#!/usr/local/bin/bash
umask 002
unset PRODUCTS
. /sdss/ups/etc/setups.sh

# Following exported for use in sdssmake environment
export upsRoot=/sdss/ups
export upsSetup=/sdss/ups/etc/setups.sh
export newVers=v1_6
export flavDir=Linux-2-4-2-3-2

cd Linux-2-4-2-3-2
mv fftw fftw.REMOVING
rm -rf fftw.REMOVING >& /dev/null &
echo "---------------------------------------------------------------"
date
echo -e "\nBegin cvs export -r v1_6 fftw"
setup sdsscvs
cvs export -r v1_6 fftw
rc=$?
if (( rc != 0 ))
then
    echo "error: \`cvs export -r v1_6 fftw' exited $rc"
    exit 1
fi
date
echo "Ended cvs export -r v1_6 fftw"
buildProds=(sdsstools tex)
echo "---------------------------------------------------------------"
echo -e "\nBegin setup of products required for building"
for item in ${buildProds[@]}
do
    echo "Begin \`setup $item'"
    setup $item
    rc=$?
    if (( rc != 0 ))
    then
        echo "error: \`setup $item' exited $rc"
        exit 1
    echo "Ended \`setup $item'"
    fi
done
date
echo "Ended setup of products required for building"
echo "---------------------------------------------------------------"
echo "native compiler desired (default), will unset gcc if already set"
[ ! -z "$SETUP_GCC" ] && unsetup gcc
date
echo "Using \"default\" gcc = $(which gcc)"
echo "Completed \`unsetup gcc' if previously set"
echo "---------------------------------------------------------------"
echo -e "\nBegin setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/fftw fftw "
setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/fftw fftw 
rc=$?
if (( rc != 0 ))
then
    echo "error: \`setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/fftw fftw ' exited $rc"
    exit 1
fi
echo "Ended \`setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/fftw fftw '"
echo "---------------------------------------------------------------"
echo -e "\nBegin setup tex"
setup tex
rc=$?
if (( rc != 0 ))
then
    echo "error: \`setup tex' exited $rc"
    exit 1
fi
echo "Completed \`setup tex'"
if (( ${#SETUP_POSTGRES} > 0 ))
then
    unsetup postgres
fi
echo "---------------------------------------------------------------"
date
echo "Build for product=fftw cvsTag=v1_6 declareVersion=v1_6"
echo "fftw -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/fftw"
echo -e "\nBuild with this environment:\n"
set | grep "_DIR" | sort
echo "---------------------------------------------------------------"
echo -e "\nTEXINPUTS=$TEXINPUTS"
echo -e "\n$PATH"
echo "---------------------------------------------------------------"
export FFTW_INSTALL_DIR=/sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
echo "Using FFTW_INSTALL_DIR=/sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2"
echo "---------------------------------------------------------------"
date
if [ ! -f fftw/Makefile ]
then
    if [ -f fftw/Makefile.in -a -f fftw/configure ]
    then
        cd fftw
        ./configure --prefix=/sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
        cd ..
    else
        echo "error: no Makefile.in or configure script"
        exit 2
    fi
fi
echo "Begin sdssmake -fullopt all"
cd fftw
pwd
eval sdssmake -fullopt all
rc=$?
if (( rc != 0 ))
then
    echo "error: \`sdssmake -fullopt all' exited $rc"
    exit 1
fi
cd ..
echo "Ended sdssmake -fullopt all"
echo "---------------------------------------------------------------"
date
echo "Begin sdssmake install"
export FFTW_DIR=/sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
echo "FFTW is $FFTW"
echo -e "Making product installation directory"
mkdir -m 775 -p /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
cd fftw
sdssmake install
rc=$?
if (( rc != 0 ))
then
    echo "error: \`sdssmake install' exited $rc"
    exit 1
fi
cd ..
echo "Ended sdssmake install"
upsDclComArgs="fftw v1_6  -f Linux+2.4-2.3.2 -r fftw/v1_6/Linux-2-4-2-3-2 -z /sdss/ups/db"
if [ -f fftw/ups/fftw.table ]
then
    upsTblArgs="-m fftw.table -M ups"
else
    # product ups directory has not table file, copying from earlier version
    Gargs="-r /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2  fftw v1_6 -m v1_6.table"
    ups copy -W -f Linux+2.4-2.3.2 -G "$Gargs" fftw v1_4
    # remove all -z flags from copied table file
    perl -p -i -e 's/\b -z .*\b//g' /sdss/ups/db/fftw/v1_6.table
    upsTblArgs="-m v1_6.table"
fi
echo "Begin \`ups declare $upsDclComArgs $upsTblArgs'"
ups declare $upsDclComArgs $upsTblArgs
echo "Ended \`ups declare $upsDclComArgs $upsTblArgs'"
chmod g+w /sdss/ups/prd/fftw/v1_6
chown jhendry.products /sdss/ups/prd/fftw/v1_6
echo -e "\nSeal the product"
{
    echo "Seal for FFTW"
    date
    echo "=== _DIR environment"
    set | grep "_DIR" | sort
    for _dir in bin etc lib
    do
        if [ -d /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2/$_dir ]
        then
            for _f in /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2/$_dir/*
            do
                if [ -f $_f ]
                then
                    echo "=== sum for $_dir/*"
                    sum /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2/$_dir/*
                    break
                fi
            done
        fi
    done
    echo "=== ls -lR for FFTW"
    chmod -R a-w /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
    ls -lR /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
} > seal.fftw.v1_6.v1_6.Linux+2.4-2.3.2
chmod +w /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
cp seal.fftw.v1_6.v1_6.Linux+2.4-2.3.2 /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
chgrp -R products /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
chmod a-w /sdss/ups/prd/fftw/v1_6/Linux-2-4-2-3-2
echo "ALL DONE"
