#ifndef lint
static char SccsId[] = "@(#)dispmask.c   1.1  9/2/93";
#endif

/* Module:	dispmask.c (Display Mask)
 *
 * Purpose:	Manipulate image masks.
 *
 * Subroutine:	dispmask_putMaskGlyph()		returns: void
 *		dispmask_updateXimage()		returns: void
 *		dispmask_toggleVisibility()	returns: void
 *		dispmask_allocColor()		returns: void
 *		dispmask_fixColorTable()	returns: void
 *		dispmask_fixMaskLUT()		returns: void
 *		dispmask_fudgeMask()		returns: void
 *
 * Xlib calls:	XPutPixel()
 *
 * Modified:	{0} Tom Nicinski	initial version		 2 Sep 1993
 *              {n} <who> -- <does what> -- <when>
 */

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>           /* X window stuff                             */
#include <X11/Xutil.h>          /* X window manager stuff                     */
#include "hfiles/define.h"	/* Define MIN, MAX, DONT_CARE, etc.           */
#include "hfiles/struct.h"	/* Declare structure types                    */
#include "hfiles/extern.h"      /* Extern main parameter structures           */

#define	DISPMASK_DEBUG	0	/* Use printf's to debug some things          */

/******************************************************************************/

/*
 * Subroutine:	dispmask_putMaskGlyph
 *
 * Purpose:	Generate a masking glyph (this is useful when the image is
 *		zoomed).
 *
 * Xlib call:   XPutPixel()
 */

#ifdef	ANSIC
void dispmask_putMaskGlyph
   (
             long  int	 upLeftX,	/* IN:    X coord of upper left corner*/
             long  int	 upLeftY,	/* IN:    Y coord of upper left corner*/
                   char	 pixVal,	/* IN:    Pixel value for glyph       */
             long  int	 width,		/* IN:    Max width  in pixels        */
             long  int	 height		/* IN:    Max height in pixels        */
   )
#else
void dispmask_putMaskGlyph (upLeftX, upLeftY, pixVal, width, height)

             long  int	 upLeftX;	/* IN:    X coord of upper left corner*/
             long  int	 upLeftY;	/* IN:    Y coord of upper left corner*/
                   char	 pixVal;	/* IN:    Pixel value for glyph       */
             long  int	 width;		/* IN:    Max width  in pixels        */
             long  int	 height;	/* IN:    Max height in pixels        */
#endif

   {	/* dispmask_putMaskGlyph */

   /*
    * Declarations.
    */

   register  long  int	 lcl_x;
   register  long  int	 lcl_xEnd;
   register  long  int	 lcl_y;
   register  long  int	 lcl_yEnd;
             long  int	 lcl_xBeg;
             long  int	 lcl_yBeg;
             long  int	 lcl_xIndent;
             long  int	 lcl_yIndent;

             long  int	 lcl_cutL;
             long  int	 lcl_cutR;
             long  int	 lcl_cutT;
             long  int	 lcl_cutB;

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   lcl_xEnd = upLeftX + width;			/* Reduce computation below   */
   lcl_yEnd = upLeftY + height;			/* Reduce computation below   */

   lcl_cutL = (upLeftX  >=                   0)   ? 0 : -upLeftX;
   lcl_cutT = (upLeftY  >=                   0)   ? 0 : -upLeftY;
   lcl_cutR = (lcl_xEnd <= (int)(dispbox.width))  ? 0 : lcl_xEnd - (int)(dispbox.width);
   lcl_cutB = (lcl_yEnd <= (int)(dispbox.height)) ? 0 : lcl_yEnd - (int)(dispbox.height);

   switch (dispmask.glyph)
      {
      case M_GLYPH_BOX :
      /*
       * Draw the glyph as a box within the pixel.
       *
       *      .-- xIndent --.
       *      |             |
       *     |-->|       |<--|
       *     |---- width ----|
       * -+- +-+-+-+-+-+-+-+-+ -+-	Cutting down the glyph is necessary
       *  |  | | | | | | | | |  | cutT	when the glyph would go outside the
       *  |  +-+-+-+-+-+-+-+-+ -+-	boundaries of the X image.
       *  |  | | | | | | | | |
       *  |  +-+-+-+-+-+-+-+-+
       *     | | | |X|X|X| | |
       *  h  +-+-+-+-+-+-+-+-+
       *  e  | | | | | |X| | |
       *  i  +-+-+-+-+-+-+-+-+
       *  g  | | | | | |X| | |
       *  h  +-+-+-+-+-+-+-+-+
       *  t  | | | |X|X|X| | |
       *     +-+-+-+-+-+-+-+-+ ---
       *  |  | | | | | | | | |  ^
       *  |  +-+-+-+-+-+-+-+-+  | cutB
       *  |  | | | | | | | | |  |
       * -+- +-+-+-+-+-+-+-+-+ -+-
       *     |---->|       |-|
       *      |             |
       *      `-- cutL      `-- cutR
       */

      lcl_xIndent = width  / 4;
      lcl_yIndent = height / 4;

      lcl_xBeg = upLeftX + ((lcl_cutL > lcl_xIndent) ? lcl_cutL : lcl_xIndent);
      lcl_yBeg = upLeftY + ((lcl_cutT > lcl_yIndent) ? lcl_cutT : lcl_yIndent);

      lcl_xEnd += -1     - ((lcl_cutR > lcl_xIndent) ? lcl_cutR : lcl_xIndent);
      lcl_yEnd += -1     - ((lcl_cutB > lcl_yIndent) ? lcl_cutB : lcl_yIndent);

      /*
       * Draw a box within the pixel.
       *
       *   o   Draw the top, bottom, left, and right sides.
       */

      if (lcl_cutT <= lcl_yIndent)
         {
         for (lcl_x = lcl_xBeg; lcl_x <= lcl_xEnd; lcl_x++)
            {
            XPutPixel (&dispbox.image, lcl_x, lcl_yBeg, pixVal);
            }
         }
      if (lcl_cutB <= lcl_yIndent)
         {
         for (lcl_x = lcl_xBeg; lcl_x <= lcl_xEnd; lcl_x++)
            {
            XPutPixel (&dispbox.image, lcl_x, lcl_yEnd, pixVal);
            }
         }
      if (lcl_cutL <= lcl_xIndent)
         {
         for (lcl_y = lcl_yBeg + 1; lcl_y <= lcl_yEnd; lcl_y++)
            {
            XPutPixel (&dispbox.image, lcl_xBeg, lcl_y, pixVal);
            }
         }
      if (lcl_cutL <= lcl_xIndent)
         {
         for (lcl_y = lcl_yBeg + 1; lcl_y <= lcl_yEnd; lcl_y++)
            {
            XPutPixel (&dispbox.image, lcl_xEnd, lcl_y, pixVal);
            }
         }
      break;

      /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

      default :
      /*
       * Draw the glyph as an "X".  Aspect ratio is   N O T   taken into con-
       * sideration.
       *
       *     |---- width ----|
       * -+- +-+-+-+-+-+-+-+-+ -+-	Cutting down the glyph is necessary
       *  |  | | | | | | | | |  | cutT	when the glyph would go outside the
       *  |  +-+-+-+-+-+-+-+-+ -+-	boundaries of the X image.
       *  |  | | | | | | |X| |
       *  |  +-+-+-+-+-+-+-+-+
       *     | | |X| | |X| | |
       *  h  +-+-+-+-+-+-+-+-+
       *  e  | | | |X|X| | | |
       *  i  +-+-+-+-+-+-+-+-+
       *  g  | | | |X|X| | | |
       *  h  +-+-+-+-+-+-+-+-+
       *  t  | | |X| | |X| | |
       *     +-+-+-+-+-+-+-+-+ ---
       *  |  | | | | | | | | |  ^
       *  |  +-+-+-+-+-+-+-+-+  | cutB
       *  |  | | | | | | | | |  |
       * -+- +-+-+-+-+-+-+-+-+ -+-
       *     |-->|         |-|
       *      |             |
       *      `-- cutL      `-- cutR
       *
       *   o   Draw the diagonal from upper left to lower right.
       */

      if (lcl_cutL > lcl_cutT) { lcl_x     = upLeftX + lcl_cutL; lcl_y = upLeftY + lcl_cutL; }
                          else { lcl_x     = upLeftX + lcl_cutT; lcl_y = upLeftY + lcl_cutT; }
      if (lcl_cutR > lcl_cutB) { lcl_xEnd -= lcl_cutR; }
                          else { lcl_xEnd -= lcl_cutB; }

      for ( ; (lcl_x < lcl_xEnd); lcl_x++, lcl_y++)
         {
         XPutPixel (&dispbox.image, lcl_x, lcl_y, pixVal);
         }

      /*
       *   o   Draw the diagonal from lower left to upper right.
       */

      if (lcl_cutL > lcl_cutB) { lcl_x     = upLeftX + lcl_cutL; lcl_y = upLeftY + height - lcl_cutL - 1; }
                          else { lcl_x     = upLeftX + lcl_cutB; lcl_y = upLeftY + height - lcl_cutB - 1; }
      if (lcl_cutR > lcl_cutT) { lcl_xEnd -= lcl_cutR; }
                          else { lcl_xEnd -= lcl_cutT; }

      for ( ; (lcl_x < lcl_xEnd); lcl_x++, lcl_y--)
         {
         XPutPixel (&dispbox.image, lcl_x, lcl_y, pixVal);
         }
      break;
      }

   }	/* dispmask_putMaskGlyph */

/******************************************************************************/

/*
 * Subroutine:	dispmask_updateXimage
 *
 * Purpose:     Copy the dispbox image and overlay the mask on top. Any
 *		transformations performed on the dispbox are done on mask:
 *
 *		   panning (zooming)
 *
 *		This updating is performed only when the masked image is to be
 *		visible and needs updating.
 *
 * Xlib call:   XPutPixel()
 */

#ifdef	ANSIC
void dispmask_updateXimage (void)
#else
void dispmask_updateXimage ()
#endif

   {	/* dispmask_updateXimage */

   /*
    * Declarations.
    */

                           char	*lcl_dataSave;	/* For switching in X image   */
   register                unsigned char *lcl_dispmask;	/* Mask datum                 */
                     long  int	 lcl_maskXini;	/* Mask column coordinate     */
   register          long  int	 lcl_maskX;	/* Mask column coordinate     */
                     long  int	 lcl_maskY;	/* Mask row    coordinate     */
   register          long  int	 lcl_maskXend;	/* Non-inclusive column       */
                     long  int	 lcl_maskYend;	/* Non-inclusive row          */
                     long  int	 lcl_XimgXini;	/* X image column coordinate  */
   register          long  int	 lcl_XimgX;	/* X image column coordinate  */
                     long  int	 lcl_XimgY;	/* X image row    coordinate  */

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * Determine whether the masked image should be updated.
    *
    *   o   If dispmask.update.hard is not true and none of the dispmask.XXX
    *       saved contexts changed, or the masked image is not visible, the
    *       masked image is not updated. This is meant to save overall time.
    *
    *       NOTE:  There is no need to check for changes in the colormap
    *              (usually on a color X visual (screen), but not limited to
    *              that), including inversion of the image.  Such changes to
    *              the colormap are reflected immediately by X (and the display
    *              hardware).  Only conditions (changes) that require SAO to
    *              regenerate the image data need to be tested here:
    *
    *                 color.scale.mode		linear, square root, log, etc.
    *
    *                 color.halftone.mode	dithering, etc.
    *
    *                 color.halftone.inverse	inverted monochrome colors
    *
    *                 color.colormap_mode	since this controls mono/color
    *
    *          NOTE:  color.inverse		is NOT checked.  It's only used
    *						for color mode, in which case,
    *						the masking colors are not
    *						affected.
    */

   if ((dispbox.image.data == (char *)(0)) || (dispmask.image.data == (char *)(0)))
      {
      dispmask.update.hard = 1;			/* Show we'll need to update  */
      dispmask.visible     = 0;			/* Mask is not visible now    */
      goto rtn_return;
      }

   if (       !dispmask.valid
       ||     !dispmask.visible
       || (   !dispmask.update.hard
           && (dispmask.update.scaleMode    == color.scale.mode)
           && (dispmask.update.halfMode     == color.halftone.mode)
           && (dispmask.update.halfInverse  == color.halftone.inverse)
           && (dispmask.update.colormapMode == color.colormap_mode)
      )   )
      {
      goto rtn_return;
      }

   /*
    * Copy the dispbox X image to the dispmask X image.
    *
    * NOTE:  This code assumes that the dispbox and dispmask images were set up
    *        properly, that is, they are the same size and do not overlap.
    */

   memcpy (dispmask.image.data, dispbox.image.data, dispbox.data_size);

   lcl_dataSave = dispbox.image.data;			/* For XPutPixel ()   */
                  dispbox.image.data = dispmask.image.data;

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * Determine the bounds of mask subarea that we'll be looking at.  The
    * following examples are for non-zoomed images (note that all arrays and
    * positions are 0-indexed):
    *
    *       |----- dispbox.width ------|
    *    -+-+--------------------------+
    *     d |         |                |
    *     i |         |                |	iadd_outx/y are coord.imgtodisp.
    *     s |     iadd_outy            |	iadd_outx/y.  See the note below
    *     p |   i     |                |	for a detailed explanation of
    *     b |   a     V                |	their meanings.
    *     o |   d    +-----------------+---+
    *     x |   d    |\\\\\\\\\\\\\\\\\|   |
    *     . |---_--->|\\\\\\\\\\\\\\\\\|   |
    *     h |   o    |\\\   pixel   \\\|  lcl_maskY
    *     e /   u    /\\\    and    \\\/   |
    *     i /   t    /\\\ mask data \\\/   |
    *     g |   x    |\\\\\\\\\\\\\\\\\|   |
    *     h |        |\\\\\\\\\\\\\\\\\|   |
    *     t |        |\\\\\\\\\\\\\\\\\|   V
    *    -+-+--------+-----------------+- lcl_maskYend
    *                |                 |
    *                +- lcl_maskX --> lcl_maskXend
    *
    *
    *  .------------ index 0 for buffer  column
    *  |         .-- index 0 for X image column
    *  V         V
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * | | | | | | | | | | | | | | | | | | | -+- <-- index 0 for buffer  row
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+  |
    * | | | | | | | | | | | | | | | | | | |  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ iadd_outy = -4.5, add_outy = -5.0
    * | | | | | | | | | | | | | | | | | | |  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+  |
    * | | | | |X|X|X|X|X|X|X|X|X|X|X|X|X|X|  |  <-- X image border
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ -+-
    * | | | | |X| | | | | | | | | | | | | |     <-- index 0 for X image row
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * | | | | |X| | | | | | | | | | | | | |	add_outx is the distance from
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+	the inside edge of the border to
    * | | | | |X| | | | | | | | | | | | | |	the left edge of the first pixel
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+	iadd_outx is the distance from
    *  |--------|				the inside edge of the border to
    *     |					the middle of the first pixel.
    *     `-- iadd_outx = -4.5, add_outx = -5.0
    *
    *
    *    .---------- index 0 for X image column
    *    |       .-- index 0 for buffer  column
    *    V       V
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |X|X|X|X|X|X|X|X|X|X|X|X|X|X|X|X|X|X|     <-- X image border
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ -+-
    * |X| | | | | | | | | | | | | | | | | |  |  <-- index 0 for X image row
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+  |
    * |X| | | | | | | | | | | | | | | | | |  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ iadd_outy =  3.5, add_outy =  3.0
    * |X| | | | | | | | | | | | | | | | | |  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+  |
    * |X| | | | |*|*|*|*|*|*|*|*|*|*|*|*|*| -+- <-- index 0 for buffer  row
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |X| | | | |*|*|*|*|*|*|*|*|*|*|*|*|*|
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |X| | | | |*|*|*|*|*|*|*|*|*|*|*|*|*|
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    *   |--------|
    *       |
    *       `--- iadd_outx = 4.5, add_outx = 4.0
    *
    * When zooming by magnifying the image, iadd_outx and add_outx will differ
    * even more.  Consider only the x direction and the magnified (8 fold) edge
    * pixel of the image which is offset from the X image border:
    *
    *  .------------ index 0 for buffer  column
    *  |         .-- index 0 for X image column
    *  V         V
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+	add_outx is the distance from
    * |X| | | | |*|*|*|*|*|*|*|*| | | | | |	the inside edge of the border to
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+	the middle of the edge pixel.
    * |X| | | | |*|*|*|*|*|*|*|*| | | | | |	iadd_outx is the distance from
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+	the inside edge of border to the
    * |X| | | | |*|*|*|*|*|*|*|*| | | | | |	middle of the "middle" pixel.
    *   |--------------|
    *   |--------|   |
    *        |       `--- iadd_outx = 8.5	ixzoom = 8
    *        `--- add_outx = 4.5
    *
    * When zooming, by reducing the image, only one pixel of the reduced set of
    * pixels is displayed (SAOimage decimates pixels rather than averaging).
    *
    * +-+-+ -.				The pixel in the lower right corner is
    * | | |  |				displayed.  add_outx/y and iadd_outx/y
    * +-+-+  | disptoimg.iny_outy	have very similar meanings as above, but
    * | |O|  | (ixzoom = 2)		they refer to the first (top and left)
    * +-+-+ -'				pixel that is displayed.
    */

   if (!coord.imgtodisp.zoom)
      {
      /*
       * A non-zoomed image.  This is done as a special case to improve perfor-
       * mance.
       *
       *   o   There is no need to worry about a pixel only being partially
       *       visible -- it's either completely visible or not.
       */

      if ((long int)(coord.imgtodisp.iadd_outx) >= 0)
         {
         lcl_XimgXini =  (long int)(coord.imgtodisp.iadd_outx);
         lcl_maskXini =   0;
         lcl_maskXend =  (coord.img.width                  <= (dispbox.width - lcl_XimgXini))
                      ?   coord.img.width                   : (dispbox.width - lcl_XimgXini);
         }
      else
         {
         lcl_XimgXini =   0;
         lcl_maskXini = -(long int)(coord.imgtodisp.iadd_outx) + 1; /* + 1 is for X image border */
         lcl_maskXend = ((coord.img.width  - lcl_maskXini) <=  dispbox.width)
                      ?   coord.img.width                   : (dispbox.width + lcl_maskXini);
         }

      if ((long int)(coord.imgtodisp.iadd_outy) >= 0)
         {
         lcl_XimgY    =  (long int)(coord.imgtodisp.iadd_outy);
         lcl_maskY    =   0;
         lcl_maskYend =  (coord.img.height                 <= (dispbox.height - lcl_XimgY))
                      ?   coord.img.height                  : (dispbox.height - lcl_XimgY);
         }
      else
         {
         lcl_XimgY    =   0;
         lcl_maskY    = -(long int)(coord.imgtodisp.iadd_outy) + 1; /* + 1 is for X image border */
         lcl_maskYend = ((coord.img.height - lcl_maskY)    <=  dispbox.height)
                      ?   coord.img.height                  : (dispbox.height + lcl_maskY);
         }

      /*
       * Go through each pixel in the mask and determine whether it should overlay
       * the original image pixel.
       *
       * NOTE:  The mask at this point must have been "corrected" with dispmask_
       *        fixMask.
       */

#if DISPMASK_DEBUG
      printf ("XimgX/Y: %3d/%3d  maskX/Y: %2d-%2d/%2d-%2d  inx/i/add_outx: %3.1f/%7.3f/%7.3f  inx/i/add_outx: %5.3f/%7.3f/%7.3f\n",
           lcl_XimgXini, lcl_XimgY, lcl_maskXini, lcl_maskXend, lcl_maskY, lcl_maskYend,
           coord.imgtodisp.inx_outx, coord.imgtodisp.iadd_outx, coord.imgtodisp.add_outx,
           coord.disptoimg.inx_outx, coord.disptoimg.iadd_outx, coord.disptoimg.add_outx);
#endif
      for ( ; lcl_maskY < lcl_maskYend; lcl_maskY++, lcl_XimgY++)
         {
         lcl_XimgX    = lcl_XimgXini;
         lcl_dispmask = dispmask.buf + (lcl_maskY * coord.img.width) + lcl_maskXini;
         for (lcl_maskX = lcl_maskXini; lcl_maskX < lcl_maskXend; lcl_dispmask++, lcl_maskX++, lcl_XimgX++)
            {
            if (dispmask.lut[*lcl_dispmask] != dispmask.color.pixTransparent)
               {
               XPutPixel (&dispbox.image, lcl_XimgX, lcl_XimgY, dispmask.lut[*lcl_dispmask]);
               }
            }
         }
      }
      /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   else
      {
      /*
       * The image is zoomed.
       *
       *   o   coord.imgtodisp.inx_outx and .iny_outy are multiplication factors
       *       used to convert a data pixel index to the X image index (less any
       *       offset from the origin).
       *
       *       coord.disptoimg works in reverse from coord.imgtodisp.  It takes
       *       X image coordinates as input and returns data coordinates.
       */

      if (coord.imgtodisp.multiply)
         {
         /*
          * The image is magnified.
          */

         if ((lcl_maskXini =                                               coord.disptoimg.iadd_outx)     < 0)
            {
              lcl_maskXini =                                                                                0;
            }
         if ((lcl_maskXend = (dispbox.width  * coord.disptoimg.inx_outx) + coord.disptoimg.iadd_outx + 1) > coord.img.width)
            {
              lcl_maskXend =                                                                                coord.img.width;
            }
         if ((lcl_maskY    =                                               coord.disptoimg.iadd_outy)     < 0)
            {
              lcl_maskY    =                                                                                0;
            }
         if ((lcl_maskYend = (dispbox.height * coord.disptoimg.iny_outy) + coord.disptoimg.iadd_outy + 1) > coord.img.height)
            {
              lcl_maskYend =                                                                                coord.img.height;
            }

         lcl_XimgXini = (lcl_maskXini * coord.imgtodisp.inx_outx) + coord.imgtodisp.add_outx;
         lcl_XimgY    = (lcl_maskY    * coord.imgtodisp.iny_outy) + coord.imgtodisp.add_outy;

#if DISPMASK_DEBUG
       printf ("XimgX/Y: %3d/%3d  maskX/Y: %2d-%2d/%2d-%2d  inx/i/add_outx: %3.1f/%7.3f/%7.3f  inx/i/add_outx: %5.3f/%7.3f/%7.3f\n",
            lcl_XimgXini, lcl_XimgY, lcl_maskXini, lcl_maskXend, lcl_maskY, lcl_maskYend,
            coord.imgtodisp.inx_outx, coord.imgtodisp.iadd_outx, coord.imgtodisp.add_outx,
            coord.disptoimg.inx_outx, coord.disptoimg.iadd_outx, coord.disptoimg.add_outx);
#endif
         for ( ; lcl_maskY < lcl_maskYend; lcl_maskY++, lcl_XimgY += coord.imgtodisp.iyzoom)
            {
            lcl_XimgX    = lcl_XimgXini;
            lcl_dispmask = dispmask.buf + (lcl_maskY * coord.img.width) + lcl_maskXini;
            for (lcl_maskX = lcl_maskXini; lcl_maskX < lcl_maskXend; lcl_dispmask++, lcl_maskX++,
                                                                                     lcl_XimgX += coord.imgtodisp.ixzoom)
               {
               if (dispmask.lut[*lcl_dispmask] != dispmask.color.pixTransparent)
                  {
                  dispmask_putMaskGlyph (lcl_XimgX, lcl_XimgY, dispmask.lut[*lcl_dispmask], coord.imgtodisp.ixzoom,
                                                                                            coord.imgtodisp.iyzoom);
                  }
               }
            }

         }
         /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
      else
         {
         /*
          * The image is reduced.
          */

         if ((lcl_maskXini =                                               coord.disptoimg.iadd_outx) < (coord.disptoimg.ixzoom-1))
            {
              lcl_maskXini =                                                                             coord.disptoimg.ixzoom-1;
            }
         if ((lcl_maskXend = (dispbox.width  * coord.disptoimg.inx_outx) + coord.disptoimg.iadd_outx + 1) > coord.img.width)
            {
              lcl_maskXend =                                                                                coord.img.width;
            }
         if ((lcl_maskY    =                                               coord.disptoimg.iadd_outy) < (coord.disptoimg.iyzoom-1))
            {
              lcl_maskY    =                                                                             coord.disptoimg.iyzoom-1;
            }
         if ((lcl_maskYend = (dispbox.height * coord.disptoimg.iny_outy) + coord.disptoimg.iadd_outy + 1) > coord.img.height)
            {
              lcl_maskYend =                                                                                coord.img.height;
            }

         lcl_XimgXini = (lcl_maskXini * coord.imgtodisp.inx_outx) + coord.imgtodisp.add_outx;
         lcl_XimgY    = (lcl_maskY    * coord.imgtodisp.iny_outy) + coord.imgtodisp.add_outy;

#if DISPMASK_DEBUG
       printf ("XimgX/Y: %3d/%3d  maskX/Y: %2d-%2d/%2d-%2d  inx/i/add_outx: %3.1f/%7.3f/%7.3f  inx/i/add_outx: %5.3f/%7.3f/%7.3f\n",
            lcl_XimgXini, lcl_XimgY, lcl_maskXini, lcl_maskXend, lcl_maskY, lcl_maskYend,
            coord.imgtodisp.inx_outx, coord.imgtodisp.iadd_outx, coord.imgtodisp.add_outx,
            coord.disptoimg.inx_outx, coord.disptoimg.iadd_outx, coord.disptoimg.add_outx);
#endif
         for ( ; lcl_maskY < lcl_maskYend; lcl_maskY += coord.disptoimg.iyzoom, lcl_XimgY++)
            {
            lcl_XimgX    = lcl_XimgXini;
            lcl_dispmask = dispmask.buf + (lcl_maskY * coord.img.width) + lcl_maskXini;
            for (lcl_maskX = lcl_maskXini; lcl_maskX < lcl_maskXend; lcl_dispmask += coord.disptoimg.ixzoom,
                                                                     lcl_maskX    += coord.disptoimg.ixzoom,
                                                                     lcl_XimgX++)
               {
               if (dispmask.lut[*lcl_dispmask] != dispmask.color.pixTransparent)
                  {
                  XPutPixel (&dispbox.image, lcl_XimgX, lcl_XimgY, dispmask.lut[*lcl_dispmask]);
                  }
               }
            }

         }   /* if (coord.imgtodisp.multiply) else */
      }   /* if (!coord.imgtodisp.zoom) else */

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * All done.
    */

   dispbox.image.data           = lcl_dataSave;	/* Restore original X image   */

   dispmask.update.hard         = 0;		/* We've updated the mask     */
   dispmask.update.scaleMode    = color.scale.mode;
   dispmask.update.halfMode     = color.halftone.mode;
   dispmask.update.halfInverse  = color.halftone.inverse;
   dispmask.update.colormapMode = color.colormap_mode;

rtn_return:

   return;

   }	/* dispmask_updateXimage */

/******************************************************************************/

/*
 * Subroutine:	dispmask_toggleVisibility
 *
 * Purpose:     Toggle the visibility setting of the mask.  In addition,
 *		perform any operations that need to make the output look
 *		appropriate:
 *
 *		   o	redraw the mouse pointer coordinates
 *
 *		These operations are only performed when the masked image is
 *		valid.
 *
 * Xlib call:   
 */

#ifdef	ANSIC
void dispmask_toggleVisibility (void)
#else
void dispmask_toggleVisibility ()
#endif

   {	/* dispmask_toggleVisibility */

   if (dispmask.valid && dispmask.buf != NULL)
      {
      dispmask.visible ^= 1;
      magnify_disp_text ();
      }

   return;

   }	/* dispmask_toggleVisibility */

/******************************************************************************/

/*
 * Subroutine:	dispmask_allocColor
 *
 * Purpose:     Allocate a color for use with masks.
 *
 *		NOTE:	Color allocating here is independent of the color allo-
 *			cating done by SAO elsewhere.  But, the same colormap
 *			is used.
 *
 * Returns:
 *
 *       0	No color was allocated.
 *
 *   non-0	A  color (either the requested or default) was allocated.
 *
 * Xlib call:	XParseColor (), XAllocColor ()
 */

#ifdef	ANSIC
unsigned long  int dispmask_allocColor
   (
                  char	*colorName,	/* IN:    Name/hex-spec of color      */
                  char	*colorDefault,	/* IN:    Name/hex-spec of default    */
   XColor		*colorCell,	/* OUT:   Fills in pixel info         */
                  char	*nameFail,	/* IN:    Failure message (format str)*/
                  char	*defaultFail,	/* IN:    Failure message (format str)*/
                  char	*defaultSuccess	/* IN:    Used default color ("   str)*/
   )
#else
unsigned long  int dispmask_allocColor (colorName, colorDefault, colorCell, nameFail, defaultFail, defaultSuccess)

                  char	*colorName;	/* IN:    Name/hex-spec of color      */
                  char	*colorDefault;	/* IN:    Name/hex-spec of default    */
   XColor		*colorCell;	/* OUT:   Fills in pixel info         */
                  char	*nameFail;	/* IN:    Failure message (format str)*/
                  char	*defaultFail;	/* IN:    Failure message (format str)*/
                  char	*defaultSuccess;/* IN:    Used default color ("   str)*/
#endif

   {	/* dispmask_allocColor */

   /*
    * Declarations.
    */

                           int	 lcl_status;

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if ((lcl_status = XParseColor (color.display, color.colormap, colorName, colorCell)) != 0)
      {
        lcl_status = XAllocColor (color.display, color.colormap, colorCell);
      }

   if (lcl_status == 0)
      {
      if (nameFail != 0)
         {
         fprintf (stderr, nameFail, colorName);
         }
      if (colorDefault != 0)
         {
         if ((lcl_status = XParseColor (color.display, color.colormap, colorDefault, colorCell)) != 0)
            {
              lcl_status = XAllocColor (color.display, color.colormap, colorCell);
            }
         if  (lcl_status == 0)
            {
            if (defaultFail    != 0) { fprintf (stderr, defaultFail,               colorDefault); }
            }
         else
            {
            if (defaultSuccess != 0) { fprintf (stderr, defaultSuccess, colorName, colorDefault); }
            }
         }
      fflush (stderr);
      }

   return (lcl_status);

   }	/* dispmask_allocColor */

/******************************************************************************/

/*
 * Subroutine:	dispmask_fixColorTable
 *
 * Purpose:     Fix the display mask color lookup table to correspond to the
 *		X visual (screen) we're using.
 *
 *		NOTE:	Color allocating here is independent of the color allo-
 *			cating done by SAO elsewhere.  But, the same colormap
 *			is used.
 *
 * Xlib call:	None
 */

#ifdef	ANSIC
void dispmask_fixColorTable
   (
                  int	 nameSize	/* IN:    Size of names in bytes      */
   )
#else
void dispmask_fixColorTable (nameSize)

                  int	 nameSize;	/* IN:    Size of names in bytes      */
#endif

   {	/* dispmask_fixColorTable */

   /*
    * Declarations.
    */

                           int	 lcl_status;
            unsigned long  int	 lcl_monochrome;	/* Monochrome screen? */
                           char	*lcl_colorDefault;
                           char	*lcl_strPosBeg;
                           char	*lcl_strPosEnd;
            unsigned long  int	 lcl_strIdx;
            unsigned long  int	 lcl_colorIdx;
            XColor		 lcl_colorRGB;	/* RGB color from database    */

   static                  char	*lcl_colorTransparent = "(transparent)";

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * Select the default color cell (pixel) value to use for masking.
    *
    *   o   An attempt is made to not use color.ncolors as an indicator of a
    *       monochrome screen.  This was done to keep this color allocation
    *       independent of SAO color allocations.
    */

   lcl_monochrome = (   (color.visual->bits_per_rgb == 1)
                     && (color.visual->map_entries  <= 2));

   switch (color.visual->class)
      {
      case GrayScale:
      case StaticGray:
           lcl_colorDefault = (lcl_monochrome) ? dispmask.color.def.mono : dispmask.color.def.grey;
           break;

      case PseudoColor:
      case StaticColor:
      case DirectColor:
      case TrueColor:
           lcl_colorDefault = dispmask.color.def.color;
           break;

      default:
           lcl_colorDefault = dispmask.color.def.mono;	/* Assume monochrome  */
           break;
      }

   if (lcl_colorDefault == 0)
      {
       lcl_colorDefault = "white";	/* Arbitrary, but universal, color    */
      }

   lcl_status = dispmask_allocColor (lcl_colorDefault, dispmask.color.def.mono, &lcl_colorRGB,
                        "WARNING: Unable to allocate default mask color (%s)\n",
                        "WARNING: Unable to allocate monochrome default mask color (%s)\n",
                        "WARNING: Default mask color (%s) degraded to monochrome default mask color (%s)\n");
   dispmask.color.pixDefault = (lcl_status == 0) ? 0 : lcl_colorRGB.pixel;

   /*
    *   o   For monochrome systems, SAO inverts the image (don't know why or
    *       where).  Thus, pixel values for white and black need to be inverted
    *       (since there are only two pixel values, 0 and 1, this is easy and
    *       hardwired).  There is also no need to really allocate the color, as
    *       no monochrome system is likely to have settable pixel intensities.
    */

   if (lcl_monochrome)
      {
      dispmask.color.pixDefault = (dispmask.color.pixDefault == 0) ? 1 : 0;
      }

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * The color table information consists of one string (not necessarily null-
    * terminated) of the colors to be used. They are delimited by COLORDELIMITER
    * with COLORDELIMITER also terminating the string.  The string length is
    * passed.
    *
    *   o   Delimiters are converted to nulls to "properly" terminate the names.
    */

   dispmask.color.names [nameSize] = 0;	/* Null-terminate over COLORDELIMITER */
   dispmask.color.cnt              = 0;
   lcl_colorIdx                    = 0;	/* Used for error messages            */

   lcl_strPosBeg = dispmask.color.names;

   while (!lcl_monochrome)		/* Don't waste time for monochrome    */
      {
      if ((lcl_strPosEnd = strchr (lcl_strPosBeg, '|'/*COLORDELIMITER*/)) == 0)
         {
         if (lcl_strPosBeg < &(dispmask.color.names [nameSize]))
            {
             lcl_strPosEnd = &(dispmask.color.names [nameSize]);
            }
         else
            {
            break;
            }
         }
       lcl_colorIdx++;			/* Got another color off the list     */
      *lcl_strPosEnd = 0;		/* Null terminate (on COLORDELIMITER) */
      if (dispmask.color.cnt >= (dispmaskColorMax))
         {
         fprintf (stderr, "WARNING: Too many mask colors; ignoring index %d (%s)\n", lcl_colorIdx, lcl_strPosBeg);
         fflush  (stderr);
         }
      else
         {
         dispmask.color.cnt++;		/* Show we've gotten a color          */
         dispmask.color.name [dispmask.color.cnt] = lcl_strPosBeg;
         }
       lcl_strPosBeg = lcl_strPosEnd + 1;	/* Onto the next list element */
      };

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * Look up the colors from X and allocate color cell (pixel) values.
    *
    * NOTE: Color cells previously used are   N O T   freed (XFreeColors ()).
    *       The problem is that this color cell allocation does not consider
    *       the colors setup by SAO (color.pixvalmap) -- mainly because of an
    *       ignorance of how these colors are changed.  Therefore, it is pos-
    *       sible to run out of colormap space after "extended" use.
    *
    *   o   The transparent orignal mask value is zero (0).  If any pixel value
    *       is this also, the "fixed" transparent mask value will need to be
    *       something other than zero (0).
    *
    *          o   Under monochrome conditions, this can be handled with simple
    *              hardwiring of values.
    *
    *          o   Under greyscale and color conditions, the "fixed" transparent
    *              pixel value will be the index of the mask color whose pixel
    *              value is zero (0).
    *
    *              NOTE:  There is no need for a special test if dispmask.color.
    *                     pixDefault is zero (0) as it is only used to fill in
    *                     erroneous table values;  it is not used independently
    *                     anywhere else.
    */

   if (lcl_monochrome)
      {
      dispmask.color.cnt = 0;			/* No colors on monochrome    */
      if (dispmask.color.pixDefault == 0)	/* Original transparency val? */
         {
         dispmask.color.pixTransparent = 1;	/* Hardwire it                */
         }
      }
   else
      {
      for (lcl_colorIdx = 1; lcl_colorIdx <= dispmask.color.cnt; lcl_colorIdx++)
         {
         dispmask.color.pix [lcl_colorIdx] =
            (dispmask_allocColor (dispmask.color.name [lcl_colorIdx], lcl_colorDefault, &lcl_colorRGB,
                                  "WARNING: Mask color (%s) not available\n",
                                  "WARNING: Unable to allocate default mask color (%s)\n",
                                  "WARNING: Mask color (%s) degraded to default mask color (%s)\n") == 0)
            ? 0 : lcl_colorRGB.pixel;

         if (dispmask.color.pix [lcl_colorIdx] == 0) /* Orig transparent val? */
            {
             dispmask.color.pixTransparent = lcl_colorIdx;	/* Fix up     */
            }
         }
      }

   /*
    *   o   Final fixup of transparency values.
    *
    *       The original mask value of zero (0) indicating transparency is used
    *       as the index into the color table.
    */

   dispmask.color.name [0] = lcl_colorTransparent;	/* Hardwired value    */
   dispmask.color.pix  [0] = dispmask.color.pixTransparent;

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * All done.
    */

rtn_return:

   return;

   }	/* dispmask_fixColorTable */

/******************************************************************************/

/*
 * Subroutine:	dispmask_fixMaskLUT
 *
 * Purpose:     Translate original mask lookup table to use colorcell values.
 *
 *		NOTE:  dispmask_fixColorTable must have been called already.
 *
 * Xlib call:   None
 */

#ifdef	ANSIC
void dispmask_fixMaskLUT (void)
#else
void dispmask_fixMaskLUT ()
#endif

   {	/* dispmask_fixMaskLUT */

   /*
    * Declarations.
    */

                  int		 lcl_LUTidx;

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   /*
    * There are two values used to indicate transparency.  The first is zero (0)
    * in the original mask LUT.  When the mask LUT is converted from original
    * values to X colorcell values, the transparency value may no longer be zero
    * (0); it will be dispmask.color.pixTransparent (in dispmask.color.pix[0]
    * also).
    */

   for (lcl_LUTidx = 0; lcl_LUTidx < MASKLUTTOP; lcl_LUTidx++)
      {
      dispmask.lut[lcl_LUTidx] = dispmask.color.pix[dispmask.lut[lcl_LUTidx]];
      }

   return;

   }	/* dispmask_fixMaskLUT */

/******************************************************************************/

/*
 * Subroutine:	dispmask_fudgeMask
 *
 * Purpose:     Generate a fudged mask used for testing.
 *
 * Xlib call:   None
 */

#ifdef	ANSIC
void dispmask_fudgeMask (void)
#else
void dispmask_fudgeMask ()
#endif

   {	/* dispmask_fudgeMask */

   /*
    * Declarations.
    */

   register unsigned long  int	 lcl_i;

#define	dispmaskFudgeMaskLen	24	/* Must be even and greater than 4    */
#define	dispmaskFudgePut(row,col)	dispmask.buf [((row) * coord.img.width) + (col)] = 1

   /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if (dispmask.buf == NULL)
      {
      goto rtn_return;
      }

   memset ((void *)(dispmask.buf), 0, dispmask.buf_sz);

   /*
    * Generate masks around the corners to test positioning.
    */

   if (   (((dispmaskFudgeMaskLen) * 3) < coord.img.width)
       && (((dispmaskFudgeMaskLen) * 3) < coord.img.height))
      {
      for (lcl_i = 0; lcl_i < (dispmaskFudgeMaskLen); lcl_i++)
         {
         /*
          * Horizontal lines.
          */

         dispmaskFudgePut (                     0,                         lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (                     1,                         lcl_i +  1);
         dispmaskFudgePut (                     5,                         lcl_i +  5);
         dispmaskFudgePut (                    10,                         lcl_i + 10);
         dispmaskFudgePut (coord.img.height - (10 + 1),                    lcl_i + 10);
         dispmaskFudgePut (coord.img.height - ( 5 + 1),                    lcl_i +  5);
         dispmaskFudgePut (coord.img.height - ( 1 + 1),                    lcl_i +  1);
         dispmaskFudgePut (coord.img.height - ( 0 + 1),                    lcl_i +  (dispmaskFudgeMaskLen));

         dispmaskFudgePut (                   dispbox.height -   ( 0 + 1), lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (                   dispbox.height -   ( 5 + 1), lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (                   dispbox.height -   (10 + 1), lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (coord.img.height - dispbox.height -   (10 + 1), lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (coord.img.height - dispbox.height -   ( 5 + 1), lcl_i +  (dispmaskFudgeMaskLen));
         dispmaskFudgePut (coord.img.height - dispbox.height -   ( 0 + 1), lcl_i +  (dispmaskFudgeMaskLen));

         /*
          * Vertical lines.
          */

         dispmaskFudgePut (                        lcl_i,                                  1);
         dispmaskFudgePut (                        lcl_i +  ((dispmaskFudgeMaskLen) - 1),  0);
         dispmaskFudgePut (                        lcl_i +  5,                             5);
         dispmaskFudgePut (                        lcl_i + 10,                            10);
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i + 10),                           10);
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i +  5),                            5);
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i +  ((dispmaskFudgeMaskLen) - 1)), 0);
         dispmaskFudgePut (coord.img.height - 1 -  lcl_i,                                  1);

         dispmaskFudgePut (                        lcl_i + (dispmaskFudgeMaskLen),  dispbox.width - ( 0 + 1));
         dispmaskFudgePut (                        lcl_i + (dispmaskFudgeMaskLen),  dispbox.width - ( 5 + 1));
         dispmaskFudgePut (                        lcl_i + (dispmaskFudgeMaskLen),  dispbox.width - (10 + 1));
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i + (dispmaskFudgeMaskLen)), dispbox.width - (10 + 1));
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i + (dispmaskFudgeMaskLen)), dispbox.width - ( 5 + 1));
         dispmaskFudgePut (coord.img.height - 1 - (lcl_i + (dispmaskFudgeMaskLen)), dispbox.width - ( 0 + 1));
         }
      }

   /*
    * Generate an "X" somewhere in the image.
    *
    * NOTE:  This is hardwired, c'est la vie.
    */

   if ((coord.img.height >= (94 + 21)) && (coord.img.width >= (75 + 21)))
      {
      for (lcl_i = 0;     lcl_i < 21;   lcl_i++)
         {
         dispmaskFudgePut (94 + lcl_i, 75 + lcl_i);
         dispmaskFudgePut (94 + lcl_i, 95 - lcl_i);
         }
      }

   /*
    * All done.
    */

rtn_return:

   return;

   }	/* dispmask_fudgeMask */

/******************************************************************************/
