char g_dscname[51] = "";	/* Display name global variable */
int  fsaonumber = 0;		/* fsao number global variable */

#ifdef IMTOOL
#include <stdio.h>			/* stderr, NULL, etc. */
#include <X11/Xlib.h>                   /* X window stuff */
#include <X11/Xutil.h>                  /* window manager stuff */
#include "hfiles/constant.h"
#include "hfiles/struct.h"
#include "hfiles/extern.h"
#include "imtool.h"			/* moved to dervish product */

/* ??? NEED TO PUT THIS IN A COMMON INCLUDE FILE - SAME AS IN DSCCMD.H ??? */
typedef enum
   {
   S_CIRCLE,
   S_POINT,
   S_ELLIPSE,
   S_RECTANGLE,
   S_UNKNOWN,
   S_UNSUPPORTED
   } CMD_SAOSHAPES;

static int dsc_trigger = 0;
Cursor shcursor;
static char *NO_OUT_MSG = "Warning: output connection not open\n";
#define      DSC_SZ_GEOM	40
#define      DSC_SZ_IMCURVAL	40+DSC_SZ_GEOM
/* #define      SZ_IMCURVAL	160 Original definition in SAOimage */

/*
 * Subroutine:  set_curpos_to_dsc_trigger
 * Purpose:     Set up mode to send cursor position when mouse button is struck
 * Note:        Equivalent imtool subroutine: gio_readcursor()
 */
void set_curpos_to_dsc_trigger ( )
{
  if( control.IRAF_out.open == 0 ) {
    (void)fprintf(stderr, NO_OUT_MSG);
    return;
  }
  dsc_trigger = 1;

  /* change pointer appearance for this function, if we are in control mode */
  if ( control.mode == COP) {
    XDefineCursor(dispbox.display, dispbox.ID, shcursor);
  }
}


/*
 * Subroutine:  check_dsc_trigger
 * Purpose:     Check if dsc_trigger is set.
 */
int check_dsc_trigger ( )
{
  return (dsc_trigger);
}


/*
 * Subroutine:  trigger_curpos_to_dsc
 * Purpose:     Send cursor position to DSC (ala IRAF) in response to
 *              trigger event                                         
 */
int trigger_curpos_to_dsc ( event )
     XKeyEvent *event; 
{
  float fileX, fileY;
  char  curval[DSC_SZ_IMCURVAL];
  char  geomShape[DSC_SZ_GEOM];
  CMD_SAOSHAPES geomShapeType;
  void i_transform();

  if( control.IRAF_out.open == 0 ) {
    (void)fprintf(stderr, NO_OUT_MSG);
    return(0);
  }

  /* Instead of calling send_curpos_to_iraf (or some DSC equivalent, just
     send the coords directly through the fifo */

  /* Get the coordinates for the last point */
  fileX = cursor.file.X;
  fileY = cursor.file.Y;

  switch( cursor.type ) {
  case COP_Circle:
    geomShapeType = S_CIRCLE;
    fileX = cursor.file.X;
    fileY = cursor.file.Y;
    /* Resultant string: N/A, radius, N/A */
    sprintf(geomShape, "%.2f", cursor.file.Xdim);
    break;
  case COP_Ellipse:
    geomShapeType = S_ELLIPSE;
    /* Resultant string: rotation angle, X-radius, Y-radius */
    sprintf(geomShape, "%6.3f %.2f %.2f", cursor.rot.angle,
            cursor.file.Ydim, cursor.file.Xdim);			/*efb*/
    break;
  case COP_Box:
    geomShapeType = S_RECTANGLE;
    /* Resultant string: rotation angle, width, height */
    sprintf(geomShape, "%6.3f %.2f %.2f", cursor.rot.angle,
            (cursor.file.Ydim * 2.0), (cursor.file.Xdim * 2.0));	/*efb*/
    break;
  case COP_Point:
    geomShapeType = S_POINT;
    /* Get the current coordinates */
    i_transform(&coord.disptofile, event->x, event->y, &fileX, &fileY);
    /* Resultant string: N/A, N/A, N/A */
    geomShape[0] = '\0';		/* There are no relevant parameters */
    break;
  case COP_Polygon:
    geomShapeType = S_UNSUPPORTED;
    geomShape[0] = '\0';		/* There are no relevant parameters */
    break;
  default:
    geomShapeType = S_UNKNOWN;
    geomShape[0] = '\0';		/* There are no relevant parameters */
    break;
  }

  /* Must distinguish

  /* Encode the following: shape, mouse button, future, x-position, y-position,
     geomShape */

  sprintf(curval, "%d %d %d %d %d %10.3f %10.3f %s",          
                  (int)geomShapeType, (int)control.event.xbutton.button, 
                  (int)(control.event.xbutton.state & ControlMask),
                  (int)(control.event.xbutton.state & ShiftMask),
                  0, 
                  fileY, fileX, geomShape); 				/*efb*/

  /* Send it to the client program. */
  (void)write_connection(&control.IRAF_out, curval, DSC_SZ_IMCURVAL);

  return( 1 );
}


/*
 * Subroutine:  read_dscRegion
 * Purpose:     Read FITS image (region) sent from DSC.
 * Note:        This routine will take the place of the 'read_disk' routine
 *		for DSC.
 */
int read_dscRegion(a_fd, a_buf, a_nbytes)
     int    a_fd;
     char   *a_buf;                 /* i: buffer to receive read data */
     int    a_nbytes;               /* i: number of bytes expected */
{
int		    bytes, nleft;
char		    *bufptr;

bytes = SZ_FIFOBUF;
bufptr = a_buf;
for (nleft = a_nbytes; nleft > 0; nleft -= bytes)
   {
   if (nleft < SZ_FIFOBUF)
      {bytes = nleft;}
   if (!read_dscPipe(a_fd, bufptr, bytes))
      {
      perror("read error on DSC pipe");
      (void)fflush(stderr);
      goto err_exit;
      }
   bufptr += bytes;
   }

return(a_nbytes);

err_exit:
return(0);
}


/*
 * Subroutine:  read_dscPipe
 * Purpose:     Read DSC data sent down pipe.
 */
int read_dscPipe(a_fd, a_bufptr, a_nbytes)
     int    a_fd;
     char   *a_bufptr;              /* i: buffer to receive read data */
     int    a_nbytes;               /* i: number of bytes expected */
{
int		    bytes_xfer, nbytes;
char		    *bufptr;

bufptr = a_bufptr;
nbytes = a_nbytes;
bytes_xfer = read(a_fd, bufptr, nbytes);
if (bytes_xfer < 0)
   {goto err_exit;}
if (bytes_xfer < nbytes)
   {
   nbytes -= bytes_xfer;
   bufptr += bytes_xfer;
   bytes_xfer = read(a_fd, bufptr, nbytes);
   if (bytes_xfer < nbytes)
      {goto err_exit;}
   }

return(1);

err_exit:
return(0);
}



/*
 * Subroutine:  parse_dscname
 * Purpose:     Parse command line for DSC name of display window
 * Returns:     1 if one of its parameters was found, else 0
 */
int parse_dscname ( argc, argv, argi, got_color )
     int argc;          /* total number of arg tokens */
     char *argv[];      /* array of arg tokens */
     int *argi;         /* first arg to check, (returned as last arg used) */
     int *got_color;    /* status word knows coloring parameter was found */
{
int i;
int usage();
void set_submenu_toggle();

i = *argi;
/* number of planes */
if((strcmp(argv[i], "-dscname") == 0))
   {
   if((++i >= argc) || (strlen(argv[i]) > 50))
      {return( usage("No DSC display name", argc, argv, i-1, i) );}
   strcpy(g_dscname, argv[i]);
   *argi = i;
   return(1);
   }
else
   {
   return(0);
   }
}
/*
 * Subroutine:  parse_fsaonumber
 * Purpose:     Parse command line for fsao number 
 * Returns:     1 if one of its parameters was found, else 0
 */
int parse_fsaonumber ( argc, argv, argi, got_color )
     int argc;          /* total number of arg tokens */
     char *argv[];      /* array of arg tokens */
     int *argi;         /* first arg to check, (returned as last arg used) */
     int *got_color;    /* status word knows coloring parameter was found */
{
int i;
int usage();
void set_submenu_toggle();

i = *argi;
/* number of planes */
if((strcmp(argv[i], "-fsaonumber") == 0))
   {
   if((++i >= argc) || (strlen(argv[i]) > 50))
      {return( usage("No fsao number", argc, argv, i-1, i) );}
   fsaonumber = atoi(argv[i]);
   *argi = i;
   return(1);
   }
else
   {
   return(0);
   }
}
#endif /* IMTOOL */
