/* ====================================================================================== */

#include "tcl.h"
/* ================================================================================ */
/* Subroutine to define online help messages for a non file-based tcl help.
 * The help is on regular tcl commands plus some generals such as help itself,
 * ftcl tcl and tclx
 *					M.Vittone Feb 10th, 93 			    */

/* -------------------------------------------------------------------------------- */

/* The following is the list of tcl/ftcl specific commands defined in 
   regular tcl without extension plus some general help items                       */

 char *help_cmd_list[] ={ "append", "array", "break", "case", "catch",
			 "cd", "close", "concat", "continue", "echo",
			 "eof", "error", "eval", "exit",
			 "expr", "file", "flush", "for", "foreach",
			 "format", "gets", "global", "history", "if",
			 "incr", "info", "join", "lappend", "lindex",
			 "linsert", "list", "llength", "lrange", 
			 "lreplace", "lsearch", "lsort", "open", "proc",
			 "puts", "pwd", "read", "regexp", "regsub",
			 "rename", "return", "scan", "seek", "set",
			 "source", "split", "string", "tell", "trace",
			 "unset", "uplevel", "upvar", "while",
			 "help_cmd", "ftcl", "ftclParams", "inipar", 
			  "getparf", "getpari", "getpars" };

/* Here it follows the text for the messages associated to the commands		     */

 char *help_cmd_msg []= {
	  "\n"							/*  --- append ---  */
          "\t append varName value ?value value ...? \n"
               "\t\t Append all of the value arguments to the current  value \n"
               "\t\t of  variable  varName.  If varName doesn't exist, it is \n"
               "\t\t given a value equal to the  concatenation  of  all  the \n"
               "\t\t value  arguments.   This  command provides an efficient \n"
               "\t\t way to build  up  long  variables  incrementally.   For \n"
               "\t\t example,  ``append  a  $b'' is much more efficient than \n"
               "\t\t ``set a $a$b'' if $a is long. \n",
	   "\n"							/*  --- array ---   */
           "\t array option arrayName ?arg arg ...? \n"
               "\t\t This command performs one of several operations on  the \n"
               "\t\t variable  given  by  arrayName.   ArrayName must be the \n"
               "\t\t name  of  an  existing  array  variable.   The   option \n"
               "\t\t argument  determines  what action is carried out by the \n"
               "\t\t command.  To have a full detailed description on option \n"
	       "\t\t please consult the tcl man pages. 			     \n"
	       "\t\t As as example, the use of option \"names\" is shown: \n"
	       "\n"
               "\t\t array names arrayName \n"
	       "\n"
               "\t\t Returns  a list containing the names of all of the \n"
               "\t\t elements in the array.  If there are  no  elements \n"
               "\t\t in the array then an empty string is returned. \n" ,
	   "\n"							/*  --- break ---   */
           "\t break \n"
               "\t\t This command may be invoked only inside the body  of  a \n"
               "\t\t loop  command  such  as  for  or  foreach or while.  It \n"
               "\t\t returns  a  TCL_BREAK  code  to  signal  the  innermost \n"
               "\t\t containing loop command to return immediately. \n",
	   "\n"							/*  --- case ---   */
           "\t case string ?in? patList body ?patList body ...? \n"
           "\t case string ?in? {patList body ?patList body ...?} \n"
               "\t\t Match string against each of the patList  arguments  in \n"
               "\t\t order.   If  one  matches,  then evaluate the following \n"
               "\t\t body argument by passing  it  recursively  to  the  Tcl \n"
               "\t\t interpreter,  and return the result of that evaluation. \n"
               "\t\t Each patList argument consists of a single  pattern  or \n"
               "\t\t list  of patterns.  Each pattern may contain any of the \n"
               "\t\t wild-cards described under string match.  If a  patList \n"
               "\t\t argument  is  default,  the  corresponding body will be \n"
               "\t\t evaluated if no patList matches string.  If no  patList \n"
               "\t\t argument  matches  string and no default is given, then \n"
               "\t\t the case command returns an empty string. \n"
	       "\t\t For a full description of the command usage please refer  \n"
	       "\t\t to tcl man pages. \n" ,
	   "\n"							/*  --- catch ---  */
           "\t catch command ?varName? \n"
               "\t\t The catch command may be used to  prevent  errors  from \n"
               "\t\t aborting  command  interpretation.  Catch calls the Tcl \n"
               "\t\t interpreter recursively to execute command, and  always \n"
               "\t\t returns  a  TCL_OK  code, regardless of any errors that \n"
               "\t\t might occur while executing command.  The return  value \n"
               "\t\t from catch is a decimal string giving the code returned \n"
               "\t\t by the Tcl interpreter after executing  command.   This \n"
               "\t\t will  be 0 (TCL_OK) if there were no errors in command; \n"
               "\t\t otherwise it will have a non-zero  value  corresponding \n"
               "\t\t to  one  of the exceptional return codes (see tcl.h for \n"
               "\t\t the  definitions  of  code  values).   If  the  varName \n"
               "\t\t argument  is  given,  then  it  gives  the  name  of  a \n"
               "\t\t variable; catch will set the value of the  variable  to \n"
               "\t\t the string returned from command (either a result or an \n"
               "\t\t error message). \n" ,
	   "\n"							/*  ---  cd   ---  */
           "\t cd ?dirName? \n"
               "\t\t Change the current working directory to dirName, or  to \n"
               "\t\t the   home   directory   (as   specified  in  the  HOME \n"
               "\t\t environment variable) if  dirName  is  not  given.   If \n"
               "\t\t dirName  starts  with  a tilde, then tilde-expansion is \n"
               "\t\t done as described for Tcl_TildeSubst.  Returns an empty \n"
               "\t\t string.   This command can potentially be disruptive to \n"
               "\t\t an  application,  so  it  may  be   removed   in   some \n"
               "\t\t applications. \n" ,
	   "\n"							/*  --- close ---  */
           "\t close fileId \n"
               "\t\t return value from a previous  invocation  of  the  open \n"
               "\t\t command;  after  this  command,  it  should not be used \n"
               "\t\t anymore.   If  fileId  refers  to  a  command  pipeline \n"
               "\t\t instead of a file, then close waits for the children to \n"
               "\t\t complete.  The normal result  of  this  command  is  an \n"
               "\t\t empty  string,  but  errors  are  returned if there are \n"
               "\t\t problems in closing the file or waiting for children to \n"
               "\t\t complete. \n" ,
	   "\n"							/*  --- concat---  */
           "\t concat arg ?arg ...? \n"
               "\t\t This  command  treats  each  argument  as  a  list  and \n"
               "\t\t concatenates  them  into a single list.  It permits any \n"
               "\t\t number of arguments.  For example, the command \n"
	   "\n" 
               "\t\t\t concat a b {c d e} {f {g h}} \n"
	   "\n" 
               "\t\t will return \n"
	   "\n" 
               "\t\t\t a b c d e f {g h} \n"
               "\t\t as its result. \n" ,
	   "\n"							/*  -- continue -- */
           "\t continue \n"
               "\t\t This command may be invoked only inside the body  of  a \n"
               "\t\t loop  command  such  as  for  or  foreach or while.  It \n"
               "\t\t returns a  TCL_CONTINUE code to  signal  the  innermost \n"
               "\t\t containing  loop  command  to skip the remainder of the \n"
               "\t\t loop's body but continue with the next iteration of the \n"
               "\t\t loop. \n" ,
	   "\n"							/*  --- echo ---   */
           "\t echo arg ? arg.. ?\n"
	   "\t echo $arg \n"
	       "\t\t This command returns the args string  as entered by \n"
	       "\t\t the user in the first syntax. In the second syntax  \n"
	       "\t\t variable substitution is performed and the value of \n"
	       "\t\t a defined variable is returned. \n" ,
	   "\n"							/*  --- eof  ---   */
           "\t eof fileId \n"
               "\t\t Returns 1 if an end-of-file condition has  occurred  on \n"
               "\t\t fileId,  0 otherwise.  FileId must have been the return \n"
               "\t\t value from a previous call to open, or it may be stdin, \n"
               "\t\t stdout,  or  stderr to refer to one of the standard I/O \n"
               "\t\t channels. \n" ,
	   "\n"							/*  --- error ---  */
           "\t error message ?info? ?code? \n"
               "\t\t Returns  a  TCL_ERROR  code,   which   causes   command \n"
               "\t\t interpretation to be unwound.  Message is a string that \n"
               "\t\t is returned to the application to  indicate  what  went \n"
               "\t\t wrong. \n"
	       "\n" 
               "\t\t If the info argument is provided and is  non-empty,  it \n"
               "\t\t is  used  to  initialize the global variable errorInfo. \n"
               "\t\t errorInfo is used to accumulate a stack trace  of  what \n"
               "\t\t was  in  progress  when  an  error  occurred; as nested \n"
               "\t\t commands unwind, the Tcl interpreter  adds  information \n"
               "\t\t to  errorInfo.   If the info argument is present, it is \n"
               "\t\t used to initialize errorInfo and the first increment of \n"
               "\t\t unwind  information  will  not  be  added  by  the  Tcl \n"
               "\t\t interpreter.  In other words,  the  command  containing \n"
               "\t\t the  error command will not appear in errorInfo; in its \n"
               "\t\t place will be info.  This feature  is  most  useful  in \n"
               "\t\t conjunction  with  the catch command: if a caught error \n"
               "\t\t cannot be handled successfully, info  can  be  used  to \n"
               "\t\t return  a  stack trace reflecting the original point of \n"
               "\t\t occurrence of the error. \n"
	       "\t\t For a full description of the command usage please refer  \n"
	       "\t\t to tcl man pages. \n" ,
 	       "\n"
	   "\n"							/*  --- eval ---   */
           "\t eval arg ?arg ...? \n"
               "\t\t Eval  takes  one  or  more  arguments,  which  together \n"
               "\t\t comprise  a  Tcl command (or collection of Tcl commands \n"
               "\t\t separated  by  newlines  in  the  usual   way).    Eval \n"
               "\t\t concatenates  all  its arguments in the same fashion as \n"
               "\t\t the concat command, passes the concatenated  string  to \n"
               "\t\t the Tcl interpreter recursively, and returns the result \n"
               "\t\t of that evaluation (or any error generated by it). \n" ,
	   "\n"							/*  --- exit ---   */
           "\t exit ?returnCode? \n"
               "\t\t Terminate  the  process,  returning  returnCode  to the \n"
               "\t\t parent  as  the  exit  status.   If  returnCode   isn't \n"
               "\t\t specified then it defaults to 0. \n" ,
	   "\n"							/*  --- expr ---   */
           "\t expr arg \n"
               "\t\t Calls the expression processor  to  evaluate  arg,  and \n"
               "\t\t returns  the  result  as  a  string.   See  the section \n"
               "\t\t EXPRESSIONS in tcl Man Pages for a detailed description \n" ,
	   "\n"							/*  --- file ---   */
           "\t file option name ?arg arg ...? \n"
               "\t\t Operate on a file or a file name.  Name is the name  of \n"
               "\t\t a   file;  if  it  starts  with  a  tilde,  then  tilde \n"
               "\t\t substitution is done before executing the command  (see \n"
               "\t\t the  manual  entry  for  Tcl_TildeSubst  for  details). \n"
               "\t\t Option indicates what to do with the  file  name.   Any \n"
               "\t\t unique  abbreviation  for  option  is  acceptable.   \n"
	       "\t\t For a full description of all the options please refer  \n"
	       "\t\t to tcl man pages. \n"
	       "\n"
	       "\t\t Under VxWorks only the following options are valid:  \n"
	       "\n"
	       "\t\t\t 	dirname, extension, root or tail \n"
	       "\n"
	       "\t\t Under UNIX, the following options are defined: \n"
	       "\n"
	       "\t\t\t atime   dirname   executable  exists   extensions \n"
	       "\t\t\t isdirectory  isfile  mtime  owned  readable \n"
	       "\t\t\t rootname  size  stat  tail  writable \n" ,
	   "\n"							/*  --- flush ---  */
           "\t flush fileId \n"
               "\t\t Flushes any output that has been buffered  for  fileId. \n"
               "\t\t FileId  must have been the return value from a previous \n"
               "\t\t call to open, or it may be stdout or stderr  to  access \n"
               "\t\t one  of  the  standard  I/O streams; it must refer to a \n"
               "\t\t file that was opened for writing.  This command returns \n"
               "\t\t an empty string. \n" ,
	   "\n"							/*  ---  for  ---  */
           "\t for start test next body \n"
               "\t\t For is a looping command, similar in structure to the C \n"
               "\t\t for  statement.   The  start,  next, and body arguments \n"
               "\t\t must be Tcl command strings, and test is an  expression \n"
               "\t\t string.    The   for  command  first  invokes  the  Tcl \n"
               "\t\t interpreter  to  execute  start.   Then  it  repeatedly \n"
               "\t\t evaluates  test as an expression; if the result is non- \n"
               "\t\t zero it invokes  the  Tcl  interpreter  on  body,  then \n"
               "\t\t invokes  the  Tcl interpreter on next, then repeats the \n"
               "\t\t loop.  The command terminates when test evaluates to 0. \n"
               "\t\t If  a  continue command is invoked within body then any \n"
               "\t\t remaining commands in the current execution of body are \n"
               "\t\t skipped;  processing  continues  by  invoking  the  Tcl \n"
               "\t\t interpreter on next, then evaluating test, and  so  on. \n"
               "\t\t If a break command is invoked within body or next, then \n"
               "\t\t the for command will return immediately.  The operation \n"
               "\t\t of  break and continue are similar to the corresponding \n"
               "\t\t statements in C.  For returns an empty string. \n" ,
	   "\n"							/*  -- foreach --  */
           "\t foreach varname list body \n"
               "\t\t In this command, varname is the  name  of  a  variable, \n"
               "\t\t list is a list of values to assign to varname, and body \n"
               "\t\t is a collection of Tcl commands.   For  each  field  in \n"
               "\t\t list (in order from left to right), foreach assigns the \n"
               "\t\t contents of the field to  varname  (as  if  the  lindex \n"
               "\t\t command had been used to extract the field), then calls \n"
               "\t\t the Tcl interpreter to execute  body.   The  break  and \n"
               "\t\t continue  statements  may  be invoked inside body, with \n"
               "\t\t the same effect as in  the  for  command.   Foreach  an \n"
               "\t\t empty string. \n" ,
	   "\n"							/*  --- format --- */
           "\t format formatString ?arg arg ...? \n"
               "\t\t This command generates a formatted string in  the  same \n"
               "\t\t way  as the C sprintf procedure (it uses sprintf in its \n"
               "\t\t implementation).  FormatString indicates how to  format \n"
               "\t\t the  result,  using  %  fields  as  in sprintf, and the \n"
               "\t\t additional arguments, if  any,  provide  values  to  be \n"
               "\t\t substituted  into  the  result.   All  of  the  sprintf \n"
               "\t\t options  are  valid;  see  the  sprintf  man  page  for \n"
               "\t\t details.   Each  arg  must match the expected type from \n"
               "\t\t the  %  field  in  formatString;  the  format   command \n"
               "\t\t converts  each  argument to the correct type (floating, \n"
               "\t\t integer,  etc.)  before  passing  it  to  sprintf   for \n"
               "\t\t formatting.   The only unusual conversion is for %c; in \n"
               "\t\t this case the argument must be a decimal string,  which \n"
               "\t\t will  then  be  converted  to  the  corresponding ASCII \n"
               "\t\t character value.  Format does backslash substitution on \n"
               "\t\t its  formatString  argument,  so backslash sequences in \n"
               "\t\t formatString will be  handled  correctly  even  if  the \n"
               "\t\t argument is in braces.  The return value from format is \n"
               "\t\t the formatted string. \n" ,
	   "\n"							/*  --- gets ---   */
           "\t gets fileId ?varName? \n"
               "\t\t Reads the next line from the file given by  fileId  and \n"
               "\t\t discards the terminating newline character.  If varName \n"
               "\t\t is specified, then the line is placed in  the  variable \n"
               "\t\t by  that  name  and  the return value is a count of the \n"
               "\t\t number of characters read (not including the  newline). \n"
               "\t\t If  the  end  of the file is reached before reading any \n"
               "\t\t characters then -1 is returned and varName is set to an \n"
               "\t\t empty  string.   If  varName  is not specified then the \n"
               "\t\t return value  will  be  the  line  (minus  the  newline \n"
               "\t\t character) or an empty string if the end of the file is \n"
               "\t\t reached before reading any characters.  An empty string \n"
               "\t\t will  also be returned if a line contains no characters \n"
               "\t\t except the newline, so eof  may  have  to  be  used  to \n"
               "\t\t determine  what really happened.  If the last character \n"
               "\t\t in the file is  not  a  newline  character,  then  gets \n"
               "\t\t behaves   as   if  there  were  an  additional  newline \n"
               "\t\t character at the end of the file.  FileId must be stdin \n"
               "\t\t or  the  return  value from a previous call to open; it \n"
               "\t\t must refer to a file that was opened for reading. \n" ,
	   "\n"							/*  -- global --   */
           "\t global varname ?varname ...? \n"
               "\t\t This command is ignored unless a Tcl procedure is being \n"
               "\t\t interpreted.    If  so,  then  it  declares  the  given \n"
               "\t\t varname's to be  global  variables  rather  than  local \n"
               "\t\t ones.   For  the duration of the current procedure (and \n"
               "\t\t only while executing in  the  current  procedure),  any \n"
               "\t\t reference  to  any  of  the varnames will be bound to a \n"
               "\t\t global variable instead of a local one. \n" ,
	   "\n"							/*  -- history --  */
           "\t history ?option? ?arg arg ...? \n"
               "\t\t Note:  this command may not be available  in  all  Tcl- \n"
               "\t\t based applications.  Typically, only those that receive \n"
               "\t\t command  input  in  a  typescript  form  will   support \n"
               "\t\t history.   The  history command performs one of several \n"
               "\t\t operations  related   to   recently-executed   commands \n"
               "\t\t recorded  in  a  history  list.  \n"
	       "\t\t For a detailed description please refer to tcl man pages\n" ,
	   "\n"							/*  ---  if  ---   */
           "\t if test ?then? trueBody ?else? ?falseBody? \n"
               "\t\t The if command evaluates test as an expression (in  the \n"
               "\t\t same  way that expr evaluates its argument).  The value \n"
               "\t\t of the expression must be numeric; if  it  is  non-zero \n"
               "\t\t then  trueBody  is  called  by  passing  it  to the Tcl \n"
               "\t\t interpreter.   Otherwise  falseBody  is   executed   by \n"
               "\t\t passing  it  to the Tcl interpreter.  The then and else \n"
               "\t\t arguments are optional  ``noise  words''  to  make  the \n"
               "\t\t command easier to read.  FalseBody is also optional; if \n"
               "\t\t it isn't specified then the  command  does  nothing  if \n"
               "\t\t test  evaluates  to  zero.  The return value from if is \n"
               "\t\t the value of the last command executed in  trueBody  or \n"
               "\t\t falseBody,  or  the  empty  string if test evaluates to \n"
               "\t\t zero and falseBody isn't specified. \n" ,
	   "\n"							/*  --- incr ---   */
           "\t incr varName ?increment? \n"
               "\t\t Increment the value stored in the variable  whose  name \n"
               "\t\t is   varName.   The  value  of  the  variable  must  be \n"
               "\t\t integral.  If increment  is  supplied  then  its  value \n"
               "\t\t (which  must  be  an  integer) is added to the value of \n"
               "\t\t variable varName;  otherwise 1  is  added  to  varName. \n"
               "\t\t The new value is stored as a decimal string in variable \n"
               "\t\t varName and also returned as result. \n" ,
	   "\n"							/*  --- info ---   */
           "\t info option ?arg arg ...? \n"
               "\t\t Provide information about various internals to the  Tcl \n"
               "\t\t interpreter.  For all the legal options please refer to \n"
	       "\t\t tcl man pages. \n",
	   "\n"							/*  --- join ---   */
           "\t join list ?joinString? \n"
               "\t\t The list argument must  be  a  valid  Tcl  list.   This \n"
               "\t\t command returns the string formed by joining all of the \n"
               "\t\t elements of list together  with  joinString  separating \n"
               "\t\t each   adjacent   pair  of  elements.   The  joinString \n"
               "\t\t argument defaults to a space character. \n" ,
	   "\n"							/*  -- lappend --  */
           "\t lappend varName value ?value value ...? \n"
               "\t\t Treat the variable given  by  varName  as  a  list  and \n"
               "\t\t append  each  of  the value arguments to that list as a \n"
               "\t\t separate element, with  spaces  between  elements.   If \n"
               "\t\t varName  doesn't  exist,  it  is created as a list with \n"
               "\t\t elements given by  the  value  arguments.   Lappend  is \n"
               "\t\t similar  to  append except that the values are appended \n"
               "\t\t as list elements rather than raw  text.   This  command \n"
               "\t\t provides  a  relatively efficient way to build up large \n"
               "\t\t lists.  For example, ``lappend  a  $b''  is  much  more \n"
               "\t\t efficient  than ``set a [concat $a [list $b]]'' when $a \n"
               "\t\t is long. \n" ,
 	   "\n"							/*  -- lindex --   */
           "\t lindex list index \n"
               "\t\t Treats  list  as  a  Tcl  list and returns the index'th \n"
               "\t\t element from it (0 refers to the first element  of  the \n"
               "\t\t list).   In extracting the element, lindex observes the \n"
               "\t\t same rules concerning braces and quotes and backslashes \n"
               "\t\t as  the  Tcl  command  interpreter;  however,  variable \n"
               "\t\t substitution and command substitution do not occur.  If \n"
               "\t\t index  is  negative  or  greater  than  or equal to the \n"
               "\t\t number of elements in value, then an  empty  string  is \n"
               "\t\t returned. \n" ,
	   "\n"							/*  -- linsert --  */
           "\t linsert list index element ?element element ...? \n"
               "\t\t This command produces a new list from list by inserting \n"
               "\t\t all of the element arguments just  before  the  indexth \n"
               "\t\t element  of  list.  Each element argument will become a \n"
               "\t\t separate element of the new list.   If  index  is  less \n"
               "\t\t than  or  equal  to  zero,  then  the  new elements are \n"
               "\t\t inserted at the beginning of the  list.   If  index  is \n"
               "\t\t greater  than or equal to the number of elements in the \n"
               "\t\t list, then the new elements are appended to the list. \n" ,
	   "\n"							/*  --- list ---   */
           "\t list arg ?arg ...? \n"
               "\t\t This command returns a list comprised of all the  args. \n"
               "\t\t Braces  and backslashes get added as necessary, so that \n"
               "\t\t the index command may be used  on  the  result  to  re- \n"
               "\t\t extract  the  original arguments, and also so that eval \n"
               "\t\t may be used to execute the resulting  list,  with  arg1 \n"
               "\t\t comprising  the  command's  name  and  the  other  args \n"
               "\t\t comprising  its  arguments.   List  produces   slightly \n"
               "\t\t different  results  than  concat:   concat  removes one \n"
               "\t\t level of grouping before forming the list,  while  list \n"
               "\t\t works   directly  from  the  original  arguments.   For \n"
               "\t\t example, the command \n"
	       "\n" 
                    "\t\t\t list a b {c d e} {f {g h}} \n"
	       "\n" 
               "\t\t will return \n"
	       "\n" 
                    "\t\t\t a b {c d e} {f {g h}} \n"
               "\t\t while concat with the same arguments will return \n"
	       "\n" 
                    "\t\t\t a b c d e f {g h} \n" ,
	   "\n"							/*  -- llength --  */
           "\t llength list \n"
               "\t\t Treats  list  as  a  list  and returns a decimal string \n"
               "\t\t giving the number of elements in it. \n" ,
	   "\n"							/*  --- lrange --- */
           "\t lrange list first last \n"
               "\t\t List  must  be  a  valid  Tcl  list.  This command will \n"
               "\t\t return a new list consisting of elements first  through \n"
               "\t\t last,  inclusive.  Last may be end (or any abbreviation \n"
               "\t\t of it) to refer to the last element of  the  list.   If \n"
               "\t\t first  is  less  than zero, it is treated as if it were \n"
               "\t\t zero.  If last is greater than or equal to  the  number \n"
               "\t\t of  elements  in  the list, then it is treated as if it \n"
               "\t\t were end.  If first is greater than last then an  empty \n"
               "\t\t string  is returned.  Note: ``lrange list first first'' \n"
               "\t\t does not always produce the  same  result  as  ``lindex \n"
               "\t\t list  first'' (although it often does for simple fields \n"
               "\t\t that aren't enclosed  in  braces);  it  does,  however, \n"
               "\t\t produce exactly the same results as ``list [lindex list \n"
               "\t\t first]'' \n" ,
	   "\n"							/*  -- lreplace -- */
           "\t lreplace list first last ?element element ...? \n"
               "\t\t Returns  a  new  list  formed  by replacing one or more \n"
               "\t\t elements of list with  the  element  arguments.   First \n"
               "\t\t gives  the  index  in  list  of the first element to be \n"
               "\t\t replaced.  If first is less than zero then it refers to \n"
               "\t\t the  first  element  of list;  the element indicated by \n"
               "\t\t first must exist in the list.  Last gives the index  in \n"
               "\t\t list  of  the  last element to be replaced;  it must be \n"
               "\t\t greater than or equal to first.  Last may  be  end  (or \n"
               "\t\t any  abbreviation  of it) to indicate that all elements \n"
               "\t\t between first  and  the  end  of  the  list  should  be \n"
               "\t\t replaced.   The  element arguments specify zero or more \n"
               "\t\t new arguments to be added to the list in place of those \n"
               "\t\t that were deleted.  Each element argument will become a \n"
               "\t\t separate element of the list.  If no element  arguments \n"
               "\t\t are specified, then the elements between first and last \n"
               "\t\t are simply deleted. \n" ,
	   "\n"							/*  -- lsearch --  */
           "\t lsearch list pattern \n"
               "\t\t Search  the  elements  of  list  to  see if one of them \n"
               "\t\t matches pattern.  If so, the command returns the  index \n"
               "\t\t of  the  first  matching  element.  If not, the command \n"
               "\t\t returns -1.  Pattern matching is done in the  same  way \n"
               "\t\t as for the string match command. \n" ,
	   "\n"							/*  --- lsort ---  */
           "\t lsort list \n"
               "\t\t Sort  the  elements  of  list,  returning a new list in \n"
               "\t\t sorted order.  ASCII sorting is used, with  the  result \n"
               "\t\t in increasing order. \n" ,
	   "\n"							/*  --- open ---   */
           "\t open fileName ?access? \n"
               "\t\t Opens a file and returns an identifier that may be used \n"
               "\t\t in future invocations of commands like read, write, and \n"
               "\t\t close.  FileName gives the name of the file to open. \n"
	       "\t\t For a full description please refer to tcl man pages \n"	         
               "\t\t The access argument indicates the way the file is to be \n"
               "\t\t accessed. It may have any of the following values: \n"
	       "\n"
               "\t\t r  \t Read   only; the file must exist \n"
               "\t\t r+ \t Read and Write; the file must exist \n"
               "\t\t w  \t Write only; it creates a new file if it does not exist. \n"
               "\t\t w+ \t Read and Write; it creates a new file if it does not exist. \n"
               "\t\t a  \t Write only. The file must exist. New data is appended. \n"
               "\t\t a+ \t Read and Write. The file must exist and is set to the EOF \n" ,
	   "\n"							/*  --- proc ---   */
           "\t proc name args body \n"
               "\t\t The proc command creates a new Tcl  command  procedure, \n"
               "\t\t name,  replacing  any  existing  command there may have \n"
               "\t\t been  by  that  name.   Whenever  the  new  command  is \n"
               "\t\t invoked,  the  contents of body will be executed by the \n"
               "\t\t Tcl interpreter.  Args specifies the  formal  arguments \n"
               "\t\t to  the  procedure.   It  consists  of a list, possibly \n"
               "\t\t empty, each of whose elements specifies  one  argument. \n"
               "\t\t Each  argument specifier is also a list with either one \n"
               "\t\t or two fields.  If there is only a single field in  the \n"
               "\t\t specifier,  then  it  is  the  name of the argument; if \n"
               "\t\t there are two fields, then the first  is  the  argument \n"
               "\t\t name  and  the second is its default value.  braces and \n"
               "\t\t backslashes may be used in the  usual  way  to  specify \n"
               "\t\t complex default values. \n"
	       "\n"
	       "\t\t For a full description of the command proc please refer \n"
	       "\t\t the tcl man pages.  \n" ,
	   "\n"							/*  --- puts ---   */
           "\t puts fileId string ?nonewline? \n"
               "\t\t Writes the characters given by string to the file given \n"
               "\t\t by  fileId.   Puts normally outputs a newline character \n"
               "\t\t after string, but this feature  may  be  suppressed  by \n"
               "\t\t specifying  the nonewline argument.  Output to files is \n"
               "\t\t buffered internally by Tcl; the flush  command  may  be \n"
               "\t\t used to force buffered characters to be output.  FileId \n"
               "\t\t must have been the return value from a previous call to \n"
               "\t\t open,  or it may be stdout or stderr to refer to one of \n"
               "\t\t the standard I/O channels; it must refer to a file that \n"
               "\t\t was opened for writing. \n" ,
	   "\n"							/*  ---  pwd ---   */
           "\t pwd \n"
               "\t\t Returns the path name of the current working directory. \n" ,
	   "\n"							/*  --- read ---   */
           "\t read fileId \n"
	   "\n" 
           "\t read fileId nonewline \n"
	   "\n" 
           "\t read fileId numBytes \n"
               "\t\t In  the first form, all of the remaining bytes are read \n"
	       "\t\t from the file given by fileId; they are returned as the \n"
               "\t\t result of the command.  If nonewline is specified as an \n"
               "\t\t additional argument, then the  last  character  of  the \n"
               "\t\t file  is  discarded  if  it is a newline.  In the third \n"
               "\t\t form, the extra argument specifies how  many  bytes  to \n"
               "\t\t read;   exactly  this  many  bytes  will  be  read  and \n"
               "\t\t returned, unless there are fewer  than  numBytes  bytes \n"
               "\t\t left in the file; in this case, all the remaining bytes \n"
               "\t\t are returned.  FileId must be stdin or the return value \n"
               "\t\t from  a  previous call to open; it must refer to a file \n"
               "\t\t that was opened for reading. \n" ,
	   "\n"							/*  -- regexp --   */
           "\t regexp ?-indices? ?-nocase? exp string ?matchVar? ?subMatchVar...? \n"
               "\t\t Determines whether the regular expression  exp  matches \n"
               "\t\t part or all of string and returns 1 if it does, 0 if it \n"
               "\t\t doesn't.  See REGULAR EXPRESSIONS  on tcl man pages for \n"
	       "\t\t complete information on the syntax of exp and how it is \n"
	       "\t\t matched against string.\n"
	        "\t\t For a full  description of  regexp please refer to tcl \n"	       
	       "\t\t man pages \n" ,	       
	   "\n"							/*  -- regsub --   */
           "\t regsub ?-all? ?-nocase? exp string subSpec varName \n"
	       "\t\t This command matches the regular expression exp against \n"
               "\t\t string using the rules described in REGULAR EXPRESSIONS \n"
               "\t\t above.   If there is no match, then the command returns \n"
               "\t\t 0 and does nothing else.  If there is a match, then the \n"
               "\t\t command  returns  1  and  also  copies  string  to  the \n"
               "\t\t variable whose name is given by varName.  When  copying \n"
	        "\t\t For a full  description of  regexp please refer to tcl \n"	       
	       "\t\t man pages \n" ,	       
	   "\n"							/*  -- rename --   */
           "\t rename oldName newName \n"
               "\t\t Rename the command that used to be  called  oldName  so \n"
               "\t\t that  it is now called newName.  If newName is an empty \n"
               "\t\t string (e.g. {}) then oldName is deleted.   The  rename \n"
               "\t\t command returns an empty string as result. \n" ,
	   "\n"							/*  -- return --   */
           "\t return ?value? \n"
               "\t\t Return immediately from the current procedure (or  top- \n"
               "\t\t level  command  or  source  command), with value as the \n"
               "\t\t return value.  If value  is  not  specified,  an  empty \n"
               "\t\t string will be returned as result. \n" ,
	   "\n"							/*  --- scan ---   */
           "\t scan string format varname1 ?varname2 ...? \n"
               "\t\t This command parses fields from an input string in  the \n"
               "\t\t same  fashion  as the C sscanf procedure.  String gives \n"
               "\t\t the input to be parsed  and  format  indicates  how  to \n"
               "\t\t parse  it,  using  %  fields  as in sscanf.  All of the \n"
               "\t\t sscanf options are valid; see the sscanf man  page  for \n"
               "\t\t details.   Each  varname  gives the name of a variable; \n"
               "\t\t when a field is scanned  from  string,  the  result  is \n"
               "\t\t converted  back  into  a  string  and  assigned  to the \n"
               "\t\t corresponding varname.  The only unusual conversion  is \n"
               "\t\t for %c.  For %c conversions a single character value is \n"
               "\t\t converted to a decimal string, which is  then  assigned \n"
               "\t\t to  the  corresponding  varname;  no field width may be \n"
               "\t\t specified for this conversion. \n" ,
	   "\n"							/*  --- seek ---   */
           "\t seek fileId offset ?origin? \n"
               "\t\t Change  the  current  access  position for fileId.  The \n"
               "\t\t offset and origin arguments  specify  the  position  at \n"
               "\t\t which  the  next  read  or write will occur for fileId. \n"
               "\t\t Offset must be a number (which  may  be  negative)  and \n"
               "\t\t origin must be one of the following: \n"
	       "\n" 
               "\t\t start \n"
                    "\t\t\t The  new access position will be origin bytes from \n"
                    "\t\t\t the start of the file. \n"
               "\t\t current \n"
                    "\t\t\t The  new access position will be origin bytes from \n"
                    "\t\t\t the current access  position;  a  negative  origin \n"
                    "\t\t\t moves the access position backwards in the file. \n"
               "\t\t end \n"
                    "\t\t\t The  new access position will be origin bytes from \n"
                    "\t\t\t the end of the file.  A negative origin places the \n"
                    "\t\t\t access  position  before  the  end-of-file,  and a \n"
                    "\t\t\t positive origin places the access  position  after \n"
                    "\t\t\t the end-of-file. \n"
	       "\n" 
               "\t\t The origin argument defaults  to  start.   FileId  must \n"
               "\t\t have  been  the  return  value  from a previous call to \n"
               "\t\t open, or it may be stdin, stdout, or stderr to refer to \n"
               "\t\t one of the standard I/O channels.  This command returns \n"
               "\t\t an empty string. \n" ,
	   "\n"							/*  ---  set ---   */
           "\t set varname ?value? \n"
               "\t\t Returns the value of variable  varname.   If  value  is \n"
               "\t\t specified,  then  set  the  value  of varname to value, \n"
               "\t\t creating a new variable if one doesn't  already  exist, \n"
               "\t\t and  return  its  value.   If  varName contains an open \n"
               "\t\t parenthesis and ends with a close parenthesis, then  it \n"
               "\t\t refers  to an array element:  the characters before the \n"
               "\t\t open parenthesis are the name of  the  array,  and  the \n"
               "\t\t characters between the parentheses are the index within \n"
               "\t\t the  array.   Otherwise  varName  refers  to  a  scalar \n"
               "\t\t variable.   If  no  procedure  is  active, then varname \n"
               "\t\t refers to a global variable.  If a procedure is active, \n"
               "\t\t then varname refers to a parameter or local variable of \n"
               "\t\t the procedure,  unless  the  global  command  has  been \n"
               "\t\t invoked to declare varname to be global. \n" ,
	   "\n"							/*  -- source --   */
           "\t source fileName \n"
               "\t\t Read file fileName and pass the  contents  to  the  Tcl \n"
               "\t\t interpreter as a sequence of commands to execute in the \n"
               "\t\t normal fashion.  The return  value  of  source  is  the \n"
               "\t\t return  value  of  the  last  command executed from the \n"
               "\t\t file.  If an error occurs in executing the contents  of \n"
               "\t\t the  file,  then  the  source  command will return that \n"
               "\t\t error.  If a return command is invoked from within  the \n"
               "\t\t file, the remainder of the file will be skipped and the \n"
               "\t\t source command will return  normally  with  the  result \n"
               "\t\t from  the  return  command.   If fileName starts with a \n"
               "\t\t tilde, then it is tilde-substituted as described in the \n"
               "\t\t Tcl_TildeSubst manual entry. \n" ,
	   "\n"							/*  --- split ---  */
           "\t split string ?splitChars? \n"
               "\t\t Returns a list created  by  splitting  string  at  each \n"
               "\t\t character  that  is  in  the splitChars argument.  Each \n"
               "\t\t element  of  the  result  list  will  consist  of   the \n"
               "\t\t characters   from   string  between  instances  of  the \n"
               "\t\t characters in splitChars.  Empty list elements will  be \n"
               "\t\t generated  if  string  contains  adjacent characters in \n"
               "\t\t splitChars, or if the first or last character of string \n"
               "\t\t is  in  splitChars.   If  splitChars is an empty string \n"
               "\t\t then  each  character  of  string  becomes  a  separate \n"
               "\t\t element of the result list.  SplitChars defaults to the \n"
               "\t\t standard white-space characters.  For example, \n"
	       "\n" 
                    "\t\t\t split \"comp.unix.misc\" . \n"
	       "\n" 
               "\t\t returns \"comp unix misc\" and \n"
	       "\n" 
                    "\t\t\t split \"Hello world\" {} \n"
	       "\n" 
               "\t\t returns \"H e l l o { } w o r l d\". \n" ,
	   "\n"							/*  -- string --   */
           "\t string option arg ?arg ...? \n"
	       "\t\t Perform one of several string operations, depending  on \n"
               "\t\t option.   For a  full  description of  all the  options \n"
	       "\t\t please refer to tcl man pages. An example follows : \n"
               "\n" 
               "\t\t string compare string1 string2 \n"
                    "\t\t\t Perform  a  character-by-character  comparison  of \n"
                    "\t\t\t strings string1 and string2 in the same way as the \n"
                    "\t\t\t C strcmp procedure.  Return -1, 0, or 1, depending \n"
                    "\t\t\t on whether string1 is lexicographically less than, \n"
                    "\t\t\t equal to, or greater than string2. \n" ,
	   "\n"							/*  --- tell ---   */
           "\t tell fileId \n"
               "\t\t Returns  a  decimal  string  giving  the current access \n"
               "\t\t position in fileId.  FileId must have been  the  return \n"
               "\t\t value from a previous call to open, or it may be stdin, \n"
               "\t\t stdout, or stderr to refer to one of the  standard  I/O \n"
               "\t\t channels. \n" ,
	   "\n"							/*  --- trace ---  */
           "\t trace option ?arg arg ...? \n"
               "\t\t Cause Tcl commands  to  be  executed  whenever  certain \n"
	       "\t\t operations  are  invoked.   At  present,  only variable \n"
               "\t\t tracing is implemented.  \n"
	       "\t\t For a full description, please refer to tcl man pages \n" ,
	   "\n"							/*  --- unset ---  */
           "\t unset name ?name name ...? \n"
               "\t\t Remove  one or more variables.  Each name is a variable \n"
               "\t\t name, specified in any of the ways  acceptable  to  the \n"
               "\t\t set  command.   If  a  name  refers to an element of an \n"
               "\t\t array, then that element is removed  without  affecting \n"
               "\t\t the  rest of the array.  If a name consists of an array \n"
               "\t\t name with no parenthesized index, then the entire array \n"
               "\t\t is  deleted.  The unset command returns an empty string \n"
               "\t\t as result.  An error occurs if  any  of  the  variables \n"
               "\t\t doesn't exist, or if any of the variables has an active \n"
               "\t\t trace. \n" ,
	   "\n"							/*  -- uplevel --  */
           "\t uplevel ?level? command ?command ...? \n"
	       "\t\t All of the command arguments  are  concatenated  as  if \n"
               "\t\t they  had  been  passed  to  concat; the result is then \n"
               "\t\t evaluated in the variable context indicated  by  level. \n"
               "\t\t Uplevel  returns  the  result  of  that evaluation.  If \n"
               "\t\t level is an integer, then it gives a distance  (up  the \n"
               "\t\t procedure  calling  stack) to move before executing the \n"
               "\t\t command.  If level consists of # followed by  a  number \n"
               "\t\t then  the  number  gives  an absolute level number.  If \n"
               "\t\t level is omitted then it defaults to 1.   Level  cannot \n"
               "\t\t be  defaulted if the first command argument starts with \n"
               "\t\t a digit or #.   \n"
	       "\t\t For a full description, please refer to tcl man pages \n" ,
	   "\n"							/*  --- upvar ---  */
           "\t upvar ?level? otherVar myVar ?otherVar myVar ...? \n"
               "\t\t This command arranges for one or more  local  variables \n"
               "\t\t in  the  current  procedure to refer to variables in an \n"
               "\t\t enclosing procedure call or to global variables.  Level \n"
               "\t\t may  have  any  of  the forms permitted for the uplevel \n"
               "\t\t command, and may be omitted if the first letter of  the \n"
               "\t\t first  otherVar  isn't # or a digit (it defaults to 1). \n"
               "\t\t For each otherVar argument, upvar makes the variable by \n"
               "\t\t that  name in the procedure frame given by level (or at \n"
               "\t\t global level, if level is #0) accessible in the current \n"
               "\t\t procedure  by the name given in the corresponding myVar \n"
               "\t\t argument.  The variable  named  by  otherVar  need  not \n"
               "\t\t exist  at the time of the call;  it will be created the \n"
               "\t\t first time myVar is referenced, just like  an  ordinary \n"
               "\t\t variable.   Upvar  may  only  be  invoked  from  within \n"
               "\t\t procedures.  Neither otherVar or myVar may refer to  an \n"
               "\t\t element of an array.  Upvar returns an empty string. \n"
	       "\t\t For a full description, please refer to tcl man pages \n" ,
	   "\n"							/*  --- while ---  */
           "\t while test body \n"
               "\t\t The while command evaluates test as an  expression  (in \n"
               "\t\t the  same  way  that expr evaluates its argument).  The \n"
               "\t\t value of the expression must be numeric; if it is  non- \n"
               "\t\t zero  then  body  is  executed by passing it to the Tcl \n"
               "\t\t interpreter.  Once body has been executed then test  is \n"
               "\t\t evaluated   again,   and   the  process  repeats  until \n"
               "\t\t eventually test evaluates  to  a  zero  numeric  value. \n"
               "\t\t Continue  commands  may  be  executed  inside  body  to \n"
               "\t\t terminate the current iteration of the loop, and  break \n"
               "\t\t commands may be executed inside body to cause immediate \n"
               "\t\t termination of the while command.   The  while  command \n"
               "\t\t always returns an empty string. \n" ,
 	   "\n help \n"
               "\t\t This is the online, not file-based help for ftcl. \n"
               "\t\t It allows to display text message on tcl commands. \n"
               "\n"
               "\t\t Typing help with NO arguments, a list of ALL the known \n"
               "\t\t facilities will be displayed. \n"
               "\n"
               "\t\t Typing  help ftcl a list of  all the built-in SPECIFIC \n"
               "\t\t tcl commands for which a help message is defined, will \n"
               "\t\t be listed. \n"
               "\t\t Available options: \n"
               "\t\t help * \n"
               "\t\t\t List ALL commands and full messages associated . \n"
               "\t\t help cmd \n"
               "\t\t\t Show the message text associated to cmd \n"
               "\t\t help cmd -full \n"
               "\t\t\t Show the message text associated to cmd and the facility\n"
	       "\t\t help -facil \n"
	       "\t\t\t List all available commands. \n"
               "\t\t help -facil=myfacility \n"
               "\t\t\t Only the commands with facility = myfacility will be listed \n"
               "\t\t help -search=pattern \n"
               "\t\t\t List all commands whose messages contain pattern \n"
               "\t\t help -search=pattern -full\n"
               "\t\t\t List all commands whose messages contain pattern ; the full\n"
               "\t\t\t text message will be displayed. \n" ,
	     "\n"
               "\t\t The following built-in ftcl commands are defined: \n"
               "\n"
               "\t\t append    array   break  case   cat   cd    close  concat\n"
               "\t\t continue  echo    eof    error  eval  exit  expr   file  \n"
               "\t\t flush     for     foreach       format      gets   global \n"
               "\t\t history   if      incr   info   join    lappend    lindex \n"
               "\t\t linsert   list    llenght       lrange  lreplace   lsearch \n"
               "\t\t lsort     open    proc   puts   pwd     read       regexp \n"
               "\t\t regsub    rename  return  scan  seek    set        source \n"
               "\t\t split     string  tell    trace unset   uplevel    upvar \n"
               "\t\t while \n" ,
 	   "\n ftclParams\n"
	       "\t\t C, FORTRAN and Tcl interfaces to read named configuration \n"
	       "\t\t parameters from a file. \n"
	       "\t\t There are four separate functions, one to open the configuration file,\n"
	       "\t\t one to read Integers, one to read Strings, and one to read Floating \n"
	       "\t\t Point Numbers. The functions return 0 on success, and 1 on failure. \n"
	       "\t\t For a detailed information on the C or FORTRAN interface please refer to \n"
	       "\t\t manpages doing a man on ftclParams.\n"
	       "\t\t The configuration file is a flat ASCII text file which can have lines \n"
	       "\t\t in a format like: \n"
	       "\n"
	       "\t\t 	# Parameter file example_par residinng in current_dir directory \n"
	       "\t\t    Value1	3	#example of integer  \n"
	       "\t\t    Value2	3.5	#example of floating \n"
	       "\t\t    Value3	three	#example of a string \n"
	       "\n"
	       "\t\t TCL interface.\n"
	       "\t\t Four commands are defined: inipar, getpari, getparf, getpars.\n"
	       "\t\t Values are returned in the interp->result string, with interp->error\n"
	       "\t\t returning TCL_ERROR or TCL_OK.\n"
	       "\n"
	       "\t\t inipar example_par current_dir\n"
	       "\t\t getparf Value2\n"
	       "\t\t getpari Value1\n"
	       "\t\t getpars Value3\n",
 	   "\n inipar\n"
	       "\t\t See ftclParams",
 	   "\n getparf\n"
	       "\t\t See ftclParams ",
 	   "\n getpari\n"
	       "\t\t See ftclParams ",
 	   "\n getpars\n"
	       "\t\t See ftclParams " };
/* =============================================================================== */

/* ftclX command help information -- added by C. Huang 1/5/1996 */

char *tclXhelp_cmd_list[] = {
	"alarm", "auto_commands", "auto_load", "auto_load_file",
	"auto_packages", "bsearch", "buildpackageindex", "catclose",
	"catgets", "catopen", "cequal", "cexpand", "chgrp", "chmod",
	"chown", "chroot", "cindex", "clength", "cmdtrace",
	"commandloop", "convert_lib", "convertclock", "copyfile",
	"crange", "csubstr", "ctoken", "ctype", "dirs", "dup", "echo",
	"edprocs", "execl", "fcntl", "flock", "fmathcmds", "fmtclock",
	"for_array_keys", "for_file", "for_recursive_glob", "fork",
	"frename", "fstat", "funlock", "getclock", "id", "infox",
	"initialize", "intersect", "intersect3", "intro", "keyedlists",
	"keyldel", "keylget", "keylkeys", "keylset", "kill", "lassign",
	"lempty", "lgets", "link", "lmatch", "loadlibindex", "loop",
	"lrmdups", "lvarcat", "lvarpop", "lvarpush", "max", "min",
	"mkdir", "nice", "packagelib", "pipe", "popd", "profile",
	"profrep", "pushd", "random", "read_file", "readdir",
	"recursive_glob", "replicate", "rmdir", "saveprocs",
	"scancontext", "scanfile", "scanmatch", "searchpath", "select",
	"server_open", "showproc", "signal", "sleep", "sync", "system",
	"times", "translit", "umask", "union", "unlink", "variables",
	"wait", "write_file"
};

char *tclXhelp_cmd_msg[] = {
    "\n"
    "\t alarm seconds \n"
        "\t\t Instructs  the  system  to send a SIGALRM signal in \n"
        "\t\t the specified number of seconds.  This is a  float- \n"
        "\t\t ing  point number, so fractions of a section may be \n"
        "\t\t specified.  If seconds is 0.0, any  previous  alarm \n"
        "\t\t request  is canceled.  Only one alarm at a time may \n"
        "\t\t be active; the command returns the number  of  sec- \n"
        "\t\t onds  left in the previous alarm.  On systems with- \n"
        "\t\t out the setitimer system call, seconds  is  rounded \n"
        "\t\t up to an even number of seconds. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t auto_commands  ?-loaders?   Lists the names of all known loadable \n"
        "\t\t procedures and commands procedures.  If  -loaders  is  specified, \n"
        "\t\t the  command  that will be executed to load the command will also \n"
        "\t\t be returned. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t auto_load ?command? \n"
        "\t\t Attempt  to  load  the specified command from a loadable library. \n"
        "\t\t loading the package containing the  procedure.   If  the  package \n"
        "\t\t indexes  have  not  been  loaded  for  all  package  libraries in \n"
        "\t\t auto_path, they will be loaded.  Out-of-date library indexes will \n"
        "\t\t be  rebuilt if they are writable.  The procedure returns 1 if the \n"
        "\t\t command was sucessfully loaded, or 0 if it was not. \n"
        "\n"
        "\t\t Duplicated package names are skipped,  the  first  package  of  a \n"
        "\t\t given  name  found  in  the path is loaded.  If the auto_path has \n"
        "\t\t changed since the last load, indexes will be reloaded  (duplicate \n"
        "\t\t packages will not be redefined). \n"
        "\n"
        "\t\t If  command is not specified, the indexes will be loaded, if they \n"
        "\t\t have not alreay been loaded or  if  the  auto_path  variable  has \n"
        "\t\t changed, but no command will be loaded. \n"
        "\n"
        "\t This  command  overrides the standard Tcl procedure of the \n"
        "\t same name. \n"
        "\n"
        "\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t auto_load_file file \n"
        "\t\t Source  a  file,  as  with  the  source  command,  except  search \n"
        "\t\t auto_path for the file. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t auto_packages ?-location? \n"
        "\t\t Returns a list of the names of all defined packages. If -location \n"
        "\t\t is specified, a list of pairs of package name and the .tlib  path \n"
        "\t\t name, offset and length of the package within the library. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t bsearch fileId key ?retvar? ?compare_proc? \n"
        "\t\t Search  an  opened  file fileId containing lines of \n"
        "\t\t text sorted into ascending order for a match.   Key \n"
        "\t\t contains  the string to match.  If retvar is speci- \n"
        "\t\t fied, then the line from the file  is  returned  in \n"
        "\t\t retvar, and the command returns 1 if key was found, \n"
        "\t\t and 0 if it wasn't.  If retvar is not specified  or \n"
        "\t\t is  a  null name, then the command returns the line \n"
        "\t\t that was found, or an empty string  if  key  wasn't \n"
        "\t\t found. \n"
        "\n"
        "\t\t By  default,  the  key is matched against the first \n"
        "\t\t white-space separated  field  in  each  line.   The \n"
        "\t\t field  is  treated  as  an  ASCII  string.  If com- \n"
        "\t\t pare_proc is specified, then it defines the name of \n"
        "\t\t a  Tcl procedure to evaluate against each line read \n"
        "\t\t from the sorted file during the  execution  of  the \n"
        "\t\t bsearch command.  Compare_proc takes two arguments, \n"
        "\t\t the key and a line extracted from  the  file.   The \n"
        "\t\t compare  routine  should  return a number less than \n"
        "\t\t zero if the key is less than the line, zero if  the \n"
        "\t\t key  matches  the line, or greater than zero if the \n"
        "\t\t key is greater than the line.   The  file  must  be \n"
        "\t\t sorted  in  ascending  order  according to the same \n"
        "\t\t criteria compare_proc uses to compare the key  with \n"
        "\t\t the line, or errouenous results will occur. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t buildpackageindex libfilelist \n"
        "\t\t Build  index  files  for  package  libraries.   The argument lib- \n"
        "\t\t filelist is a list of package libraries.  Each name must end with \n"
        "\t\t the suffix .tlib.  A corresponding .tndx file will be built.  The \n"
        "\t\t user must have write access  to  the  directory  containing  each \n"
        "\t\t library. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t catclose ?-fail|-nofail? cathandle \n"
        "\t\t Close  the  message catalog specified by cathandle. \n"
        "\t\t Normally, errors are ignored.  If -fail  is  speci- \n"
        "\t\t fied,  any  errors closing the message catalog file \n"
        "\t\t are returned.  The  option  -nofail  specifies  the \n"
        "\t\t default  behavior  of  not returning an error.  The \n"
        "\t\t use of -fail only makes sense if it was also speci- \n"
        "\t\t fied in the call to catopen. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t catgets catHandle setnum msgnum defaultstr \n"
        "\t\t Retrieve   a   message   form  a  message  catalog. \n"
        "\t\t CatHandle should be a Tcl  message  catalog  handle \n"
        "\t\t that  was  returned by catopen.  Setnum is the mes- \n"
        "\t\t sage set number, and msgnum is the message  number. \n"
        "\t\t If  the message catalog was not opened, or the mes- \n"
        "\t\t sage set or message number cannot  be  found,  then \n"
        "\t\t the default string, defaultstr, is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t catopen ?-fail|-nofail? catname \n"
        "\t\t Open  the  message  catalog catname.  This may be a \n"
        "\t\t relative path name, in which case the NLSPATH envi- \n"
        "\t\t ronment  variable  is  searched to find an absolute \n"
        "\t\t path to the message catalog.  A handle in the  form \n"
        "\t\t msgcatN is returned.  Normally, errors are ignored, \n"
        "\t\t and in the case of a failed call to catopen, a han- \n"
        "\t\t dle  is  returned  to  an unopened message catalog. \n"
        "\t\t (This handle may still be  passed  to  catgets  and \n"
        "\t\t catclose,  causing  catgets  to  simply  return the \n"
        "\t\t default string, as described above.  If  the  -fail \n"
        "\t\t option  is  specified,  an error is returned if the \n"
        "\t\t open  fails.   The  option  -nofail  specifies  the \n"
        "\t\t default  behavior  of  not  returning an error when \n"
        "\t\t catopen fails to open a specified message  catalog. \n"
        "\t\t If  the  handle  from a failed catopen is passed to \n"
        "\t\t catgets, the default string is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t cequal string1 string2 \n"
        "\t\t This  command  compares  two  strings for equality. \n"
        "\t\t Returns 1 if string1 and string2 are the  identical \n"
        "\t\t and 0 if they are not.  This command is a short-cut \n"
        "\t\t for string compare and  avoids  the  problems  with \n"
        "\t\t string expressions being treated unintentionally as \n"
        "\t\t numbers. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t cexpand string \n"
        "\t\t Expand  backslash  sequences  in  string  to  there \n"
        "\t\t actual characters.   No  other  substitution  takes \n"
        "\t\t place. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t chgrp group filelist \n"
        "\t\t Set  the group id of each file in the list filelist \n"
        "\t\t to group, which can be either a  group  name  or  a \n"
        "\t\t numeric group id. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t chmod mode filelist \n"
        "\t\t Set  permissions  of  each of the files in the list \n"
        "\t\t filelist to mode, where mode is an absolute numeric \n"
        "\t\t mode   or  symbolic  permissions  as  in  the  UNIX \n"
        "\t\t chmod(1) command.  To specify a mode as  octal,  it \n"
        "\t\t should be prefixed with a \"0\" (e.g. 0622). \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t chown owner|{owner group} filelist \n"
        "\t\t Set  owner  of  each  file  in the list filelist to \n"
        "\t\t owner, which can be a user name or numeric user id. \n"
        "\t\t If the first parameter is a list, then the owner is \n"
        "\t\t set to the first element of the list and the  group \n"
        "\t\t is set to the second element.  Group can be a group \n"
        "\t\t name or numeric group id.  If group is {}, then the \n"
        "\t\t file  group  will  be set to the login group of the \n"
        "\t\t specified user. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t chroot dirname \n"
        "\t\t Change  root  directory to dirname, by invoking the \n"
        "\t\t POSIX chroot(2) system  call.   This  command  only \n"
        "\t\t succeeds if running as root. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t cindex string indexExpr \n"
        "\t\t Returns  the  character  indexed  by the expression \n"
        "\t\t indexExpr (zero based) from string. \n"
        "\n"
        "\t\t If the expression indexExpr starts with the  string \n"
        "\t\t end,  then  end  is  replaced with the index of the \n"
        "\t\t last character in the string.   If  the  expression \n"
        "\t\t starts  with  len,  then  len  is replaced with the \n"
        "\t\t length of the string. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t clength string \n"
        "\t\t Returns  the  length of string in characters.  This \n"
        "\t\t command is a shortcut for: \n"
        "\n"
        "\t\t\t string length string \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t cmdtrace level|on ?noeval? ?notruncate? ?procs? ?fileid? \n"
        "\t\t Print  a  trace statement for all commands executed \n"
        "\t\t at depth of level or below (1 is  the  top  level). \n"
        "\t\t If  on  is specified, all commands at any level are \n"
        "\t\t traced.  The following options are available: \n"
        "\n"
        "\t\t noeval Causes arguments to  be  printed  unevaluated. \n"
        "\t\t\t If  noeval  is  specified,  the  arguments are \n"
        "\t\t\t printed before  evaluation.   Otherwise,  they \n"
        "\t\t\t are printed afterwards. \n"
        "\n"
        "\t\t\t If  the command line is longer than 60 charac- \n"
        "\t\t\t ters, it is truncated to 60  and  a  \"...\"  is \n"
        "\t\t\t postpended  to  indicate  that  there was more \n"
        "\t\t\t output than was displayed.   If  an  evaluated \n"
        "\t\t\t argument contains a space, the entire argument \n"
        "\t\t\t will be enclosed inside of  braces  (`{}')  to \n"
        "\t\t\t allow  the  reader  to  visually  separate the \n"
        "\t\t\t arguments from each other. \n"
        "\n"
        "\t\t notruncate \n"
        "\t\t\t Disables the truncation of commands and evalu- \n"
        "\t\t\t ated arguments. \n"
        "\n"
        "\t\t procs  Enables  the  tracing of procedure calls only. \n"
        "\t\t\t Commands that  aren't  procedure  calls  (i.e. \n"
        "\t\t\t calls  to  commands that are written in C, C++ \n"
        "\t\t\t or some object-compatible  language)  are  not \n"
        "\t\t\t traced if the procs option is specified.  This \n"
        "\t\t\t option  is  particularly  useful  for  greatly \n"
        "\t\t\t reducing  the  output of cmdtrace while debug- \n"
        "\t\t\t ging. \n"
        "\n"
        "\t\t fileid This is a file id as returned by the open com- \n"
        "\t\t\t mand.   If  specified,  then  the trace output \n"
        "\t\t\t will be written to the file rather  than  std- \n"
        "\t\t\t out.  A stdio buffer flush is done after every \n"
        "\t\t\t line is written so that the trace may be moni- \n"
        "\t\t\t tored externally or provide useful information \n"
        "\t\t\t for debugging problems that cause core  dumps. \n"
        "\t The  most  common use of this command is to enable tracing \n"
        "\t to a file during the development.  If a failure occurs,  a \n"
        "\t trace is then available when needed.  Command tracing will \n"
        "\t slow down the execution of code, so it should  be  removed \n"
        "\t when  code is debugged.  The following command will enable \n"
        "\t tracing to a file for the remainder of the program: \n"
        "\n"
        "\t\t\t cmdtrace on [open cmd.log w] \n"
        "\n"
        "\t cmdtrace off \n"
        "\t\t Turn off all tracing. \n"
        "\n"
        "\t cmdtrace depth \n"
        "\t\t Returns the current maximum trace level, or zero if \n"
        "\t\t trace is disabled. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t commandloop ?prompt1? ?prompt2? \n"
        "\t\t Create  an interactive command loop for the current \n"
        "\t\t TCL interpreter.  This  command  receives  commands \n"
        "\t\t from  stdin  and  executes  them.  It is useful TCL \n"
        "\t\t scripts that do not normally converse interactively \n"
        "\t\t with  a user through a Tcl command interpreter, but \n"
        "\t\t which sometimes want to enter this mode. \n"
        "\n"
        "\t\t Prompt1 is a Tcl command  that is evaluated to out- \n"
        "\t\t put  a prompt string.  The old value of tcl_prompt1 \n"
        "\t\t is saved and it is set to this value for the  dura- \n"
        "\t\t tion  of  the  command  loop.  Prompt2 is a command \n"
        "\t\t that  is  evaluated  to  output   the   ``downlevel \n"
        "\t\t prompt'',  which  is the prompt which is issued for \n"
        "\t\t continuation input.  The old value  of  tcl_prompt2 \n"
        "\t\t is  saved and it is set to this value for the dura- \n"
        "\t\t tion of the command loop. \n"
        "\n"
        "\t\t When the command terminates, the variables for  the \n"
        "\t\t prompt  hooks  will  be set to their old value.  If \n"
        "\t\t these arguments are not specified, the prompt hooks \n"
        "\t\t use their current value. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t convert_lib tclIndex packagelib ?ignore? \n"
        "\t\t Convert  a  Ousterhout  style  tclIndex  index file and associate \n"
        "\t\t source files into a package library  packagelib.   If  packagelib \n"
        "\t\t does  not  have  a .tlib extension, one will be added.  Any files \n"
        "\t\t specified in tclIndex  that  are  in  the  list  ignore  will  be \n"
        "\t\t skipped.   Files  listed  in  ignore should just be the base file \n"
        "\t\t names, not full paths. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t convertclock dateString ?GMT|{}? ?baseClock? \n"
        "\t\t Convert  dateString  to an integer clock value (see \n"
        "\t\t getclock).  This command can parse and convert vir- \n"
        "\t\t tually  any standard date and/or time string, which \n"
        "\t\t can include standard time zone mnemonics.  If  only \n"
        "\t\t a  time  is specified, the current date is assumed. \n"
        "\t\t If  the  string  does  not  contain  a  time   zone \n"
        "\t\t mnemonic,  the  local  time zone is assumed, unless \n"
        "\t\t the GMT argument is specified, in  which  case  the \n"
        "\t\t clock  value is calculated assuming that the speci- \n"
        "\t\t fied time is relative to Greenwich Mean  Time.   If \n"
        "\t\t baseClock  is specified, it is taken as the current \n"
        "\t\t clock value.  This is useful  for  determining  the \n"
        "\t\t time on a specific day. \n"
        "\n"
        "\t\t The character string consists of zero or more spec- \n"
        "\t\t ifications of the following form: \n"
        "\n"
        "\t\t time -  A  time  of  day,  which  is  of  the  form \n"
        "\t\t hh[:mm[:ss]]  [meridian]  [zone] or hhmm [meridian] \n"
        "\t\t [zone].  If no meridian is specified, hh is  inter- \n"
        "\t\t preted on a 24-hour clock. \n"
        "\n"
        "\t\t date - A specific month and day with optional year. \n"
        "\t\t The acceptable formats are mm/dd[/yy],  yyyy/mm/dd, \n"
        "\t\t monthname  dd[, yy], dd monthname [yy], and day, dd \n"
        "\t\t monthname yy.  The  default  year  is  the  current \n"
        "\t\t year.   If  the year is less then 100, then 1900 is \n"
        "\t\t added to it. \n"
        "\n"
        "\t\t relative time - A  specification  relative  to  the \n"
        "\t\t current  time.   The format is number unit; accept- \n"
        "\t\t able units are year, fortnight, month,  week,  day, \n"
        "\t\t hour,  minute  (or  min), and second (or sec).  The \n"
        "\t\t unit can be specified as a singular or  plural,  as \n"
        "\t\t in 3 weeks.  These modifiers may also be specified: \n"
        "\t\t tomorrow, yesterday, today, now, last, this,  next, \n"
        "\t\t ago. \n"
        "\n"
        "\t\t The actual date is calculated according to the fol- \n"
        "\t\t lowing steps.  First, any absolute date and/or time \n"
        "\t\t is processed and converted.  Using that time as the \n"
        "\t\t base, day-of-week specifications are added.   Next, \n"
        "\t\t relative specifications are used.  If a date or day \n"
        "\t\t is specified, and no absolute or relative  time  is \n"
        "\t\t given,  midnight is used.  Finally, a correction is \n"
        "\t\t applied so that the correct hour of the day is pro- \n"
        "\t\t duced after allowing for daylight savings time dif- \n"
        "\t\t ferences. \n"
        "\n"
        "\t\t convertclock ignores case when parsing  all  words. \n"
        "\t\t The names of the months and days of the week can be \n"
        "\t\t abbreviated to  their  first  three  letters,  with \n"
        "\t\t optional  trailing  period.  Periods are ignored in \n"
        "\t\t any timezone or meridian values. \n"
        "\n"
        "\t\t Note that convertclock will convert symbolic  time- \n"
        "\t\t zone  names,  but  these  are  not standardized and \n"
        "\t\t there are  conflicts  with  various  parts  of  the \n"
        "\t\t world.   Use  GMT when trying to produce a portable \n"
        "\t\t time that can then be converted back to  a  numeric \n"
        "\t\t value. \n"
        "\n"
        "\t\t The  only  dates  in the range 1902 and 2037 may be \n"
        "\t\t converted.  Some examples are: \n"
        "\n"
        "\t\t\t convertclock \"14 Feb 92\" \n"
        "\t\t\t convertclock \"Feb 14, 1992 12:20 PM PST\" \n"
        "\t\t\t convertclock \"12:20 PM Feb 14, 1992\" \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t copyfile ?-bytes num|-maxbytes num? fromFileId toFileId \n"
        "\t\t Copies  the  rest  of  the  file specified by from- \n"
        "\t\t FileId, starting from its current position, to  the \n"
        "\t\t file  specified by toFileId, starting from its cur- \n"
        "\t\t rent position. \n"
        "\n"
        "\t\t If -bytes is specified, then num bytes are  copied. \n"
        "\t\t If  less  than num bytes are available, an error is \n"
        "\t\t returned.  If  -maxbytes  is  specified,  then  num \n"
        "\t\t bytes  are  copied but no error is returned if less \n"
        "\t\t are available. \n"
        "\n"
        "\t\t The command returns the number of bytes  that  were \n"
        "\t\t copied. \n"
        "\n"
        "\t\t The -bytes option is particularly useful for mixing \n"
        "\t\t binary data in with ASCII commands  or  data  in  a \n"
        "\t\t data stream. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t crange string firstExpr lastExpr \n"
        "\t\t Returns  a range of characters from string starting \n"
        "\t\t at the character indexed by the  expression  first- \n"
        "\t\t Expr  (zero-based)  until  the character indexed by \n"
        "\t\t the expression lastExpr. \n"
        "\n"
        "\t\t If the expression firstExpr or lastExpr starts with \n"
        "\t\t the string end, then end is replaced with the index \n"
        "\t\t of the  last  character  in  the  string.   If  the \n"
        "\t\t expression  starts  with  len, then len is replaced \n"
        "\t\t with the length of the string. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t csubstr string firstExpr lengthExpr \n"
        "\t\t Returns  a range of characters from string starting \n"
        "\t\t at the character indexed by the  expression  first- \n"
        "\t\t Expr (zero-based) for lengthExpr characters. \n"
        "\n"
        "\t\t If  the  expression  firstExpr or lengthExpr starts \n"
        "\t\t with the string end, then end is replaced with  the \n"
        "\t\t index  of the last character in the string.  If the \n"
        "\t\t expression starts with len, then  len  is  replaced \n"
        "\t\t with the length of the string. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t ctoken strvar separators \n"
        "\t\t Parse  a  token  out  of  a  character string.  The \n"
        "\t\t string to parse is contained in the variable  named \n"
        "\t\t strvar.   The string separators contains all of the \n"
        "\t\t valid  separator  characters  for  tokens  in   the \n"
        "\t\t string.  All leading separators are skipped and the \n"
        "\t\t first token is returned.  The variable strvar  will \n"
        "\t\t be  modified to contain the remainder of the string \n"
        "\t\t following the token. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t ctype ?-failindex var? class string \n"
        "\t\t ctype  determines  whether all characters in string \n"
        "\t\t are of the specified class.  It returns 1  if  they \n"
        "\t\t are  all of class, and 0 if they are not, or if the \n"
        "\t\t string  is  empty.   This  command  also   provides \n"
        "\t\t another  method  (besides  format and scan) of con- \n"
        "\t\t verting between an ASCII character and its  numeric \n"
        "\t\t value.  The following ctype commands are available: \n"
        "\n"
        "\t\t ctype ?-failindex var? alnum string \n"
        "\t\t\t Tests that all characters  are  alphabetic  or \n"
        "\t\t\t numeric characters as defined by the character \n"
        "\t\t\t set. \n"
        "\n"
        "\t\t ctype ?-failindex var? alpha string \n"
        "\t\t\t Tests that all characters are alphabetic char- \n"
        "\t\t\t acters as defined by the character set. \n"
        "\n"
        "\t\t ctype ?-failindex var? ascii string \n"
        "\t\t\t Tests that all characters are an ASCII charac- \n"
        "\t\t\t ter (a non-negative number less than 0200). \n"
        "\n"
        "\t\t ctype char number \n"
        "\t\t\t Converts the  numeric  value,  string,  to  an \n"
        "\t\t\t ASCII  character.  Number must be in the range \n"
        "\t\t\t 0 through 255. \n"
        "\n"
        "\t\t ctype ?-failindex var? cntrl string \n"
        "\t\t\t Tests that all characters are ``control  char- \n"
        "\t\t\t acters'' as defined by the character set. \n"
        "\n"
        "\t\t ctype ?-failindex var? digit string \n"
        "\t\t\t Tests  that  all  characters are valid decimal \n"
        "\t\t\t digits, i.e. 0 through 9. \n"
        "\n"
        "\t\t ctype ?-failindex var? graph string \n"
        "\t\t\t Tests that all characters within are any char- \n"
        "\t\t\t acter  for  which  ctype print is true, except \n"
        "\t\t\t for space characters. \n"
        "\n"
        "\t\t ctype ?-failindex var? lower string \n"
        "\t\t\t Tests that all characters are  lowercase  let- \n"
        "\t\t\t ters as defined by the character set. \n"
        "\n"
        "\t\t ctype ord character \n"
        "\t\t\t Convert  a  character into its decimal numeric \n"
        "\t\t\t value.  The first character of the  string  is \n"
        "\t\t\t converted. \n"
        "\n"
        "\t\t ctype ?-failindex var? space string \n"
        "\t\t\t Tests  that all characters are either a space, \n"
        "\t\t\t horizontal-tab, carriage return, newline, ver- \n"
        "\t\t\t tical-tab, or form-feed. \n"
        "\t\t ctype ?-failindex var? print string \n"
        "\t\t\t Tests  that  all characters are a space or any \n"
        "\t\t\t character for which ctype alnum or ctype punct \n"
        "\t\t\t is  true  or  other  ``printing character'' as \n"
        "\t\t\t defined by the character set. \n"
        "\n"
        "\t\t ctype ?-failindex var? punct string \n"
        "\t\t\t Tests that all characters are made up  of  any \n"
        "\t\t\t of  the  characters  other  than  the ones for \n"
        "\t\t\t which alnum, cntrl, or space is true. \n"
        "\n"
        "\t\t ctype ?-failindex var? upper string \n"
        "\t\t\t Tests that all characters are  uppercase  let- \n"
        "\t\t\t ters as defined by the character set. \n"
        "\n"
        "\t\t ctype ?-failindex var? xdigit string \n"
        "\t\t\t Tests  that all characters are valid hexadeci- \n"
        "\t\t\t mal digits, that is 0 through 9, a  through  f \n"
        "\t\t\t or A through F. \n"
        "\n"
        "\t\t If  -failindex  is  specified,  then  the  index into \n"
        "\t\t string of the first character that did not match  the \n"
        "\t\t class is returned in var. \n"
        "\n"
        "\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t dirs   This  procedure lists the directories in the direc- \n"
        "\t\t tory stack. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t dup fileId ?targetFileId? \n"
        "\t\t Duplicate  an  open  file.  A new file id is opened \n"
        "\t\t that addresses the same file as fileId. \n"
        "\n"
        "\t\t If targetFileId is specified, the the file  is  dup \n"
        "\t\t to this specified file id.  Normally this is stdin, \n"
        "\t\t stdout, or stderr.  The  dup  command  will  handle \n"
        "\t\t flushing  output and closing this file.  The target \n"
        "\t\t file should be open if its one of stdin, stdout, or \n"
        "\t\t stderr and the process is not going to do an execl. \n"
        "\t\t Otherwise internal C code that uses  one  of  these \n"
        "\t\t files  via  direct  access to stdio FILE struct may \n"
        "\t\t behave strangely or fail. \n"
        "\n"
        "\t\t The procedure shown below will create a child  pro- \n"
        "\t\t cess and set its standard input and output files to \n"
        "\t\t a pair of pipe files we pass as arguments.  Finally \n"
        "\t\t the  program  does an execl of a specified command, \n"
        "\t\t with the program's stdin and stdout coming from and \n"
        "\t\t going to our pair of pipes. \n"
        "\n"
        "\t\t\t proc ChildProcess {cmd inPipe outPipe} { \n"
        "\t\t\t if {[set childPid [fork]] == 0} { \n"
        "\t\t\t\t dup $inPipe stdin \n"
        "\t\t\t\t close $inPipe \n"
        "\t\t\t\t dup $outPipe stdout \n"
        "\t\t\t\t close $outPipe \n"
        "\n"
        "\t\t\t\t execl $cmd \n"
        "\t\t\t\t # will never make it here... \n"
        "\t\t\t } \n"
        "\t\t\t return $childPid \n"
        "\t\t\t } \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t echo ?str ...? \n"
        "\t\t Writes  zero  or  more  strings to standard output, \n"
        "\t\t followed by a newline. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t edprocs ?proc...? \n"
        "\t\t This  procedure writes the named procedures, or all \n"
        "\t\t currently defined procedures, to a temporary  file, \n"
        "\t\t then  calls  an  editor  on it (as specified by the \n"
        "\t\t EDITOR environment variable, or vi if none is spec- \n"
        "\t\t ified),  then  sources  the  file back in if it was \n"
        "\t\t changed. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t execl ?-argv0 argv0? prog ?arglist? \n"
        "\t\t Do  an execl, replacing the current program (either \n"
        "\t\t Extended Tcl or an application  with  Extended  Tcl \n"
        "\t\t embedded  into  it) with prog and passing the argu- \n"
        "\t\t ments in the list arglist. \n"
        "\n"
        "\t\t The -argv0 options specifies that argv0  is  to  be \n"
        "\t\t passed to the program as argv [0] rather than prog. \n"
        "\n"
        "\t\t Note: If you are using execl in  a  Tk  application \n"
        "\t\t and it fails, you may not do anything that accesses \n"
        "\t\t the X server or you will receive a BadWindow  error \n"
        "\t\t from  the X server.  This includes executing the Tk \n"
        "\t\t version of the exit command.  We suggest using  the \n"
        "\t\t following command to abort Tk applications after an \n"
        "\t\t execl failure: \n"
        "\n"
        "\t\t\t kill [id process] \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t fcntl fileId attribute ?value? \n"
        "\t\t This command either sets or clears a file option or \n"
        "\t\t returns its current value.  If value are not speci- \n"
        "\t\t fied,  then  the  current  value  of  attribute  is \n"
        "\t\t returned. The following attributes  may  be  speci- \n"
        "\t\t fied: \n"
        "\n"
        "\t\t RDONLY  - The file is opened for reading only. (Get \n"
        "\t\t only) \n"
        "\n"
        "\t\t WRONLY - The file is opened for writing only.  (Get \n"
        "\t\t only) \n"
        "\n"
        "\t\t RDWR  - The file is opened for reading and writing. \n"
        "\t\t (Get only) \n"
        "\n"
        "\t\t READ - If the file is readable. (Get only). \n"
        "\n"
        "\t\t WRITE - If the file is writable. (Get only). \n"
        "\n"
        "\t\t APPEND - The file is opened for append-only writes. \n"
        "\t\t All writes will be forced to the end of the file. \n"
        "\n"
        "\t\t NONBLOCK  -  The  file  is to be accessed with non- \n"
        "\t\t blocking I/O.  See  the  read  system  call  for  a \n"
        "\t\t description  of how it affects the behavior of file \n"
        "\t\t reads. \n"
        "\n"
        "\t\t CLOEXEC - Close the file on an  process  exec.   If \n"
        "\t\t the  execl  command  or some other mechanism causes \n"
        "\t\t the process to do an exec, the file will be  closed \n"
        "\t\t if this option is set. \n"
        "\n"
        "\t\t NOBUF  -  The  file  is  not buffered. If set, then \n"
        "\t\t there no stdio buffering for the file. \n"
        "\n"
        "\t\t LINEBUF - Output the file will  be  line  buffered. \n"
        "\t\t The  buffer will be flushed when a newline is writ- \n"
        "\t\t ten, when the buffer is  full,  or  when  input  is \n"
        "\t\t requested. \n"
        "\n"
        "\t\t The APPEND, NONBLOCK, and CLOEXEC attributes may be \n"
        "\t\t set or cleared by specifying the attribute name and \n"
        "\t\t a value 1 to set the attribute and 0 to clear it. \n"
        "\n"
        "\t\t The NOBUF and LINEBUF attributes may only be set (a \n"
        "\t\t value of 1) and only one  of  the  options  may  be \n"
        "\t\t selected.   Once set, it may not be changed.  These \n"
        "\t\t options should be set  before  any  I/O  operations \n"
        "\t\t have been done on the file or data may be lost. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t flock options fileId ?start? ?length? ?origin? \n"
        "\t\t This  command  places  a lock on all or part of the \n"
        "\t\t file specified by fileId.  The lock is either advi- \n"
        "\t\t sory  or  mandatory,  depending on the mode bits of \n"
        "\t\t the file.  The lock is placed beginning at relative \n"
        "\t\t byte  offset  start  for length bytes.  If start or \n"
        "\t\t length is omitted or empty, zero  is  assumed.   If \n"
        "\t\t length is zero, then the lock always extents to end \n"
        "\t\t of file, even if the  file  grows.   If  origin  is \n"
        "\t\t \"start\",  then the offset is relative to the begin- \n"
        "\t\t ning of the file. If it is \"current\", it  is  rela- \n"
        "\t\t tive  to  the  current access position in the file. \n"
        "\t\t If it is \"end\", then it is relative to the  end-of- \n"
        "\t\t file  (a  negative  is  before the EOF, positive is \n"
        "\t\t after).  If origin is omitted, start is assumed. \n"
        "\n"
        "\t\t The following options are recognized: \n"
        "\n"
        "\t\t -read - Place a read lock on  the  file.   Multiple \n"
        "\t\t processes  may  be  accessing  the  file with read- \n"
        "\t\t locks. \n"
        "\n"
        "\t\t -write - Place a write lock on the file.  Only  one \n"
        "\t\t process may be accessing a file if there is a write \n"
        "\t\t lock. \n"
        "\n"
        "\t\t -nowait - If specified, then the process  will  not \n"
        "\t\t block  if  the lock can not be obtained.  With this \n"
        "\t\t option, the  command  returns  1  if  the  lock  is \n"
        "\t\t obtained and 0 if it is not. \n"
        "\n"
        "\t\t See  your  system's fcntl system call documentation \n"
        "\t\t for full details of the behavior of  file  locking. \n"
        "\t\t If locking is being done on ranges of a file, it is \n"
        "\t\t best to use unbuffered file access (see  the  fcntl \n"
        "\t\t command). \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t The following procedures provide command interfaces to the \n"
        "\t expr math functions. They take the same arguments  as  the \n"
        "\t expr functions and may take expressions as arguments. \n"
        "\n"
        "\t\t abs         acos        asin       atan2 \n"
        "\t\t atan        ceil        cos        cosh \n"
        "\t\t double      exp         floor      fmod \n"
        "\t\t hypot       int         log10      log \n"
        "\t\t pow         round       sin        sinh \n"
        "\t\t sqrt        tan         tanh \n"
        "\n"
        "\t These procedures are provided by Extended Tcl. \n",
    "\n"
    "\t fmtclock clockval ?format? ?GMT|{}? \n"
        "\t\t Converts  a  Unix  integer  time  value,  typically \n"
        "\t\t returned by getclock, convertclock, or  the  atime, \n"
        "\t\t mtime,  or  ctime  options  of the file command, to \n"
        "\t\t human-readable form.   The  format  argument  is  a \n"
        "\t\t string  that describes how the date and time are to \n"
        "\t\t be formatted.  Field descriptors consist of a ``%'' \n"
        "\t\t followed  by  a  field  descriptor  character.  All \n"
        "\t\t other characters are copied into the result.  Valid \n"
        "\t\t field descriptors are: \n"
        "\n"
        "\t\t\t %% - Insert a %. \n"
        "\t\t\t %a - Abbreviated weekday name. \n"
        "\t\t\t %A - Full weekday name \n"
        "\t\t\t %b - Abbreviated month name. \n"
        "\t\t\t %B - Full month name. \n"
        "\t\t\t %d - Day of month (01 - 31). \n"
        "\t\t\t %D - Date as %m/%d/%y. \n"
        "\t\t\t %e - Day of month (1-31), no leading zeros. \n"
        "\t\t\t %h - Abbreviated month name. \n"
        "\t\t\t %H - Hour (00 - 23). \n"
        "\t\t\t %I - Hour (00 - 12). \n"
        "\t\t\t %j - Day number of year (001 - 366). \n"
        "\t\t\t %m - Month number (01 - 12). \n"
        "\t\t\t %M - Minute (00 - 59). \n"
        "\t\t\t %n - Insert a new line. \n"
        "\t\t\t %p - AM or PM. \n"
        "\t\t\t %r - Time as %I:%M:%S %p. \n"
        "\t\t\t %R - Time as %H:%M. \n"
        "\t\t\t %S - Seconds (00 - 59). \n"
        "\t\t\t %t - Insert a tab. \n"
        "\t\t\t %T - Time as %H:%M:%S. \n"
        "\t\t\t %U - Week number of year (01 - 52), Sunday is the first \n"
        "\t\t\t day of the week. \n"
        "\t\t\t %w - Weekday number (Sunday = 0). \n"
        "\t\t\t %W - Week number of year (01 - 52), Monday is the first \n"
        "\t\t\t day of the week. \n"
        "\t\t\t %x - Local specific date format. \n"
        "\t\t\t %X - Local specific time format. \n"
        "\t\t\t %y - Year within century (00 - 99). \n"
        "\t\t\t %Y - Year as ccyy (e.g. 1990) \n"
        "\t\t\t %Z - Time zone name. \n"
        "\n"
        "\t\t If  format  is not specified, \"%a %b %d %H:%M:%S %Z \n"
        "\t\t %Y\" is used.  If GMT is specified, the time will be \n"
        "\t\t formated as Greenwich Mean Time. If the argument is \n"
        "\t\t not specified or is empty, then the local  timezone \n"
        "\t\t will be used as defined by the TIMEZONE environment \n"
        "\t\t variable. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t for_array_keys var array_name code \n"
        "\t\t This  procedure  performs  a foreach-style loop for \n"
        "\t\t each key in the named array.  The  break  and  con- \n"
        "\t\t tinue statements work as with foreach. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t for_file var filename { code } \n"
        "\t\t This  procedure implements a loop over the contents \n"
        "\t\t of a file.  For each line in filename, it sets  var \n"
        "\t\t to the line and executes code. \n"
        "\n"
        "\t\t The  break and continue commands work as with fore- \n"
        "\t\t ach. \n"
        "\n"
        "\t\t For example, the command \n"
        "\n"
        "\t\t\t for_file line /etc/passwd {echo $line} \n"
        "\n"
        "\t\t would echo all the lines in the password file. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t for_recursive_glob var dirlist globlist code \n"
        "\t\t This  procedure  performs a foreach-style loop over \n"
        "\t\t recursively  matched  files.   All  directories  in \n"
        "\t\t dirlist  are  recursively searched (breadth-first), \n"
        "\t\t comparing each file found  against  the  file  glob \n"
        "\t\t patterns  in  globlist.  For each matched file, the \n"
        "\t\t variable var is set to the file path  and  code  is \n"
        "\t\t evaluated.  Symbolic links are not followed. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t fork \n"
        "\t\t Fork the current Tcl process.  Fork returns zero to \n"
        "\t\t the child process and the  process  number  of  the \n"
        "\t\t child  to the parent process.  If the fork fails, a \n"
        "\t\t Tcl error is generated. \n"
        "\n"
        "\t\t If an execl is not going to be performed before the \n"
        "\t\t child  process  does  output, or if a close and dup \n"
        "\t\t sequence is going to  be  performed  on  stdout  or \n"
        "\t\t stderr,  then a flush should be issued against std- \n"
        "\t\t out, stderr and any other open output  file  before \n"
        "\t\t doing  the fork. Otherwise characters from the par- \n"
        "\t\t ent process pending in the buffers will  be  output \n"
        "\t\t by both the parent and child processes. \n"
        "\n"
        "\t\t Note:  If you are forking in a Tk based apllication \n"
        "\t\t you must execl before doing any  window  operations \n"
        "\t\t in  the child or you will receive a BadWindow error \n"
        "\t\t from the X server. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t frename oldPath newPath \n"
        "\t\t Renames  oldPath to newPath.  This command does not \n"
        "\t\t support renaming across  file  systems  unless  the \n"
        "\t\t rename  system  call  supports  them.   If renaming \n"
        "\t\t accross file systems is desired, exec the  mv  com- \n"
        "\t\t mand. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t fstat fileId ?item?|?stat arrayvar? \n"
        "\t\t Obtain status information about an open file. \n"
        "\n"
        "\t\t The following keys are used to identify data items: \n"
        "\n"
        "\t\t o atime - The time of last access. \n"
        "\n"
        "\t\t o ctime - The time of last file status change \n"
        "\n"
        "\t\t o dev - The device containing a directory  for  the \n"
        "\t\t file.  This value uniquely identifies the file sys- \n"
        "\t\t tem that contains the file. \n"
        "\n"
        "\t\t o gid - The group ID of the file's group. \n"
        "\n"
        "\t\t o ino - The  inode  number.   This  field  uniquely \n"
        "\t\t identifies the file in a given file system. \n"
        "\n"
        "\t\t o mode - The mode of the file (see the mknod system \n"
        "\t\t call). \n"
        "\n"
        "\t\t o mtime - Time when the data in the file  was  last \n"
        "\t\t modified. \n"
        "\n"
        "\t\t o nlink - The number of links to the file. \n"
        "\n"
        "\t\t o size - The file size in bytes. \n"
        "\n"
        "\t\t o tty  - If the file is associated with a terminal, \n"
        "\t\t then 1 otherwise 0. \n"
        "\n"
        "\t\t o type - The type of the  file  in  symbolic  form, \n"
        "\t\t which  is one of the following values: file, direc- \n"
        "\t\t tory, characterSpecial, blockSpecial,  fifo,  link, \n"
        "\t\t or socket. \n"
        "\n"
        "\t\t o uid - The user ID of the file's owner. \n"
        "\n"
        "\t\t If  one  of  these  keys is specified as item, then \n"
        "\t\t that data item is returned. \n"
        "\n"
        "\t\t If stat arrayvar is specified, then the information \n"
        "\t\t is  returned  in  the array arrrayvar.  Each of the \n"
        "\t\t above keys indexes an element of the array contain- \n"
        "\t\t ing the data. \n"
        "\n"
        "\t\t If  only  fileId  is specified, the command returns \n"
        "\t\t the data as a keyed list. \n"
        "\n"
        "\t\t The following value may be returned only if explic- \n"
        "\t\t itly  asked  for,  it will not be returned with the \n"
        "\t\t array or keyed list forms: \n"
        "\n"
        "\t\t o remotehost  -  If  fileId  is  a  TCP/IP   socket \n"
        "\t\t connection,  then a list is returned with the first \n"
        "\t\t element being the remote host IP address.   If  the \n"
        "\t\t remote  host  name  can be found, it is returned as \n"
        "\t\t the second element of the list. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t funlock fileId ?start? ?length? ?origin? \n"
        "\t\t Remove  a  locked  from  a file that was previously \n"
        "\t\t placed with the flock command.  The  arguments  are \n"
        "\t\t the same as for the flock command, see that command \n"
        "\t\t for more details. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t getclock \n"
        "\t\t Return  the  current  date  and  time  as a system- \n"
        "\t\t dependent integer value.  The unit of the value  is \n"
        "\t\t seconds,  allowing  it to be used for relative time \n"
        "\t\t calculations. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t id options \n"
        "\t\t This  command  provides a means of getting, setting \n"
        "\t\t and converting user, group and process ids.  The id \n"
        "\t\t command has the following options: \n"
        "\n"
        "\t\t id user ?name? \n"
        "\n"
        "\t\t id userid ?uid? \n"
        "\t\t\t Set  the real and effective user ID to name or \n"
        "\t\t\t uid, if the name (or uid) is valid and permis- \n"
        "\t\t\t sions  allow  it.  If the name (or uid) is not \n"
        "\t\t\t specified,  the  current  name  (or  uid)   is \n"
        "\t\t\t returned. \n"
        "\n"
        "\t\t id convert userid uid \n"
        "\n"
        "\t\t id convert user name \n"
        "\t\t\t Convert  a  user  ID number to a user name, or \n"
        "\t\t\t vice versa. \n"
        "\n"
        "\t\t id group ?name? \n"
        "\n"
        "\t\t id groupid ?gid? \n"
        "\t\t\t Set the real and effective group ID to name or \n"
        "\t\t\t gid, if the name (or gid) is valid and permis- \n"
        "\t\t\t sions allow it.  If the group name (or gid) is \n"
        "\t\t\t not specified, the current group name (or gid) \n"
        "\t\t\t is returned. \n"
        "\n"
        "\t\t id convert groupid gid \n"
        "\n"
        "\t\t id convert group name \n"
        "\t\t\t Convert a group ID number to a group name,  or \n"
        "\t\t\t vice versa. \n"
        "\n"
        "\t\t id effective user \n"
        "\n"
        "\t\t id effective userid \n"
        "\t\t\t Return  the  effective user name, or effective \n"
        "\t\t\t user ID number, respectively. \n"
        "\n"
        "\t\t id effective group \n"
        "\n"
        "\t\t id effective groupid \n"
        "\t\t\t Return the effective group name, or  effective \n"
        "\t\t\t group ID number, respectively. \n"
        "\n"
        "\t\t id process \n"
        "\t\t\t Return  the process ID of the current process. \n"
        "\n"
        "\t\t id process parent \n"
        "\t\t\t Return the process ID of  the  parent  of  the \n"
        "\t\t\t current process. \n"
        "\t\t id process group \n"
        "\t\t\t Return  the  process  group  ID of the current \n"
        "\t\t\t process. \n"
        "\n"
        "\t\t id process group set \n"
        "\t\t\t Set the process group ID of the  current  pro- \n"
        "\t\t\t cess to its process ID. \n"
        "\n"
        "\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t infox option \n"
        "\t\t Return  information about Extended Tcl, or the cur- \n"
        "\t\t rent  application.   The  following  infox  command \n"
        "\t\t options are available: \n"
        "\n"
        "\t\t version \n"
        "\t\t\t Return  the  version  number  of Extended Tcl. \n"
        "\t\t\t The version number for Extended Tcl is  gener- \n"
        "\t\t\t ated  by  combining  the  base  version of the \n"
        "\t\t\t standard Tcl code with a letter indicating the \n"
        "\t\t\t version  of  Extended Tcl being used.  This is \n"
        "\t\t\t the documentation for version 7.0a. \n"
        "\n"
        "\t\t patchlevel \n"
        "\t\t\t Return the patchlevel for Extended Tcl. \n"
        "\n"
        "\t\t have_fsync \n"
        "\t\t\t Return 1 if the fsync system call is available \n"
        "\t\t\t and  the  sync  command  will  sync individual \n"
        "\t\t\t files.  0 if it is not available and the  sync \n"
        "\t\t\t command will always sync all file buffers. \n"
        "\n"
        "\t\t have_msgcats \n"
        "\t\t\t Return  1  if  XPG message catalogs are avail- \n"
        "\t\t\t able, 0 if  they  are  not.   The  catgets  is \n"
        "\t\t\t designed  to continue to function without mes- \n"
        "\t\t\t sage catalogs, always  returning  the  default \n"
        "\t\t\t string. \n"
        "\n"
        "\t\t have_posix_signals \n"
        "\t\t\t Return 1 if Posix signals are available (block \n"
        "\t\t\t and unblock options available for  the  signal \n"
        "\t\t\t command).   0 is returned if Posix signals are \n"
        "\t\t\t not available. \n"
        "\n"
        "\t\t have_sockets \n"
        "\t\t\t Return 1 if sockets are available (server_open \n"
        "\t\t\t command  and  fstat  removehost option).  0 is \n"
        "\t\t\t returned if sockets are not available. \n"
        "\n"
        "\t\t appname \n"
        "\t\t\t Return the symbolic application  name  of  the \n"
        "\t\t\t current  application  linked with the Extended \n"
        "\t\t\t Tcl library.  The C variable  tclAppName  must \n"
        "\t\t\t be  set by the application to return an appli- \n"
        "\t\t\t cation specific value for this variable. \n"
        "\n"
        "\t\t applongname \n"
        "\t\t\t Return a natural language name for the current \n"
        "\t\t\t application.  The  C  variable  tclLongAppName \n"
        "\t\t\t must be set by the application  to  return  an \n"
        "\t\t\t application  specific value for this variable. \n"
        "\t\t appversion \n"
        "\t\t\t Return the  version  number  for  the  current \n"
        "\t\t\t application.   The  C  variable  tclAppVersion \n"
        "\t\t\t must be set by the application  to  return  an \n"
        "\t\t\t application-specific  value for this variable. \n"
        "\n"
        "\t\t apppatchlevel \n"
        "\t\t\t Return the patchlevel for the current applica- \n"
        "\t\t\t tion.  The C variable tclAppPatchlevel must be \n"
        "\t\t\t set by the application to return  an  applica- \n"
        "\t\t\t tion-specific value for this variable. \n"
        "\n"
        "\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t When Extended Tcl is installed, the standard runtime files \n"
        "\t are places in the Tcl master directory, which  is  config- \n"
        "\t ured  when  Tcl  is built.  This master directory normally \n"
        "\t contains the Tcl initialization  file  (TclInit.tcl),  the \n"
        "\t standard  Tcl  library file (tcl.tlib) and the help files. \n"
        "\t The Tcl master directory is named after the version of Tcl \n"
        "\t it  is  associated  with,  e.g. /usr/local/tclX/7.0a.  The \n"
        "\t path to the Tcl master directory  is  available  from  the \n"
        "\t info  library  command.   The  location  of the Tcl master \n"
        "\t directory can be overridden with the TCL_LIBRARY  environ- \n"
        "\t ment variable. \n"
        "\n"
        "\t The  first step in initializing the Tcl shell is to locate \n"
        "\t the Tcl initialization file, normally TclInit.tcl.  If  an \n"
        "\t environment  variable TCLINIT exists, it contains the path \n"
        "\t to the Tcl initialization file.  If the  TCLINIT  environ- \n"
        "\t ment  variable  is  not  set, the file TclInit.tcl is used \n"
        "\t from the default Tcl master directory. \n"
        "\n"
        "\t Tcl then  evaulates  the  Tcl  initialization  file.   The \n"
        "\t auto_path variable is initialized to the Tcl master direc- \n"
        "\t tory and may be augmented by the intialization file or the \n"
        "\t application.   Other  procedures and variables used by the \n"
        "\t Extended Tcl shell are also defined by this file. \n"
        "\n"
        "\t If the Tcl is invoked interactively, it will source a file \n"
        "\t named  .tclrc  in the user's home directory, if it exists. \n"
        "\t Tcl is viewed primarily as a programming language, not  an \n"
        "\t interactive  shell,  so the .tclrc is intended for use for \n"
        "\t loading development utilities,  not  to  support  applica- \n"
        "\t tions,  which  should not have to rely on the user's envi- \n"
        "\t ronment in such a manner. \n"
        "\n"
        "\t The Extended Tcl Tk shell, wishx, has an additional master \n"
        "\t directory and initialization file.  It use the environment \n"
        "\t variable TK_LIBRARY to override the  default  location  of \n"
        "\t the Tk master directory. \n"
        "\n"
        "\t This functionallity is provided by Extended Tcl. \n",
    "\n"
    "\t intersect lista listb \n"
        "\t\t Procedure to return the logical intersection of two \n"
        "\t\t lists.  The returned list will be sorted. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t intersect3 lista listb \n"
        "\t\t Procedure to intersects two lists, returning a list \n"
        "\t\t containing three lists:  The first list returned is \n"
        "\t\t everything in lista that wasn't in listb.  The sec- \n"
        "\t\t ond list  contains  the  intersection  of  the  two \n"
        "\t\t lists, and the third list contains all the elements \n"
        "\t\t that were in  listb  but  weren't  in  lista.   The \n"
        "\t\t returned lists will be sorted. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t tcl ?-qn? ?-f? script?|?-c command? ?args? \n"
        "\n"
        "\t Tcl  starts  the interactive Tcl command interpreter.  The \n"
        "\t Tcl shell provides an environment for  writing,  debugging \n"
        "\t and  executing  Tcl scripts.  The functionality of the Tcl \n"
        "\t shell can be  easily  obtained  by  any  application  that \n"
        "\t includes Tcl. \n"
        "\n"
        "\t The  tcl command, issued without any arguments, invokes an \n"
        "\t interactive Tcl  shell,  allowing  the  user  to  interact \n"
        "\t directly  with Tcl, executing any Tcl commands at will and \n"
        "\t viewing their results. \n"
        "\n"
        "\t If script is specified, then the script is  executed  non- \n"
        "\t interactively  with  any additional arguments, args, being \n"
        "\t supplied in the global Tcl variable `argv'.  If command is \n"
        "\t supplied, then this command (or semicolon-separated series \n"
        "\t of commands) is executed, with `argv' containing any args. \n"
        "\n"
        "\t The  Tcl  shell is intended as an environment for Tcl pro- \n"
        "\t gram development and execution.  While it is not  a  full- \n"
        "\t featured  interactive  shell,  it  provides  a comfortable \n"
        "\t environment for the interactive development of  Tcl  code. \n"
        "\t Note  that  the  package library code described here over- \n"
        "\t rides the unknown command provided as part of the standard \n"
        "\t Berkeley   Tcl   library  facility,  although  Tcl  source \n"
        "\t libraries coded to that standard can be loaded and used by \n"
        "\t Extended Tcl. \n"
        "\n"
        "\t The following command line flags are recognized by the Tcl \n"
        "\t shell command line parser: \n"
        "\n"
        "\t -q     Quick initialization flag.  The  Tcl  initiaization \n"
        "\t\t file is not evaluated and the auto_path variable is \n"
        "\t\t not set.   Tcl  auto-load  libraries  will  not  be \n"
        "\t\t available. \n"
        "\n"
        "\t -n     No  procedure  call stack dump.  The procedure call \n"
        "\t\t stack will not be displayed when an  error  occurs, \n"
        "\t\t only  the  error message.  Useful in the #! line of \n"
        "\t\t already debugged scripts. \n"
        "\n"
        "\t -f     Takes the next argument as  a  script  for  Tcl  to \n"
        "\t\t source, rather than entering interactive mode.  The \n"
        "\t\t -f flag is optional.  Normally the  first  argument \n"
        "\t\t that  does  not  start  with  a `-' is taken as the \n"
        "\t\t script to execute unless the `-c' option is  speci- \n"
        "\t\t fied.   Any  following  arguments are passed to the \n"
        "\t\t script via argv, thus any other Tcl shell  command- \n"
        "\t\t line flags must precede this option. \n"
        "\n"
        "\t -c     Take the next argument as a Tcl command to execute. \n"
        "\t\t It may contain series of commands to execute, sepa- \n"
        "\t\t rated  by  `;'.  Any following arguments are passed \n"
        "\t\t in argv, thus, as with  -f,  any  other  Tcl  shell \n"
        "\t\t flags must precede this option. \n"
        "\n"
        "\t --     Mark the end of the arguments to the Tcl shell. All \n"
        "\t\t arguments following this  are  passed  in  the  Tcl \n"
        "\t\t variable  argv.   This  is useful to pass arguments \n"
        "\t\t without attempting to execute a Tcl script. \n"
        "\n"
        "\t The result string returned by a command executed from  the \n"
        "\t Tcl  shell  command  line  is  normally echoed back to the \n"
        "\t user.  If an error occurs, then the string result is  dis- \n"
        "\t played,  along  with the error message.  The error message \n"
        "\t will be preceded by the string ``Error:''. \n"
        "\n"
        "\t The set command is a special  case.   If  the  command  is \n"
        "\t called  to  set a variable (i.e. with two arguments), then \n"
        "\t the result will not be echoed.  If only one argument,  the \n"
        "\t name  of  a  variable, is supplied to set, then the result \n"
        "\t will be echoed. \n"
        "\n"
        "\t If an unknown Tcl command  is  entered  from  the  command \n"
        "\t line,  then  the Unix command path, specified in the envi- \n"
        "\t ronment variable PATH, will be searched for a  command  of \n"
        "\t the  same  name.  If the command is found, it will be exe- \n"
        "\t cuted with any arguments remaining on the Tcl command line \n"
        "\t being passed as arguments to the command.  This feature is \n"
        "\t provided to enhance the interactive environment for devel- \n"
        "\t oping Tcl scripts. \n"
        "\n"
        "\t Automatic  execution  of  programs  in this manner is only \n"
        "\t supported from the command line, not in script files or in \n"
        "\t procedures,  to  reduce  confusion and mistakes while pro- \n"
        "\t gramming in Tcl.  Scripts should use the Tcl exec or  sys- \n"
        "\t tem commands to run Unix commands. \n"
        "\n"
        "\t This functionallity is provided by Extended Tcl. \n",
    "\n"
    "\t KEYED LISTS \n"
        "\t Extended Tcl defines a special type of list referred to as \n"
        "\t keyed lists.  These lists provided a structured data  type \n"
        "\t built  upon standard Tcl lists.  This provides a function- \n"
        "\t ality similar to structs in the C programming language. \n"
        "\n"
        "\t A keyed list is a list in which each  element  contains  a \n"
        "\t key  and  value  pair.   These element pairs are stored as \n"
        "\t lists themselves, where the key is the  first  element  of \n"
        "\t the  list,  and  the  value  is the second.  The key-value \n"
        "\t pairs are refered to as fields.  This is an example  of  a \n"
        "\t keyed list: \n"
        "\n"
        "\t\t\t {{NAME  {Frank  Zappa}} {JOB {musician and com- \n"
        "\t\t poser}}} \n"
        "\n"
        "\t If the variable person  contained  the  above  list,  then \n"
        "\t keylget person NAME would return {Frank Zappa}.  Executing \n"
        "\t the command: \n"
        "\n"
        "\t\t keylset person ID 106 \n"
        "\n"
        "\t would make person contain \n"
        "\n"
        "\t\t\t {{ID 106} {NAME {Frank Zappa}}  {JOB  {musician \n"
        "\t\t and composer}}} \n"
        "\n"
        "\t Fields may contain subfields; `.' is the seperator charac- \n"
        "\t ter.  Subfields are actually fields  where  the  value  is \n"
        "\t another  keyed  list.  Thus the following list has the top \n"
        "\t level fields ID and NAME,  and  subfields  NAME.FIRST  and \n"
        "\t NAME.LAST: \n"
        "\n"
        "\t\t\t {ID 106} {NAME {{FIRST Frank} {LAST Zappa}}} \n"
        "\n"
        "\t There  is  no  limit  to the recursive depth of subfields, \n"
        "\t allowing one to build complex data structures. \n"
        "\n"
        "\t Keyed lists are constructed and accessed via a  number  of \n"
        "\t commands.   All  keyed  list  management commands take the \n"
        "\t name of the variable containing the keyed list as an argu- \n"
        "\t ment  (i.e.  passed by reference), rather than passing the \n"
        "\t list directly. \n"
        "\n"
        "\t This functionallity is provided by Extended Tcl. \n",
    "\n"
    "\t keyldel listvar key \n"
        "\t\t Delete  the  field  specified by key from the keyed \n"
        "\t\t list in the variable listvar.   This  removes  both \n"
        "\t\t the key and the value from the keyed list. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t keylget listvar ?key? ?retvar | {}? \n"
        "\t\t Return the value associated with key from the keyed \n"
        "\t\t list in the variable listvar.   If  retvar  is  not \n"
        "\t\t specified,  then  the value will be returned as the \n"
        "\t\t result of the command.  In this case, if key is not \n"
        "\t\t found in the list, an error will result. \n"
        "\n"
        "\t\t If retvar is specified and key is in the list, then \n"
        "\t\t the value is returned in the  variable  retvar  and \n"
        "\t\t the command returns 1 if the key was present within \n"
        "\t\t the list.  If key isn't in the  list,  the  command \n"
        "\t\t will  return  0, and retvar will be left unchanged. \n"
        "\t\t If {} is specified for retvar,  the  value  is  not \n"
        "\t\t returned,  allowing the Tcl programmer to determine \n"
        "\t\t if a key is present in a keyed list without setting \n"
        "\t\t a variable as a side-effect. \n"
        "\n"
        "\t\t If  key  is omitted, then a list of all the keys in \n"
        "\t\t the keyed list is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t keylkeys listvar ?key? \n"
        "\t\t Return the a list of the keyes in the keyed list in \n"
        "\t\t the variable listvar.  If keys is  specified,  then \n"
        "\t\t it  is the name of a key field  who's subfield keys \n"
        "\t\t are to be retrieve. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t keylset listvar key value ?key2 value2 ...? \n"
        "\t\t Set  the  value  associated  with key, in the keyed \n"
        "\t\t list contained in the variable listvar,  to  value. \n"
        "\t\t If  listvar does not exists, it is created.  If key \n"
        "\t\t is not currently in the list, it will be added.  If \n"
        "\t\t it  already  exists,  value  replaces  the existing \n"
        "\t\t value.  Multiple keywords and values may be  speci- \n"
        "\t\t fied, if desired. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t kill ?-pgroup ?signal? idlist \n"
        "\t\t Send  a  signal  to  the  each  process in the list \n"
        "\t\t idlist, if permitted.  Signal, if present,  is  the \n"
        "\t\t signal  number  or the symbolic name of the signal, \n"
        "\t\t see the signal system call manual page.  The  lead- \n"
        "\t\t ing  ``SIG''  is optional when the signal is speci- \n"
        "\t\t fied by its symbolic name.  The default  for  signo \n"
        "\t\t is 15, SIGTERM. \n"
        "\n"
        "\t\t If  -pgroup is specified, the numbers in idlist are \n"
        "\t\t take as process group ids and the signal is sent to \n"
        "\t\t all  of  the process in that process group.  A pro- \n"
        "\t\t cess group id of 0 specifies  the  current  process \n"
        "\t\t group. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lassign list var ?var...? \n"
        "\t\t Assign  successive  elements of a list to specified \n"
        "\t\t variables.  If there are more variable  names  than \n"
        "\t\t fields,  the  remaining  variables  are  set to the \n"
        "\t\t empty string.  If  there  are  more  elements  than \n"
        "\t\t variables,  a  list  of  the unassigned elements is \n"
        "\t\t returned. \n"
        "\n"
        "\t\t For example, \n"
        "\n"
        "\t\t\t lassign {dave 100 200 {Dave Foo}} name uid gid longName \n"
        "\n"
        "\t\t Assigns name to ``dave'', uid to  ``100'',  gid  to \n"
        "\t\t ``200'', and longName to ``Dave Foo''. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lempty list \n"
        "\t\t Determine  if  the  specified  list  is  empty.  If \n"
        "\t\t empty, 1 is returned,  otherwise,  0  is  returned. \n"
        "\t\t This  command is an alternative to comparing a list \n"
        "\t\t to an empty string. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lgets fileId ?varName? \n"
        "\t\t Reads  the  next  Tcl  list  from the file given by \n"
        "\t\t fileId and discards the terminating newline charac- \n"
        "\t\t ter.   This  command differs from the gets command, \n"
        "\t\t in that it reads Tcl lists rather than  lines.   If \n"
        "\t\t the list contains a newline, then that newline will \n"
        "\t\t be returned as part of the result.  Only a  newline \n"
        "\t\t not quoted as part of the list indicates the end of \n"
        "\t\t the list.  There is no  corresponding  command  for \n"
        "\t\t outputing  lists,  as  puts will do this correctly. \n"
        "\t\t If varName is specified, then the line is placed in \n"
        "\t\t the variable by that name and the return value is a \n"
        "\t\t count of the number of characters read (not includ- \n"
        "\t\t ing  the  newline).   If  the  end  of  the file is \n"
        "\t\t reached before reading any characters  then  -1  is \n"
        "\t\t returned and varName is set to an empty string.  If \n"
        "\t\t varName is not specified then the return value will \n"
        "\t\t be  the  line  (minus  the newline character) or an \n"
        "\t\t empty string if the end  of  the  file  is  reached \n"
        "\t\t before  reading  any  characters.   An empty string \n"
        "\t\t will also be returned if a line contains no charac- \n"
        "\t\t ters except the newline, so eof may have to be used \n"
        "\t\t to determine what really happened. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t link ?-sym? srcpath destpath \n"
        "\t\t Create  a  directory entry, destpath, linking it to \n"
        "\t\t the existing file, srcpath.  If -sym is  specified, \n"
        "\t\t a  symbolic  link, rather than a hard link, is cre- \n"
        "\t\t ated.  (The -sym option is only available  on  sys- \n"
        "\t\t tems that support symbolic links.) \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lmatch ?mode? list pattern \n"
        "\t\t Search  the  elements  of list, returning a list of \n"
        "\t\t all elements matching pattern.  If none  match,  an \n"
        "\t\t empty list is returned. \n"
        "\n"
        "\t\t The mode argument indicates how the elements of the \n"
        "\t\t list are to be matched against pattern and it  must \n"
        "\t\t have one of the following values: \n"
        "\n"
        "\t\t -exact  The  list  element must contain exactly the \n"
        "\t\t same string as pattern. \n"
        "\n"
        "\t\t -glob Pattern is  a  glob-style  pattern  which  is \n"
        "\t\t matched  against  each  list element using the same \n"
        "\t\t rules as the string match command. \n"
        "\n"
        "\t\t -regexp Pattern is treated as a regular  expression \n"
        "\t\t and  matched  against  each  list element using the \n"
        "\t\t same rules as the regexp command. \n"
        "\n"
        "\t\t If mode is omitted then it defaults to -glob. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t loadlibindex libfile.tlib \n"
        "\t\t Load the package library index of the library file libfile (which \n"
        "\t\t must have the suffix .tlib).  Package library indexes  along  the \n"
        "\t\t auto_path are loaded automatically on the first demand_load; this \n"
        "\t\t command is provided to explicitly load libraries that are not  in \n"
        "\t\t the  path.   If  the  index  file  (with a .tndx suffix) does not \n"
        "\t\t exists or is out of date, it will be  rebuilt  if  the  user  has \n"
        "\t\t directory  permissions  to  create it. If a package with the same \n"
        "\t\t name as a package in libfile.tlib has already  been  loaded,  its \n"
        "\t\t definition  will  be  overridden by the new package.  However, if \n"
        "\t\t any procedure has actually been used from the previously  defined \n"
        "\t\t package, the procedures from libfile.tlib will not be loaded. \n"
        "\n"
        "\t\t This command will also load an index built by mkindex.tcl program \n"
        "\t\t supplied with standard Tcl.  This file must be named  \"tclIndex\". \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t loop var first limit ?increment? body \n"
        "\t\t Loop  is  a looping command, similar in behavior to \n"
        "\t\t the Tcl for statement, except that the loop  state- \n"
        "\t\t ment  achieves substantially higher performance and \n"
        "\t\t is easier to code when  the  beginning  and  ending \n"
        "\t\t values  of  a loop are known, and the loop variable \n"
        "\t\t is to be incremented by a known, fixed amount every \n"
        "\t\t time through the loop. \n"
        "\n"
        "\t\t The  var  argument  is  the name of a Tcl variable \n"
        "\t\t that will contain the loop index.  The  loop  index \n"
        "\t\t is  set  to  the value specified by first.  The Tcl \n"
        "\t\t interpreter is  invoked  upon  body  zero  or  more \n"
        "\t\t times,  where var is incremented by increment every \n"
        "\t\t time through the loop, or by one  if  increment  is \n"
        "\t\t not  specified.  Increment can be negative in which \n"
        "\t\t case the loop will count downwards. \n"
        "\n"
        "\t\t When var reaches limit, the loop terminates without \n"
        "\t\t a  subsequent  execution of body.  For instance, if \n"
        "\t\t the original loop parameters would  cause  loop  to \n"
        "\t\t terminate,  say  first  was one, limit was zero and \n"
        "\t\t increment was not specified  or  was  non-negative, \n"
        "\t\t body is not executed at all and loop returns. \n"
        "\n"
        "\t\t The  first, limit and increment are integer expres- \n"
        "\t\t sions.  They are only evaulated once at the  begin- \n"
        "\t\t ning of the loop. \n"
        "\n"
        "\t\t If  a  continue command is invoked within body then \n"
        "\t\t any remaining commands in the current execution  of \n"
        "\t\t body  are  skipped,  as  in  the for command.  If a \n"
        "\t\t break command is invoked within body then the  loop \n"
        "\t\t command  will  return immediately.  Loop returns an \n"
        "\t\t empty string. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lrmdups list \n"
        "\t\t Procedure to remove duplicate elements from a list. \n"
        "\t\t The returned list will be sorted. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t lvarcat var string ?string...? \n"
        "\t\t This  command treats each string argument as a list \n"
        "\t\t and concatenates them to the end of the contents of \n"
        "\t\t var,  forming  a a single list.  The list is stored \n"
        "\t\t back into var and also returned as the result.   if \n"
        "\t\t var does not exist, it is created. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lvarpop var ?indexExpr? ?string? \n"
        "\t\t The  lvarpop  command  pops  (deletes)  the element \n"
        "\t\t indexed by the expression indexExpr from  the  list \n"
        "\t\t contained  in  the variable var.  If index is omit- \n"
        "\t\t ted, then 0 is assumed.  If string,  is  specified, \n"
        "\t\t then the deleted element is replaced by string. The \n"
        "\t\t replaced or  deleted  element  is  returned.   Thus \n"
        "\t\t ``lvarpop  argv  0''  returns  the first element of \n"
        "\t\t argv, setting argv to contain the remainder of  the \n"
        "\t\t string. \n"
        "\n"
        "\t\t If  the expression indexExpr starts with the string \n"
        "\t\t end, then end is replaced with  the  index  of  the \n"
        "\t\t last element in the list.  If the expression starts \n"
        "\t\t with len, then len is replaced with the  length  of \n"
        "\t\t the list. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t lvarpush var string ?indexExpr? \n"
        "\t\t The  lvarpush command pushes (inserts) string as an \n"
        "\t\t element in the list contained in the variable  var. \n"
        "\t\t The  element  is inserted before position indexExpr \n"
        "\t\t in the  list.  If  index  is  omitted,  then  0  is \n"
        "\t\t assumed.  If var does not exists, it is created. \n"
        "\n"
        "\t\t If  the expression indexExpr starts with the string \n"
        "\t\t end, then end is replaced with  the  index  of  the \n"
        "\t\t last element in the list.  If the expression starts \n"
        "\t\t with len, then len is replaced with the  length  of \n"
        "\t\t the list.  Note the a value of end means insert the \n"
        "\t\t string before the last element. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t max num1 num2 ?..numN? \n"
        "\t\t Returns  the  argument that has the highest numeric \n"
        "\t\t value. The arguments, numN may be any  interger  or \n"
        "\t\t floating point values. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t min num1 num2 ?..numN? \n"
        "\t\t Returns  the  argument  that has the lowest numeric \n"
        "\t\t value. The arguments, numN may be any  interger  or \n"
        "\t\t floating point values. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t mkdir ?-path? dirList \n"
        "\t\t Create each of the directories in the list dirList. \n"
        "\t\t The mode on the new directories is 777, modified by \n"
        "\t\t the  umask.   If  -path is specified, then any non- \n"
        "\t\t existent  parent  directories  in   the   specified \n"
        "\t\t path(s) are also created. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t nice ?priorityincr? \n"
        "\t\t Change  or return the process priority.  If priori- \n"
        "\t\t tyincr  is  omitted,  the   current   priority   is \n"
        "\t\t returned.  If priorityincr is positive, it is added \n"
        "\t\t to the current  priority  level,  up  to  a  system \n"
        "\t\t defined maximum (normally 19), \n"
        "\n"
        "\t\t Negative  priorityincr values cumulatively increase \n"
        "\t\t the program's priority down  to  a  system  defined \n"
        "\t\t minimum  (normally  -19);  increasing priority with \n"
        "\t\t negative niceness values will  only  work  for  the \n"
        "\t\t superuser. \n"
        "\n"
        "\t\t The new priority is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t TCL LOADABLE LIBRARIES AND PACKAGES \n"
        "\t Extended  Tcl supports standard Tcl tclIndex libraries and \n"
        "\t package libraries. A package library file can contain mul- \n"
        "\t tiple independent Tcl packages.  A package is a named col- \n"
        "\t lection of related Tcl procedures and initialization code. \n"
        "\n"
        "\t The package library file is just a regular Unix text file, \n"
        "\t editable with your favorite text editor, containing  pack- \n"
        "\t ages  of  Tcl  source  code. The package library file name \n"
        "\t must have the suffix .tlib.  An index file with the suffix \n"
        "\t .tndx,  corresponding  to  the package library.  The .tndx \n"
        "\t will be automatically created by Tcl whenever it is out of \n"
        "\t date  or  missing  (provided  there is write access to the \n"
        "\t directory. \n"
        "\n"
        "\t The variable auto_path contains a list of directories that \n"
        "\t are  searched  for  libraries.   The first time an unknown \n"
        "\t command trap is take, the indexes for  the  libraries  are \n"
        "\t loaded  into  memory. If the auto_path variable is changed \n"
        "\t during execution of a program,  it  will  be  re-searched. \n"
        "\t Only  the  first  package of a given name found during the \n"
        "\t execution of a program is loaded.  This can be  overridden \n"
        "\t with loadlibindex command. \n"
        "\n"
        "\t The start of a package is delimited by: \n"
        "\n"
        "\t\t #@package: package_name proc1 ?..procN? \n"
        "\n"
        "\t These  lines must start in column one.  Everything between \n"
        "\t the #@package: keyword and the next #@package: keyword  or \n"
        "\t a  #@packend keyword, or the end of the file, becomes part \n"
        "\t of  the  named   package.    The   specified   procedures, \n"
        "\t proc1..procN, are the entry points of the package.  When a \n"
        "\t command named in a package specification is  executed  and \n"
        "\t detected  as an unknown command, all code in the specified \n"
        "\t package will be sourced.  This package should  define  all \n"
        "\t of  the  procedures  named on the package line, define any \n"
        "\t support procedures required by  the  package  and  do  any \n"
        "\t package-specific  initialization.   Packages  declarations \n"
        "\t maybe continued on subsequent  lines  using  standard  Tcl \n"
        "\t backslash  line  continuations.   The #@packend keyword is \n"
        "\t useful to make sure only the minimum required  section  of \n"
        "\t code  is  sourced.  Thus for example a large comment block \n"
        "\t at the beginning of the next file won't be loaded. \n"
        "\n"
        "\t Care should be taken  in  defining  package_name,  as  the \n"
        "\t first  package  found  in the path by with a given name is \n"
        "\t loaded.  This can be useful in developing new  version  of \n"
        "\t packages installed on the system. \n"
        "\n"
        "\t For example, in a package source file, the presence of the \n"
        "\t following line: \n"
        "\n"
        "\t\t #@package: directory_stack pushd popd dirs \n"
        "\t says that the text lines following that line in the  pack- \n"
        "\t age  file  up  to  the next package line or the end of the \n"
        "\t file is  a  package  named  directory_stack  and  that  an \n"
        "\t attempt  to  execute  either  pushd, popd or dirs when the \n"
        "\t routine is not  already  defined  will  cause  the  direc- \n"
        "\t tory_stack portion of the package file to be loaded. \n"
        "\n"
        "\t This functionallity is provided by Extended Tcl. \n",
    "\n"
    "\t pipe ?fileId_var_r fileId_var_w? \n"
        "\t\t Create  a  pipe.   If fileId_var_r and fileId_var_r \n"
        "\t\t are specified, then pipe will set  the  a  variable \n"
        "\t\t named  fileId_var_r  to  contain  the fileId of the \n"
        "\t\t side of the pipe that was opened for  reading,  and \n"
        "\t\t fileId_var_w will contain the fileId of the side of \n"
        "\t\t the pipe that was opened for writing. \n"
        "\n"
        "\t\t If the fileId variables are not specified,  then  a \n"
        "\t\t list  containing  the  read  and  write  fileIdw is \n"
        "\t\t returned as the result of the command. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t popd \n"
        "\t\t This  procedure  pops  the top directory entry from \n"
        "\t\t the directory stack and make it the current  direc- \n"
        "\t\t tory. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t profile ?-commands? on \n"
        "\t\t profile off arrayVar \n"
        "\t\t This  command is used to collect a performance pro- \n"
        "\t\t file of a Tcl script.  It collects data at the  Tcl \n"
        "\t\t procedure  level.  The  number of calls to a proce- \n"
        "\t\t dure, and the amount of real and CPU time  is  col- \n"
        "\t\t lected.  Time is also collected for the global con- \n"
        "\t\t text.  The procedure data is collected by bucketing \n"
        "\t\t it  based  on the procedure call stack, this allows \n"
        "\t\t determination of how much time is spent in  a  par- \n"
        "\t\t ticular procedure in each of it's calling contexts. \n"
        "\n"
        "\t\t The on option enables profile data  collection.  If \n"
        "\t\t the  -commands option is specifed, data on all com- \n"
        "\t\t mands within a procedure is  collected  as  well  a \n"
        "\t\t procedures.   Multiple  occurrences  of  a  command \n"
        "\t\t within a procedure are not distinguished, but  this \n"
        "\t\t data may still be useful for analysis. \n"
        "\n"
        "\t\t The  off  option  turns off profiling and moves the \n"
        "\t\t data collected to the array arrayVar.  The array is \n"
        "\t\t address  by  a  list  containing the procedure call \n"
        "\t\t stack.  Element zero is the top of the  stack,  the \n"
        "\t\t procedure  that  the data is for.  The data in each \n"
        "\t\t entry is a list consisting of  the  procedure  call \n"
        "\t\t count  and  the real time and CPU time in millisec- \n"
        "\t\t onds spent in the procedure (and all procedures  it \n"
        "\t\t called).  The list is in the form {count real cpu}. \n"
        "\t\t A Tcl procedure profrep is  supplied  for  reducing \n"
        "\t\t the data and producing a report \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t profrep profDataVar sortKey stackDepth ?outFile? ?userTi- \n"
        "\t\t tle? \n"
        "\t\t This procedure generates a report from data collect \n"
        "\t\t from  the profile command.  ProfDataVar is the name \n"
        "\t\t of the array containing the data  returned  by  the \n"
        "\t\t profile command. SortKey indicates which data value \n"
        "\t\t to sort by.  It should be one of \"calls\", \"cpu\"  or \n"
        "\t\t \"real\".   StackDepth is the stack depth to consider \n"
        "\t\t significant.  A value of  1  reports  by  procedure \n"
        "\t\t only,  a  value  of  2 reports by procedure and its \n"
        "\t\t caller, etc. OutFile is the name of file  to  write \n"
        "\t\t the  report  to.   If  omitted,  stdout is assumed. \n"
        "\t\t UserTitle is an optional title line to add to  out- \n"
        "\t\t put. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t pushd ?dir? \n"
        "\t\t This procedure pushs the current directory onto the \n"
        "\t\t directory stack and cd to the specified  directory. \n"
        "\t\t If the directory is not specified, then the current \n"
        "\t\t directory is pushed, but remains unchanged. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t random limit | seed ?seedval? \n"
        "\t\t Generate a pseudorandom integer number greater than \n"
        "\t\t or equal to zero and less than limit.  If  seed  is \n"
        "\t\t specified,  then the command resets the random num- \n"
        "\t\t ber generator to a starting point derived from  the \n"
        "\t\t seedval.  This allows one to reproduce pseudorandom \n"
        "\t\t number sequences for testing purposes.  If  seedval \n"
        "\t\t is  omitted,  then the seed is set to a value based \n"
        "\t\t on current system state and the current time,  pro- \n"
        "\t\t viding  a  reasonably interesting and ever-changing \n"
        "\t\t seed. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t read_file ?-nonewline? fileName \n"
        "\t\t read_file fileName numBytes \n"
        "\t\t This  proecure  reads the file fileName and returns \n"
        "\t\t the contents as a string.  If -nonewline is  speci- \n"
        "\t\t fied,  then  the last character of the file is dis- \n"
        "\t\t carded if it is a newline.  The second form  speci- \n"
        "\t\t fies  exactly  how  many  bytes  will  be  read and \n"
        "\t\t returned, unless  there  are  fewer  than  numBytes \n"
        "\t\t bytes  left  in  the  file;  in  this case, all the \n"
        "\t\t remaining bytes are returned. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t readdir dirPath \n"
        "\t\t Returns  a  list  containing  the  contents  of the \n"
        "\t\t directory dirPath.  The directory entries  \".\"  and \n"
        "\t\t \"..\" are not returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t recursive_glob dirlist globlist \n"
        "\t\t This   procedure  returns  a  list  of  recursively \n"
        "\t\t matches files.   All  directories  in  dirlist  are \n"
        "\t\t recursively   searched  (breadth-first),  comparing \n"
        "\t\t each file found against the file glob  patterns  in \n"
        "\t\t globlist.  Symbolic links are not followed. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t replicate string countExpr \n"
        "\t\t Returns  string,  replicated  the  number  of times \n"
        "\t\t indicated by the expression countExpr. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t rmdir ?-nocomplain? dirList \n"
        "\t\t Remove each of the directories in the list dirList. \n"
        "\t\t If -nocomplain is specified, then  errors  will  be \n"
        "\t\t ignored. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t saveprocs fileName ?proc...? \n"
        "\t\t This  prodcure  saves  the  definition of the named \n"
        "\t\t procedure, or all currently defined  procedures  if \n"
        "\t\t none is specified, to the named file. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t scancontext ?option? \n"
        "\t\t This  command  manages  file scan contexts.  A scan \n"
        "\t\t context is a collection of regular expressions  and \n"
        "\t\t commands  to  execute  when that regular expression \n"
        "\t\t matches a line of the file.   A  context  may  also \n"
        "\t\t have  a single default match, to be applied against \n"
        "\t\t lines that do not match any of the regular  expres- \n"
        "\t\t sions.   Multiple  scan contexts may be defined and \n"
        "\t\t they may be reused on multiple files.  A scan  con- \n"
        "\t\t text  is identified by a context handle.  The scan- \n"
        "\t\t context command takes the following forms: \n"
        "\n"
        "\t scancontext create \n"
        "\t\t Create a new scan context.  The  scanmatch  command \n"
        "\t\t is  used to define patterns in the context.  A con- \n"
        "\t\t texthandle is returned, which  the  Tcl  programmer \n"
        "\t\t uses  to refer to the newly created scan context in \n"
        "\t\t calls to the Tcl file scanning commands. \n"
        "\n"
        "\t scancontext delete contexthandle \n"
        "\t\t Delete  the  scan  context  identified  by  contex- \n"
        "\t\t thandle,  and  free all of the match statements and \n"
        "\t\t compiled regular expressions  associated  with  the \n"
        "\t\t specified context. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t scanfile ?-copyfile copyFileId? contexthandle fileId \n"
        "\t\t Scan  the  file  specified by fileId, starting from \n"
        "\t\t the current file position.  Check all  patterns  in \n"
        "\t\t the scan context specified by contexthandle against \n"
        "\t\t it, executing the match commands  corresponding  to \n"
        "\t\t patterns matched. \n"
        "\n"
        "\t\t If  the  optional  -copyfile argument is specified, \n"
        "\t\t the next argument is a file ID to which  all  lines \n"
        "\t\t not  matched  by any pattern (excluding the default \n"
        "\t\t pattern) are to be written. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t scanmatch ?-nocase? contexthandle ?regexp? commands \n"
        "\t\t Specify  Tcl  commands, to be evaluated when regexp \n"
        "\t\t is matched by a scanfile  command.   The  match  is \n"
        "\t\t added  to  the  scan  context  specified by contex- \n"
        "\t\t thandle.  Any number of  match  statements  may  be \n"
        "\t\t specified  for a give context.  Regexp is a regular \n"
        "\t\t expression (see the regexp command).  If -nocase is \n"
        "\t\t specified  as  the  first  argument, the pattern is \n"
        "\t\t matched regardless of alphabetic case. \n"
        "\n"
        "\t\t If regexp is not specified, then a default match is \n"
        "\t\t specified  for the scan context.  The default match \n"
        "\t\t will be executed when a line of the file  does  not \n"
        "\t\t match any of the regular expressions in the current \n"
        "\t\t scancontext. \n"
        "\n"
        "\t\t The array matchInfo is available to  the  Tcl  code \n"
        "\t\t that  is  executed  when  an expression matches (or \n"
        "\t\t defaults).  It contans information about  the  file \n"
        "\t\t being  scanned  and  where within it the expression \n"
        "\t\t was matched. \n"
        "\n"
        "\t\t matchInfo is local to the top level  of  the  match \n"
        "\t\t command unless declared global at that level by the \n"
        "\t\t Tcl global command.  If it  is  to  be  used  as  a \n"
        "\t\t global,  it must be declared global before scanfile \n"
        "\t\t is called (since scanfile sets the matchInfo before \n"
        "\t\t the  match  code  is  executed, a subsequent global \n"
        "\t\t will override the local variable).   The  following \n"
        "\t\t array entries are available: \n"
        "\n"
        "\t\t matchInfo(line) \n"
        "\t\t\t Contains the text of the line of the file that \n"
        "\t\t\t was matched. \n"
        "\n"
        "\t\t matchInfo(offset) \n"
        "\t\t\t The byte offset into the  file  of  the  first \n"
        "\t\t\t character of the line that was matched. \n"
        "\n"
        "\t\t matchInfo(linenum) \n"
        "\t\t\t The  line number of the line that was matched. \n"
        "\t\t\t This is relative to the  first  line  scanned, \n"
        "\t\t\t which  is  usually,  but  not necessarily, the \n"
        "\t\t\t first line of the file.   The  first  line  is \n"
        "\t\t\t line number one. \n"
        "\n"
        "\t\t matchInfo(handle) \n"
        "\t\t\t The  file  id  (handle)  of the file currently \n"
        "\t\t\t being scanned. \n"
        "\n"
        "\t\t matchInfo(copyHandle) \n"
        "\t\t\t The file id (handle) of the file specified  by \n"
        "\t\t\t the  -copyfile  option.   The element does not \n"
        "\t\t\t exist if -copyfile was not specified. \n"
        "\t\t matchInfo(submatch0) \n"
        "\t\t\t Will contain the characters matching the first \n"
        "\t\t\t parenthesized  subexpression.  The second will \n"
        "\t\t\t be contained in submatch1, etc. \n"
        "\n"
        "\t\t matchInfo(subindex0) \n"
        "\t\t\t Will contain the a list of  the  starting  and \n"
        "\t\t\t ending  indices  of  the  string  matching the \n"
        "\t\t\t first parenthesized subexpression.  The second \n"
        "\t\t\t will be contained in subindex1, etc. \n"
        "\n"
        "\t All scanmatch patterns that match a line will be processed \n"
        "\t in the order in which their specifications were  added  to \n"
        "\t the scan context.  The remainder of the scanmatch pattern- \n"
        "\t command pairs may be skipped for a file line if a continue \n"
        "\t is  executed  by the Tcl code of a preceding, matched pat- \n"
        "\t tern. \n"
        "\n"
        "\t If a return is executed in the body of the match  command, \n"
        "\t the  scanfile  command currently in progress returns, with \n"
        "\t the value passed to return as its return value. \n"
        "\n"
        "\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t searchpath path file \n"
        "\t\t Search  all  directories  in  the  specified path, which is a Tcl \n"
        "\t\t list, for the specified file.  Returns the full path name of  the \n"
        "\t\t file,  or  an  empty  string  if  the requested file could not be \n"
        "\t\t found. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t select readfileIds ?writefileIds? ?exceptfileIds? ?time- \n"
        "\t\t out? \n"
        "\t\t This command allows an Extended Tcl program to wait \n"
        "\t\t on  zero or more files being ready for for reading, \n"
        "\t\t writing, have an exceptional condition pending,  or \n"
        "\t\t for  a  timeout  period  to  expire.   readFileIds, \n"
        "\t\t writeFileIds,  exceptFileIds  are  each  lists   of \n"
        "\t\t fileIds, as returned from open, to query.  An empty \n"
        "\t\t list ({}) may be specified if  a  category  is  not \n"
        "\t\t used. \n"
        "\n"
        "\t\t The  files  specified  by  the readFileIds list are \n"
        "\t\t checked to see if data is  available  for  reading. \n"
        "\t\t The writeFileIds are checked if the specified files \n"
        "\t\t are  clear  for  writing.   The  exceptFileIds  are \n"
        "\t\t checked  to  see  if  an  exceptional condition has \n"
        "\t\t occured  (typically,  an  error).   The  write  and \n"
        "\t\t exception  checking is most useful on devices, how- \n"
        "\t\t ever, the read checking is very useful when  commu- \n"
        "\t\t nicating  with  multiple  processes  through pipes. \n"
        "\t\t Select considers data pending in  the  stdio  input \n"
        "\t\t buffer  for  read files as being ready for reading, \n"
        "\t\t the files do.  not have to be unbuffered. \n"
        "\n"
        "\t\t Timeout is a floating point timeout value, in  sec- \n"
        "\t\t onds.  If an empty list is supplied (or the parame- \n"
        "\t\t ter is omitted), then no timeout is  set.   If  the \n"
        "\t\t value is zero, then the select command functions as \n"
        "\t\t a poll of the files, returning immediately even  if \n"
        "\t\t none are ready. \n"
        "\n"
        "\t\t If  the  timeout  period  expires  with none of the \n"
        "\t\t files becomming ready, then the command returns  an \n"
        "\t\t empty  list.   Otherwise the command returns a list \n"
        "\t\t of three elements, each of those elements is a list \n"
        "\t\t of  the  fileIds  that are ready in the read, write \n"
        "\t\t and exception classes.  If  none  are  ready  in  a \n"
        "\t\t class,  then  that  element  will be the null list. \n"
        "\t\t For example: \n"
        "\n"
        "\t\t\t select {file3 file4 file5} {file6 file7} {} 10.5 \n"
        "\n"
        "\t\t could return \n"
        "\n"
        "\t\t\t {file3 file4} {file6} {} \n"
        "\n"
        "\t\t or perhaps \n"
        "\n"
        "\t\t\t file3 {} {} \n"
        "\n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t server_open ?option? host service \n"
        "\t\t Open a TCP/IP connection to a server of host on the \n"
        "\t\t port specified by  service.   The  server  is  then \n"
        "\t\t accessed  using the standard Tcl file I/O commands. \n"
        "\t\t Host may be a host name or an IP address.  Port may \n"
        "\t\t be  a port number of a service name.  Option is one \n"
        "\t\t of -buf or -nobuf.  If  no  options  is  specified, \n"
        "\t\t -buf is the default. \n"
        "\n"
        "\t\t If  -buf  is  specified, then the file is buffered. \n"
        "\t\t In this case, a pair of Tcl file ids  are  returned \n"
        "\t\t in  a  list.  The first id is open for read access, \n"
        "\t\t the second for write.  When writing  to  the  file, \n"
        "\t\t the flush command must be used to force data in the \n"
        "\t\t buffer to be sent to the server.  The close command \n"
        "\t\t must  be  called against both file ids when through \n"
        "\t\t with the socket.  Buffered access  will  result  in \n"
        "\t\t significantly  better performance when reading data \n"
        "\t\t and will also improve a series of writes being done \n"
        "\t\t with  out  intermixed reads.  The fcntl command may \n"
        "\t\t be used to make one of these files unbuffered. \n"
        "\n"
        "\t\t if -nobuf is specified, then the file is unbuffered \n"
        "\t\t and  a  single  file  id, open for both reading and \n"
        "\t\t writing, is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t showproc ?procname ...? \n"
        "\t\t This  procedure  lists  the definition of the named \n"
        "\t\t procedures.  Loading them  if  it  is  not  already \n"
        "\t\t loaded.   If  no  procedure names are supplied, the \n"
        "\t\t definitions of all currently loaded procedures  are \n"
        "\t\t returned. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t signal action siglist ?command? \n"
        "\t\t Specify  the  action  to take when a Unix signal is \n"
        "\t\t received by Extended Tcl, or a program that  embeds \n"
        "\t\t it.   Siglist  is  a list of either the symbolic or \n"
        "\t\t numeric Unix signal (the SIG prefix  is  optional). \n"
        "\t\t Action  is  one of the following actions to be per- \n"
        "\t\t formed on receipt of the signal. \n"
        "\n"
        "\t\t default - Perform system default action when signal \n"
        "\t\t is received (see signal system call documentation). \n"
        "\n"
        "\t\t ignore - Ignore the signal. \n"
        "\n"
        "\t\t error - Generate a catchable Tcl error.  It will be \n"
        "\t\t as  if  the  command  that  was running returned an \n"
        "\t\t error.  The error code will be in the form: \n"
        "\t\t\t POSIX SIG signame \n"
        "\t\t For the death of child signal, signame will  always \n"
        "\t\t be  SIGCHLD,  rather  than SIGCLD, to allow writing \n"
        "\t\t portable code. \n"
        "\n"
        "\t\t trap - When the signal occurs, execute command  and \n"
        "\t\t continue  execution  if an error is not returned by \n"
        "\t\t command.  The  command  will  be  executed  in  the \n"
        "\t\t global  context.  The command will be edited before \n"
        "\t\t execution, replacing occurrences of \"%S\"  with  the \n"
        "\t\t signal  name.  Occurrences of \"%%\" result in a sin- \n"
        "\t\t gle \"%\".  This editing occurs just before the  trap \n"
        "\t\t command  is  evaluated.   If  an error is returned, \n"
        "\t\t then  follow  the  standard  Tcl  error  mechanism. \n"
        "\t\t Often command will just do an exit. \n"
        "\n"
        "\t\t get  -  Retrieve the current settings of the speci- \n"
        "\t\t fied signals.  A keyed list will be  returned  were \n"
        "\t\t the  keys  are one of the specified signals and the \n"
        "\t\t values are a list cosisting of the  action  associ- \n"
        "\t\t ated  with  the  signal,  a  0 if the signal may be \n"
        "\t\t delivered (not block) and a 1 if it is blocked. The \n"
        "\t\t actions maybe one of `default',`ignore', `error' or \n"
        "\t\t `trap.  If the action is trap, the third element is \n"
        "\t\t the command associated with the action. \n"
        "\n"
        "\t\t block  -  Block  the  specified  signals from being \n"
        "\t\t received. (Posix systems only). \n"
        "\n"
        "\t\t unblock  -  Allow  the  specified  signal   to   be \n"
        "\t\t received.  Pending  signals  will not occur. (Posix \n"
        "\t\t systems only). \n"
        "\n"
        "\t\t The signal action will  remain  enabled  after  the \n"
        "\t\t specified  signal  has  occurred.  The exception to \n"
        "\t\t this is SIGCHLD on systems without  Posix  signals. \n"
        "\t\t For  these systems, SIGCHLD is not be automatically \n"
        "\t\t reenabled.  After a SIGCHLD signal is  received,  a \n"
        "\t\t call to wait must be performed to retrieve the exit \n"
        "\t\t status of the child process before issuing  another \n"
        "\t\t signal SIGCHLD ... command.  For code that is to be \n"
        "\t\t portable between both types of  systems,  use  this \n"
        "\t\t approach. \n"
        "\n"
        "\t\t Signals  are  not processed until after the comple- \n"
        "\t\t tion of the Tcl command that is executing when  the \n"
        "\t\t signal is received.  If an interactive Tcl shell is \n"
        "\t\t running, then the SIGINT will be set to error, non- \n"
        "\t\t interactive  Tcl  sessions  leave  SIGINT unchanged \n"
        "\t\t from when the process started (normally default for \n"
        "\t\t foreground  processes  and  ignore for processes in \n"
        "\t\t the background). \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t sleep seconds \n"
        "\t\t Sleep the Extended Tcl process for seconds seconds. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t sync ?fileId? \n"
        "\t\t If  fileId  is  not specified, or if it is and this \n"
        "\t\t system does not  support  the  fsync  system  call, \n"
        "\t\t issues a sync system call to flush all pending disk \n"
        "\t\t output.  If fileId is specified and the system does \n"
        "\t\t support  the  fsync system call, issues an fsync on \n"
        "\t\t the file corresponding to the specified Tcl  fileId \n"
        "\t\t to force all pending output to that file out to the \n"
        "\t\t disk. \n"
        "\n"
        "\t\t If fileId is specified, the file must be  writable. \n"
        "\t\t A  flush  will  be issued against the fileId before \n"
        "\t\t the sync. \n"
        "\n"
        "\t\t The infox have_fsync command can be used to  deter- \n"
        "\t\t mine if \"sync fileId\" will do a sync or a fsync. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t system command \n"
        "\t\t Executes  command  via the system(3) call.  Differs \n"
        "\t\t from exec in that system doesn't  return  the  exe- \n"
        "\t\t cuted  command's  standard  output  as  the  result \n"
        "\t\t string, and system goes through the Unix  shell  to \n"
        "\t\t provide wildcard expansion, redirection, etc, as is \n"
        "\t\t normal from an sh command line.  The exit  code  of \n"
        "\t\t the command is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t times \n"
        "\t\t Return a list containing the process and child exe- \n"
        "\t\t cution times in the form: \n"
        "\t\t\t utime stime cutime cstime \n"
        "\t\t Also see the times(2) system call manual page.  The \n"
        "\t\t values are in milliseconds. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t translit inrange outrange string \n"
        "\t\t Translate characters in string, changing characters \n"
        "\t\t occuring in inrange to the corresponding  character \n"
        "\t\t in  outrange.  Inrange  and outrange may be list of \n"
        "\t\t characters or a range in the form `A-M'.  For exam- \n"
        "\t\t ple: \n"
        "\t\t\t translit a-z A-Z foobar \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t umask ?octalmask? \n"
        "\t\t Sets  file-creation mode mask to the octal value of \n"
        "\t\t octalmask.  If octalmask is  omitted,  the  current \n"
        "\t\t mask is returned. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t union lista listb \n"
        "\t\t Procedure  to  return  the logical union of the two \n"
        "\t\t specified  lists.   Any  duplicate   elements   are \n"
        "\t\t removed. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n",
    "\n"
    "\t unlink ?-nocomplain? filelist \n"
        "\t\t Delete  (unlink)  the  files whose names are in the \n"
        "\t\t list filelist.  If -nocomplain is  specified,  then \n"
        "\t\t errors will be ignored. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t The  following  variables  are  set and/or used by the Tcl \n"
        "\t shell. \n"
        "\n"
        "\t argv0  Contains the name of the Tcl program  specified  on \n"
        "\t\t the command line or the name that the Tcl shell was \n"
        "\t\t invoked under if no program  was  specified.   argc \n"
        "\t\t Contains a count of the number of argv arguments (0 \n"
        "\t\t if none).  argv A  list  containing  the  arguments \n"
        "\t\t passed  in  from  the command line, excluding argu- \n"
        "\t\t ments used by the Tcl shell.  The first element  is \n"
        "\t\t the first passed argument, not the program name. \n"
        "\n"
        "\t tcl_interactive \n"
        "\t\t Set  to 1 if Tcl shell is invoked interactively, or \n"
        "\t\t 0 if the Tcl shell is directly executing a  script. \n"
        "\t\t Normally  checked by scripts so that they can func- \n"
        "\t\t tion as a standalone application  if  specified  on \n"
        "\t\t the  command  line, but merely load in and not exe- \n"
        "\t\t cute if loaded during an interactive invocation  of \n"
        "\t\t Tcl. \n"
        "\n"
        "\t auto_path \n"
        "\t\t Path  to search to locate Tcl scripts.  Used by the \n"
        "\t\t auto_load command and the TclX unknown command han- \n"
        "\t\t dler.  The path is a Tcl list of directory names. \n"
        "\n"
        "\t tcl_prompt1 \n"
        "\t\t Contains code to run to output the prompt used when \n"
        "\t\t interactively prompting for commands. \n"
        "\n"
        "\t tcl_prompt2 \n"
        "\t\t Contains code to run to output the prompt used when \n"
        "\t\t interactively  prompting  for  continuation  of  an \n"
        "\t\t incomplete command. \n"
        "\n"
        "\t TCLXENV \n"
        "\t\t Array that contains information used internally  by \n"
        "\t\t various  Tcl  procedures  that are part of the TclX \n"
        "\t\t shell.  Don't change this  array  unless  you  know \n"
        "\t\t what your doing. \n",
    "\n"
    "\t wait ?-nohang? ?-untraced? ?-pgroup? ?pid? \n"
        "\t\t Waits  for a process created with the execl command \n"
        "\t\t to terminate, either due to an untrapped signal  or \n"
        "\t\t call to exit system call.  If the process id pid is \n"
        "\t\t specified, they wait  on  that  process,  otherwise \n"
        "\t\t wait on any child process to terminate. \n"
        "\n"
        "\t\t If  -nohang  is specified, then don't block waiting \n"
        "\t\t on a process to terminate.  If no process is  imme- \n"
        "\t\t diately   available,  return  an  empty  list.   If \n"
        "\t\t -untraced is specified then  the  status  of  child \n"
        "\t\t processes  that  are  stopped, and whose status has \n"
        "\t\t not yet been reported since they stopped, are  also \n"
        "\t\t returned.   If  -pgroup  is specfied and pid is not \n"
        "\t\t specified, then wait on  any  child  process  whose \n"
        "\t\t process  groupd ID is they same as the calling pro- \n"
        "\t\t cess. If pid is specified with -pgroup, then it  is \n"
        "\t\t take  as a process group ID, waiting on any process \n"
        "\t\t in that process group to terminate. \n"
        "\n"
        "\t\t Wait returns a list containing three elements:  The \n"
        "\t\t first element is the process id of the process that \n"
        "\t\t terminated.  If the process  exited  normally,  the \n"
        "\t\t second  element  is  `EXIT', and the third contains \n"
        "\t\t the numeric exit code.  If the  process  terminated \n"
        "\t\t due  to  a signal, the second element is `SIG', and \n"
        "\t\t the third contains the signal name.  If the process \n"
        "\t\t is  currently  stopped  (on  systems  that  support \n"
        "\t\t SIGSTP), the second element is `STOP', followed  by \n"
        "\t\t the signal name. \n"
        "\n"
        "\t\t Note  that  it  is possible to wait on processes to \n"
        "\t\t terminate that were create in the  background  with \n"
        "\t\t the  exec command.  However, if any other exec com- \n"
        "\t\t mand is executed after the process terminates, then \n"
        "\t\t the  process status will be reaped by the exec com- \n"
        "\t\t mand and will not be available to the wait command. \n"
        "\n"
        "\t\t This command is provided by Extended Tcl. \n",
    "\n"
    "\t write_file fileName string ?string...? \n"
        "\t\t This  procedure writes the specified strings to the \n"
        "\t\t named file. \n"
        "\n"
        "\t\t This procedure is provided by Extended Tcl. \n"
};


int ftclMsgDefine (interp)

    Tcl_Interp *interp;

{

    int  msg_size=0;
    int	 i =0;

/* ------------------------------------------------------------------------------- */
    msg_size = (sizeof (help_cmd_list) / sizeof(help_cmd_list[0]) );

/* Definig messages for ftcl commands : we use "ftcl" as facility name.		  */

   for (i =0; i<msg_size; i++)
   {
     ftclHelpDefine ( interp, "ftcl", help_cmd_list[i], help_cmd_msg[i]); 
   }    

    msg_size = (sizeof (tclXhelp_cmd_list) / sizeof(tclXhelp_cmd_list[0]) );

/* Definig messages for tclX commands : we use "ftcl" as facility name.		  */

   for (i =0; i<msg_size; i++)
   {
     ftclHelpDefine ( interp, "ftcl", tclXhelp_cmd_list[i], tclXhelp_cmd_msg[i]); 
   }    

    return TCL_OK;
}

