#Convert from deg to dd.mmss
proc dms {deg} {
   if {$deg >= 0} then {set isign 1} else {set isign -1}
   set deg [expr {[fabs $deg] * 3600.}]
   set dd [floor [expr {$deg/3600.}]]
   set mm [floor [expr {($deg - $dd*3600)/60.}]]
   set ss [expr {$deg - $dd*3600 - $mm*60}]
   set val [expr {$isign*($dd + $mm/100. +$ss/10000.)}]
   }

#routine to convert from dd.mmss to deg.
proc deg {dms} {
   if {$dms >= 0} then {set isign 1} else {set isign -1}
   set dms [expr {[fabs $dms] * 10000.}]
   set dd [int [floor [expr {$dms/10000.}]]]
   set mm [int [floor [expr {($dms - $dd*10000)/100.}]]]
   set ss [expr {$dms - $dd*10000 - $mm*100}]
   set val [expr {$isign*($dd + $mm/60. +$ss/3600.)}]
   }

# Convert from deg to dd:mm:ss.s
proc ddmmss {deg} {
   if {$deg >= 0} then {set isign " "} else {set isign "-"}
   set deg [expr {[fabs $deg] * 3600.}]
   set dd [int [floor [expr {$deg/3600.}]]]
   set mm [int [floor [expr {($deg - $dd*3600)/60.}]]]
   set ss [expr {$deg - $dd*3600 - $mm*60}]
   format "%s%02d:%02d:%04.1f" $isign $dd $mm $ss
   }

# Convert from hr to hh:mm:ss.ss
proc hhmmss {hr} {
   set hr [expr {[fabs $hr] * 3600.}]
   set hh [int [floor [expr {$hr/3600.}]]]
   set mm [int [floor [expr {($hr - $hh*3600)/60.}]]]
   set ss [expr {$hr - $hh*3600 - $mm*60}]
   format "%02d:%02d:%02.2f" $hh $mm $ss
   }

# Convert from hh:mm:ss.ss to hr (works on degrees as well)
proc hr {hhmmss} {
   set list [split $hhmmss :]
   set ihr [lindex $list 0]
   set min [lindex $list 1]
   set sec [lindex $list 2]
#Strip any leading sign
   set sign 1
   if {[string index $ihr 0] == "-"} then {set sign -1}
   set ihr [string trim $ihr +-]
#Must trim leading 0's - else TCL treats numbers as octal.  Bummer
#Actually, since there is never more than 1 digit after the leading 0, TCL
#will work OK in this case.
   regsub {^0*([0-9])} $ihr \\1 ihr
   regsub {^0*([0-9])} $min \\1 min
   regsub {^0*([0-9])} $sec \\1 sec
   return [expr $sign*($ihr+($min/60.)+($sec/3600.))]
   }

########################################################################
#Works only for the north.

proc eqToStripe {ra dec} {
   set list [eqToSurvey $ra $dec]
   set eta [keylget list eta]
   set stripe [expr round(($eta+32.5)/2.5 + 10)]
   set incl [expr ($stripe-10)*2.5]
   set list [eqToGC $ra $dec -node 95 -inclination $incl]
   echo $list
   return $stripe
   }

##########################################################################
#Convert equatorial to azel, including position angle relative to zenith
#of north.
#Return as a keyed list.  Input is actually hour angle, not ra.
#Third argument is latitude.

proc eqToAzel {ha dec {phi 32.8}} {
   set DEG2RAD [expr 3.14159265358979/180.]
   set rha [expr $ha*$DEG2RAD]
   set rdec [expr $dec*$DEG2RAD]
   set rphi [expr $phi*$DEG2RAD]
   set x1 [expr cos($rdec)*sin($rha)]
   set y1 [expr cos($rdec)*cos($rha)]
   set z1 [expr sin($rdec)]
   set x $x1
   set y [expr $y1*sin($rphi) - $z1*cos($rphi)]
   set z [expr $z1*sin($rphi) + $y1*cos($rphi)]
   set z [expr max(min($z,1.),-1.)]
   set el [expr asin($z)/$DEG2RAD]
   set az [expr atan2(-1.*$x,-1.*$y)/$DEG2RAD]

#Position angle (ccw w.r.t. zenith direction)
   set pa [expr atan2(-sin($rha)*cos($rphi), cos($rdec)*sin($rphi) - \
	cos($rha)*sin($rdec)*cos($rphi))/$DEG2RAD]
   keylset list az [format %.5f $az]
   keylset list el [format %.5f $el]
   keylset list pa [format %.5f $pa]
   return $list
   }

#############################################################
proc azelToEq {az el {phi 32.8}} {
   set list [atAzEl2HaDec $az $el $phi]
   set ha [keylget list ha]
   set dec [keylget list dec]
   set newlist [eqToAzel $ha $dec $phi]
   set pa [keylget newlist pa]

#Position angle of zenith w.r.t. north
   set pa [expr -1.*$pa]
   keylset list pa [format %.5f $pa]
   return $list
   }
