global rootDir
set rootDir /data/dp30.a/data

###############################################################
#Input is a list of lists (many variations possible).
#Expans any range specifications.  Return a single list

proc fieldExpand {args} {
   set outlist ""
   foreach arg $args {
	foreach field $arg {
	   if {[regexp {([0-9]+)-([0-9]+)} $field all f1 f2]} {
		loop i $f1 [expr $f2+1] {
		   lappend outlist $i
		   }
	   } else {
		lappend outlist $field
		}
	   }
	}
   return $outlist
   }

####################################################################
#Don't conflict with sdssmosaic commands.

#Read astrometric solutions. Store as chains in astrom array
#For now, asdir points to a specific directory, so we can only process
#one run at a time.

proc astromFields {run mu1 mu2} {
#We read 6 distinct tables.  Easiest to return results in an array of chains
   global asdir

#Array to store starting mu of fields for camcol 3
   global astromCoeff
   if {![info exists astromCoeff]} {
	set astromCoeff() ""
	}
   if {[lsearch $astromCoeff() $run] < 0} {
	if {![info exists asdir($run)]} {
	   astromDir $run
	   }
	lappend astromCoeff($run) $run
	set astromCoeff($run) ""
	set topdir $asdir($run)

#Get node and inclination from pdu header
	set camcol 3
	set filter r
	set hdu [expr ($camcol - 1)*5]
	set chain [fits2Schema [format $topdir/asTrans-%06d.fit $run] \
	   ASTRANS -hdu $hdu]
	chainEach hndl $chain {
	   set field [exprGet $hndl.field]
	   set a [exprGet $hndl.a]

#Get modulo 360
	   set a [expr fmod($a+360., 360.)]
	   lappend astromCoeff($run) $field
	   set astromCoeff($run,$field) $a
	   }
	genericChainDel $chain
	}
   set mu1 [expr $mu1-.15]
   set mu1 [expr fmod($mu1+360.,360.)]
   set mu2 [expr fmod($mu2+360.,360.)]

   if {$mu1 <= $mu2} {
	set sign 1
   } else {
	set sign -1
	}
   set fields ""
   foreach field $astromCoeff($run) {
	set a $astromCoeff($run,$field)
	if {$sign > 0} {
	   if {$a >= $mu1 && $a <= $mu2} {lappend fields $field}
	} else {
	   if {$a >= $mu2 || $a >= $mu2} {lappend fields $field}
	   }
	}
   if {[llength $fields] == 0} return

   return $fields
   }

####################################################################
#Determine directory (including rerun) for astrom info given a run.

proc astromDir {run} {
   global rootDir
   global asdir

#Root directory for a given run
   set dataDir $rootDir/$run

#Get reruns.  I will use them as follows:
#	Highest priority are single digit reruns in reverse numerical order.
#	Next highest are 2-digit reruns, etc.

   set dirs [glob -nocomplain $dataDir/*]
   set reruns1 ""
   set reruns2 ""
   set reruns3 ""
   set reruns4 ""
   foreach dir $dirs {
	if {![file isdirectory $dir]} continue
	set tail [file tail $dir]
	if {[regexp {^[0-9]$} $tail]} {lappend reruns1 $tail}
	if {[regexp {^[0-9][0-9]$} $tail]} {lappend reruns2 $tail}
	if {[regexp {^[0-9][0-9][0-9]$} $tail]} {lappend reruns3 $tail}
	if {[regexp {^[0-9][0-9][0-9][0-9]$} $tail]} {lappend reruns4 $tail}
	}
   set reruns1 [lsort -decreasing -integer $reruns1]
   set reruns2 [lsort -decreasing -integer $reruns2]
   set reruns3 [lsort -decreasing -integer $reruns3]
   set reruns4 [lsort -decreasing -integer $reruns4]
   set reruns [join "$reruns1 $reruns2 $reruns3 $reruns4"]

### astrom directory
   set asdir($run) $dataDir/astrom
   foreach rerun $reruns {
	if {$rerun >= 30 && $rerun < 40} continue
	if {$rerun > 99} continue
	set dir $dataDir/$rerun/astrom
	if {![file exists $dir]} continue
	set files [glob -nocomplain $dir/asTrans*]
	if {[llength $files] == 0} continue
	set asdir($run) $dir
	break
	}
   echo asdir($run) = $asdir($run)
   return
   }

####################################################################
#Determine directory (including rerun) for tsObj files

proc tsDir {run} {
   global rootDir
   global tsdir

#Root directory for a given run
   set dataDir $rootDir/$run

#Get reruns.  I will use them as follows:
#	Highest priority are single digit reruns in reverse numerical order.
#	Next highest are 2-digit reruns, etc.

   set dirs [glob -nocomplain $dataDir/*]
   set reruns1 ""
   set reruns2 ""
   set reruns3 ""
   set reruns4 ""
   foreach dir $dirs {
	if {![file isdirectory $dir]} continue
	set tail [file tail $dir]
	if {[regexp {^[0-9]$} $tail]} {lappend reruns1 $tail}
	if {[regexp {^[0-9][0-9]$} $tail]} {lappend reruns2 $tail}
	if {[regexp {^[0-9][0-9][0-9]$} $tail]} {lappend reruns3 $tail}
	if {[regexp {^[0-9][0-9][0-9][0-9]$} $tail]} {lappend reruns4 $tail}
	}
   set reruns1 [lsort -decreasing -integer $reruns1]
   set reruns2 [lsort -decreasing -integer $reruns2]
   set reruns3 [lsort -decreasing -integer $reruns3]
   set reruns4 [lsort -decreasing -integer $reruns4]
   set reruns [join "$reruns1 $reruns2 $reruns3 $reruns4"]

### astrom directory
   set tsdir($run) $dataDir/calibChunks
   set tsdir($run,rerun) ""
   foreach rerun $reruns {
	if {$rerun >= 30 && $rerun < 40} continue
	if {$rerun > 99} continue
	set dir $dataDir/$rerun/calibChunks
	if {![file exists $dir/1]} continue
	set tsdir($run) $dir
	set tsdir($run,rerun) $rerun
	break
	}
   echo tsdir($run) = $tsdir($run)
   return
   }

