#Emulate caxes

############################################################################
#Bind mouse functions.  This is the closest that I can get.

proc axcur {{nrow 7} {ncol 7}} {
   imgGlobals
   set coords [getcur]
   if {$coords == ""} return

   set row [int [lindex $coords 0]]
   set col [int [lindex $coords 1]]
   set reg [lindex $coords 2]
   set saoNum [lindex $coords 3]
   if {[catch {set hndl [imgAxes $reg $row $col $nrow $ncol]} msg]} {
	puts stdout $msg
	return
	}
   if {[info exists img(axes)]} {
	handleSetFromHandle $img(axes) $hndl
	genericDel $hndl
   } else {
	set img(axes) $hndl
	}
   set img(saoNum) $saoNum
   return [format "%10.2f %10.2f" [exprGet $img(axes).row] \
	[exprGet $img(axes).col]]
   }

############################################################3
#Like axcur, but use caxes instead

proc caxcur {{nrow 7} {ncol 7}} {
   imgGlobals
   set coords [getcur]
   if {$coords == ""} return

   set row [int [lindex $coords 0]]
   set col [int [lindex $coords 1]]
   set reg [lindex $coords 2]
   set saoNum [lindex $coords 3]
   if {[catch {set hndl [imgCaxes $reg $row $col $nrow $ncol]} msg]} {
	puts stdout $msg
	return
	}
   if {[info exists img(axes)]} {
	handleSetFromHandle $img(axes) $hndl
	genericDel $hndl
   } else {
	set img(axes) $hndl
	}
   set img(saoNum) $saoNum
   return [format "%10.2f %10.2f" [exprGet $img(axes).row] \
	[exprGet $img(axes).col]]
   }

############################################################################
#Bind mouse functions.  Just set axes to nearest pixel

proc cur {} {
   imgGlobals
   set coords [getcur]
   if {$coords == ""} return

#getcur returns integer coords of pixel.  Center is .5 higher
   set row [expr [lindex $coords 0] + .5]
   set col [expr [lindex $coords 1] + .5]
   set reg [lindex $coords 2]
   set saoNum [lindex $coords 3]
   if {![info exists img(axes)]} {
	set img(axes) [genericNew AXES]
	}
   handleSet $img(axes).row $row
   handleSet $img(axes).col $col
   set img(saoNum) $saoNum
   return [format "%10.2f %10.2f" [exprGet $img(axes).row] \
	[exprGet $img(axes).col]]
   }

#########################################################
#Draw axes in an saoDisplay

proc axDraw {chan} {
   imgGlobals
   if {![info exists img(axes)]} {error "No axes"}

#Channel can be either a letter or number. Convert to number.
   if {[regexp {[a-zA-Z]} $chan]} {
	if {![info exists saoNumber($chan)]} {error "No channel"}
	set chan $saoNumber($chan)
	}
   if {[lsearch [saoGetRegion] $chan] < 0} {error "No channel"}
   set row [exprGet $img(axes).row]
   set col [exprGet $img(axes).col]
   set ang [exprGet $img(axes).pos_maj]
   set eps [exprGet $img(axes).eps]
   set maj [exprGet $img(axes).rmax]
   set min [expr $maj*(1.-$eps)]
   saoLabel off $chan
   saoDrawEllipse $row $col $maj $min $ang -i -s $chan
   return
   }

##############################################################
#Close equivalent to axes

proc axes {reg row col {nrow 7} {ncol 7}} {
   imgGlobals
   if {[catch {set hndl [imgAxes $reg $row $col $nrow $ncol]}]} {
	catch {genericDel $hndl}
	return
	}
   if {[info exists img(axes)]} {
	handleSetFromHandle $img(axes) $hndl
	genericDel $hndl
   } else {
	set img(axes) $hndl
	}
   return
   }

######################################################3
proc axSet {elem args} {
   imgGlobals
   if {![info exists img(axes)]} {
	set img(axes) [genericNew AXES]
	}
   if {$elem == "axes"} {
	if {[llength $args] != 2} {
	   error "Need 2 numbers for axes"
	   }
	handleSet $img(axes).row [lindex $args 0]
	handleSet $img(axes).col [lindex $args 1]
	return
	}
   if {$elem == "anginc"} {
	handleSet $img(axes).pos_maj [expr [exprGet $img(axes).pos_maj] \
	   + $args]
	return
	}
   if {$elem == "ang" || $elem == "angle"} {set elem pos_maj}
   handleSet $img(axes).$elem $args
   return
   }

######################################################3
#Wrapper to do axes and show results conveniently
proc go {{n 7}} {
   imgGlobals
   axcur $n $n
   saoReset
   axDraw $img(saoNum)
   show axes
   return
   }

#Wrapper to do axes and show results conveniently
#Use caxes for this
proc cgo {{n 7}} {
   imgGlobals
   caxcur $n $n
   saoReset
   axDraw $img(saoNum)
   show axes
   return
   }

