#Read imgHistory.txt and specHistory.txt files and compare with SDSS-II
#baseline.

proc baseline {{stdout stdout}} {

#First, read baselines

   set legChain [param2Chain legacy-II.par hdr]
   set segChain [param2Chain segue-II.par hdr]

#Create tables with incremental and cumulative
   set legbase() ""
   set imgcum 0.
   set speccum 0.
   chainEach hndl $legChain {
	foreach var "period date imaging spectro" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend legbase() $date
	set legbase($date,period) $period
	set legbase($date,img) $imaging
	set legbase($date,spec) $spectro
	set imgcum [expr $imgcum + $imaging]
	set speccum [expr $speccum + $spectro]
	set speccum [format %.1f $speccum]
	set legbase($date,imgcum) $imgcum
	set legbase($date,speccum) $speccum
	}

   set segbase() ""
   set imgcum 0.
   set speccum 0.
   chainEach hndl $segChain {
	foreach var "period date imaging spectro" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend segbase() $date
	set segbase($date,period) $period
	set segbase($date,img) $imaging
	set segbase($date,spec) $spectro
	set imgcum [expr $imgcum + $imaging]
	set speccum [expr $speccum + $spectro]
	set speccum [format %.1f $speccum]
	set segbase($date,imgcum) $imgcum
	set segbase($date,speccum) $speccum
	}

#Now read data files

   set fid [open imgHistory.txt]

#Look for NORTH
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} {close $fid; return}
	if {[regexp NORTH $line]} break
	}

   set uniquecum 0.
   set leg2cum 0.
   set format "%10s %10s %10s %10s %10s"
   puts $stdout "              LEGACY IMAGING"
   puts $stdout [format $format Date Actual Baseline Actual Baseline]
   puts $stdout [format $format ""   increment increment cumulative cumulative]
   while {1} {
	set cum0 $uniquecum
	set line [string trim [gets $fid]]
	if {[eof $fid]} {close $fid; return}
	if {[string length $line] == 0} break
	set date [lindex $line 0]
	set uniquecum [lindex $line 4]
	if {[lsearch $legbase() $date] < 0} continue
	set inc [expr $uniquecum - $cum0]
	set leg2cum [expr $leg2cum + $inc]
	puts $stdout [format $format $legbase($date,period) $inc \
	   $legbase($date,img) $leg2cum $legbase($date,imgcum)]
	}

#Look for SEGUE.
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} {close $fid; return}
	if {[regexp SEGUE $line]} break
	}

   set uniquecum 0.
   set format "%10s %10s %10s %10s %10s"
   puts $stdout "              SEGUE IMAGING"
   puts $stdout [format $format Date Actual Baseline Actual Baseline]
   puts $stdout [format $format ""   increment increment cumulative cumulative]
   while {1} {
	set cum0 $uniquecum
	set line [string trim [gets $fid]]
	if {[eof $fid]} break
	if {[string length $line] == 0} break
	set date [lindex $line 0]
	set uniquecum [lindex $line 4]
	if {[lsearch $segbase() $date] < 0} continue
	set inc [expr $uniquecum - $cum0]
	puts $stdout [format $format $segbase($date,period) $inc \
	   $segbase($date,img) $uniquecum $segbase($date,imgcum)]
	}
   close $fid

#Now for spectroscopy
   set fid [open specHistory.txt]
   gets $fid
   gets $fid
   set northcum 0
   set north2cum 0.
   puts $stdout "              LEGACY SPECTRO"
   puts $stdout [format $format Date Actual Baseline Actual Baseline]
   puts $stdout [format $format ""   increment increment cumulative cumulative]
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	set date [lindex $line 0]
	set cum0 $northcum
	set northcum [lindex $line 4]
	set north [expr $northcum - $cum0]
	if {[lsearch $legbase() $date] < 0} continue
	set north2cum [expr $north2cum + $north]
	puts $stdout [format $format $legbase($date,period) $north \
	   $legbase($date,spec) $north2cum $legbase($date,speccum)]
	}
   close $fid

#Read file again for segue
   set fid [open specHistory.txt]
   gets $fid
   gets $fid
   set segcum 0.
   puts $stdout "              SEGUE SPECTRO"
   puts $stdout [format $format Date Actual Baseline Actual Baseline]
   puts $stdout [format $format ""   increment increment cumulative cumulative]
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	set date [lindex $line 0]
	set cum0 $segcum
	set bright [lindex $line 7]
	set faint [lindex $line 8]
	set segcum [expr ($bright + 2.*$faint)/3.]
	set seg [expr $segcum - $cum0]
	if {[lsearch $segbase() $date] < 0} continue
	set seg [format %.1f $seg]
	set segcum [format %.1f $segcum]
	puts $stdout [format $format $segbase($date,period) $seg \
	   $segbase($date,spec) $segcum $segbase($date,speccum)]
	}
   close $fid
   return
   }

