#Compare bias levels vs opECalib.  This is for imager CCDs
#only.

#Top level functions:

#   biasCheck <run>	Compare biases

proc biasCheckSetup {run} {
   global biasCheck
   global dataDir

   if {[info exists dataDir]} {
	set topdir $dataDir
   } else {
	set topdir /data/dp30.a/data/$run
	}

#Look for the top level directory
   if {![file exists $topdir]} {
	error "No top directory $topdir for run $run"
	}

#Look for field dir
   set dir $topdir/fields
   if {![file exists $dir]} {
	error "No fields directory"
	return
	}

#Get list of files for camcol 1
   set files [lsort [glob -nocomplain $dir/1/idR-[format %06d $run]-g1-*.fit]]
   if {[llength $files] <= 1} {
	error "Not enough files found in directory $dir"
	}
#Take middle frame
   set file [lindex $files [expr [llength $files]/2]]
   set file [file tail $file]
   if {![regexp "idR-[format %06d $run]-g1-(\[0-9\]+).fit" \
	$file all field]} {
	error "Can't decode $file to get field"
	}
   set field [string trimleft $field 0]
   echo Directory $dir, Field $field

#Cache results
   set biasCheck(run) $run
   set biasCheck(dir) $dir
   set biasCheck(field) $field
   set biasCheck(topdir) $topdir

   return
   }

###################################################################
#Copmute statistics for left and right halves

proc biasNoiseStats {reg} {
   set nrow [exprGet $reg.nrow]
   set lreg [subRegNew $reg $nrow 1 0 4]
   set rreg [subRegNew $reg $nrow 1 0 2123]
   foreach side "l r" {
	set list [regStatsFind [set ${side}reg] -niterations 1]
	set ${side}mean [format %6.2f [keylget list mean]]
	set ${side}sigma [format %5.2f [keylget list sigma]]
	}
#   echo Offset. Left: $lmean +/- $lsigma "  " \
#	Right: $rmean +/- $rsigma
   regDel $lreg
   regDel $rreg
   return [list $lsigma $rsigma]
   }

###################################################################
#Copmute statistics for left and right halves vs. opECalib

proc biasOpCompare {list row col opChain} {
   set subchain [chainSearch $opChain "{camRow = $row} {camCol = $col}"]
   if {[chainSize $subchain] == 1} {
	   set hndl [chainElementGetByPos $subchain 0]
	   set lbias [exprGet $hndl.biasLevel0]
	   set rbias [exprGet $hndl.biasLevel1]
	   set lrn [format %6.2f [exprGet $hndl.readNoiseDN0]]
	   set rrn [format %6.2f [exprGet $hndl.readNoiseDN1]]
	   handleDel $hndl
   } else {
	return "1 1"
	}
   chainDel $subchain
   set lsig [lindex $list 0]
   set rsig [lindex $list 1]

#We return the ratio of measure rn divided by opECalib noise, left & right
#sides
   set ldiff [format %.2f [expr abs($lsig/$lrn)]]
   set rdiff [format %.2f [expr abs($rsig/$rrn)]]
   return [list $ldiff $rdiff]
   }

###################################################################

proc biasCheck {args} {
   set opts { \
	{biasCheck "Compare biases in a new run.  Menu will be presented \
	   to pick reference bias"} \
	{{[run]} INTEGER -1 run "new bias run"} \
	}
   if {[shTclParseArg $args $opts biasCheck] == 0} {
	return 
	}
   global biasCheck

#Allow caching of setup stuff for a specific run
   if {![info exists biasCheck($run)]} {
	biasCheckSetup $run
	}
   set dir $biasCheck(dir)
   set topdir $biasCheck(topdir)
   set field $biasCheck(field)

#Try reading opECalib
   set file [glob -nocomplain $topdir/logs/opECalib*.par]
   if {[llength $file] == 1} {
	set opChain [param2Chain $file hdr]
   } else {
	set opChain ""
	}

#Begin the work
   set reg [regNew]

#Now read bias vectors from photo gangs
   set mindiff 10
   set maxdiff 0
   set row(u) 3
   set row(g) 5
   set row(r) 1
   set row(i) 2
   set row(z) 4
   foreach camcol "1 2 3 4 5 6" {
	foreach fil "u g r i z" {
	   set file $dir/$camcol/idR-[format %06d $run]-$fil$camcol-[format \
		%04d $field].fit
	   regReadAsFits $reg $file
	   set list [biasNoiseStats $reg]
	   set diffs [biasOpCompare $list $row($fil) $camcol \
		$opChain]
echo CCD $row($fil)$camcol ($fil) Diffs $diffs
	   set mindiff [eval min $mindiff $diffs]
	   set maxdiff [eval max $maxdiff $diffs]
	   }
	}
   echo mindiff is [format %.2f $mindiff], maxdiff is [format %.2f $maxdiff]
   if {$maxdiff > 2.} {
	return 1
   } else {
	return 0
	}
   }


######################################################################
proc biasCheckJob {plan} {
   global dataDir
   inipar $plan
   set dataDir [getpars dataDir]
   set run [getpari run]
   if {[catch {set result [biasCheck $run]} msg]} {
        echo $msg
	echo biasCheck failed!
	return 1
   } else {
	return $result
	}
   }

#########################################################################
#Should have done this years ago.  Subtract off bias overscan so both
#sides of a region have same zero point.

proc biasSub {reg} {
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
   set subreg [subRegNew $reg $nrow [expr $ncol/2] 0 0]
   set sky [imgNtile $reg 0 $nrow 0 20 .5]
   regAddWithDbl $subreg [expr -1.*$sky]
   regDel $subreg
   set subreg [subRegNew $reg $nrow [expr $ncol/2] 0 [expr $ncol/2]]
   set sky [imgNtile $reg 0 $nrow [expr $ncol-20] $ncol .5]
   regAddWithDbl $subreg [expr -1.*$sky]
   regDel $subreg
   return
   }
