###################################################################
proc plotInit {} {
   if {[pgQinf state] == "OPEN"} {pgEnd}
   pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
   return
   }

########################################################################
#Work with counts in the next few routines.

proc apsum {hndl filter nbin} {
   global area
    if {[exprGet $hndl.psfcounts<$filter>] <= 0} return
   set sum 0
   for {set i 0} {$i < $nbin} {incr i} {
	set cnts [exprGet $hndl.profmean<$filter><$i>]
	set sum [expr $cnts*$area($i) + $sum]
	}
   set diff [expr $sum/[exprGet $hndl.psfcounts<$filter>]]
   handleSet $hndl.diff $diff
   handleSet $hndl.apmag<$filter> $sum
   return
    }

###################################################################
#Calibrate a chain with ADU's
#Sort by column as well
proc chainSumCalib {chain filter nbin mincol maxcol} {
   mtGlobals
   global flagVal
   if {![info exists flagVal]} {
	flagsDefine
	}
   set diffs ""
   set n 0
   chainEach hndl $chain {
	if {[exprGet $hndl.profmean<$filter><0>] == 0} continue
	if {[exprGet $hndl.psfcounts<$filter>] < 1.e4} continue
	set flags [exprGet $hndl.flags<$filter>]
	if {($flags & $flagVal(BRIGHT)) > 0} continue
	if {($flags & $flagVal(SATUR)) > 0 || ($flags & $flagVal(INTERP)) \
	   > 0} continue
	set col [exprGet $hndl.colc<2>]
	if {$col < $mincol || $col > $maxcol} continue
	apsum $hndl $filter $nbin
	if {[exprGet $hndl.psfcounts<$filter>] > 1.e5} continue
	lappend diffs [exprGet $hndl.diff]
	incr n
	}
   return $diffs
   }

##################################################################
proc photomSumCheck {run camcol field1 field2 nbin ifil \
	{mincol 0} {maxcol 2048}} {
   mtGlobals
   global seeing

   set diffs ""
   loop field $field1 $field2 {
echo Field $field
	set chain [objcRead $run $camcol $field]
	catch {append diffs [chainSumCalib $chain $ifil $nbin $mincol $maxcol]}
	append diffs " "
	genericChainDel $chain
	}
   set diffs [lsort -real $diffs]
   set n [llength $diffs]
   set q1 [lindex $diffs [expr $n/4]]
   set median [lindex $diffs [expr $n/2]]
   set q3 [lindex $diffs [expr 3*$n/4]]
   set median [format %.2f [expr $median-1]]
   set sig [expr ($q3-$q1)/(2.*.68)]
   set sig [format %.2f $sig]
   echo Median = $median, sig = $sig, n = $n
   return "$median $sig"
   }

#########################################################################
########################################################################
#Work with psp profiles in the next few routines.

proc pspDef {} {
   if {[lsearch typesList PSPOBJ] < 0} {
	typedef struct {
	   int prof_nprof<5>;
	   float prof_mean<5><15>;
	   float prof_cum<5><15>;
	   float psf_width<5>;
	   } PSPOBJ
	}
   return
   }

######################################################################
proc pspsum {hndl filter} {
   global area
   set sum 0
   set n [exprGet $hndl.prof_nprof<$filter>]
   set irad [expr $n-1]
   for {set i 0} {$i <= $irad} {incr i} {
	set cnts [exprGet $hndl.prof_mean<$filter><$i>]
	set sum [expr $cnts*$area($i) + $sum]
	handleSet $hndl.prof_cum<$filter><$i> $sum
	}
   return
    }

#######################################################################
#Routines that work on tsObj files.
#Convert counts per pixel to a surface brightness.
#Several of the following routines are less useful now that radial
#profiles are in 0th mag star units.
proc calib {cnts} {
   global zero extinct secz
    set line [lcaliApply [expr $cnts/.157] 10. 8000. 0. 53.9 53.9 $secz \
	$zero $extinct 0 0 1 0 1]
   set mag [expr [keylget line calMag]]
   return [format %.3f $mag]
   }

#########################################################################
#Uncalibrate a surface brightness.
proc uncalib {mag} {
   global zero extinct secz
   set cnts [lcaliInvert $mag 0. 53.9 $secz $zero $extinct 0 0 1]
   set cnts [expr $cnts*.157]
   return [format %.2f $cnts]
   }

#Input handle to a tsobj object; recompute aperture magnitude
#Zeropoint, secz, extinct are hardwired for run 1140, camcol 1

########################################################################
proc apmag {hndl filter irad} {
   global area
   global zero extinct secz
    if {![info exists zero]} {
	set zero -24.026; set extinct .0832; set secz 1.25
    }
   set sum 0
   for {set i 0} {$i <= $irad} {incr i} {
	set cnts [uncalib [exprGet $hndl.profmean<$filter><$i>]]
	set sum [expr $cnts*$area($i) + $sum]
	}
#Convert back to cnts/pixel
   set sum [expr $sum*.157]
   set mag [calib $sum]
   set diff [format %.2f [expr $mag - [exprGet $hndl.psfcounts<$filter>]]]
#   echo [format %.2f $mag] [format %.2f [exprGet $hndl.psfcounts<$filter>]] \
#	$diff
   handleSet $hndl.diff $diff
   handleSet $hndl.apmag<$filter> $mag
   return
   }

###################################################################
#Calibrate a chain
proc chainCalib {chain run camcol field filter irad} {
   mtGlobals
   global zero extinct secz
   if {![info exists fcalib($run,$camcol)]} {
	fcalibRead $run
	}
   set hndl [chainGet $fcalib($run,$camcol) 0]
   if {$hndl == ""} return
   set zero [exprGet $hndl.a<$filter>]
   set extinct [exprGet $hndl.k<$filter>]
   handleDel $hndl
   if {![info exists astrom($run,$camcol)]} {
	astromPatch $run
	}
   set subchain [chainSearch $astrom($run,$camcol) "{field = $field}"]
   set hndl [chainGet $subchain 0]
   chainDel $subchain
   if {$hndl == ""} return
   set secz [exprGet $hndl.airmass]
   handleDel $hndl
   set diffs ""
   chainEach hndl $chain {
	if {[exprGet $hndl.profmean<$filter><0>] == 0} continue
	if {[exprGet $hndl.psfcounts<$filter>] > 19.} continue
	apmag $hndl $filter $irad
	if {[exprGet $hndl.psfcounts<$filter>] < 16.} continue
	lappend diffs [exprGet $hndl.diff]
	}
   set diffs [lsort -real $diffs]
   set median [lindex $diffs [expr [llength $diffs]/2]]
#   echo Median = $median
   return $median
   }

##################################################################
proc photomCheck {run camcol} {
   mtGlobals
   global seeing
   if {![info exists seeing($run,$camcol)]} {seeingRead $run}
   set list ""
   loop field [lindex $seeing($run,$camcol) 0] [lindex $seeing($run,$camcol) \
	end] 50 {
	set chain [tsobjRead $run $camcol $field]
	set median [chainCalib $chain $run $camcol $field 2 6]
	echo Seeing = [format %.2f $seeing($run,$camcol,$field,r)], \
	   Median = $median
	genericChainDel $chain
	lappend list "[format %.2f $seeing($run,$camcol,$field,r)] $median"
	}
   return $list
   }

#################################################################
#Compare north and south runs
proc runCompare {nrun srun camcol} {
   mtGlobals
   astromPatch $nrun
   astromPatch $srun
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}

#Seeing
   global seeing
   if {![info exists seeing($nrun,$camcol)]} {seeingRead $nrun}
   if {![info exists seeing($srun,$camcol)]} {seeingRead $srun}
   set hndl1 [chainGet $astrom($nrun,$camcol) 0]
   set hndl2 [chainGet $astrom($nrun,$camcol) [expr [chainSize \
	$astrom($nrun,$camcol)]-1]]
   set startField [exprGet $hndl1.field]
   set endField [exprGet $hndl2.field]
   handleDel $hndl1
   handleDel $hndl2
   tobjDef
   loop i $startField $endField 50 {
	set field1 $i
	set field2 [expr min($field1+10,$endField)]
	set cdFields $field1-$field2
	set matchFields [cdOverlap $nrun $srun $camcol $cdFields]
	if {[string length $matchFields] == 0} continue
	set nchain [chainNew TOBJ]

#Read object lists for north and south runs.
	foreach field  [fieldExpand $cdFields] {
	   chainJoin $nchain [tsobjRead $nrun $camcol $field]
	   }
	set schain [chainNew TOBJ]
	foreach field [fieldExpand $matchFields] {
	   chainJoin $schain [tsobjRead $srun $camcol $field]
	   }

#Seeing
	set nseeing [format %.2f \
	   $seeing($nrun,$camcol,[lindex [fieldExpand $cdFields] 0],r)]
	set sseeing [format %.2f \
	   $seeing($srun,$camcol,[lindex [fieldExpand $matchFields] 0],r)]
	echo
	echo seeing = $nseeing, $sseeing

#Compute aperture mags.
	chainCalib $nchain $nrun $camcol [lindex [fieldExpand $cdFields] 0] \
	   2 6
	chainCalib $schain $srun $camcol [lindex [fieldExpand $matchFields] \
	   0] 2 6

#Match up
	set matchChain [chainMatchType $nchain $schain \
	   "{ra ra .001} {dec dec .001}"]
	set subChain [chainSearch $matchChain \
	   "{16 < object1->psfcounts<2> < 19} \
	   {16 < object2->psfcounts<2> < 19}"]

#Residuals for psf mags.
	chainTransZXY $subChain res1 object1->psfcounts<2> \
	   object2->psfcounts<2> 0 1 -1
	chainFSort $subChain res1
	set hndl [chainGet $subChain [expr [chainSize $subChain]/2]]
	set median [format %.2f [exprGet $hndl.res1]]
	handleDel $hndl
	set n [chainSize $subChain]
	echo PSF mag median residual for fields $cdFields = $median, n=$n

#Repeat using aperture mags
	chainTransZXY $subChain res1 object1->apmag<2> \
	   object2->apmag<2> 0 1 -1
	chainFSort $subChain res1
	set hndl [chainGet $subChain [expr [chainSize $subChain]/2]]
	set median [format %.2f [exprGet $hndl.res1]]
	handleDel $hndl
	set n [chainSize $subChain]
	echo Aperture mag median for fields $cdFields = $median, n=$n
	global XMIN XMAX YMIN YMAX
	set XMIN 15; set XMAX 20; set YMIN -.3; set YMAX .3
	cplot $subChain object1->apmag<2> res1
	chainDel $subChain
	genericChainDel $matchChain
	genericChainDel $nchain
	genericChainDel $schain
	}
   }

##############################################################
#Look at trans's from one and find overlapping frames from matching run
#Use global variables to do everything

proc cdOverlap {cdRun matchRun camcol cdFields} {
   mtGlobals
   set matchFields ""
   if {$matchRun == ""} return
   set camcol [lindex $camcols 0]
   if {![info exists astrom($cdRun,$camcol)]} {astromPatch $cdRun}
   if {![info exists astrom($matchRun,$camcol)]} {astromPatch $matchRun}

   set fields [fieldExpand $cdFields]
   set matchFields ""
   foreach field $fields {
#Get handle to field in cdRun
        set hndl [chainGetOne $astrom($cdRun,$camcol) field $field]
        if {$hndl == ""} {
           error "Can't find primary field $field for run $cdRun"
           }
        set a1 [exprGet $hndl.a]
        set b [exprGet $hndl.b]
	set nrowalign 1361
        set a2 [expr $a1 + $nrowalign*$b]
        handleDel $hndl
#Find matching field(s) in matchRun
#Disaster can strike when we wrap around 0 degrees!  This is if target
#chain isn't "wrapped" yet.
#OK, I will first try without wrapping the source; then with wrapping the
#source.
#I do encounter both situations.
        set matchChain [chainSearch $astrom($matchRun,$camcol) \
                "{a >= $a1} {a <= $a2}"]

        if {[chainSize $matchChain] == 0} {
           chainDel $matchChain
           set a1 [fmod $a1 360.]
           set a2 [expr $a1 + $nrowalign*$b]
           set matchChain [chainSearch $astrom($matchRun,$camcol) \
                "{a >= $a1} {a <= $a2}"]
           }

        set match ""
        chainInit $matchChain iter
        while {1} {
           set hndl [chainNext iter]
           if {$hndl == ""} break
#echo Field $field matches run $matchRun, field [exprGet $hndl.field]
           set match [exprGet $hndl.field]
           }
        if {$match != ""} {lappend matchFields $match}
        chainDel $matchChain
        }
#Reset matchFields to look like field1-fieldn.  This is needed for
#creating the mosaic directory name.
   if {[llength $matchFields] > 0} {
        set matchFields [lindex $matchFields 0]-[lindex $matchFields end]
        }

   return $matchFields
   }

