#Get CAS, DAS statistics
#Run from sdsswww1 to get gridjob statistics.
#Be sure to setup lynx first!
#Top level command: allQuery

#HST Statistic:
#From STSCI newsletter vol 22, no 2 (about July 2005)
#MAST distributed 62 GB/day in Jan 2005.  Retrieval time (no reprocessing)
#is .17 hrs.

if {![info exists env(LYNX_DIR)]} {
   echo Caution: lynx not set up!
   }

#######################################################################
#The following documents the actual SQL query inside a proc so I can cut-and-
#paste into a web browser box.

proc justadummy {} {

Cas:
select
    datepart(year,thetime) as yy,
    datepart(month,thetime) as mm,
    datepart(day,thetime) as dd,
    sum(rows) as nrows
from weblog..sqllogutc
group by
    datepart(year,thetime),
    datepart(month,thetime),
    datepart(day,thetime)
order by yy,mm,dd

Casjobs:
select
    datepart(year,timestart) as yy,
    datepart(month,timestart) as mm,
    datepart(day,timestart) as dd,
    sum(rows) as nrows
from batch.jobs
group by
    datepart(year,timestart),
    datepart(month,timestart),
    datepart(day,timestart)
order by yy,mm,dd

}

#######################################################################
proc casQuery {http {stdout ""}} {
   if {$stdout == ""} {set stdout stdout}

   set url "http://${http}/tools/search/x_sql.asp"
   append url "?cmd=select%20datepart(year,thetime)%20as%20yy,"
   append url "datepart(month,thetime)%20as%20mm,datepart(day,thetime)"
   append url "%20as%20dd,sum(rows)%20as%20nrows,count(rows)%20as%20njob"
   append url "%20from%20weblog..sqllogutc"
   append url "%20group%20by%20datepart(year,thetime),datepart(month,thetime),"
   append url "datepart(day,thetime)%20order%20by%20yy,mm,dd&format=csv"

   if {[catch {set lines [exec lynx -source $url]} msg]} {
	error $msg
	}
   set lines [split $lines \n]
   set lines [lrange $lines 1 end]
   set cum() ""
   foreach line $lines {
	set line [split $line ,]
	set year [lindex $line 0]
	set mon [lindex $line 1]
	set day [lindex $line 2]
	set cnt [lindex $line 3]
	set njob [lindex $line 4]
	set item $year-$mon
	if {[lsearch $cum() $item] < 0} {
	   lappend cum() $item
	   set cum($item) 0
	   set cum($item,njob) 0
	   }
	set cum($item) [expr $cum($item) + $cnt.]
	set cum($item,njob) [expr $cum($item,njob) + $njob]
	}
   set sum 0
   foreach item $cum() {
	puts $stdout "$item [format %.0f [expr $cum($item)*100./1.e9]] GB \
	   in $cum($item,njob) jobs"
	set sum [expr $sum + $cum($item)*100./1.e12]
	}
   return $sum
   }

#######################################################################
proc casJobQuery {http {stdout ""}} {


   set url "http://${http}/casjobs/CasJobs.asmx/SubmitShortJob?"
   append url "wsid=13481113&"
   append url "target=BatchAdmin&"
   append url "qry=select%20"
   append url "datepart(year,timestart)%20as%20yy,"
   append url "datepart(month,timestart)%20as%20mm,"
   append url "datepart(day,timestart)%20as%20dd,"
   append url "count(jobid)%20as%20njob,"
   append url "sum(rows)%20as%20nrows%20"
   append url "from%20batch.jobs%20"
   append url "group%20by%20"
   append url "datepart(year,timestart),"
   append url "datepart(month,timestart),"
   append url "datepart(day,timestart)%20"
   append url "order%20by%20yy,mm,dd&"
   append url "taskname=noname&"
   append url "autocomplete=0"

   if {[catch {set lines [exec lynx -source $url]} msg]} {
	error $msg
	}

#Need to parse xml - jeez, what a pain!
   set sum [casJobQueryParse $lines $stdout]
   return $sum
   }

###############################################################
proc casJobQueryParse {lines {stdout ""}} {
   if {$stdout == ""} {set stdout stdout}
   set lines [split $lines \n]
   set cum() ""
   while {[llength $lines] > 0} {
	set line [lvarpop lines]
	if {![regexp {<YY[^>]+>([0-9]+)</YY[^>]+>} $line all year]} continue
	set line [lvarpop lines]
	regexp {<MM[^>]+>([0-9]+)</MM[^>]+>} $line all mon
	set line [lvarpop lines]
	regexp {<DD[^>]+>([0-9]+)</DD[^>]+>} $line all day
	set line [lvarpop lines]
	regexp {<NJOB[^>]+>([0-9]+)</NJOB[^>]+>} $line all njob
	set line [lvarpop lines]
	regexp {<NROWS[^>]+>([0-9]+)</NROWS[^>]+>} $line all cnt
	set item $year-$mon
	if {[lsearch $cum() $item] < 0} {
	   lappend cum() $item
	   set cum($item) 0
	   set cum($item,njob) 0
	   }
	set cum($item) [expr $cum($item) + $cnt]
	set cum($item,njob) [expr $cum($item,njob) + $njob]
	}
   set sum 0
   foreach item $cum() {
	puts $stdout "$item [format %.0f [expr $cum($item)*100./1.e9]] GB \
	   in $cum($item,njob) jobs"
	set sum [expr $sum + $cum($item)*100./1.e12]
	}
   return $sum
   }

###############################################################
proc allCasQuery {{stdout ""}} {
   if {$stdout == ""} {set stdout stdout}
   puts $stdout ""
   puts $stdout "Collaboration (and Public) (DR2)"
   set sum 0
   puts $stdout FNAL

#cas.sdss.org is the same machine as skyserver2.fnal.gov
   set url cas.sdss.org/collab/en
   if {![catch {set result [casQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo casQuery failed for $url
	echo $msg
	}
   puts $stdout JHU
   set url skyserver.sdss.org/collab/en
   if {![catch {set result [casQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo casQuery failed for $url
	echo $msg
	}
   puts $stdout ""
   puts $stdout "Casjobs (Public)"
   puts $stdout FNAL

#casjobs.sdss.org is the same machine as skyserver3.fnal.gov
   set url casjobs.sdss.org/DR3/en
   if {![catch {set result [casJobQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo $msg
	echo casJobQuery failed for $url
	}

   puts $stdout JHU

#skyserver.sdss.org is the same machine as skyserver.pha.jhu.edu
   set url skyserver.sdss.org
   if {![catch {set result [casJobQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo $msg
	echo casJobQuery failed for $url
	}
   puts $stdout ""

#Fermi DR3
   puts $stdout "FNAL DR3"
   set url cas.sdss.org/DR3/en
   if {![catch {set result [casQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo $msg
	echo casQuery failed for $url
	}
   puts $stdout ""

   puts $stdout "FNAL Astronomers"
   set url cas.sdss.org/astro/en
   if {![catch {set result [casQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo $msg
	echo casQuery failed for $url
	}

#Fermi DR4
   puts $stdout "FNAL DR4"
   set url cas.sdss.org/DR4/en
   if {![catch {set result [casQuery $url $stdout]} msg]} {
	set sum [expr $sum + $result]
   } else {
	echo $msg
	echo casQuery failed for $url
	}
   puts $stdout ""

#   casQuery cas.sdss.org/dr2/en $stdout
   return $sum
   }

############################################################
proc rsyncQuery {{stdout ""}} {
   if {$stdout == ""} {set stdout stdout}

#For some reason, lynx does not run on sdsswww1!!!  So I will try to read
#the file directly if available
   set file /home/s1/sdssdp/WWW/web-farm-stats/rsyncBytesPerDay
   if {[file exists $file]} {
	set fid [open $file]
	set lines [read $fid]
	close $fid
   } else {
	set url "http://www-sdss.fnal.gov/sdssdp/web-farm-stats/rsyncBytesPerDay"
	if {[catch {set lines [exec lynx -source $url]} msg]} {
	   error $msg
	   }
	}
   set lines [split $lines \n]
   set lines [lrange $lines 1 end]
   set cum() ""
   foreach line $lines {
	set line [string trim $line]
	if {[string length $line] == 0} continue
	set date [lindex $line 0]
	set cnt [lindex $line end]
	set date [split $date /]
	set mon [lindex $date 0]
	set day [lindex $date 1]
	set year [lindex $date 2]
	set item $year-$mon
	if {[lsearch $cum() $item] < 0} {
	   lappend cum() $item
	   set cum($item) 0
	   }

#Go to TB right away.
	set cum($item) [expr $cum($item) + $cnt./1.e12]
	}
   set sum 0.
   foreach item $cum() {
	puts $stdout "$item [format %.1f $cum($item)] TB"
	set sum [expr $sum + $cum($item)]
	}
   return $sum
   }

############################################################
proc httpQuery {{stdout ""}} {
   if {$stdout == ""} {set stdout stdout}

#For some reason, lynx does not run on sdsswww1!!!  So I will try to read
#the file directly if available
   set file /home/s1/sdssdp/WWW/web-farm-stats/webBytesPerDay
   if {[file exists $file]} {
	set fid [open $file]
	set lines [read $fid]
	close $fid
   } else {
	set url "http://www-sdss.fnal.gov/sdssdp/web-farm-stats/webBytesPerDay"
	if {[catch {set lines [exec lynx -source $url]} msg]} {
	   error $msg
	   }
	}
   set lines [split $lines \n]
   set lines [lrange $lines 1 end]
   set cum() ""
   foreach line $lines {
	set line [string trim $line]
	if {[string length $line] == 0} continue
	set date [lindex $line 0]
	set cnt [lindex $line end]
	set date [split $date /]
	set mon [lindex $date 0]
	set day [lindex $date 1]
	set year [lindex $date 2]
	set item $year-$mon
	if {[lsearch $cum() $item] < 0} {
	   lappend cum() $item
	   set cum($item) 0
	   }

#Go to TB right away.
	set cum($item) [expr $cum($item) + $cnt./1.e12]
	}
   set sum 0.
   foreach item $cum() {
	puts $stdout "$item [format %.1f $cum($item)] TB"
	set sum [expr $sum + $cum($item)]
	}
   return $sum
   }

#####################################################
proc gsiftpQuery {{stdout ""}} {
   if {$stdout == ""} {set stdout stdout}
   set file /export/data/www/data/gsiftp/usage/gridftp.log
   if {![file exists $file]} {return 0}
   set fid [open $file]
   set sum 0
   set n 0
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	if {![regexp {NBYTES=([0-9]+)} $line all nbyte]} continue
	set sum [expr $sum + [expr $nbyte*1.]]
	incr n
	}
#Go to GB
   set sum [format %.3f [expr $sum/1.e12]]
   puts $stdout "$sum TB in $n jobs"
   close $fid
   return $sum
   }
#####################################################
proc allQuery {{file ""}} {
   if {$file == ""} {set stdout stdout} else {set stdout [open $file w]}
   set sum 0
   set sum [expr $sum + [allCasQuery $stdout]]
   puts $stdout ""
   puts $stdout Rsync
   set sum [expr $sum + [rsyncQuery $stdout]]
   puts $stdout ""
   puts $stdout Http
   set sum [expr $sum + [httpQuery $stdout]]
   puts $stdout ""
   puts $stdout Gsiftp
   set sum [expr $sum + [gsiftpQuery $stdout]]
   puts $stdout ""
   puts $stdout "Grand total: [format %.1f $sum] Terabytes"
   if {$file != ""} {close $stdout}
   return
   }
