#
#Commands
#	chunkStatus
#	Run a report on opdb and create odChunkStatus.par
#These are targetChunks.

################################################################
#Create odChunkStatus.par
#Must be connected to opdb first!  (Look in opdb.tcl)

proc chunkStatus {} {
   global photo_segments runTable

   if {![inSession]} {opdbConnect}
#Create run inventory - this gets me the most current rerun.
   if {![info exists runTable]} {
	echo Creating run inventory
	runInventory
	}

#Create a file
   set mjd [expr [getclock]/86400 + 40587]
   set file odChunkStatus-$mjd.par
   echo Output file is $file
   set fid [open $file w]
   puts $fid "#Chunk status"
   puts $fid "#Created [fmtclock [getclock]]"
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int camcol;"
   puts $fid "   int startField;"
   puts $fid "   int endField;"
   puts $fid "   double slongStart;"
   puts $fid "   double slongEnd;"
   puts $fid "   int version;"
   puts $fid "   int startMu;"
   puts $fid "   int endMu;"
   puts $fid "   int startMuSeg;"
   puts $fid "   int endMuSeg;"
   puts $fid "   char status<20>;"
   puts $fid "   } CHUNK;"

#Get a pointer to all chunks
   set citr [queryGet arChunk $photo_segments ""]

   while {[$citr next]} {

	set version [$citr -> skyVersion]
	set startMu [$citr -> startMu]
	set endMu [$citr -> endMu]

#Create a new segment iterator
	set sitr [objNew ooItr(arSegment) -transient]
	$citr -> segments $sitr
	while {[$sitr next]} {
	   set startField [$sitr -> field0]
	   set nfields [$sitr -> nFields]
	   set endField [expr $startField + $nfields - 1]
	   set startMuSeg [$sitr -> startMu]
	   set endMuSeg [$sitr -> endMu]
	   set status [$sitr -> status]
	   set status [enumIntToString arSegmentStatus $status]
	   regsub AR_SEGMENT_STATUS_ $status "" status
	   set status [string tolower $status]
	   set frames [$sitr -> framesPipelineRun]
	   set run [$frames -> run]
	   set rerun [$frames -> rerun]
	   set imgrun [$frames -> imagingRun]
	   set camcol [$frames -> camCol]
	   set stripe [$imgrun -> stripe]
	   set stripNum [$imgrun -> strip]
	   set strip [enumIntToString arStrip $stripNum]
	   regsub {AR_STRIP_} $strip "" strip

#Munn must have changed the friggen data model?
	   set strip [string index $strip 0]
	   objDel $imgrun
	   objDel $frames
	   set slongStart [expr round([$sitr -> startMu]/3600. - 185.)]
	   if {$slongStart < -180} {set slongStart [expr $slongStart+360]}
	   if {$slongStart > 180.} {set slongStart [expr $slongStart-360]}
	   set slongEnd [expr round([$sitr -> endMu]/3600. - 185.)]
	   if {$slongEnd < -180} {set slongEnd [expr $slongEnd+360]}
	   if {$slongEnd > 180} {set slongEnd [expr $slongEnd-360]}
###	   if {$camcol != 3} continue
#Not clear which rerun to use!  Do them all - the chunk rerun
#is not necessarily the latest.
#	   if {$rerun != $runTable($run)} continue
	   puts $fid "CHUNK $stripe $strip $run $rerun $camcol $startField \
		$endField $slongStart $slongEnd $version $startMu $endMu \
		$startMuSeg $endMuSeg $status"
	   }
	objDel $sitr
	}
   handleDel $citr
   close $fid
   return
   }

#######################################################################
########################################################################
#Make one summary plot of data in currentStripeStatus.par
#Make a gif unless Xwindow is open.

proc chunkPlotSummary {} {
   global stats
   set files [lsort [glob -nocomplain odChunkStatus*.par]]
   set file [lindex $files end]
   set chain [param2Chain $file hdr]
   set flag 0
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin chunkSummary.gif/VGIF
	pgGeomSet 700 900 0 0
	pgAsk 0
	set flag 1
	}


#Routine to make a plot of the survey area that has been completed.

    set plotLamMin -65
    set plotLamMax 65
    set plotStripeMin -75
    set plotStripeMax 1128

    pgIden
    pgWindow -65 65 -75 1128
    pgBox BCNITS 10.0 2 BC 0.0 0
    pgLabel {lambda} {stripe} {}
    pgSci 1
    pgPtext -30 1250 0 0 "TARGET RESOLVED CHUNK STATUS: [fmtclock [getclock] \
	    {%Y %h %d}]"
    pgSci 5
    set surveyArea [imgArea]
    pgPtext -65 1220 0 0 "Total Area : \
	[format %.0f $surveyArea]"
    if {[info exists stats]} {
 	pgSci 5
	set unique [expr $stats(northunique) + $stats(southunique) + \
	   $stats(equnique)]
	set percent [expr $unique*100./$surveyArea]
	pgPtext -65 1190 0 0 "Unique Area Imaged: [format %.0f $unique] \
	   ([format %.0f $percent])% "
	pgSci 5
	set pend 0
	set pend [expr $stats(northunreduced) + $stats(southunreduced) + \
	   $stats(equnreduced)]
	set percent [expr $pend*100./$surveyArea]
	pgPtext -65 1155 0 0 "(Processing Pending :  [format %.0f $pend])"
	}
    set astrotoolsDir [envscan \$ASTROTOOLS_DIR]
    set chn [param2Chain $astrotoolsDir/etc/atStripeDef.par hhh]

    
    pgScr 17 0.5 0.5 0.5
    pgScr 18 0.3 0.3 0.3
    pgScr 19 0.1 0.1 0.1

    pgSci 17
    set yval 0
    set width 10
    set blw 4
    set slw 1

    chainEach scan $chn {
        set ylist ""
        set ylist2 ""
        set stripe [exprGet $scan.stripeNumber]
	echo stripe is $stripe
        if {$stripe == 76} {set yval -25}
        if {$stripe == 82} {set yval -50}
        if {$stripe == 86} {set yval -75}
        set lambdaMin [exprGet $scan.lambdaMin]
        set lambdaMax [exprGet $scan.lambdaMax]

        if {$stripe == 76 || $stripe == 82 || $stripe == 86 } {

            set lambdaMin [expr $lambdaMin - 180]
            set lambdaMax [expr $lambdaMax -180]
            if {$lambdaMin < -180 } {
                set lambdaMin [expr $lambdaMin +360]
            }
            if {$lambdaMax < -180 } {
                set lambdaMax [expr $lambdaMax +360]
            }
        }
        
        pgSci 1
        pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $blw-1]
        set yval [expr $yval + $blw]
        pgSci 18
        pgRect $lambdaMin $lambdaMax [expr $yval+1] [expr $yval + $width-1]
        set yval [expr $yval + $width]
        pgSci 1
        pgSch 0.9
        pgPtext -70 [expr $yval - $width/2] 0.0 0.0 $stripe
        pgSch 1.0
        pgSci 17
        pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $slw-1]
        set yval [expr $yval + $slw]
        pgSci 18
        pgRect $lambdaMin $lambdaMax [expr $yval+1] [expr $yval + $width-1]
        set yval [expr $yval + $width]
        pgSci 1

        if {$stripe == 45} {
            pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $blw-1]          
        }

    }

   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	set strip [exprGet $hndl.strip]
	set lambdaStart [exprGet $hndl.slongStart]
	set lambdaEnd [exprGet $hndl.slongEnd]
	set len [expr $lambdaEnd-$lambdaStart]
	if {$lambdaEnd < $lambdaStart} {
	   set len [expr $len+360.]
	   }
	if {$lambdaStart > 90.} {
	   set lambdaStart [expr $lambdaStart-180.]
	   }
	if {$lambdaStart < -90.} {
	   set lambdaStart [expr $lambdaStart+180.]
	   }
	chunkScanPlot $stripe $lambdaStart 0 $len
	}
   if {$flag == 1} {echo pgEnd; pgEnd}
   return
}

#####################################################################
proc chunkScanPlot {stripe lambdaStart eta lengthOfRun} {
    
    if {$stripe == 100} {set stripe 10}
    if {$stripe == 76} {set stripe 0}
    if {$stripe == 82} {set stripe -1}
    if {$stripe == 86} {set stripe -2}
     
    set yval 0
    set width 10
    set blw 4
    set slw 1
    
   pgSci 6
   set yval [expr ($stripe-1)*$width*2 + $stripe*$blw \
		+ $stripe*$slw]
        pgRect [expr $lambdaStart] [expr $lambdaStart+$lengthOfRun] \
		[expr $yval] [expr $yval + 2.*$width - 3]
    
    pgSci 1
#    pgEnd
}

########################################################################
#Get target chunks actually used by tiling.  This is a subset of all
#targe chunks.  Create tileTargetChunk.par that has same format as
#odChunkStatus.

proc tileTargetSummary {} {

#Read tileChunk.par (which is created by specdb) and crawl the
#/opdb/d1/data/tileChunk directories to get the boundaries.

   set chain [param2Chain tileChunk.par hdr]

#Get a list of chunkIds.
   set chunkIds ""
   chainEach hndl $chain {
	set chunkId [exprGet $hndl.chunkId]
	if {[lsearch $chunkIds $chunkId] < 0} {
	   lappend chunkIds $chunkId
	   }
	}
   genericChainDel $chain

#Create output file
   set outfile tileTargetChunk.par
   set fid [open $outfile w]
   puts $fid "#Created [fmtclock [getclock]]"
   puts $fid "#Target chunks used by tiling; subset of all target chunks"
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int camcol;"
   puts $fid "   int startField;"
   puts $fid "   int endField;"
   puts $fid "   double slongStart;"
   puts $fid "   double slongEnd;"
   puts $fid "   int version;"
   puts $fid "   int startMu;"
   puts $fid "   int endMu;"
   puts $fid "   int startMuSeg;"
   puts $fid "   int endMuSeg;"
   puts $fid "   char status<20>;"
   puts $fid "   } CHUNK;"

   set drdir /data/dp15.a/data/tiling_dr5
   set opdir /opdb/d1/data/tileChunk
   foreach chunkId $chunkIds {
	if {$chunkId < 4} continue
	set dir $drdir/chunk$chunkId
	set file $dir/tiBoundariesOut.par
 	if {![file exists $file]} {
	   set dir $opdir/chunk$chunkId
	   set file $dir/tiling/tiBoundariesOut.par
	   if {![file exists $file]} {
		echo No file $file
		continue
		}
	   }
	if {[catch {set chain [param2Chain $file hdr]}]} {
	   echo Trouble reading chunk $chunkId file $file
	   continue
	   }

	chainEach hndl $chain {
	   set stripe [exprGet $hndl.stripe]
	   set coordType [exprGet $hndl.coordType]
	   set slongStart [exprGet $hndl.lambdamu<0>]
	   set slongEnd [exprGet $hndl.lambdamu<1>]
	   set node [nodeFromStripeNumber $stripe]
	   set incl [inclFromStripeNumber $stripe]
	   if {$coordType == 3 || $coordType == 1} {
		set startMu $slongStart
		set endMu $slongEnd
		set slongStart [expr $startMu - 185.]
		set slongEnd [expr $endMu - 185.]
	   } elseif {$coordType == 2} {
		set startMu [expr $slongStart + 185.]
		set endMu [expr $slongEnd + 185.]
	   } else {
		continue
		}
	   set slongStart [expr round(fmod($slongStart+720.+180.,360.))-180]
	   set slongEnd [expr round(fmod($slongEnd+720.+180,360.))-180]
	   set startMu [expr round(fmod($startMu+720.,360.))]
	   set endMu [expr round(fmod($endMu+720.,360.))]
	   set startMuSeg [expr $startMu*3600]
	   set endMuSeg [expr $endMu*3600]
	   echo chunk $chunkId
	   foreach strip "N S" {
		foreach camcol "1 2 3 4 5 6" {
		   puts $fid "CHUNK $stripe $strip 1 1 $camcol 1 1 \
			$slongStart $slongEnd 0 $startMuSeg $endMuSeg \
			$startMuSeg $endMuSeg primary"
		   }
		}
	   }
	}
   close $fid
   echo Created $outfile
   return
   }

########################################################################

#Plot target Chunks that have actually been used by tiling.
#chunkPlotSummary plots all target chunks read from opdb.  However, not
#all target chunks are tiled yet.  For the NSF review, it would be nice
#to know what remains to be tiled.  For this, I first want just the chunks
#that have been tiled.

proc tileTargetChunkPlot {} {
   global stats
   set files [lsort [glob -nocomplain tileTargetChunk*.par]]
   set file [lindex $files end]
   set chain [param2Chain $file hdr]
   set flag 0
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin tileTargetChunk.gif/VGIF
	pgGeomSet 700 900 0 0
	pgAsk 0
	set flag 1
	}


#Routine to make a plot of the survey area that has been completed.

    set plotLamMin -65
    set plotLamMax 65
    set plotStripeMin -75
    set plotStripeMax 1128

    pgIden
    pgWindow -65 65 -75 1128
    pgBox BCNITS 10.0 2 BC 0.0 0
    pgLabel {lambda} {stripe} {}
    pgSci 1
    pgPtext -30 1250 0 0 "TARGET RESOLVED CHUNK STATUS: [fmtclock [getclock] \
	    {%Y %h %d}]"
    pgSci 5
    set surveyArea [imgArea]
    pgPtext -65 1220 0 0 "Total Area : \
	[format %.0f $surveyArea]"
    if {[info exists stats]} {
 	pgSci 5
	set unique [expr $stats(northunique) + $stats(southunique) + \
	   $stats(equnique)]
	set percent [expr $unique*100./$surveyArea]
	pgPtext -65 1190 0 0 "Unique Area Imaged: [format %.0f $unique] \
	   ([format %.0f $percent])% "
	pgSci 5
	set pend 0
	set pend [expr $stats(northunreduced) + $stats(southunreduced) + \
	   $stats(equnreduced)]
	set percent [expr $pend*100./$surveyArea]
	pgPtext -65 1155 0 0 "(Processing Pending :  [format %.0f $pend])"
	}
    set astrotoolsDir [envscan \$ASTROTOOLS_DIR]
    set chn [param2Chain $astrotoolsDir/etc/atStripeDef.par hhh]

    
    pgScr 17 0.5 0.5 0.5
    pgScr 18 0.3 0.3 0.3
    pgScr 19 0.1 0.1 0.1

    pgSci 17
    set yval 0
    set width 10
    set blw 4
    set slw 1

    chainEach scan $chn {
        set ylist ""
        set ylist2 ""
        set stripe [exprGet $scan.stripeNumber]
	echo stripe is $stripe
        if {$stripe == 76} {set yval -25}
        if {$stripe == 82} {set yval -50}
        if {$stripe == 86} {set yval -75}
        set lambdaMin [exprGet $scan.lambdaMin]
        set lambdaMax [exprGet $scan.lambdaMax]

        if {$stripe == 76 || $stripe == 82 || $stripe == 86 } {

            set lambdaMin [expr $lambdaMin - 180]
            set lambdaMax [expr $lambdaMax -180]
            if {$lambdaMin < -180 } {
                set lambdaMin [expr $lambdaMin +360]
            }
            if {$lambdaMax < -180 } {
                set lambdaMax [expr $lambdaMax +360]
            }
        }
        
        pgSci 1
        pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $blw-1]
        set yval [expr $yval + $blw]
        pgSci 18
        pgRect $lambdaMin $lambdaMax [expr $yval+1] [expr $yval + $width-1]
        set yval [expr $yval + $width]
        pgSci 1
        pgSch 0.9
        pgPtext -70 [expr $yval - $width/2] 0.0 0.0 $stripe
        pgSch 1.0
        pgSci 17
        pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $slw-1]
        set yval [expr $yval + $slw]
        pgSci 18
        pgRect $lambdaMin $lambdaMax [expr $yval+1] [expr $yval + $width-1]
        set yval [expr $yval + $width]
        pgSci 1

        if {$stripe == 45} {
            pgRect -65.0 65.0 [expr $yval+1] [expr $yval + $blw-1]          
        }

    }

   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	set strip [exprGet $hndl.strip]
	set lambdaStart [exprGet $hndl.slongStart]
	set lambdaEnd [exprGet $hndl.slongEnd]
	set len [expr $lambdaEnd-$lambdaStart]
	if {$lambdaEnd < $lambdaStart} {
	   set len [expr $len+360.]
	   }
	if {$lambdaStart > 90.} {
	   set lambdaStart [expr $lambdaStart-180.]
	   }
	if {$lambdaStart < -90.} {
	   set lambdaStart [expr $lambdaStart+180.]
	   }
	chunkScanPlot $stripe $lambdaStart 0 $len
	}
   if {$flag == 1} {echo pgEnd; pgEnd}
   return
}


