#Create a dark run report using existing log files, etc.  Jeez, I make a lot
#of stuff!

#Also, get target drilling numbers for quarterly report time.
#Commands:
#   darkRunReport <monyy> [monyy]
#   drillReport chunkmin chunkmax

#Run from /data/dp0.b/users/skent/mt
global specRerun

#Rerun 24 exists, but it looks like we are using 23.
#set specRerun 24
set specRerun 23

proc darkRunReport {name {file ""}} {
   global runData specRerun
   if {[info exists runData]} {unset runData}
   if {$file == ""} {
	set outid stdout
   } else {
	set outid [open $file w]
	}

#First, get mjd limits from a file.
   set chain [param2Chain darkrun.par hdr]
   chainEach hndl $chain {
	set run [string tolower [exprGet $hndl.name]]
	if {$run == "$name"} {
	   set beginMjd [exprGet $hndl.beginMjd]
	   set midMjd [exprGet $hndl.midMjd]
	   set endMjd [exprGet $hndl.endMjd]
	   }
	}
   genericChainDel $chain
   if {![info exists beginMjd]} {
	echo Dark run $name not found!
	return
	}
   puts $outid "Dark Run $name $beginMjd <= MJD <= $endMjd"
   set nnight [expr $endMjd-$beginMjd+1]
   puts $outid "Number of nights: $nnight"

#Expand mjd limits in case data were taken during shake
   incr beginMjd -1
   incr endMjd 1

#For certain months, expand the limits for supernova observing
   if {[regexp sep $name] || [regexp oct $name] || [regexp nov $name]} {
	set chains [param2Chain moon.par hdr]
	foreach chain $chains {
	  if {[chainTypeDefine $chain] == "NEW"} {set newChain $chain} else {
		set fullChain $chain}
	   }
	genericChainDel $newChain

#Get dates of full moon around this dark run.
	set preFull [expr $midMjd - 30]
	set postFull [expr $midMjd + 30]
	chainEach hndl $fullChain {
	   set mjd [exprGet $hndl.mjd]
	   if {$mjd < $midMjd} {
		set preFull [max $mjd $preFull]
		}
	   if {$mjd > $midMjd} {
		set postFull [min $mjd $postFull]
		}
	   }
	genericChainDel $fullChain
	puts $outid "Supernova extended mjd limits: $preFull to $postFull"
	set beginMjd $preFull
	set endMjd $postFull
	}
   puts $outid ""

#Cumulative statistics
   puts $outid \
"Cumulative Statistics by Quarter Compared with Baseline: "

   baseline $outid
   puts $outid ""

#Get details of imaging
   puts $outid "IMAGING DETAILS"
   set chain [param2Chain run.par hdr]
   set mjds ""
   set runData() ""
   chainEach hndl $chain {
	set vars "mjd stripe strip muStart muEnd slongStart slongEnd \
	   frameStart frameEnd quality"
	foreach var $vars {
	   set $var [exprGet $hndl.$var]
	   }
	set run [exprGet $hndl.run]
	if {$mjd < $beginMjd || $mjd > $endMjd} continue
	if {$quality == "bad"} continue
	lappend runData() $run
	set runData($run,mjd) $mjd
	foreach var $vars {
	   set runData($run,$var) [set $var]
	   }
	set runData($run,stuff) ""
	set runData($run,calib) ""
	set runData($run,export) ""
	set runData($run,rerun) ""
	if {[lsearch $mjds $mjd] < 0} {lappend mjds $mjd}
	}

# Get better limits if data have been processed.  Use currentStripeStatus
   set chain [param2Chain currentStripeStatus.par hdr]
   set runCurrent() ""
   chainEach hndl $chain {
	foreach var "run lambdaStart lambdaEnd" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $runData() $run] < 0} continue
	set muStart [expr $lambdaStart + 185.]
	set muEnd [expr $lambdaEnd + 185.]
	if {$muStart > 360.} {set muStart [expr $muStart-360.]}
	if {$muEnd > 360.} {set muEnd [expr $muEnd-360.]}
	if {[lsearch $runCurrent() $run] < 0} {
	   lappend runCurrent() $run
	   set runCurrent($run,muStart) $muStart
	   set runCurrent($run,muEnd) $muEnd
	   }
	set runCurrent($run,muStart) [angMin $runCurrent($run,muStart) \
	   $muStart]
	set runCurrent($run,muEnd) [angMax $runCurrent($run,muEnd) \
	   $muEnd]
	}
   foreach run $runCurrent() {
	set runData($run,muStart) [angMax $runData($run,muStart) \
	   $runCurrent($run,muStart)]
	set runData($run,muEnd) [angMin $runData($run,muEnd) \
	   $runCurrent($run,muEnd)]
	}
   genericChainDel $chain

#Get data processing status
   set chains [param2Chain idMetric.par hdr]
   set chain ""
   foreach ch $chains {
	if {[chainTypeDefine $ch] == "IDMETRIC"} {
	   set chain $ch
	} else {
	   set detail $ch
	   }
	}
   if {$chain == ""} {
	echo No idMetric chain!!!
	return
	}

   chainEach hndl $chain {
	foreach var "run spool stuff calib" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $runData() $run] < 0} continue
	if {$stuff >= 0} {
	   set runData($run,stuff) Yes
	   }
	if {$calib >= 0} {
	   set runData($run,calib) Yes
	   }
	}
   genericChainDel $chain

#Get the rerun info.
#If run is not from current dark run, get basic info at least.
   set extraRun() ""
   chainEach hndl $detail {
	foreach var "run mjd rerun spoolMJD stuffMJD calibMJD version" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $runData() $run] < 0} {
	    if {$stuffMJD == ""} continue
	    if {$stuffMJD < $beginMjd} continue
	    if {$stuffMJD > $endMjd+10} continue
	    lappend extraRun() $run
	    set extraRun($run,rerun) $rerun
	    set extraRun($run,stuffMJD) $stuffMJD
	    set extraRun($run,version) $version
	    continue
	    }
	lappend runData($run,rerun) $rerun
	}
   genericChainDel $detail

   puts $outid "Number of nights with imaging: [llength $mjds]"
   puts $outid "Number of runs: [llength $runData()]"
   set format "%-4s   %5s   %6s %5s  %5s  %5s  %5s %6s %6s %6s %6s"
   puts $outid [format $format Run MJD Stripe Strip Start End Area \
	Rerun Calib Stuff Export]
   set totarea 0
   foreach run [lsort -integer $runData()] {
	if {[llength $runData($run,rerun)] > 1} {
	   set runData($run,rerun) [eval max $runData($run,rerun)]
	   }

#See if export directory exists
	set file /data/dp3.b/data/$run/$runData($run,rerun)/calibChunks/summary.html
	if {[file exists $file]} {
	   set runData($run,export) Yes
	   }
	set lambdaStart [format %.0f [expr $runData($run,muStart)-185]]
	set lambdaEnd [format %.0f [expr $runData($run,muEnd)-185]]
	set diff [expr $lambdaEnd-$lambdaStart]
	if {$diff < 0} {set diff [expr $diff+360.]}
	set area [format %.0f [expr $diff*1.25]]
	puts $outid [format $format $run $runData($run,mjd) \
	   $runData($run,stripe) $runData($run,strip) \
	   $lambdaStart $lambdaEnd $area $runData($run,rerun) \
	   $runData($run,calib) $runData($run,stuff) $runData($run,export)]
	set totarea [expr $totarea + $area]
	}
   set totarea [format %.0f $totarea]
   puts $outid ""
   puts $outid "Total Area Scanned: $totarea square degrees"

   if {[llength $extraRun()] > 0} {
	puts $outid ""
	puts $outid "OTHER IMAGING RUNS PROCESSED"
	set format "%8s  %8s  %12s"
	puts $outid [format $format Run   Rerun   PhotoVersion]
	foreach run [lsort -integer $extraRun()] {
	   puts $outid [format $format \
		$run $extraRun($run,rerun)  $extraRun($run,version)]
	   }
	}

#Spectro status
#Fetch report.  For quick action, I will use mapReport*.par
   set files [lsort [glob -nocomplain mapReport*.par]]
   set file [lindex $files end]

#Fix one segue plate with too damn many mjds.
   mapReportFix $file
   set chain [param2Chain $file hdr]
   set specData() ""
   set mjds ""
   chainEach hndl $chain {
	foreach var "plateId tileId tileRun date_done nexp mjdobs" {
	   set $var [exprGet $hndl.$var]
	   }
	set mjd [lindex [lindex $mjdobs 0] end]
	set date_done [lindex $date_done 0]
	if {$mjd < $beginMjd || $mjd > $endMjd} continue
	if {$date_done != ""} {
	   set status Done
	} else {
	   set status Not_Done

#Don't bother with partially finished plates
	   continue
	   }
	set id [format %04d $plateId]-$mjd
	lappend specData() $id
	set specData($id,plateId) $plateId
	set specData($id,mjd) $mjd
	if {[lsearch $mjds $mjd] < 0} {lappend mjds $mjd}
	foreach var "tileId tileRun nexp status" {
	   set specData($id,$var) [set $var]
	   }
	}

   genericChainDel $chain

#Get data processing status - mainly see if it has been inspected
   set file sdproc-$specRerun.par
   set chain [param2Chain $file hdr]
   foreach id $specData() {
	set plateId $specData($id,plateId)
	set mjd $specData($id,mjd)
	set subchain [chainSearch $chain "{plate = $plateId} {mjd = $mjd}"]
	set specData($id,inspect) ""
	if {[chainSize $subchain] > 0} {
	   set hndl [chainGet $subchain 0]
	   set inspectMJD [exprGet $hndl.inspectMJD]
	   if {$inspectMJD > 0} {set specData($id,inspect) Y}
	   }
	chainDel $subchain
	}
   genericChainDel $chain

   puts $outid ""
   puts $outid "SPECTROSCOPY DETAIL"
   puts $outid "Number of nights with data: [llength $mjds]"
   puts $outid "Number of pluggings: [llength $specData()]"
   puts $outid "Current spectro processing rerun: $specRerun"
   set format "%5s  %5s  %4s %15s  %4s  %8s  %8s"
   puts $outid [format $format Plate MJD tile Program Nights APO-Status \
	Inspect]
   foreach id [lsort $specData()] {
	puts $outid [format $format $specData($id,plateId) \
	   $specData($id,mjd) $specData($id,tileId) $specData($id,tileRun) \
	   $specData($id,nexp) $specData($id,status) $specData($id,inspect)]
	}

#PT Patches
   set files [lsort [glob -nocomplain odPatchStatus*.par]]
   set file [lindex $files end]
   set chains [param2Chain $file hdr]
   foreach ch $chains {
	if {[chainTypeDefine $ch] == "PATCHSTAT"} {
	   set chain $ch
	} else {
	   genericChainDel $ch
	   }
	}
   set nobs 0
   set patches ""
   set mjds ""
   chainEach hndl $chain {
	foreach var "name mjdobs quality" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$quality == "bad"} continue
	if {$mjdobs < $beginMjd || $mjdobs > $endMjd} continue
	incr nobs
	if {[lsearch $patches $name] < 0} {
	   lappend patches $name
	   }
	if {[lsearch $mjds $mjdobs] < 0} {
	   lappend mjds $mjdobs
	   }
	}
   genericChainDel $chain

#Look for pending runs
   set npend 0
   set chain [param2Chain mdMetric.par hdr]
   set subChain [chainSearch $chain "{mjd >= $beginMjd} {mjd <= $endMjd}"]
   chainEach hndl $subChain {
	set delay [exprGet $hndl.delay]
	if {$delay < 0} {incr npend}
	}
   chainDel $subChain
   genericChainDel $chain
   puts $outid ""
   puts $outid "PT DETAIL"
   puts $outid "Number of nights with data: [llength $mjds]"
   puts $outid "Number of nights pending data processing: $npend"
   puts $outid "Number of good sequences: $nobs"
   puts $outid "Number of unique patches: [llength $patches]"
   puts $outid "Patches:"
   set patches [lsort $patches]
   loop i 0 [llength $patches] 10 {
	set n2 [expr $i+10]
	set list [lrange $patches $i $n2]
	puts $outid "   $list"
	}
   if {$outid != "stdout"} {
	close $outid
	}
   return
   }


#################################################################
#Get maximum of two angles that can wrap around 360.

proc angMax {ang1 ang2} {
   if {$ang1-$ang2 < -180.} {
	return $ang1
	}
   if {$ang1-$ang2 > 180.} {
	return $ang2
	}
   return [max $ang1 $ang2]
   }

#################################################################
#Get minimum of two angles that can wrap around 360.

proc angMin {ang1 ang2} {
   if {$ang1-$ang2 < -180.} {
	return $ang2
	}
   if {$ang1-$ang2 > 180.} {
	return $ang1
	}
   return [min $ang1 $ang2]
   }

global env
if {![info exists env(LYNX_DIR)]} {
   echo Warning: lynx not setup
   }

#################################################################
#Generate a report for Boroski's quarterly report.  Input is first and
#last chunk id numbers.  Too many variables to be able to do this
#automatically.  Actually, I could use the plateRun info.  Someday ...

proc drillReport {chunk1 chunk2} {
   set files [lsort [glob -nocomplain plateList*.par]]
   set file [lindex $files end]
   if {$file == ""} return
   set chain [param2Chain $file hdr]
   set drill() ""
   chainEach hndl $chain {
	set tileRun [exprGet $hndl.tileRun]
	if {![regexp {([^0-9]+)([0-9]+)} $tileRun all prog chunk]} {
	   continue
	   }
	if {$chunk < $chunk1 || $chunk > $chunk2} continue
	if {[lsearch $drill() $prog] < 0} {
	   lappend drill() $prog
	   set drill($prog) 0
	   }
	incr drill($prog)
	}
   set ntot 0
   foreach prog [lsort $drill()] {
	echo $prog $drill($prog)
	incr ntot $drill($prog)
	}
   echo Total: $ntot
   return
   }

#########################################################################
#Map report file can be a problem.  Fix here

proc mapReportFix {file} {
   set fid [open $file]
   set tmpid [open tmpfile w]
   while {1} {
	if {[eof $fid]} break
	set line [gets $fid]
	if {[regexp {^MAPSTAT} $line]} {
	   if {![info complete $line]} {
		echo Problem: $line
		set rest [gets $fid]
		append line $rest
		}
	   }
	puts $tmpid $line
	}
   close $fid
   close $tmpid
   exec cp $file $file.bak
   exec cp tmpfile $file
   exec rm tmpfile
   return
   }
