#Check list of plates vs. plateList

proc dr1 {} {
   global dr1 plate
   if {![info exists dr1]} {
	set dr1 [param2Chain dr1_plates.par hdr]
	}

#plate chain holds all plates.
   if {![info exists plate]} {
	set file [lindex [lsort [glob plateList-*.par]] end]
	set plate [param2Chain $file hdr]
	}
   set chunks ""
   set south ""
   set mjdmax 0
   set dr1List ""
   chainEach hndl $dr1 {
	set plateId [exprGet $hndl.plate]
	lappend dr1List $plateId
	set mjd [exprGet $hndl.mjd]
	set hndl [chainGetOne $plate plateId $plateId]
	set prog [exprGet $hndl.tileRun]
	handleDel $hndl
	if {![regexp {chunk([0-9]+)} $prog all id]} {
	   echo Plate $plateId has program $prog!
	   continue
	   }
	if {[lsearch $chunks $id] < 0} {lappend chunks $id}
	set mjdmax [max $mjdmax $mjd]
	if {$mjd > 52100} {
	   if {[lsearch $south $id] < 0} {lappend south $id}
	   }
	}
   echo MJDMAX: $mjdmax
   set south [lsort -integer $south]
   echo South chunks? $south
   set chunks [lsort -integer $chunks]
   echo All chunks: $chunks

#Compare with dr1 list
   set chain [param2Chain edrPlate.par hdr]
   chainEach hndl $chain {
	set plateId [exprGet $hndl.plateId]
	set h [chainGetOne $plate plateId $plateId]
	set prog [exprGet $h.tileRun]
	handleDel $h
	if {[lsearch $dr1List $plateId] < 0} {
	   echo EDR plate not in dr1 list: $plateId program $prog
	   }
	}
   genericChainDel $chain
   return
   }

####################################################################
#Compute footprint area
#Fetch files from www-sdss.fnal.gov website.

proc targetFetch {} {
   echo Fetching dr1targ.par
   exec lynx -dump -width 120 "http://das.sdss.org/DR1/doc/coverage/tsChunk.dr1targ.par" \
  > dr1targ.par
   return
   }

############################################################################
proc bestFetch {} {
   echo Fetching dr1best.par
   exec lynx -dump -width 120 "http://das.sdss.org/DR1/doc/coverage/tsChunk.dr1best.par" \
  > dr1best.par
   return
   }

######################################################################
proc dr1foot {} {
   targetFetch
   bestFetch
   set chain [param2Chain dr1targ.par hdr]
   set targfoot 0
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripeNumber]
	set startMu [expr [exprGet $hndl.startMu]/3600. - 185.]
	set endMu [expr [exprGet $hndl.endMu]/3600. - 185.]
	if {$stripe <= 45} {
	   set incr [expr (sin($endMu/57.3) - sin($startMu/57.3))*57.3 \
		*(1.25/6.)]
	} else {
	   set incr [expr ($endMu-$startMu)*(1.25/6.)]
	   }
	if {$incr < 0} {set incr [expr -1.*$incr]}
	set targfoot [expr $targfoot + $incr]
	}
   set targfoot [format %.1f $targfoot]
   echo target footprint = $targfoot
   genericChainDel $chain

   set chain [param2Chain dr1best.par hdr]
   set bestfoot 0
   set bestunique 0
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripeNumber]
	set startMu [expr [exprGet $hndl.startMu]/3600. - 185.]
	set endMu [expr [exprGet $hndl.endMu]/3600. - 185.]
	if {$stripe <= 45} {
	   set incr [expr (sin($endMu/57.3) - sin($startMu/57.3))*57.3 \
		*(1.25/6.)]
	} else {
	   set incr [expr ($endMu-$startMu)*(1.25/6.)]
	   }
	if {$incr < 0} {set incr [expr -1.*$incr]}
	set bestfoot [expr $bestfoot + $incr]
	set incr [expr ($endMu - $startMu)]
	if {$incr < 0} {set incr [expr $incr + 360.]}
	set bestunique [expr $bestunique + $incr*1.25/6.]
	}
   set bestfoot [format %.1f $bestfoot]
   echo best footprint = $bestfoot
   echo best unique [format %.1f $bestunique]
   genericChainDel $chain

   return
   }

	
