#Get list of runs, plates for dr2

#Useful commands:
#   dr2runs
#   dr2plates	- be careful that specQualityList.par file is not in the
#	middle of being regenerated!
#   dr2publish
#   plateVerify - make sure all plates have runs in targetList
#   chunkCheck - look for unexported chunks.

#Fixes for DR2:

#   Runs to exclude that would otherwise be on final list
global exclude
set exclude "94 125 1033 1056 1755"

#   Runs to include that would otherwise be left off the final list
global include
set include "2738 3325 3388 2141"

#   Chunks that we don't want to include (outside survey area, bad seeing, etc)
global chunkOut
set chunkOut "82-1589998"

#Bad MJD's (bad flexure)
set badMJD(574) 52366
#set badMJD(298) 51955
#set badMJD(336) 51999
#set badMJD(827) 52312
#set badMJD(902) 52409
set badMJD(968) 52412

#Electronic noise with better MJD:
set badMJD(293) 51689
set badMJD(352) 51694

#Red Monster - there are other plates
set badMJD(360) 51780
set badMJD(412) 51931

#######################################################################
#Publish results

proc dr2publish {} {
   global targetList finalList runData dr1target dr1best idList chunkData \
	segData
   global finalPlateList plateData dr1Plate
   set tstamp [fmtclock [getclock] "%Y-%h-%d"]
   set mjd [expr ([getclock]/86400) + 40587]
   set fid [open dr2target.par w]
   puts $fid "#DR2 Target runs"
   puts $fid "#Read and Destroy!"
   puts $fid ""
   puts $fid "date $tstamp"
   puts $fid "mjd $mjd"
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int mjd;"
   puts $fid "   int stripe;"
   puts $fid "   char strip\[2\];"
   puts $fid "   char release\[10\];"
   puts $fid "   } TARGRUN"
   puts $fid ""
   foreach run $targetList {
	if {[lsearch $dr1target $run] >= 0} {
	   set release dr1
	} else {
	   set release dr2
	   }
	puts $fid "targrun $run $runData($run,mjd) $runData($run,stripe) \
	   $runData($run,strip) $release"
	}
   close $fid

   set fid [open dr2best.par w]
   puts $fid "#DR2 Best runs"
   puts $fid "#Read and Destroy!"
   puts $fid ""
   puts $fid "date $tstamp"
   puts $fid "mjd $mjd"
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int mjd;"
   puts $fid "   int stripe;"
   puts $fid "   char strip\[2\];"
   puts $fid "   char release\[10\];"
   puts $fid "   } BESTRUN"
   puts $fid ""
   foreach run $finalList {
	if {[lsearch $dr1best $run] >= 0} {
	   set release dr1
	} else {
	   set release dr2
	   }
	puts $fid "bestrun $run $runData($run,mjd) $runData($run,stripe) \
	   $runData($run,strip) $release"
	}
   close $fid

   set fid [open dr2spectro.par w]
   puts $fid "#DR2 Plates"
   puts $fid "#Read and Destroy!"
   puts $fid ""
   puts $fid "date $tstamp"
   puts $fid "mjd $mjd"
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int plateId;"
   puts $fid "   int mjd;"
   puts $fid "   char release\[10\];"
   puts $fid "   } PLATEMJD"
   puts $fid ""
   foreach plateId [lsort -integer $finalPlateList] {
	if {[lsearch $dr1Plate $plateId] >= 0} {
	   set release dr1
	} else {
	   set release dr2
	   }
	puts $fid "platemjd $plateId $plateData($plateId,bestmjd) $release"
	}
   close $fid

#Target chunks
   set fid [open dr2targchunk.par w]
   puts $fid "#DR2 Target chunks"
   puts $fid "#Read and Destroy!"
   puts $fid ""
   puts $fid "date $tstamp"
   puts $fid "mjd $mjd"
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   int mustart;"
   puts $fid "   int muend;"
   puts $fid "   } TSCHUNK"
   puts $fid ""

   foreach id $idList {
	puts $fid "TSCHUNK $chunkData($id,stripe) \
	   $chunkData($id,startMu) $chunkData($id,endMu)"
	}
   close $fid

#Target segments
   set fid [open dr2targseg.par w]
   puts $fid "#DR2 Target segments"
   puts $fid "#Read and Destroy!"
   puts $fid ""
   puts $fid "date $tstamp"
   puts $fid "mjd $mjd"
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int camCol;"
   puts $fid "   int rerun;"
   puts $fid "   int stripeNumber;"
   puts $fid "   char strip\[2\];"
   puts $fid "   int startMu;"
   puts $fid "   int endMu;"
   puts $fid "   int field0;"
   puts $fid "   int nFields;"
   puts $fid "   char status\[20\];"
   puts $fid "   } TSSEG"
   puts $fid ""

   foreach id $idList {
	set stripe $chunkData($id,stripe)
	foreach strip "N S" {
	   foreach camcol "1 2 3 4 5 6" {
		foreach segid $chunkData($id,$strip,$camcol,segid) {
		set run $chunkData($id,$strip,$camcol,$segid,run)
		set rerun $chunkData($id,$strip,$camcol,$segid,rerun)
		   if {$segData($segid,$camcol,status) != "primary"} {
			continue
			}
		   puts $fid "TSSEG $run $camcol $rerun $stripe $strip \
			$segData($segid,$camcol,startMuSeg) \
			$segData($segid,$camcol,endMuSeg) \
			$segData($segid,$camcol,field0) \
			$segData($segid,$camcol,nfield) \
			$segData($segid,$camcol,status)"
		   }
		}
	   }
	}
   close $fid

   return
   }

##################################################################
#Check if target chunks exist
proc chunkCheck {} {
   global chunkData segData
   set targetRoot /data/dp3.b/data/targetRoot
   set file dr2targchunk.par
   set chain [param2Chain $file hdr]
   chainEach hndl $chain {
	foreach var "stripe mustart muend" {
	   set $var [exprGet $hndl.$var]
	   }
	set name stripe[format %02d $stripe]_mu${mustart}_0
	set id $stripe-$mustart
	set dir $targetRoot/$name
	if {![file exists $dir]} {
	   echo No directory for chunk $name
	   set lambdaStart [expr $mustart/3600.-185.]
	   set lambdaEnd [expr $muend/3600.-185.]

#To match to my stripeStatus plots, subtract 180 for southern stripes
	   if {$stripe > 45 && $stripe < 100} {
		set lambdaStart [expr $lambdaStart-180.]
		set lambdaEnd [expr $lambdaEnd-180.]
		}
	   if {$lambdaStart < -180.} {set lambdaStart [expr $lambdaStart+360.]}
	   if {$lambdaStart > 180.} {set lambdaStart [expr $lambdaStart-360.]}
	   if {$lambdaEnd < -180.} {set lambdaEnd [expr $lambdaEnd+360.]}
	   if {$lambdaEnd > 180.} {set lambdaEnd [expr $lambdaEnd-360.]}
	   set lambdaStart [format %.1f $lambdaStart]
	   set lambdaEnd [format %.1f $lambdaEnd]
	   echo "   " Lambda $lambdaStart to $lambdaEnd
	   echo "   " North runs $chunkData($id,N,run)
	   echo "   " South runs $chunkData($id,S,run)
	   }
	}
   genericChainDel $chain
   return
   }

##################################################################
proc dr2runs {} {
   global chunkData segData targetList finalList global runData idList
   global dr1target dr1best

#Read list of target chunks
   set files [lsort [glob odChunkStatus*.par]]
   set file [lindex $files end]
   set chunkChain [param2Chain $file hdr]
   if {[info exists chunkData]} {unset chunkData}
   if {[info exists segData]} {unset segData}
   if {[info exists runData]} {unset runData}
   set chunkData() ""

#The chunk file combines chunk and segment information together.
#A chunk is a section of 12 camcols.  A segment is 1 camcol, 1 run.
#A camcol of a chunk may have multiple segments with different runs.
   global chunkOut
   chainEach hndl $chunkChain {
	foreach var "stripe strip run rerun camcol version \
		startField endField startMu endMu startMuSeg endMuSeg \
		version status" {
	   set $var [exprGet $hndl.$var]
	   }

#Just do target chunks.  Best chunks are almost always the same except in
#the south.
	if {$version != 0} continue

#Ignore secondary status.  Otherwise, I will get duplicate segments
	if {$status != "primary"} continue

#chunkid is the name of the chunk.
#I will accumulate run,rerun info in chunkData.
	set chunkid $stripe-$startMu

#Segments need their own id - sigh.
	set segid $run-$startMuSeg

#Chunks with runs that I want to exclude are not eliminated by the
#exclude mechanism (those runs are deleted only from best).  So I have
#an explicit list of chunks to omit.
	if {[lsearch $chunkOut $chunkid] >= 0} continue

#Collect segment information.  I assume all camcols go from 1 to 6
	set segData($segid,$camcol,startMuSeg) $startMuSeg
	set segData($segid,$camcol,endMuSeg) $endMuSeg
	set segData($segid,$camcol,run) $run
	set segData($segid,$camcol,field0) $startField
	set nfield [expr $endField-$startField+1]
	set segData($segid,$camcol,nfield) $nfield
	set segData($segid,$camcol,status) $status

#The id is unique to a chunk.
	if {[lsearch $chunkData() $chunkid] < 0} {
	   lappend chunkData() $chunkid
	   foreach i "1 2 3 4 5 6" {
		set chunkData($chunkid,$strip,$i,segid) ""
		}
	   }
	lappend chunkData($chunkid,$strip,$camcol,segid) $segid
	set chunkData($chunkid,$strip,$camcol,$segid,run) $run
	set chunkData($chunkid,$strip,$camcol,$segid,rerun) $rerun
	set chunkData($chunkid,stripe) $stripe
	set chunkData($chunkid,startMu) $startMu
	set chunkData($chunkid,endMu) $endMu
	}

#Get list of all runs
   set runChain [param2Chain run.par hdr]
   set dr1List ""
   set dr2List ""
   set runData() ""
   chainEach hndl $runChain {
       foreach var "run mjd stripe strip quality" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$quality == "bad"} continue
	lappend runData() $run
	set runData($run,mjd) $mjd
	set runData($run,strip) $strip
	set runData($run,stripe) $stripe
	set runData($run,targrerun) ""
	set runData($run,bestrun) ""

#Format run assignments - there were some exceptions later
	if {$mjd < 52091} {
	   lappend dr1List $run
	} elseif {$mjd < 52473} {
	   lappend dr2List $run
	   }
	}
   genericChainDel $runChain

#Now get final list run list
   set targetList ""
   set finalList ""
   set idList ""
   foreach id $chunkData() {
        if {![info exists chunkData($id,S,3,segid)] || ![info exists \
	   chunkData($id,N,3,segid)]} {
	   echo Gak! Chunk $id does not have both halves!
	   continue
	   }

#For any target chunk, I require that all runs be on the dr2 list.
#Otherwise bookkeeping of plates is a nightmare.
	set in 1
	foreach sid $chunkData($id,S,3,segid) {
	   set srun $chunkData($id,S,3,$sid,run)
	   foreach nid $chunkData($id,N,3,segid) {
		set nrun $chunkData($id,N,3,$nid,run)
		if {[lsearch $dr1List $srun] < 0 && \
		   [lsearch $dr2List $srun] < 0} {
		   set in 0
		   }
		if {[lsearch $dr1List $nrun] < 0 && \
		   [lsearch $dr2List $nrun] < 0} {
		   set in 0
		   }
		}
	   }
	if {$in == 0} continue

#Keep track of ids.
	lappend idList $id
	foreach sid $chunkData($id,S,3,segid) {
	   set srun $chunkData($id,S,3,$sid,run)
	   foreach nid $chunkData($id,N,3,segid) {
		set nrun $chunkData($id,N,3,$nid,run)
		if {[lsearch $finalList $srun] < 0} {
		   lappend finalList $srun
		   lappend targetList $srun
		   }
		if {[lsearch $finalList $nrun] < 0} {
		   lappend finalList $nrun
		   lappend targetList $nrun
		   }
		}
	   }
	}

#Fixups to final list
   global include exclude
   foreach run $exclude {
	set index [lsearch $finalList $run]
	if {$index >= 0} {set finalList [lreplace $finalList $index $index]}
	}

   foreach run $include {
	set index [lsearch $finalList $run]
	if {$index < 0} {lappend finalList $run}
	}

#Compare with dr1 target and best lists
   set dr1target [param2Chain dr1targ.par hdr]
   set repeatList ""
   chainEach hndl $dr1target {
	foreach var "run" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $repeatList $run] >= 0} continue
	lappend repeatList $run
	if {[lsearch $targetList $run] >= 0} {
	   echo DR1 target run $run on targetList
	} else {
	   echo DR1 target run $run NOT on targetList!
	   lappend targetList $run
	   }
	}
   genericChainDel $dr1target

   set dr1best [param2Chain dr1best.par hdr]
   set repeatList ""
   chainEach hndl $dr1best {
	foreach var "run" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $repeatList $run] >= 0} continue
	lappend repeatList $run
	if {[lsearch $finalList $run] >= 0} {
	   echo DR1 best run $run on finalList
	} else {
	   echo DR1 best run $run NOT on any finalList!
	   lappend finalList $run
	   }
	}
   genericChainDel $dr1target

   set targetList [lsort -integer $targetList]
   set finalList [lsort -integer $finalList]
echo targetList $targetList
echo finalList $finalList
   return
   }

#########################################################################
#Define the HOLETYPE schema in advance
proc holeDef {} {
typedef enum {OBJECT, COHERENT_SKY, GUIDE, LIGHT_TRAP, ALIGNMENT, QUALITY} HOLETYPE;

typedef enum {GALAXY, QSO, STAR_BHB, STAR_BROWN_DWARF, STAR_SUB_DWARF, STAR_CATY_VAR, STAR_RED_DWARF, STAR_WHITE_DWARF, REDDEN_STD, SPECTROPHOTO_STD, HOT_STD, ROSAT_A, ROSAT_B, ROSAT_C, ROSAT_D, SERENDIPITY_BLUE, SERENDIPITY_FIRST, SERENDIPITY_RED, SERENDIPITY_DISTANT, SERENDIPITY_MANUAL, QA, SKY, NA, STAR_PN, STAR_CARBON} OBJTYPE;
}

#########################################################################
proc dr2plates {} {
   global tileData plateData mapData finalPlateList
   global dr1Plate
   global specData princetonData
   global badMJD

   holeDef
   foreach var "tileData plateData mapData" {
	if {[info exists $var]} {unset $var}
	}

#Fetch quality
   specQualityFetch
   specQualityRead

#Fetch Princeton view of the world
   princetonRead

#Read plateList - this gives most complete info
   set files [lsort [glob plateList*.par]]
   set file [lindex $files end]
   set plateChain [param2Chain $file hdr]
   set tileData() ""
   set plateData() ""
   chainEach hndl $plateChain {
	foreach var "plateId tileId tileRun raCen decCen tileStatus \
	   plateStatus" {
	   set $var [exprGet $hndl.$var]
	   }

#Track plates and tiles in separate lists
	if {[lsearch $tileData() $tileId] < 0} {
	   lappend tileData() $tileId
	   set tileData($tileId,raCen) $raCen
	   set tileData($tileId,decCen) $decCen
	   set tileData($tileId,tileRun) $tileRun
	   set tileData($tileId,tileStatus) $tileStatus

#This should be a list of good plates
	   set tileData($tileId,plateId) ""
	   }
	lappend plateData() $plateId
	set plateData($plateId,tileId) $tileId
	set plateData($plateId,plateStatus) $plateStatus

#This should be a list of good maps
	set plateData($plateId,mapId) ""

#This is the mjd with the highest s/n
	set plateData($plateId,bestmjd) ""
	}
   genericChainDel $plateChain

#Read dr1 list.  In resolving multiple plates per tile, use DR1 choice
#if it exists.
   set dr1Plate ""
   set dr1Tile ""
   set dr1Chain [param2Chain dr1spectro.par hdr]

#Run through old dr1 list and cache info
   chainEach hndl $dr1Chain {
	set plateId [exprGet $hndl.plate]
	set mjd [exprGet $hndl.mjd]
	set tileId $plateData($plateId,tileId)
	lappend dr1Plate $plateId
	lappend dr1Tile $tileId
	set dr1TileData($tileId,plateId) $plateId
	set dr1TileData($tileId,mjd) $mjd
	}
   genericChainDel $dr1Chain

#Read list of plate maps
   set files [lsort [glob mapReport*.par]]
   set file [lindex $files end]
   set mapChain [param2Chain $file hdr]
   set mapData() ""
   set dr1List ""
   set dr2List ""
   chainEach hndl $mapChain {
	foreach var "mapId plateId mjdobs date_done" {
	   set $var [exprGet $hndl.$var]
	   }
	set date_done [lindex $date_done 0]
	set mjdobs [lindex $mjdobs 0]
	eval set mjd [eval max $mjdobs 0]

#Don't use bad flexure plates
	if {[info exists badMJD($plateId)]} {
	   if {[lsearch $badMJD($plateId) $mjd] >= 0} continue
	   }
#Don't check mjd yet.  My criterion will be that we should release data only
#if the latest mjd for a plate is within our data release window.
#	if {$mjd > 52473} continue
	if {$date_done == ""} continue
	if {$plateId < 266} continue
	if {![info exists plateData($plateId,tileId)]} {
	   echo Gak! plate $plateId with data is not in plateInfo file!
	   continue
	   }
	set tileId $plateData($plateId,tileId)
	set tileRun $tileData($tileId,tileRun)
	if {![regexp chunk $tileRun]} continue

#A given plate may have multiple good observations.
	if {[lsearch $tileData($tileId,plateId) $plateId] < 0} {
	   lappend tileData($tileId,plateId) $plateId
	   }
	lappend plateData($plateId,mapId) $mapId
	lappend mapData() $mapId
	set mapData($mapId,mjd) $mjd
	set mapData($mapId,plateId) $plateId
	}
   genericChainDel $mapChain

#Now go through tiles.  For those on the existing dr1list, select the dr1
#plate for my new list, else select the higher numbered plate.
   foreach tileId $tileData() {
	if {[lsearch $dr1Tile $tileId] >= 0} {
	   set plateId $dr1TileData($tileId,plateId)
	   set mjd $dr1TileData($tileId,mjd)

#Some DR1 plates have bad MJD!
	   if {![info exists badMJD($plateId)] || \
	      [lsearch $badMJD($plateId) $mjd] < 0} {
		set plateData($plateId,bestmjd) $mjd
		lappend dr1List $plateId
		continue
		}
	   }
	if {$tileData($tileId,plateId) == ""} continue

#If multiple plates for a tile, take maximum plateId.  However, for tileId 159,
#plate 360 was used in dr1, but it is flaky data, and we want to use newer
#mjd, but plate 362 is for same tile and we would select it by the following
#algorithm, which is not what is desired.  I'll just echo mutiple plateId's
#per tile and see what happens.
	set plateId [eval max $tileData($tileId,plateId) 0]

#I will just hardwire the tileId 159 case for now
	if {$tileId == 159} {
	   set plateId [eval min $tileData($tileId,plateId) 99999]
	   }

#For now, just get the candidate mjds
	set mjds ""
	foreach mapId $plateData($plateId,mapId) {
	   set mjd $mapData($mapId,mjd)
	   lappend mjds $mjd
	   }

#Which data release?
	set bestmjd [eval max $mjds 0]
	set bestsn 0
	foreach mjd $mjds {
	   if {![info exists specData($plateId,$mjd,minsn)]} {
		echo Gak, no quality info for plateId $plateId mjd $mjd
		continue
		}

#If psf >= 1.15, skip this mjd.  If it is the only mjd, we will still use it.
	   if {$specData($plateId,$mjd,psf) > 1.15} continue
	   if {$specData($plateId,$mjd,minsn) > $bestsn} {
		set bestsn $specData($plateId,$mjd,minsn)
		set bestmjd $mjd
		}
	   }
	set mjd $bestmjd
	set plateData($plateId,bestmjd) $mjd
	if {$mjd < 52091} {
	   lappend dr1List $plateId
	} elseif {$mjd < 52473} {
	   lappend dr2List $plateId
	} else {
	   continue
	   }

#Check psf quality
	if {$specData($plateId,$mjd,psf) >= 1.2} {
	   echo Plate $plateId, mjd $mjd psf too big.  Not including plate.
	   continue
	   }
#Check princeton quality
	if {[info exists princetonData($plateId,$mjd,quality)]} {
	   set quality $princetonData($plateId,$mjd,quality)
	   if {$quality != "good"} {
	      echo Plate $plateId mjd $mjd princeton says $quality \
		$princetonData($plateId,$mjd,qualcomments)
		}
	   }
#Will there be any plates released for which the imaging data is not?
#That would be difficult!
	}
   echo New dr1List is $dr1List
   echo New dr2List is $dr2List

#Compare new dr1 list with old dr1 list.
   foreach plateId $dr1List {
	if {[lsearch $dr1Plate $plateId] < 0} {
	   echo New plate $plateId not on original dr1 list
	   }
	}

#Compare old dr1 list against new lists.
   foreach plateId $dr1Plate {
	if {[lsearch $dr2List $plateId] >= 0} {
	   echo Interesting! Old dr1 plate $plateId now on dr2 list!
	} elseif {[lsearch $dr1List $plateId] < 0} {
	   echo Interesting! Old dr1 plate $plateId not found on new lists
	   }
	}

#Get list of runs in each plPlugMapP file
   set finalPlateList [join "$dr1List $dr2List"]

#Don't do this any more - everything checks OK and it is slow to run.
   return
   foreach plateId $finalPlateList {
	set mapId [lindex $plateData($plateId,mapId) end]
	set mjd $mapData($mapId,mjd)
	set logDir /data/dp9.a/data/spectro/speclog/$mjd
	set file $logDir/plPlugMapM-$mapId.par
	if {$file == ""} {
	   echo No plPlugMapM file for plate $plateId mjd $mjd!
	   continue
	   }
	set chain [param2Chain $file hdr]
	set runs ""
	chainEach hndl $chain {
	   set run [exprGet $hndl.objId<0>]
	   set holeType [exprGet -enum $hndl.holeType]
	   if {$holeType != "OBJECT"} continue
	   if {[lsearch $runs $run] < 0} {lappend runs $run}
	   }
	genericChainDel $chain

#Cache info with plateData, even though it proper belongs with the tile.
	echo plateId $plateId run $runs
	set plateData($plateId,runs) $runs
	}
   return
   }

####################################################################
#Make sure target runs for all plates in dr2 are on targetlist
proc plateVerify {} {
   global plateData finalPlateList
   global targetList
   foreach plateId $finalPlateList {
	foreach run $plateData($plateId,runs) {
	   if {[lsearch $targetList $run] < 0} {
		echo Plate $plateId run $run not on targeList!
		}
	   }
	}
   return
   }

####################################################################
#Fetch S/N stuff - actually, fetch the Annis web page.
####################################################################
proc specQualityFetch {} {
   echo Fetching spectro quality web page
   exec lynx -dump -width 200 "http://www-sdss.fnal.gov/spectroRoot/3w_live/qualityList.par" \
  > specQualityList.par
   return
   }

####################################################################
#Read the above file.  Alas, it is still broken as a param file, so read by
#hand and save info in an array
proc specQualityRead {} {
   global specData
   if {[info exists specData]} {unset specData}
   set fid [open specQualityList.par]
   set specData() ""
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	if {![regexp {^spqcweb} $line]} continue
	set plateId [lindex $line 1]
	set mjd [lindex $line 2]
	set minsn [lindex $line 8]
	set psf [lindex $line 15]
	if {[catch {format %f $psf}]} {
	   echo Bad psf for plate $plateId mjd $mjd
	   set psf .9
	   }
	if {[lsearch $specData() $plateId] < 0} {
	   lappend specData() $plateId
	   set specData($plateId,mjd) ""
	   }
	lappend specData($plateId,mjd) $mjd
	set specData($plateId,$mjd,minsn) $minsn
	set specData($plateId,$mjd,psf) $psf
	}
   close $fid
   return
   }
####################################################################
#Read the above file.  Alas, it is still broken as a param file, so read by
#hand and save info in an array
proc princetonRead {} {
   global princetonData
   if {[info exists princetonData]} {unset princetonData}
   set files [lsort [glob princetonQuality*.par]]
   set file [lindex $files end]
   set chain [param2Chain $file hdr]
   set princetonData() ""
   chainEach hndl $chain {
	foreach var "plateId mjd quality qualcomments" {
	   set $var [exprGet $hndl.$var]
	   }
	set quality [lindex $quality 0]
	set qualcomments [lindex $qualcomments 0]
	if {[lsearch $princetonData() $plateId] < 0} {
	   lappend princetonData() $plateId
	   set princetonData($plateId,mjd) ""
	   }
	lappend princetonData($plateId,mjd) $mjd
	set princetonData($plateId,$mjd,quality) $quality
	set princetonData($plateId,$mjd,qualcomments) $qualcomments
	}
   genericChainDel $chain
   return
   }
####################################################################
#Compute footprint area
#Fetch files from www-sdss.fnal.gov website.

proc targetFetch {} {
   echo Fetching dr1targ.par
   exec lynx -dump -width 120 "http://das.sdss.org/DR1/doc/coverage/tsChunk.dr1targ.par" \
  > dr1targ.par
   return
   }

############################################################################
proc bestFetch {} {
   echo Fetching dr1best.par
   exec lynx -dump -width 120 "http://das.sdss.org/DR1/doc/coverage/tsChunk.dr1best.par" \
  > dr1best.par
   return
   }

######################################################################
proc dr2foot {} {
   targetFetch
   bestFetch
   set chain [param2Chain dr2targ.par hdr]
   set targfoot 0
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripeNumber]
	set startMu [expr [exprGet $hndl.startMu]/3600. - 185.]
	set endMu [expr [exprGet $hndl.endMu]/3600. - 185.]
	if {$stripe <= 45} {
	   set incr [expr (sin($endMu/57.3) - sin($startMu/57.3))*57.3 \
		*(1.25/6.)]
	} else {
	   set incr [expr ($endMu-$startMu)*(1.25/6.)]
	   }
	if {$incr < 0} {set incr [expr -1.*$incr]}
	set targfoot [expr $targfoot + $incr]
	}
   set targfoot [format %.1f $targfoot]
   echo target footprint = $targfoot
   genericChainDel $chain

   set chain [param2Chain dr2best.par hdr]
   set bestfoot 0
   set bestunique 0
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripeNumber]
	set startMu [expr [exprGet $hndl.startMu]/3600. - 185.]
	set endMu [expr [exprGet $hndl.endMu]/3600. - 185.]
	if {$stripe <= 45} {
	   set incr [expr (sin($endMu/57.3) - sin($startMu/57.3))*57.3 \
		*(1.25/6.)]
	} else {
	   set incr [expr ($endMu-$startMu)*(1.25/6.)]
	   }
	if {$incr < 0} {set incr [expr -1.*$incr]}
	set bestfoot [expr $bestfoot + $incr]
	set incr [expr ($endMu - $startMu)]
	if {$incr < 0} {set incr [expr $incr + 360.]}
	set bestunique [expr $bestunique + $incr*1.25/6.]
	}
   set bestfoot [format %.1f $bestfoot]
   echo best footprint = $bestfoot
   echo best unique [format %.1f $bestunique]
   genericChainDel $chain

   return
   }

#####################################################################
#Check for duplicate target segments.  (Sigh, ...)
proc dr2segCheck {} {
   set chain [param2Chain dr2targseg.par hdr]
   set list ""
   chainEach hndl $chain {
	foreach var "run camCol rerun startMu endMu" {
	   set $var [exprGet $hndl.$var]
	   }
	set id $run,$camCol,$rerun,$startMu,$endMu
	if {[lsearch $list $id] >= 0} {
	   echo Duplicate segment for $run $camCol $rerun $startMu $endMu
	} else {
	   lappend list $id
	   }
	}
   genericChainDel $chain
   return
   }

#########################################################################
proc dr2foot {} {
   set chain [param2Chain dr2targchunk.par hdr]
   set targfoot 0
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	set startMu [expr [exprGet $hndl.mustart]/3600. - 185.]
	set endMu [expr [exprGet $hndl.muend]/3600. - 185.]
	if {$stripe <= 45} {
	   set incr [expr (sin($endMu/57.3) - sin($startMu/57.3))*57.3 \
		*2.5]
	} else {
	   set incr [expr ($endMu-$startMu)*2.5]
	   }
	if {$incr < 0} {set incr [expr -1.*$incr]}
	set targfoot [expr $targfoot + $incr]
	}
   set targfoot [format %.1f $targfoot]
   echo target footprint = $targfoot
   genericChainDel $chain


   return
   }

	

