#Replace saoDisplay commands with ds9.
#This is a major hack since they use much different interfaces.

##################################################################
#Internal command - initialize.  Call this every time to make sure
#environment is set up correctly.  I use an array _ds9 to cache all
#relevant information.

proc ds9Init {} {
   global _ds9

#If first time through, make sure all unix executables exist.

#First, if _ds9 exists, check that we can run xpaget.
   if {[info exists _ds9]} {
	if {[catch {exec xpaget xpans}]} {unset _ds9}
	}
   if {![info exists _ds9]} {
	foreach pgm "ds9 xpans xpaset xpaget xpaaccess" {
	   if {![auto_execok $pgm]} {
		error "Unable to locate program $pgm needed for ds9"
		}
	   }

#Check if a name-server is running.
	set response [exec xpaaccess -c xpans]
	if {$response != "yes"} {

#Start new instance of xpans server.  Set XPA_METHOD to use local ports only.
	   global env
	   set env(XPA_METHOD) localhost
	   exec xpans &
	   set n 0
	   while {[exec xpaaccess -c xpans] != "yes"} {
		set n [expr n+.5]
		after 500
		if {$n > 10} {
		   error {Can't seem to start xpa name server}
		   }
		}
	   set _ds9() ""
	   }
	}

#Rescan for existing SAO windows
   set _ds9() ""
   set list [exec xpaget xpans]
   foreach line [split $list \n] {
	set title [lindex $line 1]
	if {[regexp {ds9-[0-9]+-([0-9]+)} $line all saoNum]} {
	   lappend _ds9() $saoNum
	   }
	}
   if {[llength $_ds9()] > 1} {
	set _ds9() [lsort -integer $_ds9()]
	}
   return
   }

#########################################################################
proc saoDisplay {reg args} {
   global _ds9
   ds9Init

   set options [lindex $args 0]
   if {[ctype digit [string index $options 0]]} {
	set saoNum $options
	set options [lindex $args 1]
   } else {
	set saoNum ""
	}

#Demand that saoNum be an integer if input.
   if {$saoNum != ""} {
	if {![ctype digit $saoNum]} {
	   error "Input saoNum $saoNum is not a number!"
	   }
	}

#If saoNum is input, use it as the number.  Else, pick one.
   if {$saoNum == ""} {
	if {[llength $_ds9()] == 0} {
	   set saoNum 1
	} elseif {[llength $_ds9()] == 1} {
	   set saoNum [expr $_ds9() + 1]
	} else {
	   set saoNum [expr [eval max $_ds9()] + 1]
	   }
	}

#Parse options into those that can only be invoked at the start and
#those that can be sent later
   set options1 ""
   set options2 ""
   set min ""
   set max ""
   while {[llength $options] > 0} {
	set token [lvarpop options]
	if {$token == "-gd"} {
	   set geom [lvarpop options]

#Geometry is not currently useful for ds9
#	   lappend options1 "-geometry" "$geom"
	} elseif {$token == "-min"} {
	   set min [lvarpop options]
	} elseif {$token == "-max"} {
	   set max [lvarpop options]
	} elseif {$token == "-sqrt"} {
	   lappend options2 [list scale sqrt]
	   }
	}

#Handle special case of limits.  Must have both min, max.
   if {$min != "" && $max != ""} {
	lappend options2 [list scale limits $min $max]
	}

#Does the window exist?
   set title ds9-[pid]-$saoNum
   if {[lsearch $_ds9() $saoNum] < 0} {
	eval exec ds9 -title $title $options1 &

#Need to wait for ds9 to start
	set n 0.
	while {1} {
	   set response [exec xpaaccess -c $title]
	   if {$response == "yes"} {
		set n -1
		break
		}
	   set n [expr $n+.5]
	   if {$n >= 15.} break
	   after 500
	   }
	if {$n > 0} {
	   error "Timed out waiting for ds9 saoNum $saoNum"
	   }
	}

   foreach option $options2 {
	eval exec xpaset -p $title $option
	}
	
#Pipe binary to ds9?  Too many combinations for my brain to handle.  Unsigned
#ints are kind of nasty - ds9 will barf, I think.  So just write out a file
#and tell ds9 to load.
   set file /tmp/tempfile-[pid].fit
   regWriteAsFits $reg $file
   exec xpaset -p $title file $file
   exec rm $file
   set _ds9($saoNum) $title
   return $saoNum
   }

########################################################################
set saoDrawBoxArgs {
    {saoDrawBox "Draw a box in an SAO Display"}
    {<row> INTEGER "" row  "row"}
    {<col> INTEGER "" col   "col"}
    {<rowsize> INTEGER "" height "rowsize"}
    {<colsize> INTEGER "" width  "colsize"}
    {<ang> INTEGER "" ang  "Position angle"}
    {-s INTEGER 0 saoNum  "Display"}
}

#Don't support -the -e or -i flags (since I don't know what they do)
proc saoDrawBox {args} {
   global _ds9
   if {![info exists _ds9]} {ds9Init}

   upvar #0 saoDrawBoxArgs formal_list
   if {[shTclParseArg $args $formal_list saoDrawBox] == 0} {return}

   if {$saoNum == 0} {set saoNum [lindex $_ds9() 0]}
   if {$saoNum == ""} return
   if {![info exists _ds9($saoNum)]} return
   exec xpaset -p $_ds9($saoNum) regions box $col $row $width  $height $ang
   return
   }

########################################################################
set saoDrawCircleArgs {
    {saoDrawCircle "Draw a circle in an SAO Display"}
    {<row> INTEGER "" row  "row"}
    {<col> INTEGER "" col   "col"}
    {<radius> INTEGER "" radius "radius"}
    {-s INTEGER 0 saoNum  "Display"}
}

#Don't support -the -e or -i flags (since I don't know what they do)
proc saoDrawCircle {args} {
   global _ds9
   if {![info exists _ds9]} ds9Init

   upvar #0 saoDrawCircleArgs formal_list
   if {[shTclParseArg $args $formal_list saoDrawCircle] == 0} {return}

   if {$saoNum == 0} {set saoNum [lindex $_ds9() 0]}
   if {$saoNum == 0} return
   if {![info exists _ds9($saoNum)]} return
   exec xpaset -p $_ds9($saoNum) regions circle $col $row $radius
   return
   }

########################################################################
set saoDrawTextArgs {
    {saoDrawText "Draw text in an SAO Display"}
    {<row> INTEGER "" row  "row"}
    {<col> INTEGER "" col   "col"}
    {<text> STRING "" text "text"}
    {-s INTEGER 0 saoNum  "Display"}
}

#Don't support -the -e or -i flags (since I don't know what they do)
proc saoDrawText {args} {
   global _ds9
   if {![info exists _ds9]} ds9Init

   upvar #0 saoDrawTextArgs formal_list
   if {[shTclParseArg $args $formal_list saoDrawText] == 0} {return}

   if {$saoNum == 0} {set saoNum [lindex $_ds9() 0]}
   if {$saoNum == 0} return
   if {![info exists _ds9($saoNum)]} return
   exec xpaset -p $_ds9($saoNum) regions text $col $row $text
   return
   }

########################################################################
set saoDrawEllipseArgs {
    {saoDrawBox "Draw an ellipse in an SAO Display"}
    {<row> INTEGER "" row  "row"}
    {<col> INTEGER "" col   "col"}
    {<rowRadius> INTEGER "" rowRadius "rowRadius"}
    {<colRadius> INTEGER "" colRadius "colRadius"}
    {<ang> INTEGER "" ang "position angle"}
    {-s INTEGER 0 saoNum  "Display"}
}

#Don't support -the -e or -i flags (since I don't know what they do)
proc saoDrawEllipse {args} {
   global _ds9
   if {![info exists _ds9]} ds9Init

   upvar #0 saoDrawEllipseArgs formal_list
   if {[shTclParseArg $args $formal_list saoDrawEllipse] == 0} {return}

   if {$saoNum == 0} {set saoNum [lindex $_ds9() 0]}
   if {$saoNum == 0} return
   if {![info exists _ds9($saoNum)]} return
   exec xpaset -p $_ds9($saoNum) regions ellipse $col $row \
	$colRadius $rowRadius $ang
   return
   }

####################################################################
#I won't support -a option because I don't know what it does

proc saoDel {{saoNum ""}} {
   global _ds9
   ds9Init
   if {$saoNum == ""} {set saoNum [lindex $_ds9() 0]}
   if {[info exists _ds9($saoNum)]} {
	exec xpaset -p $_ds9($saoNum) exit
	unset _ds9($saoNum)
	set index [lsearch $_ds9() $saoNum]
	if {$index >= 0} {set _ds9() [lreplace $_ds9() $index $index]}
	}
   return
   }

########################################################################
proc saoReset {{saoNum ""}} {
   global _ds9
   if {![info exists _ds9]} ds9Init
   if {$saoNum == ""} {set saoNum [lindex $_ds9() 0]}
   if {[info exists _ds9($saoNum)]} {
	exec xpaset -p $_ds9($saoNum) regions delete all
	}
   return
   }

########################################################################
#Do nothing.  We don't have labels in ds9
proc saoLabel {args} {
   return
   }
