#################################################################
#Read final calibrations

proc fcalibRead {args} {

#We read 6 distinct files.
   mtGlobals
   global fcalib

   set runs [fieldExpand $args]

   foreach run $runs {
        set topdir $fcdir($run)
        foreach col $camcols {
	   if {[info exists fcalib($run,$col)]} {
		catch {genericChainDel $fcalib($run,$col)}
		}
	   set file $fcdir($run)/fcPCalib-[format %06d $run]-$col.fit
	   if {![file exists $file]} {
		echo No file $file
		set fcalib($run,$col) [fcalibFake $col]
		set fcalib($run,$col,version) ""
		set fcalib($run,$col,filters) "u g r i z"
		continue
		}
           set fcalib($run,$col) [fits2Schema $file FCALIB]

#Read the version and filters
	   set hdr [hdrNew]
	   hdrReadAsFits $hdr $file
	   set version [string trim [hdrGetAsAscii $hdr VERSION]]
	   set fcalib($run,$col,version) $version
	   if {![catch {set filters [string trim [hdrGetAsAscii \
			$hdr FILTERS]]}]} {
		set fcalib($run,$col,filters) $filters
		}
	   hdrDel $hdr
           }
        }
   return
   }

####################################################################
#Read astrometric solutions. Store as chains in astrom array
#For now, asdir points to a specific directory, so we can only process
#one run at a time.

proc astromRead {args} {
#We read 6 distinct tables.  Easiest to return results in an array of chains
   mtGlobals
   global astrom

#Read every 6th fits table
   set runs [fieldExpand $args]

   foreach run $runs {
	if {![info exists asdir($run)]} {
	   dataDirs $run
	   }

        set topdir $asdir($run)

#Get node and inclination from pdu header
	set hdr [hdrNew]
	hdrReadAsFits $hdr [format $topdir/asTrans-%06d.fit $run]
	set node [hdrGetAsDbl $hdr NODE]
	set incl [hdrGetAsDbl $hdr INCL]
	set astrom($run,node) $node
	set astrom($run,incl) $incl
	set filters [hdrGetAsAscii $hdr FILTERS]
	hdrDel $hdr
        foreach camcol "1 2 3 4 5 6" {
	   foreach filter $filters {
	      if {[info exists astrom($run,$camcol,$filter)]} {
		genericChainDel $astrom($run,$camcol,$filter)
		}
	      }
	   set hdu [expr ($camcol - 1)*5]
	   foreach filter $filters {
		incr hdu
		set astrom($run,$camcol,$filter) [fits2Schema \
		   [format $topdir/asTrans-%06d.fit $run] ASTRANS \
                   -hdu $hdu]
		}
           }
        }
   return
   }


