#This file contains support code for converting Fits ascii or binary tables
#to chains and back automatically using the contrib replacements routines.
#   Old: fits2Schema	New: fits2Chain
#   Old: schema2Fits	New: chain2Fits
#
#Procedures defined here:
#	fits2Chain file type [hdr] [-hdu=n]  <--- optional: hdr set if supplied,
#						  hdu=1 if not provided.
#	chain2Fits chain file [hdr [-append]  <--- optional header written if
#						  supplied; append to existing
#						  file if -append is stated.
#	transTableShow xtbl
#	transTableAdd			Private function
#	transTableAuto type		Create a translation table from a type.
###############################################################################

#Print contents of a contrib translation table to the screen.
proc transTableShow {xtbl} {
   if {[handleType $xtbl] != "TRANSTABLE"} {error "Not a TRANSTABLE"}
   set line [format "Type: %-15s  Constructor: %-15s  Destructor: %-15s" \
	[exprGet $xtbl.schemaTypeName] [exprGet $xtbl.constructor] \
	[exprGet $xtbl.destructor]]
   regsub -all {\(null\)} $line "      " line
   echo $line
   echo ""
   chainInit $xtbl.table<0> iter
   set format "%-2s %-10s %-10s %-10s %-10s %-10s %-10s %-10s"
   set j 0
   echo [format $format I FitsName FitsType SchName SchType SchDims \
	Dimen Heaplen]
   while {1} {
	set hndl [chainNext iter]
	if {$hndl == ""} then break
	set ndim [exprGet $hndl.schemaProgram->ndim]
	set dim ""
	loop i 0 $ndim {lappend dim [exprGet $hndl.schemaProgram->dim<$i>]}
	set line [format $format $j \
	   [exprGet $hndl.fitsEntry] \
	   [exprGet $hndl.fitsType] [exprGet $hndl.schemaEntry] \
	   [exprGet $hndl.schemaProgram->type] $dim \
	   [lindex [exprGet $hndl.dimen] 0] \
	   [lindex [exprGet $hndl.heaplen] 0]]
	regsub -all {\(null\)} $line "      " line
	echo $line
	incr j
	}
   return
   }

###############################################################################

set fits2ChainArgs {
    {fits2Chain "Read in a FITS table file and create the appropriate objects.\
n" }
    {<file> STRING "" file "FITS file to be read"}
    {<type> STRING "" type "name of schema to create"}
    {[hdr]  STRING NULL hdr "optional header, set if not NULL"}
    {-hdu   STRING 1 hdu "hdu to read in file"}
    {-sc   STRING "" sc "use this schemaTrans instead of auto-generating"}    
    {-existingType   CONSTANT "" existingType "use an existing type"}
}
proc fits2Chain {args} {
   upvar #0 fits2ChainArgs formal_list

   if {[shTclParseArg $args $formal_list fits2Schema] == 0} {
       return
   }

   if {$type == "" && $hdr == "NULL"} {
      error "You mustn't omit both type and header"
   }

   set tbl [tblColNew]

   fitsRead $tbl $file -hdu $hdu
#Create new type if not in existence already
   if {[lsearch [typesList] $type] == -1} then {
	typeNewFromTbl $tbl $type
	}

   set xtbl [transTableAuto $type]
   set chain [tblToChain  $tbl $xtbl]
   if {$hdr != "NULL"} {
	hdrCopy $tbl.hdr $hdr
	}
   tblColDel $tbl
   transTableDel $xtbl
   return $chain
   }

###############################################################################
#Write a chain to a fits table.

proc chain2Fits {args} {

   set context schema2Fits
   ftclParseSave $context
   ftclFullParse {chain file hdr=NULL -append} $args
   set chain [ftclGetStr chain]
   set file [ftclGetStr file]
   set hdr  [ftclGetStr hdr]
   if {[ftclPresent append]} then {set opts -append} else \
	{set opts "-pdu MINIMAL"}
   ftclParseRestore $context

   set type [chainTypeDefine $chain]
   set xtbl [transTableAuto $type]
   set tbl [chainToTbl $chain $xtbl]
   if {$hdr != "NULL"} {
      hdrCopy $hdr $tbl.hdr
   }   
   if {[catch "eval fitsWrite $tbl $file -binary $opts"]} then {
	eval fitsWrite $tbl $file -binary -standard FALSE $opts
	}
   tblColDel $tbl
   transTableDel $xtbl
   }

#################################################################
#transTableAuto routines
#Create and fill a translation table automatically, given a type name

#First, a private function

proc transTableAdd {handle arg elemName} {
   set narg [llength $arg]
   if {$narg > 1} {regsub -all " " $arg "_" fitsName} else {set fitsName $arg}
   set fitsName [string toupper $fitsName]
   set schemaName [lindex $arg 0]
   loop i 1 $narg {
	append schemaName .[lindex $arg $i]
	}
   transEntryAdd $handle $fitsName $schemaName
   return
   }

#######################################################################
#And now, the main routine
#direction = read for going from table to schema
#	     write for going from schema to table
#argList is null for translating the entire definition; otherwise it is
#a tcl list of all arguments to be included in the table

proc transTableAuto {type {argList ""}} {
   set schList [schemaRecurseGetFromType $type]
   set handle [transTableNew $type]
   while {1} {
	if {$schList == ""} break
	set elemList [lvarpop schList]
	set elemName [lindex $elemList 0]

#Compress elemName:
        set narg [llength $elemName]
        set arg [lindex $elemName 0]
        set hdrArg $arg
        for {set i 1} {$i < $narg} {incr i} {
           set arg $arg.[lindex $elemName $i]
           set hdrArg [lindex $elemName $i]
           }

#For arrays, we need a compressed handle name with last
#arg not present
        set handleArg $handle
        for {set i 0} {$i < [expr $narg-1]} {incr i} {
           set handleArg $handleArg.[lindex $elemName $i]
           }

	set elemType [lindex $elemList 1]
	set elemIdir [lindex $elemList 2]
	set elemDim  [lindex $elemList 3]
	if {$elemDim != "NULL"} then {
	   set ndim [llength $elemDim]
	} else {
	   set ndim 0
	   }
	if {$argList != ""} then {
	   if {[lsearch $argList $arg] == -1} then continue
	   }
	if {$elemType == "TYPE"} then continue
	if {$elemType == "char" && $elemIdir == 1} then {
	   transTableAdd $handle $arg $elemName
	   continue
	   }
	if {$elemIdir != 0} then continue
	if {[schemaKindGetFromType $elemType] == "PRIM"} then {
	   transTableAdd $handle $arg $elemName
	   continue
	   }
	}
   return $handle
   }

###############################################################################
ftclHelpDefine contrib fits2Chain \
"Usage: fits2Chain <file> <TYPE>
\t Read a FITS file and convert to a chain of the specified TYPE"
return

