#Replace glob, which is broken on linux 2.4.17 going vias nfs to xfs volumes.
#Also, implement a "find" command as a backup

proc glob {args} {
   if {[lindex $args 0] == "-nocomplain"} {
	set args [lrange $args 1 end]
	set err 0
   } else {
	set err 1
	}
   set outList ""
   foreach pattern $args {
	if {[catch {glob2 $pattern} msg]} {
	   if {$err} {
		error $msg
		}
	   return
	   }
	set outList [string trim [join "$outList $msg"]]
	}
   return $outList
   }

#######################################################################
#Create a directory string and the next thing down.  If the next thing down
#has a *, feed it to glob1 for parsing.  Return a list of things for the
#next level of parsing
proc glob2 {pattern} {
   set list [split $pattern /]
   if {[llength $list] == 1} {
	return [glob1 . $list]
	}
   set item [lindex $list 0]
   if {[string index $item 0] == "/" || [llength $item] == 0} {
	set dirList [lvarpop list]
	if {$dirList == ""} {set dirList /}
   } else {
	set dirList .
	}
   while {1} {
	set item [lvarpop list]
	set outlist ""
	foreach dir $dirList {
	   if {![file exists $dir]} continue
	   if {![file isdirectory $dir]} continue
	   set outlist [string trim [join "$outlist [glob1 $dir $item]"]]
	   }
	set dirList $outlist
	if {[llength $list] == 0} break
	}
   return $dirList
   }

##########################################################
#Search a directory for a given pattern.
proc glob1 {dir pat} {
#Convert pattern to a regexp.  If none, just return the extended path.
   if {![regsub -all {\*} $pat {.*} pat]} {
	set outlist $dir/$pat
	regsub -all // $outlist / outlist
	regsub -all {\./} $outlist "" outlist
	if {[file exists $outlist]} {
	   return $outlist
	   }
	return
	}
   set list [join [exec ls $dir] \n]
   set outlist ""
   foreach item $list {
	if {[regexp ^$pat\$ $item]} {
	   lappend outlist $dir/$item
	   }
	}
   regsub -all // $outlist / outlist
   regsub -all {\./} $outlist "" outlist
   return $outlist
   }

#####################################################################
#Find files in current directory

proc find {args} {
   set flag [lindex $args 0]
   if {$flag == "-nocomplain"} {
	set args [lrange $args 1 end]
	set complain 0
   } else {
	set complain 1
	}

#Demand that we run find on only one pattern at a time
   if {[llength $args] != 1} {
	error "Invalid pattern to find: $args"
	}

   if {[llength [split $args /]] > 1} {
	set dir [file dirname $args]
   } else {
	set dir ""
	}

#Try to cd to a directory first; then we can just run a plain ls.
   if {$dir != ""} {
	set here [pwd]
	cd $dir
	}
#   set files [eval exec ls $dir]
   set files [eval exec ls]
   if {$dir != ""} {
	cd $here
	}
   set list ""
   foreach file $files {
	foreach pattern $args {
	   set tail [file tail $pattern]
	   if {[string match $tail $file]} {
		if {$dir != ""} {
		   set file $dir/$file
		   }
		lappend list $file
		continue
		}
	   }
	}
   if {[llength $list] == 0 && $complain == 1} {
	error "No files found for pattern $args"
	}
   return $list
   }


