set dir [file dirname [info script]]
#echo Sourcing glob.tcl
#source $dir/glob.tcl

#Hacked from imgStatus.tcl.  Crawl through opdb, get all run/reruns,
#Fetch field limits.  Creat runInfo.par file.  Should I read run.par
#to get all unprocessed runs as well?

############################################################
proc imgCrawl {} {
   global runTable runChain

#Connect to opdb if not in session yet
   if {![inSession]} {
	opdbConnect
	}

   if {![info exists runTable]} {
	echo Reading run inventory
	runInventory
	}
   set runs [lsort -integer $runTable()]

#Read file of excluded runs
   if {[file exists exclude]} {
	echo Reading exclude file
	param2Chain exclude hdr
	set exclude [keylget hdr exclude]
   } else {
	set exclude ""
	}

#Read run.par, produced by mopdb.  This file is a digestion of observing
#logs
   if {[file exists run.par]} {
	echo Reading run.par
	set runChain [param2Chain run.par hdr]

#Construct list of good runs
	set allGoodRuns ""
	chainEach hndl $runChain {
	   set run [exprGet $hndl.run]
	   set status [exprGet $hndl.quality]
	   if {$status != "good"} continue
	   if {[lsearch $exclude $run] >= 0} continue
	   lappend allGoodRuns $run
	   }
	}

   set fid [open runInfo.par w]
   echo Opened file id $fid
   puts $fid "#Status created on [fmtclock [getclock]]"
   puts $fid "opdb_version devel"
   puts $fid "typedef struct \{"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   int field0;"
   puts $fid "   int nfield;"
   puts $fid "   \} RUNINFO;"
   foreach run [lsort -integer $runs] {
	if {[lsearch $exclude $run] >= 0} {
	   echo Excluding $run
	   continue
	   }
	set camcol 3
	echo $run

#Initialize opdb stuff
	if {[catch {set frames [pipeGet frames $run $camcol -rerun  \
		$runTable($run)]}]} {
	   echo Error reading pipeGet
	   continue
	   }
	set field0 [$frames -> field0]
	set nfield [$frames -> nFields]

#If run is in runChain, reset field limits.  This is useful when
#run log specifies tighter limits than QA
	set runqual ""
	if {[info exists runChain]} {
	   set subchain [chainSearch $runChain "{run == $run}"]
	   if {[chainSize $subchain] == 1} {
		set obj [chainElementGetByPos $subchain 0]
		set frameStart [exprGet $obj.frameStart]
		set frameEnd [exprGet $obj.frameEnd]
		set runqual [exprGet $obj.quality]
		if {$field0 < $frameStart} {
		   echo Resetting starting field from $field0 to $frameStart \
			based on runChain
		   set nfield [expr $nfield - ($frameStart - $field0)]
		   set field0 $frameStart
		   }
		if {$frameEnd-$field0+1 < $nfield} {
		   echo Resetting nfield from $nfield to \
			[expr $frameEnd-$field0+1] based on runChain
		   set nfield [expr $frameEnd-$field0+1]
		   }
		handleDel $obj
		}
	   chainDel $subchain
	   }
	if {$runqual == "bad"} {
	   echo Skipping $run due to bad quality in runChain
	   continue
	   }
	if {$nfield < 2} continue
	set imgRun [$frames -> imagingRun]
	set stripe [$imgRun -> stripe]
	set stripNum [$imgRun -> strip]
	set strip [enumIntToString arStrip $stripNum]
	regsub {AR_STRIP_} $strip "" strip
	set strip [string index $strip 0]
	objDel $imgRun
	objDel $frames

#Put out a separate line for each rerun
	foreach rerun $runTable($run,rerun) {
	   puts $fid "runinfo $run $rerun $stripe $strip \
		$field0 $nfield"
	   }
	}

#Now fetch info for auxiliary runs that aren't in opdb yet.
#I should use logic in imgppd to get the list of all runs, but that
#isn't practical yet.  Actually, this is useless for documenting data
#processing.  The only thing I could do is crawl the dp directories,
#but that picks up lots of nonsense along with the good.

   if {[info exists allGoodRuns]} {
	set auxruns $allGoodRuns
	}
   close $fid

   if {[info exists runChain]} {
	genericChainDel $runChain
	}
   return
   }

########################################################################
#runInfo.par contains info on run/rerun in opdb.  run.par contains info
#on all runs ever taken, but nothing about data processing.
#Let read both and match up; plus crawl a set of data directories to see
#what is on disk.

#Avoid conflict with sdb command "runStatus"

proc dpRunStatus {} {
   global runInfo
   if {[info exists runInfo]} {unset runInfo}
   set runChain [param2Chain run.par hdr]
   set runInfoChain [param2Chain runInfo.par hdr]

#Decode runInfo chain into tcl array
   set runInfo() ""
   chainEach hndl $runInfoChain {
	foreach var "run rerun stripe strip field0 nfield" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $runInfo() $run] < 0} {
	   lappend runInfo() $run
	   set runInfo($run,rerun) ""
	   }
	lappend runInfo($run,rerun) $rerun
	set runInfo($run,field0) $field0
	set runInfo($run,nfield) $nfield
	set runInfo($run) $rerun
	set runInfo($run,stripe) $stripe
	set runInfo($run,strip) $strip
	}
   genericChainDel $runInfoChain

#Compare run.par with runInfo.  Find runs that are not yet in opdb.
#runData are runs with good data that have been processed.
#I do not count runs with bad data or that might have been for test purposes,
#even though some of those are in opdb.
   set runData ""
   chainEach hndl $runChain {
	set run [exprGet $hndl.run]
	set status [exprGet $hndl.quality]
	if {$status == "bad"} continue
	if {[lsearch $runInfo() $run] < 0} {
	   echo Run $run has good data but is not yet in opdb.

#Should I check for partial results yet?
	   continue
	   }
	lappend runData $run
	}
   set runInfo(runData) $runData

#Now go on a fishing expedition.  Try to find key files for every run, rerun
#with good data.

   set fid [open datadirs]
   set lines [split [read $fid] \n]
   close $fid
   set datadirs ""

   foreach line $lines {
	set line [string trim $line]
	if {[string index $line 0] == "#"} continue
	if {[string length $line] == 0} continue
	lappend datadirs $line
	}
   echo datadirs $datadirs
   foreach run $runData {
	echo $run

#Set up list of products and names of files to search for
	set field [expr $runInfo($run,field0) + $runInfo($run,nfield)/2]
	set prod() "objc mask atlas fieldstat nfcalib astrom"
	set prod(objc) objcs/3/fpObjc-[format %06d \
	   $run]-3-[format %04d $field].fit*
	set prod(mask) objcs/3/fpM-[format %06d \
	   $run]-g3-[format %04d $field].fit*
	set prod(atlas) objcs/3/fpAtlas-[format %06d \
	   $run]-3-[format %04d $field].fit*
	set prod(fieldstat) objcs/3/fpFieldStat-[format %06d \
	   $run]-3-[format %04d $field].fit*
	set prod(nfcalib) nfcalib/fcPCalib-[format %06d $run]-3.fit*
	set prod(astrom) astrom/asTrans-[format %06d $run].fit*
	foreach rerun $runInfo($run,rerun) {
	   foreach dir $datadirs {
		set testdir $dir/$run/$rerun
		if {![file exists $testdir]} continue
		foreach p $prod() {
		   if {[info exists runInfo($run,$rerun,$p)]} continue
		   set test [glob -nocomplain $testdir/$prod($p)]
		   if {$test != ""} {
echo $run $rerun $p $dir
			set runInfo($run,$rerun,$p) $testdir
			}
		   }
		}

#Check for 6 columns for objc, atlas, mask
	   foreach p "objc atlas, mask" {
		if {![info exists runInfo($run,$rerun,$p)]} continue
		set cols ""
		foreach camcol "1 2 3 4 5 6" {
		   set testdir $runInfo($run,$rerun,$p)/objcs/$camcol
		   if {[file exists $testdir]} {lappend cols $camcol}
		   }
		if {[llength $cols] < 6} {
		   lappend runInfo($run,$rerun,$p) $cols
		   }
		}
	   }
	}
   return
   }

#####################################################################
#Summary of above

proc dpRunStatusSummary {} {
   global runInfo
   if {![info exists runInfo]} {dpRunStatus}

   foreach run $runInfo(runData) {

#Is there at least one rerun with OK info?
	set runInfo($run,okrerun) ""
	foreach rerun $runInfo($run,rerun) {
	   set missing ""
	   foreach prod "objc mask atlas fieldstat nfcalib astrom" {
		if {![info exists runInfo($run,$rerun,$prod)]} {
		   lappend missing $prod
		   }
		}
	   if {[llength $missing] > 0} {
		echo Run $run, rerun $rerun missing $missing
	   } else {
		set runInfo($run,okrerun) $rerun
		}
	   }
	}
   foreach run $runInfo(runData) {
	if {$runInfo($run,okrerun) == ""} {
	   echo Run $run, no complete rerun
	} else {
	   echo Run $run, complete rerun $runInfo($run,okrerun)
	   }
	}
   return
   }

###################################################################
proc runInfo {run} {
   global runInfo
   if {![info exists runInfo]} {
	dpRunStatus
	}
   if {![info exists runInfo($run)]} {
	echo No info for run $run
	return
	}
   foreach rerun $runInfo($run,rerun) {
	foreach prod "astrom nfcalib objc mask fieldstat atlas" {
	   if {[info exists runInfo($run,$rerun,$prod)]} {
		echo Run $run rerun $rerun $prod $runInfo($run,$rerun,$prod)
		}
	   }
	echo
	if {$runInfo($run,okrerun) != ""} {
	   echo Run $run rerun $runInfo($run,okrerun) is complete
	   }
	}
   return
   }

####################################################################
#Create html page!
proc runDataHtml {} {
   global runInfo
   if {![info exists runInfo]} {
	dpRunStatus
	dpRunStatusSummary
	}

   set outid [open runData.html w]
   puts $outid "<html><head><title>Data Location by Run</title></head>"
   puts $outid "<body><h1 align=center>Location of SDSS Imaging Data</h1>"
   puts $outid "<table align=center border=1>"
   puts $outid "<tr>"
   loop icol 0 3 {
	puts $outid "<th>Run</th><th>Rerun</th><td>astrom</td> \
		<td>nfcalib</td> \
		<td>objc</td><td>atlas</td><td>fieldstat</td><td>mask</td>"
	}
   puts $outid "</tr>"
   set npcol [expr ([llength $runInfo(runData)]-1)/3+1] 
   loop i 0  $npcol {
	set runs ""
	set nrerun 1
	foreach icol "0 1 2" {
	   set j [expr $i + $icol*$npcol]
	   set run [lindex $runInfo(runData) $j]
	   if {$run == ""} continue
	   lappend runs $run
	   set nrerun [max $nrerun [llength $runInfo($run,rerun)]]
	   }
	loop k 0 $nrerun {
	   puts $outid "<tr>"
	   foreach run $runs {
		if {$run == ""} continue
		set rerun [lindex $runInfo($run,rerun) $k]
		if {$rerun == ""} continue
		puts $outid "<th>$run</th><th>$rerun</th>"
		foreach p "astrom nfcalib objc atlas fieldstat mask" {
		   if {[info exists runInfo($run,$rerun,$p)]} {
			set text "<a href=$runInfo($run,$rerun,$p)>$p</a>"
		   } else {
			set text "&nbsp;"
			}
		   puts $outid "<td>$text</td>"
		   }
		}
	   puts $outid "</tr>"
	   }
	}
   puts $outid "</table>"
   puts $outid "</body></html>"
   close $outid
   return
   }

