##############################################################################
#<AUTO>
#
# FILE: imgQuality.tcl
#
# <HTML>
# Routines to create a file with quality info.
#</HTML>
#</AUTO>
##############################################################################
proc imgQuality {} {
    if {![inSession]} {
	opdbConnect
	}

#Make a list of runs
   global runTable
   if {![info exists runTable]} {runInventory}

   set fid [open currentStripeQuality.par w]
   puts $fid "#Status created on [fmtclock [getclock]]"
   puts $fid "opdb_version devel"
   puts $fid "typedef struct \{"
   puts $fid "   int run;"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   double lambdaStart;"
   puts $fid "   double lambdaEnd;"
   puts $fid "   int fieldStart;"
   puts $fid "   int fieldEnd;"
   puts $fid "   char status<20>;"
   puts $fid "   \} STRIPEQUALITY;"

   foreach run [lsort -integer $runTable()] {
	set rerun $runTable($run)
	imgQualitySummary $run $rerun $fid
	}
   close $fid
   return
   }

##############################################################################
#Print summary of quality for a run
#Read quality back out of the opdb.
proc imgQualitySummary {run rerun fid} {

    # Translate the enums
    foreach enum [enumGet arFieldQuality] {

#Convert num string to a common name
	if {![regexp "AR_FQ_(.+)" $enum all name]} {
	    echo Can't decode enum $enum
	    set name $enum
	}
    	set bit($name) [enumStringToInt arFieldQuality $enum]
    }

#Read from db
    if {[inSession]} {sessionEnd}
    sessionBegin -mrow -readonly
    photoInit -readonly
    set totalQuality($run) ""
    loop camCol 3 4 {
	lappend totalQuality($run) $camCol
	set totalQuality($run,$camCol) ""
        set frames [pipeGet frames $run $camCol -rerun $rerun]
        set asCalib [$frames -> currentAstrometricCalibration]
	set startField [$frames -> field0]
	set endField [expr $startField + [$frames -> nFields] - 1]
	set nfield [$frames -> nFields]
	set imgRun [$frames -> imagingRun]
	set stripe [$imgRun ->  stripe]
	set strip [$imgRun -> strip]
	regsub {AR_STRIP_} $strip "" strip
	set strip [string index $strip 0]
	objDel $imgRun

        loop field $startField [expr $endField +1] {
	    lappend totalQuality($run,$camCol) $field
            set olist [$frames -> objectList $field]

#Munn now wants to know which quality.  0 = current, 1 = target
	    set skyVersion 0
	    set nqual [$olist -> quality $skyVersion]
	    set quality [enumIntToString arDataQuality $nqual]

	    set goodMask [$olist -> goodMask $skyVersion]
	    set acceptableMask [$olist -> acceptableMask $skyVersion]
	    set badMask [$olist -> badMask $skyVersion]
	    set holeMask [$olist -> holeMask $skyVersion]
	    regexp "AR_DQ_(.+)" $quality all quality
	    set quality [string tolower $quality]
            objDel $olist

	    if {$field == $startField} {
		set qprev $quality
		set field1 $field
	 	}

#Don't allow holes
	    if {$quality == "hole" && $holeMask == 0} {
		set quality undetermined
		}
	    if {$quality == "hole"} {
		set quality $qprev
		}
#Require that a mask not be blank
	    if {$quality == "good" && $goodMask == 0} {
		set quality undetermined
		}
	    if {$quality == "acceptable" && $acceptableMask == 0} {
		set quality undetermined
		}
	    if {$quality == "bad" && $badMask == 0} {
		set quality undetermined
		}

	    if {$field == $startField} {
		set qprev $quality
		set field1 $field
	 	}
	    if {$quality == $qprev} {
		set field2 $field
	    } else {
		set list [getLimits [list $field1 $field2] $asCalib]
		set lambdaStart [lindex $list 0]
		set lambdaEnd [lindex $list 1]
		if {$qprev != "undetermined"} {
		   puts $fid "STRIPEQUALITY $run $stripe $strip $lambdaStart \
			$lambdaEnd $field1 $field2 $qprev"
		   }
		set field1 $field
		set field2 $field
		set qprev $quality
		}
	}
	set list [getLimits [list $field1 $field2] $asCalib]
	set lambdaStart [lindex $list 0]
	set lambdaEnd [lindex $list 1]
	if {$qprev != "undetermined"} {
	   puts $fid "STRIPEQUALITY $run $stripe $strip $lambdaStart \
	   $lambdaEnd $field1 $field2 $qprev"
	   }
    }
    objDel $asCalib
    objDel $frames
    sessionEnd
    return
}
    


