set dir [file dirname [info script]]

global dataRoot
set dataRoot /data/dp30.a/data

#echo Sourcing glob.tcl
#source $dir/glob.tcl

#Create a file currentStripeStatus.par that contains all good segments.
#from a set of runs.
#This proc must run from within opdb
#Must run runInventory first, then unset undesired runs.
#
#Commands
#	imgStrategy
#		Connect to & disconnectd from opdb
#		Call imgStatus, stripeStatus, stripePlan, stripePlotGif
#	imgFetch
#		Fetch run.par from mopdb
#       imgStatus
#               Create currentStripeStatus.par
#		user is prompted for additional runs to include (data
#			is fetched from ppd directories).
#       stripeStatus <stripes> -mjdmax <max-mjd>
#               Read currentStripeStatus.par and echo which segments
#               still need to be observed.  Also fills in more imgStatus
#		info.  -mjdmax is optional.  Now writes out a file
#		todo-<mjd>.par giving all the areas left to scan.
#	statSummary
#		Print out summary of north/south areas imaged, etc.
#	imgHistory
#		After running imgStatus, this process runs stripeStatus
#		multiple
#		times and computes cumulative statistics on a quarterly
#		basis
#	runHistory
#		After running imgStatus, this process runs stripeStatus
#		multiple
#		times and computes cumulative statistics on a dark run
#		basis
#	imgHistorySummary
#		Print out history information.
#	stripePlan -mjd <mjd> (off in imgPlan.tcl) (mjd is optional)
#
#A helper proc
proc getLimits {segment asCalib} {
   set field1 [lindex $segment 0]
   set fieldn [lindex $segment end]
   set trans [$asCalib -> trans 31 $field1]
   set a1 [$trans a]
   set mjdhndl [$trans mjd]
   set mjdstart [$mjdhndl MJDTime]
   handleDel $mjdhndl
   handleDel $trans
   set trans [$asCalib -> trans 31 $fieldn]
   set an [$trans a]
   set bn [$trans b]

#Increment an to be the coordinate at row 1361
   set an [expr $an+1361.*$bn]

   handleDel $trans
   if {[catch {set lambdaStart [expr $a1-185.]}]} {
	echo Problem with $a1 field $field1
	set lambdaStart -185.
	}
   if {$lambdaStart > 180.} {
	set lambdaStart [expr $lambdaStart-360.]
	}
   if {$lambdaStart < -180.} {
	set lambdaStart [expr $lambdaStart+360.]
	}
   if {[catch {set lambdaStart [format %.4f $lambdaStart]}]} {
	echo $lambdaStart too small!
	}
   if {[catch {set lambdaEnd [expr $an-185.]}]} {
	echo Problem with $an field $fieldn
	set lambdaEnd -185.
	}
   if {$lambdaEnd > 180.} {
	set lambdaEnd [expr $lambdaEnd-360.]
	}
   if {$lambdaEnd < -180.} {
	set lambdaEnd [expr $lambdaEnd+360.]
	}
   if {[catch {set lambdaEnd [format %.4f $lambdaEnd]}]} {
	echo $lambdaEnd too small!
	}
   set mjdstart [format %.4f $mjdstart]
   return "$lambdaStart $lambdaEnd $mjdstart"
   }

###############################################################
#Another helper proc
#This is used by auxRunStatus

proc getFieldLimits {segment frameChain node incl} {
   set field1 [lindex $segment 0]
   set fieldn [lindex $segment end]

   set hndl1 [chainGetOne $frameChain frame $field1]
   if {$hndl1 == ""} {
	error "In getFieldLimits, no field $field1 in frameChain $frameChain"
	}
   set ra [exprGet $hndl1.ra]
   set dec [exprGet $hndl1.dec]
   handleDel $hndl1

#For survey stripes, the following works fine.  For non-survey stripe, however,
#survey coords don't work.  I will compute GC coords instead and subtract
#185 from mu to define an effective lambda.  This is the exact survey long
#for survey stripes, and an effective lambda otherwise.  Actually, I want
#great circle long. in the end anyway, so the new logic is more "logical".
#   set list [eqToSurvey $ra $dec]
#   set lambda [keylget list lambda]
#   set a1 [expr $lambda+185.]
   set list [eqToGC $ra $dec -node $node -inclination $incl]
   set a1 [keylget list mu]
   set hndln [chainGetOne $frameChain frame $fieldn]
   if {$hndln == ""} {
	error "In getFieldLimits, no field $fieldn in frameChain $frameChain"
	}
   set ra [exprGet $hndln.ra]
   set dec [exprGet $hndln.dec]
   handleDel $hndln
#   set list [eqToSurvey $ra $dec]
#   set lambda [keylget list lambda]
#   set an [expr $lambda+185.]
   set list [eqToGC $ra $dec -node $node -inclination $incl]
   set an [keylget list mu]

#Increment an to row 1361.
   set an [expr $an+.396*1361./3600.]

   if {[catch {set lambdaStart [expr $a1-185.]}]} {
	echo Problem with $a1 field $field1
	set lambdaStart -185.
	}
   if {$lambdaStart > 180.} {
	set lambdaStart [expr $lambdaStart-360.]
	}
   if {$lambdaStart < -180.} {
	set lambdaStart [expr $lambdaStart+360.]
	}
   if {[catch {set lambdaStart [format %.4f $lambdaStart]}]} {
	echo $lambdaStart too small!
	}
   if {[catch {set lambdaEnd [expr $an-185.]}]} {
	echo Problem with $an field $fieldn
	set lambdaEnd -185.
	}
   if {$lambdaEnd > 180.} {
	set lambdaEnd [expr $lambdaEnd-360.]
	}
   if {$lambdaEnd < -180.} {
	set lambdaEnd [expr $lambdaEnd+360.]
	}
   if {[catch {set lambdaEnd [format %.4f $lambdaEnd]}]} {
	echo $lambdaEnd too small!
	}
   return "$lambdaStart $lambdaEnd"
   }

############################################################
#A "fetching procedure".  Go off to mopdb and fetch the current list of
#good runs with starting, stopping fields.  This list is a digestion of
#the observing logs and includes some information r.e. photometricity
#or tracking that cannot be determined easily from the processed data.
#Must have "lynx" setup.
#Fail gracefully - don't wipe out existing run.par if sdsshost2 is down.

proc imgFetch {} {
   if {[file exists run.par.tmp]} {
	echo Deleting old run.par.tmp
	exec rm -f run.par.tmp
	}
   if {[catch {exec lynx -dump -width 120 \
"http://sdsshost2.apo.nmsu.edu:8020/runReportFile.tml?host=&db=mopdb" \
	> run.par.tmp}]} {
	echo Failed to fetch data.  Use existing run.par
	return
	}
   exec cp run.par.tmp run.par
   exec rm run.par.tmp
   return
   }

############################################################
#Need to break up imgStatus, etc.
proc runStatus {} {
   runInventorySave
   imgFetch
   imgQuality
   return
   }

############################################################
#Need to work around memory problems.  Break up imgStatus into pieces.
#My guess is that before long we will need to call this run by run.
proc seeingStatus {} {
   global runTable runChain
   global quality

#Connect to opdb if not in session yet
   if {![inSession]} {
	opdbConnect
	}

#   if {![info exists runTable]} {
#	echo Reading run inventory
#	runInventory
#	}

#Initialization
   global runTable 
   source runTable.tcl

#Runs contains all scans stuffed into opdb.
   set runs [lsort -integer $runTable()]

#Keep track of troublesome runs - sometimes I miss errors
   if {[file exists trouble]} {
	exec rm trouble
	}
   exec echo #Troublesome runs > trouble

#Read file of excluded runs
   if {[file exists exclude]} {
	echo Reading exclude file
	param2Chain exclude hdr
	set exclude [keylget hdr exclude]
   } else {
	set exclude ""
	}

#Read run.par, produced by mopdb.  This file is a digestion of observing
#logs
   if {[file exists run.par] && ![info exists runChain]} {
	echo Reading run.par
	set runChain [param2Chain run.par hdr]

#Construct list of good runs
#This list might include runs not in opdb, and might exclude bad runs that
#were still stuffed into opdb.
	set allGoodRuns ""
	chainEach hndl $runChain {
	   set run [exprGet $hndl.run]
	   set status [exprGet $hndl.quality]
	   if {$status != "good"} continue
	   if {[lsearch $exclude $run] >= 0} continue
	   lappend allGoodRuns $run
	   }
	}

#Read currentQualityStatus.par, if it exists.  This gives actual status
#in opdb when that has been set properly.
   if {[file exists currentStripeQuality.par]} {
	echo Reading currentStripeQuality.par
	set curChain [param2Chain currentStripeQuality.par hdr]
	}

   opdbConnect
   foreach run $runs {
	if {[lsearch $exclude $run] >= 0} {
	   echo Excluding $run
	   continue
	   }
#Always read seeing from opdb - this avoids problem with caching old
#astroline stuff
	echo Read seeing for run $run

	if {[catch {seeingReadFromOPDB $run $runTable($run)} msg]} {
	   echo Can't read seeing for run $run
	   exec echo Error in seeingReadFrom OPDB for run $run >> trouble
	   echo msg is $msg
	   set index [lindex $runTable()]
	   set runTable() [lreplace $runTable() $index $index]
	   continue
	   }
	seeingStat $run
	seeingJudge $run
	seeingJudgeDetail $run

#Make a gif every time
	seeingJudgePlotGif $run

#If curChain exists, use it to override our seeing-only quality.
	if {[info exists curChain]} {
	   set subchain [chainSearch $curChain "{run = $run}"]
	   chainEach hndl $subchain {
		foreach var "fieldStart fieldEnd status" {
		   set $var [exprGet $hndl.$var]
		   }
		echo Current quality $status in fields $fieldStart to $fieldEnd
		loop field $fieldStart [expr $fieldEnd+1] {
		   if {$status == "undetermined"} continue
		   set quality($run,$field) $status
		   }
		}
	   chainDel $subchain
	   }
	}

#Rewrite runTable because it might have been altered.
   set runTable(allGoodRuns) $allGoodRuns
   arrayWrite runTable
   arrayWrite quality
   opdbDisconnect
   return
   }

####################################################################
proc imgStatus {} {

#Explanantion of statitics:
#   gross is area scanned (from idReport files).  This is split into
#	reduced (in opdb) (Should be in golden directory)
#	unreduced (in idReport files in ppd, haven't made it to golden yet).
#   net is total area processed (throwing out ramp-up time, bad start or
#	end of a run, or other area than cannot be processed).
#   good is area that passes seeing statistics.
#   unique is area inside survey boundary with overlap
#	within a stripe removed
#   footprint is area after stripe overlap is removed.

   global stats

   set stats(northrun) 0
   set stats(northruns) ""
   set stats(northreduced) 0
   set stats(northunreduced) 0

#Gross is the sum of reduced and unreduced
   set stats(northgross) 0
   set stats(northnet) 0
   set stats(northgood) 0

   set stats(southruns) ""
   set stats(southrun) 0
   set stats(southreduced) 0
   set stats(southunreduced) 0
   set stats(southgross) 0
   set stats(southnet) 0
   set stats(southgood) 0

#South Equatorial stuff
   set stats(eqruns) ""
   set stats(eqrun) 0
   set stats(eqreduced) 0
   set stats(equnreduced) 0
   set stats(eqgross) 0
   set stats(eqnet) 0
   set stats(eqgood) 0

#Segue stuff
   set stats(segueruns) ""
   set stats(seguerun) 0
   set stats(seguereduced) 0
   set stats(segueunreduced) 0
   set stats(seguegross) 0
   set stats(seguenet) 0
   set stats(seguegood) 0

#Everything else
   set stats(otherruns) ""
   set stats(otherrun) 0
   set stats(otherreduced) 0
   set stats(otherunreduced) 0
   set stats(othergross) 0
   set stats(othernet) 0
   set stats(othergood) 0

#Initialization
   global runTable 
   source runTable.tcl
   global quality
   source quality.tcl

#Reread run.par, produced by mopdb.  This file is a digestion of observing
#logs.  Used to reset fields limits.
   global runChain
   if {[file exists run.par] && ![info exists runChain]} {
	echo Reading run.par
	set runChain [param2Chain run.par hdr]
	}

#Read file of excluded runs
   if {[file exists exclude]} {
	echo Reading exclude file
	param2Chain exclude hdr
	set exclude [keylget hdr exclude]
   } else {
	set exclude ""
	}

   set fid [open currentStripeStatus.par w]
   echo Opened file id $fid
   puts $fid "#Status created on [fmtclock [getclock]]"
   puts $fid "opdb_version devel"
   puts $fid "typedef struct \{"
   puts $fid "   int run;"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   double lambdaStart;"
   puts $fid "   double lambdaEnd;"
   puts $fid "   double mjdStart;"
   puts $fid "   char status<20>;"
   puts $fid "   \} STRIPESTATUS;"
   foreach run [lsort -integer $runTable()] {
	if {[lsearch $exclude $run] >= 0} {
	   echo Excluding $run
	   continue
	   }
	set camcol 3
	echo $run

#First, get the idReport area.  Code to increment is further down.
	global photo_runs
	if {[catch {set hndl [queryGetOne arRun $photo_runs \
		"id_ == $run"]}]} {\
	   echo Can't find arRun for $run!
	   exec echo Can't find arRun for run $run >> trouble
	   set stats($run,gross) 0.
	   set stats($run,mjd) 0
	} else {
	   set muEnd [$hndl -> muEnd]
	   set muStart [$hndl -> muStart]
	   set mjdhndl [$hndl -> mjdStart]
	   set mjd [$mjdhndl MJD]
	   handleDel $mjdhndl

#Catch occassional errors - e.g., -9999 in a field.
	   if {$muEnd == -9999 || $muStart == -9999} {
		if {[info exists runChain]} {
		   set one [chainGetOne $runChain run 1749]
		   if {$one != ""} {
			set muStart [exprGet $one.muStart]
			set muEnd [exprGet $one.muEnd]
			handleDel $one
			}
		   }
		}
	   if {$muEnd < $muStart} {set muEnd [expr $muEnd + 360.]}
	   set gross [expr ($muEnd-$muStart) * 1.25]
	   set stats($run,gross) $gross
	   set stats($run,reduced) $stats($run,gross)
	   set stats($run,unreduced) 0
	   set stats($run,mjd) $mjd
	   objDel $hndl
	   }

#Initialize opdb stuff
	if {[catch {set frames [pipeGet frames $run $camcol -rerun  $runTable($run)]}]} {
	   echo Error reading pipeGet
	   exec echo Trouble running pipeGet for run $run, camcol \
		$camcol >> trouble
	   continue
	   }
	set field0 [$frames -> field0]
	set nfield [$frames -> nFields]

#If run is in runChain, reset field limits.  This is useful when
#run log specifies tighter limits than QA
	set runqual ""
	if {[info exists runChain]} {
	   set subchain [chainSearch $runChain "{run == $run}"]
	   if {[chainSize $subchain] == 1} {
		set obj [chainElementGetByPos $subchain 0]
		set frameStart [exprGet $obj.frameStart]
		set frameEnd [exprGet $obj.frameEnd]
		set runqual [exprGet $obj.quality]
		if {$field0 < $frameStart} {
		   echo Resetting starting field from $field0 to $frameStart \
			based on runChain
		   set nfield [expr $nfield - ($frameStart - $field0)]
		   set field0 $frameStart
		   }
		if {$frameEnd-$field0+1 < $nfield} {
		   echo Resetting nfield from $nfield to \
			[expr $frameEnd-$field0+1] based on runChain
		   set nfield [expr $frameEnd-$field0+1]
		   }
		handleDel $obj
		}
	   chainDel $subchain
	   }
	if {$runqual == "bad"} {
	   echo Skipping $run due to bad quality in runChain
	   continue
	   }
	if {$nfield < 2} continue
	loop field $field0 [expr $field0+$nfield] {
	   if {![info exists quality($run,$field)]} {
		set quality($run,$field) bad
		}
	   if {$run == 1458 && $field <= 300} {
		set quality($run,$field) bad
		}
	   }
	set imgRun [$frames -> imagingRun]
	set stripe [$imgRun -> stripe]
	set stripNum [$imgRun -> strip]
	set strip [enumIntToString arStrip $stripNum]
	regsub {AR_STRIP_} $strip "" strip
	set strip [string index $strip 0]
	objDel $imgRun

#Cache some info
	set stats($run,stripe) $stripe
	set stats($run,strip) $strip
	set asCalib [$frames -> currentAstrometricCalibration]
	set status $quality($run,$field0)

#Total length of run - just use nfield.  This is area in sq. deg.
	set stats($run,net) \
	   [format %.1f [expr ($nfield * 1361 * .396/3600.)*1.25]]
	set nfgood 0
	if {$status == "good" || $status == "acceptable"} {
	   set segment $field0
	   set badsegment ""
	   set nfgood 1
	} else {
	   set segment ""
	   set badsegment $field0
	   }
	loop field [expr $field0+1] [expr $field0+$nfield] {
	   if {$quality($run,$field) == "good" || $quality($run,$field) ==  "acceptable"} {
		if {[llength $badsegment] > 0} {
		   set limits [getLimits $badsegment $asCalib]
		   puts $fid "STRIPESTATUS $run $stripe $strip $limits BAD"
echo "STRIPESTATUS $run $stripe $strip $limits BAD"
		   }
		lappend segment $field
		set badsegment ""
		incr nfgood
	   } else {
		if {[llength $segment] > 0} {
		   set limits [getLimits $segment $asCalib]
		   puts $fid "STRIPESTATUS $run $stripe $strip $limits GOOD"
echo "STRIPESTATUS $run $stripe $strip $limits GOOD"
		   }
		set segment ""
		lappend badsegment $field
		}
	   }

#Cleanup any remaining segment
	if {[llength $segment] > 0} {
	   set limits [getLimits $segment $asCalib]
	   puts $fid "STRIPESTATUS $run $stripe $strip $limits GOOD"
echo "STRIPESTATUS $run $stripe $strip $limits GOOD"
	   }
	if {[llength $badsegment] > 0} {
	   set limits [getLimits $badsegment $asCalib]
	   puts $fid "STRIPESTATUS $run $stripe $strip $limits BAD"
echo "STRIPESTATUS $run $stripe $strip $limits BAD"
	   }

	objDel $asCalib
	objDel $frames
	set stats($run,good) [expr ($nfgood * 1361.*.396/3600.)*1.25]
	if {$stripe <= 45} {
	   incr stats(northrun)
	   lappend stats(northruns) $run
	   set stats(northnet) [expr $stats(northnet) + $stats($run,net)]
	   set stats(northgross) [expr $stats(northgross) + \
		$stats($run,gross)]
	   set stats(northreduced) [expr $stats(northreduced) + \
		$stats($run,reduced)]
	   set stats(northunreduced) [expr $stats(northunreduced) + \
		$stats($run,unreduced)]
 	   set stats(northgood) [expr $stats(northgood) + \
		$stats($run,good)]
	} elseif {$stripe == 76 || $stripe == 86} {
	   incr stats(southrun)
	   lappend stats(southruns) $run
	   set stats(southnet) [expr $stats(southnet) + $stats($run,net)]
	   set stats(southgross) [expr $stats(southgross) + \
		$stats($run,gross)]
	   set stats(southreduced) [expr $stats(southreduced) + \
		$stats($run,reduced)]
	   set stats(southunreduced) [expr $stats(southunreduced) + \
		$stats($run,unreduced)]
 	   set stats(southgood) [expr $stats(southgood) + \
		$stats($run,good)]
	} elseif {$stripe == 82} {
	   incr stats(eqrun)
	   lappend stats(eqruns) $run
	   set stats(eqnet) [expr $stats(eqnet) + $stats($run,net)]
	   set stats(eqgross) [expr $stats(eqgross) + \
		$stats($run,gross)]
	   set stats(eqreduced) [expr $stats(eqreduced) + \
		$stats($run,reduced)]
	   set stats(eqreduced) [expr $stats(equnreduced) + \
		$stats($run,unreduced)]
 	   set stats(eqgood) [expr $stats(eqgood) + \
		$stats($run,good)]
	} elseif {$stripe == 72 || $stripe == 79 || $stripe == 90 || \
		$stripe > 200} {
	   incr stats(seguerun)
	   lappend stats(segueruns) $run
	   set stats(seguenet) [expr $stats(seguenet) + $stats($run,net)]
	   set stats(seguegross) [expr $stats(seguegross) + \
		$stats($run,gross)]
	   set stats(seguereduced) [expr $stats(seguereduced) + \
		$stats($run,reduced)]
	   set stats(seguereduced) [expr $stats(segueunreduced) + \
		$stats($run,unreduced)]
 	   set stats(seguegood) [expr $stats(seguegood) + \
		$stats($run,good)]
	} else {
	   incr stats(otherrun)
	   lappend stats(otherruns) $run
	   set stats(othernet) [expr $stats(othernet) + $stats($run,net)]
	   set stats(othergross) [expr $stats(othergross) + \
		$stats($run,gross)]
	   set stats(otherreduced) [expr $stats(otherreduced) + \
		$stats($run,reduced)]
	   set stats(otherreduced) [expr $stats(otherunreduced) + \
		$stats($run,unreduced)]
 	   set stats(othergood) [expr $stats(othergood) + \
		$stats($run,good)]
	   }
	}

#Now fetch info for auxiliary runs that aren't in opdb yet.
#I should use logic in imgppd to get the list of all runs, but that
#isn't practical yet.
   set allGoodRuns $runTable(allGoodRuns)
   if {[info exists allGoodRuns]} {
	set auxruns $allGoodRuns
   } elseif {[file exists extras]} {
	echo Fetching auxiliary runs from file 'extras'
	set exid [open extras]
	set auxruns ""
	while {1} {
	   set line [string trim [gets $exid]]
	   if {[eof $exid]} break
	   if {$line == ""} continue
	   if {[string index $line 0] == "#"} continue
	   foreach run $line {
		lappend auxruns $run
		}
	   }
	close $exid
   } else {
	puts stdout "Auxiliary runs (from ppd directory): " nonewline
	set auxruns [gets stdin]
	}

#We now normally fetch auxruns from run.par.  In some cases there are
#run not properly entered into opdb that will be caught here - we
#use astroline files to judge them.
   set stats(auxruns) ""
   foreach run $auxruns {
	if {[lsearch $stats(northruns) $run] < 0 && \
	   [lsearch $stats(southruns) $run] < 0 && \
           [lsearch $stats(eqruns) $run] < 0 && \
           [lsearch $stats(segueruns) $run] < 0 && \
	   [lsearch $stats(otherruns) $run] < 0} {
           lappend stats(auxruns) $run
	   auxRunStatus $run $fid
	   }
	}
   close $fid

#Print out final stats
   echo
   echo North:
   echo Number of runs = $stats(northrun)
   echo Total area scanned = $stats(northgross)
   echo "           " reduced = $stats(northreduced)
   echo "           " unreduced = $stats(northunreduced)
   echo Total area processed = $stats(northnet)
   echo Total good area processed = $stats(northgood)
   echo
   echo South:
   echo Number of runs = $stats(southrun)
   echo Total area scanned = $stats(southgross)
   echo "           " reduced = $stats(southreduced)
   echo "           " unreduced = $stats(southunreduced)
   echo Total area processed = $stats(southnet)
   echo Total good area processed = $stats(southgood)
   echo
   echo South equatorial:
   echo Number of runs = $stats(eqrun)
   echo Total area scanned = $stats(eqgross)
   echo "           " reduced = $stats(eqreduced)
   echo "           " unreduced = $stats(equnreduced)
   echo Total area processed = $stats(eqnet)
   echo Total good area processed = $stats(eqgood)
   echo
   echo Segue:
   echo Number of runs = $stats(seguerun)
   echo Total area scanned = $stats(seguegross)
   echo "           " reduced = $stats(seguereduced)
   echo "           " unreduced = $stats(segueunreduced)
   echo Total area processed = $stats(seguenet)
   echo Total good area processed = $stats(seguegood)
   echo
   echo Other:
   echo Number of runs = $stats(otherrun)
   echo Total area scanned = $stats(othergross)
   echo "           " reduced = $stats(otherreduced)
   echo "           " unreduced = $stats(otherunreduced)
   echo Total area processed = $stats(othernet)
   echo Total good area processed = $stats(othergood)
   if {[info exists runChain]} {
	genericChainDel $runChain
	}
   if {[info exists curChain]} {
	genericChainDel $curChain
	}
   opdbDisconnect
   return
   }

####################################################################
#Evaulate status of a run not in the database
proc auxRunStatus {run {fid {}}} {
   global dataRoot
   global seeing aseeing quality
   global stats
   global runChain
   if {[info exists seeing]} {unset seeing}
   if {[info exists aseeing]} {unset aseeing}
   if {![file exists seeing-$run.tcl]} {
echo astroRead $run
	if {[catch {astroRead $run}]} {
	   exec echo In auxRunStatus, problem reading astrline for run $run \
		>> trouble
	   echo Problem reading astroline files for run $run!
	   return
	   }

	astroCopy
   } else {
echo seeingRead $run
	seeingRead $run
	}

#Find idFrameLog file in PPD directory
#Look in data processing directory if not in PPD.
   set files [glob -nocomplain $dataRoot/ppd/*/idFrameLog-[format  %06d $run]-01.par]
   set frameLogFile [lindex $files 0]
   if {![file exists $frameLogFile]} {
	set files [glob -nocomplain \
$dataRoot/$run/*/logs/idFrameLog-[format  %06d $run]-01.par]
	set frameLogFile [lindex $files 0]
	}
echo Checking for file $frameLogFile
   if {[string length $frameLogFile] == ""} return

#If file is from data processing directory, this does not include MJD in
#the full path name.  I will instead look for the idReport file in this
#directory and get the MJD from that.
   set frameDir [file dirname $frameLogFile]
   set files [glob -nocomplain $frameDir/idReport*.par]
   set reportFile [lindex $files 0]
   if {[string length $reportFile] == 0} return
   if {![regexp {([0-9]+)} [file tail $reportFile] all mjd]} return
echo mjd is $mjd

#Get the stripe info
   set file $dataRoot/ppd/skent/idReport-$mjd.out
   if {[file exists $file]} {
	set repotFile $file
	}
   if {![file exists $reportFile]} {
	echo No idReport file found for mjd $mjd
	return
	}
   if {[catch {set chains [param2Chain $reportFile hdr]}]} {
	exec echo In auxRunStatus, error reading idReport for mjd $mjd \
	   for run $run >> trouble
	echo Error reading idReport file for $mjd
	return
	}
   foreach chain $chains {
	if {[chainTypeDefine $chain] == "IMAGINGRUN"} {break}
	}
   set hndl [chainGetFirst $chain run $run]
   set stripe [exprGet $hndl.stripe]
   set strip [exprGet $hndl.strip]

#Catch a mistake in logs for mjd 53500
   if {$run == 5313} {set strip S}
   set node [exprGet $hndl.node]
   set incl [exprGet $hndl.incl]

#Initialize stats if not setup yet.
   if {![info exist stats]} {
	set stats(northrun) 0
	set stats(northruns) ""
	set stats(northgross) 0
	set stats(northreduced) 0
	set stats(northunreduced) 0
	set stats(northnet) 0
	set stats(northgood) 0

	set stats(southrun) 0
	set stats(southruns) ""
	set stats(southgross) 0
	set stats(southreduced) 0
	set stats(southunreduced) 0
	set stats(southnet) 0
	set stats(southgood) 0

	set stats(eqrun) 0
	set stats(eqruns) ""
	set stats(eqgross) 0
	set stats(eqreduced) 0
	set stats(equnreduced) 0
	set stats(eqnet) 0
	set stats(eqgood) 0

	set stats(seguerun) 0
	set stats(segueruns) ""
	set stats(seguegross) 0
	set stats(seguereduced) 0
	set stats(segueunreduced) 0
	set stats(seguenet) 0
	set stats(seguegood) 0

	set stats(otherrun) 0
	set stats(otherruns) ""
	set stats(othergross) 0
	set stats(otherreduced) 0
	set stats(otherunreduced) 0
	set stats(othernet) 0
	set stats(othergood) 0
	}

   set stats($run,stripe) $stripe
   set stats($run,strip) $strip
   foreach chain $chain {
	genericChainDel $chain
	}

echo Running seeing stat
   seeingStat $run
   seeingJudge $run
   seeingJudgeDetail $run
   if {![file exists seeing-$run.gif]} {
	seeingJudgePlotGif $run
	}

#Read idFrameLog for one set of CCDs
   set file $dataRoot/ppd/skent/idFrameLog-[format %06d $run]-01.out
   if {[file exists $file]} {
	set frameLogFile $file
	}
   if {![file exists $frameLogFile]} {
	echo Can't find file $frameLogFile!!!
	return
	}
   set frameChain [param2Chain $frameLogFile hdr]
   set hndl [chainGet $frameChain 0]
   set field0 [exprGet $hndl.frame]
   set nfield [chainSize $frameChain]
   handleDel $hndl

#If run is in runChain, reset field limits.  This is useful when
#run log specifies tighter limits than QA
	set runqual ""
	if {[info exists runChain]} {
	   set subchain [chainSearch $runChain "{run == $run}"]
	   if {[chainSize $subchain] == 1} {
		set obj [chainElementGetByPos $subchain 0]
		set frameStart [exprGet $obj.frameStart]
		set frameEnd [exprGet $obj.frameEnd]
		set runqual [exprGet $obj.quality]
		if {$field0 < $frameStart} {
		   echo Resetting starting field from $field0 to $frameStart \
			based on runChain
		   set nfield [expr $nfield - ($frameStart - $field0)]
		   set field0 $frameStart
		   }
		if {$frameEnd-$field0+1 < $nfield} {
		   echo Resetting nfield from $nfield to \
			[expr $frameEnd-$field0+1] based on runChain
		   set nfield [expr $frameEnd-$field0+1]
		   }
		handleDel $obj
		}
	   chainDel $subchain
	   }

#Trim run for  rampup and rampdown
   set field0 [expr $field0 + 8]

#Recent night logs overcount by 18 or 19 fields!  I suspect they include
#phantom fields plus ramps.  Also, photo skips the last field due to lack
#of padding.  I'll be a little conservative.
#I think the idReport file doen't add on the phantom fields.

   set nfield [expr $nfield - 19]
   if {$runqual == "bad"} {
	echo Skipping run $run due to bad quality in runChain
	return
	}

#Total length of run - just use nfield.  This is area in sq. deg.
   set stats($run,mjd) $mjd
   set stats($run,net) [format %.1f [expr ($nfield * 1361 * .396/3600.)*1.25]]
   set stats($run,gross) $stats($run,net)
   set stats($run,reduced) 0
   set stats($run,unreduced) $stats($run,gross)
   if {$nfield < 2} {
	genericChainDel $frameChain
	return
	}
   set nfgood 0
   loop field $field0 [expr $field0+$nfield] {
	if {![info exists quality($run,$field)]} {
	   set quality($run,$field) bad
	   }
	}
   set status $quality($run,$field0)
   if {$status == "good" || $status == "acceptable"} {
	set segment $field0
	set badsegment ""
	set nfgood 1
   } else {
	set segment ""
	set badsegment $field0
	}
   loop field [expr $field0+1] [expr $field0+$nfield] {
	if {$quality($run,$field) == "good" || $quality($run,$field) ==  "acceptable"} {
	   if {[llength $badsegment] > 0} {
		set limits [getFieldLimits $badsegment $frameChain $node $incl]
		puts $fid "STRIPESTATUS $run $stripe $strip $limits $mjd BAD"
echo "STRIPESTATUS $run $stripe $strip $limits $mjd BAD"
		}
	   lappend segment $field
	   set badsegment ""
	   incr nfgood
	} else {
	   if {[llength $segment] > 0} {
		set limits [getFieldLimits $segment $frameChain $node $incl]
		puts $fid "STRIPESTATUS $run $stripe $strip $limits $mjd GOOD"
echo "STRIPESTATUS $run $stripe $strip $limits $mjd GOOD"
		}
	   set segment ""
	   lappend badsegment $field
	   }
       }

#Cleanup any remaining segment
   if {[llength $segment] > 0} {
	set limits [getFieldLimits $segment $frameChain $node $incl]
	puts $fid "STRIPESTATUS $run $stripe $strip $limits $mjd GOOD"
echo "STRIPESTATUS $run $stripe $strip $limits $mjd GOOD"
	}
   if {[llength $badsegment] > 0} {
	set limits [getFieldLimits $badsegment $frameChain $node $incl]
	puts $fid "STRIPESTATUS $run $stripe $strip $limits $mjd BAD"
echo "STRIPESTATUS $run $stripe $strip $limits $mjd BAD"
	}

   genericChainDel $frameChain
   set stats($run,good) [expr ($nfgood * 1361.*.396/3600.)*1.25]
   if {$stripe <= 45} {
	   incr stats(northrun)
	   lappend stats(northruns) $run
	   set stats(northgross) [expr $stats(northgross) \
		+ $stats($run,gross)]
	   set stats(northreduced) [expr $stats(northreduced) \
		+ $stats($run,reduced)]
	   set stats(northunreduced) [expr $stats(northunreduced) \
		+ $stats($run,unreduced)]
	   set stats(northnet) [expr $stats(northnet) + $stats($run,net)]
 	   set stats(northgood) [expr $stats(northgood) + \
		+ $stats($run,good)]
   } elseif {$stripe == 76 || $stripe == 86} {
	   incr stats(southrun)
	   lappend stats(southruns) $run
	   set stats(southgross) [expr $stats(southgross) \
		+ $stats($run,gross)]
	   set stats(southreduced) [expr $stats(southreduced) \
		+ $stats($run,reduced)]
	   set stats(southunreduced) [expr $stats(southunreduced) \
		+ $stats($run,unreduced)]
	   set stats(southnet) [expr $stats(southnet) + $stats($run,net)]
 	   set stats(southgood) [expr $stats(southgood) + \
		$stats($run,good)]
   } elseif {$stripe == 82} {
	   incr stats(eqrun)
	   lappend stats(eqruns) $run
	   set stats(eqgross) [expr $stats(eqgross) \
		+ $stats($run,gross)]
	   set stats(eqreduced) [expr $stats(eqreduced) \
		+ $stats($run,reduced)]
	   set stats(equnreduced) [expr $stats(equnreduced) \
		+ $stats($run,unreduced)]
	   set stats(eqnet) [expr $stats(eqnet) + $stats($run,net)]
 	   set stats(eqgood) [expr $stats(eqgood) + \
		$stats($run,good)]
   } elseif {$stripe == 72 || $stripe == 79 || $stripe == 90 || \
		$stripe > 200} {
	   incr stats(seguerun)
	   lappend stats(segueruns) $run
	   set stats(seguegross) [expr $stats(seguegross) \
		+ $stats($run,gross)]
	   set stats(seguereduced) [expr $stats(seguereduced) \
		+ $stats($run,reduced)]
	   set stats(segueunreduced) [expr $stats(segueunreduced) \
		+ $stats($run,unreduced)]
	   set stats(seguenet) [expr $stats(seguenet) + $stats($run,net)]
 	   set stats(seguegood) [expr $stats(seguegood) + \
		$stats($run,good)]
   } else {
	   incr stats(otherrun)
	   lappend stats(otherruns) $run
	   set stats(othergross) [expr $stats(othergross) \
		+ $stats($run,gross)]
	   set stats(otherreduced) [expr $stats(otherreduced) \
		+ $stats($run,reduced)]
	   set stats(otherunreduced) [expr $stats(otherunreduced) \
		+ $stats($run,unreduced)]
	   set stats(othernet) [expr $stats(othernet) + $stats($run,net)]
 	   set stats(othergood) [expr $stats(othergood) + \
		$stats($run,good)]
	}
   return
   }

####################################################################
#For Hirsch Cohen, compute total area of North survey
#Modified to get main north survey area (stripes 10 to 37).
#I want footprint, not unique

proc imgArea {} {
   set sum 0
   loop i 10 38 {
        set list [imgStripeLimits $i]
        set start [keylget list lambdaMin]
        set end [keylget list lambdaMax]
        set sum [expr $sum + 57.3*sin($end/57.3) - 57.3*sin($start/57.3)]
        }
   set sum [expr $sum*2.5]
   echo North footprint (stripes 10-37): [format %.0f $sum] sq. deg.
   set totsum $sum
   set sum 0
   foreach i "76 82 86" {
        set list [imgStripeLimits $i]
        set start [keylget list lambdaMin]
        set end [keylget list lambdaMax]
        set sum [expr $sum + $end + 360 - $start]
        }
   set sum [expr $sum*2.5]
   echo South footprint (stripes 76, 82, 86): [format %.0f $sum] sq. deg.
   set totsum [expr $totsum+$sum]
   return [format %.0f $totsum]
   }

####################################################################
#In the current photoSch, the segue stripes are not defined.  I will
#implement a more generalized routine here.

proc imgStripeLimits {instripe} {
   global SEGUE
   if {![info exists SEGUE]} {
	set segue [param2Chain segue.par hdr]
	set SEGUE() ""
	chainEach hndl $segue {
	   foreach var "stripe node incl muMin muMax" {
		set $var [exprGet $hndl.$var]
		}
	   if {[lsearch $SEGUE() $stripe] >= 0} continue
	   lappend SEGUE() $stripe
	   set SEGUE($stripe,node) $node
	   set SEGUE($stripe,incl) $incl
	   set lmin [expr $muMin-185.]
	   if {$lmin < -180.} {set lmin [expr $lmin+360.]}
	   set lmax [expr $muMax-185.]
	   if {$lmax < -180.} {set lmax [expr $lmax+360.]}
	   set SEGUE($stripe,lambdaMin) $lmin
	   set SEGUE($stripe,lambdaMax) $lmax
	   }
	genericChainDel $segue
	}
   set stripe $instripe
   if {[lsearch $SEGUE() $stripe] >= 0} {
	set list ""
	keylset list lambdaMin $SEGUE($stripe,lambdaMin)
	keylset list lambdaMax $SEGUE($stripe,lambdaMax)
	keylset list eta 0.
   } else {
	set list [stripeLimits $stripe]
	}
   return $list
   }

####################################################################
#Compute status of a set of stripes.  Read currentStripeStatus, block
#out sections of a stripe that are done, figure out what is left to
#do.  At some point I will want to expand this to output sections that
#have been scanned once but are bad and might be lower priority to rescan.

proc stripeStatus {args} {
   set opts { \
        {stripeStatus "Compute remaining area to scan"} \
        {{[stripes]} STRING "" stripes "List of stripes (blank = all)"} \
	{-mjdmax INTEGER -1 mjdmax "Maximum mjd"} \
    }

   if {[shTclParseArg $args $opts stripeStatus] == 0} {
	        return 
	}

   global curStatus

   if {$mjdmax < 0} {set mjdmax [expr [getclock]/86400 + 40587]}

#Create a file with a list of all segments.  Use mjdmax for the name of the
#file.
   set outfile todo-$mjdmax.par
   echo Output file is $outfile
   set fid [open $outfile w]
   puts $fid "#Create [fmtclock [getclock]]"
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   float slongStart;"
   puts $fid "   float slongEnd;"
   puts $fid "   float muStart;"
   puts $fid "   float muEnd;"
   puts $fid "   } IMGTODO;"

   if {[info exists curStatus]} {
	genericChainDel $curStatus
	unset curStatus
	}

   if {![info exists curStatus]} {
	set curStatus [param2Chain currentStripeStatus.par hdr]
	}

   if {$stripes == ""} {
	loop i 1 46 {lappend stripes $i}
	lappend stripes 76 82 86

#Segue stripes
	set segue [param2Chain segue.par hdr]
	chainEach hndl $segue {
	   set stripe [exprGet $hndl.stripe]
	   if {[lsearch $stripes $stripe] < 0} {
		lappend stripes $stripe
		}
	   }
	genericChainDel $segue
	}
  
#For future use
   global stats
   set stats(mjdmax) $mjdmax

#Save in the stats array
   set stats(northunique) 0
   set stats(southunique) 0
   set stats(equnique) 0
   set stats(segueunique) 0
   set stats(otherunique) 0
   set stats(northfoot) 0
   set stats(southfoot) 0
   set stats(eqfoot) 0
   set stats(seguefoot) 0
   set stats(otherfoot) 0

#For JP, stats of southern equatorial stripe.  This is the good area
#in the southern equatorial stripe zone, regarless of overlap.
   set stats(southeq) 0

#Sum is area left to do
#area is total area of stripes
   set sum 0
   set area 0
   set nonoverlap 0
   foreach stripe $stripes {
	set unique 0
	set footprint 0
	foreach strip "S N" {
	   set list [imgStripeLimits $stripe]
	   set lmin [keylget list lambdaMin]
	   set lmax [keylget list lambdaMax]

#Footprint calculation wants lmin and lmax between -90 and 90
#For stripes < 100, if the limits are 360, trim back to 180.
	   if {$stripe < 100} {
		if {$lmin == -180 && $lmax == 180} {
		   if {$stripe <= 45} {
			set lmin -90
			set lmax 90
		   } else {
			set lmin 90
			set lmax -90
			}
		   }
		}
	   if {$lmin < -90} {set lmin [expr $lmin+360.]}
	   if {$lmax < -90} {set lmax [expr $lmax+360.]}
	   set lmin [expr round($lmin)]
	   set lmax [expr round($lmax)]

#This condition can be met for segue stripes:
	   if {$lmax < $lmin} {set lmax [expr $lmax+360.]}

#Cache survey limits for southern equatorial stripe calc.
	   set lambdaMin $lmin
	   set lambdaMax $lmax

	   set area [expr $area + $lmax - $lmin]

#Total survey stripe length
	   set stripeLen [expr $lmax - $lmin]

#Effective area of footprint.  Ah, this makes sense only if lmin, lmax
#are in range -90 to +90.
	   set footLen [expr 57.3*(sin($lmax/57.3) - sin($lmin/57.3))]
	   if {$footLen < 0} {set footLen [expr -1.*$footLen]}
	   echo
	   echo Stripe limits $lmin $lmax
	   set toDo [list [list $lmin $lmax]]
	   chainEach hndl $curStatus {
		foreach var "run stripe strip lambdaStart lambdaEnd \
			mjdStart status" {
		   set c$var [exprGet $hndl.$var]
		   }
		if {$stripe != $cstripe || $strip != $cstrip} continue
		if {$cmjdStart > $mjdmax} continue
		if {$clambdaStart < -90} {
		   set clambdaStart [expr $clambdaStart+360.]
		   }
		if {$clambdaEnd < -90} {
		   set clambdaEnd [expr $clambdaEnd+360.]
		   }
		if {$clambdaEnd < $clambdaStart} {
		   set clambdaEnd [expr $clambdaEnd + 360.]
		   }

#I might need to normalize clmin, clmax to fall within the stripe limits
		if {$clambdaEnd < $lambdaMin} {
		   set clambdaStart [expr $clambdaStart + 360.]
		   set clambdaEnd [expr $clambdaEnd + 360.]
		   }

		if {$clambdaStart > $lambdaMax} {
		   set clambdaStart [expr $clambdaStart - 360.]
		   set clambdaEnd [expr $clambdaEnd - 360.]
		   }

		set nonoverlap [expr $nonoverlap + $clambdaEnd-$clambdaStart]
		if {![regexp GOOD $cstatus]} continue

#Check for overlaps of to-do list
		set clmin [format %.4f $clambdaStart]
		set clmax [format %.4f $clambdaEnd]
		set clmin [expr round($clmin)]
		set clmax [expr round($clmax)]

		echo Good stripe $crun $clmin $clmax

		loop i 0 [llength $toDo] {
		   set pair [lindex $toDo $i]
		   set lmin [lindex $pair 0]
		   set lmax [lindex $pair 1]

#Six possibilities
		   if {$clmin < $lmin} {
			if {$clmax <= $lmin} continue
			if {$clmax > $lmin && $clmax < $lmax} {
			   set lmin $clmax
			   set pair [list $lmin $lmax]
			   set toDo [lreplace $toDo $i $i $pair]
			   continue
			   }
			set toDo [lreplace $toDo $i $i ""]
			continue
			}
		   if {$clmin >= $lmin && $clmin < $lmax} {
			if {$clmax < $lmax} {
			   set pair1 [list $lmin $clmin]
			   set toDo [lreplace $toDo $i $i $pair1]
			   set pair2 [list $clmax $lmax]
			   set todo [lappend toDo $pair2]
			   continue
			   }
			set pair [list $lmin $clmin]
			set toDo [lreplace $toDo $i $i $pair]
			continue
			}
		   }
		while {1} {
		   set index [lsearch $toDo {}]
		   if {$index < 0} break
		   set toDo [lreplace $toDo $index $index]
		   }
#Southern equatorial stripe
		if {$stripe == 82} {

echo clambdaStart $clambdaStart, clambdaEnd $clambdaEnd
#Adjust strip limits to fall within survey boundary.
		   if {$clambdaStart < $lambdaMin} {
			set clambdaStart $lambdaMin
			}
		   if {$clambdaStart > $lambdaMax} {
			set clambdaEnd $lambdaMax
			}
		   if {$clambdaStart > $clambdaEnd} continue

#I actually compute multiplicity (sum of total length / 2*stripeLen).
#The factor 2 is to correct for strip vs stripe.
		   set stats(southeq) [expr $stats(southeq) + \
			.5*($clambdaEnd - $clambdaStart) / $stripeLen]
		   }
		}
	   loop i 0 [llength $toDo] {
		set pair [lindex $toDo $i]
		set lmin [lindex $pair 0]
		set lmax [lindex $pair 1]
		if {$lmin > 180} {set lmin [expr $lmin-360]}
		if {$lmax > 180} {set lmax [expr $lmax-360]}
		set pair [list $lmin $lmax]
		set toDo [lreplace $toDo $i $i $pair]
		}
	   foreach pair $toDo {
		echo Stripe $stripe$strip Must do $pair
		set start [lindex $pair 0]
		set end [lindex $pair 1]

#Include mu coords, which helps for segue.
		set muStart [expr $start + 185.]
		if {$muStart > 360.} {set muStart [expr $muStart - 360.]}
		set muEnd [expr $end + 185.]
		if {$muEnd > 360.} {set muEnd [expr $muEnd - 360.]}
		puts $fid "IMGTODO $stripe $strip $start $end $muStart \
		   $muEnd"

#Wrap end to be bigger than start
		if {$end < $start} {set end [expr $end+360.]}
		set sum [expr $sum + $end - $start]
		set unique [expr $unique + $end - $start]
		set diff [expr (57.3) * \
		   (sin($end/57.3) - sin($start/57.3))]
		if {$diff < 0.} {set diff [expr -1.*$diff]}
		set footprint [expr $footprint + $diff]
		}
	   }
#Total unique area.  We have summed unobserved length along 2 strips.
#Convert to done square degrees.
	set unique [expr (2*$stripeLen - $unique) * 1.25]
	set footprint [expr (2*$footLen - $footprint) * 1.25]

#Track individual stripes - sometimes useful to have.
	set stats($stripe,unique) $unique
	if {$stripe <= 45} {
	   set stats(northunique) [expr $stats(northunique) + $unique]
	   set stats(northfoot) [expr $stats(northfoot) + $footprint]
	} elseif {$stripe == 76 || $stripe == 86} {
	   set stats(southunique) [expr $stats(southunique) + $unique]
	   set stats(southfoot) [expr $stats(southfoot) + $footprint]
	} elseif {$stripe == 82} {
	   set stats(equnique) [expr $stats(equnique) + $unique]
	   set stats(eqfoot) [expr $stats(eqfoot) + $footprint]
	} elseif {$stripe == 72 || $stripe == 79 || $stripe == 90 || \
		$stripe > 200} {
	   set stats(segueunique) [expr $stats(segueunique) + $unique]
	   set stats(seguefoot) [expr $stats(seguefoot) + $footprint]
	} else {
	   set stats(otherunique) [expr $stats(otherunique) + $unique]
	   set stats(otherfoot) [expr $stats(otherfoot) + $footprint]
	   }
	}
   close $fid
   set sum [expr $sum*2.5/2.]
   set area [expr $area*2.5/2.]
   set nonoverlap [expr $nonoverlap*2.5/2.]
   echo ""
   echo Total area is $area
   echo Still must do $sum square degrees.
   echo Area done (overlaps removed) = [expr $area - $sum]
   echo Area done (overlaps not removed) = $nonoverlap
   echo North unique area = $stats(northunique)
   echo South unique area = $stats(southunique)
   echo Equator unique area = $stats(equnique)
   echo Segue unique area = $stats(segueunique)
   echo Other unique area = $stats(otherunique)
   return
   }

####################################################################
#Compute status of data processing.

proc dpStatus {args} {
   set opts { \
        {dpStatus "Compute data processing stats"} \
        {{[stripes]} STRING "" stripes "List of stripes (blank = all)"} \
	{-mjdmax INTEGER -1 mjdmax "Maximum mjd"} \
    }

   if {[shTclParseArg $args $opts stripeStatus] == 0} {
	        return 
	}

   global curStatus

   if {$stripes == ""} {
	loop i 1 46 {lappend stripes $i}
	lappend stripes 76 82 86

#Segue stripes
	set segue [param2Chain segue.par hdr]
	chainEach hndl $segue {
	   set stripe [exprGet $hndl.stripe]
	   if {[lsearch $stripes $stripe] < 0} {
		lappend stripes $stripe
		}
	   }
	genericChainDel $segue
	}
  
   if {$mjdmax < 0} {set mjdmax [expr [getclock]/86400 + 40587]}

#For future use
   global stats runTable
   set stats(mjdmax) $mjdmax

#Save in the stats array
   set stats(northdp) 0
   set stats(southdp) 0
   set stats(eqdp) 0
   set stats(seguedp) 0
   set stats(otherdp) 0

#Sum is area left to do
#area is total area of stripes
   foreach zone "north south eq segue other" {
	foreach run $stats(${zone}runs) {
	   if {![info exists runTable($run,rerun)]} continue
	   set stripe $stats($run,stripe)

#Just multiply gross by number of reruns.
	   set stats($run,dp) [expr $stats($run,gross) * \
		[llength $runTable($run,rerun)]]
	   set stats(${zone}dp) [expr $stats(${zone}dp) + $stats($run,dp)]
	   }
	echo ""
	echo Zone $zone, area processed $stats(${zone}dp)
	}
   return
   }

##################################################################
proc statSummary {{file ""}} {
   global stats
   if {![info exists stats]} return
   if {$file != ""} {
	set stdout [open $file w]
   } else {
	set stdout stdout
	}

#Recompute the gross, net, and good sums based upon mjdmax.  The
#unique and foot sums were computed in stripeStatus
   set mjdmax $stats(mjdmax)
   set date [fmtclock [expr ($mjdmax-40587)*86400] "%d-%h-%Y"]
   echo Summary for data as of $date
   echo
   foreach zone "north south eq segue other" {
	foreach type "gross reduced unreduced net good dp" {
	   set stats(${zone}${type}) 0
	   }
       foreach run [lsort $stats(${zone}runs)] {
	   if {$stats($run,mjd) > $mjdmax} continue
	   foreach type "gross reduced unreduced net good dp" {
		if {$type == "dp" && ![info exists stats($run,dp)]} continue
		set stats(${zone}${type}) [expr \
		   [set stats(${zone}${type})] + $stats($run,$type)]
		}
	   }
	}

#South footprint = southUnique
   set stats(southfoot) $stats(southunique)
   set stats(eqfoot) $stats(equnique)
   set stats(seguefoot) $stats(segueunique)

   puts $stdout [format "%-8s %8s %8s %8s %8s %8s %8s" \
	Zone Gross Net Good Unique Footprint Reprocessings]
   puts $stdout ""
   puts $stdout [format "%-8s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f" North \
	$stats(northgross) \
	$stats(northnet) $stats(northgood) $stats(northunique) \
	$stats(northfoot) $stats(northdp)]
   puts $stdout [format "%-8s %8.0f" Reduced $stats(northreduced)]
   puts $stdout [format "%-8s %8.0f" Unreduc. $stats(northunreduced)]
   puts $stdout ""
   puts $stdout [format "%-8s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f" South \
	$stats(southgross) \
	$stats(southnet) $stats(southgood) $stats(southunique) \
	$stats(southfoot) $stats(southdp)]
   puts $stdout [format "%-8s %8.0f" Reduced $stats(southreduced)]
   puts $stdout [format "%-8s %8.0f" Unreduc. $stats(southunreduced)]
   puts $stdout ""
   puts $stdout [format "%-8s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f" Equator \
	$stats(eqgross) \
	$stats(eqnet) $stats(eqgood) $stats(equnique) \
	$stats(eqfoot) $stats(eqdp)]
   puts $stdout [format "%-8s %8.0f" Reduced $stats(eqreduced)]
   puts $stdout [format "%-8s %8.0f" Unreduc. $stats(equnreduced)]
   puts $stdout ""
   puts $stdout [format "%-8s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f" Segue \
	$stats(seguegross) \
	$stats(seguenet) $stats(seguegood) $stats(segueunique) \
	$stats(seguefoot) $stats(seguedp)]
   puts $stdout [format "%-8s %8.0f" Reduced $stats(seguereduced)]
   puts $stdout [format "%-8s %8.0f" Unreduc. $stats(segueunreduced)]
   puts $stdout ""
   puts $stdout [format "%-8s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f" Other \
	$stats(othergross) \
	$stats(othernet) $stats(othergood) $stats(otherunique) \
	$stats(otherfoot) $stats(otherdp)]
   puts $stdout [format "%-8s %8.0f" Reduced $stats(otherreduced)]
   puts $stdout [format "%-8s %8.0f" Unreduc. $stats(otherunreduced)]
   puts $stdout ""
   puts $stdout "South equatorial multiplicity = [format %.2f $stats(southeq)]"

#I really need to document the date and runs in this
   puts $stdout ""
   puts $stdout "Runs: [lsort -integer [join "$stats(northruns) \
	   $stats(southruns) $stats(eqruns) $stats(segueruns) \
	   $stats(otherruns)"]]"
   if {$stdout != "stdout"} {
	close $stdout
	}
   return
   }

##################################################################
#Print out details (and not sums)
proc statDetail {} {
   global stats
   if {![info exists stats]} return

   foreach zone "north south eq segue other" {
	puts stdout ""
	puts stdout [format "%-8s %8s %8s %8s" [string toupper $zone] \
	   Gross Net Good]
	foreach run [lsort -integer $stats(${zone}runs)] {
	   puts stdout [format "%-8s %8.0f %8.0f %8.0f" $run \
		$stats($run,gross) \
		$stats($run,net) $stats($run,good)]
	   }
	}

   return
   }

#######################################################################
#Get history of imaging by dark run.

proc runHistory {} {
   global imgHistory stats
   set imgHistory() ""
   set now [expr [getclock]/86400 + 40587]
   set chains [param2Chain moon.par hdr]
   foreach chain $chains {
	if {[chainTypeDefine $chain] != "FULL"} {
	   genericChainDel $chain
	} else {
echo Full moon
	   set full $chain
	   }
	}
   if {![info exists full]} {
	echo No full moon chain - gak!
	return
	}
   chainEach hndl $full {
	set date [lindex [exprGet $hndl.date] 0]
	set year [lindex $date 0]
	set mon [join [lrange $date 1 2] -]
	set mjd [exprGet $hndl.mjd]

#Sometimes I like to see how we are doing in the current run.
	if {$mjd > $now+30} continue
	if {[lsearch $imgHistory() $year] < 0} {
	   lappend imgHistory() $year
	   set imgHistory($year) ""
	   }
	lappend imgHistory($year) $mon
	set imgHistory($year,$mon,$mjd) $mjd
	stripeStatus -mjdmax $mjd
	dpStatus -mjdmax $mjd
	statSummary
	foreach zone "north south eq segue other" {
		foreach type "gross reduced unreduced net good unique foot \
			dp" {
		   set imgHistory($year,$mon,$zone,$type) \
			$stats(${zone}${type})
		   }
	   }
	}
   genericChainDel $full
   return
   }

#######################################################################
#Get history of imaging.

proc imgHistory {} {
   global imgHistory stats
   set imgHistory() ""
   set now [expr [getclock]/86400 + 40587]
   foreach year "1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008" {
#	if {$year > [fmtclock [getclock] %Y]} break
	lappend imgHistory() $year
	set imgHistory($year) ""
	foreach mon "Jan Apr Jul Oct" {
	   set mjd [expr [convertclock "01 $mon $year"]/86400 + 40587]

#Sometimes I like to see how we are doing in the current quarter.
	   if {$mjd > $now+90} break
	   lappend imgHistory($year) $mon
	   stripeStatus -mjdmax $mjd
	   dpStatus -mjdmax $mjd
	   statSummary
	   foreach zone "north south eq segue other" {
		foreach type "gross reduced unreduced net good unique foot \
			dp" {
		   set imgHistory($year,$mon,$zone,$type) \
			$stats(${zone}${type})
		   }
		}
	   }
	}
   return
   }

###############################################################
proc imgHistorySummary {{file ""}} {
   global imgHistory stats
   if {$file != ""} {
	set stdout [open $file w]
   } else {
	set stdout stdout
	}
   foreach zone "north south eq segue" {
	puts $stdout ""
	puts $stdout [format "%-8s    %8s %8s \
		%8s %8s %8s %8s" [string toupper $zone] \
		Gross Net Good Unique Foot Reprocessed]
	foreach year $imgHistory() {
	   foreach mon $imgHistory($year) {
		puts $stdout [format "%4s-%3s %8.0f %8.0f \
		   %8.0f %8.0f %8.0f %8.0f" $year $mon \
		   $imgHistory($year,$mon,$zone,gross) \
		   $imgHistory($year,$mon,$zone,net) \
		   $imgHistory($year,$mon,$zone,good) \
		   $imgHistory($year,$mon,$zone,unique) \
		   $imgHistory($year,$mon,$zone,foot) \
		   $imgHistory($year,$mon,$zone,dp)]
		}
	   }
	}

#Create imgStatus.par with summed numbers for web pages
   set mjdnow [expr int([getclock]/86400+40587)]
   foreach year $imgHistory() {
	foreach mon $imgHistory($year) {
	   set fid [open imgStatus.par w]
           puts $fid "date [fmtclock [getclock] "%Y %h %d"]"
           puts $fid "mjd $mjdnow"
	   foreach type "gross net good unique foot dp" {
		   set $type [expr $imgHistory($year,$mon,north,$type) + \
		   $imgHistory($year,$mon,south,$type) + \
		   $imgHistory($year,$mon,eq,$type)]
		   puts $fid "$type [format %.0f [set $type]]"
		   }
#Baseline is 5 yr plan for unique observing of north, south, and eq areas.
#Taken from Kron's plan and text.
	   set baseline [expr 7707 + 475 + 270]
	   puts $fid "baseline $baseline"
	   close $fid
	   }
	}
#I really need to document the date and runs in this
   puts $stdout ""
   puts $stdout "Runs: [lsort -integer [join "$stats(northruns) \
	   $stats(southruns) $stats(eqruns) $stats(segueruns) \
	   $stats(otherruns)"]]"
   if {$stdout != "stdout"} {
	close $stdout
	}
   return
   }

###############################################################
#Create an html file with history of imaging
proc imgHistoryHtml {} {
   global imgHistory stats
   set stdout [open imgHistory.html w]
   puts $stdout "<head>"
   puts $stdout "<title>Imaging History</title>"
   puts $stdout "</head>"
   puts $stdout ""
   puts $stdout "<body>"
   puts $stdout "<h1 align=center>Imaging History</h1>"
   puts $stdout "<h3 align=center>[fmtclock [getclock] %Y-%h-%d]</h3>"
   foreach zone "north south eq segue other" {
	puts $stdout "<p><table border=1 align=center>"
	puts $stdout "<tr><th colspan=7><b>[string toupper $zone] \
		</b></th></tr>"
	puts $stdout "<tr><th>Cumulative Date</th> \
		<th>Gross</th><th>Net</th><th>Good</th> \
		<th>Unique</th> \
		<th>Footprint</th><th>Reprocessed</th></tr>"
	foreach year $imgHistory() {
	   foreach mon $imgHistory($year) {
		puts $stdout "<tr><td>01-$mon-$year</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,gross)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,net)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,good)]</td> \
		   <td>[format %.0f \
			$imgHistory($year,$mon,$zone,unique)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,foot)]</td> \
		   <td>[format %.0f \
			$imgHistory($year,$mon,$zone,dp)]</td></tr>"
		}
	   }
	puts $stdout "</table>"
	}

#I really need to document the date and runs in this
   puts $stdout ""
   puts $stdout "<p>Runs: [lsort -integer [join "$stats(northruns) \
	   $stats(southruns) $stats(eqruns) $stats(segueruns) \
	   $stats(otherruns)"]]"
   puts $stdout "<p>Definitions (all areas in square degrees):<ul>"
   puts $stdout "<li><b>Gross</b>: Total area scanned and considered \
	   \"reducible\""
   puts $stdout "<li><b>Net</b>: Total area after trimming \
	   rampup fields, setup fields, etc."
   puts $stdout "<li><b>Good</b>: Total area meeting survey requirements \
	   on seeing, tracking, etc."
   puts $stdout "<li><b>Unique</b>: Total area with duplicate scans within
	stripes removed. This area is compared with the baseline plan."
   puts $stdout "<li><b>Footprint</b>: Total area on the sky after \
	   eliminating overlaps between stripes.  This is the total unique
	area available for science."
   puts $stdout "</body>"
   if {$stdout != "stdout"} {
	close $stdout
	}
   return
   }

###############################################################
#Create an html file with history of imaging by dark run - must have run
#runHistory first!
proc runHistoryHtml {} {
   global imgHistory stats
   set stdout [open runHistory.html w]
   puts $stdout "<head>"
   puts $stdout "<title>Imaging History by Dark Run</title>"
   puts $stdout "</head>"
   puts $stdout ""
   puts $stdout "<body>"
   puts $stdout "<h1 align=center>Imaging History</h1>"
   puts $stdout "<h3 align=center>[fmtclock [getclock] %Y-%h-%d]</h3>"
   foreach zone "north south eq segue other" {
	puts $stdout "<p><table border=1 align=center>"
	puts $stdout "<tr><th colspan=7><b>[string toupper $zone] \
		</b></th></tr>"
	puts $stdout "<tr><th>Cumulative Date</th> \
		<th>Gross</th><th>Net</th><th>Good</th> \
		<th>Unique</th> \
		<th>Footprint</th><th>Reprocessed</th></tr>"
	foreach year $imgHistory() {
	   foreach mon $imgHistory($year) {
		puts $stdout "<tr><td>$mon-$year</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,gross)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,net)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,good)]</td> \
		   <td>[format %.0f \
			$imgHistory($year,$mon,$zone,unique)]</td> \
		   <td>[format %.0f $imgHistory($year,$mon,$zone,foot)]</td> \
		   <td>[format %.0f \
			$imgHistory($year,$mon,$zone,dp)]</td></tr>"
		}
	   }
	puts $stdout "</table>"
	}

#I really need to document the date and runs in this
   puts $stdout ""
   puts $stdout "<p>Runs: [lsort -integer [join "$stats(northruns) \
	   $stats(southruns) $stats(eqruns) $stats(segueruns) \
	   $stats(otherruns)"]]"
   puts $stdout "<p>Definitions (all areas in square degrees):<ul>"
   puts $stdout "<li><b>Gross</b>: Total area scanned and considered \
	   \"reducible\""
   puts $stdout "<li><b>Net</b>: Total area after trimming \
	   rampup fields, setup fields, etc."
   puts $stdout "<li><b>Good</b>: Total area meeting survey requirements \
	   on seeing, tracking, etc."
   puts $stdout "<li><b>Net</b>: Total area with duplicate scans within
	stripes removed. This area is compared with the baseline plan."
   puts $stdout "<li><b>Footprint</b>: Total area on the sky after \
	   eliminating overlaps between stripes.  This is the total unique
	area available for science."
   puts $stdout "</body>"
   if {$stdout != "stdout"} {
	close $stdout
	}
   return
   }

