################################################################
#Read a compressed or uncompressed file.
proc regRead {reg file} {
    if {![regexp {\.R$} $file]} {
        return [regReadAsFits $reg $file]
    }
#Run sdssdecompress and create a temp file
   set tmp /tmp/decomp-[pid].fit
   exec sdssdecompress < $file > $tmp
   set hndl [regReadAsFits $reg $tmp]
   exec rm $tmp
   return $hndl
}

###############################################################
#Read either a regular or compress region.  Use a pipe to sdssdecompress in
# case of the latter.

proc regRead {reg file {filter ""} {field ""}} {

#Regular file
   if {[regexp {.fit$} $file]} {
	return [regReadAsFits $reg $file]
	}

#Run sdssdecompress and create a temp file
   if {[regexp {\.R$} $file]} {

#Name of file after decompression.
	set tmp /tmp/decomp-[pid].fit

#Sometimes compressed files get corrupted.
	exec cp $file $tmp.R

#Pipes don't seem to work reliably with sdssdecompress, so run on
#a named file.
	if {[catch {exec sdssdecompress $tmp.R}]} {
	    catch {exec /bin/rm $tmp.R}
	   return
	   }
	set hndl [regReadAsFits $reg $tmp]
	exec /bin/rm $tmp
	return $hndl
	}

#Run sdssdecompress and create a temp file
   if {[regexp {\.gz$} $file] || [regexp {\.Z$} $file]} {

#Name of file after decompression.
	set tmp /tmp/decomp-[pid].fit

#Sometimes compressed files get corrupted.
	exec cp $file $tmp.gz

#Pipes don't seem to work reliably with sdssdecompress, so run on
#a named file.
	if {[catch {exec gunzip $tmp.gz}]} {
	    catch {exec /bin/rm $tmp.gz}
	   return
	   }
	set hndl [regReadAsFits $reg $tmp]
	exec /bin/rm $tmp
	return $hndl
	}

#Extract from a tar file
   if {[regexp {.tar$} $file]} {

#Determine as much as possible about the tar file
	set tail [file tail $file]
	if {![regexp {fpC-([0-9]+)-[0-9]-([0-9]).tar} $file all run camcol]} {
	   echo Can't decode tar file name $file
	   return
	   }
	set here [pwd]
	cd /tmp
	set outfile fpC-$run-$filter$camcol-[format %04d $field].fit.R
	if {[catch {exec tar xvf $file $outfile} msg]} {
	   cd $here
	   echo $msg
	   return
	   }

#Now decompress
	set tmp /tmp/decomp-[pid].fit
	if {[catch {exec sdssdecompress < $outfile > $tmp}]} {
	   return
	   }
	exec rm $outfile
	set hndl [regReadAsFits $reg $tmp]
	exec rm $tmp
	cd $here
	return $hndl
	}
   return
   }


###############################################################
#Drat, corrected frames are in a tar file.  Extract into auxdir

proc tarExtract {run field1 field2} {
   cdGlobals

   if {![info exists auxdir]} {
	echo Need an auxdir to unpack tar files.
	return
	}

#Extract from a tar file.  For simplicity, put all in one directory.
   set here [pwd]
   foreach camcol $camcols {
	if {[info exists auxphoto($run)]} {
	   set dir $auxphot($run)/$camcol
	} else {
	   set dir $corrdir($run)/$camcol
	   }
	set tfile [lindex [glob $dir/*.tar] 0]
	cd $auxdir
	set outfiles ""
	for {set field $field1} {$field <= $field2} {incr field} {
	   foreach filter $rgbfil {
		set outfile fpC-[format %06d $run]-$filter$camcol-[format \
		   %04d $field].fit.R
		lappend outfiles $outfile
		}
	   }

	echo Extracting [llength $outfiles] files from $tfile
	if {[catch {eval exec tar xvf $tfile $outfiles} msg]} {
	   echo $msg
	   }
	}
   cd $here
   return
   }

#################################################3

#Commands to handle opening/closing file
proc openimage {file} {
#Allow for wild-card in file name
#Globbing uses several special characters that must be escaped
   regsub -all {\-} $file {\-} file
   regsub -all {\?} $file {\?} file
   regsub -all {\[} $file {\[} file
   regsub -all {\]} $file {\]} file
   set file [glob -nocomplain -- $file]

#If more than one, take the 1st
   set file [lindex $file 0]
   if {![file exist $file]} then {error "No such file $file"}
   set handle [regNew -name $file]
   if {[catch "regRead $handle $file"]} then {
	regDel $handle
	error "Error opening $file"
	}
   return $handle
   }

##########################################################################
proc status {} {
	set list [handleListFromType REGION]
	   echo [format "%5s%1s %-20s %8s %8s %8s %8s" ID " " Name Nrow Ncol \
		Type Nsubreg]
	for {set i 0} {$i < [llength $list]} {incr i} {
	   set handle [lindex $list $i]
	   set info [regInfoGet $handle]

#regInfoGet does not return properly formatted list if name has a space in it.
#	   set name [file tail [keylget info name]]
	   set name [lindex [exprGet $handle.name] 0]
	   set nrow [keylget info nrow]
	   set ncol [keylget info ncol]
	   set type [keylget info type]
	   set nsub [keylget info nSubReg]
	   if {[keylget info isSubReg] != 0} then {set flag *} else \
		{set flag ""}
	   echo [format "%5s%1s %-20s %8d %8d %8s %8s" $handle $flag \
		$name $nrow $ncol $type $nsub]
	   }
	puts stdout "Memory in use: [memBytesInUse] bytes"
        puts stdout "Number of handles: [llength [handleList]]"
	}

proc closeimage {handle} {
	regDel $handle
	}
