#Compute performance metrics for processing data.

#mt: Run from sdssdp4
#Imaging: Run from sdssdp11

#Commands:
#   mdMetrics	(Runs mtMetricsCompute and mdPlot)  (must be on sdssdp4)
#   idMetrics	(Runs idMetricsCompute and idPlot)  (must be on sdssdp11)
#   dpMetrics   Compute total TBytes processed.
###########################################################################

global dataRoot
set dataRoot /data/dp30.a/data

global mtTopDir
set mtTopDir $dataRoot/mt/dp/apo20/run

#Temporarily:
set mtTopDir /data/dp40.c/data/mt/dp/apo20/run

####################################################################
#For the PT, my metric is quantized by mjd of observing.  I compute the
#time needed for the first processing and stuffing of an MJD of observing.

#Query mtdb.  Fetch all MJD's that have been stuffed and get stuffing dates.
proc mtMJDProc {} {
   global mtMJDProc
   global mt_kali
   if {[info exists mtMJDProc]} {unset mtMJDProc}
   if {![inSession]} mtdbConnect
   set kali [queryGet arKali $mt_kali {}]
   set mtMJDProc() ""
   while {[$kali next]} {

#Get the observing MJD
	set excalRun [$kali -> excalRun]
	set framesRun [$excalRun -> mTFramesRun]
	set log [$framesRun -> mTLog]
	set mjdobs [$log -> mjd]
	objDel $log
	objDel $framesRun
	objDel $excalRun
	set stamp [objNew -transient ooItr(arStamp)]
	$kali -> comments $stamp
	$stamp next
	set time [$stamp -> time]
	set mjdproc [$time MJDTime]
	handleDel $time
	objDel $stamp
	if {[lsearch $mtMJDProc() $mjdobs] < 0} {
	   lappend mtMJDProc() $mjdobs
	   set mtMJDProc($mjdobs) ""
	   }
	lappend mtMJDProc($mjdobs) [expr int($mjdproc)]
	}
   objDel $kali
   set mtMDJProc() [lsort -integer $mtMJDProc()]
   return
   }

####################################################################
#Fetch patches from MOPDB and make a list of all MJDs with good data.
#The idea is to find nights with unreduced data.

proc mtMJDObs {} {
   global mtTopDir
   global mtMJDObs
   if {[info exists mtMJDObs]} {unset mtMJDObs}
   set mtMJDObs() ""

#Can't get list of MJDOBS from APO - oops!  Instead, crawl a local directory
#with mtReport files copied automatically via a cron job.  This should be
#good enough.
   set files [lsort [glob -nocomplain \
	$mtTopDir/mtreports_raw/mdReport*.par]]
   foreach file $files {
	set tail [file tail $file]
	if {![regexp {mdReport-([0-9]+)\.par} $tail all mjdobs]} continue

#Look for secondary fields.
	if {[catch {set n [exec grep " Sec " $file]}]} continue
	if {[llength $n] == 0} continue
	lappend mtMJDObs() $mjdobs
	}
   set mtMDJObs() [lsort -integer $mtMJDObs()]
   return
   }

####################################################################
#Get bad mjds
proc mtMJDBad {} {
   global mtMJDBad
   if {[info exists mtMJDBad]} {unset mtMJDBad}
   if {![inSession]} mtdbConnect
   set mtMJDBad() [mjdsGet -kali]
   return
   }

###################################################################
#Merge results and compute metrics.
proc mtMetricsCompute {} {
   global mtMJDProc mtMJDObs mtMJDBad
   global mtTopDir

#Assemble the data
   if {![inSession]} {mtdbConnect}

   echo Fetching data from mtdb
   mtMJDProc

   echo Locating all mdReports with data
   mtMJDObs

   echo Locating all mjds with bad data
   mtMJDBad
   mtdbDisconnect

#First, go through all mtMJDObs and find out what is processed, what is bad,
#and what is pending

   set fid [open mdMetric.par w]
   puts $fid "#Created [fmtclock [getclock]]"
   puts $fid "typedef struct {"
   puts $fid "   int mjd;"
   puts $fid "   int delay;"
   puts $fid "   } MTMETRIC;"

   foreach mjdobs $mtMJDObs() {
	if {$mjdobs < 51564} continue
	if {[lsearch $mtMJDProc() $mjdobs] >= 0} {
	   set delay [expr [lindex $mtMJDProc($mjdobs) 0] - $mjdobs]
	   echo mjdobs = $mjdobs, delay = $delay
	   puts $fid "mtmetric $mjdobs $delay"
	} elseif {[lsearch $mtMJDBad() $mjdobs] >= 0} {
	   echo mjdobs = $mjdobs bad

#Sometimes a night is processed but not stuffed because of patch naming
#problems
	} elseif {[file exists \
		$mtTopDir/listed_by_mjd/mjd$mjdobs/exPhotom-$mjdobs.fit]} {
	   echo mjdobs = $mjdobs processed but not stuffed
	} else {
	   echo mjdobs = $mjdobs pending
	   puts $fid "mtmetric $mjdobs -9999"
	   }
	}
   close $fid

#Sanity checks
   foreach mjd $mtMJDProc() {
	if {$mjd < 51564} continue
	if {[lsearch $mtMJDObs() $mjd] < 0} {

#Sometimes a night is processed but there were no secondaries
	   if {[file exists $mtTopDir/mtreports_raw/mdReport-$mjd.par]} \
		continue
	   echo mjd $mjd was processed but has no mdReport!
	   }
	}

#This test is no good because I don't report mdReports with no secondaries.
   foreach mjd $mtMJDBad() {
	if {$mjd < 51564} continue
	if {[lsearch $mtMJDObs() $mjd] < 0} {
#	   echo mjd $mjd is bad but has no mdReport!
	   }
	}
   return
   }

#######################################################################
#Make plot of mt metrics
proc mdPlot {device} {
   if {![file exists mdMetric.par]} {
	echo Please run mdMetric first!
	return
	}
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin mdMetrics$device
	pgAsk 0
	}
   set chain [param2Chain mdMetric.par hdr]
   set mindelay -10
   set maxdelay 200
   set minmjd [expr [convertclock "Jan 01, 2000"]/86400 + 40587]
   set maxmjd 0
   chainEach hndl $chain {
	set maxmjd [max $maxmjd [exprGet $hndl.mjd]]
	}
   pgSci 1
   pgSch 1
   pgSlw 2
   set dev [string toupper [lindex [split $device /] 1]]
   if {$dev == "GIF"} {pgSlw 1}

#Dates: Convert to year
   set minyr 2000
   set maxyr [expr $minyr + 1.05*($maxmjd - $minmjd)/365.25]
   pgScr 0 1 1 1
   pgScr 1 0 0 0
   pgEnv $minyr $maxyr $mindelay $maxdelay 0 0
   pgLine "$minyr $maxyr" "0 0"
   set now [fmtclock [getclock] "%Y-%h-%d"]
   pgLabel "Year" "Processing Delay" "PT DATA PROCESSING METRICS: $now"
   set xlist ""
   set ylist ""
   chainEach hndl $chain {
	set mjd [exprGet $hndl.mjd]
	set mjd [expr ($mjd-$minmjd)/365.25 + 2000]
	set delay [exprGet $hndl.delay]
	if {$delay > $maxdelay} {
		set delay $maxdelay
		}
	if {$delay < $mindelay} {
	   set delay $mindelay
	   }
	lappend xlist $mjd
	lappend ylist $delay
	}
   pgSci 2
   loop i 0 [llength $xlist] {
	pgPoint [lindex $xlist $i] [lindex $ylist $i] 3
	}
   pgText 2000 210 "* Delay"
   genericChainDel $chain
   if {[pgQinf DEVICE] != ""} {pgEnd}
   return
   }

###########################################################################
proc mdMetrics {} {
   mtMetricsCompute
   mdPlot .ps/CPS
   mdPlot .gif/GIF
   return
   }

###########################################################################
#Get metrics for imaging.  Must run this one on sdssdp25 with photoSch
#I also need fpostgres to query the sdssdp database.

#All depended on opdb now eliminated!

proc idMetricsCompute {} {
   global runTable 
   global dataRoot

   set mname(01) Jan
   set mname(02) Feb
   set mname(03) Mar
   set mname(04) Apr
   set mname(05) May
   set mname(06) Jun
   set mname(07) Jul
   set mname(08) Aug
   set mname(09) Sep
   set mname(10) Oct
   set mname(11) Nov
   set mname(12) Dec

#First, get run info from opdb.
#   runTable(run,mjd) is the observing date,
#   runTable(run,rerun) is a list of all reruns,
#   runTable(run,rerun,mjdProc) is the frames processing date.

#For here, don't refetch if runTable exists

   if {![info exists runTable]} {
	if {[file exists runTable.tcl]} {
	   source runTable.tcl
	} else {
	   echo Running runInventory
	   runInventory
	   }
	}
#   if {![inSession]} opdbConnect
#   runMJD  This info now comes from run.par

#Read run.par and add any new runs
   echo Reading run.par
   set chain [param2Chain run.par hdr]

#Keep a list of official runs
   set runparlist ""
   chainEach hndl $chain {
	set run [exprGet $hndl.run]
	lappend runparlist $run
	set quality [exprGet $hndl.quality]

#Fetch mjd from run.par file
	set runTable($run,mjd) [exprGet $hndl.mjd]
	set runTable($run,quality) $quality

	if {[lsearch $runTable() $run] >= 0} continue
	if {$quality == "bad"} continue
	echo Extra run $run
	lappend runTable() $run
	set runTable($run,strip) [exprGet $hndl.strip]
	set runTable($run,stripe) [exprGet $hndl.stripe]
	set runTable($run,rerun) ""
	set runTable($run,photo) ""
	set runTable($run,field0) [exprGet $hndl.frameStart]

#Actually, fieldn is the last field, not the number of fields.
 	set runTable($run,fieldn) [exprGet $hndl.frameEnd]
	}
   chainDel $chain

#Run a separate job to get info out of sdssdp database and into a file.  I
#cannot run internally very easily.
   echo Fetching info from postgres
   exec idproc

#Read the file
   set chain [param2Chain idproc.par hdr]
   chainEach hndl $chain {
	foreach var "run rerun version startMJD endMJD" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $runTable() $run] < 0} continue

#Unfortunately, many of the early reruns have now
#been deleted from opdb!  This messes up my accounting.
#If there is a rerun in postgres that is not in opdb, I will check to see if
#it is a valid rerun.  If so, I will add all info to runTable, fetching
#timestamps from log files if necessary.
	if {[lsearch $runTable($run,rerun) $rerun] < 0} {

	   set runRoot $dataRoot/$run/$rerun
	   if {![file exists $runRoot]} continue
	   if {$startMJD < 0} continue

#I need to recreate the following time stamps:  mjdProc, mjdCalib
#endMJD is the stuff date.  Some of the southern runs were never stuffed
#to save on opdb space, so don't want to skip such reruns.  I will look for
#timestamps in other files.

	   set file1 $runRoot/dplogs/framesEnder.$run.$rerun.log
	   if {![file exists $file1]} continue
	   set file2 $runRoot/nfcalib/fcPCalib-[format %06d $run]-1.fit
	   if {![file exists $file2]} continue

#Data processing ran, so let's get timestamps.
	   if {[catch {set line [exec grep "Results reported at" \
		$file1]}]} continue
	   if {![regexp {Results reported at (.+)} $line all date]} continue
	   set mon [lindex $date 1]
	   set day [lindex $date 2]
	   set yr [lindex $date 4]
	   set tstamp [convertclock "$mon $day, $yr"]
	   set mjdProc [expr $tstamp/86400+40587]
	   set hdr [hdrNew]
	   hdrReadAsFits $hdr $file2
	   set tstamp [hdrGetAsAscii $hdr FCALI_ID]
	   hdrDel $hdr

#Decode FITS format timestamp
	   if {![regexp {([0-9]+)-([0-9]+)-([0-9]+)} $tstamp all yr mon day]} \
		continue
	   if {[catch {set tstamp [convertclock "$day $mname($mon) $yr"]}]} \
		continue
	   set mjdCalib [expr $tstamp/86400+40587]
	   lappend runTable($run,rerun) $rerun
	   set runTable($run,$rerun,mjdProc) $mjdProc
	   set runTable($run,$rerun,mjdCalib) $mjdCalib
	   if {$endMJD < 0} {set endMJD $mjdCalib}
	   }
	set runTable($run,$rerun,version) $version
	set runTable($run,$rerun,startMJD) $startMJD
	set runTable($run,$rerun,endMJD) $endMJD
	}
   genericChainDel $chain

#Fill in startMJD from opdb if not present in postgres
   set fid [open idMetric.par w]
   puts $fid "#Created [fmtclock [getclock]]"
   puts $fid "#Structure to hold summary information"
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int mjd;"
   puts $fid "   int field0;"
   puts $fid "   int fieldn;"
   puts $fid "   int spool;"
   puts $fid "   int stuff;"
   puts $fid "   int calib;"
   puts $fid "   int nrerun;"
   puts $fid "   } IDMETRIC;"
   puts $fid ""
   puts $fid "#Structure to hold detailed information for each rerun"
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int mjd;"
   puts $fid "   int rerun;"
   puts $fid "   int field0;"
   puts $fid "   int fieldn;"
   puts $fid "   int spoolMJD;"
   puts $fid "   int stuffMJD;"
   puts $fid "   int calibMJD;"
   puts $fid "   char version<12>;"
   puts $fid "   } IDMETRICDETAIL;"
   puts $fid ""
   foreach run [lsort -integer $runTable()] {

#If not on official list, skip.  Such runs include the gamma ray burst runs.
#Actually, I really should have them in run.par anyway.
	if {[lsearch $runparlist $run] < 0} {
	   echo Run $run not in run.par, skipping
	   continue
	   }

#If quality is bad, skip
	if {[info exists runTable($run,quality)]} {
	   if {$runTable($run,quality) == "bad"} continue
	   }
	foreach rerun $runTable($run,rerun) {
	   if {![info exists runTable($run,$rerun,startMJD)]} {
		set runTable($run,$rerun,startMJD) \
		   $runTable($run,$rerun,mjdProc)
		set runTable($run,$rerun,endMJD) \
		   $runTable($run,$rerun,mjdProc)
		set runTable($run,$rerun,version) Unknown
		}
	   }

#Use the earliest rerun.
	set rerun [lindex [lsort -integer $runTable($run,rerun)] 0]
	if {$rerun != ""} {
	   set begin [expr $runTable($run,$rerun,startMJD) - \
		$runTable($run,mjd)]
	   set end [expr $runTable($run,$rerun,endMJD) - \
		$runTable($run,mjd)]
	   set mjdCalib $runTable($run,$rerun,mjdCalib)
	   if {$mjdCalib != ""} {set calib [expr $mjdCalib - \
		$runTable($run,mjd)]}
	   set nrerun [llength $runTable($run,rerun)]
	} else {
	   set begin -9999
	   set end -9999
	   set calib -9999
	   }
	echo Run $run Spool $begin, Stuff $end, Calib = $calib,  \
		n(rerun) = $nrerun
	puts $fid "idmetric $run $runTable($run,mjd) $runTable($run,field0) \
	   $runTable($run,fieldn) $begin $end $calib $nrerun"

	foreach rerun $runTable($run,rerun) {
#Make sure all MJD's are numbers
	   foreach var "startMJD endMJD mjdCalib" {
		if {$runTable($run,$rerun,$var) == ""} {
		   set runTable($run,$rerun,$var) -9999
		   }
		}
	   puts $fid "idmetricdetail $run $runTable($run,mjd) $rerun \
		$runTable($run,field0) $runTable($run,fieldn) \
		$runTable($run,$rerun,startMJD) $runTable($run,$rerun,endMJD) \
		$runTable($run,$rerun,mjdCalib) $runTable($run,$rerun,version)"
	   }
	}
   close $fid

   return
#Final check - look at each run, look at earliest rerun, see if there
#are reruns in data directories that are even earlier.  Much check for
#complete set of files, however.
#I will not do this routinely - the only runs that show up are 1458 and 2582.

   foreach run [lsort -integer $runTable()] {
#If not on official list, skip.  Such runs include the gamma ray burst runs.
#Actually, I really should have them in run.par anyway.
	if {[lsearch $runparlist $run] < 0} {
	   echo Run $run not in run.par, skipping
	   continue
	   }

#If quality is bad, skip
	if {[info exists runTable($run,quality)]} {
	   if {$runTable($run,quality) == "bad"} continue
	   }
#Use the earliest rerun.
	set rerun [lindex [lsort -integer $runTable($run,rerun)] 0]
	set runRoot $dataRoot/$run
	set dirs [glob -nocomplain $runRoot/*]
	if {[llength $dirs] == 0} {
	   echo No data for $runRoot!
	   continue
	   }
	set reruns ""
	foreach dir $dirs {
	   set tail [file tail $dir]
	   if {![ctype digit $tail]} continue
	   set file1 $runRoot/$tail/dplogs/framesEnder.$run.$tail.log
	   if {![file exists $file1]} continue
	   set file2 $runRoot/$tail/nfcalib/fcPCalib-[format %06d $run]-1.fit
	   if {![file exists $file2]} continue
	   lappend reruns $tail
	   }
	set first [lindex [lsort -integer $reruns] 0]
	if {$first == ""} continue
	if {$first < $rerun} {
	   echo Run $run data directory $first lower than first database \
		run $rerun
	   }
	}
   return
   }

#######################################################################
#Make plot of imaging metrics
proc idPlot {device} {
   if {![file exists idMetric.par]} {
	echo Please run idMetric first!
	return
	}
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin idMetrics$device
	pgAsk 0
	}
   set chains [param2Chain idMetric.par hdr]
   foreach ch $chains {
	if {[chainTypeDefine $ch] == "IDMETRIC"} {
	   set chain $ch
	} else {
	   set qchain $ch
	   }
	}
   set mindelay -10
   set maxdelay 200
   set minmjd [expr [convertclock "Jan 01, 2000"]/86400 + 40587]
   set maxmjd 0
   chainEach hndl $chain {
	set maxmjd [max $maxmjd [exprGet $hndl.mjd]]
	}
   pgSci 1
   pgSch 1
   pgSlw 2
   set dev [string toupper [lindex [split $device /] 1]]
   if {$dev == "GIF"} {pgSlw 1}

#Dates: Convert to year
   set minyr 2000
   set maxyr [expr $minyr + 1.05*($maxmjd - $minmjd)/365.25]
   pgScr 0 1 1 1
   pgScr 1 0 0 0
   pgEnv $minyr $maxyr $mindelay $maxdelay 0 0
   pgLine "$minyr $maxyr" "0 0"
   set now [fmtclock [getclock] "%Y-%h-%d"]
   pgLabel "Year" "Processing Delay" "IMAGING DATA PROCESSING METRICS: $now"
   set xlist ""
   set y1list ""
   set y2list ""
   set y3list ""
   chainEach hndl $chain {
	set mjdmax [exprGet $hndl.mjd]
	set mjd [expr ($mjdmax-$minmjd)/365.25 + 2000]
	foreach var "spool stuff calib" {
	   set $var [exprGet $hndl.$var]
	   if {[set $var] > $maxdelay} {
		set $var $maxdelay
		}
	   if {[set $var] < $mindelay} {
		set $var $mindelay
		}
	   }
	lappend xlist $mjd
	lappend y1list $spool
	lappend y2list $stuff
	lappend y3list $calib
	set yr2 [expr int(2*$mjd)]
	if {$spool > 0} {lappend spoolMed($yr2) $spool}
	if {$stuff > 0} {lappend stuffMed($yr2) $stuff}
	if {$calib > 0} {lappend calibMed($yr2) $calib}

#Queue stuff:  Put run on the queue for each day from data taking to start
#of spool
	if {$spool < 0} {set spool [expr [getclock]/86400 + 40587]}
	if {$spool > $mjdmax} {
	   loop i $mjdmax $spool {
		set mjdqueue [expr $i/7]
		if {![info exists queue($mjdqueue)]} {
		   set queue($mjdqueue) 1
		} else {
		   incr queue($mjdqueue)
		   }
		}
	   }
	}
   pgSci 2
#   pgLine $xlist $y1list
   loop i 0 [llength $xlist] {
	pgPoint [lindex $xlist $i] [lindex $y1list $i] 3
	}
   foreach yr2 [lsort -integer [array names spoolMed]] {
	set median [lindex $spoolMed($yr2) [expr [llength $spoolMed($yr2)]/2]]
	if {$median > $maxdelay-5} {set median [expr $maxdelay-5]}
	set x1 [expr $yr2/2.]
	set x2 [expr ($yr2+1)/2.]
echo x1 $x1 median spool $median
	pgPoint [expr ($x1+$x2)/2.] $median 3
	pgLine "$x1 $x2" "$median $median"
	}
   pgText 2000 210 "* Spool"
   pgSci 3
#   pgLine $xlist $y2list
   loop i 0 [llength $xlist] {
	pgPoint [lindex $xlist $i] [lindex $y2list $i] 3
	}
   foreach yr2 [lsort -integer [array names stuffMed]] {
	set median [lindex $stuffMed($yr2) [expr [llength $stuffMed($yr2)]/2]]
	if {$median > $maxdelay-5} {set median [expr $maxdelay-5]}
	set x1 [expr $yr2/2.]
	set x2 [expr ($yr2+1)/2.]
echo x1 $x1 median stuff $median
	pgPoint [expr ($x1+$x2)/2.] $median 3
	pgLine "$x1 $x2" "$median $median"
	}
   pgText 2000.3 210 "* Stuff"
   pgSci 4
#   pgLine $xlist $y3list
   loop i 0 [llength $xlist] {
	pgPoint [lindex $xlist $i] [lindex $y3list $i] 3
	}
   foreach yr2 [lsort -integer [array names calibMed]] {
	set median [lindex $calibMed($yr2) [expr [llength $calibMed($yr2)]/2]]
	if {$median > $maxdelay-5} {set median [expr $maxdelay-5]}
	set x1 [expr $yr2/2.]
	set x2 [expr ($yr2+1)/2.]
echo x1 $x1 median calib $median
	pgPoint [expr ($x1+$x2)/2.] $median 3
	pgLine "$x1 $x2" "$median $median"
	}
   pgText 2000.6 210 "* Calib"
   genericChainDel $chain



   if {[pgQinf DEVICE] == ""} {
	pgAsk 1
   } else {
	pgEnd
	pgBegin idQueue$device
	}

#Now compute queue length.  I have already store the time between data
#taking and the first spooling.  Now add in all time between spooling and
#stuffing or calibrating (whichever is bigger)
   chainEach hndl $qchain {
	set mjdstart [exprGet $hndl.mjd]
	set spool [exprGet $hndl.spoolMJD]
	if {$spool < 0} continue
	set calib [exprGet $hndl.calibMJD]
	set stuff [exprGet $hndl.stuffMJD]
	set end [max $calib $stuff]
	if {$end > $spool} {
	   loop i $spool $end {
		set mjdqueue [expr $i/7]
		if {![info exists queue($mjdqueue)]} {
		   set queue($mjdqueue) 1
		} else {
		   incr queue($mjdqueue)
		   }
		}
	   }
	}

#Now plot
   set maxqueue 50
   pgSci 1
   pgScr 0 1 1 1
   pgScr 1 0 0 0
   pgEnv $minyr $maxyr 0 $maxqueue 0 0
   pgLabel "Year" "Queue length" "IMAGING DATA PROCESSING QUEUE: $now"
   foreach mjdqueue [lsort -integer [array names queue]] {
	set mjdmax [expr $mjdqueue*7]
	set mjd [expr ($mjdmax-$minmjd)/365.25 + 2000]
	set x1 [expr $mjd-.009582]
	set x2 [expr $mjd+.009582]
	pgRect $x1 $x2 0 [expr $queue($mjdqueue)/7.]
#	pgPoint $mjd [expr $queue($mjdqueue)/7.] 3
	}
   if {[pgQinf DEVICE] == ""} {
	pgAsk 1
   } else {
	pgEnd
	pgBegin idOutput$device
	}

#Now compute delivered square degrees.
   chainEach hndl $qchain {
	set mjd [exprGet $hndl.stuffMJD]
	set calib [exprGet $hndl.calibMJD]
	set end [max $mjd $calib]
	if {$end <= 0} continue
	set field0 [exprGet $hndl.field0]
	set nfield [expr [exprGet $hndl.fieldn] - $field0 + 1]
	set sqdeg [expr int($nfield/100.*18.75)]
	set mjdout [expr $end/7]
	if {![info exists output($mjdout)]} {
	   set output($mjdout) $sqdeg
	} else {
	   incr output($mjdout) $sqdeg
	   }
	}
   genericChainDel $qchain

#Now plot
   set maxout 1000.
   pgSci 1
   pgSfs 1
   pgSlw 2
   set dev [string toupper [lindex [split $device /] 1]]
   if {$dev == "GIF"} {pgSlw 1}
   pgScr 0 1 1 1
   pgScr 1 0 0 0
   pgEnv $minyr $maxyr 0 $maxout 0 0
   pgLabel "Year" "Delivered Data" "IMAGING DATA PROCESSING OUTPUT: $now"

   pgSfs 1
   foreach mjdout [lsort -integer [array names output]] {
	set mjdmax [expr $mjdout*7]
	set mjd [expr ($mjdmax-$minmjd)/365.25 + 2000]
	set x1 [expr $mjd-.009582]
	set x2 [expr $mjd+.009582]
	pgRect $x1 $x2 0 $output($mjdout)
#	pgPoint $mjd [expr $output($mjdout)] 3
	}
   if {[pgQinf DEVICE] != ""} {
	pgEnd
   } else {
	pgAsk 0
	}
   return
   }

#######################################################################
proc idMetrics {} {
   idMetricsCompute
   idPlot .ps/CPS
   idPlot .gif/GIF
   return
   }


################################################################
#Run all the metrics jobs
#Check for machine name.  If on sdssdp11, run spectro and imaging.
#If on sdssdp4, run mt

proc metricsJob {} {
   set machine [exec funame]
   if {$machine == "sdssdp4"} {
	echo Running mt metrics on sdssdp4\; go to sdssdp11 to run \
	   imaging/spectro
	mtMetrics
	mdPlot
   } elseif {$machine == "sdssdp25"} {
	echo Running imaging/spectro metrics on sdssdp25\; go to sdssdp4 \
	   to run mt
	idMetrics
	idPlot
	}
   return
   }

######################################################################
#Compute total data processed, including reprocessing.  Because opdb no longer
#contains a full record, I will crawl the imaging data directory, search for
#all reruns that are not logical links, fetch the raw frame count from
#run.par, and compute an approximate data volume that way.

proc dpMetrics {} {
   global dataRoot
   set runChain [param2Chain run.par hdr]
   set runTable() ""
   chainEach hndl $runChain {
	set run [exprGet $hndl.run]
	set frameStart [exprGet $hndl.frameStart]
 	set frameEnd [exprGet $hndl.frameEnd]
	set quality [exprGet $hndl.quality]
	lappend runTable() $run
	set runTable($run,frameStart) $frameStart
	set runTable($run,frameEnd) $frameEnd
	set runTable($run,quality) $quality
	set runTable($run,len) [expr $frameEnd-$frameStart+1]
	}
   genericChainDel $runChain
   set dirs [glob -nocomplain $dataRoot/*]

#Total number of fields
   set ntot 0
   foreach dir $dirs {
	set run [file tail $dir]
	if {![ctype digit $run]} continue
	if {[lsearch $runTable() $run] < 0} continue
	if {$runTable($run,quality) == "bad"} continue
	set subdirs [glob -nocomplain $dir/*]
	set nrerun 0
	foreach subdir $subdirs {
	   set rerun [file tail $subdir]
	   if {![ctype digit $rerun]} continue
	   if {[file type $subdir] != "directory"} continue
	   incr nrerun 1
	   }
	set ntot [expr $ntot + $nrerun*$runTable($run,len)]
	}

#Data volume in TB
   set volume [expr $ntot*.17e-3]
   echo Volume: [format %.1f $volume]
   return
   }
