##############################################################
#Compute various astronomical quantities.

proc mtEphemeris {mjdate} {

#Define globals on-the-fly whenever this routine is run.
   set globals "beginDate endDate mjdSunrise mjdSunset mjdMornTwi \
	mjdEveTwi mjdMoonrise mjdMoonset phase lstMid dst"
   eval global $globals
   eval uplevel 1 global $globals

#Get a timestamp for this time
   set tstamp [tstampFromMJD $mjdate]

#Sidereal time at midnight.  Returned value is in degrees.
#I set longitude to 0 because otherwise it computes lst at APO for UT=0
#not local midnight.  I'm a bit lazy.  NOTE: This is lst at midnight standard
#time, regardless of DST flag.  [Is this what I want? No  - see below for
#dst correction]
   set lstMid [string trim [tstampToLst $tstamp -long 0]]
   tstampDel $tstamp

#Results are compute in UT hrs

#Sunrise and set
   set riseSet [mtRiseSet $mjdate 10 90.75]
   set mjdSunrise [mjdToX [lindex $riseSet 0] $mjdate]
   set mjdSunset  [mjdToX [lindex $riseSet 1] $mjdate]

#Twilit
   set riseSet [mtRiseSet $mjdate 10 108]
   set mjdMornTwi [mjdToX [lindex $riseSet 0] $mjdate]

   set mjdEveTwi  [mjdToX [lindex $riseSet 1] $mjdate]

#Moonrise and set
   set riseSet [mtRiseSet $mjdate 3 90.75]
   set mjdMoonrise [mjdToX [lindex $riseSet 0] $mjdate]
   set mjdMoonset  [mjdToX [lindex $riseSet 1] $mjdate]

   set line ""
   set vars "mjdEveTwi mjdMornTwi mjdMoonrise mjdMoonset"
   foreach global $vars {
	lappend line [set $global]
	}
   set list [list $vars $line]
   return $list
   }
		
#####################################################################
#Convert an MJD to UT hours since a reference local midnight.

proc mjdToX {mjd mjdate} {
   set hr [expr ($mjd-$mjdate)*24.]
#   set hr [expr int($hr+0.5)]
   return $hr
   }

###################################################################
#Compute rise and set times of specified object for APO
#mjdate is integer MJD that falls in the midst of a night.  Typically, this
#actually occurs around 5 pm for APO.

#
# The timezone input to mjdRiseSet is used only to make sure that times are
# centered around local midnight.  I am suspicious that there may be sidereal
# time ambiguities not properly treated in astrotools.
#
#In fact, there appear to be a couple of bugs in mjdRiseSet.  First,
#the variable "time" is the amount of sidereal time to be added to int(mjd)
#for the desired phenomena; this is converted to days using 24 hr, which is
#wrong - should use 23h 56m for one sidereal day.  Second, the resolution of
#the proper day also suffers from the same problem.  We should fix up "time"
#to be module the right amount

proc mtRiseSet {mjdate obj zd} {
   global at_APO_25_longitude at_APO_25_latitude

#Hack for stupid renaming of at_ compiled constants.
   if {![info exists at_APO_25_longitude]} {
	global at_APOLongitude2_5m
	set at_APO_25_longitude $at_APOLongitude2_5m
	}

   if {![info exists at_APO_25_latitude]} {
	global at_APOLatitude2_5m
	set at_APO_25_latitude $at_APOLatitude2_5m
	}

#Get mjd at local midnight.  Note that longitude is + for EAST.
   set mjd [expr [int $mjdate] - $at_APO_25_longitude/360.]

#Zenith distance is astronomical twilight for sun; normal rise, set for others
#   if {$obj == 10} {set zd 108} else {set zd 90}
#Input instead.

#Get nominal ra, dec
   set tstamp [tstampFromMJD $mjd]
   set keylist [planetRaDec $tstamp $obj]
   tstampDel $tstamp
   set ra [keylget keylist ra]
   set dec [keylget keylist dec]

#I think this routine returns UT; time zone is just to get the proper
#approximate local time.
#
   set mjdset [mjdRiseSet $ra $dec [abs $at_APO_25_longitude] \
	$at_APO_25_latitude $mjdate 7 $zd set]
   set mjdrise [mjdRiseSet $ra $dec [abs $at_APO_25_longitude] \
	$at_APO_25_latitude $mjdate 7 $zd rise]

#Repeat with improved ra, dec
   set tstamp [tstampFromMJD $mjdset]
   set keylist [planetRaDec $tstamp $obj]
   tstampDel $tstamp
   set ra [keylget keylist ra]
   set dec [keylget keylist dec]
   set mjdset [mjdRiseSet $ra $dec [abs $at_APO_25_longitude] \
	$at_APO_25_latitude $mjdate 7 $zd set]

#Repeat with improved ra, dec
   set tstamp [tstampFromMJD $mjdrise]
   set keylist [planetRaDec $tstamp $obj]
   tstampDel $tstamp
   set ra [keylget keylist ra]
   set dec [keylget keylist dec]
   set mjdrise [mjdRiseSet $ra $dec [abs $at_APO_25_longitude] \
	$at_APO_25_latitude $mjdate 7 $zd rise]
#echo Setting: [mtMJDDate $mjdset]
#echo Rising: [mtMJDDate $mjdrise]
   return "$mjdrise $mjdset"
   }

###################################################################3
#Return printable version of mjd
proc mtMJDDate {mjd} {
   set tstamp [tstampFromMJD $mjd]
   set year [exprGet $tstamp.year]
   set mon [exprGet $tstamp.month]
   set day [exprGet $tstamp.day]
   set hr [exprGet $tstamp.hour]
   set min [exprGet $tstamp.minute]
   set sec [exprGet $tstamp.second]
   tstampDel $tstamp

#Get unix time.  I use the tclX clock routines
   set unix [convertclock "$mon/$day/$year $hr:$min:[int $sec]" GMT]
   return [fmtclock $unix "%d-%b-%y" GMT]
   }
#########################################################################
