### Routines to read seeing from opdb and create seeing-xxxx.tcl files
### Must setup photoSch on sdssdp4 to run.
### Useful commands:
### opdbConnect - defines the OO_FD_BOOT variable, begins a session, opens
###	necessary containers
### runInventory - queries for all runs & reruns & saves in runTable array.
### calibInventory - prints all runs without photometric calibs.  Reads
###    run.par to get more run parameters.
### seeingReadFromOPDB run rerun - Reads seeing, creates seeing array,
###	also writes to a seeing-run.tcl file.  If file exists, database is
###	skipped (but you would need to source the file to create the seeing
###	array entries).
### opdbDisconnect

proc opdbConnect {} {
   if {[inSession]} return
   dbInit -nMaxPages 5000
   sessionBegin -readonly -mrow
   photoInit -readonly
   }

#######################################################################
proc opdbDisconnect {} {
   if {![inSession]} return
   sessionEnd
   }

#######################################################################
#Read seeing and write to a file.
#This code is more or less a copy of that in the dp product in etc/dpSeeing.tcl
proc seeingReadFromOPDB {run rerun} {
   global seeing psfQuality fpQuality
   if {![inSession]} {opdbConnect}
   set seeing($run) ""
   set psfQuality($run) ""
   foreach camcol "1 2 3 4 5 6" {
	set seeing($run,$camcol) ""
	if {[catch {set frames [pipeGet frames $run $camcol -rerun $rerun]}]} \
	   continue
	lappend seeing($run) $camcol
	lappend psfQuality($run) $camcol
echo Run $run, camcol $camcol
	set psCalib [$frames -> postageStampCalibration]

#This is not the final photometric calibration!!!
	set transArray [$psCalib -> photometricFieldCalibrations]
	set psfArray [$psCalib -> photometricFieldCalibrations2]
	set n [$psCalib -> nFields]
	set first [$psCalib -> field0]
	loop i 0 $n {
	   set field [expr $i + $first]
	   lappend seeing($run,$camcol) $field
	   lappend psfQuality($run,$camcol) $field
	   set trans [$transArray elem $i]
	   foreach filter "u g r i z" {
		set fhandle [$trans $filter]
		set seeing($run,$camcol,$field,$filter) [format %.4f \
		   [$fhandle psf_width]]
		handleDel $fhandle
		}
	   handleDel $trans

#Read quality
	   set psf [$psfArray elem $i]
	   set psfStatus [$psf psp_status]
	   handleDel $psf

#Here I keep the numerical value (whereas setQuality converts to a status)
#The status is the max over all 5 filters, which is good.
	   set psfQuality($run,$camcol,$field) $psfStatus
	   }
	handleDel $transArray
	handleDel $psfArray
	objDel $psCalib
	objDel $frames
#	puts $fid "set seeing($run,$camcol) \"$seeing($run,$camcol)\""
	}
#   puts $fid "set seeing($run) \"$seeing($run)\""
#   close $fid
   return
   }

####################################################################
#Convert opdb timestamp to mjd
proc mjdStamp {tstamp} {
   set tstamp [split [lindex $tstamp 0] T]
   set tstamp [split [lindex $tstamp 0] -]
   set yr [lindex $tstamp 0]
   set mon  [lindex $tstamp 1]
   set day [lindex $tstamp 2]

#Sigh, opdb can have a bad time stamp, it seems
   if {[catch {set time [convertclock $mon/$day/$yr]}]} {
	echo In mjdStamp, encountered bad tstamp $tstamp.
	return 0
	}
   set mjd [expr int($time/86400)+40587]
   return $mjd
   }

####################################################################
proc runInventory {} {
   global runTable photo_frames
   if {![inSession]} opdbConnect
   if {[info exists runTable]} {unset runTable}
   set runTable() ""
   set itr [queryGet arFramesPipeRun $photo_frames "camCol_ = 4"]
   while {[$itr next]} {
#	if {[$itr -> camCol] != 4} continue
	set run [$itr -> run]
	if {$run == 2259 && $rerun == 40} continue
	set field0 [$itr -> field0]
	set nFields [$itr -> nFields]
	set fieldn [expr $field0 + $nFields - 1]
	if {[lsearch $runTable() $run] < 0} {
	   lappend runTable() $run
	   }
	set runTable($run,field0) $field0
	set runTable($run,fieldn) $fieldn
	set rerun [$itr -> rerun]
	if {[catch {set hndl [$itr -> postageStampCalibration]}]} {
	    echo No PS calibration, run $run, rerun $rerun
	    continue
	   }
	objDel $hndl

#EEARGH!  Yanny stuck in some test reruns.  Need to trash.
	if {$rerun > 100} {
	   echo Skipping run $run rerun $rerun
	   continue
	   }
	if {$rerun >= 10 && $rerun < 20} {
	   echo Skipping run $run rerun $rerun
	   continue
	   }
#Fetch the stripe, strip
	set hndl [$itr -> imagingRun]
	if {![$hndl isValid]} {
	   echo Invalid imagingRun handle for run $run rerun $rerun. Skipping
	   continue
	   }
	set stripe [$hndl -> stripe]
	set strip [$hndl -> strip]
	objDel $hndl
	set runTable($run,stripe) $stripe
	set runTable($run,strip) $strip

	if {[catch {set hndl [$itr -> currentPhotometricCalibration]}]} {
	    echo No photo calibration, run $run, rerun $rerun
	    continue
	   }
	set runTable($run,$rerun,photo) 1
	if {![$hndl isValid]} {
	   echo No photometric calibration run $run rerun $rerun
	   set runTable($run,$rerun,photo) 0
	   set mjdCalib ""
	} else {

#Get the mjd
	   set id [$hndl -> id]
	   set mjdCalib [mjdStamp $id]
	   if {$mjdCalib == 0} {
		echo Badk photo calib mjdStamp for run $run, rerun $rerun
		}
	   }

	set runTable($run,$rerun,mjdCalib) $mjdCalib
	objDel $hndl
	if {![info exists runTable($run,rerun)]} {
	   set runTable($run,rerun) $rerun
	} else {

	   lappend runTable($run,rerun) $rerun
	   }

#Fetch the processing date
	set runTable($run,$rerun,mjdProc) [mjdStamp [$itr -> id]]
	if {$runTable($run,$rerun,mjdProc) == 0} {
	   echo Bad processing mjdStamp for run $run rerun $rerun
	   }
	}
   objDel $itr

#Compute the optimum rerun.  Because setQuality keeps getting reset,
#I will use the most recent photo V5_2 rerun (rerun < 10) if present, else
#the largest absolutely

#Hmm, let's use a rerun > 40 if present (and check on limits).
   foreach run $runTable() {
	set rerun -1
	set max -1
	foreach r $runTable($run,rerun) {
	   if {$r >= 40} {set rerun [max $rerun $r]}
	   set max [max $max $r]
	   }
	if {$rerun == -1} {set rerun $max}
	if {$rerun == -1} {
	   echo No reruns for run $run!
	   continue
	   }
	set runTable($run) $rerun
	set runTable($run,photo) $runTable($run,$runTable($run),photo)

#Fetch the processing date
	set runTable($run,$rerun) $runTable($run,$rerun,mjdProc)
	}
   return
   }

####################################################################
#To work around memory problems, do runInventory and write out results.
proc runInventorySave {} {
   global runTable
   runInventory
   arrayWrite runTable
   opdbDisconnect
   return
   }

####################################################################
#After running runInventory, get observing mjd.  This is SLOW
#Get mjd of run from arRun object
proc runMJD {} {
   global runTable
   foreach run $runTable() {
	global photo_runs
	set hndl [queryGetOne arRun $photo_runs "id_ = $run"]
	set mjdhndl [$hndl -> mjdStart]
	set mjd [$mjdhndl MJD]
	handleDel $mjdhndl
	objDel $hndl
	set runTable($run,mjd) $mjd
	echo $run $mjd
	}
   return
   }

####################################################################
#After running runInventory, list all runs without calibrations.
#Also, create a file odRunStatus that gives the status of a run.
#I would like to read this into another proc that tracks the status of
#run calibrations.

proc calibInventory {} {
   global runTable
   if {![info exists runTable]} {runInventory}
   if {[file exists run.par]} {
	set chain [param2Chain run.par hdr]
	}
   set mjd [expr ([getclock]/82640) + 40587]
   set file odRunStatus-$mjd.par
   echo Creating file $file
   set fid [open $file w]
   puts $fid "#Created [fmtclock [getclock]]"
   puts $fid "mjd $mjd"
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   int iscalib;"
   puts $fid "   }; RUNPHOTOSTATUS"
   puts $fid ""
   foreach run [lsort -integer $runTable()] {
	set stripe 0
	set strip ""
	set status ""

#Skip gamma ray burst run
	if {$run == 2143} continue
	if {[info exists chain]} {
	   set hndl [chainGetOne $chain run $run]
	   if {$hndl != ""} {
		set stripe [exprGet $hndl.stripe]
		set strip [exprGet $hndl.strip]
		set status [string tolower [exprGet $hndl.quality]]
		handleDel $hndl
	   } else {
		echo Run $run not in run.par
		}
	   }
	if {$status == "bad"} continue
	puts $fid "runphotostatus $run $runTable($run,rerun) $stripe $strip \
	   $runTable($run,photo)"
	if {$runTable($run,photo)} continue
	echo No calibration for run $run rerun $runTable($run,rerun) \
		strip $stripe$strip

#Test to see if calibrations exist
	set test \
/data/dp3.b/data/$run/$runTable($run,rerun)/nfcalib/fcPCalib-[format \
	   %06d $run]-1.fit
	if {[file exists $test]} {
	   echo "   " There may be a calibration on disk still.
	   }
	}
   if {[info exists chain]} {
	chainEach hndl $chain {
	   set run [exprGet $hndl.run]
	   set stripe [exprGet $hndl.stripe]
	   set strip [exprGet $hndl.strip]
	   set status [exprGet $hndl.quality]
	   if {[string tolower $status] == "bad"} continue
	   if {[lsearch $runTable() $run] >= 0} continue
	   set rerun -1
	   echo Data not processed for $run

#See if I have web pages.  If so, data may just not have been stuffed
	   set test /data/dp7.c/users/skent/sdssmosaic/www/run-$run
	   if {[file exists $test]} {
		echo "   " Web pages exist for $run
		}	
#Test to see if calibrations exist
	   set test /data/dp3.b/data/$run/0/nfcalib/fcPCalib-[format \
		%06d $run]-1.fit
	   if {[file exists $test]} {
		echo "   " A calibration seems to exist for $run
		}
	   puts $fid "runphotostatus $run $rerun $stripe $strip 0"
	   }
	}
   close $fid
   if {[info exists chain]} {
	genericChainDel $chain
	}
   return
   }

####################################################################
#Procs to work with mt database
proc mtdbConnect {} {
   sessionBegin -readonly -mrow
   mtInit -readonly
   }

#######################################################################
proc mtdbDisconnect {} {
   sessionEnd
   }

