#Match up patches that are "good" with actual scans
#Commands
#	patchStatusPlot
#		Read run.par, patch.par and make overlay plots
#		run.par comes from rundb, patch.par comes from patchStatusFetch
#		Also read odPatchReport.par to get actual seqeunces.
#	patchStripePlot
#		Plot all patches on a map of all stripes.
#	patchStatus
#		Run a report on mopdb and create odPatchStatus.par
#	patchStatusFetch
#		Fetch a report from MOPDB and dump to patch.par
#	patchStatusHistory
#		Read latest odPatchStatus file and report on done patches.
#	patchStrategy
#	patchSkyFetch
#	patchSkyAnalyze
#	patchSkyList
#		Read odPatchStatus file and check sky values.

set mtDir /data/dp30.a/data/mt/dp/apo20/run/listed_by_mjd

#Temporary
set mtDir /data/dp40.c/data/mt/dp/apo20/run/listed_by_mjd

proc patchStatusPlot {{runs ""}} {
   global patchChain secChain odFile

#Get odPatchReport file.  Used only to fetch sequence numbers.
   set files [glob -nocomplain odPatchStatus*.par]
   if {[llength $files] > 0} {
	set files [lsort $files]
	set odFile [lindex $files end]
   } else {
	set odFile ""
	}
    if {![info exists patchChain]} {
	set patchChain [param2Chain patch.par hdr]
    }
   if {![info exists secChain]} {
	global env
	if {[info exists env(MTSTDS_DIR)]} {
	   set secChain [param2Chain $env(MTSTDS_DIR)/secondary/mt_patch.par \
		hdr]
	   }
	}
   set runChain [param2Chain run.par hdr]
   chainFSort $runChain run
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}
   chainEach hndl $runChain {
	foreach var "run stripe strip muStart muEnd" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$runs != "" && [lsearch $runs $run] < 0} continue

	set node 95.
	set incl [expr ($stripe-10)*2.5]
	if {$incl > 90.} {set incl [expr $incl-180.]}

#Try using stripeLimits for segue stripes.
	if {$stripe > 95} {
	   set list [stripeLimits $stripe]
	   set incl [keylget list incl]
	   set node [keylget list node]
	   }

#If astrom file exists, read it and fetch muStart, muEnd.
	global astrom masterdir
	if {![info exists masterdir]} {set masterdir /data/dp3.b/data}
	if {[info exists astrom($run,3,r)] || ![catch {astromRead $run}]} {
	   set hndl [chainGet $astrom($run,3,r) 0]
	   set old $muStart
	   set muStart [exprGet $hndl.a]
	   echo Change muStart from $old to [format %.0f $muStart]
	   handleDel $hndl
	   set hndl [chainGet $astrom($run,3,r) [expr [chainSize \
		$astrom($run,3,r)] - 1]]
	   set old $muEnd
	   set muEnd [exprGet $hndl.a]
	   echo Change muEnd from $old to [format %.0f $muEnd]
	   handleDel $hndl
	   echo Using asCalib for calibrations
	   }
	echo Run $run stripe $stripe
	if {$muStart > $muEnd} {set muEnd [expr $muEnd+360]}
	pgSci 1
	pgEnv $muStart $muEnd -1.3 1.3 0 0
	pgLabel "mu" "nu" "Run $run"
	pgSci 6
	set off 0
	if {$strip == "S"} {set off -1}
	if {$strip == "N"} {set off 1}

#Separation of 2 scan lines in degrees
	set sep .212
	foreach scan "1 2 3 4 5 6" {
	   set nucenter [expr (2.*($scan-3.5) + 0.5*$off)*$sep]
	   set numin [expr $nucenter-0.5*$sep]
	   set numax [expr $nucenter+0.5*$sep]
	   pgPoly "$muStart $muEnd $muEnd $muStart $muStart" \
		"$numin $numin $numax $numax $numin"
	   }
	set stripe1 [expr $stripe-1]
	set stripe2 [expr $stripe+1]

#If we have observations of a patch but haven't finished all repeats, those
#patches don't make it into goodPatch and miss any existing observations.
#Well, life is tough.
	set goodPatch [chainSearch $patchChain "{$stripe1 <= stripe \
	   <= $stripe2}"]
echo Number of patches = [chainSize $goodPatch]
	chainEach patch $goodPatch {
	   foreach var "name ra dec" {
		set $var [exprGet $patch.$var]
		}
#Get min, max ra and dec and then rotate.
	   set width .683
	   set ra1 [expr $ra - $width*.5/cos($dec/57.3)]
	   set dec1 [expr $dec - $width*.5]
	   set ra2 [expr $ra + $width*.5/cos($dec/57.3)]
	   set dec2 [expr $dec - $width*.5]
	   set ra3 [expr $ra + $width*.5/cos($dec/57.3)]
	   set dec3 [expr $dec + $width*.5]
	   set ra4 [expr $ra - $width*.5/cos($dec/57.3)]
	   set dec4 [expr $dec + $width*.5]
	   set list [eqToGC $ra $dec -node $node -inclination $incl]
	   set mu [keylget list mu]
	   set nu [keylget list nu]
	   set list [eqToGC $ra1 $dec1 -node $node -inclination $incl]
	   set mu1 [keylget list mu]
	   set nu1 [keylget list nu]
	   set list [eqToGC $ra2 $dec2 -node $node -inclination $incl]
	   set mu2 [keylget list mu]
	   set nu2 [keylget list nu]
	   set list [eqToGC $ra3 $dec3 -node $node -inclination $incl]
	   set mu3 [keylget list mu]
	   set nu3 [keylget list nu]
	   set list [eqToGC $ra4 $dec4 -node $node -inclination $incl]
	   set mu4 [keylget list mu]
	   set nu4 [keylget list nu]
	   if {$mu < $muStart-180} {set mu [expr $mu+360]}
	   if {$mu > $muStart+180} {set mu [expr $mu-360]}
	   foreach var "mu1 mu2 mu3 mu4" {
		if {[set $var] < $mu - 180} {
		   set $var [expr [set $var] + 360]
		   }
		if {[set $var] > $mu + 180} {
		   set $var [expr [set $var] - 360]
		   }
		}
	   pgSci 3
	   pgPoly "$mu1 $mu2 $mu3 $mu4 $mu1" \
		"$nu1 $nu2 $nu3 $nu4 $nu1"
	   pgSci 1
	   set mut [expr ($mu+1.)]
	   set nut $nu
	   pgText $mut $nut $name

#List sequence numbers of good patches.
	   if {$odFile != ""} {
		set seqs ""
		catch {set seqs [split [string trim \
		   [exec grep $name $odFile]] \n]}
		foreach line $seqs {
		   set seqId [lindex $line 3]
		   set quality [lindex $line 4]
		   if {$quality == "bad"} continue
		   set nut [expr $nut-.1]
		   pgText $mut $nut $seqId
		   }
		}
	   }
	chainDel $goodPatch

	if {![info exists secChain]} continue

#Plot all secondary patches, so we can see the standard positions.
	set goodPatch [chainSearch $secChain "{[expr $stripe-1] <= stripe \
	   <= [expr $stripe+1]}"]
	chainEach patch $goodPatch {
	   foreach var "name ra dec" {
		set $var [exprGet $patch.$var]
		}
	   set incl [expr ($stripe-10)*2.5]
	   if {$incl > 90.} {set incl [expr $incl-180.]}

#Try using stripeLimits for segue stripes.
	   if {$stripe > 90} {
		set list [stripeLimits $stripe]
		set incl [keylget list incl]
		set node [keylget list node]
		}

	   set list [eqToGC $ra $dec -node $node -inclination $incl]
	   set mu [keylget list mu]
	   if {$mu < $muStart-180} {set mu [expr $mu+360]}
	   if {$mu > $muStart+180} {set mu [expr $mu-360]}
	   set nu [keylget list nu]
	   set width .683
	   set numin [expr $nu-0.5*$width]
	   set numax [expr $nu+0.5*$width]
	   set muStart [expr $mu-0.1*$width]
	   set muEnd [expr $mu+0.1*$width]
	   pgSci 4
	   pgPoly "$muStart $muEnd $muEnd $muStart $muStart" \
		"$numin $numin $numax $numax $numin"
	   pgSci 4
	   set mut [expr ($mu+1.)]
	   set nut $nu
	   pgText $mut $nut $name
	   }
	chainDel $goodPatch
	}
   genericChainDel $runChain
   return
   }

################################################################
#Make a plot of all done patchs on an overlay of all stripes.
proc patchStripePlot {} {

#Get odPatchReport file
   set file patch.par
   echo Reading $file
   set patchChain [param2Chain $file hdr]

#Read all secondary patches
   if {![info exists secChain]} {
	global env
	if {[info exists env(MTSTDS_DIR)]} {
	   set secChain [param2Chain $env(MTSTDS_DIR)/secondary/mt_patch.par \
		hdr]
	   }
	}

   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; sleep 1; pgGeomSet 800 800 10 25
	pgAsk 0
	}

   pgSci 1
   pgSlw 1
   pgSch 1
   pgScr 0 1 1 1
   pgScr 1 0 0 0
   pgEnv -70 70 -58 58 0 0
   pgLabel "Survey Longitude" "Survey Latitude" ""
   loop stripe 45 0 -1 {
	set list [stripeLimits $stripe]
	set amin [keylget list lambdaMin]
	set amax [keylget list lambdaMax]
	set eta [keylget list eta]
	set etamin [expr $eta-1.25]
	set etamax [expr $eta+1.25]
	pgSci 4
	pgLine "$amin $amin $amax $amax $amin" \
	   "$etamin $etamax $etamax $etamin $etamin"

#Get patches
	set subChain [chainSearch $secChain "{stripe = $stripe}"]
	chainEach hndl $subChain {
	   set lambda [exprGet $hndl.lambda]
	   set eta [exprGet $hndl.eta]
	   set lmin [expr $lambda-.33]
	   set lmax [expr $lambda+.33]
	   set emin [expr $eta-.33]
	   set emax [expr $eta+.33]
	   pgSci 2
	   pgPoly "$lmin $lmin $lmax $lmax $lmin" \
		"$emin $emax $emax $emin $emin"
	   }
	chainDel $subChain
	set subChain [chainSearch $patchChain "{stripe = $stripe}"]
	chainEach hndl $subChain {
	   set ra [exprGet $hndl.ra]
	   set dec [exprGet $hndl.dec]
	   set name [exprGet $hndl.name]
	   if {![regexp {[0-9]+[ABCD][0-9]+} $name]} continue
	   set list [eqToSurvey $ra $dec]
	   set lambda [keylget list lambda]
	   set eta [keylget list eta]
	   set lmin [expr $lambda-.33]
	   set lmax [expr $lambda+.33]
	   set emin [expr $eta-.33]
	   set emax [expr $eta+.33]
	   pgSci 3
	   pgPoly "$lmin $lmin $lmax $lmax $lmin" \
		"$emin $emax $emax $emin $emin"

	   }
	chainDel $subChain
	}
   genericChainDel $patchChain
   genericChainDel $secChain
   return
   }

################################################################
#Create odPatchStatus.par
#Must be connected to opdb first!  (Look in opdb.tcl)

proc patchStatus {} {
   if {![inSession]} {
	mtdbConnect
	}
   set chain [patchesGet]

#Change "undetermined" to "unknown"
#Different behavior: Remove sequences that are unknown.  Why?  In mopdb,
#if I get "undetermined", I unverify the sequence.  In many cases, I have
#declared the sequence to be bad by hand, which also sets the verify state on.
#If a new status is "unknown", the sequence in opdb is reverted to unverified,
#which is not always the desired effect.

   set outChain [chainNew [chainTypeDefine $chain]]
   loop i 0 [chainSize $chain] {
	set hndl [chainElementGetByPos $chain $i]
	set quality [exprGet $hndl.quality]
	set name [exprGet $hndl.name]
	if {![regexp {^[0-9]+[A-Da-d][0-9]+} $name]} {
	   handleDel $hndl
	   continue
	   }
	if {$quality == "undetermined"} {
	   handleSet $hndl.quality "unknown"
	   handleDel $hndl
	} else {
	   chainAdd $outChain $hndl
	   }
	}
   set mjd [expr [getclock]/86400 + 40587]
   set file odPatchStatus-$mjd.par
   echo File is $file
   set hdr ""
   keylset hdr mjd $mjd

#Now make a list of bad mjd's
   chain2Param $file $outChain $hdr
   chainDel $outChain
   set fid [open $file a]
   set mjds [mjdsGet -kali]
   puts $fid ""
   puts $fid "typedef struct {"
   puts $fid "   int mjd;"
   puts $fid "   } BADMJD;"
   foreach mjd $mjds {
	puts $fid "BADMJD $mjd"
	}
   close $fid
   genericChainDel $chain
   mtdbDisconnect
   return $file
   }

#########################################################################
#Fetch patch status from mopdb

proc patchStatusFetch {} {
   if {[info exists patch.par.tmp]} {
	exec rm patch.par.tmp
	}
   if {[info exists patch.par.html]} {
	exec rm patch.par.html
	}

#For some bizarre reason, lynx -source no longer works with mopdb! Use wget.
   if {[catch {exec wget -O patch.par.html "http://sdsshost2.apo.nmsu.edu:8020/patchReportFile.tml?host=&db=mopdb" >& /dev/null}]} {
	echo Failed.  Continue with old patch.par
	return
	}
   set fid [open patch.par.html]
   set lines [read $fid]
   close $fid
   if {![regexp {<pre>(.+)</pre>} $lines all lines]} {
	echo Failed to decode html.
	return
	}
   set fid [open patch.par.tmp w]
   puts $fid $lines
   close $fid
   exec rm patch.par.html
   exec cp patch.par.tmp patch.par
   exec rm patch.par.tmp
   return
}

##############################################################
#Read odPatchStatus file and compute history
#odPatchStatus is the opdb version of patch history.

proc patchStatusHistory {{stdout ""}} {
   global patchHistory

#Get odPatchReport file
   set files [glob -nocomplain odPatchStatus*.par]
   if {[llength $files] <= 0} {
       echo No odReport files!
       return
	}
   set files [lsort $files]
   set odFile [lindex $files end]
   echo Reading $odFile
   set chains [param2Chain $odFile hdr]
   set patchChain [lindex $chains 0]
   genericChainDel [lindex $chains 1]

   set patchHistory() ""

#Create a patchHistory.txt file
   if {$stdout != ""} {
	echo Creating $stdout
	set stdout [open $stdout w]
   } else {
	set stdout stdout
	}

#Now
   set mjdnow [expr [getclock]/86400 + 40587]
   puts $stdout "     Date       All    Unique  Unique  N(night)"
   puts $stdout "               Seqs.   North   South"
   puts $stdout "                      Patches"
   foreach year "2000 2001 2002 2003 2004 2005" {
        lappend patchHistory() $year
        set patchHistory($year) ""
        foreach mon "Jan Apr Jul Oct" {
           lappend patchHistory($year) $mon
           set mjd [expr [convertclock "01 $mon $year"]/86400 + 40587]

#Allow for current quarter, but nothing beyond.
	   if {$mjd > $mjdnow+90} break

#Include data from Mar31/Apr01, e..g. so allow equality.
	   set subchain [chainSearch $patchChain "{mjdobs <= $mjd}"]
	   set nnorth 0; set nsouth 0; set nall 0
	   set mjdlist ""
	   set allPatchList ""
	   set northPatchList ""
	   set southPatchList ""
	   chainEach hndl $subchain {
		set quality [exprGet $hndl.quality]
		if {$quality == "bad"} {continue}
		set name [exprGet $hndl.name]

#Keep track of unique patches - avoid duplicate exposures.
		if {[lsearch $allPatchList $name] < 0} {
		   incr nall
		   lappend allPatchList $name
		   }

#Note that I am not counting the older patches with a-d.
		if {[regexp {([0-9]+)[ABCD][0-9]+} $name all stripe]} {
		   if {$stripe <= 45} {
			set zone north
		   } elseif {$stripe < 100} {
			set zone south
		   } else {
			continue
			}
		   if {[lsearch [set ${zone}PatchList] $name] < 0} {
			incr n$zone
			lappend ${zone}PatchList $name
			}
		   }
		set mjdobs [exprGet $hndl.mjdobs]
		if {[lsearch $mjdlist $mjdobs] < 0} {
		   lappend mjdlist $mjdobs
		   }
		}
	   set format "%4s-%3s %8.0f %8.0f %8.0f %8.0f"
	   puts $stdout [format $format $year $mon $nall $nnorth $nsouth \
		[llength $mjdlist]]
	   chainDel $subchain

#Create ptStatus.par for John Inkmann.  Overwrite continuously, so the last
#time through produces the most recent status.
	   set fid [open ptStatus.par w]
	   puts $fid "date [fmtclock [getclock] "%Y %h %d"]"
	   puts $fid "mjd $mjdnow"
	   puts $fid "all $nall		#All sequences"
	   puts $fid "npatch [expr $nnorth+$nsouth] \
		#Unique new patches observed"
	   puts $fid "ntotal 1520	#Total number of patches to observe"
	   close $fid
	   }
	}
   genericChainDel $patchChain
   if {$stdout != "stdout"} {
	close $stdout
	}


#Read patch.par and compare its view of the world with odPatchStatus.
#Just check for existence of DONE VERIFIED patches.
   if {![file exists patch.par]} return
   set patchChain [param2Chain patch.par hdr]
   chainEach hndl $patchChain {
	set name [exprGet $hndl.name]
	set status [exprGet $hndl.status]
	set verified [exprGet $hndl.verified]
	if {$status != "DONE"} continue
	if {$verified != "VERIFIED"} continue

#Skip old patches.
	if {![regexp {[0-9](A|B|C|D)} $name]} continue
	if {[lsearch $northPatchList $name] < 0 && \
		[lsearch $southPatchList $name] < 0} {
	   echo Patch $name is done, verified in mopdb but not opdb!
	   }
	}

   return
   }

#######################################################################
proc patchStrategy {} {
   patchStatusFetch
   set file [patchStatus]
   patchStatusHistory patchHistory.txt

#Now compute metrics
   mdMetrics

   echo
   echo Do: scp $file sdsshost2.apo.nmsu.edu:/p/mopdata/v1_1/run
   echo Enter this file into mopdb
   return
   }

####################################################################
proc patchSkyFetch {} {
   global mjdData
   if {[info exists mjdData]} {unset mjdData}
   set files [lsort [glob -nocomplain odPatchStatus*.par]]
   set file [lindex $files end]
   if {$file == ""} {
	echo No odPatchStatus file found
	return
	}
   set chains [param2Chain $file hdr]
   set chain [lindex $chains 0]
   genericChainDel [lindex $chains 1]
   set mjdData() ""
   chainEach hndl $chain {
	foreach var "name mjdobs seqId quality" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$quality == "bad"} continue
	if {[lsearch $mjdData() $mjdobs] < 0} {
	   lappend mjdData() $mjdobs
	   set mjdData($mjdobs) ""
	   }
	lappend mjdData($mjdobs) $seqId
	set mjdData($mjdobs,$seqId,name) $name
	}
   genericChainDel $chain
   if {![inSession]} {mtdbConnect}
   foreach mjd $mjdData() {
	if {[llength $mjdData($mjd)] <= 0} continue
	if {[catch {set frames [mtframesGet $mjd]} msg]} {
	   echo No mtFrames for mjd $mjd
	   continue
	   }
	set itr [objNew ooItr(arMTField) -transient]
	set yn [$frames -> mTFields $itr]
	while {$yn} {
	   set seq [$itr -> mTSequence]
	   set seqId [$seq -> seqId]
	   objDel $seq
	   if {[lsearch $mjdData($mjd) $seqId] >= 0} {
		set frame [$itr -> frame 0]
		set sky [$frame sky]
		set usky [$sky value]
		handleDel $sky
		handleDel $frame
		set mjdData($mjd,$seqId,sky) $usky
		}
	   set yn [$itr next]
	   }
	handleDel $itr
	objDel $frames
	}

#Fill in missing patches from FITS files.  (Why not in mtdb?)
   patchSkyFetchFromFits
   return
   }

#########################################################################
#Fetch skies from FITS file in case where not in database.
proc patchSkyFetchFromFits {} {
   global mjdData mtDir mtDir
   if {![info exists mjdData]} {
	patchSkyFetch
	}
   foreach mjd $mjdData() {
	set file $mtDir/mjd${mjd}/mdReport-${mjd}.par
	set chains [param2Chain $file hdr]
	if {[info exists expChain]} {unset expChain}
	foreach ch $chains {
	   if {[chainTypeDefine $ch] == "EXP"} {
		set expChain $ch
	   } else {
		genericChainDel $ch
		}
	   }
	if {![info exists expChain]} {
	   echo No mdReport found for mjd $mjd
	   continue
	   }
	foreach seqId $mjdData($mjd) {
	   set subchain [chainSearch $expChain "{exposure = $seqId}"]
	   if {[chainSize $subchain] == 0} {
		echo No exposure for mjd $mjd sequence $seqId
		chainDel $subchain
		continue
		}
	   set hndl [chainGet $subchain 0]
	   chainDel $subchain
	   set mjdData($mjd,$seqId,mjd) [exprGet $hndl.mjd]
	   set mjdData($mjd,$seqId,ra) [exprGet $hndl.ra]
	   set mjdData($mjd,$seqId,dec) [exprGet $hndl.dec]
	   handleDel $hndl
	   if {[info exists mjdData($mjd,$seqId,sky)]} continue
	   set file $mtDir/mjd${mjd}/mtObj-[format %08d $seqId].fit
	   set chain [fits2Schema $file AAA]
	   set hndl [chainGet $chain 0]
	   set filter [exprGet $hndl.filter]
	   set sky [exprGet $hndl.sky]
	   if {$filter != "u"} {echo Whoops, filter no u!}
	   set mjdData($mjd,$seqId,sky) $sky
	   handleDel $hndl
	   genericChainDel $chain
	   }
	genericChainDel $expChain
	}
   return
   }
#####################################################################
#Analyze skies

proc patchSkyList {} {
   global mjdData patchData
   if {![info exists mjdData]} {patchSkyFetch}
   if {[info exists patchData]} {unset patchData}
   set patchData() ""
   set skies ""
   foreach mjd $mjdData() {
	foreach seqId $mjdData($mjd) {
	   if {![info exists mjdData($mjd,$seqId,sky)]} {
		echo No sky for mjd $mjd, seqId $seqId, patch \
		   $mjdData($mjd,$seqId,name)
		continue
		}
	   set sky [format %.2f $mjdData($mjd,$seqId,sky)]
	   set name $mjdData($mjd,$seqId,name)
	   lappend skies $sky
	   if {[lsearch $patchData() $name] < 0} {
		lappend patchData() $name
		set patchData($name) ""
		}
	   lappend patchData($name) $sky
	   }
	}
   return $skies
   }

##########################################################################
#Analyze patch sky data.  If a patch has no good observation with sky <
#threshold, flag it for reobservation.

proc patchSkyAnalyze {thresh} {
   global mjdData patchData
   if {![info exists patchData]} {
	patchSkyList
	}
   set patchList ""
   foreach name $patchData() {
	if {[llength $patchData($name)] == 0} {
	   echo No data for $name
	   lappend patchList $name
	   continue
	   }
	if {[llength $patchData($name)] == 1} {
	   set sky $patchData($name)
	} else {
	   set sky [eval min $patchData($name)]
	   }
	if {$sky > $thresh} {
	   lappend patchList $name
	   }
	}
   set patchList [lsort $patchList]
   set fid [open patchList w]
   loop i 0 [llength $patchList] 10 {
	loop j $i [min [expr $i+10] [llength $patchList]] {
	   puts $fid [format %-8s [lindex $patchList $j]] nonewline
	   }
	puts $fid ""
	}
   close $fid
   return [lsort $patchList]
   }

######################################################################
#Correlate u band brightness with predicted sky brightness from moon.

proc patchSkyMoon {} {
   global mjdData
   pgEnv 30 15 0 300 0 0
   foreach mjd $mjdData() {
	foreach seqId $mjdData($mjd) {
	   if {![info exists mjdData($mjd,$seqId,sky)]} {
		echo No sky for mjd $mjd, seqId $seqId, patch \
		   $mjdData($mjd,$seqId,name)
		continue
		}
	   set sky [format %.2f $mjdData($mjd,$seqId,sky)]
	   set list [exec moonSkyMag $mjdData($mjd,$seqId,mjd) \
		$mjdData($mjd,$seqId,ra) $mjdData($mjd,$seqId,dec)]
	   set bright [keylget list bright]
	   set ramoon [keylget list ramoon]
	   if {$bright == 0.} {set bright [expr 25. + [random 50]/10.]}
	   pgPoint $bright $sky 1
	   }
	}
   return
   }
