#Photometry in a circular aperture

proc circle {reg {rad 5}} {
   imgGlobals
   if {![info exists img(sky)]} {error "Need some sky first"}
   if {![info exists img(axes)]} {error "Need some axes first"}
   set row [exprGet $img(axes).row]
   set col [exprGet $img(axes).col]
   set list [imgCircle $reg $row $col $rad]
   set totcnts [lindex $list 0]
   set pixels [lindex $list 1]
   set skycnts [expr $img(sky)*$pixels]
   set img(phototot) $totcnts
   set img(photonet) [expr $totcnts-$skycnts]
   set img(photoarea) $pixels
   return [format "Total %10.2f, Sky %10.2f, Net %10.2f" \
	$totcnts $skycnts $img(photonet)]
   return
   }

#####################################################################
#Photometry in a rectangle.

proc rectangle {reg {xmax 5} {ymax 4}} {
   imgGlobals
   if {![info exists img(sky)]} {error "Need some sky first"}
   if {![info exists img(axes)]} {error "Need some axes first"}
   set row [exprGet $img(axes).row]
   set col [exprGet $img(axes).col]
   set ang [exprGet $img(axes).pos_maj]
   set list [imgBoxPhoto $reg $row $col $ang $xmax $ymax]
   set totcnts [lindex $list 0]
   set pixels [lindex $list 1]
   set skycnts [expr $img(sky)*$pixels]
   set img(phototot) $totcnts
   set img(photonet) [expr $totcnts-$skycnts]
   set img(photoarea) $pixels

#Compute noise if readnoise, gain are provided
#Because atlas images have fake sky level, we use explicit skynoise
#to get the combined sky photon noise plus readnoise.
   if {[info exists sky(skynoise)] && [info exists img(gain)]} {
	set img(photonoise) [expr sqrt($img(photonet)/img(gain) + \
	   img(photoarea)*pow(sky(skynoise),2))]
   } else {
	set img(photonoise) 0
	}
   return [format "Total %10.2f, Sky %10.2f, Net %10.2f" \
	$totcnts $skycnts $img(photonet)]
   return
   }

