#Check log files vs. actual frames for spectro data for a given mjd
#Must run sdFix first to correct lots of bugs.  Use .out files from sdFix.

#Commands herein:
#	sdFix			Fix sdReport files in current directory.
#	sdppd [outfile]		Summary of sdReport.out files
#	logFrame [outfile]	Obsolete - looked at spectro data area
#
#Also, imaging commands
#       idFix [mjds]		Fix idReport files from parallel directories
#       imgppd [outfile]	Summary of idReport.out files  All mjds at
#				the moment.

###############################################################
#Input is a list of lists (many variations possible).
#Expans any range specifications.  Return a single list

global rootDir
set rootDir /data/dp30.a/data

set ddir [file dirname [info script]]
#source $ddir/seeing.tcl

proc fieldExpand {args} {
   set outlist ""
   foreach arg $args {
        foreach field $arg {
           if {[regexp {([0-9]+)-([0-9]+)} $field all f1 f2]} {
                loop i $f1 [expr $f2+1] {
                   lappend outlist $i
                   }
           } else {
                lappend outlist $field
                }
           }
        }
   return $outlist
   }

###############################################################
#Make summary listings of sdReport files.
#Cache info for use in planning other stuff

proc sdppd {{outfile ""}} {
   global specData
   if {[info exists specData]} {unset specData}
   set specData() ""
   if {$outfile == ""} {
	set stdout stdout
   } else {
	set stdout [open $outfile w]
	}

#Read opHdrFix.par if it exists
   if {[file exists opHdrFix.par]} {
	echo Reading opHdrFix.par
	set hdrChain [param2Chain opHdrFix.par hdr]
	chainEach hndl $hdrChain {
	   set root [exprGet $hndl.fileroot]
	   set key [exprGet $hndl.keyword]
	   set val [exprGet $hndl.value]
	   if {![regexp {sdR-..-(.+)} $root all root]} {
		echo Can't decode root $root
		continue
		}
	   set roots $root

#Expand ranges if present
	   if {[regexp {(.+)\[([0-9])-([0-9])} $root all prefix l1 l2]} {
		set roots ""
		for {set l $l1} {$l <= $l2} {incr l} {
		   lappend roots $prefix$l
		   }
		}
	   foreach root $roots {
		set exposure [string trimleft $root 0]
		set ${key}($exposure) $val
echo $exposure $key $val
		}
	   }
	genericChainDel $hdrChain
	}

   foreach file [glob -nocomplain sdReport*.out] {
	set sdReport $file

	regexp {sdReport-([0-9]+)} $sdReport all mjd
#After looking at stuff with Annis, we find that the files names on disk
#are wrong; the exposure and sequence numbers look OK so far.

	set chains [param2Chain $sdReport hdr]
	foreach chain $chains {
	   if {[chainTypeDefine $chain] == "SEXP"} {
		set expChain $chain
		break
		}
	   }
	if {![info exists expChain]} {error "Can't find a SEXP chain!"}
	set names "seqId plateId mapId exposure expTime flavor camera"
	if {[info exists exp]} {
	   unset exp
	   unset seq
	   unset map
	   unset expTime
	   }
	chainEach hndl $expChain {
	   foreach name $names {
		set $name [exprGet $hndl.$name]
		lappend list [set $name]
		}
	   if {$flavor != "target" && $flavor != "science"} continue
	   if {$camera != "b1"} continue

	   if {[info exists PLATEID($exposure)]} {
echo Overriding plateId for $exposure
		set plateId $PLATEID($exposure)
		}
	   if {[info exists MAPID($exposure)]} {
echo Overriding mapId for $exposure
		set mapId $MAPID($exposure)
		}

#Assemble a list of plates and sequences and maps
	   if {![info exists exp($plateId)]} {
		set exp($plateId) $exposure
		set exptime($plateId) $expTime
	   } else {
	       if {[lsearch $exp($plateId) $exposure] >= 0} continue
		lappend exp($plateId) $exposure
		incr exptime($plateId) $expTime
		}

	   if {![info exists seq($plateId)]} {
		set seq($plateId) $seqId
	   } else {
	       if {[lsearch $seq($plateId) $seqId] >= 0} continue

#It appears that if the sequence number is different, it was just bumped
#up.  I will presume that we a plate only once per night.
#I can do better by looking for changes in plateId to identify a new
#sequence but that code is not in here yet.
#In the meantime, I will not track multiple sequence numbers.
#		lappend seq($plateId) $seqId
		}

	   if {![info exists map($plateId)]} {
		set map($plateId) $mapId
	   } else {
	       if {[lsearch $map($plateId) $mapId] >= 0} continue
		lappend map($plateId) $mapId
		}
	   }
	puts $stdout "MJD $mjd"
	puts $stdout "plateId nexp exptim   mapid    sequences  exposures"
	foreach plateId [lsort -integer [array names exp]] {
	   puts $stdout [format "%4d   %2d    %4.0f     %s        %s        %s" \
		$plateId [llength \
		$exp($plateId)] $exptime($plateId) $map($plateId) $seq($plateId) $exp($plateId)]

#Cache data
	   if {[lsearch $specData() $mjd] < 0} {
		lappend specData() $mjd
		set specData($mjd) ""
		}

#I will  once again assume I observe a plate only once per night.
#I will track observing by plateId, not mapId
	   if {![info exists specData($mjd)]} {
		set specData($mjd) $plateId
	   } else {
		lappend specData($mjd) $plateId
		}
	   set specData($mjd,$plateId,mapId) $map($plateId)
	   set specData($mjd,$plateId,seqId) $seq($plateId)
	   set specData($mjd,$plateId,exptime) $exptime($plateId)
	   set specData($mjd,$plateId,exposure) $exp($plateId)
	   }
	foreach chain $chains {
	   genericChainDel $chain
	   }
	puts $stdout ""
	}
   if {$outfile != ""} {close $stdout}
   return
   }

######################################################################
#Make a list of all expsoures as found in all data directories in
#the spectro working area, /data/dp2.a/spectro.  Obsolete command.

proc logFrame {{outfile ""}} {
   if {$outfile == ""} {
	set stdout stdout
   } else {
	set stdout [open $outfile w]
	}
   foreach file [glob -nocomplain sdReport*.out] {
	set sdReport $file
	regexp {sdReport-([0-9]+)} $sdReport all mjd
	logFrameMJD $mjd $stdout
	}
   if {$outfile != ""} {close $stdout}
   return
   }
######################################################################
#Check log files from actual frames for spectro data for a given mjd

proc logFrameMJD {mjd stdout} {
   set dataDir /data/dp2.a/spectro/$mjd
   if {![file exists $dataDir]} {
	echo No data for $dataDir
	return
	}
   foreach file [glob -nocomplain $dataDir/sdR-01-*.fit] {

#After looking at stuff with Annis, we find that the files names on disk
#are wrong; the exposure and sequence numbers look OK so far.

	set hdr [hdrNew]
	hdrReadAsFits $hdr $file
	set names "MJD SEQID EXPOSURE FLAVOR CAMERAS"
	foreach name "MJD SEQID PLATEID EXPOSURE" {
		set [string tolower $name] [hdrGetAsInt $hdr $name]
		}
	foreach name "FLAVOR CAMERAS" {
		set [string tolower $name] [string trim [hdrGetAsAscii \
		   $hdr $name]]
		}
	foreach name "EXPTIME" {
		set [string tolower $name] [hdrGetAsDbl $hdr $name]
		}
	if {$exptime < 0.} {
	   echo Bad exptime: exposure=$exposure, exptime=$exptime
	   set exptime 900
	   }
	if {$flavor != "target" && $flavor != "science"} continue
	if {$cameras != "b1"} continue
#Assemble a list of plates
	if {![info exists exp($plateid)]} {
		set exp($plateid) $exposure
		set exptim($plateid) [int $exptime]
	} else {
	       if {[lsearch $exp($plateid) $exposure] >= 0} continue
		lappend exp($plateid) $exposure
		incr exptim($plateid) [int $exptime]
		}
	}
   puts $stdout "MJD $mjd"
   puts $stdout "plateId nexp exptim   exposures"
   foreach plateId [lsort -integer [array names exp]] {
	puts $stdout [format "%4d   %2d    %4.0f     %s" $plateId [llength \
		$exp($plateId)] $exptim($plateId) $exp($plateId)]
	}
   puts $stdout ""
   return
   }

##################################################################
#Fix up lines with bad program (not embedded in quotes).
proc sdFix {{mjds ""}} {
   global rootDir
   set datadir $rootDir/ppd
   if {$mjds == ""} {
	set files [glob -nocomplain $datadir/*/sdReport-*.par]
echo $files
   } else {
	set mjds [fieldExpand $mjds]
	set files ""
	foreach mjd $mjds {
	   set dir $datadir/$mjd
	   set file [glob -nocomplain \
	       $dir/sdReport-*.par]
	   if {$file != ""} {lappend files $file}
	   }
       }
   foreach file $files {
	echo $file
	set tail [file tail $file]
	if {![file exists $tail]} {
	   exec cp $file ./$tail
	   }
	set file $tail
	set pre [file root $file]
	set out $pre.out
	set inid [open $file]
	set outid [open $out w]
	while {1} {
	   set line [gets $inid]
	   if {[eof $inid]} break
	   regsub " binning -9999  -9999 " $line "\"binning 1 1\" " line
	   regsub " binning 1   1 " $line "\"binning 1 1\" " line

#Embedded quotes in sdcomment
	   if {[regexp {^(sdcomment[^"]+)(".+")$} $line all prefix body]} {
		set line "$prefix [list $body]"
		}
	   puts $outid $line
	   }
	close $outid
	close $inid
	}
   return
   }

##################################################################
#Fix up idReport files (bad comment lines).
#Input desired mjds, or work on all directories
proc idFix {{mjds ""}} {
   global rootDir
   set datadir $rootDir/ppd
   if {$mjds == ""} {
	set files [glob \
	   $datadir/*/idReport-\[0-9\]\[0-9\]\[0-9\]\[0-9\]\[0-9\].par]
   } else {
	set mjds [fieldExpand $mjds]
	set files ""
	foreach mjd $mjds {
	   set dir $datadir/$mjd
	   set file [glob -nocomplain \
	       $dir/idReport-\[0-9\]\[0-9\]\[0-9\]\[0-9\]\[0-9\].par]
	   if {$file != ""} {lappend files $file}
	   }
	}
   foreach file $files {
	echo $file
	set pre [file root [file tail $file]]
	set out $pre.out
	set inid [open $file]
	set outid [open $out w]
	while {1} {
	   set line [gets $inid]
	   if {[eof $inid]} break
	   regsub {^(typedef.*COMMENT)$} $line \
{typedef struct { int run; int frame; char comment[240]; } COMMENT} line
	   if {[lindex $line 0] == "comment" && [llength $line] == 3} {
		set line [list [lindex $line 0] [lindex $line 1] 1 \
		[lindex $line 2]]
		}
	   regsub -all {{}} $line -9999 line
	   puts $outid $line
	   }
	close $outid
	close $inid
	}
   return
   }

######################################################################
#Same as ppd, but for imaging data
#
proc imgppd {{mjds ""} {outfile ""}} {
   global rootDir
   set datadir $rootDir/ppd
   if {$outfile == ""} {
	set stdout stdout
   } else {
	set stdout [open $outfile w]
#Create a .par file as well
	set parfile [file root $outfile].par
	set parid [open $parfile w]
	puts $parid "typedef struct {"
	puts $parid "   int run;"
	puts $parid "   int mjd;"
	puts $parid "   int stripe;"
	puts $parid "   char strip<2>;"
	puts $parid "   float slongStart;"
	puts $parid "   float slongEnd;"
	puts $parid "   float muStart;"
	puts $parid "   float muEnd;"
	puts $parid "   float seeing;"
	puts $parid "   int firstFrame;"
	puts $parid "   int lastFrame;"
	puts $parid "} RUN;"
	}

   if {$mjds == ""} {
	set files [glob \
	   $datadir/skent/idReport-\[0-9\]\[0-9\]\[0-9\]\[0-9\]\[0-9\].out]
   } else {
	set mjds [fieldExpand $mjds]
	set files ""
	foreach mjd $mjds {
	   set dir $datadir/skent
	   set file [glob -nocomplain $dir/idReport-$mjd.out]
	   if {$file != ""} {lappend files $file}
	   }
	}

   foreach file [lsort $files] {
	set idReport $file

	regexp {idReport-([0-9]+)} $idReport all mjd
#After looking at stuff with Annis, we find that the files names on disk
#are wrong; the exposure and sequence numbers look OK so far.

	set chains [param2Chain $idReport hdr]
	if {[info exists expChain]} {unset expChain}
	foreach chain $chains {
	   if {[chainTypeDefine $chain] == "IMAGINGRUN"} {
		set expChain $chain
		break
		}
	   }
	if {![info exists expChain]} {
	   echo "Can't find an IMAGINRUN chain!"
	   continue
	   }
	set names "stripe strip run flavor node incl muStart lastFrame mark"
	if {[info exists exp]} {
	   unset exp
	   unset expTime
	   }
	set runs ""
	if {[info exists data]} {unset data}
	chainEach hndl $expChain {
	   foreach name $names {
		set $name [string trim [exprGet -enum $hndl.$name]]
		lappend list [set $name]
		}
	   if {[lsearch $runs $run] < 0} {
		lappend runs $run
		}
	   foreach name $names {
		set data($run,$name) [set $name]
		}
	   }
	foreach chain $chains {
	   genericChainDel $chain
	   }
	if {[llength $runs] == 0} continue
	set first 0
	foreach run $runs {
	   if {$data($run,flavor) != "science" && \
		$data($run,flavor) != "calibration"} continue
	   if {$data($run,mark) != "END"} {
		puts $stdout ""
		puts $stdout  "Caution: No END record found for next run"
		set data($run,lastFrame) 0
		}

#Read idFrameLog for one set of CCDs
	   if {[catch {idFrameLogFix $mjd $run} msg]} {
		echo Problem reading idFrameLog for mjd $mjd, run $run:
		echo $msg
		echo Continuing ...
		continue
		}
	   set file idFrameLog-[format %06d $run]-01.out
	   set frameChain [param2Chain $file hdr]
	   loop i 0 [chainSize $frameChain] {
		set hndl1 [chainGet $frameChain $i]
		set ra1 [exprGet $hndl1.ra]
		set dec1 [exprGet $hndl1.dec]
		set frame1 [exprGet $hndl1.frame]
		handleDel $hndl1
		if {$ra1 >= 0} break
		}
	   loop i 1 [chainSize $frameChain] {
		set hndl2 [chainGet $frameChain $i]
#Guard against bogus ra, dec values when tracking stops
		if {[exprGet $hndl2.ra] < 0} {
		   handleDel $hndl2
		   continue
		   }
		set ra2 [exprGet $hndl2.ra]
		set dec2 [exprGet $hndl2.dec]
		set frame2 [exprGet $hndl2.frame]
		handleDel $hndl2
		}
	   if {$data($run,stripe) == 82 || $data($run,stripe) == 10} {
		set dec1 0
		set dec2 0
		}
	   set coords1 [eqToSurvey $ra1 $dec1]
	   set coords2 [eqToSurvey $ra2 $dec2]
	   set lambda1 [keylget coords1 lambda]
	   set lambda2 [keylget coords2 lambda]

#Get great circle coordinates also!
	   set gcoord1 [eqToGC $ra1 $dec1 -node $data($run,node) \
		-inclination $data($run,incl)]
	   set gcoord2 [eqToGC $ra2 $dec2 -node $data($run,node) \
		-inclination $data($run,incl)]
	   set mu1 [keylget gcoord1 mu]
	   set mu2 [keylget gcoord2 mu]
	   genericChainDel $frameChain

#Compute seeing
	   set seeing ""
	   catch {set seeing [seeing $mjd $run]}
	   if {$seeing == ""} {set seeing 0} 
	   if {$first == 0} {
		puts $stdout ""
		puts $stdout "MJD $mjd"
		puts $stdout "run stripe strip slongStart slongEnd muStart muEnd  seeing firstFrame lastFrame"
		set first 1
		}
#Use 1st, last frame numbers from idFrameLog instead of idReport.
	   puts $stdout [format \
"%-7s %-4s %-6s %-7.0f %-7.0f %-8.0f %-8.0f %-6.2f %-5s %-8s" \
		$run $data($run,stripe) $data($run,strip) $lambda1 $lambda2 \
		$mu1 $mu2 $seeing $frame1 $frame2]
	   if {[info exist parid]} {
		puts $parid [format \
		"RUN %s %s %s %s %.0f %.0f %.0f %.0f %.2f %s %s" \
		$run $mjd $data($run,stripe) $data($run,strip) $lambda1 \
		$lambda2 $mu1 $mu2 $seeing $frame1 $frame2]
#		$run $mjd $data($run,stripe) $data($run,strip) $lambda1 \
#		$lambda2 $mu1 $mu2 $seeing $data($run,lastFrame)]
		}
	   }
	}
   if {$outfile != ""} {close $stdout}
   if {[info exists parid]} {close $parid}
   return
   }

##################################################################
#Fix up idFrameLog files (bad comment lines).  Look in parallel directories.
proc idFrameLogFix {mjd run} {
   global rootDir
   set datadir $rootDir/ppd/$mjd
   set file $datadir/idFrameLog-[format %06d $run]-01.par
#	echo $file
	set pre [file root [file tail $file]]
	set out $pre.out
	set inid [open $file]
	set outid [open $out w]
        set oldframe 0
	while {1} {
	   set line [gets $inid]
	   if {[eof $inid]} break
	   if {[regexp {^frameinfo[ ]+([0-9]+)} $line all frame]} {
		if {$frame < $oldframe} {
		   echo Mutiple copies of idFrameLog for run $run
		   break
		   }
		set oldframe $frame
		}
	   puts $outid $line
	   }
	close $outid
	close $inid
   return
   }
#####################################################################
#Dammit, put out stripe limits in GC coords

proc GCLimits {args} {
   foreach stripe $args {
	set limits [stripeLimits $stripe]
	set lambdaMin [keylget limits lambdaMin]
	set lambdaMax [keylget limits lambdaMax]
	set eta [keylget limits eta]
	set node 95
	set incl [expr $eta+32.5]
	set gcoord1 [surveyToGC $lambdaMin $eta -node $node -inclination $incl]
	set gcoord2 [surveyToGC $lambdaMax $eta -node $node -inclination $incl]
	set mu1 [format %.0f [keylget gcoord1 mu]]
	set mu2 [format %.0f [keylget gcoord2 mu]]
	set nu1 [format %.0f [keylget gcoord1 nu]]
	set nu2 [format %.0f [keylget gcoord2 nu]]
	puts stdout "Stripe $stripe mu1=$mu1 mu2=$mu2"
	}
   return
   }

#Check log files vs. actual frames for spectro data for a given mjd
#Must run sdFix first to correct lots of bugs.  Use .out files from sdFix.

#Commands herein:
#	sdFix			Fix sdReport files in current directory.
#	sdppd [outfile]		Summary of sdReport.out files
#	logFrame [outfile]	Obsolete - looked at spectro data area
#
#Also, imaging commands
#       idFix [mjds]		Fix idReport files from parallel directories
#       imgppd [outfile]	Summary of idReport.out files  All mjds at
#				the moment.

###############################################################
#Input is a list of lists (many variations possible).
#Expans any range specifications.  Return a single list

set ddir [file dirname [info script]]
#source $ddir/seeing.tcl

proc fieldExpand {args} {
   set outlist ""
   foreach arg $args {
        foreach field $arg {
           if {[regexp {([0-9]+)-([0-9]+)} $field all f1 f2]} {
                loop i $f1 [expr $f2+1] {
                   lappend outlist $i
                   }
           } else {
                lappend outlist $field
                }
           }
        }
   return $outlist
   }

######################################################################
#On an unrelated note ... check on a range of fields for the Roy William
#error

proc williams {run rerun camcol filter fields} {
   global rootDir
   set dataDir $rootDir/$run/$rerun/corr/$camcol
    if {![file exists $dataDir]} {
	error "No directory $dataDir"
	}
   set fields [fieldExpand $fields]
   foreach field $fields {
	set file fpC-[format %06d $run]-$filter$camcol-[format \
	   %04d $field].fit
	if {![file exists $dataDir/$file.gz]} {
	   error "No file $file"
	   }
	exec cp $dataDir/$file.gz .
	exec chmod +w ./$file.gz
	exec gunzip $file.gz

	set hndl [regNew]
	regReadAsFits $hndl $file
	exec rm $file
	set line [hdrGetLine $hndl.hdr FIELD]
	if {[regexp NA $line]} {
	   echo Bad FIELD for field $field: [string trim $line]
	   }
	regDel $hndl
	}
   return
   }


