proc profNew {} {
   return [genericNew PROF]
   }

proc profDel {hndl} {
   if {[handleType $hndl] != "PROF"} then {error "Not a handle to a PROF"}
   genericDel $hndl
   }

proc profile {reg rmin rmax {nderiv 0}} {
   imgGlobals
   if {[info exists img(profile)]} {
	catch {genericChainDel $img(profile)}
	}
   if {![info exists img(axes)]} {error "Need some axes"}
   if {![info exists img(sky)]} {error "Need some sky"}
   set hndl [imgProfile $reg $img(axes) $rmin $rmax $nderiv $img(sky)]
   set img(profile) $hndl
   return $hndl
   }

#Plot contours of a PROFILE in an saoDisplay

proc contour {chan skip} {
   imgGlobals
   if {![info exists saoNumber($chan)]} {error "Bad channel"}
   if {[lsearch [saoGetRegion] $saoNumber($chan)] < 0} {error "Bad channel"}
   if {![info exists img(profile)]} return
   chainInit $img(profile) iter
   set n 0
   saoLabel off $saoNumber($chan)
   while {1} {
	set hndl [chainNext iter]
	if {$hndl == ""} break
	incr n
	if {$n < $skip} continue
	set n 0
	set xc [int [exprGet $hndl.xc]]
	set yc [int [exprGet $hndl.yc]]
	set ecc [exprGet $hndl.ecc]
	set ang [int [exprGet $hndl.ang]]
	set maj [exprGet $hndl.radius]
	set min [expr $maj*sqrt(1.-$ecc*$ecc)]
	saoDrawEllipse $xc $yc [int $maj] [int $min] $ang -i \
	   -s $saoNumber($chan)
	}
   return
   }

########################################################################
#Procs to make manipulate profiles.

#Compute sums
proc profSum {} {
   global img
   loop i 0 [chainSize $img(profile)] {
	if {$i == 0} {
	   set hndl [chainGet $img(profile) 0]
	   handleSet $hndl.cum 0.
	   handleDel $hndl
	   continue
	   }
	set hndl0 [chainGet $img(profile) [expr $i-1]]
	set hndl1 [chainGet $img(profile) $i]
	set rad0 [exprGet $hndl0.rad]
	set rad1 [exprGet $hndl1.rad]
	set cnts0 [expr [exprGet $hndl0.cnts] - $img(sky)]
	set cnts1 [expr [exprGet $hndl1.cnts] - $img(sky)]
	set cum0 [exprGet $hndl0.cum]
	set cum1 [expr $cum0 + .5*($cnts0+$cnts1)*3.14159 * \
	   (pow($rad1,2)-pow($rad0,2))]
	handleSet $hndl1.cum $cum1
	handleDel $hndl0
	handleDel $hndl1
	}
   return
   }

###################################################################
#Convert to log
proc profLog {} {
   global img
   chainScale $img(profile) cnts [expr -1.*$img(sky)] 1
   chainEach hndl $img(profile) {
	set cnts [exprGet $hndl.cnts]
	if {$cnts > 0} {handleSet $hndl.mag [expr log10($cnts)]}
	set cum [exprGet $hndl.cum]
	if {$cum > 0} {handleSet $hndl.cummag [expr log10($cum)]}
	}
   return
   }

##########################

#Match 2 chains between specified radii; compute scaling inclusive
proc profScale {chain1 chain2 irad1 irad2} {
   global img
   set avg 0
   set n 0
   loop i $irad1 [expr $irad2+1] {
       set hndl1 [chainGet $chain1 $i]
       set hndl2 [chainGet $chain2 $i]
       set cnts1 [expr [exprGet $hndl1.cnts] - $img(sky)]
       set cnts2 [expr [exprGet $hndl2.cnts] - $img(sky)]
	set ratio [expr $cnts1/$cnts2]
	incr n
	set avg [expr $avg+$ratio]
	handleDel $hndl1
	handleDel $hndl2
	}
   set median [expr $avg/$n]
   return $median
   }

############################################
#Merge 2 chains.  We include irad elements from chain1
#New chain is just points to objects in original chains.

proc profMerge {chain1 chain2 irad} {
   set type [chainTypeDefine $chain1]
   set outChain [chainNew $type]
   loop i 0 $irad {
	chainAdd $outChain [chainGet $chain1 $i]
	}
   loop i $irad [chainSize $chain2] {
	chainAdd $outChain [chainGet $chain2 $i]
	}
   return $outChain
   }

