global dataRoot
set dataRoot /data/dp30.a/data

#Extract seeing statistics for a run
### Useful commands:
### seeingRead run - reads seeing.  Possible sources are a file seeing-run.tcl
###	or psField files.  masterdir is hardwired for now.
### seeingWrite run - write out seeing array to seeing-run.tcl file
### seeingStat run - computes median seeing for each CCD in 10 field
###	stretches
### seeingJudge run - computes median of medians from seeingStat combining
###	5 groups, 6 camcols, 5 filters.  Applied Kolmogorov correction.
###	Also computes worst median of all groups, camcols, filters.
###	Assigns quality to each group of fields based on these 2 stats.
### seeingJudgeSummary - determines what fraction of all fields are good
###	and bad
### seeingJudgeDetail run - Expands results of seeingJudge and assigns good/bad
###	quality to each field in a run
### seeingJudgePlot run - makes a plot of median per field, min per field,
###	and max per field.  Note that these values are not quite the same
###	as were used by seeingJudge (they are computed field by field, not
###	grouping 10 fields together).
### seeingJudgePlotGif run - makes a gif version of above.
### seeingHtml - find all seeing.gif files and make an html link page.
### seeingPlot run camcol filter - plot seeing for specified CCD
### seeingQualityWrite run - write out qaSeeing file with seeing, quality.
### seeingHist - make differential & cumulative histograms.  All runs
### runningHist - groups fields & computes histogram relative to median
### seeingHistPlot
### cumHistPlot
### astroRead run - read astroline files
### astroCopy     - copy astroline seeing to seeing and scale to match psp.
### seeingCompare run
### Various aperture correction reading & plotting routines.

proc seeingRead {run} {
   global seeing
   if {![info exists seeing()]} {
	set seeing() ""
	}
   if {[lsearch $seeing() $run] < 0} {lappend seeing() $run}
   if {[file exists seeing-$run.tcl]} {
	echo Reading seeing-$run.tcl
	source seeing-$run.tcl
	return
	}
   foreach camcol "1 2 3 4 5 6" {
	seeingReadCol $run $camcol
	}
   return
   }

#####################################################################
#Cache seeing results to a file
proc seeingWrite {run} {
   global seeing
   set fid [open seeing-$run.tcl w]
   puts $fid "set seeing($run) \"1 2 3 4 5 6\""
   foreach camcol $seeing($run) {
	puts $fid "set seeing($run,$camcol) \"$seeing($run,$camcol)\""
	foreach field $seeing($run,$camcol) {
	   foreach filter "u g r i z" {
		puts $fid "set seeing($run,$camcol,$field,$filter) \
		   $seeing($run,$camcol,$field,$filter)"
		}
	   }
	}
   close $fid
   }

#####################################################################

proc seeingReadCol {run camcol} {
   mtGlobals
   set topdir $masterdir
   global seeing
   set dir $topdir/$run/objcs/$camcol
   set files [lsort [glob $dir/psField*.fit]]
   if {![info exists seeing($run)]} {
	set seeing($run) ""
	}
   if {[lsearch $seeing($run) $camcol] < 0} {lappend seeing($run) $camcol}
   set seeing($run,$camcol) ""
   foreach file $files {
#echo $file
	if {![regexp {.+-([0-9]+)\.fit} $file all field]} continue
	set field [string trimleft $field 0]
echo $run $camcol $field
	lappend seeing($run,$camcol) $field
	seeingReadField $run $camcol $field
	}
   return
   }

#############################################################
proc seeingReadField {run camcol field} {
   mtGlobals
   global seeing
   set topdir $masterdir
   set dir $topdir/$run/objcs/$camcol
   set file \
      $dir/psField-[format %06d $run]-$camcol-[format %04d $field].fit
   set chain [fits2Schema $file PPP -hdu 6]
   set hndl [chainGet $chain 0]
   loop i 0 5 {
	set filter [lindex "u g r i z" $i]
	set fwhm [exprGet $hndl.psf_width<$i>]
	set seeing($run,$camcol,$field,$filter) $fwhm
	}
   handleDel $hndl
   genericChainDel $chain
   return
   }

#####################################################################

set nsmooth 10

set medlim 1.7
set maxlim 2.0
set gradlim .2

set scale(u) .89
set scale(g) .95
set scale(r) 1
set scale(i) 1.04
set scale(z) 1.04

proc seeingStat {run} {
   global median seeing nsmooth
   if {![info exists median()]} {set median() ""}
   if {[lsearch $median() $run] < 0} {lappend median() $run}
   set median($run) ""
   foreach camcol "1 2 3 4 5 6" {
	set median($run,$camcol) ""
	set fields $seeing($run,$camcol)
	if {[llength $fields] == 0} continue
	lappend median($run) $camcol
	set firstfield [lindex $fields 0]
	set lastfield [lindex $fields end]

#Sometime a field gets dropped.  Check, and if so, stick in fake values
	set nfield [expr $lastfield-$firstfield+1]
	if {$nfield != [llength $fields]} {
	   foreach filter "u g r i z" {
		loop i $firstfield [expr $lastfield+1] {
		   if {![info exists seeing($run,$camcol,$i,$filter)]} {
			echo Missing field $i, filter $filter
			set seeing($run,$camcol,$i,$filter) 3.
			}
		   }
		}
	   }

	foreach filter "u g r i z" {
	   loop i $firstfield [expr $lastfield+1] $nsmooth { 
		set list ""
		set jlist ""
		set gradlist ""
		loop j $i [min [expr $i+$nsmooth] [expr $lastfield+1]] {
		   lappend list $seeing($run,$camcol,$j,$filter)
		   lappend jlist $j
		   if {$j != $i} {
			set grad [format %.4f \
			   [expr abs($seeing($run,$camcol,$j,$filter) - \
			   $seeing($run,$camcol,[expr $j-1],$filter))]]
			lappend gradlist $grad
			}
		   }
		set list [lsort -real $list]
		set jlist [lsort -integer $jlist]
		set gradlist [lsort -real $gradlist]

#We tabulate median at median frame # of a string of frames.
		set jmed [lindex $jlist [expr \
		   [llength $jlist]/2]]
		if {$filter == "u"} {
		   lappend median($run,$camcol) $jmed
		   }
		set median($run,$camcol,$jmed,$filter) [lindex $list [expr \
		   [llength $list]/2]]
		set median($run,$camcol,$jmed,$filter,grad) \
			[lindex $gradlist [expr [llength $gradlist]/2]]
		}
	   }

	}
   }

#####################################################################
#Determine which segments of data are bad.  Must run seeingStat first.
#Go through medians, group by 5 fields of 10.  Take median of medians.
#Demand that median of medians not exceed 1.7 arcsec, and that no
#single point exceed 2.0 arcsec.  Otherwise, tag this group as bad.

proc seeingJudge {run} {
   global median quality scale
   global medlim maxlim gradlim

   set camcol1 [lindex $median($run) 0]
   if {![info exists quality()]} {set quality() ""}
   if {[lsearch $quality() $run] < 0} {lappend quality() $run}
   set quality($run) ""
   loop i 0 [llength $median($run,$camcol1)] {
	set medlist ""
	set max 0
	set maxgrad 0
	set g1index [max 0 [expr $i-2]]
	set g2index [min [expr [llength $median($run,$camcol1)]-1] \
		[expr $i+2]]
	set group [lindex $median($run,$camcol1) $i]
	lappend quality($run) $group
	foreach camcol $median($run) {
	   loop j $g1index $g2index {
		set g [lindex $median($run,$camcol1) $j]
		foreach filter "u g r i z" {
		   if {![info exists median($run,$camcol,$g,$filter)]} {
			echo Missing median for run $run, camcol $camcol, \
			   group $g, filter $filter
			continue
			}
		   set medscale [format %.4f [expr $scale($filter) * \
			$median($run,$camcol,$g,$filter)]]
		   lappend medlist $medscale
		   set max [max $max $medscale]
		   }
		}
	   }
	set medlist [lsort -real $medlist]
	set med [lindex $medlist [expr [llength $medlist]/2]]
	set quality($run,$group,med) $med
	set quality($run,$group,max) $max
	if {$med <= $medlim && $max <= $maxlim} {
	   set quality($run,$group) good
	} elseif {$max <= $maxlim} {
#	   echo Marginal run $run group $group, median = $med, max = $max
	   set quality($run,$group) acceptable
	} else {
#	   echo Reject run $run group $group, median = $med, max = $max
	   set quality($run,$group) bad
	   }
	}
   return
   }

#####################################################################
proc seeingJudgeSummary {} {
   global quality
   set ngood 0; set nbad 0
   foreach run $quality() {
	foreach group $quality($run) {
	   if {$quality($run,$group) == "good"} {
		incr ngood
	   } else {
		incr nbad
		}
	   }
	}
   echo Fraction of good is [format %.2f [expr $ngood*1.0/($ngood+$nbad)]]
   return
   }

   
#####################################################################
#After running seeingRead, seeingStat, seeingJudge, run this proc to
#assign a quality to each field.

proc seeingJudgeDetail {run} {
   global seeing quality scale

   if {![info exists seeing($run)]} {
	echo No data for run $run
	return
	}
   if {[llength $seeing($run)] < 6} {
	echo Less than 6 camcols
	return
	}

#Set up quality
   set fields ""
   foreach group $quality($run) {
	set f1 [expr $group-5]
	set f2 [expr $group+5]
	for {set field $f1} {$field <= $f2} {incr field} {
	   set quality($run,$field) $quality($run,$group)
	   lappend fields $field
	   }
	}

#Get better list of fields
   global seeing
   set quality($run) $seeing($run,3)
   return
   }

################################################################3
#Proc to run seeingStat, seeingJudge, seeingJudgeDetail
proc seeingRun {run} {
   seeingStat $run
   seeingJudge $run
   seeingJudgeDetail $run
   return
   }

################################################################3
#Proc to run seeingStat, seeingJudge, seeingJudgeDetail
proc seeingQualityWrite {run} {
   global seeing quality scale
   set file qaSeeing-$run.par
   echo File is $file
   set fid [open $file w]
   puts $fid "typedef struct {"
   puts $fid "   int field;"
   puts $fid "   float seeing;"
   puts $fid "   char quality<20>;"
   puts $fid "   } SEEINGQA"
   foreach field $seeing($run,3) {
	set seeingList ""
	if {![info exists quality($run,$field)]} continue
	if {![info exists seeing($run,1,$field,r)]} continue
	foreach camcol "1 2 3 4 5 6" {
	   if {[lsearch $seeing($run,$camcol) $field] < 0} {
		echo No data for camcol $camcol, field $field
		continue
		}
	   foreach filter "u g r i z" {
		set seeScale [expr $seeing($run,$camcol,$field,$filter) \
		   * $scale($filter)]
		lappend seeingList $seeScale
		}
	   }
	set seeingList [lsort -real $seeingList]
	set median [lindex $seeingList [expr [llength $seeingList]/2]]
	set median [format %.2f $median]
	puts $fid "seeingqa $field $median $quality($run,$field)"
	}
   close $fid
   return
   }

################################################################3
#Top level proc to make qa files
proc seeingQa {run} {
   seeingRead $run
   seeingRun $run
   seeingQualityWrite $run
   return
   }

################################################################3
#Must have run seeingJudgeDetail first.

proc seeingJudgePlotGif {run} {
   global seeing quality scale
#Plot
   if {[pgQinf STATE] == "OPEN"} {pgEnd}
   pgBegin seeing/seeing-$run.gif/GIF
   pgGeomSet 1200 400 10 25
   pgAsk 1
   seeingJudgePlot $run
   pgEnd
   return
   }

################################################################3
#Must have run seeingJudgeDetail first.

proc seeingJudgePlot {run} {
   global seeing quality scale
#Plot
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 1200 400 10 25; pgAsk 1
	}

   pgSci 1
#   pgEnv 0 800 0 2 0 0
#Kessler wants 3 arcsec limit.  Blech!
   pgEnv 0 800 0 3 0 0
   set mjd [runMjdGet $run]
   pgLabel Field "Median Seeing" "Run $run $mjd"
   pgSci 2
   pgLine "0 1000" "1.7 1.7"

   foreach field $seeing($run,3) {
	set seeingList ""
	if {![info exists quality($run,$field)]} continue
	if {![info exists seeing($run,1,$field,r)]} continue
	foreach camcol "1 2 3 4 5 6" {
	   if {[lsearch $seeing($run,$camcol) $field] < 0} {
		echo No data for camcol $camcol, field $field
		continue
		}
	   foreach filter "u g r i z" {
		set seeScale [expr $seeing($run,$camcol,$field,$filter) \
		   * $scale($filter)]
		lappend seeingList $seeScale
		}
	   }
	set seeingList [lsort -real $seeingList]
	set median [lindex $seeingList [expr [llength $seeingList]/2]]
	set low [lindex $seeingList 0]
	set high [lindex $seeingList end]
	if {$quality($run,$field) == "good"} {
	   pgSci 5
	} elseif {$quality($run,$field) == "acceptable"} {
	   pgSci 6
	} else {
	   pgSci 7
	   }
	pgPoint $field $median 3
	pgLine "$field $field" "$low $high"
	}
   return
   }


#####################################################################
#Make an html page
proc seeingHtml {} {

   set outfile seeingStatus.html
   set fid [open $outfile w]
   puts $fid "<head>\n"
   puts $fid "<title>seeingStatus</title>\n"
   puts $fid "</head>\n"
   puts $fid "<body>\n"
   puts $fid "<h1 align=center>Seeing Status by Run</h1>\n"
   set files [glob -nocomplain seeing/seeing*.gif]
   set runs ""
   foreach file $files {
	if {![regexp {seeing-([0-9]+).gif} $file all run]} continue
	lappend runs $run
	}
   foreach run [lsort -integer $runs] {
	set mjd [runMjdGet $run]
	puts $fid "<li>Run <a href=seeing/seeing-$run.gif>$run</a> $mjd\n"
	}
   puts $fid "</body>\n"
   close $fid
   return
   }

#####################################################################
proc runMjdGet {inrun} {
   global runmjd runstripe runstrip

#Read run.par, if it exists, to get mjd of each run
   if {[file exists run.par] && ![info exists runmjd]} {
	set chain [param2Chain run.par hdr]
	chainEach hndl $chain {
	   set run [exprGet $hndl.run]
	   set stripe [exprGet $hndl.stripe]
	   set strip [exprGet $hndl.strip]
	   set mjd [exprGet $hndl.mjd]
	   set runmjd($run) $mjd
	   set runstripe($run) $stripe
	   set runstrip($run) $strip
	   }
	genericChainDel $chain
	}
   if {[info exists runmjd($inrun)]} {
	return " (MJD $runmjd($inrun)) ($runstripe($inrun) $runstrip($inrun))"
   } else {
	return
	}
   }

#####################################################################
proc seeingPlot {run camcols filters} {
   global seeing median
   pgSlw 3
   pgSci 1
   pgEnv 0 [lindex $seeing($run,[lindex $camcols 0]) end] 1 2 0 0
   pgLabel Field FWHM "Run $run, camcol $camcols, filter $filters"
   foreach camcol $camcols {
	foreach filter $filters {
	   foreach field $seeing($run,$camcol) {
		pgSci 2
		pgSch .7
		pgPoint $field $seeing($run,$camcol,$field,$filter) 2
		pgSch 2
		if {![info exists median($run,$camcol,$field,$filter)]} \
			continue
		if {$median($run,$camcol,$field,$filter) == ""} continue
		pgSci 1
		pgPoint $field $median($run,$camcol,$field,$filter) 3
		pgSch 1
		}
	   pgSch 1
	   }
	}
   return
   }

#####################################################################
#Histogram the seeing accumulated so far
#seeingCum contains the cumulative frame counts.

proc seeingHist {} {
   global seeing seeingHist seeingCum
   global scale

#Bin by camcol,filter, and .1 increments in seeing out to 3 arcsec.
   foreach camcol "1 2 3 4 5 6" {
	for {set bin 0} {$bin <= 30} {incr bin} {
	   foreach filter "u g r i z" {
		set seeingHist($camcol,$filter,$bin) 0
		}
	   }
	}
   foreach run $seeing() {
	if {![info exists seeing($run)]} continue
	foreach camcol $seeing($run) {
	   foreach filter "u g r i z" {
		set median($filter) ""
		foreach field $seeing($run,$camcol) {
		   set bin [expr \
			round(10. * $scale($filter) * \
			   $seeing($run,$camcol,$field,$filter))]
		   if {$bin > 30} continue
		   incr seeingHist($camcol,$filter,$bin)
		   lappend median($filter) $bin
		   }
		set median($filter) [lindex [lsort -real $median($filter)] \
		   [expr [llength $median($filter)]/2]]
		echo Run $run Camcol $camcol Filter $filter \
		   Median $median($filter)
		}
	   }
	}

#Now compute cumulatives
#Bin by camcol,filter, and .1 increments in seeing out to 3 arcsec.
   foreach camcol "1 2 3 4 5 6" {
	foreach filter "u g r i z" {
	   set seeingCum($camcol,$filter,0) 0
	   }
	foreach filter "u g r i z" {
	   for {set bin 1} {$bin <= 30} {incr bin} {
		set seeingCum($camcol,$filter,$bin) [expr \
		   $seeingCum($camcol,$filter,[expr $bin-1]) + \
		   $seeingHist($camcol,$filter,$bin)]
		set n $seeingCum($camcol,$filter,$bin)
		}

#Normalize
	   if {$n == 0} {set n 1.}
	   for {set bin 0} {$bin <= 30} {incr bin} {
		set seeingCum($camcol,$filter,$bin) [expr \
		   $seeingCum($camcol,$filter,$bin)/($n*1.)]
		}
	   }

#Now merge filters
	for {set bin 0} {$bin <= 30} {incr bin} {	
	   set seeingCum($camcol,$bin) 0
	   foreach filter "u g r i z" {
		set seeingCum($camcol,$bin) [expr \
		   $seeingCum($camcol,$bin) + \
		   $seeingCum($camcol,$filter,$bin)/5.]
		}
	   }
	}

#Now merge columns
   for {set bin 0} {$bin <= 30} {incr bin} {
	set seeingCum($bin) 0
	foreach camcol "1 2 3 4 5 6" {
	   set seeingCum($bin) [expr \
		$seeingCum($bin) + $seeingCum($camcol,$bin)/6.]
	   }
	}
   return
   }

#####################################################################
#Histogram the seeing accumulated so far
#We divide runs into blocks of 10 fields or so, make histogram relative
#to median.  This will give me a feel for the typical scatter that we encounter
#in short blocks of time.

proc runningHist {} {
   global seeing seeingHist seeingCum scale

   set ngroup 50

   if {[info exists seeingHist]} {unset seeingHist}
#Bin by .1 increments between -1 and +1 arcsec.
   for {set bin -10} {$bin <= 10} {incr bin} {
	set seeingHist($bin) 0
	}

   foreach run $seeing() {
	if {![info exists seeing($run)]} continue
	foreach camcol $seeing($run) {
	   foreach filter "u g r i z" {
		set median($filter) ""
		loop i 0 [llength $seeing($run,$camcol)] $ngroup {

#First, compute median in group.
		   foreach field [lrange $seeing($run,$camcol) $i \
			[expr $i+$ngroup]] {
			set bin [expr \
			   round(10. * $scale($filter) * \
			   $seeing($run,$camcol,$field,$filter))]
			if {$bin > 30} continue
			lappend median($filter) $bin
			}
		   set m1 [lindex [lsort -real $median($filter)] \
			[expr [llength $median($filter)]/2]]
		   set m2 [lindex [lsort -real $median($filter)] \
			[expr ([llength $median($filter)]+1)/2]]
		   set median($filter) [expr ($m1+$m2)/2.]

#Next, increment histogram
		   foreach field [lrange $seeing($run,$camcol) $i \
			[expr $i+$ngroup]] {
			set bin [expr \
			   round(10. * $scale($filter) * \
			   $seeing($run,$camcol,$field,$filter) - \
			   $median($filter))]
			if {$bin < -10 || $bin > 10} continue
			incr seeingHist($bin)
			}
		   }
		}
	   }
	}

#Cumulative- compute cumulative to bin midpoint.
   if {[info exists seeingCum]} {unset seeingCum}
   set n 0
   for {set bin -10} {$bin <= 10} {incr bin} {
	set n [expr $n+$seeingHist($bin)]
	}
   set n [expr $n*1.0]
   set cum 0
   set seeingCum(-10) [expr $seeingHist(-10)/(2.*$n)]
   for {set bin -9} {$bin <= 10} {incr bin} {
	set cum [expr $cum + ($seeingHist([expr $bin-1]) + \
	   $seeingHist($bin))/(2.*$n)]
	set seeingCum($bin) [format %.3f $cum]
	}
   return
   }

#####################################################3
#Plot histogram

proc seeingHistPlot {camcol} {
   global seeingHist
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgAsk 0; pgGeomSet 400 400 10 25
	}
   pgSci 1
   pgSlw 3
   pgEnv 0 3 0 1000 0 0
   pgLabel "Seeing (arcsec)" "Number" "Seeing statistics for camcol \
$camcol"
   set ci(u) 4
   set ci(g) 5
   set ci(r) 3
   set ci(i) 7
   set ci(z) 2
#   foreach camcol "1 2 3 4 5 6" {
	foreach filter "u z g i r" {
	   pgSci $ci($filter)
	   for {set bin 0} {$bin <= 30} {incr bin} {
		set b1 [expr ($bin-.5)/10.]
		set b2 [expr ($bin+.5)/10.]
		set amp $seeingHist($camcol,$filter,$bin)
		pgLine "$b1 $b1 $b2 $b2" "0 $amp $amp 0"
		}
	   }
#	}
   return
   }

##################################################################
#Plot histogram

proc seeingCumPlot {} {
   global seeingHist seeingCum
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgAsk 0; pgGeomSet 400 400 10 25
	}
   pgSci 1
   pgSlw 3
   pgEnv 1 2.5 0 1 0 0
   pgLabel "Seeing (arcsec)" "Cumulative Frames" "Spring 2000"
   set ci(u) 4
   set ci(g) 5
   set ci(r) 3
   set ci(i) 7
   set ci(z) 2

   set ci(1) 2
   set ci(2) 3
   set ci(3) 5
   set ci(4) 5
   set ci(5) 3
   set ci(6) 2
   foreach camcol "1 2 3 4 5 6" {
	pgSci $ci($camcol)
	for {set bin 0} {$bin <= 30} {incr bin} {
	   set b1 [expr ($bin-.5)/10.]
	   set b2 [expr ($bin+.5)/10.]
	   set amp $seeingCum($camcol,$bin)
	   pgLine "$b1 $b1 $b2 $b2" "0 $amp $amp 0"
	   }
	}
   return
   }

########################################################################
#Read astroline files and decode seeing information
proc astroRead {run} {
   global aseeing
   global dataRoot

   if {![info exists aseeing()]} {set aseeing() ""}
   if {[lsearch $aseeing() $run] < 0} {lappend aseeing() $run}
   set aseeing($run) ""
   set fieldOff(u) 4
   set fieldOff(g) 8
   set fieldOff(r) 0
   set fieldOff(i) 2
   set fieldOff(z) 6

#Look for ppd directory
   set files [glob -nocomplain \
	$dataRoot/ppd/*/idAstroline-c1-[format %06d $run].par]
   if {$files != ""} {
	set dir [file dirname [lindex $files 0]]
   } else {
	set dir .
	}
   foreach camcol "1 2 3 4 5 6" {
	lappend aseeing($run) $camcol
	set aseeing($run,$camcol) ""
	set file $dir/idAstroline-c$camcol-[format %06d $run].par
	set chain [param2Chain $file hdr]
	chainEach hndl $chain {
	   set frame [exprGet $hndl.frame]
	   lappend aseeing($run,$camcol) $frame
	   foreach ifil "1 2 3 4 5" {
		set filter [lindex "x r i u z g" $ifil]

#FWHM is avg of major & minor axes; convert sigma in pixels to FWHM in arcsec
		set width [expr ([exprGet $hndl.rMajor$ifil] + \
		   [exprGet $hndl.rMinor$ifil])*.94/2.]
		set field [expr $frame - $fieldOff($filter)]
		set aseeing($run,$camcol,$field,$filter) [format %.2f $width]

#Set frame index seeing to 0 just for sanity
		if {![info exists aseeing($run,$camcol,$frame,$filter)]} {
		   set aseeing($run,$camcol,$frame,$filter) 0
		   }
		}
	   }
	genericChainDel $chain
	}
   return
   }

########################################################################
#Rename aseeing to seeing
proc astroCopy {} {
   global aseeing seeing
   if {![info exists aseeing]} {
	echo No aseeing found
	return
	}
   if {[info exists seeing]} {unset seeing}
   foreach elem [array names aseeing] {
	set seeing($elem) $aseeing($elem)
	}

#Rescale seeing
   foreach run $seeing() {
     foreach camcol "1 2 3 4 5 6" {
	foreach field $seeing($run,$camcol) {
	   foreach filter "u g r i z" {
		set seeing($run,$camcol,$field,$filter) [expr 1.09* \
		   $seeing($run,$camcol,$field,$filter)]
		}
	   }
	}
     }
   return
   }

########################################################################
#Compare astroline and psp.  Must have run seeingRead and astroRead first.

proc seeingCompareCF {run camcol filter} {
   global seeing aseeing
   set list ""
   
   foreach field $aseeing($run,$camcol) {
	if {![info exists aseeing($run,$camcol,$field,$filter)]} continue
	if {![info exists seeing($run,$camcol,$field,$filter)]} continue
	lappend list [expr $seeing($run,$camcol,$field,$filter) / \
	   $aseeing($run,$camcol,$field,$filter)]
	}
   set list [lsort -real $list]
   set m [lindex $list [expr [llength $list]/2]]
   return [format %.3f $m]
   }

########################################################################

proc seeingComparePlotCF {run camcol filter} {
   global seeing aseeing
   set list ""
   
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}
   pgSci 1
   pgEnv 0 [expr [llength $seeing($run,$camcol)+15]] 1 2 0 0
   pgLabel Field Seeing "Run $run"
   pgSch 2
   foreach field $aseeing($run,$camcol) {
	if {![info exists aseeing($run,$camcol,$field,$filter)]} continue
	if {![info exists seeing($run,$camcol,$field,$filter)]} continue
	pgSci 2
	pgPoint $field $aseeing($run,$camcol,$field,$filter) 2
	pgSci 4
	pgPoint $field $seeing($run,$camcol,$field,$filter) 5
	}
   pgSci 1
   pgSch 1
   return
   }

#####################################################################
proc seeingCompare {run} {
   set list ""
   foreach camcol "1 2 3 4 5 6" {
	foreach filter "u g r i z" {
	   lappend list [seeingCompareCF $run $camcol $filter]
	   }
	}
   set list [lsort -real $list]
   set m [lindex $list [expr [llength $list]/2]]
   return [format %.3f $m]
   }

#####################################################################

proc apcorReadCol {run camcol} {
   mtGlobals
   set topdir $masterdir
   global apcor
   set dir $topdir/$run/objcs/$camcol
   set files [lsort [glob $dir/psField*.fit]]
   set seeing($run,$camcol) ""
   foreach file $files {
#echo $file
	if {![regexp {.+-([0-9]+)\.fit} $file all field]} continue
	set field [string trimleft $field 0]
echo $run $camcol $field
	lappend apcor($run,$camcol) $field
	apcorReadField $run $camcol $field
	}
   return
   }

#############################################################
proc apcorReadField {run camcol field} {
   global apcor
   mtGlobals
   set topdir $masterdir
   set dir $topdir/$run/objcs/$camcol
   set file \
      $dir/psField-[format %06d $run]-$camcol-[format %04d $field].fit
   pspDef
   set chain [fits2Schema $file PSPOBJ -hdu 6]
   set hndl [chainGet $chain 0]
   loop i 0 5 {
	set filter [lindex "u g r i z" $i]
	pspsum $hndl $i
	set sum1 [exprGet $hndl.prof_cum<$i><4>]
	set sum2 [exprGet $hndl.prof_cum<$i><6>]
	if {$sum1 > 0 && $sum2 > 0} {
	   set ratio [expr $sum2/$sum1]
	} else {
	   set ratio 0
	   }
	set apcor($run,$camcol,$field,$filter,n) [exprGet $hndl.prof_nprof<$i>]
	set apcor($run,$camcol,$field,$filter) $ratio
	if {$apcor($run,$camcol,$field,$filter,n) >= 8} {
	   set sum1 [exprGet $hndl.prof_cum<$i><6>]
	   set sum2 [exprGet $hndl.prof_cum<$i><7>]
	   set ratio [expr $sum2/$sum1]
#	   echo "   " $run $camcol $field $filter ratio 8/6 = [format \
#		%.3f $ratio]
	   set apcor($run,$camcol,$field,$filter,ratio) $ratio
	   }

	}
   handleDel $hndl
   genericChainDel $chain
   return
   }

#####################################################3
#Write out ratios

proc apcorWrite {run} {
   global apcor seeing
   set fid [open apcor-$run.par w]
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int camcol;"
   puts $fid "   int field;"
   puts $fid "   float ratio<5>;"
   puts $fid "   float seeing<5>;"
   puts $fid "   } APCOR;"
   foreach camcol "1 2 3 4 5 6" {
	if {![info exists apcor($run,$camcol)]} continue
	if {![info exists seeing($run,$camcol)]} {
	   seeingReadCol $run $camcol
	   }
	set fields $apcor($run,$camcol)
	foreach field [lsort -integer $fields] {
	   set filters "u g r i z"
	   set ratios ""
	   set seeings ""
	   foreach filter $filters {
		lappend ratios [format %.4f $apcor($run,$camcol,$field,$filter)]
		lappend seeings [format %.2f \
		   $seeing($run,$camcol,$field,$filter)]
		}
	   puts $fid [list apcor $run $camcol $field \
		$ratios $seeings]
	   }
	}
   close $fid
   return
   }

#####################################################3
#Interpolate apcor ratios across bad fields

proc apcorInterp {run} {
   global apcor
   foreach camcol "1 2 3 4 5 6" {
	if {![info exists apcor($run,$camcol)]} continue
	set fields $apcor($run,$camcol)
	set filters "u g r i z"
	   foreach filter $filters {
		foreach field [lsort -integer $fields] {
		   if {![info exists apcor($run,$camcol,$field,$filter)\
			]} continue
		   if {$apcor($run,$camcol,$field,$filter) > 1} continue
		   set field1 [expr $field-1]
		   set field2 [expr $field+1]

#Look for valid value either before or after this field
		   if {[info exists apcor($run,$camcol,$field1,$filter) \
			]} {
			echo Fixup for $run,$camcol,$field,$filter, new value \
			   = $apcor($run,$camcol,$field1,$filter)
			set apcor($run,$camcol,$field,$filter) \
			   $apcor($run,$camcol,$field1,$filter)
			continue
			}
		   if {[info exists apcor($run,$camcol,$field2,$filter) \
			]} {
			echo Fixup for $run,$camcol,$field,$filter, new value \
			   = $apcor($run,$camcol,$field2,$filter)
			set apcor($run,$camcol,$field,$filter) \
			   $apcor($run,$camcol,$field2,$filter)
			continue
			}
		   }
		}
	}
   return
   }

