# Try to pipe data to a separate executable.

proc cmdExec {args} {
#Pipe1 is used to pipe data from parent to child
#Pipe2 is used to pipe data from child to parent
   set pipe1 [pipe]
   set pipe2 [pipe]
#puts stdout "$pipe1 $pipe2"
   set rpipe1 [lindex $pipe1 0]
   set wpipe1 [lindex $pipe1 1]
   set rpipe2 [lindex $pipe2 0]
   set wpipe2 [lindex $pipe2 1]
#   fcntl $rpipe2 NONBLOCK 1
   fcntl $wpipe1 NOBUF 1
   fcntl $wpipe2 NOBUF 1
   flush stdout
   flush stderr
   set pid [eval exec [join $args] <@ $rpipe1 >@ $wpipe2 2>@ stderr &]
#Code for parent process
   close $rpipe1
   close $wpipe2
#Return read, write pipes and pid
   return "$rpipe2 $wpipe1 $pid"
   }

######################################################################
#Dither reg1; output to reg2
proc regDither {reg1 nlevel min max} {
   imgGlobals
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
#   sky4 $reg1
#   set min [max 0 [expr $img(sky) - 2.*$img(skynoise)]]
#   set max [expr $min+[min [max 255 [expr 30.*$img(skynoise)]] 65535]]
   set reg2 [regNew $nrow $ncol -type U8]
   set list [cmdExec regDither $nrow $ncol 2 $min $max $nlevel]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg1.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $reg2.rows_u8<$i><0> $ncol
	}
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

####################################################################
#Square root scaling of reg1 in place
proc regSqrt {reg1 min max {bias 0}} {
   imgGlobals
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
   set reg2 [regNew $nrow $ncol]
   set list [cmdExec regSqrt $nrow $ncol 2 $min $max $bias]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg1.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $reg2.rows_u16<$i><0> [expr 2*$ncol]
	}
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

####################################################################
#Row median - return new region
proc regMedian {reg1 interval} {
   imgGlobals
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
   set reg2 [regNew $nrow $ncol]
   set list [cmdExec regMedian $nrow $ncol 2 $interval]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg1.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $reg2.rows_u16<$i><0> [expr 2*$ncol]
	}
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

####################################################################
#Smooth - return a new region
#Only works on unsigned shorts!
proc regSmooth {reg1} {
   imgGlobals
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
   set reg2 [regNew $nrow $ncol]
   set list [cmdExec regSmooth $nrow $ncol 2]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   binWrite $wid $reg1.rows_u16<0><0> [expr 2*$ncol]
   loop i 1 $nrow {
	binWrite $wid $reg1.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $reg2.rows_u16<[expr $i-1]><0> [expr 2*$ncol]
	}
   binRead $rid $reg2.rows_u16<[expr $nrow-1]><0> [expr 2*$ncol]
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

########################################################################
#Expand a region by a factor 3
proc reg3 {reg1} {
   imgGlobals
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
   set reg2 [regNew [expr 3*$nrow] [expr 3*$ncol] -type U8]
   set list [cmdExec reg3 $nrow $ncol 1]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
     loop j 0 3 {
	binWrite $wid $reg1.rows_u8<$i><0> [expr $ncol]
	flush $wid
	binRead $rid $reg2.rows_u8<[expr 3*$i + $j]><0> [expr 3*$ncol]
	}
    }
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

########################################################################
#Decimate a region
proc regDecimate {reg1 decimate} {
   imgGlobals
   if {$decimate <1} return
   set nrow [exprGet $reg1.nrow]
   set ncol [exprGet $reg1.ncol]
   set type [exprGet -enum $reg1.type]
   if {[regexp 8 $type]} {set nbyte 1}
   if {[regexp 16 $type]} {set nbyte 2}
   if {[regexp 32 $type]} {set nbyte 4}
   regexp {TYPE_(.+)} $type all suffix
   set suffix [string tolower $suffix]
   set TYPE [string toupper $suffix]
   set reg2 [regNew [expr $nrow/$decimate] [expr $ncol/$decimate] -type $TYPE]
   set list [cmdExec regDecimate $nrow $ncol $nbyte $decimate]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow/$decimate {
	binWrite $wid $reg1.rows_$suffix<[expr int(($i+.5)*$decimate)]><0> \
	   [expr $ncol*$nbyte]
	flush $wid
	binRead $rid $reg2.rows_$suffix<$i><0> [expr ($ncol/$decimate)*$nbyte]
	}
   wait $pid
   close $rid
   close $wid
   return $reg2
   }

####################################################################
#Gamma enhancement of rgb images.  Must be unsigned shorts
proc regGamma {rreg greg breg gamma sigr sigg sigb} {
   imgGlobals
   set nrow [exprGet $rreg.nrow]
   set ncol [exprGet $rreg.ncol]
   set rreg2 [regNew $nrow $ncol]
   set greg2 [regNew $nrow $ncol]
   set breg2 [regNew $nrow $ncol]
   set list [cmdExec regGamma $nrow $ncol 2 $gamma $sigr $sigg $sigb]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $rreg.rows_u16<$i><0> [expr 2*$ncol]
	binWrite $wid $greg.rows_u16<$i><0> [expr 2*$ncol]
	binWrite $wid $breg.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $rreg2.rows_u16<$i><0> [expr 2*$ncol]
	binRead $rid $greg2.rows_u16<$i><0> [expr 2*$ncol]
	binRead $rid $breg2.rows_u16<$i><0> [expr 2*$ncol]
	}
   wait $pid
   close $rid
   close $wid
   return "$rreg2 $greg2 $breg2"
   }

####################################################################
#Make a PPM file.  Inputs must be uchars.
#nrep is pixel repetition factor.
proc regPPM {rreg greg breg nrep file} {
   imgGlobals
   set nrow [exprGet $rreg.nrow]
   set ncol [exprGet $rreg.ncol]
   set list [cmdExec regPPM $nrow $ncol 1 $nrep $file]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $rreg.rows_u8<$i><0> $ncol
	binWrite $wid $greg.rows_u8<$i><0> $ncol
	binWrite $wid $breg.rows_u8<$i><0> $ncol
	flush $wid
	}
   wait $pid
   close $rid
   close $wid
   return
   }

####################################################################
#Square root scaling of reg1 in place
proc regSwap {reg} {
   imgGlobals
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
   set list [cmdExec regSwap $nrow $ncol 2]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg.rows_u16<$i><0> [expr 2*$ncol]
	flush $wid
	binRead $rid $reg.rows_u16<$i><0> [expr 2*$ncol]
	}
   wait $pid
   close $rid
   close $wid
   return
   }

#########################################################################
#Make a gif.  Region must be u8
proc regGif {reg nred ngreen nblue outfile} {
   imgGlobals
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
   set list [cmdExec gifMain $nrow $ncol 1 $nred $ngreen $nblue $outfile]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg.rows_u8<$i><0> $ncol
	flush $wid
	}
   wait $pid
   close $rid
   close $wid
   return
   }

#########################################################################
#Make a png.  Region must be u8
proc regPng {reg nred ngreen nblue outfile} {
   imgGlobals
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
   set list [cmdExec pngMain $nrow $ncol 1 $nred $ngreen $nblue $outfile]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg.rows_u8<$i><0> $ncol
	flush $wid
	}
   wait $pid
   close $rid
   close $wid
   return
   }

#########################################################################
#Ishift.  Arbitrary region
#Note: This procedure seems to give problems when combined with other region
#routines.  I will implement a slower but safer version below
proc regIshift {reg rshift cshift} {
   imgGlobals
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
#Get nbyte and suffix
   set type [exprGet -enum $reg.type]
   if {[regexp 8 $type]} {set nbyte 1}
   if {[regexp 16 $type]} {set nbyte 2}
   if {[regexp 32 $type]} {set nbyte 4}
   regexp {TYPE_(.+)} $type all suffix
   set suffix [string tolower $suffix]
#   set TYPE [string toupper $suffix]

#Shift the rows.  Just cycle the pointers
#Temporary handle for a pointer
   loop j 0 [expr int(abs($rshift))] {
	set tmp($j)  [genericNew PTR]
	}
#Shift rows left
   if {$rshift < 0} {
	loop j 0 [expr -1*$rshift] {
	   handleSetFromHandle $tmp($j) $reg.rows_$suffix<$j>
	   }
	loop i 0 [expr $nrow+$rshift] {
	   handleSetFromHandle $reg.rows_$suffix<$i> \
		$reg.rows_$suffix<[expr $i-$rshift]>
	   }
	loop j 0 [expr -1*$rshift] {
	   handleSetFromHandle $reg.rows_$suffix<[expr $j+$nrow+$rshift]> \
		$tmp($j)
	   }
   } else {

#Shift rows right
	loop j 0 $rshift {
	   handleSetFromHandle $tmp($j) $reg.rows_$suffix<[expr $j+$nrow-$rshift]>
	   }
	loop i $rshift $nrow {
	   handleSetFromHandle $reg.rows_$suffix<[expr $nrow+$rshift-$i-1]> \
		$reg.rows_$suffix<[expr $nrow-$i-1]>
	   }
	loop j 0 $rshift {
	   handleSetFromHandle $reg.rows_$suffix<$j> \
		$tmp($j)
	   }
	}
	
#Temporary handle for a pointer
   loop j 0 [expr int(abs($rshift))] {
	genericDel $tmp($j)
	}

   set list [cmdExec regIshift $nrow $ncol $nbyte $cshift]
   set rid [lindex $list 0]
   set wid [lindex $list 1]
   set pid [lindex $list 2]
   loop i 0 $nrow {
	binWrite $wid $reg.rows_$suffix<$i><0> [expr $ncol*$nbyte]
	flush $wid
	binRead $rid $reg.rows_$suffix<$i><0> [expr $ncol*$nbyte]
	}
   wait $pid
   close $rid
   close $wid
   return
   }

##########################################################################
#New proc regIshift - just use Dervish commands and copy.
proc regIshift {reg rshift cshift} {
   set nrow [exprGet $reg.nrow]
   set ncol [exprGet $reg.ncol]
   if {$rshift > 0} {
	set rstart 0
	set nr [expr $nrow-$rshift]
   } else {
	set rstart [expr -1*$rshift]
	set nr [expr $nrow+$rshift]
	}
   if {$cshift > 0} {
	set cstart 0
	set nc [expr $ncol-$cshift]
   } else {
	set cstart [expr -1*$cshift]
	set nc [expr $ncol+$cshift]
	}
   set subreg [subRegNew $reg $nr $nc $rstart $cstart]
   set newreg [regNewFromReg $subreg]
   regDel $subreg
   regSetWithDbl $reg 0
   set subreg [subRegNew $reg $nr $nc [expr $rstart+$rshift] [expr $cstart+$cshift]]
   regPixCopy $newreg $subreg
   regDel $subreg
   regDel $newreg
   return
   }

#
