#Steve Kent's modifications to Mark Schaffer's sdReport.tcl file.
#Use latest home areas.  Simplify code where possible.  Don't be so verbose.
#Make comparison with ppd and/or speclog
#Finally, run sdHdrFix to fix up problems that Schlegel finds in headers.

##############################
### Commands
###   makeAllReports	Create new sdReports, skip over existing
###   specLogOut	Temporary for now - Crawl speclog area and
###			create sdHdrFix files (in homeDir)
###			with exposures commented out in original sdReports
###   sdHdrFix		Fix up headers for Schlegel fixes.  Also look for
###			sdHdrFix files in speclog and in homeDir
###   ppdCompare	Compare new files with either speclog pr ppd
###   sdReportCheck	Check for bad tileId/plateId

set homeDir /data/dp7.c/users/skent/mt/sdReport
set sdDir /data/dp9.a/data/spectro/rawdata;

#speclog files
set logDir /data/dp9.a/data/spectro/fspeclog

set ppdDir /data/dp30.a/data/ppd
set goldDir /data/dp30.a/data/golden

#I can either check against ppd or speclog.
set checkDir $logDir

proc makeAllReports {} {
    global homeDir sdDir

    if {![file exists $homeDir]} {mkdir $homeDir}


    set mjdirs [glob -nocomplain $sdDir/*];
    set mjdirss [lsort $mjdirs];

    foreach mjdir $mjdirs {
	set mjd [file tail $mjdir]
	if {![ctype digit $mjd]} continue

	if {$mjd <= 51433} continue

#Check if sdReport file already exists.
	set fname $homeDir/sdReport-$mjd.par
	if {[file exists $fname]} {
	   echo MJD $mjd already done.
	   continue
	   }
	makeSdReport $mjdir $homeDir $mjd;
    }
}

#########################################################################
#Make sdReport file for 1 mjd.

proc makeSdReport {rootDir homeDir mjd} {

#    echo "Making sdReport for $mjd...";

    if {[catch {set imageList [glob $rootDir/sdR*.fit*]}]} {

	return;

    }

#sortImages strips off leading directory info.
    set imageList [sortImages $imageList];
echo imageList is $imageList
    global file;
    set file [open $homeDir/sdReport-$mjd.par w];

    set observer [getObserver $homeDir $rootDir [lindex $imageList 1]];

    makeHeader $mjd $observer;

    makeFile $mjd $homeDir $rootDir $imageList;

    flush $file;
    close $file;
    

    echo "Completed sdReport for $mjd";
}

############################################################################
#Read header of file.  If file does not exist, look for .gz version
proc headerRead {file} {
   global homeDir

   if {![file exists $file]} {
	set file $file.gz
	}
   if {![file exists $file]} {
	echo Can't find file $file
	return
	}
   if {[file size $file] == 0} {
	echo Zero file size for $file!
	return
	}
   set hdr [hdrNew]
   if {[regexp {\.gz$} $file]} {
	exec cp $file $homeDir/local.fit.gz
	if {[catch {exec gunzip $homeDir/local.fit.gz}]} {
	   echo Can't unzip $file
	   return
	   }
	if {[catch {hdrReadAsFits $hdr $homeDir/local.fit}]} {
	   echo "Can't read $homeDir/local.fit"
	   return
	   }
	exec rm $homeDir/local.fit
   } else {
	if {[catch {hdrReadAsFits $hdr $file}]} {
	   echo "Can't read $file"
	   return
	   }
	}
   return $hdr
   }

############################################################################
#Helper routine called by makeSdReport. Feed in list of images.
#Skeleton of sdReport file has already been created.

proc makeFile {mjd homeDir rootDir imageList} {

    global file;
    global ppdDir goldDir

    set oldPlate "";


#Fetch old sdReport from ppd and extract comments
    set comflag 0
    if {![catch {set chains [param2Chain $ppdDir/$mjd/sdReport-$mjd.par hdr]}]} {
	foreach chain $chains {
	   if {[chainTypeDefine $chain] != "SDCOMMENT"} {
		genericChainDel $chain
	   } else {
		set comChain $chain
		set comflag 1
		}
	   }
	}
    foreach image $imageList {

	if {$image == "SPACE"} {
	    puts $file "";
	    flush $file;
	    continue;
	}

	set fileExposure [split $image -];
	set fileExposure [lindex $fileExposure 2];
	set fileExposure [split $fileExposure .];
	set fileExposure [lindex $fileExposure 0];
	set fileExposure [string trimleft $fileExposure 0];

	set hdr [headerRead $rootDir/$image]
	if {$hdr == ""} {
	   echo Problem with image $rootDir/$image\; skipping
	   continue
	   }
	set seqId [hdrGetAsInt $hdr SEQID];
	set flavor [hdrGetAsAscii $hdr FLAVOR];  

#Replace science with target
	if {$flavor == "science"} {set flavor target}

#New bug (alas) - flavor can be non-existent!  These seem to be targets.
#Actually, for mjd 52525, the string in the header for FLAVOR is illegal.
#
	if {$flavor == ""} {set flavor target}
	set quality [hdrGetAsAscii $hdr QUALITY];        
	set program [string trimright [hdrGetAsAscii $hdr PROGRAM] " "];
	set exposure [hdrGetAsInt $hdr EXPOSURE];

	if {$exposure != $fileExposure} {
	    echo "Filename $fileExposure off of exposure $exposure";
	}

	set tileId [hdrGetAsInt $hdr TILEID];
	set plateId [hdrGetAsInt $hdr PLATEID];
	set mapId [hdrGetAsInt $hdr MAPID];

	if {[catch {set cameras [hdrGetAsAscii $hdr CAMERAS]}]} {
	    set cameras "unknown";
	}
	

	set expTime [hdrGetAsDbl $hdr EXPTIME];
	set mjd [format "%5.5f" [tai2MJD [hdrGetAsDbl $hdr TAI]]];
	set ra [format "%3.4f" [hdrGetAsDbl $hdr RADEG]];
	set dec [format "%3.4f" [hdrGetAsDbl $hdr DECDEG]];
	set tai [hdrGetAsAscii $hdr TAIHMS];
	set target [string trimright [hdrGetAsAscii $hdr NAME] " "];
	    
	if {$oldPlate != $plateId} {
	    set oldPlate $plateId;

#Bug fixed here - Mark was using exposure, not fileExposure
	    set trueSeqId $fileExposure;
	}

	if {$program == ""} { set program "unknown"}
	if {$target == ""} { set target "unknown"}
	    
#Look for existing comments
	if {$comflag == 1 && [string trim $cameras] == "b1"} {
	   set subchain [chainSearch $comChain "{frame = $fileExposure}"]
	   chainEach hndl $subchain {
		set comment [exprGet $hndl.comment]

#Purge excessive braces
		if {[llength $comment] == 1} {set comment [lindex $comment 0]}
		if {[llength $comment] == 1} {set comment [lindex $comment 0]}
		puts $file [list sdcomment $fileExposure $fileExposure \
		   $comment]
		}
	   chainDel $subchain
	   }
	puts $file "sexp $trueSeqId $tileId $plateId $mapId $flavor $quality \"$program\" $fileExposure $expTime $mjd $ra $dec $cameras $target $tai"
	flush $file;
	hdrDel $hdr;

    }
    if {$comflag == 1} {genericChainDel $comChain}
}

    
#######################################################################
proc getObserver {homeDir rootDir fileName} {

#    echo "Getting observer."


    if {![file exists $rootDir/$fileName]} {
	return "unknown";
    }

    set hdr [headerRead $rootDir/$fileName]
    if {$hdr == ""} {
	echo Problem fetching observer from file $rootDir/$fileName
	return Unknown
	}

    set observer [hdrGetAsAscii $hdr OBSERVER];
    hdrDel $hdr;

    return $observer;
}

##########################################################################
proc makeHeader {mjd observer} {

#    echo "Making header."

    global file;
    global goldDir ppdDir

    set spectroCCD(-9999)               disabled;
    set spectroCCD(1)                   b1;
    set spectroCCD(4)                   r1;
    set spectroCCD(2)                   r2;
    set spectroCCD(3)                   b2;
    
    puts $file "mjd \t $mjd \t #[fmtclock [getclock] "%Y/%m/%d %H:%M:%S.0" GMT]";
    puts $file "observers \t $observer";
    puts $file "version \t atFermi \t# version of IOP";
    puts $file "mcpVersion \t atFermi \t# version of MCP";
    puts $file "cameraVersion \t atFermi \t# Camera code version";
    puts $file "parameters \t # Tunable parameters file (if any)";
    puts $file "parametersDir \t # Directory containing the 'parameters'";
    puts $file " \t # file (optional --- defaults to '.')";
    puts $file "";
    puts $file "equinox 2000 \t # Equinox of telescope coordinates (years)";
    puts $file "fileNameFormat      sdR-%02s-%08d.fit \t # Format to convert ";
    puts $file "\t\t # frame ID to raw file name";
    puts $file "";
    puts $file "# CCD configuration files";
    puts $file "configDir \t .";

#Need a real opConfig file.  Try ppd, golden, or fake it.
#NOTE: If we have stuff imaging data, there is a desire to have sdReport
#config files match the idReport config files (since the stuffer checks).
#However, I am not reading idReport here, since one does not know if it
#is up to date.
    set configFile [lindex [glob -nocomplain $goldDir/$mjd/opConfig*.par] 0]
    if {$configFile == ""} {
	set configFile [glob -nocomplain $ppdDir/$mjd/opConfig*.par]
	}
    if {$configFile == ""} {set configFile none}
    puts $file "ccdConfig \t [file tail $configFile]";

    set configFile [lindex [glob -nocomplain $goldDir/$mjd/opECalib*.par] 0]
    if {$configFile == ""} {
	set configFile [glob -nocomplain $ppdDir/$mjd/opECalib*.par]
	}
    if {$configFile == ""} {set configFile none}
    puts $file "ccdECalib \t [file tail $configFile]";

    set configFile [glob -nocomplain $goldDir/$mjd/opBC*.par]
    if {$configFile == ""} {
	set configFile [glob -nocomplain $ppdDir/$mjd/opBC*.par]
	}
    if {$configFile == ""} {set configFile none}
    puts $file "ccdBC \t [file tail $configFile]";

    set configFile [glob -nocomplain $goldDir/$mjd/opCamera*.par]
    if {$configFile == ""} {
	set configFile [glob -nocomplain $ppdDir/$mjd/opCamera*.par]
	}
    if {$configFile == ""} {set configFile none}
    puts $file "ccdCamera \t [file tail $configFile]";

    puts $file "ccdVoltages \t idCCDLog-%d-${mjd}.par";
    puts $file ""
    foreach ccd [array names spectroCCD] {
	puts $file "$spectroCCD($ccd) \t $ccd \t # which CCD goes to which camera";
        }
    puts $file "";
    puts $file "typedef struct {";
    puts $file "\t int       seqId; \t # sequence number.";
    puts $file "\t int       tileId; \t # tile ID";
    puts $file "\t int       plateId; \t # plugged plate ID";
    puts $file "\t int       mapId; \t # mapped plate ID";
    puts $file "\t char      flavor\[20\]; \t # Flavor of this sequence (e.g., dome flat, cal lamp, etc.)";
    puts $file "\t char      quality\[20\];";
    puts $file "\t char      program\[20\];";
    puts $file "\t int       exposure;\t # Unique identifier. ";
    puts $file "\t double    expTime; \t # Exposure time. (seconds) ";
    puts $file "\t double    mjd; \t # MJD at the start of the exposure. (MJD)";
    puts $file "\t double    ra;";
    puts $file "\t double    dec;";
    puts $file "\t char      camera\[20\]; \t# Cameras used (e.g., b1r1b2r2)";
    puts $file "\t char      targetName\[20\];";
    puts $file "\t char      taihms\[20\]; \t # HH:MM:SS.SS of exposure start (UT)";
    puts $file "} SEXP;";
    puts $file "";
    puts $file "typedef struct {";
    puts $file "\t int seqNum;";
    puts $file "\t int frame;";
    puts $file "\t char comment\[240\];";
    puts $file "} SDCOMMENT;";
    puts $file "";
    flush $file;

    return;
}
    
#######################################################
proc sortImages {imageList} {

#    echo "Sorting images."

    set sortedList "";
    set finalList "";

    foreach image $imageList {
	set image [split $image -];
	set camera [lindex $image 1];
	
	if {$camera == "b1"} {
	    set camera 0;
	} elseif {$camera == "r1"} {
	    set camera 1;
	} elseif {$camera == "b2"} {
	    set camera 2;
	} elseif {$camera == "r2"} {
	    set camera 3;
	} else {
	    continue;
	}

	set exposure [lindex $image 2];
	set exposure [split $exposure .];
	set exposure [lindex $exposure 0];

	lappend sortedList "$exposure.$camera"
    }

    set sortedList [lsort $sortedList];
    set oldExposure "";

    foreach image $sortedList {
	set image [split $image .];
	set exposure [lindex $image 0];
	set camera [lindex $image 1];

	if {$exposure != $oldExposure} {
	    set oldExposure $exposure;
	    lappend finalList "SPACE";
	}

	if {$camera == 0} {
	    set camera "b1";
	} elseif {$camera == 1} {
	    set camera "r1";
	} elseif {$camera == 2} {
	    set camera "b2";
	} elseif {$camera == 3} {
	    set camera "r2";
	}

	set image "sdR-$camera-$exposure.fit";
	lappend finalList $image;
    }

    return $finalList;
}

###########################################################
proc tai2MJD { tai } {
    return [expr ($tai - 33) / 86400.0]
}

##########################################################
proc ppdCompare {} {
   global homeDir spDir checkDir

#First, find all sdReports derived from rawdata
   set files [lsort [glob -nocomplain $homeDir/sdReport*.par]]
   foreach file $files {
       set tail [file tail $file]
       if {![regexp {sdReport-([0-9]+).par} $tail all mjd]} continue

#Skip uninteresting mjds
	if {$mjd < 51564} continue

#See if we care about this file - look for "target " entries
	if {[catch {set list [exec grep "target " $file]}]} {
	   continue
	   }
	if {[llength $list] == 0} continue
       set match $checkDir/$mjd/$tail
       if {![file exists $match]} {
	   echo Rawdata mjd $mjd has no ppd sdReport file
	   continue
	} else {
	   echo MJD $mjd OK
	   }

#Now compare file contents
#First, rawdata sdReport
	set list [param2Chain $file hdr]
	set rawChain ""
	foreach ch $list {
	   if {[chainTypeDefine $ch] != "SEXP"} {
		genericChainDel $ch
	   } else {
		set rawChain $ch
		}
	   }
	if {$rawChain == ""} {
	   echo No SEXP for mjd $mjd
	   continue
	   }
#Next, ppd sdReport
	set list [param2Chain $match hdr]
	set ppdChain ""
	foreach ch $list {
	   if {[chainTypeDefine $ch] != "SEXP"} {
		genericChainDel $ch
	   } else {
		set ppdChain $ch
		}
	   }
	if {$ppdChain == ""} {
	   echo No SEXP in ppd file for mjd $mjd
	   genericChainDel $rawChain
	   continue
	   }

#Construct list of exposures from each
	set rawList ""
	set rawPlate ""
	if {[info exists rawData]} {unset rawData}
	if {[info exists rawExp]} {unset rawExp}
	chainEach hndl $rawChain {
	   foreach var "plateId exposure ra dec flavor mapId camera" {
		set $var [exprGet $hndl.$var]
		}
#	   if {$mapId < 0} continue
#	   if {$plateId < 266} continue
	   if {[lsearch $rawList $exposure] < 0} {
		lappend rawList $exposure
		set rawId($exposure,plateId) $plateId
		set rawId($exposure,flavor) $flavor
		set rawExp($exposure) ""
		}
	   lappend rawExp($exposure) $camera
	   if {[lsearch $rawPlate $plateId] < 0} {
		lappend rawPlate $plateId
		set rawData($plateId,ra) $ra
		set rawData($plateId,dec) $dec
		set rawData($plateId,flavor) $flavor
		}
	   }
	genericChainDel $rawChain

	set ppdList ""
	set ppdPlate ""
	if {[info exists ppdData]} {unset ppdData}
	if {[info exists ppdId]} {unset ppdId}
	chainEach hndl $ppdChain {
	   foreach var "plateId exposure ra dec flavor mapId camera" {
		set $var [exprGet $hndl.$var]
		}
	   if {$mapId < 0} continue
	   if {$plateId < 266} continue
	   if {[lsearch $ppdList $exposure] < 0} {
		lappend ppdList $exposure
		set ppdId($exposure,plateId) $plateId
		set ppdId($exposure,flavor) $flavor
		}
	   if {$plateId <0} continue
	   if {[lsearch $ppdPlate $plateId] < 0} {
		lappend ppdPlate $plateId
		set ppdData($plateId,ra) $ra
		set ppdData($plateId,dec) $dec
		set ppdData($plateId,flavor) $flavor
		}
	   }
	genericChainDel $ppdChain

#Check for unsynched exposures
	foreach exp $rawList {
	   if {[llength $rawExp($exp)] < 4} {
		echo MJD $mjd WARNING: Exposure $exp has \
		   [llength $rawExp($exp)] exposures
		}
	   }

#Compare some basic properties
	foreach exp $ppdList {
	   if {$ppdId($exp,flavor) == "engineering"} continue
	   if {$ppdId($exp,flavor) == "ignore"} continue
	   if {$ppdId($exp,flavor) == "dark"} continue
	   if {$ppdId($exp,flavor) == "bias"} continue
	   if {[lsearch $rawList $exp] < 0} {
		echo MJD $mjd ppd exposure $exp not in raw report
	   } else {
		if {$rawId($exp,plateId) != $ppdId($exp,plateId)} {
		   echo MJD $mjd Plate ID mismatch for exposure $exp
		   }
		}
	   }

	foreach plateId $ppdPlate {
	   if {[lsearch $rawPlate $plateId] < 0} {
		echo MJD $mjd ppd plateId $plateId not in raw report
	   } elseif {$rawData($plateId,flavor) == "target" || \
		$rawData($plateId,flavor) == "science"} {
		set radiff [expr abs($rawData($plateId,ra) - \
		   $ppdData($plateId,ra))]
		if {$radiff > .002} {
		   echo MJD $mjd plateId $plateId ra differs by $radiff \
			between raw,ppd.
		   }
		set decdiff [expr abs($rawData($plateId,dec) - \
		   $ppdData($plateId,dec))]
		if {$decdiff >.002} {
		   echo MJD $mjd plateId $plateId dec differs by $decdiff \
			between raw,ppd.
		   }
		}
	   }
	}

#Now go the other direction
   echo Processing ppd directories
   set files [lsort [glob -nocomplain $checkDir/*]]
   foreach file $files {
       set mjd [file tail $file]
	if {![ctype digit $mjd]} continue

#Skip uninteresting mjds
	if {$mjd < 51564} continue
	set reportFile $file/sdReport-$mjd.par
	if {![file exists $reportFile]} continue

#See if we care about this file - look for "target " entries
	if {[catch {set list [exec grep "target " $reportFile]}]} {
	   continue
	   }
	if {[llength $list] == 0} continue
       set match $homeDir/sdReport-$mjd.par
       if {![file exists $match]} {
	   echo PPD mjd $mjd has no rawdata sdReport file
	} else {
	   echo MJD $mjd OK
	   }
	}
   return
   }

########################################################################
#Fix up sdReports made from headers for problems in headers.  Schlegel
#has 2 mechanisms for tracking changes.  For a long times, changes
#were in idlspec2d in examples/opHdrFix.par.  Since then, they are
#stored in speclog in individual sdHdrFix.par files for the relevant MJD.
#Finally, apply some fixes from my ppd.tcl sdFix routine.

proc sdHdrFix {} {
   global homeDir logDir

#Fetch a list of all plates and make plate-tile cross-reference
   set file [lindex [lsort [glob -nocomplain plateList-*.par]] end]
   if {$file == ""} {
	echo No plateList file found!
	return
	}
   set chain [param2Chain $file hdr]
   chainEach hndl $chain {
	set plateId [exprGet $hndl.plateId]
	set tileId [exprGet $hndl.tileId]
	set plate($tileId) $plateId
	set tile($plateId) $tileId
	}
   genericChainDel $chain

#First, assemble a list of all fixes.
#Use UPS to find idlspec2d product
   set line [exec ups list -l -f Linux idlspec2d | grep Home]
   set root [string trim [lindex [split $line =] 1]]
   set file $root/examples/opHdrFix.par
   if {![file exists $file]} {
	echo No opHdrFix.par file found!
	return
	}

#Make a list of all corrections
   echo Reading from $file
   set hdrChain [param2Chain $file hdr]
   set fixList ""
   chainEach hndl $hdrChain {
	foreach var "fileroot keyword value" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend fixList [list $fileroot $keyword $value]
	}
   genericChainDel $hdrChain

#Now fetch individual sdHdrFix files
   set dirs [glob -nocomplain $logDir/*]
   foreach dir $dirs {
	set mjd [file tail $dir]
	if {![ctype digit $mjd]} continue
	set file $dir/sdHdrFix-$mjd.par
	if {![file exists $file]} continue
	echo Found sdHdrFix $file
	set chain [param2Chain $file hdr]
	chainEach hndl $chain {
	   foreach var "fileroot keyword value" {
		set $var [exprGet $hndl.$var]
		}
	   lappend fixList [list $fileroot $keyword $value]
	   }
	genericChainDel $chain
	}

#Finally, fetch sdHdrFix files in homeDir that I generate using specLogOut
   set files [glob -nocomplain $homeDir/sdHdrFix*.par]
   foreach file $files {
	echo Found sdHdrFix $file
	set chain [param2Chain $file hdr]
	chainEach hndl $chain {
	   foreach var "fileroot keyword value" {
		set $var [exprGet $hndl.$var]
		}
	   lappend fixList [list $fileroot $keyword $value]
	   }
	genericChainDel $chain
	}

#Positions of all keywords in sexp record
   set index(SEQID) 1
   set index(TILEID) 2
   set index(PLATEID) 3
   set index(MAPID) 4
   set index(FLAVOR) 5
   set index(QUALITY) 6
   set index(PROGRAM) 7
   set index(EXPOSURE) 8
   set index(EXPTIME) 9
   set index(MJD) 10
   set index(RA) 11
   set index(DEC) 12
   set index(CAMERA) 13
   set index(TARGETNAME) 14
   set index(TAIHMS) 15

#Now decode the fixes and make a new list
   set newFixList ""
   set expList ""
   foreach fix $fixList {
	set fileroot [lindex $fix 0]
	set keyword [lindex $fix 1]

#Change FITS keyword to sdReport variant if needed
	if {$keyword == "RADEG"} {set keyword RA}
	if {$keyword == "DECDEG"} {set keyword DEC}
	if {$keyword == "NAME"} {set keyword TARGETNAME}
	set value [lindex $fix 2]

#Trim useless junks from value if present
	set value [string trim $value ']
	if {![regexp {sdR-(..)-(.+)} $fileroot all cams root]} {
	   echo Can't decode root $root
	   continue
	   }
	set exps $root

#Expand ranges if present
	if {[regexp {(.+)\[([0-9])-([0-9])} $root all prefix l1 l2]} {
	   set exps ""
	   for {set l $l1} {$l <= $l2} {incr l} {
		lappend exps $prefix$l
		}
	   }

#Expand question marks
	set cams1 [string index $cams 0]
	set cams2 [string index $cams 1]
	if {$cams1 == "?"} {set cams1 "b r"}
	if {$cams2 == "?"} {set cams2 "1 2"}
	set cams ""
	foreach c1 $cams1 {
	   foreach c2 $cams2 {
		lappend cams $c1$c2
		}
	   }

#Convert to exposures
	foreach exp $exps {
	   set exp [string trimleft $exp 0]
	   lappend expList $exp
	   foreach cam $cams {
		lappend newFixList [list $exp $cam $keyword \
		   [string trim $value ']]
		}
	   }
	}

foreach fix $newFixList {
echo $fix
}

#Now fix the sdReport files.  If I have multiple fixes for a file, be
#ineffecient and apply one at a time.  I rewrite entire file.
   set files [lsort [glob -nocomplain $homeDir/sdReport-*.par]]
   foreach file $files {
	echo Checking $file
	set lines [split [read_file $file] \n]
	set outlines ""
	set nfix 0

#Resequence as well
	set oldId -10000
	set oldExpId -1000
	foreach line $lines {
	   if {[regexp "^sexp" $line] || [regexp "^SEXP" $line]} {
		set exp [lindex $line $index(EXPOSURE)]
		set cam [lindex $line $index(CAMERA)]
		if {[lsearch $expList $exp] >= 0} {
		   foreach fix $newFixList {
			set expfix [lindex $fix 0]
			set camfix [lindex $fix 1]
			if {$expfix != $exp} continue
			if {$camfix != $cam} continue
			set keyword [lindex $fix 2]
			set value [lindex $fix 3]
			if {[info exists index($keyword)]} {
			   set line [lreplace $line $index($keyword) \
				$index($keyword) $value]
			   }
			}
		   incr nfix
		   }		   

		foreach var "exposure tileId plateId mapId flavor targetName" {
		   set $var [lindex $line $index([string toupper $var])]
		   }
		if {$flavor == "science"} {
		   set flavor target
		   set line [lreplace $line $index(FLAVOR) $index(FLAVOR) \
			$flavor]
		   incr nfix
		   }
		if {$flavor == "arc" || $flavor == "flat" || $flavor == \
		   "target" || $flavor == "smear"} {
		   if {$plateId < 0 || $plateId == 9999} {
			if {[info exists plate($tileId)]} {
			   set plateId $plate($tileId)
			   set line [lreplace $line $index(PLATEID) \
				$index(PLATEID) $plateId]
			   incr nfix
			   }
			}
		   if {$tileId < 0 || $tileId == 9999} {
			if {[info exists tile($plateId)]} {
			   set tileId $tile($plateId)
			   set line [lreplace $line $index(TILEID) \
				$index(TILEID) $tileId]
			   incr nfix
			   }
			}
		   if {$mapId < 0} {
			if {[regexp {([0-9]+)-([0-9]+)-([0-9]+)} $targetName \
			   all mapplate mjdmap version]} {
			   set mapId $mjdmap$version
			   set line [lreplace $line $index(MAPID) \
				$index(MAPID) $mapId]
			   incr nfix
			   }
			}
		   }
#Check sequenceId
		set seqId [lindex $line $index(SEQID)]
		set expId [lindex $line $index(EXPOSURE)]
		set plateId [lindex $line $index(PLATEID)]

#Reset sequence ID only if exposure ID changes.
		if {$expId != $oldExpId} {
		   if {$plateId != $oldId} {
			set newSeqId $expId
			set oldId $plateId
			}
		   set oldExpId $expId
		   }
		if {$newSeqId != $seqId} {
#		   echo Revise sequence $seqId to $newSeqId
		   set line [lreplace $line $index(SEQID) $index(SEQID) \
			$newSeqId]
		   incr nfix
		   }
		}
	   lappend outlines $line
	   }
	if {$nfix > 0} {
	   echo Rewriting $file
	   set lines [join $outlines \n]
	   write_file $file $lines
	   }
	}
   return
   }

#######################################################################
#Check sdReports for bad plateId or tileId when a good one should be
#present

proc sdReportCheck {} {
   global homeDir
   set files [lsort [glob -nocomplain $homeDir/sdReport*.par]]
   foreach file $files {
	set tail [file tail $file]
	set chains [param2Chain $file hdr]
	set schain ""
	foreach chain $chains {
	   if {[chainTypeDefine $chain] != "SEXP"} {
		genericChainDel $chain
	   } else {
		set schain $chain
		}
	   }
	if {$schain == ""} continue
	set expList ""
	chainEach hndl $schain {
	   foreach var "exposure tileId plateId mapId flavor expTime" {
		set $var [exprGet $hndl.$var]
		}
	   if {[lsearch $expList $exposure] >= 0} continue
	   lappend expList $exposure
	   if {$flavor == "arc" || $flavor == "flat" || $flavor == "target" \
		|| $flavor == "smear"} {
		if {$plateId < 0 || $plateId == 9999 || $tileId < 0 || \
		   $tileId == 9999 || $mapId < 0} {
		   echo Bad data $tail $exposure $tileId $plateId $mapId \
			$flavor
		   }
if {$plateId < 266} continue
		if {$flavor == "flat" && ($expTime > 60. || $expTime < 1.)} {
		   echo Bad exposure time?  $tail $exposure $flavor $expTime
		   }
		if {$flavor == "arc" && ($expTime > 60. || $expTime < 1.)} {
		   echo Bad exposure time? $tail $exposure $flavor $expTime
		   }
		if {$flavor == "target" && $expTime < 60.} {
		   echo Bad exposure time? $tail $exposure $flavor $expTime
		   }
		}
	   if {$flavor == "science"} {
		echo $tail $exposure flavor science
		}
	   }
	genericChainDel $schain
	}
   return
   }

#########################################################
#Crawl speclog and look for exposures that are commented out.
#Create sdHdrFix files in homeDir to capture existing edits to sdReport
#files in speclog

proc specLogOut {} {
   global logDir homeDir

#Positions of all keywords in sexp record
   set index(SEQID) 1
   set index(TILEID) 2
   set index(PLATEID) 3
   set index(MAPID) 4
   set index(FLAVOR) 5
   set index(QUALITY) 6
   set index(PROGRAM) 7
   set index(EXPOSURE) 8
   set index(EXPTIME) 9
   set index(MJD) 10
   set index(RA) 11
   set index(DEC) 12
   set index(CAMERA) 13
   set index(TARGETNAME) 14
   set index(TAIHMS) 15

   set files [lsort [glob -nocomplain $logDir/*/sdReport*.par]]
   foreach file $files {
	set tail [file tail $file]
	set lines [split [read_file $file] \n]
	set expList ""
	regexp {([0-9]+)} $tail all mjd
	foreach line $lines {
	   if {![regexp "^#sexp" $line] && ![regexp "#SEXP" $line]} continue
	   set exposure [lindex $line $index(EXPOSURE)]
	   set camera [lindex $line $index(CAMERA)]
	   set combo $exposure-$camera
	   if {[lsearch $expList $combo] >= 0} continue
	   echo $tail Exposure $combo commented out
	   lappend expList $combo
	   }

#Create new sdHdrFix file
	if {[llength $expList] > 0} {
	   set fixfile $homeDir/sdHdrFix-$mjd.par
	   set fid [open $fixfile w]
	   puts $fid "typedef struct {"
	   puts $fid "  char fileroot\[20\]; # Root of file name, without any \".fit\" suffix"
	   puts $fid "  char keyword\[9\]; # Keyword name"
	   puts $fid "  char value\[80\]; # Keyword value (as a string)"
	   puts $fid "  } OPHDRFIX;"
	   puts $fid ""
	   foreach combo $expList {
		set combo [split $combo -]
		set exposure [lindex $combo 0]
		set camera [lindex $combo 1]
		puts $fid "OPHDRFIX sdR-$camera-[format %08d \
		   $exposure] FLAVOR ignore"
		}
	   close $fid
	   } 
	}
   return
   }

################################################################
#Read an sdReport file and check for same flavor in all 4 cameras of an
#exposure

proc flavorCheck {file} {
   set chains [param2Chain $file hdr]
   set chain ""
   foreach ch $chains {
	if {[chainTypeDefine $ch] == "SEXP"} {
	   set chain $ch
	} else {
	   genericChainDel $ch
	   }
	}
   set expTable() ""
   set shortList ""
   set flavorList ""
   chainEach hndl $chain {
	foreach var "exposure camera flavor" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $expTable() $exposure] < 0} {
	   lappend expTable() $exposure
	   set expTable($exposure,camera) ""
	   set expTable($exposure,flavor) ""
	   }
	lappend expTable($exposure,camera) $camera
	lappend expTable($exposure,flavor) $flavor
	}
   foreach exp $expTable() {
	if {[llength $expTable($exp,flavor)] < 4} {
	   lappend shortList $exp
	   }
	set type [lindex $expTable($exp,flavor) 0]
	foreach flavor $expTable($exp,flavor) {
	   if {$flavor != $type} {
		lappend flavorList $exp
		break
		}
	   }
	}
   if {[llength $shortList] != 0} {
	echo [file tail $file]: Not enough cameras: $shortList
	}
   if {[llength $flavorList] != 0} {
	echo [file tail $file]: Flavor mismatch: $flavorList
	}
   genericChainDel $chain
   return
   }
################################################################
proc flavorCheckAll {} {
   global homeDir
   set files [lsort [glob -nocomplain $homeDir/sdReport-*.par]]
   echo Checking [llength $files] files in $homeDir
   foreach file $files {
	flavorCheck $file
	}
   return
   }

################################################################
