###############################################################
#Create a file with the status of segue chunks, analogous to odChunkStatus
#file for regular survey.

global dataRoot
set dataRoot /data/dp30.a/data

proc segueChunk {} {
   global dataRoot

   set mjd [expr [getclock]/86400 + 40587]
   set segfile segueChunk-$mjd.par
   set fid [open $segfile w]
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   char strip<2>;"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int camcol;"
   puts $fid "   int startField;"
   puts $fid "   int endField;"
   puts $fid "   double slongStart;"
   puts $fid "   double slongEnd;"
   puts $fid "   int version;"
   puts $fid "   int startMu;"
   puts $fid "   int endMu;"
   puts $fid "   int startMuSeg;"
   puts $fid "   int endMuSeg;"
   puts $fid "   char status<20>;"
   puts $fid "   } CHUNK;"

#Read file with list of SEGUE scans
#Return if nothing to plan.
   if {![file exists segue.par]} return
   set chain [param2Chain segue.par hdr]
   if {[chainSize $chain] == 0} {
	genericChainDel $chain
	return
	}

   set bestRoot $dataRoot/bestRoot

   set stripes ""
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	if {[lsearch $stripes $stripe] < 0} {
	   lappend stripes $stripe
	   }
	}
   genericChainDel $chain

   foreach stripe $stripes {
	set files [lsort [glob -nocomplain $bestRoot/stripe$stripe*]]
	foreach file $files {
	   set tail [file tail $file]
	   if {![regexp {stripe([0-9]+)_mu([0-9]+)_([0-9]+)} \
		$tail all stripe chunk version]} continue
	   if {$stripe > 100} {
		set list [stripeLimits $stripe]
		set node [keylget list node]
		set incl [keylget list incl]
	   } else {
		set node 95.
		set incl [inclFromStripeNumber $stripe]
		}
	   set chunkFile $file/tsChunk-$stripe-$chunk-$version.par
	   if {![file exists $chunkFile]} {
		echo Weird - no chunkFile $chunkFile
		continue
		}
	   set chain [param2Chain $chunkFile hdr]
	   chainEach hndl $chain {
		foreach param "run camCol rerun stripeNumber strip startMu \
		   endMu field0 nFields status" {
		   set $param [exprGet $hndl.$param]
		   }

		set endField [expr $field0 + $nFields - 1]

#For segue stripes, survey longitude is nonsense.  However, I formally
#define it as mu - 185.
		set slongStart [expr int(round($startMu/3600.)) - 185]
		if {$slongStart >= 180} {set slongStart [expr $slongStart-360]}
		if {$slongStart < -180} {set slongStart [expr $slongStart+360]}
		set slongEnd [expr int(round($endMu/3600.)) - 185]
		if {$slongEnd >= 180} {set slongEnd [expr $slongEnd-360]}
		if {$slongEnd < -180} {set slongEnd [expr $slongEnd+360]}

#For SEGUE, best and target are the same.  For making data releases, I want
#target version
		set version 0
		puts $fid "chunk $stripe $strip $run $rerun $camCol \
		   $field0 $endField $slongStart $slongEnd $version $startMu \
		   $endMu $startMu $endMu $status"
		}
	   genericChainDel $chain
	   }
	}
   close $fid
   return $segfile
   }



