#Make a plot of segue stripes in equatorial coords.

########################################################################
#Make one summary plot of data in currentStripeStatus.par
#Make a gif unless Xwindow is open.
#Use RA, DEC

global dataRoot
set dataRoot /data/dp30.a/data

proc seguePlotEq {} {
   global stats
   set flag 0
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin segueEqSummary.gif/VGIF
	pgGeomSet 900 600 0 0
	pgAsk 0
	set flag 1
	}

#Routine to make a plot of the survey area that has been completed.

   pgEnv 360 0 -30 90 0 0
   pgLabel {RA} {DEC} {SEGUE Layout, Imaging, and Plate Coverage}

   gpPlot
	
#Plot observations.  Use currentStripeStatus.par
   set file currentStripeStatus.par
   if {[file exists $file]} {
	set chain [param2Chain $file hdr]
	chainEach hndl $chain {
	   foreach var "run stripe strip lambdaStart lambdaEnd status" {
		set $var [exprGet $hndl.$var]
		}
	   if {$stripe <= 100  && $stripe != 72 && $stripe != 79 && \
		$stripe != 90} continue
	   segueDataPlot $stripe $strip $lambdaStart $lambdaEnd $status
	   }
	genericChainDel $chain
	}

   set chain [param2Chain segue.par hdr]
   pgSci 1
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	set strip [exprGet $hndl.strip]
	set node [exprGet $hndl.node]
	set incl [exprGet $hndl.incl]
	set muStart [exprGet $hndl.muMin]
	set muEnd [exprGet $hndl.muMax]
	set len [expr $muEnd - $muStart]
	if {$muEnd < $muStart} {
	   set len [expr $len+360.]
	   }
	segueScanPlot $stripe $strip $node $incl $muStart $len
	}
   genericChainDel $chain

#Sometimes Yanny resolves chunks but does not make plates - this makes
#plate planning a challenge!  I will read the latest plateList and plot
#all segue plates.
   set file [lindex [lsort [glob -nocomplain plateList-*.par]] 0]
   if {$file == ""} return
   set chain [param2Chain $file hdr]
   chainEach hndl $chain {
	foreach var "raCen decCen tileRun" {
	   set $var [exprGet $hndl.$var]
	   }
	if {![regexp {segue[0-9]+} $tileRun]} continue
	pgSci 4
	pgCirc $raCen $decCen 1.25
	}
   genericChainDel $chain
   if {$flag == 1} {echo pgEnd; pgEnd}
   return
   }

#####################################################################
#Plot scan layout

proc segueScanPlot {stripe strip node incl muStart lengthOfRun} {
   global dataRoot
    
   pgSci 1
   if {$stripe > 100 && $stripe < 1000} {pgSci 3}

   set nu 0
   if {$strip == "S"} {
	   set nu -.625
	   }
   if {$strip == "N"} {
	   set nu .625
	   }
   set xlist ""
   set ylist ""
   set ra1 0.0001
   loop i 0 $lengthOfRun {
	set mu [expr $i + $muStart]
	set list [GCToEq $mu $nu -node $node -inclination $incl]
	set ra [keylget list ra]
	set dec [keylget list dec]
	if {$ra < $ra1} {
	   pgLine $xlist $ylist
	   set xlist ""
	   set ylist ""
	   }
	lappend xlist $ra
	lappend ylist $dec
	set ra1 $ra
	if {$i == int($lengthOfRun/2) && $nu >= 0} {
	   pgText $ra $dec $stripe
	   }
	}
   pgLine $xlist $ylist
   pgSci 1

#Plot target chunks
   set bestRoot $dataRoot/bestRoot
   set files [lsort [glob -nocomplain $bestRoot/stripe$stripe*]]
   foreach file $files {
	set tail [file tail $file]
	if {![regexp {stripe([0-9]+)_mu([0-9]+)_([0-9]+)} \
	   $tail all stripe chunk version]} continue
	if {$stripe > 100} {
	   set list [stripeLimits $stripe]
	   set node [keylget list node]
	   set incl [keylget list incl]
	} else {
	   set node 95.
	   set incl [inclFromStripeNumber $stripe]
	   }
	set chunkFile $file/tsChunk-$stripe-$chunk-$version.par
	if {![file exists $chunkFile]} {
	   echo Weird - no chunkFile $chunkFile
	   continue
	   }
	set chain [param2Chain $chunkFile hdr]
	chainEach hndl $chain {
	   foreach param "run camCol rerun stripeNumber strip startMu \
		endMu field0 nFields" {
		set $param [exprGet $hndl.$param]
		}
	   set startMu [expr $startMu/3600.]
	   set endMu [expr $endMu/3600.]
	   set nplate [expr ($endMu-$startMu)/2.2]
	   for {set i 0} {$i <= $nplate} {incr i} {
		set mu [expr $startMu + 2.2*$i]
		set nu 0.
		set list [GCToEq $mu $nu -node $node -inclination $incl]
		set ra [keylget list ra]
		set dec [keylget list dec]
		pgSci 7
		pgCirc $ra $dec 1.25
		pgSci 1
		}
	   }
	genericChainDel $chain
	}
   return
   }

#####################################################################
#Plot galactic plane - 2 lines at +/- 12.5 degrees latitude.

proc gpPlot {} {
   pgSci 4
   set xlist ""
   set ylist ""
   set ra1 .0001
   loop i 0 361 {
	set line [galToEq $i 12.5]
	set ra [keylget line ra]
	set dec [keylget line dec]
	if {$ra < $ra1} {
	   pgLine $xlist $ylist
	   set xlist ""
	   set ylist ""
	   }
	set ra1 $ra
	lappend xlist $ra
	lappend ylist $dec
	}
   pgLine $xlist $ylist

   set xlist ""
   set ylist ""
   set ra1 .0001
   loop i 0 361 {
	set line [galToEq $i -12.5]
	set ra [keylget line ra]
	set dec [keylget line dec]
	if {$ra < $ra1} {
	   pgLine $xlist $ylist
	   set xlist ""
	   set ylist ""
	   }
	set ra1 $ra
	lappend xlist $ra
	lappend ylist $dec
	}
   pgLine $xlist $ylist
   pgSci 1
   return
   }

#####################################################################
#Plot imaging data layout

proc segueDataPlot {stripe strip lambdaStart lambdaEnd quality} {
   global dataRoot
    
   if {$stripe < 100} {
	set node 95.
	set incl [inclFromStripeNumber $stripe]
   } else {
	set line [stripeLimits $stripe]
	set node [keylget line node]
	set incl [keylget line incl]
	}

   pgSci 3
   if {[string tolower $quality] == "bad"} {pgSci 2}

   set nu 0
   if {$strip == "S"} {
	   set nu -.625
	   }
   if {$strip == "N"} {
	   set nu .625
	   }
   set lengthOfRun [expr $lambdaEnd - $lambdaStart + 1.]
   if {$lengthOfRun < 0} {set lengthOfRun [expr $lengthOfRun + 360.]}
   set muStart [expr $lambdaStart + 185.]

   loop i 0 $lengthOfRun {
	set mu [expr $i + $muStart]
	set list [GCToEq $mu $nu -node $node -inclination $incl]
	set ra [keylget list ra]
	set dec [keylget list dec]
	pgCirc $ra $dec .7
	}
   pgSci 1
   return
   }


