proc show {item {arg1 ""}} {
   imgGlobals
   if {$item == "axes"} {
	if {![info exists img(axes)]} return
	set row [exprGet $img(axes).row]
	set col [exprGet $img(axes).col]
	set angle [exprGet $img(axes).pos_maj]
	set eps [exprGet $img(axes).eps]
	set sigmaj [exprGet $img(axes).sigmaj]
	set sigmin [exprGet $img(axes).sigmin]
	set rmax [exprGet $img(axes).rmax]
	puts stdout [format "Row: %7.2f, Col: %7.2f, Pos. Ang.: %7.2f, \
Ellipticity: %5.2f" $row $col $angle $eps]
	puts stdout [format "Sig maj: %7.2f, Sig min: %7.2f    Rmax: %7.0f" \
	   $sigmaj $sigmin $rmax]
	return
	}
   if {$item == "photo"} {
	if {![info exists img(photonet)]} return
	set total $img(phototot)
	set net $img(photonet)
	set area $img(photoarea)
	puts stdout [format "Total: %10.1f,   Net: %10.1f,   Area: %10.2f" \
	   $total $net $area]
	return
	} 
   if {$item == "profile"} {
	if {![info exists img(profile)]} return
	if {![info exists img(sky)]} return
	set format "%6s %8s %8s %7s %5s %10s %8s %8s"
	puts stdout [format $format Index Xc Yc Ang Ecc Flux A2 B2]
	set i 0
	chainEach hndl $img(profile) {
	   set xc [format %.2f [exprGet $hndl.xc]]
	   set yc [format %.2f [exprGet $hndl.yc]]
	   set ang [format %.2f [exprGet $hndl.ang]]
	   set ecc [format %.2f [exprGet $hndl.ecc]]
	   set intensity [exprGet $hndl.intensity]
	   set flux [expr $intensity - $img(sky)]
	   set flux [format %.2f $flux]
	   set a2 [format %.2f [exprGet $hndl.a2]]
	   set b2 [format %.2f [exprGet $hndl.b2]]
	   puts stdout [format $format $i. $xc $yc $ang $ecc $flux $a2 $b2]
	   incr i
	   }
	return
	}

#Profile as a magnitude
   if {$item == "prmag"} {
	if {![info exists img(profile)]} return
	if {![info exists img(sky)]} return
	set format "%6s %8s %8s %7s %5s %10s"
	puts stdout [format $format Index Xc Yc Ang Ecc Flux]
	set i 0
	if {$arg1 == ""} {
	   set xmag 20.
	} else {
	   set xmag $arg1
	   }
	chainEach hndl $img(profile) {
	   set xc [format %.2f [exprGet $hndl.xc]]
	   set yc [format %.2f [exprGet $hndl.yc]]
	   set ang [format %.2f [exprGet $hndl.ang]]
	   set ecc [format %.2f [exprGet $hndl.ecc]]
	   set intensity [exprGet $hndl.intensity]
	   set flux [expr $intensity - $img(sky)]
	   if {$flux > 0.} {
		set flux [expr -2.5*log10($flux) + $xmag]
	   } else {
		set flux 0.
		}
	   set flux [format %.2f $flux]
	   puts stdout [format $format $i. $xc $yc $ang $ecc $flux]
	   incr i
	   }
	return
	}

#Annular profiles use an abbreviated profile structure, although both
#handles are stashed in img(profile).
   if {$item == "annulus"} {
	if {![info exists img(profile)]} return
	if {![info exists img(sky)]} {
	   echo Need some sky!
	   return
	   }
	set format "%6s %7s %5s %10s %15s %10s"
	puts stdout [format $format Radius Ang Eps Counts Cumulative Fract]
	set i 0
	set step 1
	if {$arg1 != ""} {set step $arg1}
	set last [chainGet $img(profile) [expr [chainSize $img(profile)]-1]]
	set max [exprGet $last.cum]
	set rad [exprGet $last.rad]
	if {$max == 0} {set max 1.}
	set max [expr $max - $img(sky)*3.14159265*$rad*$rad]
	handleDel $last
	loop i 0 [chainSize $img(profile)] $step {
	   set hndl [chainGet $img(profile) $i]
	   if {$hndl == ""} break
	   set ang [format %.2f [exprGet $hndl.ang]]
	   set eps [format %.2f [exprGet $hndl.eps]]

#Need to sky subtract annulus.
	   set flux [format %.2f [expr [exprGet $hndl.cnts] - $img(sky)]]
	   set cum [expr [exprGet $hndl.cum] - $img(sky)*3.14159265*$i*$i]
	   set fract [format %.2f [expr $cum/$max]]
	   set cum [format %.2f $cum]
	   puts stdout [format $format $i. $ang $eps $flux $cum $fract]
	   handleDel $hndl
	   }
	return
	}
   if {$item == "sky"} {
	if {![info exists img(sky)]} return
	set format "Sky = %.3f, noise = %.3f"
	puts stdout [format $format $img(sky) $img(skynoise)]
	return
	}
   return
   }
