#Run this in astrotools
#Read the astrom files for SIRTF runs, compute the separation from the
#field center, take everything in a 1.4 deg radius circle about the
#center.
#Commands:
#	sirtf  (defines field boundaries; runs on fsgi03 or sdsslnx.  Needs
#			plan.par)
#	sirtfCopy (Copies files to sirtf directory.  fsgi03 only)
#	sirtfGifTar (Makes tar file of sirtf gifs.  sdsslnx only)

#On fsgi03, I will work from the dp directory, since all files are
#on disk.

if {[exec uname -n] == "fsgi03"} {
   set dir /usr/sdss/data04/skent/cdrom
} else {
   set dir /data/x1.3/skent/cdrom
   }
source $dir/init.tcl

global sirtfruns
set sirtfruns "1336 1339 1356 1359"

proc sirtf {} {
   global astrom
   global fmin fmax sirtfruns
   cdGlobals plan.par
   if {[exec uname -n] == "fsgi03"} {
	set masterdir /usr/sdss/data04/skent/dp
   } else {
	set masterdir /data/dp3.b/data
	}
   set racen [expr (17.+18./60.)*15.]
   set decen [expr 59.+30./60.]

   foreach run $sirtfruns {
	set asdir($run) $masterdir/$run/astrom
	astromRead $run
#Get min, max field, camcol foreach run
	set ccmin($run) 6
	set ccmax($run) 1
	set fmin($run) 10000
	set fmax($run) 0
	set gcmin($run) 360
	set gcmax($run) 0
	foreach camcol "1 2 3 4 5 6" {
#echo
#echo run $run camcol $camcol
	   chainEach hndl $astrom($run,$camcol,r) {
		foreach var "field a d" {
		   set $var [exprGet $hndl.$var]
		   }
		set list [GCToEq $a $d -node $astrom($run,node) -inclination \
		   $astrom($run,incl)]
		set ra [keylget list ra]
		set dec [keylget list dec]
		set cd [expr cos($dec*3.141593/180.)]
		set sep [expr sqrt(pow(($ra-$racen)*$cd,2) + \
			pow($dec-$decen,2))]
		if {$sep < 1.414} {
#		   echo $run $camcol $field
		   set ccmin($run) [min $ccmin($run) $camcol]
		   set ccmax($run) [max $ccmax($run) $camcol]
		   set fmin($run) [min $fmin($run) $field]
		   set fmax($run) [max $fmax($run) $field]
		   set gcmin($run) [min $gcmin($run) $a]
		   set gcmax($run) [max $gcmax($run) $a]
		   }
		}
	   }
	}
   foreach run $sirtfruns {

#Increase gcmax by 1 field size
	set gcmax($run) [expr $gcmax($run) + 1361.*.4/3600.]
	set gcmin($run) [format %.2f $gcmin($run)]
	set gcmax($run) [format %.2f $gcmax($run)]
	echo Run $run camccol = $ccmin($run)-$ccmax($run), field = \
	   $fmin($run)-$fmax($run) gclong = $gcmin($run)-$gcmax($run)
	}
   return
   }

#########################################################################
#Copy files into a build directory for this product

proc sirtfCopy {} {
   global fmin fmax sirtfruns
   if {![info exists fmin]} {sirtf}
   set masterdir /usr/sdss/data04/skent/dp
   set sirtfdir /usr/sdss/data04/skent/sirtf
   if {![file exists $sirtfdir]} {
	exec mkdir $sirtfdir
	}
#   set sirtfruns "1336 1339 1356 1359"
   foreach run $sirtfruns {
	set sourcedir $masterdir/$run
	if {![file exists $sourcedir]} {
	   echo No source directory $sourcedir!
	   continue
	   }
	set rundir $sirtfdir/$run
	if {[file exists $rundir]} {
	   exec rm -r $rundir
	   }
	exec mkdir $rundir

#Copy over astrom calibration
	set sfile [glob -nocomplain $sourcedir/astrom/asTrans*.fit]
	if {[file exists $sfile]} {
	   set asdir $rundir/astrom
	   exec mkdir $asdir
	   echo Copying $sfile
	   exec cp $sfile $asdir
	} else {
	   echo No astrometric source file found
	   }

#Copy over photometric calibrations
	set sfiles [glob -nocomplain $sourcedir/nfcalib/fcPCalib*.fit]
	set fcdir $rundir/nfcalib
	exec mkdir $fcdir
	foreach file $sfiles {
	   echo Copying $file
	   exec cp $file $fcdir
	   }

#Copy over objc files
	set photodir $rundir/objcs
	exec mkdir $photodir
	foreach camcol "1 2 3 4 5 6" {
	   set sdir $sourcedir/objcs/$camcol
	   if {![file exists $sdir]} {
		echo No source camcol directory $sdir
		continue
		}
	   set cdir $photodir/$camcol
	   exec mkdir $cdir
	   set fields [fieldExpand $fmin($run)-$fmax($run)]
	   foreach field $fields {
		foreach ftype "fpBIN fpAtlas fpFieldStat fpM fpObjc" {
		   set files [glob -nocomplain $sdir/$ftype-*$field.fit]
		   foreach file $files {
			echo Copying $file
			exec cp $file $cdir
			}
		   }
		}
	   }

#Copy over tsObj files
	set tsdir $rundir/calibChunks
	exec mkdir $tsdir
	foreach camcol "1 2 3 4 5 6" {
	   set sdir $sourcedir/calibChunks/$camcol
	   if {![file exists $sdir]} {
		echo No source camcol directory $sdir
		continue
		}
	   set cdir $tsdir/$camcol
	   exec mkdir $cdir
	   set fields [fieldExpand $fmin($run)-$fmax($run)]
	   foreach field $fields {
		foreach ftype "tsObj tsField" {
		   set files [glob -nocomplain $sdir/$ftype-*$field.fit]
		   foreach file $files {
			echo Copying $file
			exec cp $file $cdir
			}
		   }
		}
	   }

	}
   return
   }

#####################################################################
#This proc runs on sdsslnx

proc sirtfGifTar {} {
   cdGlobals
   global fmin fmax sirtfruns
   if {[exec uname -n] == "fsgi03"} {
	set cdrom /usr/sdss/data04/skent/cdrom
	set tardir /usr/sdss/data04/skent/cdrom
   } else {
	set cdrom /data/x1.3/skent/cdrom
	set tardir /data/x1.1/skent/cdrom
	}
   set here [pwd]
   cd $cdrom

#Compute fmin fmax   
   if {![info exists fmin]} sirtf
   set tarfile $tardir/sirtfwww.tar
   if {[file exists $tarfile]} {exec rm $tarfile}
   set command "tar cvf $tarfile www/template/objpos.htm"

#Search for mosaic-dirs to tar
   foreach run $sirtfruns {
	set rundir www/run-$run
	append command " $rundir/back.htm"
	append command " $rundir/info.htm"
	append command " $rundir/stripe.htm"
	append command " $rundir/mosdir.htm"

	set mdirs [glob -nocomplain $rundir/mosaic*]
	foreach mdir $mdirs {
	   if {![regexp mosaic-(\[0-9\]+)-(\[0-9\]+) $mdir all field1 \
		field2]} continue
	   if {$field1 > $fmax($run) || $field2 < $fmin($run)} continue
	   append command " $mdir"
	   }
	}
   echo command is $command
   eval exec $command
   echo Created $tarfile
   cd $here
   return
   }

####################################################################
#Fixup mosdir.htm for missing mosaic directories
#This should run only on fsgi03 after we have untarred sirtfwww.tar

proc sirtfMosdirFix {} {
   cdGlobals
   global sirtfruns
#Fixup mosdir.htm for missing 

   if {[exec uname -n] == "fsgi03"} {
	set topdir /usr/sdss/data04/skent/sirtf
   } else {
	set topdir /data/x1.3/skent/sirtf
	}

   foreach run $sirtfruns {
	set wwwdir $topdir/www/run-$run
	set file $wwwdir/mosdir.htm
	set outfile $wwwdir/mosdirfix.htm

#Get list of mosaics
	set mosdirs [glob -nocomplain $wwwdir/mosaic*]
	set mosaics ""
	foreach mosdir $mosdirs {
	   set mosaic [file tail $mosdir]
	   lappend mosaics $mosaic
	   }

#Open mosdir.htm, read lines, chuck any that refer to a non-existent
#mosaic
	set fid [open $file]
	set outid [open $outfile w]
	while {1} {
	   set line [gets $fid]
	   if {[eof $fid]} break
	   if {![regexp mosaic- $line]} {
		puts $outid $line
		continue
		}
	   regexp "mosaic-\[0-9\]+-\[0-9\]+" $line all
	   if {[lsearch $mosaics $all] < 0} {
		echo Dropping line $line
	   } else {
		puts $outid $line
		}
	   }
	close $fid
	close $outid
	exec mv $outfile $file
	}

#Now make master directory
   set masterfile $topdir/www/index.html
   set fid [open $masterfile w]
   puts $fid "<head><title>SIRTF FLS</title></head>"
   puts $fid "<body>"
   puts $fid "<h1 align=center>SIRTF First Light Survey</h1>"
   puts $fid "<h1 align=center>Sloan Digital Sky Survey Data</h1>"
   puts $fid "<table align=center>"
   foreach run $sirtfruns {
	puts $fid "<tr><td><a href=run-$run/stripe.htm>Run $run</a></td></tr>"
	}
   puts $fid "</table>"
   puts $fid "<p><p><p><font size=-1>The Sloan Digital Sky Survey (SDSS) \
	is a joint project of The University of Chicago, Fermilab, the \
	Institute for Advanced Study, the Japan Participation Group,
	The Johns Hopkins University, the Max-Planck-Institute for \
	Astronomy, Princeton University, the United States Naval Observatory, \
	and the University of Washington. Apache Point Observatory, site of \
	the SDSS, is operated by the Astrophysical Research Consortium. \
	Funding for the project has been provided by the Alfred P. Sloan \
	Foundation, the SDSS member institutions, the National Aeronautics \
	and Space Administration, the National Science Foundation, the U.S. \
	Department of Energy, and Monbusho. The SDSS Web site is \
	<a href=http://www.sdss.org/>http://www.sdss.org</a>."
   puts $fid "</body>"
   close $fid
   return
   }

