#Fetch photometric zero points and sky values from opdb
#Useful commands:
#skyGet run camcol [rerun] - basic routine to fetch skies
#skyPlot run camcol filter - plot sky
#skyTwiCheck run           - Check field limits for one run
#skyCheckRuns              - Check all runs in opdb

proc skyGet {run camcol {rerun ""}} {
   global skies runTable
   if {![inSession]} {
	opdbConnect
	}
   if {![info exists runTable]} {
	runInventory
	}
   if {$rerun == ""} {set rerun $runTable($run)}
   set frames [pipeGet frames $run $camcol -rerun $rerun]
   set calib [$frames -> currentPhotometricCalibration]
   set field0 [$frames -> field0]
   set nFields [$frames -> nFields]

#Fetch skies
   foreach filter "u g r i z" {
	set skies($run,$camcol,$filter) ""
	}
   set skies($run,$camcol,field) ""
   loop field $field0 [expr $nFields+$field0] {
	set objList [$frames -> objectList $field]
	foreach filter "u g r i z" {
	   set string AR_[string toupper $filter]
	   set int [enumStringToInt arFilter $string]
	   set frameStat [$objList -> frameStat $int]
	   set sky [$frameStat sky_frames]
	   handleDel $frameStat
	   lappend skies($run,$camcol,$filter) [format %.2f $sky]
	   }
	lappend skies($run,$camcol,field) $field
	objDel $objList
	}
   objDel $frames
   sessionEnd
   return
   }

#################################################################
proc skyPlot {run camcol filter} {
   global skies
   if {![info exists skies($run,$camcol,field)]} {
	skyGet $run $camcol
	}
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}
   set fmin [lindex $skies($run,$camcol,field) 0]
   set fmax [lindex $skies($run,$camcol,field) end]
   set skymax [eval max $skies($run,$camcol,$filter)]
   pgEnv $fmin $fmax 0 $skymax 0 0
   set i 0
   foreach field $skies($run,$camcol,field) {
	set sky [lindex $skies($run,$camcol,$filter) $i]
	if {$sky == ""} continue
	incr i
	pgPoint $field $sky 3
	}
   return
   }

#####################################################################
#Check for twilight.  Look in u filter, camcol 3

proc skyTwiCheck {run} {
   global skies runTable
   if {![info exists skies($run,3,field)]} {
	skyGet $run 3
	}

#Evening twilight - go in forward direction
#First, median
   set median [lindex [lsort -real $skies($run,3,u)] \
	[expr [llength $skies($run,3,u)]/2]]
   loop i 0 [llength $skies($run,3,field)] {
	set field [lindex $skies($run,3,field) $i]
	set sky [lindex $skies($run,3,u) $i]
	if {$sky < 1.5*$median} break
	}
   set fieldEve $field

#Morning twilight - go in reverse direction
#First, median
   set median [lindex [lsort -real $skies($run,3,u)] \
	[expr [llength $skies($run,3,u)]/2]]
   loop j 0 [llength $skies($run,3,field)] {
	set i [expr [llength $skies($run,3,field)] - $j - 1]
	set field [lindex $skies($run,3,field) $i]
	set sky [lindex $skies($run,3,u) $i]
	if {$sky < 1.5*$median} break
	}
   set fieldMorn $field

   set first [lindex $skies($run,3,field) 0]
   set last [lindex $skies($run,3,field) end]
   if {$fieldEve > $first} {
	echo Run $run, recommend first $first -> $fieldEve
	}
   if {$fieldMorn < $last} {
	echo Run $run, recommend last $last -> $fieldMorn
	}
   return
   }

####################################################################
proc skyCheckRuns {} {
   global runTable
   if {![info exists runTable]} {
	runInventory
	runMJD
	}
   foreach run [lsort -integer $runTable()] {
	skyTwiCheck $run
	}
   return
   }



