#Create observing plans for south and also for SEGUE.
#
#New commands:
#	seguePlan <mjd> [fid]
#	GCPlan <mjd> [fid]
#	trackPlan <mjd> [fid]
#
#South:
#Compute start, end times for south equatorial survey.
#Segments are 36 degrees long.  Start lambda is 126, 162, -162.
#Observing HA range is +/- 23 degrees.

#For now, input sidereal time at midnight

proc southPlan {mjd {outid stdout}} {
   puts $outid "--------------"
   puts $outid "South Area"
   puts $outid ""
   puts $outid "Repeat scans on stripe 82 N/S, all longitudes between \
        126 and -126."
   puts $outid ""
   puts $outid "Strip  Start   End  Start-Time-Range  Length(hr) Initial"
   puts $outid "       Longitude           (UT)                 Priority"
   puts $outid ""
   set format "%2s%2s%7d%7d%7s%7s%8.1f%7d"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO
   set stmid [expr $stmid - 7.*15.]

#For priority (N or S), compute year 3 months previously
   set yrprio [fmtclock [expr [getclock] - 3*30*86400] %Y]

#Odd years are north, even years are south
   if {($yrprio/2)*2 == $yrprio} {
	set nprio 2
	set sprio 3
   } else {
	set nprio 3
	set sprio 2
	}
   foreach lambda "126 162 -162" {
	set ra [expr $lambda+185]
	set lambdamax [expr $lambda+36]
	if {$lambdamax >= 180} {set lambdamax [expr $lambdamax-360]}
	if {$lambda == 126} {
	   set stmin [expr $ra-23]
	} else {
	   set stmin [expr $ra-13]
	   }
	set stmax [expr $ra+23]

#mtmin and mtmax are actually in UT.
	set mtmin [format %.0f [expr $stmin-$stmid]]
	set mtmax [format %.0f [expr $stmax-$stmid]]
	if {$mtmin < 0} {set mtmin [expr $mtmin+360]}
	if {$mtmin >= 360} {set mtmin [expr $mtmin-360]}
	if {$mtmax < 0} {set mtmax [expr $mtmax+360]}
	if {$mtmax >= 360} {set mtmax [expr $mtmax-360]}
	set minhr [expr $mtmin/15]
	set minmin [format %02d [expr ($mtmin-$minhr*15)*4]]
	set maxhr [expr $mtmax/15]
	set maxmin [format %02d [expr ($mtmax-$maxhr*15)*4]]
	set line [format $format 82 N $lambda $lambdamax $minhr:$minmin \
	   $maxhr:$maxmin 2.4 $nprio]
	puts $outid $line
	puts $outid ""
	}

   foreach lambda "126 162 -162" {
	set ra [expr $lambda+185]
	set lambdamax [expr $lambda+36]
	if {$lambdamax >= 180} {set lambdamax [expr $lambdamax-360]}
	if {$lambda == 126} {
	   set stmin [expr $ra-23]
	} else {
	   set stmin [expr $ra-13]
	   }
	set stmax [expr $ra+23]
	set mtmin [format %.0f [expr $stmin-$stmid]]
	set mtmax [format %.0f [expr $stmax-$stmid]]
	if {$mtmin < 0} {set mtmin [expr $mtmin+360]}
	if {$mtmin >= 360} {set mtmin [expr $mtmin-360]}
	if {$mtmax < 0} {set mtmax [expr $mtmax+360]}
	if {$mtmax >= 360} {set mtmax [expr $mtmax-360]}
	set minhr [expr $mtmin/15]
	set minmin [format %02d [expr ($mtmin-$minhr*15)*4]]
	set maxhr [expr $mtmax/15]
	set maxmin [format %02d [expr ($mtmax-$maxhr*15)*4]]
	set line [format $format 82 S $lambda $lambdamax $minhr:$minmin \
	   $maxhr:$maxmin 2.4 $sprio]
	puts $outid $line
	puts $outid ""
	}
   puts $outid "If > 15 degrees of a scan is observed, priority ==>> 1 for \
	following 2 days,"
   puts $outid "then returns to original on 3rd day."
   puts $outid "If a scan reaches the end of the last southern stripe and \
	no other scan"
   puts $outid "is available, continue scanning on the equator into the \
	Galactic plane."
   return
   }

###############################################################
#For now, read stripes from a file and output
#We do the work of stripePlan and obsPlan in one.
#We do NOT extend segments to get patches.

proc seguePlan {mjd {outid stdout}} {

#Read stripeExclude, if it exists.  By now I have several stripes with
#little bits not yet finished that are not worth chasing after.
#It is easiest just to wipe them out by hand.
   set exclude ""
   if {[file exists stripeExclude]} {
        param2Chain stripeExclude hdr
        set exclude [keylget hdr exclude]
        }

#Read file with list of SEGUE scans
#Return if nothing to plan.
   if {![file exists segue.par]} return
   set chain [param2Chain segue.par hdr]
   if {[chainSize $chain] == 0} {
	genericChainDel $chain
	return
	}
   mtstdRead
   set files [lsort [exec find . -name todo*.par]]
   set file [lindex $files end]
   if {[string length $file] == 0} return
   set todo [param2Chain $file hdr]

   puts $outid "--------------"
   puts $outid "SEGUE Area"
   puts $outid "                                      Code for SEGUE Scans:"
   puts $outid "Rank ordered                          X Pre-existing data"
   puts $outid "by priority                           E Stripe endpoint"
   puts $outid "                                      * Area is to be scanned, but no data yet"
   puts $outid "                                      1st symbol is start of scan; 2nd is"
   puts $outid "                                        end of scan"
   puts $outid ""
   puts $outid "Strip    Start   End  Start-Time-Range  Length(hr)"
   puts $outid "          Mu      Mu        (UT)"
   puts $outid ""
   set format "%4s%2s%7d%7d%7s%7s%8.1f"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO for 0H UT
   set stmid [expr $stmid - 7.*15.]
   if {$stmid < 0} {set stmid [expr $stmid+360.]}

   set stripes ""
   chainEach hndl $chain {
	set stripe [exprGet $hndl.stripe]
	set node($stripe) [exprGet $hndl.node]
	set incl($stripe) [exprGet $hndl.incl]
	if {[lsearch $stripes $stripe] < 0} {lappend stripes $stripe}
	}

   foreach stripe $stripes {
	if {$stripe < 200} continue
	if {[lsearch $exclude $stripe] >= 0} continue
	echo Segue stripe $stripe

#Low numbered stripes only get one pass.  Nominally this is supposed to
#be an O strip, but I've started using N, so let's go with that.

	if {$stripe > 290} {
	   set strips "S N"
	} else {
	   set strips "N"
	   }
	foreach strip $strips {
	   chainEach hndl $todo {
		foreach var "stripe strip slongStart slongEnd" {
		   set c$var [exprGet $hndl.$var]
		   }
		if {$stripe != $cstripe || $strip != $cstrip} continue
		set muMin [expr $cslongStart + 185.]
		if {$muMin >= 360.} {set muMin [expr $muMin-360.]}
		set muMax [expr $cslongEnd + 185.]
		if {$muMax >= 360.} {set muMax [expr $muMax-360.]}

#Stripe less than 200 are now planned as part of the normal northern survey
#observing.  We use survey coords, not GC coords for those anyway, so
#seguePlan is not appropriate.
		if {$stripe < 200} continue
		set diff [expr $muMax-$muMin]
		if {$diff < 0.} {set diff [expr $diff+360.]}
		if {$diff > 360.} {set diff [expr $diff-360.]}
		set nseg 1

#Break up segments longer than 3 hrs in length.
		if {$diff > 45.1} {set nseg 2}
		if {$diff > 90.2} {set nseg 3}

#Stripe 1260 is a real pain!!!
if {$stripe == 1260} {set nseg 10}
		set lseg [expr $diff/$nseg]
		loop i 0 $nseg {
		   set mumin [expr $muMin + $i*$lseg]
		   set mumax [expr $mumin + $lseg]
		   set utlimits [utGCLimits $mumin $mumax $node($stripe) \
			 $incl($stripe) $stmid]
		   if {[string length $utlimits] == 0} {
			echo limits not optimal for stripe $stripe
			continue
			}
		   set ut1 [hm [expr [lindex $utlimits 0]/15.]]
		   set ut2 [hm [expr [lindex $utlimits 1]/15.]]
		   set len [expr ($mumax-$mumin)/15.]
		   if {$len < 0.} {set len [expr $len+360.]}
		   if {$len > 360.} {set len [expr $len-360.]}
		   if {$mumin > 360.} {set mumin [expr $mumin-360.]}
		   if {$mumax > 360.} {set mumax [expr $mumax-360.]}
		   set mumin [format %.0f $mumin]
		   set mumax [format %.0f $mumax]
		   set line [format $format $stripe $strip $mumin $mumax \
			$ut1 $ut2 $len]
		   puts $outid $line
		   }
		puts $outid ""
		}
	   }
	}
   genericChainDel $chain
   return
   }

###############################################################
#Read arbitrary great circles from a file and output a plan
#I should also output the actual "track" command.

proc GCPlan {mjd {outid stdout}} {

#Read file with list of GC scans
   if {![file exists gc.par]} return
   set chain [param2Chain gc.par hdr]
   if {[chainSize $chain] == 0} {
	genericChainDel $chain
	return
	}

   puts $outid "--------------"
   puts $outid "GC Area"
   puts $outid ""
   puts $outid "Strip    Start   End  Start-Time-Range  Length(hr)"
   puts $outid "         Mu      Mu           (UT)"
   puts $outid ""
   set format "%4s%2s%7d%7d%7s%7s%8.1f"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO for 0H UT
   set stmid [expr $stmid - 7.*15.]
   if {$stmid < 0} {set stmid [expr $stmid+360.]}

   chainEach hndl $chain {
	foreach var "stripe strip node incl muMin muMax" {
	   set $var [exprGet $hndl.$var]
	   }
	set len [expr $muMax-$muMin]

#Only prograde scans allowed.
	if {$len < 0.} {set len [expr $len+360.]}
	set utlimits [utGCLimits $muMin $muMax $node $incl $stmid]
	if {[string length $utlimits] == 0} {
	   echo limits not optimal for stripe $stripe
	   continue
	   }
	set ut1 [hm [expr [lindex $utlimits 0]/15.]]
	set ut2 [hm [expr [lindex $utlimits 1]/15.]]
	set len [expr ($muMax-$muMin)/15.]
	set muMin [format %.0f $muMin]
	set muMax [format %.0f $muMax]
	set line [format $format $stripe $strip $muMin $muMax \
		 $ut1 $ut2 $len]
	puts $outid $line
	set command [trackMuNodeIncl $muMin $node $incl $strip]
	puts $outid $command
	puts $outid ""
	}
   genericChainDel $chain
   return
   }

###############################################################
#Read list of SDSS survey holesfrom a file and output a plan

proc holePlan {mjd {outid stdout}} {

#Read file with list of GC scans
   if {![file exists holes.par]} return
   set chain [param2Chain holes.par hdr]
   if {[chainSize $chain] == 0} {
	genericChainDel $chain
	return
	}

   puts $outid "--------------"
   puts $outid "Hole Area"
   puts $outid ""
   puts $outid "Strip    Start   End  Start-Time-Range  Length(hr)"
   puts $outid "         Long    Long         (UT)"
   puts $outid ""
   set format "%4s%2s%7d%7d%7s%7s%8.1f"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO for 0H UT
   set stmid [expr $stmid - 7.*15.]
   if {$stmid < 0} {set stmid [expr $stmid+360.]}

   chainEach hndl $chain {
	foreach var "stripe strip slongStart slongEnd muStart muEnd" {
	   set $var [exprGet $hndl.$var]
	   }

#Length in hrs.
	set len [expr ($slongEnd - $slongStart)/15.]
	if {$len < 0.} {set len [expr $len+360.]}
	set utlimits [utLimits $slongStart $slongEnd $stripe $stmid]
	if {[string length $utlimits] == 0} {
	   echo limits not optimal for stripe $stripe
	   continue
	   }
	set ut1 [hm [expr [lindex $utlimits 0]/15.]]
	set ut2 [hm [expr [lindex $utlimits 1]/15.]]
	set line [format $format $stripe $strip $slongStart $slongEnd \
		 $ut1 $ut2 $len]
	puts $outid $line
	}
   genericChainDel $chain
   return
   }

###############################################################
#Read arbitrary track commands and convert them to a GC plan

proc trackPlan {mjd {outid stdout}} {

#Read file with list of track command scans
   if {![file exists track.par]} return
   set chain [param2Chain track.par hdr]
   if {[chainSize $chain] == 0} {
	genericChainDel $chain
	return
	}

   puts $outid "--------------"
   puts $outid "Track Area"
   puts $outid ""
   puts $outid "Strip    Start   End  Start-Time-Range  Length(hr)"
   puts $outid "       Longitude           (UT)"
   puts $outid ""
   set format "%4s%2s%7d%7d%7s%7s%8.1f"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO for 0H UT
   set stmid [expr $stmid - 7.*15.]
   if {$stmid < 0} {set stmid [expr $stmid+360.]}

   chainEach hndl $chain {
	foreach var "stripe strip ra dec velra veldec len" {
	   set $var [exprGet $hndl.$var]
	   }
	set pa [expr (180./3.14159265)*atan2($velra, $veldec)]
	set list [eqToNodeIncl $ra $dec $pa]
	set node [keylget list node]
	set incl [keylget list incl]

#Round-off error can cause problems.  Let's catch the due-north and due-south
#cases at least
	if {$velra == 0 && $veldec > 0} {
	   set node $ra
	   set incl 90.
	   }

	if {$velra == 0 && $veldec < 0} {
	   set node [expr $ra+180.]
	   if {$node > 360.} {set node [expr $node - 360.]}
	   set incl 90.
	   }

#Length is supplied in degrees!!!!
	set muMin [keylget list mu]
	set muMax [expr $muMin + $len]

	set utlimits [utGCLimits $muMin $muMax $node $incl $stmid]
	if {[string length $utlimits] == 0} {
	   echo limits not optimal for stripe $stripe
	   continue
	   }
	set ut1 [hm [expr [lindex $utlimits 0]/15.]]
	set ut2 [hm [expr [lindex $utlimits 1]/15.]]
	set len [expr ($muMax-$muMin)/15.]
	set muMin [format %.0f $muMin]
	if {$muMax > 360.} {set muMax [expr $muMax-360.]}
	set muMax [format %.0f $muMax]
	set line [format $format $stripe $strip $muMin $muMax \
		 $ut1 $ut2 $len]
	puts $outid $line
	set command [trackRaDecNode $ra $dec $node $strip]
	puts $outid $command
	puts $outid ""
	}
   genericChainDel $chain
   return
   }

###############################################################
#Read arbitrary track commands and convert them to a GC plan

proc snPlan {mjd {outid stdout}} {

#Hard-wired supernova plan.
#For documentation purposes, I will label the plan with an MJD range of
#+/- 3 days.
   set mjd1 [expr $mjd - 3]
   set mjd2 [expr $mjd + 3]
   set unix1 [expr ($mjd1 - 40587)*86400]
   set unix2 [expr ($mjd2 - 40587)*86400]
   set date1 [fmtclock $unix1 "%h %d"]
   set date2 [fmtclock $unix2 "%h %d"]
   puts $outid "--------------"
   puts $outid "Supernova Area  (MJD = $mjd1 to $mjd2)"
   puts $outid "                ($date1 to $date2)"
   puts $outid ""
   puts $outid "Stripe   Start   End  Start-Time-Range  Length(hr)"
   puts $outid "          Longitude        (UT)"
   puts $outid ""
   set format "%4s%2s%7d%7d%7s%7s%10.1f"

#Approximate sidereal time in deg. at 0hr UT
   set yr [fmtclock [getclock] %Y]
   set mjdsep21 [expr [convertclock "21 sep $yr"]/86400 + 40587]
   set stmid [expr 360.*($mjd-$mjdsep21)/365.25]

#Sidereal time at APO for 0H UT
   set stmid [expr $stmid - 7.*15.]
   if {$stmid < 0} {set stmid [expr $stmid+360.]}

#Get twilight times.  These are in UT hrs.
   set list [mtEphemeris $mjd]
   set names [lindex $list 0]
   set vals [lindex $list 1]
   loop i 0 [llength $names] {
	set name [lindex $names $i]
	set $name [lindex $vals $i]
	}

#Sidereal time at evening twilight
   set stEveTwi [expr $stmid + $mjdEveTwi*15.]

#Maximum west hour angle for start
#   set haMax 15.

#Kron's plan calls for a max. air mass of 3.25 hrs, which is an elevation of
#34 degrees or air mass of 1.8 (vs. 1.2 for HA = 0).
#Since I will put out a plan for every week, the actual HA will vary
#by +/- 15 min relative to nominal, so I will shrink the interval by 15 min
#(to 3 hrs) to make sure the worst case start position is still
#within limits.

   set haMax [expr 3.*15.]
   set raStart 300.
   set runLen 120.

#End survey longitude
   set lmax [expr int($raStart + $runLen - 185.)]
   if {$lmax > 180} {set lmax [expr $lmax - 360]}
   set stripe 82

#Real plan is to alternate strips, so just put blank
#   set strip N
   set strip " "

#Latest start sidereal time (in degrees) for raStart
   set stMax [expr $haMax + $raStart]

#But don't start too late if we can't finish the scan
   if {$stMax > $stEveTwi + ($mjdMornTwi - $mjdEveTwi)*15. - $runLen} {
	set stMax [expr  $stEveTwi + ($mjdMornTwi - $mjdEveTwi)*15. - $runLen]
	}
   set stMin [expr $raStart - $haMax]

#Is this later than twilight?  If so, output a start time range.
   if {$stMax > $stEveTwi} {
	set utStart $mjdEveTwi

#Don't start too early.
	if {$stMin > $stEveTwi} {
	   set utStart [expr $utStart + ($stMin-$stEveTwi)/15.]
	   }
	set utEnd [expr $utStart + ($stMax - $stEveTwi)/15.]
	set ut1 [hm $utStart]
	set ut2 [hm $utEnd]
	set lstart [expr int($raStart - 185.)]
	set lend $lmax
	set len [format %.1f [expr $runLen/15.]]
	set line [format $format $stripe $strip $lstart $lend \
	   $ut1 $ut2 $len]

#Kron rule - don't observe if len is less than 1.5 hrs
	if {$len >= 1.5} {
	   puts $outid $line
	   }
	}

#Instructions for starting in mid-stripe
   set utStart [expr $mjdEveTwi + ($stMax - $stEveTwi)/15.]

#End of range is length minus 1 hr (the shortest length run that is
#efficient)
   set utMax [expr $utStart + ($runLen-15.)/15.]
   if {$utStart < $mjdEveTwi} {set utStart $mjdEveTwi}
   if {$utMax > $mjdMornTwi - 1.} {set utMax [expr $mjdMornTwi - 1.]}

   set inc .5
   while {$utStart < $utMax} {
	set stStart [expr $stEveTwi + ($utStart - $mjdEveTwi)*15.]
	set ra [expr $raStart + $stStart - $stMax + $inc*15./2.]
	set lstart [expr $ra - 185.]
	if {$lstart > 180.} {set lstart [expr $lstart - 360.]}
	set ut1 [hm $utStart]
	set utEnd [expr $utStart + $inc]
	set ut2 [hm $utEnd]
	set len [expr $mjdMornTwi - $utStart - $inc/2.]
	set lend [expr $lstart + 15.*$len]
	if {$lend > 180.} {set lend [expr $lend - 360.]}

#Hmm, lend can be beyond end of survey if night is long enough and utStart
#is driven by hour angle constraints, not length of scan constraints.
	if {$lend < 0 && $lend > $lmax} {
	   set lend $lmax
	   set len [expr ($lend -$lstart)/15.]
	   if {$len < 0} {set len [expr $len + 24.]}
	   }
#	puts $outid "$ut1 $ut2 $lstart $len"
	set utStart [expr $utStart + $inc]
	set lstart [format %.0f $lstart]
	set lend [format %.0f $lend]
	set len [format %.1f $len]
	set line [format $format $stripe $strip $lstart $lend \
	   $ut1 $ut2 $len]
#Kron rule - don't observe if len is less than 1.5 hrs
	if {$len >= 1.5} {
	   puts $outid $line
	   }
	}
   puts $outid ""
   return
   }

#######################################################################
#Convert ra, dec, pos. angle to node, inclination, GC longitude.
#I don't think astrotools does this actual conversion.

proc eqToNodeIncl {ra dec pa} {
   set DEG2RAD [expr 3.14159265/180.]
   set rra [expr $ra*$DEG2RAD]
   set rdec [expr $dec*$DEG2RAD]
   set rpa [expr $pa*$DEG2RAD]
   set rnode [expr $rra - atan2(sin($rdec)*cos($rdec)*cos($rdec)*sin($rpa), cos($rpa))]
   set rincl [expr atan2(sin($rdec),(cos($rdec)*sin($rra-$rnode)))]
   set node [expr $rnode/$DEG2RAD]
   set incl [expr $rincl/$DEG2RAD]
   set rmu [expr acos(cos($rdec)*cos($rra-$rnode)) + $rnode]
   set mu [expr $rmu/$DEG2RAD]
   set list ""
   keylset list node $node
   keylset list incl $incl
   keylset list mu $mu
   return $list
   }

#######################################################################
#Compute position angle of mu direction relative to RA axis
#Note: I converted atan to atan2 below but don't know if it is correct in
#all situations.  atan did not work, e.g., for south-going scans.

proc muAng {ra dec node} {
echo ra $ra dec $dec node $node
   set DEG2RAD [expr 3.141592653589/180.]
   set rra [expr ($ra-$node)*$DEG2RAD]

#If rra = 0, we are starting on the node.  Assume 90 deg. angle regardless
#of dec.  If dec = 0, things are degenerate.
   if {$rra == 0.} {return 90.}
   set rdec [expr $dec*$DEG2RAD]
   set phi [expr \
	atan2(sin($rdec)*cos($rra),sin($rra)) \
	/ $DEG2RAD]
   return $phi
   }

######################################################################
#Track command, given ra, dec, node

proc trackRaDecNode {ra dec node {strip ""}} {
   set DEG2RAD [expr 3.141592653589/180.]
   set phi [muAng $ra $dec $node]
   set rphi [expr $phi*$DEG2RAD]
   set vel .00417807
   set velx [format %.8f [expr $vel*cos($rphi)]]
   set vely [format %.8f [expr $vel*sin($rphi)]]
   set ra [format %.4f $ra]
   set dec [format %.4f $dec]
   if {$strip == "N"} {
	set command1 "OFFSET BORESIGHT 0.10488,0.,0.,0./PAbsolute/Wait"
	set command2 "TRACK $ra, $dec FK5 /SCANVELOCITY=($velx,$vely) \
	   /KEEP=(BORESIGHT)"
	return [join [list $command1 $command2] \r\n]
	}
   if {$strip == "S"} {
	set command1 "OFFSET BORESIGHT -0.10488,0.,0.,0./PAbsolute/Wait"
	set command2 "TRACK $ra, $dec FK5 /SCANVELOCITY=($velx,$vely) \
	   /KEEP=(BORESIGHT)"
	return [join [list $command1 $command2] \r\n]
	}
   set command "TRACK $ra, $dec FK5 /SCANVELOCITY=($velx,$vely)"
   return $command
   }

######################################################################
#Track command, given mu0, node, inclination
#Back up by 3 degrees to get ramp.

proc trackMuNodeIncl {mu0 node incl {strip ""}} {
   set DEG2RAD [expr 3.141592653589/180.]
   set mu0 [expr $mu0-3.]
   set list [GCToEq $mu0 0 -node $node -inclination $incl]
   set ra [keylget list ra]
   set dec [keylget list dec]
   set rdec [expr $dec*$DEG2RAD]
   set rincl [expr $incl*$DEG2RAD]
   set rmu [expr ($mu0 - $node)*$DEG2RAD]
   set phi [expr atan(cos($rmu)*sin($rincl)/cos($rincl))/$DEG2RAD]
   set rphi [expr $phi*$DEG2RAD]
   set vel .00417807
   set velx [format %.8f [expr $vel*cos($rphi)]]
   set vely [format %.8f [expr $vel*sin($rphi)]]
   set ra [format %.4f $ra]
   set dec [format %.4f $dec]
   if {$strip == "N"} {
	set command1 "OFFSET BORESIGHT 0.10488,0.,0.,0./PAbsolute/Wait"
	set command2 "TRACK $ra, $dec FK5/SCANVELOCITY=($velx,$vely)/KEEP=(BORESIGHT)"
	return [join [list $command1 $command2] \r\n]
	}
   if {$strip == "S"} {
	set command1 "OFFSET BORESIGHT -0.10488,0.,0.,0./PAbsolute/Wait"
	set command2 "TRACK $ra, $dec FK5/SCANVELOCITY=($velx,$vely)/KEEP=(BORESIGHT)"
	return [join [list $command1 $command2] \r\n]
	}
   set command "TRACK $ra, $dec FK5 /SCANVELOCITY=($velx,$vely)"
   return $command
   }

#########################################################################
#Make a list of survey stripes with full info.  I need this over in mopdb.
#Segue information is in current directory.

proc stripeList {} {
   set fid [open stripeList.par w]
echo Creating stripeList.par as $fid
   puts $fid "typedef struct {"
   puts $fid "   int stripe;"
   puts $fid "   float node;"
   puts $fid "   float incl;"
   puts $fid "   float muStart;"
   puts $fid "   float muEnd;"
   puts $fid "   } STRIPE"

   set segStripes ""
   set chain [param2Chain segstripes.par hdr]
   chainEach hndl $chain {
	foreach var "stripe node incl startMu endMu" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend segStripes $stripe
	}

   loop i 1 100 {
	if {[lsearch $segStripes $i] >= 0} continue
	set line [stripeLimits $i]
	foreach var "lambdaMin lambdaMax eta" {
	   set $var [keylget line $var]
	   }
###	if {$lambdaMin == $lambdaMax} continue
	set node 95.
	set incl [expr $eta + 32.5]
	set line [surveyToEq $lambdaMin $eta]
	set ra [keylget line ra]; set dec [keylget line dec]
	set line [eqToGC $ra $dec -node $node -incl $incl]
	set muStart [keylget line mu]
	set line [surveyToEq $lambdaMax $eta]
	set ra [keylget line ra]; set dec [keylget line dec]
	set line [eqToGC $ra $dec -node $node -incl $incl]
	set muEnd [keylget line mu]
	puts $fid "[format "stripe  %3d  %10.3f %10.3f %10.3f %10.3f" \
	   $i  $node  $incl $muStart $muEnd]"
	}
   chainEach hndl $chain {
	foreach var "stripe node incl startMu endMu" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$incl > 180.} {set incl [expr $incl - 360.]}
	set muStart $startMu
	if {$endMu > 360.} {
	   set muEnd [expr $endMu - 360.]
	} else {
	   set muEnd $endMu
	   }
	puts $fid "[format "stripe  %3d  %10.3f %10.3f %10.3f %10.3f" \
	   $stripe  $node  $incl $muStart $muEnd]"
	}
   close $fid
   genericChainDel $chain
   return
   }

