#Connect to opdb and fetch rowc, colc for an object.

#########################################################################
proc plConvert {} {
   global frames
   set chain [param2Chain specDir.par hdr]
   chainEach hndl $chain {
	set mjdobs [exprGet $hndl.mjdobs]
	set plate [exprGet $hndl.plateID]
	set file [exprGet $hndl.mapFile]
	set outdir ./[format %04d $plate]/$mjdobs
	if {[catch {plMapRead $file $outdir} msg]} {
	   echo Problem converting $file
	   echo $msg
	   }
	}
   genericChainDel $chain
   return
   }
#########################################################################
proc plMapRead {file outdir} {
   global frames

#Output file is plPlugMap-xxx-xxx-r.par
   set tail [file tail $file]

#If regsub fails, output file name is same as input
   regsub plPlugMapM $tail plPlugMapR tail
   set outFile $outdir/$tail
   if {[file exists $outFile]} {
	set mtime [file mtime $outFile]
	set now [getclock]

#Don't overwrite newer files.
	if {($now-$mtime)/86400. < 3.} {
	   echo $file newer than 3 days.
	   return
	   }
	echo Overwriting $outFile
	}

echo Input $file, output $outFile
   set fid [open $outFile w]
   puts $fid "typedef struct {"
   puts $fid "   int run;"
   puts $fid "   int rerun;"
   puts $fid "   int camcol;"
   puts $fid "   int field;"
   puts $fid "   int id;"
   puts $fid "   int rowc;"
   puts $fid "   int colc;"
   puts $fid "   int fiberId;"
   puts $fid "   } SPECOBJ;"
   set chain [param2Chain $file hdr]
   loop i 0 [chainSize $chain] {
	set hndl [chainElementGetByPos $chain $i]
	set run [exprGet $hndl.objId<0>]
	set rerun [exprGet $hndl.objId<1>]
	set camcol [exprGet $hndl.objId<2>]
	set field [exprGet $hndl.objId<3>]
	set id [exprGet $hndl.objId<4>]
	set holeType [exprGet -enum $hndl.holeType]
	set fiberId [exprGet $hndl.fiberId]
	set primTarget [exprGet $hndl.primTarget]
	set secTarget [exprGet $hndl.secTarget]
	handleDel $hndl

#Hack for Yanny changing 756 rerun from 6 to 8
	if {$run == 756 && $rerun == 6} {
	   set rerun 8
	   }
	if {$holeType != "OBJECT"} continue
	set line [specObj $run $rerun $camcol $field $id]
	puts $fid "SPECOBJ $run $rerun $camcol $field $id $line $fiberId"
#echo "SPECOBJ $run $rerun $camcol $field $id $line $fiberId"
	}
   close $fid
   genericChainDel $chain

#Clean up frames - avoid too many handles.
   foreach hndl [array names frames] {
	objDel $frames($hndl)
	}
   unset frames
   sessionEnd
   return
   }

######################################################################
#Get row, col for all objects in a plug mapping.  For now, just echo.
proc specObj {run rerun camcol field id} {
   global frames
   if {![inSession]} {
	opdbConnect
	}
   if {![info exists frames($run,$rerun,$camcol)]} {
	set frames($run,$rerun,$camcol) [pipeGet frames \
	   $run $camcol -rerun $rerun]
	}
   set objList [$frames($run,$rerun,$camcol) -> objectList $field]
   set itr [objNew ooItr(arObject) -transient]
   set object [objNew -transient ooHandle(arObject)]
   $objList -> objects $itr
   set rowc -9999
   set colc -9999
   while {[$itr next]} {
	set objId [$itr -> id]
	if {$objId != $id} continue
	set float [$itr -> objc_rowc]
	set rowc [$float value]
	handleDel $float
	set float [$itr -> objc_colc]
	set colc [$float value]
	handleDel $float
        break
	}
   objDel $itr
   objDel $objList
   objDel $object
   return "[expr round($rowc)] [expr round($colc)]"
   }

######################################################################
#Recreate a tsObj file for a single field.

proc tsObjFromField {run rerun camcol field} {
   global frames
   if {![inSession]} {
	opdbConnect
	}

#Allow null rerun - use latest rerun (using runInventory).
   if {$rerun == ""} {
	global runTable
	if {![info exists runTable]} {
	   runInventory
	   }
	set rerun $runTable($run)
	}
   if {![info exists frames($run,$rerun,$camcol)]} {
	set frames($run,$rerun,$camcol) [pipeGet frames \
	   $run $camcol -rerun $rerun]
	}
   set objList [$frames($run,$rerun,$camcol) -> objectList $field]
   set itr [objNew ooItr(arObject) -transient]
   set object [objNew -transient ooHandle(arObject)]
   $objList -> objects $itr

# Initialize calibrated object chain
   set chain [chainNew TA_CALIB_OBJ]
   while {[$itr next]} {

# Get calibrated version of the object
	$object = $itr
	if {! [$object isValid]} {error "invalid object handle"}
        set hndl [objectCalibrate $object 0]
	chainElementAddByPos $chain $hndl TAIL AFTER
	handleDel $hndl
	}
   objDel $itr
   objDel $objList
   objDel $object

   set outTbl [calibObjsWriteToTbl $chain]

   hdrInsWithInt $outTbl.hdr RUN $run
   hdrInsWithInt $outTbl.hdr RERUN $rerun
   hdrInsWithInt $outTbl.hdr CAMCOL $camcol
   hdrInsWithInt $outTbl.hdr FIELD $field

   set file [format tsObj-%06d-%d-%d-%04d.fit $run $camcol $rerun $field]
   fitsWrite $outTbl $file -binary -pdu MINIMAL

# Clean up
   tblColDel $outTbl
   genericChainDel $chain
   return
   }

#############################################################
set tsObjFromMapArgs {
    {tsObjFromMap "Generate a tsObj file from a mapped plate"}
    {<plateId>      INTEGER "" plateId  "Plate ID"}
    {[mapId]      INTEGER 0  mapId   "Mapping id (not required if only one mapping stuffed in the OPDB)"}
    {-current     CONSTANT 0 current  "Use current (best) calibrations rather than target selection calibrations"}
    {-raw         CONSTANT 0 raw      "Spit out raw values rather than calibrated values"}
}

ftclHelpDefine tsProcs tsObjFromMap [shTclGetHelpInfo tsObjFromMap \
        $tsObjFromMapArgs]

proc tsObjFromMap {args} {
    global photo_maps

    upvar #0 tsObjFromMapArgs formal_list

    # Parse the arguments.
    if {[shTclParseArg $args $formal_list tsObjFromMap] == 0} {return}
    set current [ftclPassed $args $formal_list -current]
    set raw [ftclPassed $args $formal_list -raw]
    set Arguments ""
    if {! $current} {append Arguments " -target"}
    if {$raw} {append Arguments " -raw"}

    # Get a handle to the map
    if {$mapId == 0} {
        set map [queryGetOne arPluggedPlate $photo_maps \
                "plate_.id_ == $plateId"]
    } else {
        set map [queryGetOne arPluggedPlate $photo_maps \
                "id_ == $mapId && plate_.id_ == $plateId"]
    }

    # Initialize some handles for use when iterating over objects
    set fibers [objNew -transient ooItr(arFiber)]
    set hole [objNew -transient ooHandle(arHole)]
    set tiledTarget [objNew -transient ooHandle(arTiledTarget)]
    set target [objNew -transient ooHandle(arTarget)]
    set object [objNew -transient ooHandle(arObject)]

    # Initialize calibrated object chain
    set chain [chainNew TA_CALIB_OBJ]
   # For each fiber ...
    if {! [$map -> fibers $fibers]} {
        error "couldn't initialize iterator over fibers"
    }
    while {[$fibers next]} {
        # Fetch handle to the object
        $fibers -> hole $hole
        $hole -> tiledTarget $tiledTarget
        $tiledTarget -> target $target
        $target -> object $object

        # Get calibrated version of the object
        if {! [$object isValid]} {error "invalid object handle"}
        set obj([$fibers -> spectrographId],[$fibers -> fiberId]) \
                [eval objectCalibrate $object 0 $Arguments]
    }

    # Create chain in fiber order
    loop spectrographId 1 3 {
        loop fiberId 1 321 {
            if {! [info exists obj($spectrographId,$fiberId)]} {
                set obj($spectrographId,$fiberId) [genericNew TA_CALIB_OBJ]
                foreach var {run rerun camCol field id} {
                    handleSet $obj($spectrographId,$fiberId).$var 0
                }
            }
            chainElementAddByPos $chain $obj($spectrographId,$fiberId) \
                    TAIL AFTER
            handleDel $obj($spectrographId,$fiberId)
        }
    }

    # Write out chain of calibrated objects as a tsObj file
    set outTbl [calibObjsWriteToTbl $chain]
    set mapId [$map -> id]
    set fscanMjd [expr $mapId / 100]
    set fscanId [expr $mapId % 100]
    hdrInsWithInt $outTbl.hdr PLATE_ID $plateId
    hdrInsWithInt $outTbl.hdr FSCANMJD $fscanMjd
    hdrInsWithInt $outTbl.hdr FSCANID $fscanId
    fitsWrite $outTbl [format tsObj-%04d-%05d-%02d.fit $plateId $fscanMjd \
            $fscanId] -binary -pdu MINIMAL


    # Clean up
    taGenericChainDestroy $chain
    tblColDel $outTbl
    objDel $object
    objDel $target
    objDel $tiledTarget
    objDel $hole
    objDel $fibers
    objDel $map
    return
}
