#Procedures herein:
# sp2dCrawl:	Read spPlate files
# sp2dReport:	Create a report file from sp2d data.
# plateInfo:	List info for a single plate
# reportCrawl:	Read plateReport.par
# report2dMatch:	Compare plateReport and sp2d
# spectroStatusFetch:	Fetch plate report from plate inventory db.
# mapStatusFetch:	Fetch map report from plate inventory db.
# plateListFetch:	Fetch list of tiles, plates, and programs.
# specTileReport:	Create tileReport file - used by tilePlot
# specStatusHistory	Print spectro history by quarter
# specRunHistory	Print spectro history by run
#####################################################################
#################################################################
#Crawl through 2-d directories and get spPlate info

global 2d_dir 1d_dir astrodir
set 2d_dir /data/dp9.a/data/spectro/2d_19
#set 2d_alt /data/dp9.a/data/spectro/2d_15
set 1d_dir /data/dp9.a/data/spectro/1d_19
set astrodir /data/dp3.b/data/golden

proc sp2dCrawl {} {
   global sp2d 2d_dir 2d_alt 1d_dir astrodir

   if {[info exists sp2d]} {unset sp2d}
   set logDir $astrodir

#Read the latest plateList file - to get tileRun info
#Fetch plateList info
   echo Fetching plateList info
   plateListInfo
   set files [lsort [glob -nocomplain plateList-*.par]]
   set file [lindex $files end]
   if {$file != ""} {
	set plateInfo [param2Chain $file hdr]
	}

#Need to crawl multiple root directories as we transition between sp2d
#versions.
   if {[info exists 2d_alt]} {
	set dirs1 [lsort [find -nocomplain $2d_alt/*]]
   } else {
	set dirs1 ""
	}
   set dirs2 [lsort [find -nocomplain $2d_dir/*]]
   set dirs [join "$dirs1 $dirs2"]
   set hdr [hdrNew]
   set sp2d() ""
   set sp2d(ignore) ""
   foreach dir $dirs {
	set plateId [string trimleft [file tail $dir] 0]
	if {$plateId < 266} continue
	if {![regexp {^[0-9]+$} $plateId]} continue

#If no 1d directory, skip.  This is the case for plate 317 (high secz plate!)
	if {![file exists $1d_dir/[format %04d $plateId]]} {
	   continue
	   }
	if {[lsearch $sp2d() $plateId] < 0} {
	   lappend sp2d() $plateId
	   set sp2d($plateId) ""
	   }
	set spfiles [find -nocomplain $dir/spPlate*.fits]
	foreach spfile $spfiles {
	   if {[catch {hdrReadAsFits $hdr $spfile}]} {
		echo Error reading header for $spfile
		continue
		}

#Extract interesting info from spPlate header
	   set mjd [hdrGetAsInt $hdr MJD]

#If the mjd already exists, we processed this file from another sp2d rerun.
	   if {[lsearch $sp2d($plateId) $mjd] >= 0} continue

#Cache the 2d directory info
	   set sp2d($plateId,$mjd,2dir) $dir
	   lappend sp2d($plateId) $mjd
	   set mapId [hdrGetAsInt $hdr MAPID]

	   set n [hdrGetLineTotal $hdr]
	   set exposures ""
	   loop i 0 $n {
		set line [hdrGetLineCont $hdr $i]
		if {[regexp {EXPID[0-9][0-9]} $line]} {
		   set val [string trim [lindex $line 2] ']
		   if {[regexp {b1-[0-9]+-([0-9]+)} $val all expId]} {
			set expId [string trimleft $expId 0]
			lappend exposures $expId
			}
		   }
		}
	   set sp2d($plateId,$mjd,mapId) $mapId
	   set sp2d($plateId,$mjd,exposure) $exposures

#Fetch S/N
	   foreach key "spec1_g spec1_r spec1_i spec2_g spec2_r spec2_i" {
		set sn [hdrGetAsDbl $hdr [string toupper $key]]
		set sp2d($plateId,$mjd,$key) $sn
		}
	   }

#qc files can be in either 2d or 1d.  Sigh.
	set qcfiles [find -nocomplain $dir/qc*.par]

#Now crawl 1-D for QC file
	if {[regsub {2d_} $dir {1d_} 1dir]} {
	   set 1dfiles [find -nocomplain $1dir/qc*.par]
	   set qcfiles [join "$qcfiles $1dfiles"]
	   }
	foreach qcfile $qcfiles {
	   if {![catch {exec grep ignore $qcfile}]} {

#Decode info.  This is a horrible way to track info, but alas.
		set list [split [file tail $qcfile] .]

#I am not tracking qcignore properly.  I track by mapName, which is wrong,
#I should track by mjdmax, since that is what the qcignore files say.
		set mjdmax [lindex $list 2]
		if {![catch {set map [find \
			$logDir/$mjdmax/plPlugMapM-[format %04d \
			$plateId]-*.par]}]} {
		   set map [lindex $map end]
		   regexp {plPlugMapM-[0-9]+-([0-9]+)-([0-9]+)\.par} \
			[file tail $map] all mjdMap version
		   set mapName [format %04d $plateId]-$mjdMap-$version
		   if {[lsearch $sp2d(ignore) $mapName] < 0} {
			lappend sp2d(ignore) $mapName
			if {[info exists sp2d($plateId,mjdmax)]} {
			   lappend sp2d($plateId,mjdmax) $mjdmax
			} else {
			   set sp2d($plateId,mjdmax) $mjdmax
			   }
			}
		   }
		}
	   }
	}
   hdrDel $hdr

#Reorder arrays by mapName
   if {[info exists sn]} {unset sn}

#Invert sp2d
   set sp2d(mapName) ""
   set thresh1 13.
   set thresh2 13.

   foreach plateId [lsort -integer $sp2d()] {
	set sp2d($plateId,status) Not_Done

#For combined plates, we have multiple mjdmax with the same mapId.  If I
#go through in ascending date order, I save information for the most
#combined spPlate file.
	foreach mjdmax [lsort -integer $sp2d($plateId)] {
	   set 2dir $sp2d($plateId,$mjdmax,2dir)
	   set mapName [mapName $sp2d($plateId,$mjdmax,mapId) $plateId]

#Is mapName unique?  If not, we have uncombined plates.
	   if {[lsearch $sp2d(mapName) $mapName] < 0} {
		set sp2d($mapName) ""
		lappend sp2d(mapName) $mapName
		}

#Test for uncombined exposures.  I check if exposures in new mjd for this map
#already in existing set (or if this is the first mjd).  If so, I overwrite
#old (because that is how sp2d works).  Else I add.
	   if {[lsearch $sp2d($mapName) [lindex \
		   $sp2d($plateId,$mjdmax,exposure) 0]] >= 0 || \
		   [llength $sp2d($mapName)] == 0} {
		set sp2d($mapName) $sp2d($plateId,$mjdmax,exposure)
		set sp2d($mapName,plateId) $plateId
		set sp2d($mapName,mjdmax) $mjdmax
		set sp2d($mapName,sng1) $sp2d($plateId,$mjdmax,spec1_g)
		set sp2d($mapName,snr1) $sp2d($plateId,$mjdmax,spec1_r)
		set sp2d($mapName,sni1) $sp2d($plateId,$mjdmax,spec1_i)
		set sp2d($mapName,sng2) $sp2d($plateId,$mjdmax,spec2_g)
		set sp2d($mapName,snr2) $sp2d($plateId,$mjdmax,spec2_r)
		set sp2d($mapName,sni2) $sp2d($plateId,$mjdmax,spec2_i)
		set sp2d($plateId,$mjdmax,status) Not_Done
		set sp2d($mapName,status) NOT_DONE
	   } else {
		echo Warning: Likely uncombined mjds for map $mapName

#Include this mjd only if S/N is > 2 in all bands
		if {$sp2d($plateId,$mjdmax,spec1_g) < 2 || \
		    $sp2d($plateId,$mjdmax,spec1_i) < 2 || \
		    $sp2d($plateId,$mjdmax,spec2_g) < 2 || \
		    $sp2d($plateId,$mjdmax,spec2_i) < 2} {
		   echo S/N too low anyway.
		   continue
		   }
		lappend sp2d($mapName) $sp2d($plateId,$mjdmax,exposure)
		set sp2d($mapName,mjdmax) $mjdmax
		set sp2d($mapName,sng1) [expr $sp2d($mapName,sng1) + \
		   $sp2d($plateId,$mjdmax,spec1_g)]
		set sp2d($mapName,snr1) [expr $sp2d($mapName,snr1) + \
		   $sp2d($plateId,$mjdmax,spec1_r)]
		set sp2d($mapName,sni1) [expr $sp2d($mapName,sni1) + \
		   $sp2d($plateId,$mjdmax,spec1_i)]
		set sp2d($mapName,sng2) [expr $sp2d($mapName,sng2) + \
		   $sp2d($plateId,$mjdmax,spec2_g)]
		set sp2d($mapName,snr2) [expr $sp2d($mapName,snr2) + \
		   $sp2d($plateId,$mjdmax,spec2_r)]
		set sp2d($mapName,sni2) [expr $sp2d($mapName,sni2) + \
		   $sp2d($plateId,$mjdmax,spec2_i)]
		}

#Test for done condition each time through.
	   set mapId $sp2d($plateId,$mjdmax,mapId)
	   set sn(g1) $sp2d($mapName,sng1)
	   set sn(r1) $sp2d($mapName,snr1)
	   set sn(i1) $sp2d($mapName,sni1)
	   set sn(g2) $sp2d($mapName,sng2)
	   set sn(r2) $sp2d($mapName,snr2)
	   set sn(i2) $sp2d($mapName,sni2)
	   set mean [expr ($sn(g1)+$sn(g2)+$sn(i1)+$sn(i2))/4.]
	   set min [min $sn(g1) $sn(i1) $sn(g2) $sn(i2)]
	   set sp2d($mapName,mean) $mean
	   set sp2d($mapName,min) $min

#Only the min test actually applies.
#First, reset thresh if the tileRun exists and is a special long plate
	   set t1 $thresh1
	   set t2 $thresh2
	   if {[info exists plateInfo]} {
		set hndl [chainGetOne $plateInfo plateId $plateId]
		if {$hndl != ""} {
		   set tileRun [exprGet $hndl.tileRun]
		   handleDel $hndl
		   if {[regexp {([A-Za-z]+)} $tileRun all prog]} {
			if {[lsearch "photoz lowz annis fstar loveday" $prog] \
			   >= 0} {
			   set t1 28.
			   set t2 28.
			   }
			}
		   }
		}
	   if {$min >= $t1 && $mean >= $t2} {
		set sp2d($mapName,status) DONE
		set sp2d($plateId,status) Done
		set sp2d($plateId,$mjdmax,status) Done
		}

#Now crawl 1-D for QC file
	   if {![regsub {2d_} ${2dir} {1d_} 1dir]} {
		echo "Cannot construct 1dir for ${2dir}"
		continue
		}
	   set qcFile ${1dir}/1d/spQCLog-$mjdmax-[format %04d $plateId].par

#Generate my own statistics - ignore QC file!
	   if {![file exists $qcFile] || 1} {
		echo No QC file for plate $plateId, mjdmax $mjdmax

#Fetch the spDiag file and reconstruct the lrg statistics
		set diagFile $1dir/1d/spDiag1d-$mjdmax-[format %04d \
		   $plateId].par
		if {![file exists $diagFile]} {
		   echo No file $diagFile
		   continue
		   }
		set chain [param2Chain $diagFile hdr]
		set nlrg 0; set nlrghc 0; set nmg 0; set nmghc 0
		set lrglist ""
		set mglist ""
		chainEach hndl $chain {
		   set primTarget [exprGet $hndl.primTarget]
		   set zconf [exprGet $hndl.zConfFinal]
		   if {$primTarget & 0x20} {
			incr nlrg
			if {$zconf >= 0.5} {incr nlrghc}
			lappend lrglist $zconf
			}
		   if {$primTarget & 0x40} {
			incr nmg
			if {$zconf >= 0.5} {incr nmghc}
			lappend mglist $zconf
			}
		   }
		genericChainDel $chain
		set sp2d($plateId,$mjdmax,nlrg) $nlrg
		set sp2d($plateId,$mjdmax,nlrghc) $nlrghc
		set sp2d($plateId,$mjdmax,nmg) $nmg
		set sp2d($plateId,$mjdmax,nmghc) $nmghc
		if {[llength $lrglist] > 0} {
		   set lrglist [lsort -real $lrglist]
		   set sp2d($plateId,$mjdmax,lrgmed) [lindex $lrglist \
			[expr [llength $lrglist]/20]]
		   }
		if {[llength $mglist] > 0} {
		   set mglist [lsort -real $mglist]
		   set sp2d($plateId,$mjdmax,mgmed) [lindex $mglist \
			[expr [llength $mglist]/20]]
		   }
	   } else {
		set chain [param2Chain $qcFile hdr]
		set hndl0 [chainGet $chain 0]
		set hndl1 [chainGet $chain 1]
		set sp2d($plateId,$mjdmax,nmg) [expr [exprGet $hndl0.nMG] + \
		   [exprGet $hndl1.nMG]]
		set sp2d($plateId,$mjdmax,nmghc) [expr [exprGet $hndl0.nMGHC] \
		   + [exprGet $hndl1.nMGHC]]
		set sp2d($plateId,$mjdmax,nlrg) [expr [exprGet $hndl0.nLRG] + \
		   [exprGet $hndl1.nLRG]]
		set sp2d($plateId,$mjdmax,nlrghc) [expr [exprGet \
		   $hndl0.nLRGHC] + [exprGet $hndl1.nLRGHC]]
		set sp2d($plateId,$mjdmax,nmgclass) [expr [exprGet \
		   $hndl0.nMGClass] + [exprGet $hndl1.nMGClass]]
		handleDel $hndl0
		handleDel $hndl1
		genericChainDel $chain
		}
	   }
	}
   if {[info exists plateInfo]} {
	genericChainDel $plateInfo
	}
   return
   }

######################################################################
proc sp2dCheck {} {
   global sp2d
#Do some sanity checking internal to sp2d.
#First, look for maps that have qcignore set.  Because there are version
#mismatches, I will just look at plateId and mjdmap
   foreach mapId [lsort $sp2d(mapName)] {
	set list [split $mapId -]
	set plateId [lindex $list 0]
	set mjdmap [lindex $list 1]
	set index [lsearch -regexp $sp2d(ignore) $plateId-$mjdmap]
	if {$index >= 0} {
	   echo qcignore set for [lindex $sp2d(ignore) $index] but \
spPlate exists (mapId = $mapId)
	   }
	}

#Look for plates with multiple mjdmax; see if they share same mapId, which
#means that they were not combined.  Actually, they probably are combined.
   foreach plateId [lsort -integer $sp2d()] {
	if {[llength $sp2d($plateId)] <= 1} continue
	loop i 1 [llength $sp2d($plateId)] {
	   set mjd2 [lindex $sp2d($plateId) $i]
	   loop j 0 $i {
		set mjd1 [lindex $sp2d($plateId) $j]
		if {$sp2d($plateId,$mjd1,mapId) == \
			$sp2d($plateId,$mjd2,mapId)} {

#Usually they are merged already.
#		   echo Plate $plateId, mjds $mjd1, $mjd2 should be merged
		   }
		}
	   }
	}
   return
   }

######################################################################
#Read plateList file and stuff useful info into sp2d array

proc plateListInfo {} {
   global sp2d
   set file [lindex [lsort [glob -nocomplain plateList-*.par]] end]
   if {$file == ""} {
	echo No plateList file found
	return
	}
   set chain [param2Chain $file hdr]
   chainEach hndl $chain {
	foreach var "plateId raCen decCen tileRun haMin location" {
	   set $var [exprGet -enum $hndl.$var]
	   }
	set sp2d($plateId,ra) $raCen
	set sp2d($plateId,dec) $decCen
	set sp2d($plateId,tileRun) $tileRun
	set sp2d($plateId,haMin) $haMin
	set sp2d($plateId,location) $location
	set st [expr $raCen + $haMin]
	if {$st < 0} {set st [expr $st+360]}
	if {$st >= 360} {set st [expr $st-360]}
	set sp2d($plateId,st) [format %.0f $st]
	}
   genericChainDel $chain
   return
   }

######################################################################
#Dump detailed info about a single plate
proc plateInfo {plateId} {
   global sp2d reportData princeton
   if {![info exists sp2d]} {
       echo Must run sp2dCrawl first
	return
	}
   if {![info exists reportData]} {
	echo Must run reportCrawl first
	return
	}
   if {![info exists sp2d($plateId)]} {
	echo No sp2d Info for plate $plateId
   } else {

	echo RA = $sp2d($plateId,ra), DEC = $sp2d($plateId,dec), tileRun = \
	   $sp2d($plateId,tileRun), ST = $sp2d($plateId,st), Location = \
	   $sp2d($plateId,location)
	echo mjdMax for which data exist: $sp2d($plateId)

#First, loop through each mjdmax and build up a list of unique maps for this
#plate.  I should have done this in sp2d but didn't do so.  Also, I only
#keep last mjdmax, not a list of all mjds that go into a plate.
	set mapList ""
	foreach mjdMax [lsort -integer $sp2d($plateId)] {
	   set mapName [mapName $sp2d($plateId,$mjdMax,mapId) $plateId]
	   if {[lsearch $mapList $mapName] < 0} {
		lappend mapList $mapName
		set mjd($mapName) ""
		}
	   lappend mjd($mapName) $mjdMax
	   }
	foreach mapName [lsort $mapList] {
	   echo
	   echo MapName $mapName, MJDs: $mjd($mapName)
	   echo "      " Exposures $sp2d($mapName)
	   echo "      " plateId $sp2d($mapName,plateId)
	   echo "      " Ultimate mjdmax $sp2d($mapName,mjdmax)
	   echo "      " sng1 [format %.2f $sp2d($mapName,sng1)]
	   echo "      " snr1 [format %.2f $sp2d($mapName,snr1)]
	   echo "      " sni1 [format %.2f $sp2d($mapName,sni1)]
	   echo "      " sng2 [format %.2f $sp2d($mapName,sng2)]
	   echo "      " snr2 [format %.2f $sp2d($mapName,snr2)]
	   echo "      " sni2 [format %.2f $sp2d($mapName,sni2)]
	   echo "   " Map S/N status: $sp2d($mapName,status)
	   if {[info exists reportData($mapName,status)]} {
		set status $reportData($mapName,status)
		if {$status == ""} {set status "NotDone"}
		echo  "   " Plate database status: $status
		}
	   if {[lsearch $sp2d(ignore) $mapName] >= 0} {
		echo Seems like $mapName is qcignore
		}
	   foreach mjdMax [lsort -integer $mjd($mapName)] {
		echo "   " $mjdMax: mapId $sp2d($plateId,$mjdMax,mapId)
		echo "   " $mjdMax: exposures $sp2d($plateId,$mjdMax,exposure)
		echo "   " $mjdMax: S/N(g1, r1, i1, g2, r2, i2) \
		   [format %.2f $sp2d($plateId,$mjdMax,spec1_g)] \
		   [format %.2f $sp2d($plateId,$mjdMax,spec1_r)] \
		   [format %.2f $sp2d($plateId,$mjdMax,spec1_i)] \
		   [format %.2f $sp2d($plateId,$mjdMax,spec2_g)] \
		   [format %.2f $sp2d($plateId,$mjdMax,spec2_r)] \
		   [format %.2f $sp2d($plateId,$mjdMax,spec2_i)]
#The following does not always exist, and it does not appear to be set
#correctly anyway.
#		echo "   " Status of plate $plateId, mjdMax $mjdMax: \
#		   $sp2d($plateId,$mjdMax,status)
#Does this mean qcignore?
		if {[info exists sp2d($plateId,mjdmax)]} {
		   echo QC ignore? $sp2d($plateId,mjdmax)
		   }

#Check 1d QA
		if {[info exists sp2d($plateId,$mjdMax,nmg)]} {
		   if {$sp2d($plateId,$mjdMax,nmg) > 0} {
			set fractmg [format %.2f [expr \
			   1.*$sp2d($plateId,$mjdMax,nmghc) / \
			   $sp2d($plateId,$mjdMax,nmg)]]
		   } else {
			set fractmg 0
			}
		   if {$sp2d($plateId,$mjdMax,nlrg) > 0} {
			set fractlrg [format %.2f [expr \
			   1.*$sp2d($plateId,$mjdMax,nlrghc) / \
			   $sp2d($plateId,$mjdMax,nlrg)]]
		   } else {
			set fractlrg 0
			}
		   echo "   " $mjdMax: Main galaxy: $fractmg good \
			(n=$sp2d($plateId,$mjdMax,nmg)), \
			LRG: $fractlrg good (n=$sp2d($plateId,$mjdMax,nlrg))
		   }
		}
	   }
	echo Global S/N status: $sp2d($plateId,status)

#Check if plate is on list of plate inventory database "done" list
	if {[lsearch $reportData(plateId) $plateId] >= 0} {
	   echo Plate database status: Done
	} else {
	   echo Plate databse status: NotDone
	   }
	if {[info exists princeton($plateId)]} {
	   foreach mjdmax $princeton($plateId) {
		echo Princeton QA for mjd $mjdmax: Status = \
		   $princeton($plateId,$mjdmax,quality), \
		   comment: $princeton($plateId,$mjdmax,qualcomments)
		}
	   }
	}
   return
   }

######################################################################
#Create report file from sp2d

proc sp2dReport {} {
   global sp2d
#Now go root out the S/N ratios
   set thresh1 13
   set thresh2 15
   set today [expr [getclock]/86400 + 40587]
   set file spectroReport-$today.par
set fid ""
echo   set fid [open $file w]
echo   puts $fid "#Report created [fmtclock [getclock]]"
echo   puts $fid "mjdReport $today"
echo   puts $fid "typedef struct {"
echo   puts $fid "   int plateId;"
echo   puts $fid "   char status<10>;"
echo  puts $fid "   int mjdobs;"
echo   puts $fid "   char mapId<13>;"
echo   puts $fid "   } SPECSTAT;"

   set n 0
   foreach plateId [lsort -integer $sp2d()] {
	set sp2d($plateId,status) Not_Done
	foreach mjdobs $sp2d($plateId) {
	   incr n
	   set sp2d($plateId,$mjdobs,status) Not_Done
	   set mapId $sp2d($plateId,$mjdobs,mapId)
	   set sn(g1) $sp2d($plateId,$mjdobs,spec1_g)
	   set sn(r1) $sp2d($plateId,$mjdobs,spec1_r)
	   set sn(i1) $sp2d($plateId,$mjdobs,spec1_i)
	   set sn(g2) $sp2d($plateId,$mjdobs,spec2_g)
	   set sn(r2) $sp2d($plateId,$mjdobs,spec2_r)
	   set sn(i2) $sp2d($plateId,$mjdobs,spec2_i)
	   set mean [expr ($sn(g1)+$sn(g2)+$sn(r1)+$sn(r2)+$sn(i1)+$sn(i2))/6.]
	   set min [min $sn(g1) $sn(r1) $sn(i1) $sn(g2) $sn(r2) $sn(i2)]

#Catch whacky things
	   set mean1 [expr ($sn(g1)+$sn(r1)+$sn(i1))/3.]
	   set mean2 [expr ($sn(g2)+$sn(r2)+$sn(i2))/3.]
	   if {$min >= $thresh1 && $mean >= $thresh2} {
		set sp2d($plateId,$mjdobs,status) Done
		set sp2d($plateId,status) Done
		}
	   if {$sp2d($plateId,$mjdobs,status) == "Done"} {
		set line "specstat [string trimleft $plateId 0] \
		   Done $mjdobs [mapName $mapId $plateId]"
	   } else {
		set line "specstat [string trimleft $plateId 0] \
		   Not_Done $mjdobs [mapName $mapId $plateId]"
		}
	   echo puts $fid $line
	   if {$mean1 < .7*$mean2 && $mean1 < $thresh1 && $mean2 > $thresh1} {
		echo $plateId: Left side is low: S/N(1) = $mean1, \
		   S/N(2) = $mean2
	   } elseif {$mean1 > .7*$mean2 && $mean1 > $thresh1 && $mean2 < \
		$thresh1} {
		echo $plateId: echo Right side is low: S/N(1) = $mean1, \
		   S/N(2) = $mean2
		}
	   }
	}
echo   close $fid
   echo Total $n plate/mjd combinations
   return
   }

#############################################################
#Helper proc.

proc arrayWrite {_inarray} {
   upvar 1 $_inarray array
   set fid [open $_inarray.tcl w]
   foreach name [array names array] {
	puts $fid "set ${_inarray}($name)  [list $array($name)]"
	}
   close $fid
   return
   }

################################################################
#Helper proc to convert 7 digit mapId plus plateId to mapName
proc mapName {mapId plateId} {
   if {![regexp {([0-9][0-9][0-9][0-9][0-9])([0-9][0-9])} $mapId all \
	mjdmap version]} {return $mapId$plateId}
   return "[format %04d $plateId]-$mjdmap-$version"
   }

#################################################################
#Read plateReport file
#Also mapReport, if it exists.

proc reportCrawl {} {
   global reportData
   if {[info exists reportData]} {unset reportData}
   set files [lsort [find -nocomplain plateReport*.par]]
   set file [lindex $files end]
   echo Reading $file
   if {$file == ""} {
	echo No plateReport file found
	return
	}
   set chain [param2Chain $file hdr]
   set reportData(mapName) ""
   set reportData(plateId) ""
   chainEach hndl $chain {
	foreach var "plateId tileId mapId" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend reportData(mapName) $mapId
	set reportData($mapId,plateId) $plateId
	set reportData($mapId,status) DONE
	if {[lsearch $reportData(plateId) $plateId] < 0} {
	   lappend reportData(plateId) $plateId
	   }
	}
   genericChainDel $chain

   set files [lsort [find -nocomplain mapReport*.par]]
   set file [lindex $files end]
   if {$file == ""} {
	echo No mapReport file found
	return
	}

   echo Reading $file
   set chain [param2Chain $file hdr]
   global mapData
   set mapData(mapName) ""
   set mapData(plateId) ""
   chainEach hndl $chain {
	foreach var "plateId tileId tileRun mapId mjdMap date_done mjdobs" {
	   set $var [exprGet $hndl.$var]
	   }
	lappend mapData(mapName) $mapId
	set mapData($mapId,plateId) $plateId
	set mapData($mapId,tileId) $tileId
	set mapData($mapId,tileRun) $tileRun
	set mapData($mapId,date_done) [lindex $date_done 0]
	if {$mapData($mapId,date_done) != ""} {
	   set mapData($mapId,status) DONE
	} else {
	   set mapData($mapId,status) ""
	   }
	set mjdobs [lindex $mjdobs 0]
	if {[llength $mjdobs] > 1} {set mjdobs [eval max $mjdobs]}

#Sometimes mjdobs did not get recorded by the observers.  Set to mjdMap.
	if {$mjdobs == 0} {set mjdobs $mjdMap}
	if {$mjdobs < $mjdMap} {set mjdobs $mjdMap}
	set mapData($mapId,mjdmax) $mjdobs

#Keep track of plates that are done
	if {$mapData($mapId,status) == "DONE" && \
		[lsearch $mapData(plateId) $plateId] < 0} {
	   lappend mapData(plateId) $plateId
	   }
	}
   genericChainDel $chain

#Compare reportData with mapData
   echo Comparing reportData with mapData
   foreach mapId $reportData(mapName) {
	set plateId $reportData($mapId,plateId)
	if {[lsearch $mapData(mapName) $mapId] < 0} {
	   echo plate $plateId, map $mapId has no data
	   }
	}

#Compare mapData with reportData.  We should no have any failures here.
#Compare only maps that are DONE
   echo Comparing mapData with reportData
   foreach mapId $mapData(mapName) {
	if {$mapData($mapId,date_done) == ""} continue
	set plateId $mapData($mapId,plateId)
	if {[lsearch $reportData(plateId) $plateId] < 0} {
	   echo Curious! map $mapId is not in list of plates done.
	   }
	}

#Replace reportData with mapData
   unset reportData
   foreach elem [array names mapData] {
	set reportData($elem) $mapData($elem)
	}
   return
   }

#################################################################
#Compare report file with sp2d.
proc report2dMatch {} {
   global reportData sp2d 2d_dir 2d_alt plateList

#First, run through report and see if things are in platelist
   set npass 0
   set ngood 0
   set nbad 0
   set sp2dpass ""
   echo Scanning report file:
   foreach mapName [lsort $reportData(mapName)] {
	regexp {([0-9]+)-[0-9]+-[0-9]+} $mapName all plateId
	set plateId [string trimleft $plateId 0]
	if {$plateId < 266} continue
	if {[lsearch $sp2d(mapName) $mapName] < 0} {
	   incr npass
	   lappend sp2dpass $mapName
	   continue
	   }
	set plateId $sp2d($mapName,plateId)
	set avg [format %.2f $sp2d($mapName,mean)]
	set min [format %.2f $sp2d($mapName,min)]
	set status $sp2d($mapName,status)
	if {$status == "DONE"} {
	   incr ngood
	} else {
	   if {$reportData($mapName,status) == "DONE"} {
		echo $mapName fails S/N, avg = $avg, min = $min \
		    but is DONE in plate DB.
		}
	   incr nbad
	   }
	}
   echo $ngood plates have good S/N
   echo $nbad plates have bad S/N
   echo $npass plates not found in sp2d
   echo
   foreach miss $sp2dpass {
	if {[lsearch $sp2d(ignore) $miss] >= 0} {
	   echo $miss is qcignore
	} else {
	   set mjdmax $reportData($miss,mjdmax)
	   set plateId $reportData($miss,plateId)
	   echo $miss is missing from sp2d ($plateId,$mjdmax)

#Check to see if we even made a plan file
	   set file $2d_dir/[format %04d $plateId]/spPlan2d-[format \
		%04d $plateId]-$mjdmax.par
	   if {[file exists $file]} {
		echo "   " At least a plan file exists
		}

#Check to see if it is in alternate processing directory
	   if {[info exists 2d_alt]} {
		set file $2d_alt/[format %04d $plateId]/spPlate-[format \
		%04d $plateId]-$mjdmax.fits
	   } else {
		set file ""
		}
	   if {[file exists $file]} {
		echo "   " Found in $2d_alt
		}
	   }
	}
   echo
   echo Scanning sp2d
   set ngoodagree 0
   set ngooddisagree 0
   set nbadagree 0
   set nbaddisagree 0
   foreach mapName [lsort $sp2d(mapName)] {
	set plateId $sp2d($mapName,plateId)
	set sng1 $sp2d($mapName,sng1)
	set sng2 $sp2d($mapName,sng2)
	set snr1 $sp2d($mapName,snr1)
	set snr2 $sp2d($mapName,snr2)
	set sni1 $sp2d($mapName,sni1)
	set sni2 $sp2d($mapName,sni2)
	set avg [format %6.2f $sp2d($mapName,mean)]
	set min [format %6.2f $sp2d($mapName,min)]
	set status $sp2d($mapName,status)

#Sometimes mapName differs from report file by just the version number.
#Do a fancier search first, then look for exact match.
	set list [split $mapName -]
	set match [lindex $list 0]-[lindex $list 1]-.+
	if {$status == "DONE"} {
	   set nindex [lsearch -regexp $reportData(mapName) $match]
	   if {$nindex >= 0} {
		set dbName [lindex $reportData(mapName) $nindex]
		set dbStatus $reportData($dbName,status)
	   } else {
		set dbStatus ""
		}
	   if {$dbStatus == "DONE"} {
		incr ngoodagree
		if {[lsearch $reportData(mapName) $mapName] < 0} {
		   echo Version mismatch for $mapName between sp2d and report
		   }
	   } else {
		incr ngooddisagree
		echo Plate $plateId MJDMAX $sp2d($mapName,mjdmax) \
		   mapName $mapName good S/N but not good in mapReport
		if {[lsearch $reportData(plateId) $plateId] >= 0} {
		   echo "   " plate is declared done anyway
		   incr ngooddisagree -1
		   }
		}
	} else {
	   if {[lsearch -regexp $reportData(mapName) $match] >= 0} {
		if {$reportData($mapName,status) == "DONE"} {
		   incr nbaddisagree
		   echo Plate $plateId MJDMAX $sp2d($mapName,mjdmax) \
			mapName $mapName has bad S/N yet is \
			declared DONE.
		} else {
		   incr nbadagree
		   }
		if {[lsearch $reportData(mapName) $mapName] < 0} {
		   echo Version mismatch for $mapName between sp2d and report
		   }
	   } else {

#Plates that are bad and not in mapReport are likely from shake or other nights
#of not serious data taking, so don't print out.

#		echo Plate $plateId MJDMAX $sp2d($mapName,mjdmax) \
#		   mapName $mapName has bad S/N and is not in mapReport
		incr nbadagree
		}
	   }
	}
   echo $ngoodagree plates have good S/N and are done
   echo $ngooddisagree plates have good S/N and are not done
   echo $nbadagree plates have bad S/N and are not done
   echo $nbaddisagree plates have bad S/N and are done
   return
   }

#########################################################################
#Fetch spectro status from plate database.  Call plateReportFile
#Actually, this gets the plates.

proc spectroStatusFetch {} {
   set mjd [expr [getclock]/86400+40587]
   exec lynx -dump -width 120 "http://plate-mapper.apo.nmsu.edu:8020/plateReportFile.tml?host=&db=inventory" \
         > plateReport-$mjd.par
   return
}

#########################################################################
#Fetch spectro status from plate database.
#Actually, this gets the maps.

proc mapStatusFetch {} {
   set mjd [expr [getclock]/86400+40587]
   exec lynx -dump -width 120 "http://plate-mapper.apo.nmsu.edu:8020/mapReportFile.tml?host=&db=inventory" \
         > mapReport-$mjd.par
   return
}

#########################################################################
#Fetch Schlegel quality list.
#Actually, this gets the maps.

proc princetonFetch {} {
   global princeton
   set mjd [expr [getclock]/86400+40587]
   exec lynx -dump -width 250 "http://spectro.princeton.edu/platequality.txt" \
         > princetonQuality-$mjd.txt

#Convert to .par file
   princetonConvert $mjd

#Read in to array
   if {[info exists princeton]} {unset princeton}
   set princeton() ""
   set file [lindex [lsort [glob -nocomplain princetonQuality-*.par]] end]
   if {$file == ""} {
	echo Need to be able to fetch princetonQuality!
	return
	}
   set chain [param2Chain $file hdr]
   chainEach hndl $chain {
	foreach var "plateId mjd sn2_g1 sn2_g2 sn2_i1 sn2_i2 quality \
	   qualcomments" {
	   set $var [exprGet $hndl.$var]
	   }
	set qualcomments [lindex $qualcomments 0]
	if {[lsearch $princeton() $plateId] < 0} {
	   lappend princeton() $plateId
	   set princeton($plateId) $mjd
	} else {
	   lappend princeton($plateId) $mjd
	   }
	foreach var "sn2_g1 sn2_g2 sn2_i1 sn2_i2 quality qualcomments" {
	   set princeton($plateId,$mjd,$var) [set $var]
	   }
	}
   genericChainDel $chain

   return
}

#########################################################################
proc princetonConvert {mjd} {
   set infile princetonQuality-$mjd.txt
   set outfile princetonQuality-$mjd.par
   set fid [open $infile]
   set outid [open $outfile w]
   echo outid = $outid
   puts $outid "typedef struct {"
   set header [gets $fid]
   set cols [gets $fid]

#cols is a string of dashes for each field separated by 1 space
   set cols [split $cols " "]
   set lens ""
   set nvar 0
   set end(-1) -2
   loop i 0 [llength $cols] {
	set var [lindex $header $i]
	set string [lindex $cols $i]
	set len [string length $string]
	set start($nvar) [expr $end([expr $nvar-1]) + 2]
	set end($nvar) [expr $start($nvar)+$len - 1]
	incr nvar
	lappend lens $len
	if {$var == "PLATE"} {
	   set var plateId
	   set type int
	} elseif {$var == "MJD"} {
	   set var mjd
	   set type int
	} else {
	   regsub {\%} $var n var
	   set var [string tolower $var]
	   set type char
	   incr len
	   }
	if {$type == "int"} {
	   puts $outid "   int $var;"
	} else {
	   puts $outid "   $type $var<$len>;"
	   }
	}
   puts $outid "   } PLATEQUALITY;"

   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	if {[string length [string trim $line]] == 0} break
	set outline ""
	loop i 0 $nvar {
	   set field [string trim [string range $line $start($i) $end($i)]]
	   lappend outline $field
	   }
	puts $outid "platequality $outline"
	}
   close $fid
   close $outid
   return
   }

#########################################################################
#Compare princeton data with sp2d
proc princeton2dCompare {} {
   global sp2d princeton
   if {![info exists sp2d]} {
	echo Need to run sp2dCrawl first!
	return
	}

   if {![info exists princeton]} {
	echo Need to run princetonFetch first!
	return
	}

#Now compare sp2d with princeton
   foreach plateId $sp2d() {
	foreach mjd $sp2d($plateId) {
	   if {![info exists princeton($plateId,$mjd,quality)]} {
#These are mostly intermediate mjds.
#		echo sp2d plate $plateId mjd $mjd has no princeton data
		continue
		}
	   if {![info exists sp2d($plateId,$mjd,status)]} {
		echo No sp2d quality for plate $plateId, mjd $mjd!
		continue
		}
	   if {$sp2d($plateId,$mjd,status) == "Done" && \
		$princeton($plateId,$mjd,quality) \
		== "good" && $princeton($plateId,$mjd,qualcomments) == \
		""} continue
	   if {$sp2d($plateId,$mjd,status) != "Done" && \
		$princeton($plateId,$mjd,quality) \
		!= "good"} continue
	   echo Plate $plateId mjd $mjd sp2d quality \
		$sp2d($plateId,$mjd,status) Princeton quality \
		$princeton($plateId,$mjd,quality) Comment: \
		$princeton($plateId,$mjd,qualcomments)
	   }
	}
   return
   }

#########################################################################
#Compare princeton data with dr1
proc princetondr1Compare {} {
   global dr1 princeton
   if {![info exists princeton]} {
	echo Need to run princetonFetch first!
	return
	}
   if {![file exists dr1_plates.par]} {
	echo Need dr1_plates.par!
	return
	}
   set chain [param2Chain dr1_plates.par hdr]
   if {[info exists dr1]} {
	unset dr1
	}
   set dr1() ""
   chainEach hndl $chain {
	set plateId [exprGet $hndl.plate]
	set mjd [exprGet $hndl.mjd]
	lappend dr1() $plateId
	set dr1($plateId) $mjd
	}
   genericChainDel $chain

#Now compare dr1 with princeton
   foreach plateId $dr1() {
	set mjd $dr1($plateId)
	if {![info exists princeton($plateId,$mjd,quality)]} {
	   echo sp2d plate $plateId mjd $mjd has no princeton data
	   continue
	   }
	if {$princeton($plateId,$mjd,quality) == "good" && \
	   $princeton($plateId,$mjd,qualcomments) == ""} continue
	echo Plate $plateId mjd $mjd Princeton quality \
		$princeton($plateId,$mjd,quality) Comment: \
		$princeton($plateId,$mjd,qualcomments)
	}
   return
   }

#########################################################################
#Fetch list of all plates

proc plateListFetch {} {
   set mjd [expr [getclock]/86400+40587]
   exec lynx -dump -width 120 "http://plate-mapper.apo.nmsu.edu:8020/plateReportAllFile.tml?host=&db=inventory" \
         > plateList-$mjd.par

#Make an html file
   set outid [open plateList.html w]
   set fid [open plateList-$mjd.par]
   puts $outid "<html><head><title>Plate List</title></head>"
   puts $outid "<body><pre>"
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} break
	puts $outid $line
	}
   close $fid
   puts $outid "</pre></body></html>"
   close $outid
   return
}

################################################################
#Create a report file - do it by tile, so I can read back into tileChunkPlot
#Must have run tileQuery on opdb to get tile.par
#I read info from mapReport.  Anything declared done there stays done unless
#overridden  by sp2d.
#Only tiles stored in opdb are kept here.  Thus, many secondary plates are
#NOT recorded.  Perhaps this should change some day?

proc specTileReport {} {
   global sp2d reportData

   if {![info exists sp2d]} {
	echo Running sp2dCrawl ...
	sp2dCrawl
	}
   if {![info exists reportData]} {
	echo Running reportCrawl ...
	reportCrawl
	}

#Read tile.par.  This is created by tileQuery in photoSch database.
   set mtime [file mtime tile.par]
   set mdate [fmtclock $mtime "%Y-%h-%d"]
   echo Reading tile.par.  It was generated on $mdate
   set tileChain [param2Chain tile.par hdr]
   chainEach hndl $tileChain {
	set tileId [exprGet $hndl.tileId]
	set chunkId [exprGet $hndl.chunkId]
	set raCen [exprGet $hndl.raCen]
	set decCen [exprGet $hndl.decCen]
	set tile($tileId,chunkId) $chunkId
	set tile($tileId,raCen) $raCen
	set tile($tileId,decCen) $decCen
	set tile($tileId,status) NOT_DONE
	lappend tileSurveyList [exprGet $hndl.tileId]
	}
   genericChainDel $tileChain

#Include data from Mar31/Apr01, e..g. so allow equality.
   set tileList ""
   foreach mapId $reportData(mapName) {

#Keep track of unique plates and tiles - avoid duplicate exposures.
	set plateId $reportData($mapId,plateId)
	set tileId $reportData($mapId,tileId)

#Skip old or test data
	if {$plateId < 266} continue

#Get status as estimated by the mountain; let status from data processing
#override if data have been processed.
	set status [string toupper $reportData($mapId,status)]
	set tileRun $reportData($mapId,tileRun)

#Some of these plates want to get counted - I should really divide plates
#based on tileRun name.
	if {$tileId >= 9000} continue

#What if there is a discrepancy between sp2d and the report?  In some cases,
#2d says the plate is done, but I have my doubts.
	if {[info exists sp2d($mapId,status)]} {

#If neither status is DONE, skip
	   if {$status != "DONE" && $sp2d($mapId,status) != "DONE"} continue
	   if {[string toupper $sp2d($mapId,status)] != "$status"} {
		set status MAYBE
		}
	   }
	if {$status != "DONE" && $status != "MAYBE"} continue
	if {![info exists tile($tileId,status)]} continue
	set tile($tileId,tileRun) $tileRun

#Logic here is still incomplete - I want to elevate tile status to the
#greater of the new status and the existing status.
	if {$tile($tileId,status) != "DONE"} {
	   set tile($tileId,status) $status
	   }
	if {[lsearch $tileList $tileId] < 0} {
	   lappend tileList $tileId}
	}
   set today [expr [getclock]/86400 + 40587]
   set file tileReport-$today.par
   echo Creating $file
   set fid [open $file w]
   puts $fid "#Report created [fmtclock [getclock]]"
   puts $fid "mjdReport $today"
   puts $fid "typedef struct {"
   puts $fid "   int tileId;"
   puts $fid "   int chunkId;"
   puts $fid "   char tileRun<20>;"
   puts $fid "   char status<10>;"
   puts $fid "   } TILESTAT;"

   set n 0
   foreach tileId [lsort -integer $tileList] {
	puts $fid "TILESTAT $tileId $tile($tileId,chunkId) \
	   $tile($tileId,tileRun) $tile($tileId,status)"
	}
   close $fid
   return
   }



################################################################
#The following code is not being run at present!!!  Used specRunHistory
#instead.
#
#Whoops, not true, this is still run! It creates specStatusHistory.txt file
#for John Inkmann
#
#Reconstruct history of spectroscopy by quarter.
# Must have run reportCrawl first.
#Must have run sp2dCrawl to get results of data processing.
#Must have run tileQuery on opdb to get tile.par

proc specStatusHistory {{stdout ""}} {
   global sp2d reportData

   if {![info exists sp2d]} {
	echo Running sp2dCrawl ...
	sp2dCrawl
	}
   if {![info exists reportData]} {
	echo Running reportCrawl ...
	reportCrawl
	}

   set specHistory() ""

   if {$stdout != ""} {
	echo Creating $stdout
	set stdout [open $stdout w]
   } else {
	set stdout stdout
	}	
#Loop through all maps with data.
#Definitions:
#	gross - number of maps for which data were taken
#	good - number of maps for which data are good
#	unique - number of plates for which data are good
#	footprint - number of tiles for which data are good
#

#Read tile.par.  This is created by tileQuery in photoSch database.
   set mtime [file mtime tile.par]
   set mdate [fmtclock $mtime "%Y-%h-%d"]
   echo Reading tile.par.  It was generated on $mdate
   set tileChain [param2Chain tile.par hdr]
   set tileSurveyList ""
   chainEach hndl $tileChain {
	set tileId [exprGet $hndl.tileId]
	set chunkId [exprGet $hndl.chunkId]
	set raCen [exprGet $hndl.raCen]
	set decCen [exprGet $hndl.decCen]
	set tile($tileId,chunkId) $chunkId
	set tile($tileId,raCen) $raCen
	set tile($tileId,decCen) $decCen
	lappend tileSurveyList [exprGet $hndl.tileId]
	}
   genericChainDel $tileChain
   set mjdnow [expr [getclock]/86400 + 40587]
   puts $stdout "     Date       All    Good     Unique  N(tiles) N(tiles) \
	N(tiles)"
   puts $stdout "                Obs     Obs     Plates   North    South \
	Secondary"

   foreach year "1999 2000 2001 2002 2003 2004 2005 2006 2007 2008" {
        lappend specHistory() $year
        set specHistory($year) ""
	foreach mon "Jan Apr Jul Oct" {
           lappend specHistory($year) $mon
           set mjd [expr [convertclock "01 $mon $year"]/86400 + 40587]

#Allow for current quarter, but nothing beyond.
	   if {$mjd > $mjdnow+90} break

#Include data from Mar31/Apr01, e..g. so allow equality.
	   set ngross 0; set ngood 0; set nunique 0; set nfootn 0; set nfoots 0
	   set nsec 0
	   set plateList ""
	   set tileList ""
	   set nreject 0
	   set plateReject ""
	   set nunknown 0
	   set plateUnknown ""
	   foreach mapId $reportData(mapName) {
		if {$reportData($mapId,mjdmax) > $mjd} continue
		incr ngross

#Keep track of unique plates and tiles - avoid duplicate exposures.
		set plateId $reportData($mapId,plateId)
		set tileId $reportData($mapId,tileId)

#Get status as estimated by the mountain; let status from data processing
#override if data have been processed.
		set status $reportData($mapId,status)
		set tileRun [lindex $reportData($mapId,tileRun) 0]

#Get program name from tileRun.  Some plates count more than others
		if {![regexp {^([A-Za-z]+)} $tileRun all prog]} {
		   echo Couldn't decode program $tileRun!
		   set prog ""
		   }
		set unit 1
		if {[lsearch "photoz lowz annis fstar loveday" \
		   $prog] >= 0} {set unit 2}
		if {[lsearch "tremonti" $prog] >= 0} {set unit 7}

#Skip old or test data
#		if {$plateId < 266} continue

#Some of these plates want to get counted - I should really divide plates
#based on tileRun name.
#		if {$tileId >= 9000 && ![regexp photoz $tileRun]} continue

#Delete some special runs
		if {[regexp FASTT $tileRun] || [regexp M67 $tileRun]} continue
		if {[regexp chunk $tileRun] && $plateId < 266} continue

		if {[info exists sp2d($mapId,status)]} {
#Count up plates that are done in plateDB but not done in sp2d.  These are
#not counted in the final report but might be good, because sp2d status
#is sometimes suspect
		   if {$status == "DONE" && $sp2d($mapId,status) != "DONE"} {

#Count as a reject only if no other maps exist that are DONE
			if {$sp2d($plateId,status) != "Done"} {
			   incr nreject
			   lappend plateReject $plateId
			   }
			}
		   set status $sp2d($mapId,status)
		} else {

#Map not processed.  See if other maps for this plate were done.
		   if {![info exists sp2d($plateId,status)]} {
			incr nunknown
			lappend plateUnknown $plateId
			}
		   }
		if {$status != "DONE"} continue
		incr ngood

		if {[lsearch $plateList $plateId] < 0} {
		   incr nunique
		   lappend plateList $plateId
		   }
		if {[lsearch $tileList $tileId] < 0} {

#Is tile on the official survey list?
		   if {[lsearch $tileSurveyList $tileId] >= 0} {
			if {$tile($tileId,raCen) > 90 && $tile($tileId,raCen) \
			   < 270 && [regexp chunk $tileRun]} {
			   incr nfootn $unit
			} else {
			   if {[regexp south $tileRun]} {
				incr nsec $unit
			   } elseif {[regexp chunk $tileRun]} {
				incr nfoots $unit
			   } else {
#Some photoz tiles were stuffed into opdb.  They get counted here.
				incr nsec $unit
				}
			   }
		   } else {
			incr nsec $unit
			}
		   lappend tileList $tileId
		   }
		}
	   set format "%4s-%3s %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f"
	   puts $stdout [format $format $year $mon $ngross $ngood $nunique \
		$nfootn $nfoots $nsec]

#Create specStatus.par for John Inkmann.  Overwrite continuously, so the last
#time through produces the most recent status.
           set fid [open specStatus.par w]
           puts $fid "date [fmtclock [getclock] "%Y %h %d"]"
           puts $fid "mjd $mjdnow"
           puts $fid "nall $ngross         #All plate/map observations"
           puts $fid "ngood $ngood        #Good plate/map observations"
	   puts $fid "nplate $nunique     #Number of plates observed"
	   puts $fid "ntile [expr $nfootn + $nfoots]	\
       #Number of tiles observed"

#Baseline is sum of north, south, and equatorial areas observed once.
#Taken from Kron plan.  This presumes that if we add tiles that go deeper
#in the south, those plates do NOT get counted above.
	   set ntotal [expr 1540 + 95 + 54]
           puts $fid "ntotal $ntotal       #Total number of tiles to observe"
           close $fid
	   }
	}
echo Number of plates declared done in database but rejected by sp2d = $nreject
echo Plates are [lsort -integer $plateReject]
echo Number of plates with no dp status = $nunknown
echo Plates are [lsort -integer $plateUnknown]
   if {$stdout != "stdout"} {
	close $stdout
	}

   return
   }



################################################################
#Reconstruct history of spectroscopy by dark run
#Make this in html so the observers can read it.

proc specRunHistory {} {
   global sp2d reportData

   set specHistory() ""

   echo Creating specHistory.html
   set stdout [open specHistory.html w]

#Loop through all maps with data.
#Definitions:
#	gross - number of maps for which data were taken
#	good - number of maps for which data are good
#	unique - number of plates for which data are good
#	footprint - number of tiles for which data are good
#

#Read tile.par
   set tileChain [param2Chain tile.par hdr]
   set tileSurveyList ""
   chainEach hndl $tileChain {
	set tileId [exprGet $hndl.tileId]
	set chunkId [exprGet $hndl.chunkId]
	set raCen [exprGet $hndl.raCen]
	set decCen [exprGet $hndl.decCen]
	set tile($tileId,chunkId) $chunkId
	set tile($tileId,raCen) $raCen
	set tile($tileId,decCen) $decCen
	lappend tileSurveyList [exprGet $hndl.tileId]
	}
   genericChainDel $tileChain
   set mjdnow [expr [getclock]/86400 + 40587]
   puts $stdout "<head>"
   puts $stdout "<title>Spectro History</title>"
   puts $stdout "</head>"
   puts $stdout ""
   puts $stdout "<body>"
   puts $stdout "<h1 align=center>Spectroscopy History</h1>"
   puts $stdout "<h3 align=center>[fmtclock [getclock] %Y-%h-%d]</h3>"
   puts $stdout "<table border=1 align=center>"
   puts $stdout "<tr><th>Date</th><th>All</th><th>Good</th><th>Unique</th> \
	<th>N(tiles)</th><th>N(tiles)</th><th>N(tiles)</th>\
	<th>Monthly Increment</th></tr>"
   puts $stdout "<tr><th>Obs</th><th>Plates</th><th>Plates</th>\
	<th>Plates</th> \
	<th>North</th><th>South</th><th>Secondary</th> \
	<th>Exposure-Equiv</th></tr>"

   set chains [param2Chain moon.par hdr]
   foreach chain $chains {
        if {[chainTypeDefine $chain] != "FULL"} {
           genericChainDel $chain
        } else {
echo Full moon
           set full $chain
           }
        }
   if {![info exists full]} {
        echo No full moon chain - gak!
        return
        }

   set nold 0
   chainEach hndl $full {
        set date [lindex [exprGet $hndl.date] 0]
        set year [lindex $date 0]
        set mon [lrange $date 1 2]
        set mjd [exprGet $hndl.mjd]
	if {$year < 2000} continue

#Sometimes I like to see how we are doing in the current run.
        if {$mjd > $mjdnow+30} continue
        lappend specHistory() $year
        set specHistory($year) ""
           lappend specHistory($year) $mon

	   set ngross 0; set ngood 0; set nunique 0; set nfootn 0; set nfoots 0
	   set nsec 0
	   set plateList ""
	   set tileList ""
	   set nreject 0
	   set plateReject ""
	   set nunknown 0
	   set plateUnknown ""
	   foreach mapId $reportData(mapName) {
		if {$reportData($mapId,mjdmax) > $mjd} continue

#Keep track of unique plates and tiles - avoid duplicate exposures.
		set plateId $reportData($mapId,plateId)
		set tileId $reportData($mapId,tileId)

#Get status as estimated by the mountain; let status from data processing
#override if data have been processed.
		set status $reportData($mapId,status)
		set tileRun [lindex $reportData($mapId,tileRun) 0]

#Get program name from tileRun.  Some plates count more than others
		if {![regexp {^([A-Za-z]+)} $tileRun all prog]} {
		   echo Couldn't decode program $tileRun!
		   set prog ""
		   }
		set unit 1
		if {[lsearch "photoz lowz annis fstar loveday" \
		   $prog] >= 0} {set unit 2}
		if {[lsearch "tremonti" $prog] >= 0} {set unit 7}

		incr ngross $unit

#Skip old or test data
#		if {$plateId < 266} continue

#Some of these plates want to get counted - I should really divide plates
#based on tileRun name.
#		if {$tileId >= 9000 && ![regexp photoz $tileRun]} continue

#Delete some special runs
		if {[regexp FASTT $tileRun] || [regexp M67 $tileRun]} continue
		if {[regexp chunk $tileRun] && $plateId < 266} continue

		if {[info exists sp2d($mapId,status)]} {
#Count up plates that are done in plateDB but not done in sp2d.  These are
#not counted in the final report but might be good, because sp2d status
#is sometimes suspect
		   if {$status == "DONE" && $sp2d($mapId,status) != "DONE"} {

#Count as a reject only if no other maps exist that are DONE
			if {$sp2d($plateId,status) != "Done"} {
			   incr nreject
			   lappend plateReject $plateId
			   }
			}
		   set status $sp2d($mapId,status)
		} else {

#Map not processed.  See if other maps for this plate were done.
		   if {![info exists sp2d($plateId,status)]} {
			incr nunknown
			lappend plateUnknown $plateId
			}
		   }
		if {$status != "DONE"} continue
		incr ngood $unit

		if {[lsearch $plateList $plateId] < 0} {
		   incr nunique
		   lappend plateList $plateId
		   }
		if {[lsearch $tileList $tileId] < 0} {

#Is tile on the official survey list?
		   if {[lsearch $tileSurveyList $tileId] >= 0} {
			if {$tile($tileId,raCen) > 90 && $tile($tileId,raCen) \
			   < 270 && [regexp chunk $tileRun]} {
			   incr nfootn $unit
			} else {
			   if {[regexp south $tileRun]} {
				incr nsec $unit
			   } elseif {[regexp chunk $tileRun]} {
				incr nfoots $unit
			   } else {
#Some photoz tiles were stuffed into opdb.  They get counted here.
				incr nsec $unit
				}
			   }
		   } else {
			incr nsec $unit
			}
		   lappend tileList $tileId
		} else {

#This message is generated if we have multiple good exposures for a single
#plate.
		   echo Duplicate tile $tileId for plate $plateId
		   }
		}
	   set nincr [expr $ngood - $nold]
	   set nold $ngood
	   puts $stdout "<tr><td>$mon $year</td><td>$ngross</td> \
		<td>$ngood</td><td>$nunique</td><td>$nfootn</td> \
		<td>$nfoots</td><td>$nsec</td><td>$nincr</td></tr>"

#Baseline is sum of north, south, and equatorial areas observed once.
#Taken from Kron plan.  This presumes that if we add tiles that go deeper
#in the south, those plates do NOT get counted above.
	   set ntotal [expr 1540 + 95 + 54]
	}
   puts $stdout "</table></body>"
   close $stdout

   return
   }

########################################################################
#See how effect 1d QC is
#stat can be either lrg or mg.  lrg seems to be more sensitive.
proc qcCheck {{stat lrg}} {
   global sp2d
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}

   pgEnv 0 40 0.8 1 0 0
   pgLabel "MIN S/N" "Per Cent $stat with zconf > 0.5" ""
   foreach plateId [lsort $sp2d()] {
	if {$plateId >= 673 && $plateId <= 714} continue
	foreach mjdmax $sp2d($plateId) {
	   if {[info exists sp2d($plateId,$mjdmax,n${stat})]} {
		if {$sp2d($plateId,$mjdmax,n${stat}) == 0} continue
		set avg [format %.2f [expr ($sp2d($plateId,$mjdmax,spec1_g) + \
			$sp2d($plateId,$mjdmax,spec1_i) + \
			$sp2d($plateId,$mjdmax,spec2_g) + \
			$sp2d($plateId,$mjdmax,spec2_i))/4.]]
		set min [format %.2f [min $sp2d($plateId,$mjdmax,spec1_g) \
			$sp2d($plateId,$mjdmax,spec1_i)  \
			$sp2d($plateId,$mjdmax,spec2_g)  \
			$sp2d($plateId,$mjdmax,spec2_i)]]
		set percent [format %.2f [expr \
		   $sp2d($plateId,$mjdmax,n${stat}hc) \
		   /[double $sp2d($plateId,$mjdmax,n${stat})]]]
		if {$sp2d($plateId,$mjdmax,nlrg) < 20} continue
		echo $plateId $mjdmax \
		   $sp2d($plateId,$mjdmax,n${stat}) $avg $percent

		pgPoint $min $percent 3
		}
	   }
	}
   return
   }





