##############################################################################
#Fetch quality info from opdb.  Compare with field limits in run.par
#I must have deleted a previous version of this file.
#Run additional QA checks based on sky, image size.
#The idea is to look for bad data that are flagged as good.
#Note: All info from opdb is now extracted in separate procs and cached in
#files to avoid memory leaks in opdb.
#
#Run from photoSch
#
#Top-level proc:  statusCheckRuns.
#
#Be sure to run the following procs first to prefetch info!
#Top-level proc to cache quality info:  qualitySave
#					skySave
#					fangSave
#					   (This bypasses existing runQuality
#					   files and sscPsf files.
#
#Run the following if you haven't done "statusUpdate" in a while and new
#runs have been processed.
#					runInventorySave
#
##############################################################################
global qdir

#Top level directory for quality files.  Get them out of mt directory.
set qdir quality

proc statusFetchRun {run {rerun ""} {skyVersion 0}} {
   global qdir
   global runQuality

   global runQuality-$run
   set file $qdir/runQuality-$run.tcl
   if {[file exists $file]} {
	source $file
	array set runQuality [array get runQuality-$run]
	return
	}
#echo Gak! no runQuality file found.
#exit
    if {![inSession]} {
	opdbConnect
	}

#Make a list of runs
   global runTable
   if {![info exists runTable]} {
	if {[info exists runTable.tcl]} {
	   source runTable.tcl
	} else {
	   echo Must create runTable.tcl externally (e.g. with runStatus)
	   opdbDisconnect
	   exit
	   }
	}

   if {$rerun == ""} {set rerun $runTable($run)}

   # Translate the enums
   # arFieldQuality refers to the bit masks.

   foreach enum [enumGet arFieldQuality] {

#Convert num string to a common name
	if {![regexp "AR_FQ_(.+)" $enum all name]} {
	    echo Can't decode enum $enum
	    set name $enum
	}
    	set bit($name) [enumStringToInt arFieldQuality $enum]
    }

   foreach name [array names bit] {
	set num $bit($name)
	set bit($num) $name
	}

   if {![info exists runQuality()]} {
	set runQuality() $run
   } else {
	if {[lsearch $runQuality() $run] < 0} {
	   lappend runQuality() $run
	   }
	}
   set runQuality($run) ""
#Read from db
    set totalQuality($run) ""
    loop camCol 3 4 {
	lappend totalQuality($run) $camCol
	set totalQuality($run,$camCol) ""
        set frames [pipeGet frames $run $camCol -rerun $rerun]
        set asCalib [$frames -> currentAstrometricCalibration]
	set startField [$frames -> field0]
	set endField [expr $startField + [$frames -> nFields] - 1]
	set nfield [$frames -> nFields]
	set imgRun [$frames -> imagingRun]
	set stripe [$imgRun ->  stripe]
	set strip [$imgRun -> strip]
	regsub {AR_STRIP_} $strip "" strip
	set strip [string index $strip 0]
	objDel $imgRun

        loop field $startField [expr $endField +1] {
	    lappend totalQuality($run,$camCol) $field
            set olist [$frames -> objectList $field]

#Which quality.  0 = target, 1 = best
	    set nqual [$olist -> quality $skyVersion]
	    set quality [enumIntToString arDataQuality $nqual]

	    set goodMask [$olist -> goodMask $skyVersion]
	    set acceptableMask [$olist -> acceptableMask $skyVersion]
	    set badMask [$olist -> badMask $skyVersion]
	    set holeMask [$olist -> holeMask $skyVersion]
	    regexp "AR_DQ_(.+)" $quality all quality
	    set quality [string tolower $quality]
            objDel $olist

	    if {$field == $startField} {
		set qprev $quality
	 	}

#Don't allow holes, since these can be for just one camcol.
	    if {$quality == "hole" && $holeMask == 0} {
		set quality undetermined
		}
	    if {$quality == "hole"} {
		set quality $qprev
		}
#Require that a mask not be blank
	    if {$quality == "good" && $goodMask == 0} {
		set quality undetermined
		}
	    if {$quality == "acceptable" && $acceptableMask == 0} {
		set quality undetermined
		}
	    if {$quality == "bad" && $badMask == 0} {
		set quality undetermined
		}

	    if {$field == $startField} {
		set qprev $quality
	 	}
	    set override 0
	    set list ""

#Don't bother checking goodMask because these are the default flags if set.
	    foreach mask "acceptableMask badMask holeMask" {
		loop i 0 24 {
		   set num [expr int(pow(2,$i))]
		   if {$num & [set $mask]} {
			lappend list [list $mask $bit($num)]
			if {$bit($num) == "MANUAL"} {set override 1}
			}
		   }
		}
	    lappend runQuality($run) $field
	    set runQuality($run,$field,quality) $quality
	    set runQuality($run,$field,override) $override
	    set runQuality($run,$field,mask) $list
	    }
 	objDel $asCalib
 	objDel $frames
    }
    sessionEnd
    return
}
    
#######################################################################
#Cache quality info into a file.
#AARGGHH!! The runs are too big to fetch all in one photoSch job.
#I will run a kentools job, call photoSch, and archive one run at a time.
#This is slow, but should be more effective.
#Assume that runTable is already cached.
#I should rewrite this to run qualitySaveByRun only if not already cached.

proc qualitySave {} {
   global qdir
   source runTable.tcl
   set file qualitySave
   set fid [open $file w]
   puts $fid ". /usr/local/etc/setups.sh"
   puts $fid "setup photoSch"
   puts $fid "echo Running photoSch"
   foreach run $runTable() {
	set file $qdir/runQuality-$run.tcl
	if {[file exists $file]} {
	   echo Skipping $file
	   continue
	   }
	puts $fid "photoSch -noTk -command \
	   \"source init.tcl; qualitySaveByRun $run\""
	}
   close $fid
   chmod +x qualitySave
   echo Created script \"qualitySave\"
   echo Now run \"qualitySave\"
   return
   }

#######################################################################
#Read one run and write to a file.

proc qualitySaveByRun {run} {
   global qdir
   global runTable runQuality runQuality-$run
   echo Run $run
   opdbConnect
   source runTable.tcl
   statusFetchRun $run

#Copy array to a temporary
   array set runQuality-$run [array get runQuality]
   arrayWrite runQuality-$run
   set file runQuality-$run.tcl
   exec mv $file $qdir/$file
   opdbDisconnect
   return
   }
#######################################################################
#Compress a list of numbers to a set of ranges

proc listCompress {list} {
   if {[llength $list] <= 1} {return $list}
   set outlist ""
   set l1 [lindex $list 0]
   set l2 $l1
   loop i 1 [llength $list] {
	set l [lindex $list $i]
	if {$l == $l2+1} {
	   set l2 $l
	} else {
	   if {$l2 == $l1} {
		lappend outlist $l1
	   } else {
		lappend outlist $l1-$l2
		}
	   set l1 $l
	   set l2 $l1
	   }
	}
   if {$l2 == $l1} {
	lappend outlist $l1
   } else {
	lappend outlist $l1-$l2
	}
   return $outlist
   }
#######################################################################
proc statusCheckRun {run {rerun ""} {skyVersion 0}} {
   global runQuality runChain runTable global fieldFix

   if {![info exists runChain]} {
	set runChain [param2Chain run.par hdr]
	}
   set hndl [chainGetOne $runChain run $run]
   if {$hndl == ""} {
	echo "Run $run not in run.par!"
	return
	}
   set frameStart [exprGet $hndl.frameStart]
   set frameEnd [exprGet $hndl.frameEnd]
   handleDel $hndl

   if {![info exists runQuality($run)]} {
	statusFetchRun $run $rerun $skyVersion
	}

   if {![info exists fieldFix()]} {
	set fieldFix() ""
	}
   if {[lsearch $fieldFix() $run] < 0} {
	lappend fieldFix() $run
	}
   set fieldFix($run) ""

   echo Checking run $run rerun $runTable($run)

#Check skies
   set list [skyTwiCheck $run]
   set fieldEve [lindex $list 0]
   set fieldMorn [lindex $list 1]

#Check fangs for doughnuts
   set list [fangCheckRun $run]
   set fangStart [lindex $list 0]
   set fangEnd [lindex $list 1]

#Check field limits from run.par, sky.
#Keep track of bad fields for each possible cause
   set runField ""
   set skyField ""
   set fangField ""
   foreach field $runQuality($run) {
	set quality $runQuality($run,$field,quality)
	if {$quality == "bad"} continue
	if {$field < $frameStart || $field > $frameEnd} {
#	   echo Field $field is not bad and outside run.par limits!
	   lappend runField $field
	   }
	if {$field < $fieldEve || $field > $fieldMorn} {
#	   echo Field $field is not bad and outside sky limits!
	   lappend skyField $field
	   }
	if {$field < $fangStart || $field > $fangEnd} {
#	   echo Field $field is not bad and outside fang limits!
	   lappend fangField $field
	   lappend fieldFix($run) $field
	   }
	}
   if {[llength $runField] > 0} {
	echo "   " "Fields outside run.par limits:" [listCompress $runField]
	}
   if {[llength $skyField] > 0} {
	echo "   " "Fields outside sky     limits:" [listCompress $skyField]
	}
   if {[llength $fangField] > 0} {
	echo "   " "Fields outside fang    limits:" [listCompress $fangField]
	}
   set fieldFix($run,run) $runField
   set fieldFix($run,sky) $skyField
   set fieldFix($run,fang) $fangField
   return
   }

########################################################################
#The following procs are imported from skyGet.tcl

#Fetch photometric zero points and sky values from opdb
#Useful commands:
#skyGet run camcol [rerun] - basic routine to fetch skies
#skyPlot run camcol filter - plot sky
#skyTwiCheck run           - Check field limits for one run
#skyCheckRuns              - Check all runs in opdb

proc skyGet {run camcol {rerun ""}} {
   global skies runTable
   if {[info exists skies($run,$camcol,g)]} return
   if {![inSession]} {
	opdbConnect
	}
   if {![info exists runTable]} {
	runInventory
	}
   if {$rerun == ""} {set rerun $runTable($run)}
   set frames [pipeGet frames $run $camcol -rerun $rerun]
   set calib [$frames -> currentPhotometricCalibration]
   set field0 [$frames -> field0]
   set nFields [$frames -> nFields]

#Fetch skies
   foreach filter "u g r i z" {
	set skies($run,$camcol,$filter) ""
	}
   set skies($run,$camcol,field) ""
   loop field $field0 [expr $nFields+$field0] {
	set objList [$frames -> objectList $field]
	foreach filter "u g r i z" {
	   set string AR_[string toupper $filter]
	   set int [enumStringToInt arFilter $string]
	   set frameStat [$objList -> frameStat $int]
	   set sky [$frameStat sky_frames]
	   handleDel $frameStat
	   lappend skies($run,$camcol,$filter) [format %.2f $sky]
	   }
	lappend skies($run,$camcol,field) $field
	objDel $objList
	}
   objDel $frames
   sessionEnd
   return
   }

#################################################################
#Read skies from opdb and save in a file
#One file serves all runs, so we rerun regardless.
proc skySave {} {
   global qdir
   global runTable skies
   if {![info exists runTable]} {
	if {[file exists runTable.tcl]} {source runTable.tcl}
	}
   if {![info exists runTable]} {
	runInventory
	}
   foreach run $runTable() {
	skyGet $run 3
	}
   arrayWrite skies
   opdbDisconnect
   set file skies.tcl
   exec mv $file $qdir/$file
   return
   }

#################################################################
proc skyPlot {run camcol filter} {
   global skies
   if {![info exists skies($run,$camcol,field)]} {
	skyGet $run $camcol
	}
   if {[pgQinf STATE] == "CLOSED"} {
	pgBegin; pgGeomSet 400 400 10 25; pgAsk 0
	}
   set fmin [lindex $skies($run,$camcol,field) 0]
   set fmax [lindex $skies($run,$camcol,field) end]
   set skymax [eval max $skies($run,$camcol,$filter)]
   pgEnv $fmin $fmax 0 $skymax 0 0
   set i 0
   foreach field $skies($run,$camcol,field) {
	set sky [lindex $skies($run,$camcol,$filter) $i]
	if {$sky == ""} continue
	incr i
	pgPoint $field $sky 3
	}
   return
   }

#####################################################################
#Check for twilight.  Look in u filter, camcol 3

proc skyTwiCheck {run} {
   global skies runTable
   if {![info exists skies($run,3,field)]} {
	skyGet $run 3
	}

#Evening twilight - go in forward direction
#First, median
   set median [lindex [lsort -real $skies($run,3,u)] \
	[expr [llength $skies($run,3,u)]/2]]
   loop i 0 [llength $skies($run,3,field)] {
	set field [lindex $skies($run,3,field) $i]
	set sky [lindex $skies($run,3,u) $i]
	if {$sky < 1.5*$median} break
	}
   set fieldEve $field

#Morning twilight - go in reverse direction
#First, median
   set median [lindex [lsort -real $skies($run,3,u)] \
	[expr [llength $skies($run,3,u)]/2]]
   loop j 0 [llength $skies($run,3,field)] {
	set i [expr [llength $skies($run,3,field)] - $j - 1]
	set field [lindex $skies($run,3,field) $i]
	set sky [lindex $skies($run,3,u) $i]
	if {$sky < 1.5*$median} break
	}
   set fieldMorn $field

   set first [lindex $skies($run,3,field) 0]
   set last [lindex $skies($run,3,field) end]
   if {$fieldEve > $first} {
#	echo Run $run, recommend first $first -> $fieldEve
	}
   if {$fieldMorn < $last} {
#	echo Run $run, recommend last $last -> $fieldMorn
	}
   return [list $fieldEve $fieldMorn]
   }

###########################################################################
#The following procs are imported from fang.tcl
#
#Determine start, end fields for a run based on focus log, SSC widths, and
#actual fields processed.  This runs off opdb.
#
#Single run cammands:
#   fangCheckRun run
#   fangPlotRun
#   focusLimits run
#   focusPlot run
#   runLimits run (does fangCheckRuna dn focusLimits)
#   fangCheck (does all runs in opdb)

#For fangs. read camcol 3, HDU 10 from fang files for a given run, rerun,
#compute median rmajor and use as a measure of seeing.  See if this is a good
#QA diagnostic.

#For focus, read from golden directory.

proc fangGetRun {run} {
   global qdir
   global photo_frames
   global sscPsf

#Read from sscPsf-$run.tcl if it exists
   set file $qdir/sscPsf-$run.tcl
   if {[file exists $file]} {
	source $file
	array set sscPsf [array get sscPsf-$run]
	return
	}
	
   if {![inSession]} {opdbConnect}
   set sscRun [pipeGet ssc $run 3]
   set field0 [$sscRun -> field0]
   set nfields [$sscRun -> nFields]
   set sscPsf($run) ""
   loop field $field0 [expr $field0+$nfields] {
	lappend sscPsf($run) $field
	set starList [$sscRun -> stars $field]
	set gstars [$starList -> stars g]
	set medgList ""
	set size [$gstars size]
	loop i 0 $size {
	   set hndl [$gstars elem $i]
	   set rMajor [$hndl rMajor]
	   set counts [$hndl counts]
	   handleDel $hndl
	   if {$counts > 100} {
		lappend medgList $rMajor
		}
	   }

	handleDel $gstars
	set rstars [$starList -> stars r]
	set medrList ""
	set size [$rstars size]
	loop i 0 $size {
	   set hndl [$rstars elem $i]
	   set rMajor [$hndl rMajor]
	   set counts [$hndl counts]
	   handleDel $hndl
	   if {$counts > 100} {
		lappend medrList $rMajor
		}
	   }
	handleDel $rstars
	objDel $starList
	if {[llength $medgList] > 0} {
	   set medgList [lsort -real $medgList]
	   set gmedian [lindex $medgList [expr [llength $medgList]/2]]
	} else {
	   set gmedian 0
	   }
	if {[llength $medrList] > 0} {
	   set medrList [lsort -real $medrList]
	   set rmedian [lindex $medrList [expr [llength $medrList]/2]]
	} else {
	   set rmedian 0
	   }
	set sscPsf($run,$field,g) $gmedian
	set sscPsf($run,$field,r) $rmedian
	}
   objDel $sscRun
   return
   }

#######################################################################
#Cache fang info into a file.
#AARGGHH!! The fangs are too big to fetch all in one photoSch job.
#I will run a kentools job, call photoSch, and archive one run at a time.
#This is slow, but should be more effective.
#Assume that runTable is already cached.

proc fangSave {} {
   global qdir
   source runTable.tcl
   set file fangSave
   set fid [open $file w]
   puts $fid ". /usr/local/etc/setups.sh"
   puts $fid "setup photoSch"
   puts $fid "echo Running photoSch"
   foreach run $runTable() {
	set file $qdir/sscPsf-$run.tcl
	if {[file exists $file]} {
	   echo Skipping $file
	   continue
	   }
	puts $fid "photoSch -noTk -command \
	   \"source init.tcl; fangSaveByRun $run\""
	}
   close $fid
   chmod +x fangSave
   echo Now run \"fangSave\"
   return
   }

#######################################################################
#Read one run and write to a file.

proc fangSaveByRun {run} {
   global qdir
   global sscPsf sscPsf-$run
   echo Run $run
   opdbConnect
   fangGetRun $run

#Copy array to a temporary
   array set sscPsf-$run [array get sscPsf]
   arrayWrite sscPsf-$run
   set file sscPsf-$run.tcl
   exec mv $file $qdir/$file
   opdbDisconnect
   return
   }
#######################################################################
proc fangCheckRun {run} {
   global sscPsf
   global runTable
   if {![info exists sscPsf($run)]} {fangGetRun $run}

#Make guestimate of first and last good fields
   set first [lindex $sscPsf($run) 0]
   set last [lindex $sscPsf($run) end]
   set fieldStart $first
   foreach field $sscPsf($run) {
	if {$sscPsf($run,$field,g) == 0 || $sscPsf($run,$field,r) == 0} \
	   continue
	set diff [expr $sscPsf($run,$field,g) - $sscPsf($run,$field,r)]
	if {$diff < -1.} continue
	if {$sscPsf($run,$field,r) > 3.} continue
	break
	}
   set fieldStart [expr $field]
   if {$fieldStart > $last} {set fieldStart $last}

   set fieldEnd $first
   foreach field [lsort -integer -decreasing $sscPsf($run)] {
	if {$sscPsf($run,$field,g) == 0 || $sscPsf($run,$field,r) == 0} \
	   continue
	set diff [expr $sscPsf($run,$field,g) - $sscPsf($run,$field,r)]
	if {$diff > 2.} continue
	if {$sscPsf($run,$field,g) > 3.} continue
	break
	}
   set fieldEnd [expr $field]
   if {$fieldEnd < $first} {set fieldEnd $first}

#If we are not at ends of run, apply one more trim
   if {$fieldStart != $first} {incr fieldStart 2}
   if {$fieldEnd != $last} {incr fieldEnd -2}
#   echo first $first, last $last
#   echo fieldStart $fieldStart fieldEnd $fieldEnd

#And check focus limits
#   echo Hit <cr>
#   gets stdin
#   echo Focus limits $limits

   return "$fieldStart $fieldEnd"
   }

####################################################################
proc fangPlotRun {run} {
   global sscPsf
   set xmax [lindex $sscPsf($run) end]
   if {[pgQinf STATE] == "CLOSED"} {plotInit}
   pgSci 1
   pgEnv 0 $xmax -5 5 0 0
   pgLabel "Field" "g, r and g-r seeing" "Run $run"
   set fieldStart [lindex $sscPsf($run) 0]
   foreach field $sscPsf($run) {

#Plot difference in g, r widths
	if {$sscPsf($run,$field,g) == 0 || $sscPsf($run,$field,r) == 0} \
	   continue
	set diff [expr $sscPsf($run,$field,g) - $sscPsf($run,$field,r)]
	pgSci 1
	pgPoint $field $diff 3
	pgSci 3
	pgPoint $field $sscPsf($run,$field,g) 2
	pgSci 2
	pgPoint $field -$sscPsf($run,$field,r) 2
	}
   return
   }

###########################################################################
proc focusLimits {run} {
   global photo_runs
   global rootDir
   global runTable
   global runFocus
   if {![info exists runTable]} {
	runInventory
	runMJD
	}
   set mjd $runTable($run,mjd)
   if {![info exists rootDir]} {
	echo Setting rootDir to /data/dp3.b/data
	set rootDir /data/dp3.b/data
	}
#echo mjd $mjd
   set file6 $rootDir/ppd/$mjd/idAstroline-r6-[format %06d $run].par
   set file9 $rootDir/ppd/$mjd/idAstroline-r9-[format %06d $run].par
   if {![file exists $file6] || ![file exists $file9]} {
	echo No focus files $file6 or $file9
	return
	}

#Read idAstroline files - these are more direct.  They give focus corrections
#directly, albeit with some delay.

   set chain6 [param2Chain $file6 AAA]
   set chain9 [param2Chain $file9 AAA]

#Leading astrometrics
   set frames ""
   chainEach hndl $chain6 {
	set frame [exprGet $hndl.frame]
	set focus($frame) [exprGet $hndl.bigFocusAdjustment]
	lappend frames $frame
	}
#Cache focus data
   set runFocus($run) $frames
   foreach frame $frames {
	set runFocus($run,$frame,6) $focus($frame)
	}

#Make guestimate of first and last good frames
   foreach frame $frames {
	if {$focus($frame) < -50 || $focus($frame) > 20} continue
	break
	}
   set first6 $frame

   foreach frame [lsort -integer -decreasing $frames] {
	if {$focus($frame) < -50 || $focus($frame) > 20} continue
	break
	}
   set last6 $frame

#Trailing astrometrics
   unset focus
   set frames ""
   chainEach hndl $chain9 {
	set frame [exprGet $hndl.frame]
	set focus($frame) [exprGet $hndl.bigFocusAdjustment]
	lappend frames $frame
	}
#Cache focus data
   set runFocus($run) $frames
   foreach frame $frames {
	set runFocus($run,$frame,9) $focus($frame)
	}

#Make guestimate of first and last good frames
   foreach frame $frames {
	if {$focus($frame) < -20 || $focus($frame) > 50} continue
	break
	}
   set first9 $frame
   foreach frame [lsort -integer -decreasing $frames] {
	if {$focus($frame) < -20 || $focus($frame) > 50} continue
	break
	}

   set last9 $frame

#echo first $first6 $first9
#echo last $last6 $last9
   set first [max $first6 $first9]
   set last [min $last6 $last9]

#Emperical correction for offsets.
   set first [expr $first-5]
   if {$first < [lindex $frames 0]} {set first [lindex $frames 0]}
   set last [expr $last-13]
   genericChainDel $chain6
   genericChainDel $chain9
   return [list $first $last]
   }

#######################################################################
proc focusPlot {run} {
   global runFocus
   if {![info exists runFocus]} {focusLimits}

   pgSci 1
   pgEnv 0 [lindex $runFocus($run) end] -60 60 0 0
   pgSci 7
   foreach frame $runFocus($run) {
       if {![info exists runFocus($run,$frame,6)]} continue
	pgPoint $frame $runFocus($run,$frame,6) 3
	}
   pgSci 5
   foreach frame $runFocus($run) {
       if {![info exists runFocus($run,$frame,9)]} continue
	pgPoint $frame $runFocus($run,$frame,9) 3
	}
   return
   }

#######################################################################
#Check focus logs in golden vs. ppd for all runs in opdb

proc goldenCheck {} {
   global runTable
   if {![info exists runTable]} {
	runInventory
	runMJD
	}
   foreach run $runTable() {
	set mjd $runTable($run,mjd)
	set pfile /data/dp3.b/data/ppd/$mjd/focus-$mjd.par
	set gfile /data/dp3.b/data/golden/$mjd/focus-$mjd.par
	if {![file exists $pfile]} {
	   echo Run $run $pfile not found
	   continue
	   }
	if {![file exists $gfile]} {
	   echo Run $run $gfile not found
	   continue
	   }
	set psize [file size $pfile]
	set gsize [file size $gfile]
	if {$psize != $gsize} {
	   echo File size mismatch run $run mjd $mjd
	   }
	}
   return
   }

#################################################################
#Procedure to compare frames run limits with those from focus and ssc.
proc runLimits {run} {
   global runTable

   if {![info exists runTable]} {
	runInventory
	runMJD
	}

#First, get ssc limits
   set list [fangCheckRun $run]
   set fieldStart [lindex $list 0]
   set fieldEnd [lindex $list 1]

#Get limits from astroline focus loop.

    set limits ""

#Focus check failed on run 1897!
#   set limits [focusLimits $run]

#Now look for outliers
   if {$limits != ""} {
	set fieldStart [max $fieldStart [lindex $limits 0]]
	set fieldEnd [min $fieldEnd [lindex $limits 1]]
	}

#Get first and last fields from actual pipeline run
   set first $runTable($run,field0)
   set last $runTable($run,fieldn)
   if {$first < $fieldStart} {
	echo Run $run recommend start field $first -> $fieldStart
	}
   if {$last > $fieldEnd} {
	echo Run $run recommend end field $last -> $fieldEnd
	}
   return
   }

###########################################################################
#Check all runs in opdb

proc fangCheck {} {
   global runTable

   if {![info exists runTable]} {
	runInventory
	runMJD
	}
   foreach run [lsort -integer $runTable()] {
	runLimits $run
	}
   return
   }

####################################################################
proc statusCheckRuns {} {
   global runTable skies
   global qdir

#Read cached runTable.
   source runTable.tcl
   if {![info exists runTable]} {
	runInventory
	runMJD
	}

#Read the cached data quality - hopefully don't crash the program.
   source $qdir/skies.tcl
   foreach run [lsort -integer $runTable()] {
	statusCheckRun $run
	}
   return
   }

#######################################################################
#Compare target chunks against fieldFix from statusCheckRuns

proc chunkCheck {} {
   global fieldFix
   if {![info exists fieldFix]} {statusCheckRuns}
   set files [lsort [glob -nocomplain odChunkStatus*.par]]
   set file [lindex $files end]
   set chunkChain [param2Chain $file hdr]
   chainEach hndl $chunkChain {
	foreach var "run camcol startField endField version status \
	   slongStart slongEnd startMu endMu" {
	   set $var [exprGet $hndl.$var]
	   }
	if {$version != 0} continue
	if {$status != "primary"} continue
	if {$camcol != 3} continue
	if {[lsearch $fieldFix() $run] < 0} {
	   echo Run $run not in fieldFix, continuing
	   continue
	   }
	set runField ""
	set skyField ""
	set fangField ""
	loop i $startField [expr $endField+1] {
	   foreach check "run sky fang" {
		if {[lsearch $fieldFix($run,$check) $i] >= 0} {
		   lappend ${check}Field $i
		   }
		}
	   }
	set startMu [format %.0f [expr $startMu/3600.]]
	set endMu [format %.0f [expr $endMu/3600.]]
	if {[llength $runField] > 0} {
	   echo Run $run "fields outside run.par limits:" \
		[listCompress $runField] start long $slongStart end long \
		   $slongEnd
	   }
	if {[llength $skyField] > 0} {
	   echo Run $run "fields outside sky     limits:" \
		[listCompress $skyField] start long $slongStart end long \
		   $slongEnd
	   }
	if {[llength $fangField] > 0} {
	   echo Run $run "fields outside fang    limits:" \
		[listCompress $fangField] start long $slongStart end long \
		   $slongEnd
	   }
	}
   genericChainDel $chunkChain
   return
   }

############################################################################
#Save list of field fixes to a file
#Run after statusCheckRuns

proc fieldFix {{file fieldFix.txt}} {
   global fieldFix
   echo Writing results to $file
   set fid [open $file w]
   foreach run [lsort -integer $fieldFix()] {
	if {$fieldFix($run,run) == "" && $fieldFix($run,sky) == "" && \
	   $fieldFix($run,fang) == ""} continue
	puts $fid "Run $run"
	set list ""
	foreach test "run fang sky" {
	   if {$fieldFix($run,$test) != ""} {
		puts $fid "   Fields outside $test limits: [listCompress \
		   $fieldFix($run,$test)]"
		foreach field $fieldFix($run,$test) {
		   if {[lsearch $list $field] < 0} {lappend list $field}
		   }
		}
	   }
	puts $fid "   Master list of fields to flag bad: [listCompress \
	   [lsort -integer $list]]"
	}
   close $fid
   }




