#Better routine to get stripe from eta.  Need different routines for N and S.

proc stripeNorthFromEta {eta} {
   set stripe [expr round((32.5+$eta)/2.5 + 10.)]
   return $stripe
   }

proc stripeSouthFromEta {eta} {
   set stripe [expr round((32.5+$eta)/2.5 + 82.)]
   return $stripe
   }

###################################################################
#Given RA, DEC, get all coordinates, including stripe, strip, and camcol

proc eqToAll {ra dec {verbose 1}} {
   global runChain
   set list [eqToSurvey $ra $dec]
   set lambda [keylget list lambda]
   set eta [keylget list eta]
   if {$verbose != "1"} {set verbose 0}
   if {$verbose} {
	echo survey eta = [format %.3f $eta] lambda = [format %.3f $lambda]
	}
   if {$lambda >= -90 && $lambda < 90} {
	set stripe [stripeNorthFromEta $eta]
   } else {
	set stripe [stripeSouthFromEta $eta]
	}
   set node 95.
   if {$stripe <= 45} {
	set incl [expr ($stripe-10)*2.5]
   } else {
	set incl [expr ($stripe-82)*2.5]
	}
   set list [eqToGC $ra $dec -node $node -inclination $incl]
   set mu [keylget list mu]
   set nu [keylget list nu]
   if {$verbose} {
	echo node $node incl $incl
	}
   if {$verbose} {
	echo mu [format %.3f $mu] nu [format %.3f $nu]
	}

#camcol and strip
   set icol [expr int($nu/.21868 + 7.)]
   if {int(($icol/2)*2) == $icol} {
	set strip N
	set camcol [expr ($icol)/2]
   } else {
	set strip S
	set camcol [expr ($icol+1)/2]
	}
   if {$verbose} {
	echo stripe $stripe strip $strip camcol $camcol
	}
   set returnlist ""
   keylset returnlist stripe $stripe
   keylset returnlist strip $strip
   keylset returnlist camcol $camcol
   keylset returnlist lambda $lambda
   keylset returnlist eta $eta
   keylset returnlist mu $mu
   keylset returnlist nu $nu
   keylset returnlist node $node
   keylset returnlist incl $incl

#Can we find which runs?
   set file /data/dp0.b/users/skent/mt/run.par
   if {![file exists $file]} {return $returnlist}
   if {![info exists runChain]} {
	set runChain [param2Chain $file hdr]
	}
   set subchain [chainSearch $runChain "{stripe = $stripe} \
	{strip<0> = $strip}"]
   chainEach hndl $subchain {
	foreach var "run muStart muEnd frameStart frameEnd" {
	   set $var [exprGet $hndl.$var]
	   }

#Need to deal with wraps
	set len [expr $muEnd - $muStart]
	if {$len > 0} {
	   if {$mu < $muStart || $mu > $muEnd} continue
	   set rate [expr ($frameEnd - $frameStart)/$len]
	   set field [expr $frameStart + $rate * ($mu - $muStart)]
	} else {
	   if {$mu < $muStart && $mu > $muEnd} continue
	   set len [expr $muEnd - $muStart + 360.]
	   set rate [expr ($frameEnd - $frameStart)/$len]
	   if {$mu >= $muStart} {
		set field [expr $frameStart + $rate * ($mu - $muStart)]
	   } else {
		set field [expr $frameStart + $rate * ($mu - $muStart + 360.)]
		}
	   }
	set field [expr round($field)]
	if {$verbose} {
	   echo "   " Run $run Camcol $camcol Field ~$field
	   }
	}
   chainDel $subchain
   return $returnlist
   }

###################################################################
#Given a range of ra, dec, determine all run/rerun/fields that contribute.
#I will not check for primary/secondary here.

proc boxField {ra1 ra2 dec1 dec2} {
   set returnlist ""
   set nstepra [expr int(($ra2-$ra1)/2.5) + 1]
   set nstepdec [expr int(($dec2-$dec1)) + 1]

#Construct list of runs on n & s strips
   set file /data/dp0.b/users/skent/mt/run.par
   if {![file exists $file]} {return $returnlist}
   if {![info exists runChain]} {
	set runChain [param2Chain $file hdr]
	}
   for {set i 0} {$i <= $nstepra} {incr i} {
	for {set j 0} {$j <= $nstepdec} {incr j} {
	   set ra [expr $ra1 + ($ra2-$ra1)*($i/double($nstepra))]
	   set dec [expr $dec1 + ($dec2-$dec1)*($j/double($nstepdec))]
	   set list [eqToAll $ra $dec]
	   set stripe [keylget list stripe]
	   set mu [keylget list mu]
	   foreach strip "N S" {
		set subchain [chainSearch $runChain "{stripe = $stripe} \
		{strip<0> = $strip}"]
		chainEach hndl $subchain {
		   set run [exprGet $hndl.run]
		   set quality [exprGet $hndl.quality]
		   if {$quality != "good"} continue
		   set field [astromFields $run $mu $mu]
		   if {$field == ""} continue
		   if {![info exists runField()]} {
			set runField() ""
			}
		   if {[lsearch $runField() $run] < 0} {
			lappend runField() $run
			set runField($run) ""
			}
		   eval lappend runField($run) $field
		   }
		chainDel $subchain
		}
	   }
	}

#Now expand return list
   if {[llength $runField()] == 0} return
   foreach run [lsort -integer $runField()] {
	if {[llength $runField($run)] > 1} {
	   set min [eval min $runField($run)]
	   set max [eval max $runField($run)]
	} else {
	   set min $runField($run)
	   set max $min
	   }
echo run $run min $min max $max
	for {set field $min} {$field <= $max} {incr field} {
	   lappend returnlist [list $run $field]
	   }
	}
   return $returnlist
   }

###################################################################
#Given a range of ra, dec, read tsObj files.

proc boxGet {ra1 ra2 dec1 dec2} {
   global tsdir
   set list [boxField $ra1 $ra2 $dec1 $dec2]

#Define a compressed data structure
   typedef struct {
	int run;
	int camcol;
	int rerun;
	int field;
	int id;
	double ra;
	double dec;
	double mu;
	double nu;
	float psfcounts<5>;
	int status;
	float ug;
	float gr;
	float ri;
	float iz;
	} TSOBJ;
   set outChain [chainNew TSOBJ]
   foreach pair $list {
	set run [lindex $pair 0]
	set field [lindex $pair 1]
	if {![info exists tsdir($run)]} {
	   tsDir $run
	   }
	set dataDir $tsdir($run)
	set rerun $tsdir($run,rerun)
	foreach camcol "1 2 3 4 5 6" {
	   set dir $dataDir/$camcol
	   set file $dir/tsObj-[format %06d $run]-$camcol-$rerun-[format \
		%04d $field].fit
	   set chain [fits2Schema $file TSOBJ]
	   chainJoin $outChain $chain
	   }
	}
   return $outChain
   }

###################################################################
#Given RA, DEC of telescope boresight, ccdrow and ccdcol, and strip, return
#approximate RA, DEC of CCD center.  Useful for checking coords
#reported by the observers.

proc ccdToEq {ra dec ccdrow ccdcol strip} {
   set list [eqToSurvey $ra $dec]
   set lambda [keylget list lambda]
   set eta [keylget list eta]
#   echo survey eta = [format %.3f $eta] lambda = [format %.3f $lambda]
   if {$lambda >= -90 && $lambda < 90} {
	set stripe [stripeNorthFromEta $eta]
   } else {
	set stripe [stripeSouthFromEta $eta]
	}
   echo stripe $stripe
   set node 95.
   if {$stripe <= 45} {
	set incl [expr ($stripe-10)*2.5]
   } else {
	set incl [expr ($stripe-82)*2.5]
	}
   set list [eqToGC $ra $dec -node $node -inclination $incl]
   set mu [keylget list mu]
   set nu [keylget list nu]
#   echo mu [format %.3f $mu] nu [format %.3f $nu]

#nu should be approximately 0.  Make it exactly so.
#   echo Making nu exactly 0.
   set nu 0.

#camcol and strip
   if {[string toupper $strip] == "S"} {
	set icol [expr $ccdcol*2 - 1]
   } else {
	set icol [expr $ccdcol*2]
	}
   set mu [expr $mu - ($ccdrow-3.)*.29942]
   set nu [expr ($icol-7.+.5)*.21868]
   set list [GCToEq $mu $nu -node $node -inclination $incl]
   set ra [keylget list ra]
   set dec [keylget list dec]
   echo ra [format %.3f $ra] dec [format %.3f $dec]
   echo ra [hhmmss [expr $ra/15.]] dec [ddmmss $dec]
   return
   }
