#Helper file in compiling quarterly report.
#Read latest plateList file, make a list of all plates and tileRuns in the
#specified quarter.  Construct sums.

proc targetInfo {year quarter} {
   if {$quarter == 1} {
	set months "jan feb mar"
	}
   if {$quarter == 2} {
	set months "apr may jun"
	}
   if {$quarter == 3} {
	set months "jul aug sep"
	}
   if {$quarter == 4} {
	set months "oct nov dec"
	}

   set nchar [string length $year]
   set i1 [expr $nchar-2]
   set i2 [expr $nchar-1]
   set year [string range $year $i1 $i2]

   set drillRuns ""
   foreach month $months {
	lappend drillRuns $month$year
	}

   set file [lindex [lsort [glob -nocomplain plateList*.par]] end]
   if {$file == ""} {
	echo No plateList file found
	return
	}
   set chain [param2Chain $file hdr]

   set plateRuns() ""

#Reach chain
   chainEach hndl $chain {
	foreach var "plateId tileRun plateRun" {
	   set $var [exprGet $hndl.$var]
	   }
	if {[lsearch $drillRuns $plateRun] < 0} continue
	if {[lsearch $plateRuns() $plateRun] < 0} {
	   lappend plateRuns() $plateRun
	   set plateRuns($plateRun) ""
	   }
	if {[lsearch $plateRuns($plateRun) $tileRun] < 0} {
	   lappend plateRuns($plateRun) $tileRun
	   set plateRuns($plateRun,$tileRun) ""
	   }
	lappend plateRuns($plateRun,$tileRun) $plateId
	}
   genericChainDel $chain

   set nchunk 0
   set nsurvey 0
   set nspecial 0
   set nspecialplate 0
   set surveyRun ""
   set specialRun ""

#Now construct lists
   foreach plateRun $plateRuns() {
	foreach tileRun $plateRuns($plateRun) {
	   set nplate($tileRun) [llength $plateRuns($plateRun,$tileRun)]
	   if {[regexp ^chunk $tileRun]} {
		incr nchunk
		set nsurvey [expr $nsurvey + $nplate($tileRun)]
		lappend surveyRun $tileRun
	   } else {
		incr nspecial
		set nspecialplate [expr $nspecialplate + $nplate($tileRun)]
		lappend specialRun $tileRun
		}
	   }
	}
   foreach tileRun $surveyRun {
	echo [format "%-15s %d" $tileRun $nplate($tileRun)]
	}
   foreach tileRun $specialRun {
	echo [format "%-15s %d" $tileRun $nplate($tileRun)]
	}
   set ndrill [llength $plateRuns()]
   set ntotal [expr $nsurvey + $nspecialplate]
   echo Number of drill runs: $ndrill
   echo Total plates drilled: $ntotal
   echo Survey chunks: $surveyRun
   echo "   " Number of survey chunks: $nchunk
   echo "   " Number of survey plates: $nsurvey
   echo Special chunks: [lsort $specialRun]
   echo "   " Number of special chunks: $nspecial
   echo "   " Number of special plates: $nspecialplate
   return
   }


