########################################################################
#Make a plot of all spectro data in hand, showing done and not done tiles.

########################################################################
#Make a plot of all data in hand, showing good, bad areas & overlaps.
#Use gifs.

proc tileStripePlotGif {} {

   set tileChain [param2Chain tile.par hdr]
   set plateChain [param2Chain plate.par hdr]
   set file [lindex [find tileReport*.par] end]
   set specChain [param2Chain $file hdr]
   set chunkChain [param2Chain tileChunk.par hdr]

#Decode chains
   set tile() ""
   chainEach hndl $tileChain {
	set tileId [exprGet $hndl.tileId]

#Skip southern plates.
	set chunkId [exprGet $hndl.chunkId]
	if {$chunkId == 22} continue
	if {$chunkId < 4} continue
	if {$tileId >= 9000} continue
	foreach var "chunkId raCen decCen" {
	   set tile($tileId,$var) [exprGet $hndl.$var]
	   }
	lappend tile() $tileId
	set tile($tileId,plateId) ""
	set tile($tileId,status) NOT_DONE
	}

   set plate() ""
   chainEach hndl $plateChain {
	set plateId [exprGet $hndl.plateId]
	set tileId [exprGet $hndl.tileId]
	if {$plateId < 266 || $tileId >= 9000} continue
	set plate($plateId,tileId) $tileId
	if {![info exists tile($tileId,plateId)] || $tile($tileId,plateId) \
		== ""} {
	   set tile($tileId,plateId) $plateId
	} else {
	   append tile($tileId,plateId) /$plateId
	   }
	}

#tileChunks
   set tileChunk() ""
   chainEach hndl $chunkChain {
	foreach var "chunkId stripe lmin len" {
	   set $var [exprGet $hndl.$var]
	   }
	if {![info exists tileChunk($stripe)]} {
	   set tileChunk($stripe) $chunkId
	} else {
	   lappend tileChunk($stripe) $chunkId
	   }
	set tileChunk($chunkId,$stripe,lmin) $lmin
	set tileChunk($chunkId,$stripe,len) $len
	}

#Make a web page
   set fid [open tileStatus.html w]
   puts $fid "<html>"
   puts $fid "<head>"
   puts $fid "<title>Tile Status</title>"
   puts $fid "</head>"
   puts $fid ""
   puts $fid "<body bgcolor=white>"
   puts $fid "<h1 align=center>Tile Status on [fmtclock [getclock]]</h1>\n"
   puts $fid "<table align=center border=0>\n"
   puts $fid "<tr><td width=30 height=5 bgcolor=green>&nbsp;</td><td> \
	Tile observed</td></tr>\n"
   puts $fid "<tr><td width=30 height=5 bgcolor=red>&nbsp;</td><td> \
	Tile not observed</td></tr>\n"
   puts $fid "<tr><td width=30 height=5 bgcolor=magenta>&nbsp;</td><td> \
	Tile with marginal data</td></tr>\n"
   puts $fid "</table>\n"

#Min and max of plots
   set amin -90.
   set amax 70.

#Max width of bars in pixels
   set width 800.
   set height 5
   set stripes ""
   loop i 45 0 -1 {lappend stripes $i}
   lappend stripes 76 82 86
   foreach stripe $stripes {
	set chunk($stripe,obs) ""
	set chunk($stripe,tile) ""
	}

#Assign all designed tiles to a stripe
   foreach tileId $tile() {
	set list [eqToSurvey $tile($tileId,raCen) $tile($tileId,decCen)]
	set lambda [keylget list lambda]
	set eta [keylget list eta]
	set stripe [expr round(($eta+32.5)/2.5) + 10]
	if {$lambda > 90 || $lambda < -90} {
	   set stripe [expr $stripe+72]
	   }
	lappend chunk($stripe,tile) $tileId
	}

#Assign all observed tiles to a stripe
   chainEach hndl $specChain {
	set tileId [exprGet $hndl.tileId]
	if {![info exists tile($tileId,raCen)]} continue
	set tileRun [exprGet $hndl.tileRun]
	set status [exprGet $hndl.status]

#Keep only main survey plates.
	if {![regexp chunk $tileRun]} continue
	set list [eqToSurvey $tile($tileId,raCen) $tile($tileId,decCen)]
	set lambda [keylget list lambda]
	set eta [keylget list eta]
	set stripe [expr round(($eta+32.5)/2.5) + 10]
	if {$lambda > 90 || $lambda < -90} {
	   set stripe [expr $stripe+72]
	   }
	lappend chunk($stripe,obs) $tileId
	set tile($tileId,status) $status
	}

   foreach stripe $stripes {
	set list [stripeLimits $stripe]
	set lmin [keylget list lambdaMin]
	set lmax [keylget list lambdaMax]

	if {$stripe > 45} {
	   set lmin [expr $lmin-180]
	   set lmax [expr $lmax+180]
	   }
	if {[llength $chunk($stripe,obs)] == 0} continue
	echo $stripe
	set pfile $stripe.gif
	pgBegin $pfile/GIF
	pgGeomSet 800 200 0 0
	pgSch 2
	pgScr 0 1 1 1
	pgScr 1 0 0 0
	pgScr 7 .7 .7 0
	set y 0
	pgSci 1
	pgSlw 1
	pgSch 2
	pgEnv $amin $amax -14 3 1 0
	pgLabel "Survey Longitude" "" "$stripe"
	puts $fid "<p><hr><p>\n"
	puts $fid "<li>$stripe<br>"
	puts $fid "<img src=\"$stripe.gif\"><br>\n"
	set start [max $lmin $amin]
	set end [min $lmax $amax]
	set len [expr round(($end - $start) \
	   *$width/($amax - $amin))]
	pgSlw 4
	pgSci 4
	pgLine "$start $end" "$y $y"
	pgSci 1
	pgSlw 1
	pgSch 2
	pgText [expr $amin+2] [expr $y-.1] "Limit"

	set lambdas ""
	if {[info exists array]} {unset array}
	foreach tileId $chunk($stripe,tile) {
	   set list [eqToSurvey $tile($tileId,raCen) $tile($tileId,decCen)]
	   set lambda [keylget list lambda]

#Southern stripes.
	   if {$lambda > 90.} {set lambda [expr $lambda-180.]}
	   if {$lambda < -90.} {set lambda [expr $lambda+180.]}
	   pgSfs 1
	   pgSci 2
	   pgCirc $lambda 0. 1.25
	   set array($lambda) $tileId
	   lappend lambdas $lambda
	   }

	foreach tileId $chunk($stripe,obs) {
	   set list [eqToSurvey $tile($tileId,raCen) $tile($tileId,decCen)]
	   set lambda [keylget list lambda]

#Southern stripes.
	   if {$lambda > 90.} {set lambda [expr $lambda-180.]}
	   if {$lambda < -90.} {set lambda [expr $lambda+180.]}
	   pgSfs 1
	   if {$tile($tileId,status) == "DONE"} {
		pgSci 3
	   } else {
		pgSci 6
		}
	   pgCirc $lambda 0. 1.25
	   }

	set y 0
	if {[info exists tileChunk($stripe)]} {
	   foreach chunkId $tileChunk($stripe) {
		if {$chunkId < 4 || $chunkId == 22} continue
		set start [max $tileChunk($chunkId,$stripe,lmin) $amin]
		set lmax [expr $start+$tileChunk($chunkId,$stripe,len)]
		set end [min $lmax $amax]
		set len [expr round(($end - $start) \
		   *$width/($amax - $amin))]
		pgSlw 4
		pgSci 6
		set y [expr $y-3]
		pgLine "$start $end" "$y $y"
		pgSci 1
		pgSlw 1
		pgText [expr $amin+2] [expr $y-.1] "tileChunk $chunkId"
		}
	   }
	pgEnd
	set line ""
	foreach lambda [lsort -real $lambdas] {
	   set tileId $array($lambda)
	   if {![info exists tile($tileId,plateId)]} continue
	   set plateId $tile($tileId,plateId)
	   set color red
	   if {[lsearch $chunk($stripe,obs) $tileId] >= 0} {
		if {$tile($tileId,status) == "DONE"} {
		   set color green 
		} else {
		   set color magenta
		   }
		}
	   append line "<font color=$color>$plateId</font> "
	   }
	puts $fid $line
	puts $fid "<br>"
	}
   puts $fid "</body>"
   puts $fid "</html>"
   close $fid
   return
   }



