#Compute a vector profile.

proc vector {reg nwid nstep} {
   imgGlobals
   if {![info exists img(axes)]} {error "Need some axes"}
   if {[info exists img(profile)]} {
	catch {genericChainDel $img(profile)}
	}
   set rowc [exprGet $img(axes).row]
   set colc [exprGet $img(axes).col]
   set ang [exprGet $img(axes).pos_maj]
   set hndl [imgVector $reg $rowc $colc $ang $nwid $nstep]
   set img(profile) $hndl
   set img(proftype) vector
   return $hndl
   }

alias v vector

#Graph a vector profile on the terminal screen

proc gr {option} {
   imgGlobals
   if {[string tolower $option] == "v"} {
	if {![info exists img(profile)]} return
	set cmin 1.e20; set cmax -1.e20
	chainEach hndl $img(profile) {
	   set cnts [exprGet $hndl.cnts]
	   set cmin [min $cnts $cmin]
	   set cmax [max $cnts $cmax]
	   }
	if {$cmin == $cmax} return
	if {[info exists img(sky)] && $img(sky) > $cmin} {set cmin $img(sky)}
	if {$cmin == $cmax} return

#Set up grid
	loop i 0 21 {
	   set grid($i) ""
	   loop j 0 71 {
		lappend grid($i) " "
		}
	   }
	loop i 0 21 5 {
	   set grid($i) ""
	   loop j 0 71 {
		lappend grid($i) -
		}
	   }
	loop i 0 21 {
	   loop j 0 71 10 {
		set grid($i) [lreplace $grid($i) $j $j I]
		}
	   }
	chainEach hndl $img(profile) {
	   set cnts [exprGet $hndl.cnts]
	   set rad [exprGet $hndl.rad]
	   set x [expr $rad+35]
	   if {$x < 0 || $x > 70} continue
	   set y [expr 20 - int(20*($cnts-$cmin)/($cmax-$cmin))]
	   if {$y < 0} {set y 0}
	   if {$y > 20} {set y 20}
	   set grid($y) [lreplace $grid($y) $x $x *]
	   if {$img(proftype) == "annulus"} {
		set x [expr 70-$x]
		set grid($y) [lreplace $grid($y) $x $x *]
		}
	   }
	loop i 0 21 {
	   set line [join $grid($i) ""]
	   set label ""
	   if {$i == 0} {set label [format %7.1f $cmax]} 
	   if {$i == 20} {set label [format %7.1f $cmin]} 
	   puts stdout [format %-8s%s $label $line]
	   }
	}
   return
   }

proc vmark {{chan ""}} {
   imgGlobals
   if {![info exists img(profile)]} return
   if {$chan != ""} {
	set ichan $saoNumber($chan)
	set opt " -s $ichan"
   } else {
	set ichan ""
	set opt ""
	}
   eval saoReset $ichan
   eval saoLabel off $ichan
   chainEach hndl $img(profile) {
	set row [exprGet $hndl.row]
	set col [exprGet $hndl.col]
	eval saoDrawPoint $row $col $opt
	}
   eval saoLabel off $ichan
   return
   }
