/* Function prototypes in img module */

#ifndef IMG_PROTO
#define IMG_PROTO

#include <stdio.h>
#include "dervish.h"	/* Dervish file */
#include "img_msg_c.h"
#include "imgAxes.h"
#include "imgProf.h"
#include "imgProfile.h"

/* Define PI if not already defined */

#ifndef PI
#define PI 3.14159265358979
#endif

double drand();

double ran1(int *seed);

void reseed(int seed);

double gaussRan(int n);

double pixbin (REGION *reg, double row, double col);

double xpixbin (REGION *reg, double row, double col);

void *regPixIntFunc (REGION *reg);

void *regPixDblFunc (REGION *reg);

void (*regPixIntFuncSet (REGION *reg)) (REGION *, int, int, int);

void (*regPixDblFuncSet (REGION *reg)) (REGION *, int, int, double);

int pxSize (REGION *reg);

int pxType (REGION *reg);

typedef signed short IMDATA;

IMG_CODE imgFbs(REGION *reg, int pixthresh, int pixmax, CHAIN *list);

IMG_CODE imgMbs(REGION *reg, double rowc, double colc, double sig, double count,
   double gain);

IMG_CODE imgNoise(REGION *reg, double rms);

IMG_CODE imgPhoton(REGION *reg, double gain);

IMG_CODE imgRegFindObj(REGION *reg, CHAIN    *objectlist,
   double sky, double sky_noise, double radius, double sig1, double sig2,
   double sig4, double photon, double readout);

double pixbin (REGION *reg, double row, double col);

IMG_CODE imgAnn (REGION *reg, double row, double col, int rmax,
	CHAIN *proflist);

IMG_CODE imgVec (REGION *reg, double row, double col, double ang,
	int nwidth, int nstep, CHAIN *proflist);

typedef unsigned short UDATA;

IMG_CODE imgSqrt (UDATA **rows, int nr, int nc, double rnoise, double gain,
	double quant);

IMG_CODE imgSqr (UDATA **rows, int nr, int nc, double rnoise, double gain,
	double quant);

IMG_CODE imgSim (UDATA **rows, int nr, int nc, double rnoise, double gain,
	double sky);

IMG_CODE imgAxes (REGION *reg, int row0, int col0, int npr, int npc,
        AXES *axes);

IMG_CODE imgCaxes (REGION *reg, int row0, int col0, int npr, int npc,
        AXES *axes);

IMG_CODE imgSmooth (REGION *reg);

IMG_CODE imgBleedFix (REGION *reg, int thresh);

double pxGetAsDbl (const REGION *reg, int r, int c);

void pxSetWithDbl (const REGION *reg, int r, int c, double dbl);

int syminv(double *a, double *b, int dimension, double *pvrow, double *pvcol,
        int *use);

IMG_CODE imgRegProfile (REGION *reg, CHAIN *profilelist, AXES *cetnroid,
        int start_pixel, int end_pixel, int nderiv, double sky);

IMG_CODE imgFuncFit (CHAIN *chain, double *amp, double *fwhm, double *fract);

double imgFunc (CHAIN *chain, double *amp, double *fwhm, int *index);

IMG_CODE imgStats (REGION *reg, int row1, int nrow, int col1,
   int ncol,double *mean, double *stdev, int *minval,
   int *rowmin, int *colmin, int *maxval, int *rowmax,
   int *colmax);

IMG_CODE imgHist (REGION *reg, int row1, int col1, int row2,
   int col2, int pixmin, int pixmax, int skip, int *pixval,
   int *bin);

IMG_CODE imgRowMed (REGION *regIn, REGION *regOut);

IMG_CODE imgColMed (REGION *regIn, REGION *regOut);

IMG_CODE imgRowAvg (REGION *regIn, REGION *regOut);

IMG_CODE imgColAvg (REGION *regin, REGION *regout);

IMG_CODE imgRowSub (REGION *reg, REGION *regRow, int dither);

IMG_CODE imgColSub (REGION *reg, REGION *regCol, int dither);

IMG_CODE imgConAdd (REGION *reg, double value, int dither);

IMG_CODE imgCsmooth (REGION *reg, int dither);

IMG_CODE imgDecimate (REGION *regin, REGION *regout, int factor);

IMG_CODE imgCompress (REGION *regin, REGION *regout, int factor);

IMG_CODE imgCircle (REGION *reg, double rowc, double colc, double rad,
   double *counts, double *area);

IMG_CODE imgGauss (REGION *reg, double rowc, double colc, double ang,
   double FWHM, double rnoise, double gain, int xmax, int ymax,
   double *amp, double *erramp, double *offset, double *erroff, double *chi2);

IMG_CODE imgBoxPhoto (REGION *reg, double rowc, double colc, double ang,
   int xmax, int ymax, double *counts, double *area);

int polyfit (double *x, double *y, int nelem, int ndim, double *coeff);
int polycomp (double *x, double *y, int nelem, int ndim, double *coeff,
	      double *res);
int polyrms (double *res, int n, double *rms);
int polyeval (double *val, double x, double *coeff, int ndim);

#endif
