#ifndef SHTRANSCONTRIB_H
#define SHTRANSCONTRIB_H

#include "shCProgram.h"

/* typedef for the function that copies from one data element to another */

typedef void COPYPROGRAM(void *ptr1, void *ptr2);

/* New structures to support translation between FITS tables and CHAINs */

/* First, one element */


typedef struct {
   char *fitsEntry;
   TYPE fitsType;		/* Optional type to override default from
				   schema */
   char *schemaEntry;
   PROGRAM *fitsProgram;	/* Info to get FITS side data pointer */
   PROGRAM *schemaProgram;	/* Info to get schema side data pointer */

/* An auxiliary index into a TBLCOL to get the matching field */
   int ifld;

/* Names, programs, and copy functions for heaplen variables on schema side */
   char *heaplen;
   PROGRAM *heaplenProgram;
   COPYPROGRAM *heaplenCopy;

/* Names, programs, and copy functions for dimen variables on schema side.
 * These variables are used to define the lengths of out-of-line arrays
*/
   char *dimen;
   PROGRAM **dimenProgram;
   COPYPROGRAM **dimenCopy;

/* Function to copy & convert one element of FITS data */
   COPYPROGRAM *fits2Schema;

/* Function to copy & convert one element of schema data */
   COPYPROGRAM *schema2Fits;

   } TRANSENTRY;		/* pragma SCHEMA */

/* Now, the table.  Just use it as a container for a chain */

typedef struct {
   CHAIN *table;
   char *schemaTypeName;	/* Schema side type name */
   char *constructor;		/* Auxiliary contructor proc (optional) */
   char *destructor;		/* Auxiliary destructor proc (optional) */
   int nEntry;		/* Chain size (may be unnecessary) */
   TRANSENTRY **entry;	/* Vectorized version of table for faster access */
   } TRANSTABLE;	/* pragma SCHEMA */

/* Prototype of data copy function */

COPYPROGRAM *p_shCopyFuncGet (TYPE type1, TYPE type2);

#endif
