/* Conversion from Fortran to C */
/* LZW encode an array */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "gif.h"

/****************************************************************************/
/* Procedure to determine if we need to swap.  Function returns 0 for little endian
* and 1 for big endian machines
* Set an unsigned int to 1, then look at "first" byte in an address sense.
*/

static int ifswap (void) {
   unsigned short test = 1;
   if (((unsigned char *)(&test))[0] == 1) return 0; else return 1;
   }

static int swap;

/*******************************************************************************/
/* Add code to output buffer */
static void putbits (unsigned char *buf, int *ibit, int nmax, unsigned int code,
   int nbit) {

	unsigned int ival4;
	int ibyte, ishift;

/* ibit is last bit to be entered into buf. ibit = 8 when we have written 1 full
 * byte */

/* nextbit is last bit once we enter the new code. */
	int nextbit = *ibit + nbit;
/* check to see if we overflow the output array */
	int lastbyte = (nextbit-1)/8;

	if (lastbyte >= nmax-2) {	/* caution: code below writes 3 bytes. */
	   fprintf (stderr, "Compressed image is bigger than raw image.\n");
	   exit(1);
	   }

/* ibyte is the next byte to receive the first bit of code */
	ibyte = *ibit/8;
	ishift = pow(2, fmod(*ibit, 8))+0.5;
	ival4=code*ishift + buf[ibyte]; /* shouldn't have overflow problems here.*/

/* I could rewrite the following to get rid of byte swap testing if I were to
 * do appropriate shifts and masking of code */
/* byte swapping here for big endian! */
	if (swap) {
	   buf[ibyte] = ((unsigned char *)&ival4)[3];
	   buf[ibyte+1] = ((unsigned char *)&ival4)[2];
	   buf[ibyte+2] = ((unsigned char *)&ival4)[1];
/* little endian! */
	} else {
	   buf[ibyte] = ((unsigned char *)&ival4)[0];
	   buf[ibyte+1] = ((unsigned char *)&ival4)[1];
	   buf[ibyte+2] = ((unsigned char *)&ival4)[2];
	   }
	*ibit = nextbit;
	return;
	}

/*********************** encode **********************************************/

void encode(unsigned char *img, int nwidth, int nheight, unsigned char *buf,
   int *n, int nmax, int icode) {

/* ENCODE LZW FORMAT DATA.
* ENCODING TABLE IS BUILT ON THE FLY.
* THE TABLE HAS THREE PARTS TO EACH ENTRY: THE CODE FOR THAT ENTRY,
* AN INDEX TO A PREVIOUS ENTRY THAT IS FURTHER DECODED TO YIELD THE
* FIRST PART OF THE STRING OF THE CURRENT ENTRY, AND A SINGLE CHARACTER
* THAT IS THE LAST PART OF THE STRING OF THE CURRENT ENTRY.
* TABLE HAS A MAX OF 4095 ENTRIES.  IN PRINCIPLE, THE LAST CODE CAN BE
* PARSED TO A STRING 4095 CHARACTERS LONG.  HENCE IT IS NOT PRACTICAL
* TO MAINTAIN THE FULL STRING OF EACH ENTRY IN CORE AT ONCE.
* IN PRACTICE, THE CODE NUMBER OF A PARTICULAR ENTRY WILL BE THE ARRAY
* INDEX, SO NO NEED TO STORE IT SEPARATELY.
*
* NOTE: SUBROUTINE DECODE USED BY UNGIF REVERSES WHAT IS DONE HERE.  IT
* ALSO DOES A COUPLE OF THINGS NOT DONE HERE (SO THE TWO CANNOT BE CALLED
* IN IMMEDIATE SUCCESSION FOR TESTING): DECODE EXTRACT THE PARAMETER ICODE
* FROM ITS INPUT STREAM (ICODE IS PASSED AS A DUMMY ARGUMENT HERE).  ALSO,
* DECODE PERFORMS UNBLOCKING OF DATA; ENCODE DOES NOT BLOCK DATA (THAT IS
* DONE LATER BY THE MAIN GIF ROUTINE).
*/

/* BUF MAY ALREADY PARTLY FILLED IN UP TO INDEX N */

/* ALTERNATE STRING TABLE: WE USE W AND K AS INDICES IN A TABLE THAT POINT
* TO THE NEXT W.
*/
	int wtable[4097][256];
	int *ptr;

	int i;
	int j;
	unsigned char k;
/*
* ICODE IS WRITTEN INTO OUTPUT BUFFER IN MAIN ROUTINE.
* MORE INITIALIZATION.
*/
	int ibit = (*n)*8;	/* INDEX OF LAST ENCODED BIT IN THE BUFFER */
	int npix = 0;		/* HOW MANY PIXELS HAVE WE ENCODED SO FAR. */
				/* INDEX IN IMG */

/* MASK FOR GETTING RID OF UNWANTED HIGH ORDER BITS IN INPUT ARRAY */
	unsigned char mask;
	int w = -1;
	unsigned int code;
	int ncode;
	int nbit;
	int nlevel = pow(2,icode);
	int lastflag = 0;

/***************************************************************************/

/* Set the swap flag */
	swap = ifswap();
/* fprintf (stderr, "Begin, dimesions are %d %d npix = \n", nwidth, nheight, npix);
 */
/* Mask */
	mask = nlevel - 1;

/* RESET THE BIG TABLE */
	for (j=0; j < nlevel; j++) {
	   for (i=-1; i<4096; i++) {
		wtable[i+1][j] = -1;
		}
	   }
/* INITIALIZE TABLE.  THIS CAN BE FORCED AT ANY POINT IN THE DATA STREAM
* FIRST VALIDE CODE IS 2**ICODE+2.  INITIALLY, CODES ARE ICODE+1=NBIT
* BITS IN LENGTH.  ICODE=IDEPTH NORMALLY EXCEPT THAT ICODE=2 FOR IDEPTH=1.
* THIS INCREASES DYNAMICALLY AS MORE CODES ARE ADDED
* TO THE TABLE.
*/

	for (i=0; i<nlevel; i++) {
	   wtable[-1+1][i] = i;
	   }

/* NCODE=INDEX OF LAST CODE THAT IS TAKEN. */
	ncode = nlevel + 1;
	for (i=1; i<=11; i++) {
	   if (ncode >= pow(2,i)) nbit = i+1;
	   }

/* THE FIRST THING WE WRITE IS A TABLE RESET CODE */
	code = nlevel;
	putbits (buf, &ibit, nmax, code, nbit);

/* UPDATE N EVERYTIME WE WRITE TO THE BUFFER FOR SAFETY */
/* Bug?  I used to have n = ibit/8+1, but this gives the index of the next byte
 * to receive data, not the index of the last byte with data.  ibit is the total
 * number of bits already written.
*/
	*n = (ibit-1)/8+1;

s20:

/* BEGINNING A NEW DECODING SEQUENCE.  AS WE PROGRESS THROUGH THE STRING
* TABLE, THE NEXT POSSIBLE ENTRY WILL ALWAYS BE BELOW THE CURRENT ONE.
* HERE, INITIALIZE THE CURRENT POSITION TO THE START OF THE TABLE
*/

s30:

/* Lastflag = 0 means that the last pixel is part of a compound code */
	lastflag = 0;
/* DECODE NEXT PIXEL */
/* Fiddle with npix to make index 0 based */
	if (npix >= nwidth * nheight) goto s50;
	k = img[npix];
	npix = npix + 1;

/* MASK OFF UNUSED BITS OF IMG, JUST TO BE SAFE */
	k = k & mask;
/* SEE IF WE CAN FIND A COMPOUND CODE IN THE STRING TABLE. */
	if (wtable[w+1][(int)k] >= 0) {
		w = wtable[w+1][(int)k];
		goto s30;
		}
	lastflag = 1;
/* COULDN'T FIND AN EXISTING CODE.  OUTPUT W AS CURRENT CODE AND UPDATE
* THE STRING TABLE
*/
s50:
	code = w;
	putbits(buf, &ibit, nmax, code, nbit);

/* UPDATE N EVERYTIME WE WRITE TO THE BUFFER FOR SAFETY */
	*n = (ibit-1)/8 + 1;

/* HAVE WE FINISHED YET? */
	if (npix >= nwidth * nheight) goto s100;

/* RESET TABLE IF NCODE IS GETTING TOO BIG */
	if (ncode == 4094) {
/* fprintf (stderr, "Resetting table, done with %f\n", 
        (double)npix/(nwidth*nheight)); */
		code = nlevel;
		putbits (buf, &ibit, nmax, code, nbit);

/* UPDATE N EVERYTIME WE WRITE TO THE BUFFER FOR SAFETY */
		*n = (ibit-1)/8 + 1;

/* NCODE=INDEX OF LAST CODE THAT IS TAKEN. */
		ncode = nlevel + 1;

/* HAVE TO RESET THE BIG TABLE AGAIN. */
		ptr = &(wtable[0][0]);
		for (j=0; j<nlevel*4097; j++) *ptr++ = -1;
/*
		for (j=0; j < nlevel; j++) {
		   for (i=-1; i<4096; i++) {
			wtable[i+1][j] = -1;
			}
		   }
 */
/* DON'T HAVE TO RESET ENTRIES WITH W=-1 BECAUSE WE SHOULD NEVER GET W=-1
* AGAIN
*/
	} else {

/* ENTER NEW STRING INTO CODE TABLE. */
		ncode = ncode+1;
		wtable[w+1][(int)k] = ncode;
		}
/*  RESET W */
	w = k;

/* SEE IF NBIT NEEDS ADJUSTING. */
	for (i=1; i<=11; i++) {
	   if (ncode >= pow(2,i)) nbit = i+1;
	   }
	goto s20;

s100:
/* fprintf (stderr, "Done: npix = %d\n", npix); */
/* DONE */

/* If lastflag = 1, the last byte is still unwritten */
	if (lastflag == 1) {
	   code = k;
	   putbits(buf, &ibit, nmax, code, nbit);
	   }

	code = nlevel + 1;
	putbits(buf, &ibit, nmax, code, nbit);

/* UPDATE N EVERYTIME WE WRITE TO THE BUFFER FOR SAFETY */
	*n = (ibit-1)/8 + 1;
/* fprintf (stderr, "Size of output buffer = %d\n", *n); */
	return;
	}


