/*****************************************************************************
******************************************************************************
**
** FILE:
**	fileCopy.c
**
** ABSTRACT:
**	This file contains code to copy from one file handle to another.
**
** ENTRY POINT		SCOPE	DESCRIPTION
** -------------------------------------------------------------------------
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	FTCL	 	TCL + Fermi extension
**      SHIVA
**	IMAGE
**
** AUTHORS:
**	Creation date:  Nov. 1994
**	Steve Kent
**
******************************************************************************
******************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <stdio.h>
#include "tcl.h"


/*============================================================================  
**============================================================================
**
** ROUTINE: tclReadTest
**
** DESCRIPTION:
**	Read 2 file handles, translate to descriptors, then copy until
**	input returns eof
**
** RETURN VALUES:
**	TCL_OK -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**
**============================================================================
*/

static char *tclFileTest_use =
  "USAGE: fileCopy <in-fileHndl> <out-fileHndl>";
static char *tclFileTest_hlp =
  "Copy from one file handle to another until eof on 1st.";


static int
tclFileCopy(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   int fileId1, fileId2;
   FILE *filePtr1, *filePtr2;
   int nread;
#define NBYTE 62000
   char buffer[62000];

   if(argc != 3) {
      Tcl_SetResult(interp,tclFileTest_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get file handle pointers. */
    if (Tcl_GetOpenFile(interp, argv[1], 0, 0, &filePtr1) != TCL_OK) {
        return TCL_ERROR;
	}
    if (Tcl_GetOpenFile(interp, argv[2], 0, 0, &filePtr2) != TCL_OK) {
        return TCL_ERROR;
	}
   fileId1 = fileno(filePtr1);
   fileId2 = fileno(filePtr2);
   while (!feof(filePtr1)) {
	nread = read (fileId1, buffer, NBYTE);
	if (nread == -1) break;
	if (nread == 0) break;
	write (fileId2, buffer, nread);
	}
   return TCL_OK;
   }

/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
fileCopyDeclare(Tcl_Interp *interp) 
{
   Tcl_CreateCommand (interp, "fileCopy",
		(Tcl_CmdProc *)tclFileCopy,
		(ClientData) 0, 0);

   return;
}

