/* encode a gif file. */
/* NOTE: We really do 6 bits! */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "gif.h"

/* Gif files use little-endian short ints. */

/* Return 1 if big-endian, 0 if little-endian */

/*************************************************************************/

static int ifswap(void) {
   short test=1;
   if (*(char *)&test == '\0') return 1;
   return 0;
   }

/*************************************************************************/
static short swap2(short in);

static short swap2(short in) {
   short temp;
   if (ifswap()) {
	((char *)&temp)[0] = ((char *)&in)[1];
	((char *)&temp)[1] = ((char *)&in)[0];
	return temp;
   } else {
	return in;
	}
   }

/*************************************************************************/
#define min(x,y) (x)<(y)? (x) : (y)

/*************************************************************************/
static void blockout (unsigned char *data, int ndata, FILE *file) {
	unsigned char buf[256];
	int ival4;
	int imax;
	int i,j;
/* write out data in blocks of 255 bytes. */
	for (i=0; i<ndata; i+=255) {
	   imax=min(i+255,ndata);
	   ival4=imax-i;
	   buf[0]=ival4;
	   for (j=1; j<256; j++) buf[j]=0;
	   for (j=1; j<=ival4; j++) {
		buf[j]=data[i+j-1];
		}
	   buf[0]=255;
	   fwrite (buf, 1, 256, file);
	   }
/* write out last block of 0 bytes */
	buf[0]=0;
	fwrite (buf, 1, 1, file);
	}

/****************************** gif *************************************/

void gif (unsigned char *img, int nwidth, int nheight, char *outfile,
   int nred, int ngreen, int nblue) {
	unsigned char red[256], green[256], blue[256];
	unsigned char mask;
	short buf2;
	int i, j, k, index;
/* create a temporary buffer that will hold the compressed image as
   it is created.  allocate space for 
   one image worth of bytes.  if compression is inefficient and
   actually results in expansion, the encode process will abort.
   i could equally well allocate this space in the output file, but then i
   would need to trim the file size when done, and i am not sure how
   to do that.
*/
	unsigned char *tempaddr;
/* n is an index to the most recently filled slot in the temporary buffer.
   on return, it has the number of bytes in the buffer.
*/
	int n=0;
/* encoding uses lzw algorithm (welch, ieee computer,
   vol 17, no 6, 1984).  do this in a separate subroutine.
   first, keep track of the last byte of ublocked data.  this will be needed
   for the reblocking to be done for the final file.
   that is pointless for disk files, but i didn't make up the rules.
   first, number of bits per pixel.
*/

/* If depth is 1, we would still need icode to be 2, according to GIF specs */
	int depth;
	int icode;
	int ncolor;
	int ndata;
	FILE *file;
	int ntot=nwidth*nheight;

/* Number of levels is product of red, green blue.  depth is next power of
 * 2 bigger. */
	int nlevel = nred * ngreen * nblue;

	depth = 1;
	if (nlevel > 2) depth = 2;
	if (nlevel > 4) depth = 3;
	if (nlevel > 8) depth = 4;
	if (nlevel > 16) depth = 5;
	if (nlevel > 32) depth = 6;
	if (nlevel > 64) depth = 7;
	if (nlevel > 128) depth = 8;
	icode = depth;
	ncolor = pow(2, depth);

/* Icode is a minumum of 2 */
	if (icode == 1) icode = 2;
	tempaddr = (unsigned char *)malloc(ntot);

/* Zero out temporary memory */
	for (i=0; i<ntot; i++) {
	   tempaddr[i]='\0';
	   }
/* fprintf (stderr,"begin encoding\n"); */
	encode (img, nwidth, nheight, tempaddr, &n, ntot,
		icode);

/* write the output buffer to a file
   estimate file size
   first, the number of actual data bytes is ...
*/
	ndata=n;
/*	outsize=6+7+3*ncolor+10+1+(ndata)*(1+1/254.)+3+256; */

/* final 3 bytes: 1 is for truncations error of previous calculation
                1 is for the last block of 0 bytes
                1 is for the termination character.
*/
	file = fopen (outfile, "w");

/* write out the preamble
   fill in the signature
*/
	fwrite ("GIF87a", 6, 1, file);

/* create the screen descriptor */
	buf2=swap2((short)nwidth);
	fwrite (&buf2,2,1,file);
	buf2=swap2((short)nheight);
	fwrite (&buf2,2,1,file);

/* create compound byte */
	mask=0xf0 + (depth-1);	/* this says color table follows, depth bppix */
	fwrite (&mask,1,1,file);

/* background intensity.  set to 0 */
	mask=0;
	fwrite (&mask,1,1,file);

/* and a byte of zeros to end descriptor. */
	fwrite (&mask,1,1,file);

/* write out the color table as a linear table */

	for (i=0; i<ncolor; i++) {
	   red[i]=255;
	   green[i]=255;
	   blue[i]=255;
	   }

	for (i=0; i<nred; i++) {
	   for (j=0; j<ngreen; j++) {
		for (k=0; k<nblue; k++) {
		   index = i + nred*j + nred*ngreen*k;
		   if (index >= nred*ngreen*nblue) continue;
		   red[index]=255*i/(nred-1);
		   green[index]=255*j/(ngreen-1);
		   blue[index]=255*k/(nblue-1);
		   }
		}
	  }

	for (i=0; i<ncolor; i++) {
	   fwrite (&red[i], 1,1,file);
	   fwrite (&green[i], 1,1,file);
	   fwrite (&blue[i], 1,1,file);
	   }

/* If we want to have a transparent background for the gif, the following
 * code does that; it also requires we use GIF89A above for the header
*/

/* Write a Grpahic Control Extension. This comes before the image descriptor.
 * First, write out extension introducer, graphic control label, and block
   size.
*/

/*
	 mask=0x21;
	 fwrite(&mask,1,1,file);
	 mask=0xf9;
	 fwrite(&mask,1,1,file);
	 mask=4;
	 fwrite(&mask,1,1,file);
*/

/* Next, a pile of flags; we only set the last bit, which indicates use
 * transparency
*/
/*
	mask=1;
	fwrite(&mask,1,1,file);
*/

/* Delay time is unused */

/*
	buf2=0;
	fwrite(&buf2,2,1,file);
*/

/* Index of transparent color.  I will hardwire index 0 */
/*
	mask=0;
	fwrite(&mask,1,1,file);
*/

/* Block terminator */

/*
	mask=0;
	fwrite(&mask,1,1,file);
*/

/* write out the image descriptor. */
	mask=0x2c;
	fwrite (&mask,1,1,file);
/* image left and top start indices (0 for both) */
	buf2=0;
	fwrite(&buf2,2,1,file);
	fwrite(&buf2,2,1,file);
/* image size again */
	buf2=swap2((short)nwidth);
	fwrite (&buf2,2,1,file);
	buf2=swap2((short)nheight);
	fwrite (&buf2,2,1,file);
/* another masked word; this says depthbits per pixel but no separate color map.
*/
	mask=depth-1;
	fwrite (&mask,1,1,file);
/* begin image. */
/* code size. */
	buf2=swap2((short)icode);
/*	fwrite (&buf2,1,1,file); */
	mask = icode;
	fwrite (&mask,1,1,file);
/* reblock and write out the rest of the data */
	blockout(tempaddr,ndata, file);
/* write out termination character */
	mask=0x3b;
	fwrite (&mask,1,1,file);
/* Free buffer */
	free(tempaddr);
	fclose (file);
	}
