/* encode a gif file. */
/* NOTE: We really do 6 bits! */

#include <stdio.h>
#include "dervish.h"
#include "gif8.h"
#include "math.h"
/* #include "malloc.h" */

/* Gif files use little-endian short ints. */

/* Return 1 if big-endian, 0 if little-endian */

/*************************************************************************/
static int ifswap(void) {
   short test=1;
   if (*(char *)&test == '\0') return 1;
   return 0;
   }

/*************************************************************************/
static short swap2(short in) {
   short temp;
   if (ifswap()) {
	((char *)&temp)[0] = ((char *)&in)[1];
	((char *)&temp)[1] = ((char *)&in)[0];
	return temp;
   } else {
	return in;
	}
   }

/*************************************************************************/
#define min(x,y) (x)<(y)? (x) : (y)

/*************************************************************************/
   void blockout (char *data, int ndata, FILE *file) {
	char buf[256];
	int ival4;
	int imax;
	int i,j;
/* write out data in blocks of 255 bytes. */
	for (i=0; i<ndata; i+=255) {
	   imax=min(i+254,ndata);
	   ival4=imax-i+1;
	   ((unsigned char *)buf)[0]=ival4;
	   for (j=1; j<=ival4; j++) {
		buf[j]=data[i+j-1];
		}
	   ((unsigned char *)buf)[0]=255;
	   fwrite (buf, 1, 256, file);
	   }
/* write out last block of 0 bytes */
	buf[0]=0;
	fwrite (buf, 1, 1, file);
	}

/****************************** gif8 *************************************/

void gif8 (unsigned char *img, int nwidth, int nheight, char *outfile, int DEPTH) {
	unsigned char red[256], green[256], blue[256];
	unsigned char mask;
	short buf2;
	int i;
/* create a temporary buffer that will hold the compressed image as
   it is created.  allocate space for 
   one image worth of bytes.  if compression is inefficient and
   actually results in expansion, the encode process will abort.
   i could equally well allocate this space in the output file, but then i
   would need to trim the file size when done, and i am not sure how
   to do that.
*/
	unsigned char *tempaddr;
/* n is an index to the most recently filled slot in the temporary buffer.
   on return, it has the number of bytes in the buffer.
*/
	int n=0;
/* encoding uses lzw algorithm (welch, ieee computer,
   vol 17, no 6, 1984).  do this in a separate subroutine.
   first, keep track of the last byte of ublocked data.  this will be needed
   for the reblocking to be done for the final file.
   that is pointless for disk files, but i didn't make up the rules.
   first, number of bits per pixel.
*/

/* If depth is 1, we would still need icode to be 2, according to GIF specs */
	int depth = DEPTH;
	int icode = depth;
	int ncolor = pow(2, DEPTH);
	int ndata;
	FILE *file;
	int ntot=nwidth*nheight;

/* Icode is a minumum of 2 */
	if (icode == 1) icode = 2;
	tempaddr = (unsigned char *)malloc(ntot);
/* Zero out temporary memory */
	for (i=0; i<ntot; i++) {
	   tempaddr[i]='\0';
	   }
	encode_ (img, &nwidth, &nheight, (char *)tempaddr, &n, &ntot,
		&icode);
/* write the output buffer to a file
   estimate file size
   first, the number of actual data bytes is ...
*/
	ndata=n;

/*	outsize=6+7+3*ncolor+10+1+(ndata)*(1+1/254.)+3+256; */
/* final 3 bytes: 1 is for truncations error of previous calculation
                1 is for the last block of 0 bytes
                1 is for the termination character.
*/
	file = fopen (outfile, "w");
/* write out the preamble
   fill in the signature
*/
	fwrite ("GIF87a", 6, 1, file);
/* create the screen descriptor */
	buf2=swap2((short)nwidth);
	fwrite (&buf2,2,1,file);
	buf2=swap2((short)nheight);
	fwrite (&buf2,2,1,file);
/* create compound byte */
	mask=0xf0 + (depth-1);	/* this says color table follows, depth bppix */
	fwrite (&mask,1,1,file);
/* background intensity.  set to 0 */
	mask=0;
	fwrite (&mask,1,1,file);
/* and a byte of zeros to end descriptor. */
	fwrite (&mask,1,1,file);
/* write out the color table as a linear table */
	for (i=0; i<ncolor; i++) {
	    red[i]=255*i/(ncolor-1);
	    green[i]=255*i/(ncolor-1);
	    blue[i]=255*i/(ncolor-1);
	    if (i == ncolor-1) {blue[i]=0;}
	  }

	for (i=0; i<ncolor; i++) {
	   fwrite (&red[i], 1,1,file);
	   fwrite (&green[i], 1,1,file);
	   fwrite (&blue[i], 1,1,file);
	   }
/* write out the image descriptor. */
	mask=0x2c;
	fwrite (&mask,1,1,file);
/* image left and top start indices (0 for both) */
	buf2=0;
	fwrite(&buf2,2,1,file);
	fwrite(&buf2,2,1,file);
/* image size again */
	buf2=swap2((short)nwidth);
	fwrite (&buf2,2,1,file);
	buf2=swap2((short)nheight);
	fwrite (&buf2,2,1,file);
/* another masked word; this says depthbits per pixel but no separate color map.
*/
	mask=depth-1;
	fwrite (&mask,1,1,file);
/* begin image. */
/* code size. */
	buf2=swap2((short)icode);
	fwrite (&buf2,1,1,file);
/* reblock and write out the rest of the data */
	blockout((char *)tempaddr,ndata, file);
/* write out termination character */
	mask=0x3b;
	fwrite (&mask,1,1,file);
/* Free buffer */
	free(tempaddr);
	fclose (file);
	}

