/* imgAnn.c --
*  Mar 1993 Steve Kent
*   
* Tcl functions defined in this module:
*
* imgAnn
*
*/

#include "tcl.h"
#include "dervish.h"
#include "shGeneric.h"
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "imgProto.h"
#include "img_msg_c.h"
#include "imgProf.h"

/************************** imgann - the action routine **********************/

/* Compute a radial profile in circular annuli.  Allow images of any type.
** Don't use ellipses yet, although they can be added at a future date.
*/

IMG_CODE imgAnn (REGION *reg, double row, double col, int rmax,
	CHAIN *proflist) {

   PROF *prof;
   int what;
   int i,j;
   int npix;
   int nstep;
   double r,c;
   double pix, pixsum, posang;
   double pi=3.14159265358979;
   double profsum, cnts0=0.;
   double a2, norm, cnts1;

   if (rmax <= 0) return IMG_SUCCESS;

   profsum = 0.;
   for (i=0; i<=rmax; i++) {
	prof = (PROF *)shGenericNew ("PROF");
	if (prof == NULL) return IMG_GENERIC_ERROR;
	what = shChainElementAddByPos (proflist, prof, "PROF", TAIL, AFTER);
	prof -> rad = i;
	prof -> row = row;
	prof -> col = col;
	prof -> ang = 0.;
	prof -> eps = 0.;

/* Number of steps around the circumference of the annulus */
	nstep = 2. * pi * i + 1;
	pixsum = 0.;   npix = 0;
	for (j=0; j<nstep; j++) {
	   posang = (j * 2. * pi) / nstep;
	   r = row + i * cos (posang);
	   c = col + i * sin (posang);
	   pix = pixbin(reg, r, c);
	   if (pix != 0.) {
		npix++;
		pixsum = pixsum + pix;
		}
	   }
	if (npix != 0) {prof -> cnts = pixsum / npix;}
		else {prof -> cnts = 0.;}

/* Cumulative counts */
	cnts1 = prof->cnts;
	if (i > 0) {

/* Let's try 1/(a^2+r^2) profile. */
	   if (cnts1 != cnts0) {
		a2 = (cnts1*i*i - cnts0*(i-1.)*(i-1.)) / (cnts0 - cnts1);
		norm = cnts1 * (a2 + i*i);

/* Linear intensity gradient */
		if (a2 > 0) {
		   profsum += norm * pi * log((a2+i*i)/(a2+(i-1.)*(i-1.)));
		} else {
		   profsum += pi*(i*(cnts0+cnts1) -
			(2.*cnts0+cnts1)/3.);
		   }
	   } else {
		profsum += cnts1*pi*(i*i - (i-1.)*(i-1.));
		}
	   }
	prof->cum = profsum;
	cnts0 = prof->cnts;
	}
   return IMG_SUCCESS;
   }
