/**********************************************************************
 * Interpolate in a spectrum
 *********************************************************************/

#include <math.h>
#include "dervish.h"
#include "img_msg_c.h"
#include "imgProto.h"
#include "imgInterp.h"


/*============================================================================  
**============================================================================
**
** ROUTINE: imgInterp
**
** DESCRIPTION:
**	Interpolate to a specified wavelength in a DVEC structure.
**
** RETURN VALUES:
**	Interpolated flux
**	IMG_SUCCESS	Success
**
**============================================================================
*/


IMG_CODE imgInterp	(DVEC *dvec,	/* Input spectrum */
			double wave,	/* In: wavelength */
			double *flux)	/* Out: Interplated Flux */

{

/* Local variables */
   int i, k;
   int n = dvec->wave->dimen;

   if (wave < dvec->wave->vec[0] || wave > dvec->wave->vec[n-1]) {
	*flux = 0; return IMG_SUCCESS;
	}
   k=0;
   for (i=0; i<n-1; i++) {
	if (wave <= dvec->wave->vec[i]) break;
	k++;
	}
   if (k == 0) {
	*flux = dvec->flux->vec[0];
	return IMG_SUCCESS;
	}

/* k gives index of wavelength just longward of wave */
   *flux = dvec->flux->vec[k-1] + (dvec->flux->vec[k] -
	dvec->flux->vec[k-1]) * (wave - dvec->wave->vec[k-1]) /
	(dvec->wave->vec[k] - dvec->wave->vec[k-1]);
   return IMG_SUCCESS;
}


