/*****************************************************************************
 ******************************************************************************
 **
 ** FILE:
 **     imgMain.c
 **
 ** ABSTRACT:
 **	Top level routine for running image analysis programs.
 **
 ** ENVIRONMENT:
 **     ANSI C.
 ******************************************************************************
 ******************************************************************************
 */

#include <stdio.h>
#include <sys/types.h>
#include <malloc.h>
#include "dervish.h"
/*
#include "dp.h"
*/

#define L_SHPROMPT    "img> "   /* Command line prompt */

/* These prototypes really belong in a header file */

void imgDeclare (Tcl_Interp *interp);
void fileCopyDeclare (Tcl_Interp *interp);

void atVerbsDeclare(Tcl_Interp *interp);
void smaVerbsDeclare(Tcl_Interp *interp);

void tkInterpSave(Tcl_Interp *interp);
/* void shTclBinReadDeclare(Tcl_Interp *interp); */

int main(int argc, char *argv[])
{
   CMD_HANDLE sh_handle;
   Tcl_Interp    *interp;
  

/* Define TYPE to schema ahead of dervish */

   if (shMainTcl_Init(&sh_handle, &argc, argv, L_SHPROMPT) != SH_SUCCESS) {
	printf ("Failed during shMainTcl_Init\n");
	return 1;
	}

   interp = sh_handle.interp;

   /*
    * Declare required TCL commands
    */
   ftclCreateCommands(interp);          /* Fermi enhancements to TCL   */
   ftclCreateHelp (interp);             /* Define help command         */
   ftclMsgDefine (interp);              /* Define help for tcl verbs   */
 


   /* --- PLACE YOUR FAVORITE INITIALIZATIONS HERE --- */
   ftclParseInit(interp);               /* Ftcl command parsing */
   tkInterpSave(interp);		/* Stow interp pointer for tk pgPlot
					 * emulation */
 
   /*
    * NOTE: shMainTcl_Declare is a comprehensive declaration routine for
    * Dervish.  Included are TCL command declarations for region, peek, and
    * FSAOimage.  If all commands are not desired, one can customize by
    * replacing the call to this routine with individual calls to only the
    * desired TCL command declaration routines. It runs the initialization
    * scripts, so it must go after all of the initialization routines have
    * been called.
    */

   shMainTcl_Declare(&sh_handle);       /* Required Dervish TCL commands */

/*
   Tdp_Init(interp);
*/
   /* --- PLACE YOUR FAVORITE TCL DECLARATIONS HERE --- */
 
   smaVerbsDeclare(interp);
   atVerbsDeclare(interp);
   fileCopyDeclare(interp);
   imgDeclare(interp);
/*   shTclBinReadDeclare(interp); */

   /*
    *             DO NOT PUT ANYTHING BELOW THIS LINE.
    *
    * We will now enter the main loop of Tk. There is no successfull return
    * from this loop. Anything added below this line will never be reached,
    * NEVER!!
    */
   shTk_MainLoop(&sh_handle);

   return 0;
}





