/* imgMbs.c --
*  Mar 1998 Steve Kent
*  Add gaussian star images with photon noise.
*  Also, a routine for fluctuating pixel values.  Provide a check on dervish
*  counterpart
*
*/
#include <math.h>
#include "dervish.h"
#include "shGeneric.h"
#include "imgStar.h"
#include "img_msg_c.h"
#include "imgProto.h"

#ifndef PI
#define PI 3.14159265358979
#endif

/************************** imgMbs - the action routine **********************/

IMG_CODE imgMbs (REGION *reg, double rowc, double colc, double sig, double count,
   double gain) {

   int i;
   int nr = reg->nrow;
   int nc = reg->ncol;
   double row, col;
   int irow, icol;
   double random;
   double radius, angle;

/* Pointer to function to return integer pixel value */
/*   int (*intpix)(REGION *, int, int) = regPixIntFunc (reg); */

/* Pointer to function to return double pixel value */
   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

/* Pointer to a function to set a double pixel value */
   void (*dblpixset)(REGION *, int, int, double) =
	regPixDblFuncSet (reg);

/* Keep track of dithering. */
   double dither = 0.5;

   for (i=0; i<count*gain; i++) {
	random = drand();
	if (random == 1.) continue;
/* Radius */
	radius = sqrt(-2.*log(1.-random))*sig;
/* Angle */
	angle = 2.*PI*drand();
	row = rowc + radius * cos(angle);
	col = colc + radius * sin(angle);
/* Truncate to nearest integer */
	irow = (row);
	icol = (col);
	if (irow < 0 || irow >= nr) continue;
	if (icol < 0 || icol >= nc) continue;
/* Well, if the image is of integer data type, I would need to dither each
   photon increment to avoid quantization error.  But the easiest way leads
   to an image with noise that is independent of gain.  So I will just increment
   each pixel naively and the hell with it.  If the gain is 1, this is OK for
   integer regions.  If the gain is > 1, I want to be using a floating point
   region.
   Bing -> I know how to do it.
*/
	dither += 1./gain;
	if (dither >= 1.) {
	   dblpixset (reg, irow, icol, dblpix(reg, irow, icol) + 1);
	   dither -= 1;
	   }
	}
   return IMG_SUCCESS;
}

/************************** imgNoise - add poisson noise **********************/

/* Just specify an rms noise value to add. */

IMG_CODE imgNoise (REGION *reg, double rms) {

   int i, j;
   int nr = reg->nrow;
   int nc = reg->ncol;
   double random;

/* Pointer to function to return integer pixel value */
/*   int (*intpix)(REGION *, int, int) = regPixIntFunc (reg); */

/* Pointer to function to return double pixel value */
   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

/* Pointer to a function to set a double pixel value */
   void (*dblpixset)(REGION *, int, int, double) =
	regPixDblFuncSet (reg);

   for (i=0; i<nr; i++) {
	for (j=0; j<nc; j++) {
/* Fake up gaussian noise.  Actually, photon statics aren't even really gaussian */
	   random = gaussRan(5);
	   random = random * rms;
	   dblpixset (reg, i, j, dblpix(reg, i, j) + random);
	   }
	}
   return IMG_SUCCESS;
}

/************************* imgPhoton - add photon noise *********************/

/* Assume input image is perfect.  Specify gain but nothing else. */

IMG_CODE imgPhoton (REGION *reg, double gain) {

   int i, j;
   int nr = reg->nrow;
   int nc = reg->ncol;
   double random;
   double pix;

/* Pointer to function to return integer pixel value */
/*   int (*intpix)(REGION *, int, int) = regPixIntFunc (reg); */

/* Pointer to function to return double pixel value */
   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

/* Pointer to a function to set a double pixel value */
   void (*dblpixset)(REGION *, int, int, double) =
	regPixDblFuncSet (reg);

   for (i=0; i<nr; i++) {
	for (j=0; j<nc; j++) {
	   random = gaussRan(5);
	   pix = dblpix(reg, i, j);
	   dblpixset (reg, i, j, pix + random*sqrt(pix/gain));
	   }
	}
   return IMG_SUCCESS;
}
