#include <math.h>
#include "dervish.h"
#include "img_msg_c.h"
#include "imgProto.h"

#ifndef PI
#define PI 3.14159265358979
#endif

#define sqr(x) (sqrt(fabs(x)))


/*============================================================================  
**============================================================================
**
** ROUTINE: imgCircle
**
** DESCRIPTION:
**	Compute integrated intensity in a circular aperture.
**	For pixels on the boundary of the aperture, approximate square pixel
**	by a circle.
**
** RETURN VALUES:
**	IMG_SUCCESS	Success
**
**============================================================================
*/

/* rowc, colc are center of circle */

IMG_CODE imgCircle (REGION *reg, double rowc, double colc, double rad,
   double *counts, double *area) {

/* rowp, colp are coords of the mid-point of a pixel */
/* rowl, coll are points on arc of aperture boundary that are on the line
 * connecting the circle center and the pixel center */

   double rowp, colp;

/* Local indexes */
   int i, j;

/* Local variables */
   double f, u;

/* Radius of a circle with unit area */
   double rp = 1./sqrt(PI);
   double radp, r2;

/* Pointer to function to return double pixel value */
   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

/* Threshold is peak along boundary */

   *counts = 0.;
   *area = 0.;
   for (i = (int)(rowc - rad - 2); i <= (int)(rowc + rad + 2); i++) {
	if (i < 0 || i >= reg->nrow) continue;
  	for (j = (int)(colc - rad - 2); j <= (int)(colc+rad + 2); j++) {
	   if (j < 0 || j >= reg->ncol) continue;
	   rowp = i+0.5;
	   colp = j+0.5;
	   r2 = pow(rowp-rowc,2) + pow(colp-colc,2);
	   if (r2 > pow(rad+.707, 2)) continue;
	   if (r2 < pow(rad-.707, 2)) {
		*counts += dblpix(reg,i,j);
		*area += 1.;
		continue;
		}
	   radp = sqrt(r2);
	   u = rad - radp;
	   if (u > rp) u = rp;
	   if (u < -rp) u = -rp;
	   f = .5 + asin(u/rp)/PI;
	   if (f < 0.) f=0.;
	   if (f > 1.) f=1.;
	   *counts += dblpix(reg,i,j) * f;
	   *area += f;
	   }
	}
   return IMG_SUCCESS;
}

