/******************************************************************************
******************************************************************************/


#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include <string.h>
#include <stdio.h>

#include "dervish.h"
#include "imgTypes.h"

/*============================================================================
**============================================================================
**
** LOCAL MACROS, DEFINITIONS, ETC.
**
**============================================================================
*/

/* macro to "round down", assisting in the conversion of floating point 
** types to ints.
*/ 
#define ROUND_DOWN(d) ((d)>= 0.0 ? (d) + 0.5: -(-(d) + 0.5))

static void utilerror (char *string) {printf ("%s \n", string); abort ();}

static U8 set_u8 (double dbl)
{
   if (dbl < (double)U8_MIN) return (U8_MIN);
   if (dbl > (double)U8_MAX) return (U8_MAX);
   return ROUND_DOWN(dbl);
}

static S8 set_s8 (double dbl)
{
   if (dbl < (double)S8_MIN) return (S8_MIN);
   if (dbl > (double)S8_MAX) return (S8_MAX);
   return ROUND_DOWN(dbl);
}

static U16 set_u16 (double dbl)
{
   if (dbl < (double)U16_MIN) return (U16_MIN);
   if (dbl > (double)U16_MAX) return (U16_MAX);
   return ROUND_DOWN(dbl);
}

static S16 set_s16 (double dbl)
{
   if (dbl < (double)S16_MIN) return (S16_MIN);
   if (dbl > (double)S16_MAX) return (S16_MAX);
   return ROUND_DOWN(dbl);
}

static U32 set_u32 (double dbl)
{
   if (dbl < (double)U32_MIN) return (U32_MIN);
   if (dbl > (double)U32_MAX) return (U32_MAX);
   return ROUND_DOWN(dbl);
}

static S32 set_s32 (double dbl)
{
   if (dbl < (double)S32_MIN) return (S32_MIN);
   if (dbl > (double)S32_MAX) return (S32_MAX);
   return ROUND_DOWN(dbl);
}

static FL32 set_fl32 (double dbl)
{
   return (FL32)(dbl);
}


double pxGetAsDbl (const REGION *reg, int r, int c)
{
   switch (reg->type) {
    case TYPE_U8: 	return ((double) reg->rows_u8[r][c]);
    case TYPE_S8: 	return ((double) reg->rows_s8[r][c]);
    case TYPE_U16:	return ((double) reg->rows   [r][c]);
    case TYPE_S16:	return ((double) reg->rows_s16[r][c]);
    case TYPE_U32:	return ((double) reg->rows_u32[r][c]);
    case TYPE_S32:	return ((double) reg->rows_s32[r][c]);
    case TYPE_FL32:	return ((double) reg->rows_fl32[r][c]);
    default:		utilerror ("region type is unsupported");
   }
   return (0.0);
}


void pxSetWithDbl (REGION *reg, int r, int c, double dbl)
{
   switch (reg->type) {
    case TYPE_U8: 	reg->rows_u8[r][c]   = set_u8(dbl);   return;
    case TYPE_S8: 	reg->rows_s8[r][c]   = set_s8(dbl);   return;
    case TYPE_U16:	reg->rows   [r][c]   = set_u16(dbl);  return;
    case TYPE_S16:	reg->rows_s16[r][c]  = set_s16(dbl);  return;
    case TYPE_U32:	reg->rows_u32[r][c]  = set_u32(dbl);  return;
    case TYPE_S32:	reg->rows_s32[r][c]  = set_s32(dbl);  return;
    case TYPE_FL32:	reg->rows_fl32[r][c] = set_fl32(dbl); return;
    default:		utilerror ("region type is unsupported"); 
   }
   return;
}

int pxSize (REGION *reg)
{
   switch (reg->type) {
    case TYPE_U8: return 1;
    case TYPE_S8: return 1;
    case TYPE_U16: return 2;
    case TYPE_S16: return 2;
    case TYPE_U32: return 4;
    case TYPE_S32: return 4;
    case TYPE_FL32: return 4;
    default: utilerror ("region type is unsupported"); 
   }
   return 0;
}

/* Return generic data type */

int pxType (REGION *reg)
{
   switch (reg->type) {
    case TYPE_U8: return IMG_INT;
    case TYPE_S8: return IMG_INT;
    case TYPE_U16: return IMG_INT;
    case TYPE_S16: return IMG_INT;
    case TYPE_U32: return IMG_INT;
    case TYPE_S32: return IMG_INT;
    case TYPE_FL32: return IMG_FLOAT;
    default: utilerror ("region type is unsupported"); 
   }
   return 0;
}


