/*****************************************************************************
******************************************************************************
**
** FILE:
**	imgProfile.c
**
** ABSTRACT:
**	This file contains C routines to create and delete
**	PROFILE structures.
**
** ENTRY POINT		SCOPE	DESCRIPTION
** -------------------------------------------------------------------------
** imgProfileNew		public  Create a new PROFILE
** imgProfileDel		public	Delete a PROFILE
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	dervish
**	profile
**
** AUTHORS:
**	Creation date:  2 Feb,1994
**	Mario Nonino
**
******************************************************************************
******************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dervish.h"
#include "imgProfile.h"
/*** Message codes ****/
#include "img_msg_c.h"


/*============================================================================  
**============================================================================
**
** ROUTINE: imgProfileNew
**
** DESCRIPTION:
**	Create a new PROFILE
**
** RETURN VALUES:
**	Pointer to new structure, or NULL if failure
**
** CALLS TO:
**	shMalloc
**
**============================================================================
*/
PROFILE *
imgProfileNew(void)
{

   PROFILE *profile = shMalloc(sizeof(PROFILE));
/* Update here if more complicated allocation of storage is needed */
   profile->algorithm  = CCD_ALGORITHM;
   profile->radius = 0.0;
   profile->xc = 0.0;
   profile->yc = 0.0;
   profile->ang  = 0.0;
   profile->ecc  = 0.0;
   profile->flag  = 0;
   profile->a0 = 0.0;
   profile->b0 = 0.0;
   profile->a1 = 0.0;
   profile->b1 = 0.0;
   profile->a2 = 0.0;
   profile->b2 = 0.0;
   profile->a4 = 0.0;
   profile->b4 = 0.0;
   profile->deriv = 0.0;
   profile->intensity = 0.0;
   return(profile);
}


/*============================================================================  
**============================================================================
**
** ROUTINE: imgProfileDel
**
** DESCRIPTION:
**   Delete a PROFILE
**
** RETURN VALUES:
**   PROFILE_SUCCESS   Success
**
** CALLS TO:
**   shFree
**
**============================================================================
*/
IMG_CODE
imgProfileDel(PROFILE *profile)
{
   shFree(profile);
   return(IMG_SUCCESS);
}
