#include <math.h>

/*******************Utility: random number generator*************************/

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

static int iv[NTAB];
static int iy=0;
static int seed = 17;

/* Return random number between 0 and 1.  Seed is hardwired above, but
 * it can be reset via the routine imgReseed
 */

double drand()
{
   int j;
   long k;
   double temp;

   if (seed <=0 || !iy) {
	if (-(seed) < 1) seed=1;
	else seed = -(seed);
	for (j=NTAB+7; j>=0; j--) {
	   k=(seed) / IQ;
	   seed = IA * (seed - k * IQ) - IR * k;
	   if (seed < 0) seed += IM;
	   if (j < NTAB) iv[j] = seed;
	   }
	iy = iv[0];
   	}
   k = (seed) / IQ;
   seed = IA * (seed - k * IQ) - IR * k;
   if (seed < 0) seed += IM;
   j = iy / NDIV;
   iy = iv[j];
   iv[j] = seed;
   if ((temp = AM * iy) > RNMX) return RNMX;
   else return temp;
   }

/********************************** ran1 ***********************************/
/* Old version of my random number generator. I think the functionality
* is unchanged from the original. */

double ran1(int *idum) {
   double temp;
   seed = *idum;
   temp = drand();
   *idum = seed;
   return temp;
   }

/******************************** reseed **********************************/
/* Reset the seed */

void reseed (int idum) {
   seed = idum;
   iy = 0;
   return;
   }

/******************************** ranGauss *******************************/

/* Generate an approximately Gaussian random number with sigma = 1 */
/* n specified the number of convolutions of a box-car.  A good value is
 * 3 to 5.
*/

double gaussRan (int n) {
   int k;
   double random;

   random = 0;
   if (n < 1) n = 1;
   for (k=0; k<n; k++) {
	random += (drand() - 0.5);
	}
   random = random * sqrt(12./n);
   return random;
   }
