/**************************************************************************
 * Object Finder
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <string.h>
#include "dervish.h"
#include "shGeneric.h"
#include "imgProto.h"
#include "imgStar.h"
#include "img_msg_c.h"

#define min(a,b) ((a>b) ? b : a)

/****************************************************************************/

static double vector_max(double *data,int elements)
{
double max_value = -1.e34;     /* must be redefine: ANSI_C */
register int i;


        for(i=0;i < elements;i++)
        if (*(data+i) > max_value) max_value = *(data+i);

        return max_value;
}

/****************************************************************************/

IMG_CODE
imgRegFindObj(
 	REGION 	  *reg,
	CHAIN     *starlist,
		double sky,
		double sky_noise,
		double radius,
		double sig1,
		double sig2,
		double sig4,
		double photon,
		double readout
	    )
{
   STAR *star;
   register int i,j,k,l;
   double max_value =0.0;
   double readout_noise =0.0;
   double sigma1=0.0,sigma2=0.0,sigma4=0.0;
   int nrad=0;
   int nedge=0;
   int krow=0,kcol=0;
   int nstart=0;
   double avg=0.0;
   double values[8];
   double image_value = 0.0;

   int masked_pixels = 0;
   int *pmask;  /* this must be removed ! */

/* Pointer to function to return integer pixel value */
   int (*intpix)(REGION *, int, int) = regPixIntFunc (reg);

/* Pointer to function to return double pixel value */
   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

   AXES axes;
   double counts, area;

   double sum_i=0.0, sum_k=0.0, sum_j=0.0;

/******************** Begin code *********************************/

   pmask = (int *) shMalloc((reg->ncol)*(reg->nrow)*sizeof(int));

   for (i=0;i<reg->ncol;i++)
	for(j=0;j<reg->nrow;j++)
		pmask[i + j*reg->ncol] = 0;

   nrad = (int) (radius+0.5);
   readout_noise = pow(readout,2.0);
	
   sigma1 = pow(sig1,2.0);
   sigma2 = pow(sig2,2.0);
   sigma4 = sig4*sky_noise;

   nedge = nrad + 1;
/*	nedge= min(nrad+1,4);*/
   krow = nedge +1;
   kcol = nedge +1; 

   for(j=krow;j< (reg->nrow - nedge+1);j++) {
	if( j  == krow) 
	   nstart = kcol;
	else
	   nstart = nedge + 1;
	for(i=nstart;i<(reg->ncol - nedge+1);i++) {
	   values[0] = dblpix(reg,j-1,i-1);
	   values[1] = dblpix(reg,j-1,i);
	   values[2] = dblpix(reg,j,i-1);
	   values[3] = dblpix(reg,j-1,i+1);
	   values[4] = dblpix(reg,j,i+1);
	   values[5] = dblpix(reg,j+1,i+1);
	   values[6] = dblpix(reg,j+1,i);
	   values[7] = dblpix(reg,j+1,i-1);
	   image_value = dblpix(reg,j,i);
	   max_value = vector_max(values,8);

	   if(image_value < max_value) continue;
/****** mask here!! **/
	   sum_i = 0.0;
	   sum_j = 0.0;
	   sum_k = 0.0;

/* make sure object is above sky level */
	
	   for(k=j-1;k<j+2;k++)
		for(l=i-1;l<i+2;l++)
		   sum_i += dblpix(reg,k,l);

	   sum_i -= image_value;
	   sum_i /= 8.;
	   if( sum_i <= sky) continue;

	   for(k=j-2;k<j+3;k++)
		for(l=i-2;l<i+3;l++)
		   sum_j += dblpix(reg,k,l);

	   sum_j -= (8.0 * sum_i + image_value);
	   sum_j /= 16.0;

	   if (sum_i <= sum_j ) continue;

	   for(k=j-3;k<j+4;k++)
		for(l=i-3;l<i+4;l++)
		sum_k += dblpix(reg,k,l);

	   sum_k -= (8.0 * sum_i +16.0 *sum_j + image_value);
	   sum_k /= 24.0;
	   if (sum_j <= sum_k) continue;

/* make sure peak is significant at least at sig1 level */

	   if(pow((image_value - sum_j),2.0) <= 
		(sigma1*(sum_j/photon+readout_noise))) continue;

	   if(pow((sum_i - sum_j),2.0) <= 
		(sigma2*(sum_j/photon+readout_noise))) continue;


/* make sure average level is well above sky */

	   sum_i -= sky;
	   sum_j -= sky;

	   avg = (image_value - sky + 8.0*sum_i)/9.0;

	   if (avg <= sigma4) continue;	

	   if (pmask[i + j*(reg->ncol)] == 1) continue;
	   if ((star = shGenericNew("STAR")) == NULL) return IMG_GENERIC_ERROR;
	   shChainElementAddByPos (starlist, star, "STAR", TAIL, AFTER);
	   star->row = j;
	   star->col = i;

	   for (k=j-nedge;k<j+nedge;k++)
		for(l=i-nedge;l<i+nedge;l++) {
		   pmask[l + k*(reg->ncol)] = 1;
			/*don't find peak here again */
		   masked_pixels += 1;
		   }

/*Continue if we are within nrad of the edge */
	   if((j-nrad) < 1 || (j+nrad) > reg->nrow || (i-nrad) < 1 ||
		(i+nrad) > reg->ncol) {
		continue;
		}

	   if (imgAxes (reg, j, i, nrad, nrad, &axes) == IMG_SUCCESS) {
		star->row = axes.row;
		star->col = axes.col;
		star->peak = axes.cnts;
		star->fwhm = (axes.sigmaj + axes.sigmin)*2.35/2.;
		star->eps = axes.eps;
		star->ang = axes.pos_maj;
	   } else {
		continue;
		}

/* Do photometry */

	   if (imgCircle (reg, axes.row, axes.col, radius, &counts, &area)
		== IMG_SUCCESS) {
		star->cnts = counts - sky*area;
		}

	   }
	 }

   shFree(pmask);
/*   fprintf(stderr," masked pixels %d \n",masked_pixels); */
   return IMG_SUCCESS;
   }
