/*
******************************************************************************
*	
* MODULE	 Scope 
* imgRegPrfile    profile.for from CCD (S.Kent)	2-Feb-1994 M. Nonino
* 
*   -- compute surface brigthness profile from concentric annuli
******************************************************************************
*/
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <string.h>
#include "dervish.h"

#include "imgProfile.h"
#include "imgAxes.h"
#include "imgProfile.h"
#include "imgProto.h"

static void xyaeinc(double *, double *, double *, double *, double ,
	     double , double , double , double , double ,
	     int , double );
static void ppolyfit(double *,int ,int );
static void pfunc(int, double *);
static void pderiv(int, double *);

int     nterm=0;

static	double pi = 3.14159265358979;
#define max(a,b) (((a)>(b)) ? (a) : (b))
#define min(a,b) (((a)>(b)) ? (b) : (a))
/******************************************************************************
*******************************************************************************
*/

/******************************************************************************/

IMG_CODE
 imgRegProfile(
		REGION *region,	/* input */
		CHAIN *profilelist,	/* input | output*/
		AXES *centroid,	/* input */
	        int start_pixel,	/* 11-14-1994 addition */
		int end_pixel,             
		int nderiv,		/* input */
		double sky		/* input */
		)
{


/* private */

   PROFILE **profile;
   CURSOR_T cursor;

   register int i,k,l;
   int npts=0;
   double func=0.0;

   double sum=0.0,radius=0.0;

   double a0=0.0,b0=0.0,a1=0.0,b1=0.0,a2=0.0,b2=0.0,a4=0.0,b4=0.0;
   double cs =0.0,sn =0.0;
   double a11=0.0,b11=0.0,a22=0.0,b22=0.0,xc0=0.0,yc0=0.0,ang0=0.0,ecc0=0.0;

   double ecc=0.0,ang=0.0;
   double xc=0.0,yc=0.0;


   int iter =0;

   double f_inc = 0.5;    /* must be modified!! */

   int l1=0,l2=0,nc=0;	
   int size, n_steps;
	
   double b=0.0,e=0.0,de=0.0,dx=0.0,dy=0.0,deriv=0.0;	


   double x=0.0,y=0.0,px=0.0;
   int npx=0,incflag=0;
	
   double nnz=0.0, c1=0.0,c2=0.0,c4=0.0,s1=0.0,s2=0.0,s4=0.0;
   double p1=0.0,p2=0.0,p3=0.0,p4=0.0;

   double *pro_file;
   char yn[81];

/* end */

printf ("Inputs are %d %d %d %lf\n", start_pixel, end_pixel, nderiv, sky);

/* Create or extend profile list as necessary */
   size = shChainSize(profilelist);
   if (size < end_pixel+1) {
	for (i=size; i<=end_pixel; i++) {
	   shChainElementAddByPos (profilelist, (void *)imgProfileNew(),
		"PROFILE", TAIL, AFTER);
	   }
	}
   size = shChainSize(profilelist);
/* Create array of pointers to manipulate more easily */
   profile = (PROFILE **)shMalloc(size * sizeof(PROFILE *));
   cursor = shChainCursorNew(profilelist);
   for (i=0; i<size; i++) {
	profile[i]=(PROFILE *)shChainWalk (profilelist, cursor, NEXT);
	}
   shChainCursorDel (profilelist, cursor);

   xc = centroid->row;
   yc = centroid->col;
   ang = centroid->pos_maj;
   ecc = sqrt(1. - pow(1.-centroid->eps,2.));
printf ("Input axes are %lf %lf %lf %lf\n", xc, yc, ang, ecc);

/* --central point */
   if (start_pixel == 0) {
	profile[0]->radius = 0;
	profile[0]->xc = xc;
	profile[0]->yc = yc;
	profile[0]->ang = ang;
	profile[0]->ecc = ecc;
	profile[0]->intensity = xpixbin(region,xc,yc);
printf("%6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf\n",
0., xc, yc, ang, ecc, profile[0]->intensity);
	start_pixel=start_pixel+1;
	}

   n_steps = end_pixel - start_pixel +1;
   if (n_steps <= 0) {
	shFree (profile);
	return IMG_SUCCESS;
	}

   l1 = start_pixel;
   l2 = end_pixel;
	
/* Scratch space */
   nc = (int) (2.0*pi*l2 + 1.0);
   pro_file = (double *)shMalloc(sizeof(double)*nc);

/* -- step through contours 
 * -- k = index in profile elements
 * -- radius = major axis of ellipse in pixels 
*/

/* main loop here ... */

/* Enter while loop until done */
   while (1) {
   for (k=l1; k<=l2; k++) {
	radius =  k;

/* -- a priori elements for ellipse. if iter =0 use current values for xc, etc.
 * -- if iter = 1 use a priori contour from previous pass
 */
	if (iter == 1) {
		xc = profile[k]->xc;
		yc = profile[k]->yc;
		ang = profile[k]->ang;
		ecc = profile[k]->ecc;
    	 	}

	cs = cos(ang*pi/180.0);
	sn = sin(ang*pi/180.0);

	b = radius * sqrt(1.0 - pow(ecc,2.0));
	nc = (int) (2.0*pi*radius + 1.0);
	nc /= 4;
	nc *= 4;

	de = 2.0*pi/((double) nc);
/* get intensity along a priori contour */


	for(l=0; l< nc; l++) {

		e = ((double) (l+1)) * de;
		dx = radius * cos(e);
		dy = b * sin(e);
		
		x = xc + dx*cs - dy*sn;
		y = yc + dx*sn + dy*cs;

		if(k<=2) pro_file[l] = xpixbin(region,x,y);
		       else 
			  pro_file[l] = pixbin(region,x,y);

		} /* end for(l... */

/* -- points l, nc-l, nc/2-l, nc/2 +l are equivalent. 
 * -- Fill in points lost to zones
*/
	incflag = 0;
	for (l=0; l < nc/4; l++) {
		if (pro_file[l] != 0.0
		   && pro_file[nc - l-2]!= 0.0 
		   && pro_file[nc/2 - l-2] != 0.0
		   && pro_file[nc/2 + l] != 0.0
	           )
		   continue;

		npx = 0;
		px  = 0.0;

		if(pro_file[l]        != 0.0) { npx +=1; px += pro_file[l];}
		if(pro_file[nc-l-2]   != 0.0) { npx +=1; px += pro_file[nc-l-2];}
		if(pro_file[nc/2-l-2] != 0.0) { npx +=1; px += pro_file[nc/2-l-2];}
		if(pro_file[nc/2+l]   != 0.0) { npx +=1; px += pro_file[nc/2+l];}

		if(npx == 0) {
		   incflag =1;
		   continue;
		   }

		px /= ((double) npx);

		if(pro_file[l]        == 0.0) pro_file[l]        = px;
		if(pro_file[nc-l-2]   == 0.0) pro_file[nc-l-2]   = px;
		if(pro_file[nc/2-l-2] == 0.0) pro_file[nc/2-l-2] = px;
		if(pro_file[nc/2+l]   == 0.0) pro_file[nc/2+l]   = px;

		} /* end for(l ... */

	if(pro_file[0] == 0.0) pro_file[0] = pro_file[nc/2-1];
	if(pro_file[nc/2-1] == 0.0) pro_file[nc/2-1] = pro_file[0];

	if(pro_file[0] == 0.0 &&  pro_file[nc/2-1] == 0) incflag = 1;

	if(incflag) {
/* -- countour is incomplete. Zero out complementary parts of contour */

	   for(l=0; l<nc/4-1;l++) {
		if(pro_file[l] != 0.0) continue;

		pro_file[nc/4-l-2]   = 0.0;
		pro_file[nc/4+l]     = 0.0;
		pro_file[3*nc/4-l-2] = 0.0;
		pro_file[3*nc/4+l]   = 0.0;
		}

	   if(pro_file[0] == 0.0) {
		pro_file[nc/4-1]   = 0.0;
		pro_file[3*nc/4-1] = 0.0;
		}

	   if(pro_file[nc/4-1] == 0.0) {
		pro_file[0]   = 0.0;
		pro_file[nc/2-1] = 0.0;
		}

	   nnz = 0.0;
		
	   for(i=0;i<nc/4-1;i++) 
			if(pro_file[i]) nnz +=4.0;

	   } 
	else {
		nnz = (double) nc;
		}

/* -- get Fourier coefficients */

	b0 = pro_file[nc/4-1] + pro_file[nc/2-1] + pro_file[3*nc/4-1] + pro_file[nc-1];
	a1 = - pro_file[nc/2-1] + pro_file[nc-1];
	b1 = pro_file[nc/4-1] - pro_file[3*nc/4-1];
	a2 = -pro_file[nc/4-1] + pro_file[nc/2-1] - pro_file[3*nc/4-1] + pro_file[nc-1];


	b2 = 0.0;

	a4 = b0;
	b4 = 0.0;

	if(nc > 4) {
		
		for(l=1; l< nc/4; l++) {

		   e = ((double) l) * de;

 		   c1 = cos(e);
		   s1 = sin(e);

		   c2 = pow(c1,2.0) - pow(s1,2.0);
		   s2 = 2.0 * s1 * c1;

		   c4 = pow(c2,2.0) - pow(s2,2.0);
		   s4 = 2.0 * s2 * c2;

		   p1 = pro_file[l-1];
		   p2 = pro_file[nc/2 - l-1];
		   p3 = pro_file[nc/2 + l-1];
		   p4 = pro_file[nc - l-1];
			 

		   sum = p1 + p2 + p3 + p4;

		   b0 += sum;

	
		   a1 += (c1*(p1 - p2 - p3 + p4));
		   b1 += (s1*(p1 + p2 - p3 - p4));
	
		   a2 += (c2*sum);
		   b2 += (s2*(p1 - p2 + p3 - p4));
	
		   a4 += (c4*sum);
		   b4 += (s4*(p1 - p2 + p3 - p4));

		   } /* end for(l ... */
		if( nc <= 8) {
		   a4 /= 2.0;
		   b4 /= 2.0;
		   }

		}
	else
		{
		a2 /= 2.0;
		a4 = 0.0;
		}

	if(nnz == 0.0) nnz = 1.0;

	b0 /=  nnz;
	a1  *= (2.0/nnz);
	b1  *= (2.0/nnz);
	a2  *= (2.0/nnz);
	b2  *= (2.0/nnz);
	a4  *= (2.0/nnz);
	b4  *= (2.0/nnz);

/* -- if contour is incomplete cos(4e) is totally screwed up. */

	if(incflag) a4 =0.0;

/* -- solve for new ecc, center, angle 
 * note that b0 has not been sky subtracted, so don't normalize by it*/
	profile[k]->radius = radius;
	profile[k]->xc = xc;
	profile[k]->yc = yc;
	profile[k]->ang = ang;
	profile[k]->ecc = ecc;
	profile[k]->intensity = b0;
	profile[k]->a0  = a0;
	profile[k]->b0  = b0;
	profile[k]->a1  = a1;
	profile[k]->b1  = b1;
	profile[k]->a2  = a2;
	profile[k]->b2  = b2;
	profile[k]->a4  = a4;
	profile[k]->b4  = b4; 
printf("%6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf\
\n", radius, xc, yc, ang, ecc, b0, a4, b4, a2, b2);

	if(iter == 0 && k <= nderiv) {
/* -- update xc,yc,ang,ecc using numerical derivatives */
		deriv = profile[k]->intensity + profile[k]->a2 - 
		   (profile[k-1]->intensity + profile[k-1]->a2);

/* kent's code has icenter=0 here */
		xyaeinc(&xc,&yc,&ang,&ecc,deriv,radius,a1,b1,a2,b2,0,1.0);
		} /* end if */

	} /* end for(k ... */

/* -- iterate here ... */
	
   iter = 1;

/* See if we want to update parameters */
   printf ("Iterate ? ");

/* The C I/O facility truly sucks.  The fgets function would be ideal to read
   just 1 character, but it doesn't work properly when called in succession.
   So I will just read a buffer and hope that the user doesn't type more than
   80 characters
*/

   gets (yn);
   if (yn[0] != 'y' && yn[0] != 'Y') break;

/* -- update ellipse parameters without averaging steps 
 */
	
   if (nderiv >= l1) {

	for (k = l1; k <= nderiv; k++) {
		deriv = profile[k]->intensity + profile[k]->a2 - 
		   (profile[k-1]->intensity + profile[k-1]->a2);
		
		profile[k]->deriv = deriv;
		}
	}
   else {

	if(nderiv < l2) {
/* -- don't add cos(2e) term when computing derivatives; try to cut down on
 * -- number of "deriv>0" problems.
*/	
	   for(k = nderiv+1; k<= l2; k++) 
		pro_file[k-nderiv-1] = log(profile[k]->intensity -
		   sky);

	   npts = l2 - nderiv;

	   ppolyfit(pro_file,npts,npts/8+2);

	   for(k = nderiv+1; k<= l2; k++) {
		pfunc(k-nderiv,&func);
		pderiv(k-nderiv,&deriv);
printf(" func is %lf deriv is %lf \n",func,deriv);
		deriv *= (exp(func));
		profile[k]->deriv = deriv;
		}

	   } /*end if nderiv < l2*/
	} /*end if nderiv > l1 */
 
   for(k=l1; k<= l2; k++) {
	radius = profile[k]->radius;
	xc0 = profile[k]->xc; 
	yc0 = profile[k]->yc;
	ang0 = profile[k]->ang;
	ecc0 = profile[k]->ecc;
	deriv = profile[k]->deriv;
	a11 = profile[k]->a1;
	b11 = profile[k]->b1;
	a22 = profile[k]->a2;
	b22 = profile[k]->b2;

	xyaeinc(&xc0,&yc0,&ang0,&ecc0,deriv,radius,a11,b11,a22,b22,0,f_inc);


	profile[k]->xc = xc0; 
	profile[k]->yc = yc0;
	profile[k]->ang = ang0;
	profile[k]->ecc = ecc0;

printf("%6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf\n",
radius, xc0, yc0, ang0, ecc0, profile[k]->intensity);
	}
   }   /* End of while loop */

   shFree (pro_file);
   shFree (profile);
   return IMG_SUCCESS;
}


/****************************************************************************
*****************************************************************************/
static void 
xyaeinc(double *xc, double *yc, double *ang, double *ecc, double deriv,
	     double radius, double a1, double b1, double a2, double b2,
	     int icenter, double finc)
{

double aa1=0.0,aa2 =0.0,bb1=0.0,bb2=0.0,f0=0.0,dxt=0.0,dyt=0.0,dx=0.0,dy=0.0;
double f2=0.0,psi=0.0,cs=0.0,sn=0.0,arg=0.0,temp_arg=0.0;	

	aa1 = a1/(deriv * radius);
	aa2 = a2/(deriv * radius);
	bb1 = b1/(deriv * radius);
	bb2 = b2/(deriv * radius);

	f0 = 1./(sqrt(1.0 - pow(*ecc,2.0)));

	if(aa2 > 0.0 && f0 == 1.0) {
		*ang +=90.0;
		aa2 *= -1.0;
		 		  }

	aa1  *= finc;
	aa2  *= finc;
	bb1  *= finc;
	bb2  *= finc;

		if(aa2 > 0) 
			f2 = pow(f0,2.0)/(1.0 + 4 * aa2);
		else
			f2 = pow(f0,2.0)*(1.0 - 4 * aa2);

	if (f2 < 1.0) { f2 =1.0;}
	if (f2 >=1.0) {*ecc = sqrt(1.0 - 1.0/f2);}
	if (f2 == 1.0) {
		if(bb2 == 0.0) psi =0.0;
		if(bb2 != 0.0 && aa2 == 0.0) psi = pi/2.;
		if(bb2 != 0.0 && aa2 != 0.0) psi = 0.5*atan2(bb2,aa2);
			}
		else
		       {
		temp_arg = 4.0 * f0 * bb2 /(1.0 -f2);
		arg = max(temp_arg,-1.0);
		temp_arg = min(arg,1.0);
		psi = 0.5 * asin(temp_arg);
			}



	*ang += (psi*180.0/pi);


if(icenter == 1) {
	dxt = radius * aa1;
	dyt = radius/sqrt(f2) * bb1;

	dx = -dxt;
	dy = -dyt;
	
	if(dx < -1.0) dx = -1.0;
	if(dx > 1.0)  dx =  1.0;

	if(dy < -1.0) dy = -1.0;
	if(dy > 1.0)  dy =  1.0;

/* -- rotate dx, dy by -ang degrees */

	cs = cos((*ang)*pi/180.0);
	sn = sin((*ang)*pi/180.0);
	
				*xc += dx*cs - dy*sn;
				*yc += dx*sn + dy*cs;
	    }

		*ang = fmod(*ang,180.);
		
		if(*ang < -90.) *ang += 180.;
		if(*ang > 90.0) *ang -= 180.;

		return;
}


/****************************************************************************/

static double  xmax=0.0;
static double  cmat[2080],rhs[64],prt[64],duma[64],dumb[64];
static int dumc[64];

static void
ppolyfit(double *data,int n,int norder)
{
   register int i,j,l,k;
   double res=0.0,x=0.0;

   nterm = norder;

   if(n < norder) return;

   xmax = (double) n;

   l=0;
   for (j=0; j<nterm; j++) {
	rhs[j]=0.;
	for (k=0; k<=j; k++) {
	   cmat[l]=0.;
	   l++;
	   }
	}
	
   for(i=0; i<n; i++) {
	res = data[i];

	if(res == 0.0) continue;
		
	x = ((double) i)/xmax;
	prt[0]=1.0;
		
	for(j=1; j<nterm; j++) {
		prt[j] = pow(x,j);
		}

	l = 0;

	for(j=0; j<nterm; j++) {
		rhs[j] += (prt[j]*res);
		
		for(k=0; k<=j; k++) {
		   cmat[l] += (prt[j] * prt[k]);
		   l++;
		   }
		}
	}
   syminv(cmat,rhs,nterm,duma,dumb,dumc);
}


/************************************************************************/	
static void
 pfunc(int jx, double *result)
{
double x=0.0;
register int i;

	x = ((double) jx)/xmax;

	*result = rhs[0];

	if(nterm <= 1) return;

		for(i=1;i<nterm;i++)
			*result += (pow(x,(double) i)*rhs[i]);

return;
}


/*****************************************************************************/
static void
 pderiv(int jx, double *result)
{
double x=0.0;
register int i;

	x = ((double) jx)/xmax;
	
	*result = rhs[1]/xmax;

		if (nterm <= 2) return;

	for(i=2;i<nterm;i++)
	*result += (((double) i) * pow(x,(double) (i-1)) * rhs[i]/xmax);
return;
}
