#include <math.h>
#include <string.h>
#include "dervish.h"
#include "imgProto.h"

/* Standard smoothing in place */

 IMG_CODE imgSmooth (REGION *reg) {
    int nrow, ncol;

 /* We convert everything to floats to avoid overflows, so make all buffers
  * double just to be safe.
 */
    double *buf1;
    double *buf2;
    double *buf3;
    double bufout;

 /* Pointer to function to return double pixel value */
    double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg);

 /* And to set the pixel */
    void (*dblpixset)(REGION *, int, int, double) = regPixDblFuncSet(reg);
    int i, j;

    nrow = reg->nrow;
    ncol = reg->ncol;
    if (nrow < 2 || ncol < 2) return IMG_SUCCESS;

    buf1 = (double *)shMalloc((ncol+2)*sizeof(double));
    buf2 = (double *)shMalloc((ncol+2)*sizeof(double));
    buf3 = (double *)shMalloc((ncol+2)*sizeof(double));

 /* Make buffer ncol+2 wide.  Read 1st row and duplicate; then step through
  * remaining rows.
 */

 /* buf1, 2, 3 have raw data, bufout has filtered data */
    for (j=0; j<ncol; j++) {
	 buf3[j+1] = dblpix(reg, 0, j);
	 }
    buf3[0] = buf3[1];
    buf3[ncol+1] = buf3[ncol];
    memcpy (buf2, buf3, sizeof(double)*(ncol+2));

    for (i=0; i<nrow-1; i++) {
	 memcpy (buf1, buf2, sizeof(double)*(ncol+2));
	 memcpy (buf2, buf3, sizeof(double)*(ncol+2));
	 for (j=0; j<ncol; j++) {
	    buf3[j+1] = dblpix(reg, i+1, j);
	    }
	 buf3[0] = buf3[1];
	 buf3[ncol+1] = buf3[ncol];

	 for (j=0; j<ncol; j++) {
	    bufout = (4.*buf2[j+1] + buf2[j] + buf2[j+2] + \
		 buf1[j+1] + buf3[j+1])/8.;
	    dblpixset(reg, i, j, bufout);
	    }
	 }

 /* Need to compute last row */
    memcpy (buf1, buf2, sizeof(double)*(ncol+2));
    memcpy (buf2, buf3, sizeof(double)*(ncol+2));

    for (j=0; j<ncol; j++) {
	 bufout = (4.*buf2[j+1] + buf2[j] + buf2[j+2] + \
	    buf1[j+1] + buf3[j+1])/8.;
	 dblpixset(reg, nrow-1, j, bufout);
	 }

 /* Free up malloced storage */
    shFree(buf1);
    shFree(buf2);
    shFree(buf3);
    return IMG_SUCCESS;
    }

/****************************************************************************/

/* Procedure to fix up bleed trails */

 IMG_CODE imgBleedFix (REGION *reg, int thresh) {
    int nrow, ncol;
    int pix, pix1, pix2;
    int i, j, k, c1, c2;

 /* Pointer to function to return int pixel value */
    int (*intpix)(REGION *, int, int) = regPixIntFunc (reg);

 /* And to set the pixel */
    void (*intpixset)(REGION *, int, int, int) = regPixIntFuncSet(reg);

   nrow = reg->nrow;
   ncol = reg->ncol;

   if (nrow*ncol == 0) return IMG_SUCCESS;

   for (i=0; i<nrow; i++) {
	for (j=1; j<ncol-1; j++) {
	   pix = intpix(reg, i,j);
	   if (pix < thresh) continue;

/* I found a pixel that is above the threshold.  I will look for low and
 * high column numbers of 1st pixels around this one that are below threshold
 * and use them to interpolate.
*/
	   c1 = j-1;
	   c2 = ncol-1;
	   for (k=j; k<ncol; k++) {
		if (intpix(reg, i, k) < thresh) {
		   c2 = k;
		   break;
		   }
		}
	   pix1 = intpix(reg, i, c1);
	   pix2 = intpix(reg, i, c2);
	   if (pix1 >= thresh) pix1 = pix2;
	   if (pix2 >= thresh) pix2 = pix1;
	   for (k=c1; k<=c2; k++) {
		pix = pix1 + (pix2-pix1)*(k-c1)/((double)c2-(double)c1) + 0.5;
		intpixset(reg, i, k, pix);
		}
	   }
	}
   return IMG_SUCCESS;
   }
