/*****************************************************************************
******************************************************************************
**
** FILE:
**      tclMethods.c
**
** ABSTRACT:
**      This file contains tcl support code tp convert regions to GIFs.
**	NOTE: At the moment, the gifs will have 64 colors, greyscale only.
**	This is hardwired into several locations.
**	NO MORE! (I hope)
**
** ENTRY POINT          SCOPE   DESCRIPTION
** -------------------------------------------------------------------------
** tclGif		local	Convert region to gif
**
** ENVIRONMENT:
**      ANSI C.
**
** REQUIRED PRODUCTS:
**      dervish
**      ftcl
**
** AUTHORS:
**      Creation date:  Jan 1994
**      Steve Kent
**
******************************************************************************
******************************************************************************
*/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "dervish.h"
#include "imgProto.h"
#include "gif8.h"

/*============================================================================
**============================================================================
**
** LOCAL MACROS, DEFINITIONS, ETC.
**
**============================================================================
*/

/*----------------------------------------------------------------------------
**
** GLOBAL VARIABLES
*/
static char *module = "tclGif";             /* name of this set of code */


/*============================================================================
**============================================================================
**
** ROUTINE: scale
**
** DESCRIPTION:
**      Rescale pixels down to byte size
**
** RETURN VALUES:
**	None
**
**
**============================================================================
*/

static void scale (REGION *reg, unsigned char *pixels, double min, double max,
   int depth) {

   int i,j,n;
   int pix;

   static int flag=0;
   static unsigned char lookup[65536];

/* Pointer to function to return integer pixel value */
   int (*intpix)(REGION *, int, int) = regPixIntFunc (reg);
/* Pointer to function to return double pixel value */
/*   double (*dblpix)(REGION *, int, int) = regPixDblFunc (reg); */

/* To access a pixel as a double: dblpix (reg, row, col) */
/* To access a pixel as an int:   intpix (reg, row, col) */

   if (flag == 0) {
	for (i=0; i<65536; i++) {
	   lookup[i] = (pow(2,depth)-1)*sqrt(((float)i)/65535.);
	   }
	flag=0;
	}

   n=0;
   for (i=0; i<reg->nrow; i++) {
        for (j=0; j<reg->ncol; j++) {
           pix = 65535*(intpix (reg, i, j)-min)/(max-min);
	   if (pix < 0) pix=0;
	   if (pix >65535) pix=65535;
	   pixels[n] = lookup[pix];
	   n++;
           }
        }
   return;
   }

/*============================================================================
**============================================================================
**
** ROUTINE: tclGif
**
** DESCRIPTION:
**      Convert a REGION to a gif file
**
** RETURN VALUES:
**      TCL_OK -   Successful completion.
**      TCL_ERROR -   Failed miserably.
**
**
**============================================================================
*/
static char *tclGif_use =
"USAGE: regToGif <hndl-REGION> <file-name> <min> <max> [<depth>]";
static char *tclGif_hlp =
   "Convert a REGION to a GIF file with user supplied scaling.";

static int
tclGif(
          ClientData clientData,
          Tcl_Interp *interp,
          int argc,
          char **argv
          )
{
   REGION *reg = NULL;
   unsigned char *pixels;
   double min, max;
   int depth;

   shErrStackClear();

   if (argc != 5 && argc != 6) {
        Tcl_SetResult(interp,tclGif_use,TCL_STATIC);
        return(TCL_ERROR);
        }

   if (shTclAddrGetFromName (interp, argv[1], (void *)&reg, "REGION") != TCL_OK)
        return(TCL_ERROR);
   if (Tcl_GetDouble(interp, argv[3], &min) != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble(interp, argv[4], &max) != TCL_OK) return(TCL_ERROR);
   if (argc == 6) {
	if (Tcl_GetInt(interp, argv[5], &depth) != TCL_OK) return(TCL_ERROR);
   } else {
	depth=6;
	}
   if (depth<2) depth=2;
   if (depth>7) depth=7;

/* REGION must be continuous pixels; should really check here, but I won't
 * yet */

/* Rescale the region pixels down to unsigned bytes with specified scaling
   factors.  I will compute the square root as well.
*/
   
   pixels = (unsigned char *)shMalloc(reg->nrow*reg->ncol*sizeof(char));
   scale (reg, pixels, min, max, depth);

   gif8(pixels, reg->ncol, reg->nrow, argv[2], depth);

   shFree(pixels);

   return TCL_OK;
}/*============================================================================
**============================================================================
**
** ROUTINE: shTclMethodsDeclare
**
** DESCRIPTION:
**      Declare commands to TCL
**
**
**============================================================================
*/
void
imgTclGifDeclare(Tcl_Interp *interp)
{

   shTclDeclare(interp,"regToGif", tclGif,
                (ClientData) 0,
                (Tcl_CmdDeleteProc *)NULL,
                module, tclGif_hlp, tclGif_use);
}

