/* imgVec.c --
*  Mar 1999 Steve Kent
*
* Tcl functions defined in this module:
*
* imgVec
*
*/

#include "tcl.h"
#include "dervish.h"
#include "shGeneric.h"
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "imgProto.h"
#include "img_msg_c.h"
#include "imgProf.h"

/************************** imgVec - the action routine **********************/

/* Compute a vector profile.  Specify centroid, pos angle, width,
 * and number of steps.
 * We cover -nstep to +nstep, width -width to +width
 * Output to a PROF array, although this is not really optimal.
*/

IMG_CODE imgVec (REGION *reg, double rowc, double colc, double angdeg, \
	int width, int nstep, CHAIN *proflist) {

   PROF *prof;
   int what;
   int i,j;
   int npix;
   double r,c;
   double pix, pixsum;
   double pi=3.14159265358979;
   double ang = pi*angdeg/180.;

   if (nstep < 0 || width < 0) return IMG_SUCCESS;

/* Position angle convention: 0 is increasing row direction; 90 is increasing
 * column direction.  Doesn't match astronomical convention for SDSS CCD pics.
*/

   for (i=-nstep; i<=nstep; i++) {
	prof = (PROF *)shGenericNew ("PROF");
	if (prof == NULL) return IMG_GENERIC_ERROR;
	what = shChainElementAddByPos (proflist, prof, "PROF", TAIL, AFTER);
	prof -> rad = i;
	prof -> row = rowc + i*cos(ang);
	prof -> col = colc + i*sin(ang);
	prof -> ang = ang;
	prof -> eps = 0.;
	pixsum=0.;
	npix=0;
	for (j=-width; j<=width; j++) {
	   r = rowc + i * cos (ang) - j * sin(ang) ;
	   c = colc + i * sin (ang) + j * cos(ang) ;
	   pix = pixbin(reg, r, c);
	   if (pix != 0.) {
		npix++;
		pixsum = pixsum + pix;
		}
	   }
	if (npix != 0) {prof -> cnts = pixsum / npix;}
	   else {prof -> cnts = 0.;}
	}
   return IMG_SUCCESS;
   }
