/* maskdriv.c -- Fortran callable PGPLOT driver for a MASK region */

#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "dervish.h"

static MASK *mask;
static char color;
static int imax=0;
static int imin=0;
static int jmax=0;
static int jmin=0;
static char *mask_type = "MASK";
static char namebuf[81];


void redriv_ (int *ifunc, float rbuf[], int *nbuf, char *chr, int *lchr,
	int len) {

/*PGPLOT driver for drawing plots into a MASK region */
	int ncol=8;
	int resx=100;	/* Resolution is dependent on where mask will get
			   displayed */
	int resy=100;
	Tcl_Interp *interp;
	int ierr;
	int nstep;
	int i, j;
	float stepx, stepy;
	int i0, j0, i1, j1;
	int ix, iy;
	char colorindex[] = {1, 2, 4, 8, 16, 32, 64, -128};
	int ic;

         switch (*ifunc) {

/*--- IFUNC=1, Return device name ---------------------------------------*/

         case 1 :
              strncpy( chr, mask_type, len ) ; /* 14Oct91*/
              *lchr = strlen( mask_type ) ;
              for ( i = *lchr ; i < len ; i++ )
                  chr[i] = ' ';
              break;

/*--- IFUNC=2, Return physical min and max for plot device, and range
               of color indices -----------------------------------------*/

         case 2 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              rbuf[4] = 0.0;
              rbuf[5] = (float) ncol;
              *nbuf = 6;

              break;

/*--- IFUNC=3, Return device resolution ---------------------------------*/

         case 3 :

              rbuf[0] = (float) resx;
              rbuf[1] = (float) resy;
              rbuf[2] = 1.0;             /* Device coordinates per pixel */
              *nbuf = 3;

              break;

/*--- IFUNC=4, Return misc device info ----------------------------------*/

         case 4 :

              chr[0] = 'N'; /* no interactive device */
              chr[1] = 'N'; /* no cursor is available */
              chr[2] = 'N'; /* no dashed lines */
              chr[3] = 'N'; /* no area fill available */
              chr[4] = 'N'; /* no fat lines */
              chr[5] = 'N'; /* no rectangle fill available */
              chr[6] = 'N'; /* no line of pixels available */
              chr[7] = 'N'; /* not used */
              chr[8] = 'N'; /* not used */
              chr[9] = 'N'; /* not used */
              *lchr = 10;

              break;

/*--- IFUNC=5, Return default file name ---------------------------------*/

         case 5 :

              strcpy (chr, "");  /* Default name is NULL */
              *lchr = 0;

              break;

/*--- IFUNC=6, Return default physical size of plot ---------------------*/

         case 6 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              *nbuf = 4;

              break;

/*--- IFUNC=7, Return misc defaults -------------------------------------*/

         case 7 :

              rbuf[0] = 1.0;
              *nbuf = 1;

              break;

/*--- IFUNC=8, Select plot ----------------------------------------------*/

         case 8 :

              break;

/*--- IFUNC=9, Open workstation -----------------------------------------*/

         case 9 :
	      strncpy (namebuf, chr, len);
	      namebuf[len]='\0';
/* Create dummy interp structure */
		interp = Tcl_CreateInterp();
		ierr = shTclAddrGetFromName (interp, namebuf, (void **)&mask, \
			"MASK");
		Tcl_DeleteInterp(interp);
		if (ierr != TCL_OK) {
			rbuf[1]=0.;	/* I think this is return error code */
			break;
			}
		imax = mask->ncol;
		jmax = mask->nrow;
/* Initial default color */
		color = 1;
		rbuf[1]=1;
              break;

/*--- IFUNC=10, Close workstation ---------------------------------------*/

         case 10 :
		mask=NULL;
		strcpy (namebuf,"");
		break;

/*--- IFUNC=11, Begin picture -------------------------------------------*/

         case 11 :

/* Create dummy interp structure */
		interp = Tcl_CreateInterp();
		ierr = shTclAddrGetFromName (interp, namebuf, (void **)&mask, \
			"MASK");
		Tcl_DeleteInterp(interp);
		if (ierr != TCL_OK) {
			break;
			}
		for (i=0; i<mask->ncol; i++) {
		   for (j=0; j<mask->nrow; j++) {
			mask->rows[j][i] = (char) NULL;
			}
		   }
		break;

/*--- IFUNC=12, Draw line -----------------------------------------------*/

         case 12 :

/* Create dummy interp structure */
		interp = Tcl_CreateInterp();
		ierr = shTclAddrGetFromName (interp, namebuf, (void **)&mask, \
			"MASK");
		Tcl_DeleteInterp(interp);
		if (ierr != TCL_OK) {
			break;
			}
         /* Translate input */

              i0 = (int) (rbuf[0] + 0.5) + imin;
              j0 = (int) (rbuf[1] + 0.5) + jmin;
              i1 = (int) (rbuf[2] + 0.5) + imin;
              j1 = (int) (rbuf[3] + 0.5) + jmin;

         /* Draw the line */

        nstep = sqrt(pow((float)(i1-i0),2) + pow((float)(j1-j0),2))+0.5;
        if (nstep != 0) {
	   stepx = (float)(i1-i0)/(float) nstep;
	   stepy = (float)(j1-j0)/(float) nstep;
	} else {
	   stepx = 0.;
	   stepy = 0.;
	   }
	for (i = 0; i<=nstep; i++) {
	   ix = i0 + i*stepx - 0.5;
	   iy = jmax - (j0 + i*stepy - 0.5);
	   if (ix < 0)   ix = 0;
	   if (iy < 0)   iy=0;
	   if (ix > mask->ncol-1) ix = mask->ncol-1;
	   if (iy > mask->nrow-1) iy = mask->nrow-1;
	   mask->rows[iy][ix] = mask->rows[iy][ix] | color;
	   }
	break;

/*--- IFUNC=13, Draw dot ------------------------------------------------*/

         case 13 :

/* Create dummy interp structure */
		interp = Tcl_CreateInterp();
		ierr = shTclAddrGetFromName (interp, namebuf, (void **)&mask, \
			"MASK");
		Tcl_DeleteInterp(interp);
		if (ierr != TCL_OK) {
			break;
			}
         /* Translate input */

              i0 = (int) (rbuf[0] - 0.5) + imin;
              j0 = jmax - ((int) (rbuf[1] - 0.5) + jmin);
	 	if (i0 < 0) i0 = 0;
		if (j0 < 0) j0 = 0;
		if (i0 > mask->ncol-1) i0 = mask->ncol-1;
		if (j0 > mask->nrow-1) j0 = mask->nrow-1;

         /* Draw the point */

	   mask->rows[j0][i0] = mask->rows[j0][i0] | color;

              break;

/*--- IFUNC=14, End picture ---------------------------------------------*/

         case 14 :

              break;

/*--- IFUNC=15, Select color index --------------------------------------*/

         case 15 :

         /* Translate input */

              ic = (int) (rbuf[0] + 0.5);

         /* Check input for proper range */

              if (ic < 1 || ic > ncol) {
                 ic = 1;
                 rbuf[0] = (float) ic;
                 }

         /* Change the color index - handle monochrome properly */

	      color = colorindex[ic-1];
              break;

/*--- IFUNC=16, Flush buffer. -------------------------------------------*/

         case 16 :

              break;

/*--- IFUNC=17, Read cursor. --------------------------------------------*/

         case 17 :

              break;

/*--- IFUNC=18, Erase alpha screen. -------------------------------------*/

         case 18 :

              break;

/*--- IFUNC=19, Set line style. -----------------------------------------*/

         case 19 :

              break;

/*--- IFUNC=20, Polygon fill. -------------------------------------------*/

         case 20 :

              break;

/*--- IFUNC=21, Set color representation. -------------------------------*/

         case 21 :

         /* This is ignored */

              break;

/*--- IFUNC=22, Set line width. -----------------------------------------*/

         case 22 :

              break;

/*--- IFUNC=23, Escape --------------------------------------------------*/

         case 23 :

              break;

/*--- IFUNC=24, Rectangle Fill. -----------------------------------------*/

         case 24 :

              break;

/*--- IFUNC=25, ---------------------------------------------------------*/

         case 25 :

              break;

/*--- IFUNC=26, Line of pixels ------------------------------------------*/

         case 26 :

              break;

/*--- IFUNC=?, ----------------------------------------------------------*/

         default :

              break;

              } /* End of switch */

         return;

} /* End of maskdriv */
