#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* Process data from an image via piping. Input must be unsigned int*/
/* Pass dimension information by argc mechanism; bytes come by stdin */
/* Return dithered image */

int main(int argc, char **argv) {
   int nrow, ncol, nbyte;
   unsigned short *buf;
   double pmin, pmax;
   int nbit;
   float *err0, *err1;
   unsigned char *out;
   float scale;

   int i, j, k;

   if (argc != 7) {
	fprintf (stderr, "Usage: regProc <nrow> <ncol> <byte-per-pix> \
<min> <max> <nbit>\n");
	exit(1);
	}

   nrow = atoi(argv[1]);
   ncol = atoi(argv[2]);
   nbyte = atoi(argv[3]);
   pmin = atof(argv[4]);
   pmax = atof(argv[5]);
   nbit = atoi(argv[6]);
   if (nbit < 1) exit(0);
   if (nbit > 8) nbit=8;
   scale = pow(2,nbit)-1;

   if (nrow*ncol*nbyte == 0) exit(0);

/* Allocate scratch storage for error arrays.  Pad by 2 */
   err0 = (float *)malloc((ncol+2)*sizeof(float));
   err1 = (float *)malloc((ncol+2)*sizeof(float));

/* Increment err0, err1 to point to 2nd element in each array to allow
 * negative indices later on. */

   err0++;
   err1++;

   for (i=-1; i<=ncol; i++) {
	err0[i] = .5;
	}

/* Allocate space for input, output rows */
   buf = (unsigned short *)malloc(ncol*sizeof(unsigned short));
   out = (unsigned char *)malloc(ncol*sizeof(char));

   for (i=0; i<nrow; i++) {
	fread (buf, 2*ncol, 1, stdin);
	for (j=0; j<ncol; j++) {
	   err0[j] = err1[j];
	   }
	for (j=0; j<ncol; j++) {
/* I rescale input to range 0 to nbit; then add weighted error using
 * Floyd Steinberg algorithm.  Result is truncated to next lowest integer;
 * remainder is kept as error for future pixels.
*/

	   err1[j] = (buf[j]-pmin)*scale/(pmax-pmin) + (7.*err1[j-1] +
		err0[j-1] + 5.*err0[j] + 3.*err0[j+1])/16.;

	   if (err1[j] > scale) err1[j] = scale;
	   out[j] = floor(err1[j]);
	   err1[j] -= out[j];
	   }
	fwrite (out, ncol, 1, stdout);
	fflush (stdout);
	}

/* Free up malloced storage */
   err0--;
   err1--;
   free(err0);
   free(err1);
   free(buf);
   free(out);
   fclose(stdout);
   exit(0);
   }
