#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* Process data from an image via piping. Input must be unsigned int*/
/* Pass dimension information by argc mechanism; bytes come by stdin */
/* Return image scaled by sqrt algorithm */
/* Algorithm is out = sqrt((in-thresh)*scale) + bias */

int main(int argc, char **argv) {
   int nrow, ncol, nbyte;
   unsigned short *buf;
   double min, max, bias;
/* double b, icross, con; */
   unsigned short trans[65536];

   int i, j;
   double temp;

   if (argc != 7) {
	fprintf (stderr, "Usage: regSqrt <nrow> <ncol> <nbyte> <min> \
<max> <bias>\n");
	exit(1);
	}

   nrow = atoi(argv[1]);
   ncol = atoi(argv[2]);
   nbyte = atoi(argv[3]);
   min = atof(argv[4]);
   max = atof(argv[5]);
   bias = atof(argv[6]);

   if (nrow*ncol*nbyte == 0) exit(0);

/* Create lookup table.  Rescale to 0 to 255 */

/* Use Peter Kunszt technique of converting to log table above a
 * certain threshold so we saturate more smoothly */
/*   b = 255./log(2.*max-min); */
/*   icross = min + pow(2.*b/255.,2)*(max-min); */
/*   con = 255.*sqrt((icross-min)/(max-min)) - b*log(icross-min); */
/*   fprintf (stderr, "icross = %f, con = %f\n", icross, con); */

   for (i=0; i<65536; i++) {
	temp = (i-min)/(max-min);
	if (temp < 0) temp=0;
	temp = 255.*sqrt(temp) + bias;

	/*	if (i > icross && i > min) {
	   temp = b*log(i-min) + con;
	   } */
	if (temp < 0) temp=0;
	if (temp > 255.) temp = 255.;
	trans[i] = temp;
	}

   buf = (unsigned short *)malloc(ncol*sizeof(unsigned short));

   for (i=0; i<nrow; i++) {
	fread (buf, 2*ncol, 1, stdin);
	for (j=0; j<ncol; j++) {
	   buf[j] = trans[buf[j]];
	   }
	fwrite (buf, 2*ncol, 1, stdout);
	fflush (stdout);
	}

/* Free up malloced storage */
   free(buf);
   fclose(stdout);
   exit(0);
   }
