/* tkdriv.c -- Fortran callable PGPLOT driver for a Tk canvas widget */
/* This routine shadow gfdriv, which is a driver for a GraphOn device
 * that we don't need
*/

#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "dervish.h"

static int imax=0;
static int imin=0;
static int jmax=0;
static int jmin=0;
static int ic;
static int lw;
static char *tk_type = "REG";
static char namebuf[100];
static int ncolor=16;


/**********************************************************************/
/* This function will override the pgplot version.  The device type will be
 * TK; intialize by calling pgBegin widget/TK  where widget is an existing
 * tk Canvas widget.  By making the user create it, it is possible to plot
 * into multiple tkWindows in a session.  The alternative would be to create
 * the tk widget in the pgBegin command, but we would want to destroy it
 * upon calling pgEnd for symmetry.
*/

void x2driv_ (int *ifunc, float rbuf[], int *nbuf, char *chr, int *lchr,
	int len) {

/*PGPLOT driver for drawing plots into a MASK region */
	int resx=100;	/* Resolution is dependent on where mask will get
			   displayed */
	int resy=100;
	int color;

        int nstep;
        int i;
        float stepx, stepy;
        int i0, j0, i1, j1;
        int ix, iy;
	int ierr;

	Tcl_Interp *interp = NULL;
	REGION *reg=NULL;

         switch (*ifunc) {

/*--- IFUNC=1, Return device name ---------------------------------------*/

         case 1 :
              strncpy( chr, tk_type, len ) ; /* 14Oct91*/
              *lchr = strlen( tk_type ) ;
              for ( i = *lchr ; i < len ; i++ )
                  chr[i] = ' ';
              break;

/*--- IFUNC=2, Return physical min and max for plot device, and range
               of color indices -----------------------------------------*/

         case 2 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              rbuf[4] = 0.0;
              rbuf[5] = (float) ncolor;
              *nbuf = 6;

              break;

/*--- IFUNC=3, Return device resolution ---------------------------------*/

         case 3 :

              rbuf[0] = (float) resx;
              rbuf[1] = (float) resy;
              rbuf[2] = 1.0;             /* Device coordinates per pixel */
              *nbuf = 3;

              break;

/*--- IFUNC=4, Return misc device info ----------------------------------*/

         case 4 :

              chr[0] = 'I'; /* Interactive device */
              chr[1] = 'N'; /* no cursor is available */
              chr[2] = 'N'; /* no dashed lines */
              chr[3] = 'N'; /* no area fill available */
              chr[4] = 'N'; /* no fat lines */
              chr[5] = 'N'; /* no rectangle fill available */
              chr[6] = 'N'; /* no line of pixels available */
              chr[7] = 'N'; /* not used */
              chr[8] = 'N'; /* not used */
              chr[9] = 'N'; /* not used */
              *lchr = 10;

              break;

/*--- IFUNC=5, Return default file name ---------------------------------*/

         case 5 :

              strcpy (chr, "");  /* Default name is NULL */
              *lchr = 0;

              break;

/*--- IFUNC=6, Return default physical size of plot ---------------------*/

         case 6 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              *nbuf = 4;

              break;

/*--- IFUNC=7, Return misc defaults -------------------------------------*/

         case 7 :

              rbuf[0] = 1.0;
              *nbuf = 1;

              break;

/*--- IFUNC=8, Select plot ----------------------------------------------*/

         case 8 :

              break;

/*--- IFUNC=9, Open workstation -----------------------------------------*/

         case 9 :
/* Initialize colors the 1st time through */

/* Save name of widget in namebuf */
	strncpy (namebuf, chr, len);
	for (i=0; i<len; i++) {
	   if (namebuf[i] == ' ') namebuf[i] = '\0';
	   }
	namebuf[len]='\0';
/* Check to see if the region exists. */
	interp = Tcl_CreateInterp();
	ierr=shTclAddrGetFromName(interp, namebuf, (void *)&reg, "REGION");
	Tcl_DeleteInterp(interp);
	if (ierr != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}

/* Get dimensions */
/* Note that i maps to column #, j maps to row # */
	jmax = reg->nrow;
	imax = reg->ncol;
/* Initial default color */
	ic = 1;
	rbuf[1]=1;
/* Line weight */
	lw = 1;
	break;

/*--- IFUNC=10, Close workstation ---------------------------------------*/

         case 10 :
/* We don't clear the canvas; just reset namebuf */
	strcpy (namebuf,"");
	break;

/*--- IFUNC=11, Begin picture -------------------------------------------*/

         case 11 :

/* Check to see if the region exists. */
	interp = Tcl_CreateInterp();
	ierr=shTclAddrGetFromName(interp, namebuf, (void *)&reg, "REGION");
	Tcl_DeleteInterp(interp);
	if (ierr != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}

/* Clear the region*/
/*	shRegSetWithDbl (0., reg); */
	break;

/*--- IFUNC=12, Draw line -----------------------------------------------*/

         case 12 :

/* Check to see if the region exists. */
	interp = Tcl_CreateInterp();
	ierr=shTclAddrGetFromName(interp, namebuf, (void *)&reg, "REGION");
	Tcl_DeleteInterp(interp);
	if (ierr != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}

/* Decision time: pgPlot has x, y origin at lower left.  saoDisplay
 * places it at
 * upper left.  I will use pgPlot convention, which means that pgPlot
 * coordinates don't map onto region coordinates (such as would arise from
 * manipulating the widget directly
*/
         /* Translate input */

/* Reverse y (vertical) coordinates */
              i0 = (int)(rbuf[0] + imin);
              j0 = (int)(rbuf[1] + jmin);
              i1 = (int)(rbuf[2] + imin);
              j1 = (int)(rbuf[3] + jmin);

/* Draw the line */

	color = pow(2,ic) - 1;
        nstep = sqrt(pow((float)(i1-i0),2) + pow((float)(j1-j0),2))+0.5;
        if (nstep != 0) {
           stepx = (float)(i1-i0)/(float) nstep;
           stepy = (float)(j1-j0)/(float) nstep;
        } else {
           stepx = 0.;
           stepy = 0.;
           }
        for (i = 0; i<=nstep; i++) {
           ix = i0 + i*stepx;
           iy = jmax - (j0 + i*stepy);
           if (ix < 0)   ix = 0;
           if (iy < 0)   iy=0;
           if (ix > reg->ncol-1) ix = reg->ncol-1;
           if (iy > reg->nrow-1) iy = reg->nrow-1;
	   shRegPixSetWithDbl(reg, iy, ix, color);
           }
	break;

/*--- IFUNC=13, Draw dot ------------------------------------------------*/

         case 13 :

/* Check to see if the region exists. */
	interp = Tcl_CreateInterp();
	ierr=shTclAddrGetFromName(interp, namebuf, (void *)&reg, "REGION");
	Tcl_DeleteInterp(interp);
	if (ierr != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}

/* Reverse y (vertical) coordinates */
              i0 = rbuf[0] + imin;
              j0 = jmax - (int)(rbuf[1] + jmin);

         /* Draw the point */
	   if (i0 < 0) i0 = 0;
	   if (j0 < 0) j0 = 0;
	   if (i0 > reg->ncol-1) i0 = reg->ncol-1;
	   if (j0 > reg->nrow-1) j0 = reg->nrow-1;

	color = pow(2,ic) - 1;
	shRegPixSetWithDbl (reg, j0, i0, color);
	break;

/*--- IFUNC=14, End picture ---------------------------------------------*/

         case 14 :

              break;

/*--- IFUNC=15, Select color index --------------------------------------*/

         case 15 :

         /* Translate input */

              ic = (int) (rbuf[0] + 0.5);

         /* Check input for proper range */

              if (ic < 1 || ic > ncolor) {
                 ic = 1;
                 rbuf[0] = (float) ic;
                 }

              break;

/*--- IFUNC=16, Flush buffer. -------------------------------------------*/

         case 16 :
/*
	   sprintf (command, "update");
	   Tcl_Eval (interp, command);
*/
              break;

/*--- IFUNC=17, Read cursor. --------------------------------------------*/

         case 17 :

              break;

/*--- IFUNC=18, Erase alpha screen. -------------------------------------*/

         case 18 :

              break;

/*--- IFUNC=19, Set line style. -----------------------------------------*/

         case 19 :

              break;

/*--- IFUNC=20, Polygon fill. -------------------------------------------*/

/* We could do something here, but not for now */

         case 20 :

              break;

/*--- IFUNC=21, Set color representation. -------------------------------*/

         case 21 :

	{
	   int indx;
	   unsigned char red, green, blue;

	   indx = (int)(rbuf[0]+0.5);
	   if (indx < 1 || indx > ncolor) break;
	   red = (unsigned short) (rbuf[1]*255);
	   green = (unsigned short) (rbuf[2]*255);
	   blue = (unsigned short) (rbuf[3]*255);
	   }
	break;

/*--- IFUNC=22, Set line width. -----------------------------------------*/

         case 22 :
	     lw = (int)(rbuf[0]+0.5);

              break;

/*--- IFUNC=23, Escape --------------------------------------------------*/

         case 23 :

              break;

/*--- IFUNC=24, Rectangle Fill. -----------------------------------------*/

         case 24 :

              break;

/*--- IFUNC=25, ---------------------------------------------------------*/

         case 25 :

              break;

/*--- IFUNC=26, Line of pixels ------------------------------------------*/

         case 26 :

              break;

/*--- IFUNC=?, ----------------------------------------------------------*/

         default :

              break;

              } /* End of switch */

         return;

} /* End of tkdriv */
