/*****************************************************************************
******************************************************************************
**
** FILE:
**	imgTclAnn.c
**
** ABSTRACT:
**	This file contains interface routines to compute circularly averaged
**	intensity profiles
**
** ENTRY POINT		SCOPE	DESCRIPTION
** -------------------------------------------------------------------------
** imgTclAnnDeclare	public	declare all the verbs defined in this module
** tclAnn		static	compute radial profile in annuli
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	FTCL	 	TCL + Fermi extension
**      DERVISH
**	IMAGE
**
** AUTHORS:
**	Creation date:  Oct 1993
**	Steve Kent
**
******************************************************************************
******************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "imgProf.h"
#include "img_msg_c.h"
#include "imgProto.h"

/*----------------------------------------------------------------------------
**
** GLOBAL VARIABLES
*/

static char *module = "img";		/* name of this facility */


/*============================================================================  
**============================================================================
**
** ROUTINE: tclAnn
**
** DESCRIPTION:
**	Interface to routine to compute radial intensity profile in circular
**	annuli.
**
** RETURN VALUES:
**	TCL_OK -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	imgAnn
**
**
**============================================================================
*/
/*
 * Input is handle to image, x, y, max radius
 * return handle to : a profile list
 */

static char *tclAnn_use =
  "USAGE: imgAnnulus <region-handle> row col rmax";
static char *tclAnn_hlp =
  "Compute radial profile in circular annuli";

static int
tclAnn(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   char listName[HANDLE_NAMELEN];

   REGION *reg;
   double row, col;
   int rmax;
   CHAIN *proflist;

   shErrStackClear();

/* argc is the number of arguments passed from the command line (including
** the command itself)
*/
   if(argc != 5) {
      Tcl_SetResult(interp,tclAnn_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */
/* This routine translate a handle to a pointer */
/* See tclHandle.c in dervish for a complete list of these routines */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&reg, "REGION") != TCL_OK)
	return (TCL_ERROR);

/* Get the row and column */

   if (Tcl_GetDouble (interp, argv[2], &row)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble (interp, argv[3], &col)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetInt (interp, argv[4], &rmax)  != TCL_OK) return(TCL_ERROR);


/*
 * ok, get a handle for our new PROF list
 */

   if (shTclChainHandleNew (interp, listName, "PROF", &proflist) != TCL_OK) {
      return(TCL_ERROR);
   }

/* Done with the handles, etc. Now the action begins */


   if (imgAnn (reg, row, col, rmax, proflist) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in ANNULUS", TCL_STATIC);
	return(TCL_ERROR);
	}
   
/* Return handle as the result */
      Tcl_AppendResult (interp, listName, (char*)NULL);
   return(TCL_OK);
}


/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclAnnDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"imgAnnulus", 
		(Tcl_CmdProc *)tclAnn,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclAnn_hlp, tclAnn_use);

   return;
}

