/*
 * TCL support for image utilities
 *
 * ENTRY POINT		SCOPE
 * imgTclInterpDeclare	public	declare all the verbs defined in this module
 * tclInterp		static	Compute interpolated flux
 *
 * REQUIRED PRODUCTS:
 *      DERVISH
 *
 */
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "shGeneric.h"
#include "img_msg_c.h"
#include "imgProto.h"
#include "imgInterp.h"

static char *module = "img";		/* name of this facility */

/*****************************************************************************/
/*************************** tclInterp  **************************************/
/*****************************************************************************/
/*
 * Input is handle to image, a center, and a radius
 * return counts and area
 */

static char *tclInterp_use =
  "USAGE: imgInterp <dvec> wave";
static char *tclInterp_hlp =
  "Interpolate flux to the input wavelength in a DVEC structure";

static int
tclInterp(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   DVEC *dvec = NULL;
   double wave;
   double flux;

   shErrStackClear();

   if(argc != 3) {
      Tcl_SetResult(interp,tclInterp_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&dvec, "DVEC")
	!= TCL_OK) {
	Tcl_SetResult(interp,tclInterp_use,TCL_STATIC);
	return (TCL_ERROR);
	}

/* Get the treshold and max */

   if (Tcl_GetDouble (interp, argv[2], &wave)  != TCL_OK) return(TCL_ERROR);

/* Done with the handles, etc. Now the action begins */


   if (imgInterp (dvec, wave, &flux) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in Interp", TCL_STATIC);
	return(TCL_ERROR);
	}

   sprintf (interp->result, "%f", flux);

   return(TCL_OK);
}

/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclInterpDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"imgInterp", 
		(Tcl_CmdProc *)tclInterp,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclInterp_hlp, tclInterp_use);

   return;
}

