/*
 * TCL support for image utilities
 *
 * ENTRY POINT		SCOPE
 * imgTclPhotoDeclare	public	declare all the verbs defined in this module
 * tclCircle		static	Compute photometry in circular aperture
 *
 * REQUIRED PRODUCTS:
 *	FTCL	 	TCL + Fermi extension
 *      DERVISH
 *
 */
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "shGeneric.h"
#include "img_msg_c.h"
#include "imgProto.h"

static char *module = "img";		/* name of this facility */

/*****************************************************************************/
/*************************** tclCircle     **************************************/
/*****************************************************************************/
/*
 * Input is handle to image, a center, and a radius
 * return counts and area
 */

static char *tclCircle_use =
  "USAGE: imgCircle <region-handle> row col radius";
static char *tclCircle_hlp =
  "Compute integrated photometry inside an aperture";

static int
tclCircle(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   REGION *reg = NULL;
   double rowc, colc, rad;
   double counts, area;

   shErrStackClear();

   if(argc != 5) {
      Tcl_SetResult(interp,tclCircle_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&reg, "REGION")
	!= TCL_OK) {
	Tcl_SetResult(interp,tclCircle_use,TCL_STATIC);
	return (TCL_ERROR);
	}

/* Get the treshold and max */

   if (Tcl_GetDouble (interp, argv[2], &rowc)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble (interp, argv[3], &colc)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble (interp, argv[4], &rad)  != TCL_OK) return(TCL_ERROR);


/* Done with the handles, etc. Now the action begins */


   if (imgCircle (reg, rowc, colc, rad, &counts, &area) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in Circle", TCL_STATIC);
	return(TCL_ERROR);
	}

   sprintf (interp->result, "%.2f %.2f", counts, area);

   return(TCL_OK);
}


/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclPhotoDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"imgCircle", 
		(Tcl_CmdProc *)tclCircle,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclCircle_hlp, tclCircle_use);

   return;
}

