/*
 * TCL support for image utilities
 *
 * ENTRY POINT		SCOPE
 * imgTclReadAtlasDeclare publicdeclare all the verbs defined in this module
 * tclReadAtlas		static	Read Atlas Images
 *
 * REQUIRED PRODUCTS:
 *	FTCL	 	TCL + Fermi extension
 *      DERVISH
 *
 */
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "shGeneric.h"
#include "img_msg_c.h"
#include "imgProto.h"

/* A lonely prototype */

REGION *readAtlasImage(int color, char *infile, int row);

static char *module = "img";		/* name of this facility */

/*****************************************************************************/
/*************************** tclReadAtlas **********************************/
/*****************************************************************************/
/*
 * Input is color, input file, row, outfile.
 */

static char *tclReadAtlas_use =
  "USAGE: readAtlasImage color inFile row";
static char *tclReadAtlas_hlp =
  "Read one atlas image; return a region";

static int
tclReadAtlas(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   char *inFile = NULL;
   REGION *reg = NULL;
   int color, row;
   char tclRegionName[MAXTCLREGNAMELEN];

   shErrStackClear();

   if(argc != 4) {
      Tcl_SetResult(interp,tclReadAtlas_use,TCL_STATIC);
      return(TCL_ERROR);
   }

   
   inFile = argv[2];

   if (Tcl_GetInt (interp, argv[1], &color)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetInt (interp, argv[3], &row)  != TCL_OK) return(TCL_ERROR);

/* Get a new region */

   if (shTclRegNameGet(interp, tclRegionName) != TCL_OK) {
        Tcl_SetResult(interp ,"Cannot find region",TCL_VOLATILE);
        return(TCL_ERROR);
        }

   reg = readAtlasImage(color, inFile, row);

   if (reg == NULL) {
        Tcl_SetResult (interp, "Error reading Atlas Image",TCL_STATIC);
        shTclRegNameDel(interp, tclRegionName);
        return(TCL_ERROR);
        }

/* Associate region with TCL region name */

   if (shTclRegNameSetWithAddr(interp, reg, tclRegionName) != TCL_OK) {
        shRegDel(reg);
        shTclRegNameDel(interp, tclRegionName);
        Tcl_SetResult (interp, "Error creating region",TCL_STATIC);
        return TCL_ERROR;
        }

   Tcl_SetResult (interp, tclRegionName, TCL_VOLATILE);
   return(TCL_OK);
}


/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclReadAtlasDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"readAtlasImage", 
		(Tcl_CmdProc *)tclReadAtlas,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclReadAtlas_hlp, tclReadAtlas_use);

   return;
}

