/*****************************************************************************
******************************************************************************
**
** FILE:
**	epTclObject.c
**
** ABSTRACT:
**	This file contains the tcl interface to OBJECT routines
**
** ENTRY POINT		SCOPE	DESCRIPTION
** epTclObjectDeclare public declare all the verbs defined in this module
** -------------------------------------------------------------------------
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	dervish
**	ftcl
**
** AUTHORS:
**	Creation date:  27 Jan, 1994
**	Mario Nonino
**
******************************************************************************
******************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include "dervish.h"

#include "imgProto.h"

/*============================================================================  
**============================================================================
**
** LOCAL MACROS, DEFINITIONS, ETC.
**
**============================================================================
*/

/*----------------------------------------------------------------------------
**
** GLOBAL VARIABLES
*/
static char *module = "img";	/* name of this set of code */


/*============================================================================  
**============================================================================
**
** ROUTINE: imgTclObjectFind
**
** DESCRIPTION:
**	Interface to routine for object finder
**
** RETURN VALUES:
**	TCL_OK -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	imgRegFindObj
**
**
**============================================================================
*/
/*
 * Input is handle to image, sky, skynoise, radius,sigma1,sigma2,sigma3 
 * return handle to : a object list
 */

static char *imgTclObjectFind_use =
  "USAGE: imgObjFind <region-handle> sky skynoise radius sigma1 sigma2 sigma3 \
photon readout";
static char *imgTclObjectFind_hlp =
  "Find objects in a region";

static int
imgTclObjectFind(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   char listName[HANDLE_NAMELEN];

   REGION *reg;
   double sky;
   double sky_noise;
   double radius=0.0;
   double sigma1=0.0;
   double sigma2=0.0;
   double sigma3=0.0;
   double photon =0.0;
   double readout =0.0;
   CHAIN *starlist;


   shErrStackClear();

/* argc is the number of arguments passed from the command line (including
** the command itself)
*/
   if(argc != 10) {
      Tcl_SetResult(interp,imgTclObjectFind_use,TCL_VOLATILE);
      return(TCL_ERROR);
   }

/* Get the region pointer */

   if (shTclAddrGetFromName (interp, argv[1], (void *)&reg, "REGION") 
       != TCL_OK) {
	Tcl_AppendResult (interp, "Cannot find region ", argv[1], NULL);
	return (TCL_ERROR);
	}


   if (Tcl_GetDouble(interp, argv[2], &sky) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[3], &sky_noise) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[4], &radius) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[5], &sigma1) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[6], &sigma2) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[7], &sigma3) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[8], &photon) != TCL_OK) return TCL_ERROR;
   if (Tcl_GetDouble(interp, argv[9], &readout) != TCL_OK) return TCL_ERROR;

   if ( radius <= 0.0)
  	{
	       Tcl_SetResult (interp, "Radius must be positive!!", TCL_STATIC);
		return(TCL_ERROR);
	} 
/*
 * -- get a handle for our new STAR list
 */
   if (shTclChainHandleNew (interp, listName, "STAR", &starlist) != TCL_OK) 
      return(TCL_ERROR);

/* Done with the handles, etc. Now the action begins */


   if (imgRegFindObj(reg, starlist, sky, sky_noise, radius, sigma1,\
	   sigma2, sigma3, photon, readout) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in object finder", TCL_STATIC);
	return(TCL_ERROR);
	}
   
/* Return handle as the result */
      Tcl_AppendResult (interp, listName, (char*)NULL);
   return(TCL_OK);
}


/*============================================================================  
**============================================================================
**
** ROUTINE: imgTclObjectDeclare
**
** DESCRIPTION:
** 	Declare new tcl verbs
**
** RETURN VALUES:
**	TCL_OK    -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	Various tcl and dervish routines
**
**============================================================================
*/

/*****************************************************************************/

void
imgTclFindObjDeclare(Tcl_Interp *interp) 
{

   shTclDeclare(interp,"imgFindObj", 
		(Tcl_CmdProc *)imgTclObjectFind,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, imgTclObjectFind_hlp, imgTclObjectFind_use);



} 

