/*
 * TCL support for image smoothing utilities
 *
 * ENTRY POINT		SCOPE
 * imgTclSmoothDeclare	public	declare all the verbs defined in this module
 * tclSmooth		static	find bright stars
 *
 * REQUIRED PRODUCTS:
 *	FTCL	 	TCL + Fermi extension
 *      DERVISH
 *
 */
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "shGeneric.h"
#include "img_msg_c.h"
#include "imgProto.h"

static char *module = "img";		/* name of this facility */

/*****************************************************************************/
/*************************** tclSmooth  **************************************/
/*****************************************************************************/
/*
 * Input is handle to region.  Smooth in place.
 */

static char *tclSmooth_use =
  "USAGE: imgSmooth <region-handle>";
static char *tclSmooth_hlp =
  "Smooth a region (1 4 1 function)";

static int
tclSmooth(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{

   REGION *reg = NULL;

   shErrStackClear();

   if(argc != 2) {
      Tcl_SetResult(interp,tclSmooth_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&reg, "REGION")
	!= TCL_OK) {
	Tcl_SetResult(interp,tclSmooth_use,TCL_STATIC);
	return (TCL_ERROR);
	}

/* Done with the handles, etc. Now the action begins */

   if (imgSmooth(reg) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in imgSmooth", TCL_STATIC);
	return(TCL_ERROR);
	}
   
   return(TCL_OK);
}


/*****************************************************************************/
/*************************** tclBleedFix *************************************/
/*****************************************************************************/
/*
 * Input is handle to region.  Smooth in place.
 */

static char *tclBleedFix_use =
  "USAGE: imgSmooth <region-handle> thresh";
static char *tclBleedFix_hlp =
  "Interpolate over bleed trails";

static int
tclBleedFix(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{

   REGION *reg = NULL;
   int thresh;

   shErrStackClear();

   if(argc != 3) {
      Tcl_SetResult(interp,tclBleedFix_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&reg, "REGION")
	!= TCL_OK) {
	Tcl_SetResult(interp,tclBleedFix_use,TCL_STATIC);
	return (TCL_ERROR);
	}

   if (Tcl_GetInt(interp, argv[2], &thresh) != TCL_OK) return TCL_ERROR;

/* Done with the handles, etc. Now the action begins */

   if (imgBleedFix(reg, thresh) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in imgBleedFix", TCL_STATIC);
	return(TCL_ERROR);
	}
   
   return(TCL_OK);
}


/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclSmoothDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"imgSmooth", 
		(Tcl_CmdProc *)tclSmooth,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclSmooth_hlp, tclSmooth_use);

   shTclDeclare(interp,"imgBleedFix", 
		(Tcl_CmdProc *)tclBleedFix,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclBleedFix_hlp, tclBleedFix_use);

   return;
}

