/* This is a template routine to be used to define new data types.  It is
** one of three modules that needs to be created for each data type.  Suppose
** you want to create a new data type called STAR.  Edit this file and
** make the following substitutions:
** foo_lc -> star
** Foo_mc -> Star
** FOO_uc -> STAR
** module_lc->ast
** include file is named module_lcFoo_mc
** The name of this module should be module_lcTclFOO_UC.c
** I use this mechanism in order that it work with the EDT editor.
*/

/*****************************************************************************
******************************************************************************
**
** FILE:
**	module_lcTclFOO_UC.c
**
** ABSTRACT:
**	This file contains the tcl interface to FOO_UC routines
**
** ENTRY POINT		SCOPE	DESCRIPTION
 * moduleUlcTclFoo_mcDeclare public declare all the verbs defined in this module
** -------------------------------------------------------------------------
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	dervish
**	ftcl
**
** AUTHORS:
**	Creation date:  Oct. 1993
**	Steve Kent
**
******************************************************************************
******************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <termio.h>
#include <sys/types.h>
#include "dervish.h"
#include "shTclHandle.h"
#include "module_lcFoo_mc.h"

/*============================================================================  
**============================================================================
**
** LOCAL MACROS, DEFINITIONS, ETC.
**
**============================================================================
*/

/*----------------------------------------------------------------------------
**
** GLOBAL VARIABLES
*/
static char *module = "module_lcTclFoo_mc";	/* name of this set of code */


/*============================================================================  
**============================================================================
**
** ROUTINE: tclFoo_mcNew
**
** DESCRIPTION:
** 	Return a handle to a new FOO_uc
**
** RETURN VALUES:
**	TCL_OK    -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	Various tcl and dervish routines
**	module_lcFoo_mcNew
**
**============================================================================
*/

/*****************************************************************************/

static char *tclFoo_mcNew_use =
  "USAGE: foo_lcNew";
static char *tclFoo_mcNew_hlp =
  "create a new FOO_uc";

static int
tclFoo_mcNew(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   char name[HANDLE_NAMELEN];
   FOO_uc *foo_lc;
   
   shErrStackClear();
   
   if(argc != 1) {
      Tcl_SetResult(interp,tclFoo_mcNew_use,TCL_STATIC);
      return(TCL_ERROR);
   }
/*
 * Get a handle for our new FOO_uc
 */


   if (shTclHandleNew (interp, name, "FOO_uc",
        (void *)(foo_lc=module_lcFoo_mcNew())) != TCL_OK) {
        if (foo_lc != NULL) module_lcFoo_mcDel(foo_lc);
        return(TCL_ERROR);
        }
   
   Tcl_SetResult(interp,name,TCL_VOLATILE);
   return(TCL_OK);
}


/*============================================================================  
**============================================================================
**
** ROUTINE: tclFoo_mcDel
**
** DESCRIPTION:
**	Delete a FOO_uc and its handle
**
** RETURN VALUES:
**	TCL_OK    -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	Various tcl and dervish routines
**	module_lcFoo_mcDel
**
**============================================================================
*/

/*****************************************************************************/

static char *tclFoo_mcDel_use =
  "USAGE: foo_lcDel foo_lc";
static char *tclFoo_mcDel_hlp =
  "Delete a FOO_uc";

static int
tclFoo_mcDel(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   FOO_uc *foo_lc;
   char *name;
   char *opts = "foo_lc";
   
   shErrStackClear();
   
   ftclParseSave("foo_lcDel");
   if(ftclFullParseArg(opts,argc,argv) != 0) {
	name = ftclGetStr("foo_lc");
	if (shTclAddrGetFromName (interp, name, (void *)&foo_lc, "FOO_uc")
                != TCL_OK) {
	    return(TCL_ERROR);
	    }
	}
    else {
	Tcl_SetResult(interp,tclFoo_mcDel_use,TCL_STATIC);
	return(TCL_ERROR);
	}

   module_lcFoo_mcDel(foo_lc);

/* Modification by S. Kent Aug 2, 1993 delete handle */

   p_shTclHandleDel(interp, name);

   Tcl_SetResult(interp,"",TCL_STATIC);
   return(TCL_OK);
}


/*============================================================================  
**============================================================================
**
** ROUTINE: module_lcTclFoo_mcDeclare
**
** DESCRIPTION:
** 	Declare new tcl verbs
**
** RETURN VALUES:
**	TCL_OK    -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	Various tcl and dervish routines
**
**============================================================================
*/

/*****************************************************************************/

void
module_lcTclFoo_mcDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"foo_lcNew", 
		(Tcl_CmdProc *)tclFoo_mcNew, 
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclFoo_mcNew_hlp, tclFoo_mcNew_use);

   shTclDeclare(interp,"foo_lcDel", 
		(Tcl_CmdProc *)tclFoo_mcDel, 
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL, 
		module, tclFoo_mcDel_hlp, tclFoo_mcDel_use);
} 
