/*****************************************************************************
******************************************************************************
**
** FILE:
**	imgTclVec.c
**
** ABSTRACT:
**	This file contains interface routines to compute circularly averaged
**	intensity profiles
**
** ENTRY POINT		SCOPE	DESCRIPTION
** -------------------------------------------------------------------------
** imgTclVecDeclare	public	declare all the verbs defined in this module
** tclVec		static	compute vector profile
**
** ENVIRONMENT:
**	ANSI C.
**
** REQUIRED PRODUCTS:
**	FTCL	 	TCL + Fermi extension
**      DERVISH
**	IMAGE
**
** AUTHORS:
**	Creation date:  Oct 1993
**	Steve Kent
**
******************************************************************************
******************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include "dervish.h"
#include "imgProf.h"
#include "img_msg_c.h"
#include "imgProto.h"

/*----------------------------------------------------------------------------
**
** GLOBAL VARIABLES
*/

static char *module = "img";		/* name of this facility */


/*============================================================================  
**============================================================================
**
** ROUTINE: tclVec
**
** DESCRIPTION:
**	Interface to routine to compute vector intensity profile
**
** RETURN VALUES:
**	TCL_OK -   Successful completion.
**	TCL_ERROR -   Failed miserably.
**
** CALLS TO:
**	imgVec
**
**
**============================================================================
*/
/*
 * Input is handle to image, x, y, max radius
 * return handle to : a profile list
 */

static char *tclVec_use =
  "USAGE: imgVec <region-handle> row col ang nwidth nstep";
static char *tclVec_hlp =
  "Compute vector profile; -nwidth to +nwidth wide, -nstep to +nstep long";

static int
tclVec(
	  ClientData clientData,
	  Tcl_Interp *interp,
	  int argc,
	  char **argv
	  )
{
   char listName[HANDLE_NAMELEN];

   REGION *reg;
   double row, col;
   double ang;
   int nwidth, nstep;
   CHAIN *proflist;

   shErrStackClear();

/* argc is the number of arguments passed from the command line (including
** the command itself)
*/
   if(argc != 7) {
      Tcl_SetResult(interp,tclVec_use,TCL_STATIC);
      return(TCL_ERROR);
   }

/* Get the region pointer */
/* This routine translate a handle to a pointer */
/* See tclHandle.c in dervish for a complete list of these routines */

   if (shTclAddrGetFromName(interp, argv[1], (void **)&reg, "REGION") != TCL_OK)
	return (TCL_ERROR);

/* Get the row and column */

   if (Tcl_GetDouble (interp, argv[2], &row)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble (interp, argv[3], &col)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetDouble (interp, argv[4], &ang)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetInt (interp, argv[5], &nwidth)  != TCL_OK) return(TCL_ERROR);
   if (Tcl_GetInt (interp, argv[6], &nstep)  != TCL_OK) return(TCL_ERROR);


/*
 * ok, get a handle for our new PROF list
 */

   if (shTclChainHandleNew (interp, listName, "PROF", &proflist) != TCL_OK) {
      return(TCL_ERROR);
   }

/* Done with the handles, etc. Now the action begins */


   if (imgVec (reg, row, col, ang, nwidth, nstep, proflist) != IMG_SUCCESS) {
	Tcl_SetResult(interp,"Error in VECTOR", TCL_STATIC);
	return(TCL_ERROR);
	}
   
/* Return handle as the result */
      Tcl_AppendResult (interp, listName, (char*)NULL);
   return(TCL_OK);
}


/*****************************************************************************/
/*
 * Declare my new tcl verbs to tcl
 */
void
imgTclVecDeclare(Tcl_Interp *interp) 
{
   shTclDeclare(interp,"imgVector", 
		(Tcl_CmdProc *)tclVec,
		(ClientData) 0,
		(Tcl_CmdDeleteProc *)NULL,
		module, tclVec_hlp, tclVec_use);

   return;
}

