/* tkdriv.c -- Fortran callable PGPLOT driver for a Tk canvas widget */
/* This routine shadow gfdriv, which is a driver for a GraphOn device
 * that we don't need
*/

#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "dervish.h"

static int imax=0;
static int imin=0;
static int jmax=0;
static int jmin=0;
/* static int count=0; */
static int ic;
static int lw;
static char *tk_type = "TK";
static char namebuf[100];
static char color[8][10];
static int colorinit=0;
static int ncolor=8;

/* Need to get the interp structure into a low level pgplot routine.  Here
 * is a routine that is called on program startup to copy interp to
 * static storage.  Kind of clunky, but it gets the job done.
*/

/* Here is the storage and the function */

static Tcl_Interp *tkInterp = NULL;

void tkInterpSave (Tcl_Interp *interp)
{
   tkInterp = interp;
}


/**********************************************************************/
/* This function will override the pgplot version.  The device type will be
 * TK; intialize by calling pgBegin widget/TK  where widget is an existing
 * tk Canvas widget.  By making the user create it, it is possible to plot
 * into multiple tkWindows in a session.  The alternative would be to create
 * the tk widget in the pgBegin command, but we would want to destroy it
 * upon calling pgEnd for symmetry.
*/

void txdriv_ (int *ifunc, float rbuf[], int *nbuf, char *chr, int *lchr,
	int len) {

/*PGPLOT driver for drawing plots into a MASK region */
	int resx=100;	/* Resolution is dependent on where mask will get
			   displayed */
	int resy=100;
	int i;
	float x0, y0, x1, y1;
	char command[1000];
	char *temp;

         switch (*ifunc) {

/*--- IFUNC=1, Return device name ---------------------------------------*/

         case 1 :
              strncpy( chr, tk_type, len ) ; /* 14Oct91*/
              *lchr = strlen( tk_type ) ;
              for ( i = *lchr ; i < len ; i++ )
                  chr[i] = ' ';
              break;

/*--- IFUNC=2, Return physical min and max for plot device, and range
               of color indices -----------------------------------------*/

         case 2 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              rbuf[4] = 0.0;
              rbuf[5] = (float) ncolor;
              *nbuf = 6;

              break;

/*--- IFUNC=3, Return device resolution ---------------------------------*/

         case 3 :

              rbuf[0] = (float) resx;
              rbuf[1] = (float) resy;
              rbuf[2] = 1.0;             /* Device coordinates per pixel */
              *nbuf = 3;

              break;

/*--- IFUNC=4, Return misc device info ----------------------------------*/

         case 4 :

              chr[0] = 'I'; /* Interactive device */
              chr[1] = 'N'; /* no cursor is available */
              chr[2] = 'N'; /* no dashed lines */
              chr[3] = 'N'; /* no area fill available */
              chr[4] = 'N'; /* no fat lines */
              chr[5] = 'N'; /* no rectangle fill available */
              chr[6] = 'N'; /* no line of pixels available */
              chr[7] = 'N'; /* not used */
              chr[8] = 'N'; /* not used */
              chr[9] = 'N'; /* not used */
              *lchr = 10;

              break;

/*--- IFUNC=5, Return default file name ---------------------------------*/

         case 5 :

              strcpy (chr, "");  /* Default name is NULL */
              *lchr = 0;

              break;

/*--- IFUNC=6, Return default physical size of plot ---------------------*/

         case 6 :

              rbuf[0] = 0.0;
              rbuf[1] = (float) (imax - imin);
              rbuf[2] = 0.0;
              rbuf[3] = (float) (jmax - jmin);
              *nbuf = 4;

              break;

/*--- IFUNC=7, Return misc defaults -------------------------------------*/

         case 7 :

              rbuf[0] = 1.0;
              *nbuf = 1;

              break;

/*--- IFUNC=8, Select plot ----------------------------------------------*/

         case 8 :

              break;

/*--- IFUNC=9, Open workstation -----------------------------------------*/

         case 9 :
/* Initialize colors the 1st time through */

/* Save name of widget in namebuf */
	strncpy (namebuf, chr, len);
	for (i=0; i<len; i++) {
	   if (namebuf[i] == ' ') namebuf[i] = '\0';
	   }
	namebuf[len]='\0';
	if (colorinit == 0) {
	   strcpy (color[0], "#ffffff"); /* white */
	   strcpy (color[1], "#ff0000"); /* red */
	   strcpy (color[2], "#00ff00"); /* green */
	   strcpy (color[3], "#0000ff"); /* blue */
	   strcpy (color[4], "#00ffff"); /* cyan */
	   strcpy (color[5], "#ffe0ff"); /* magenta (modified) */
	   strcpy (color[6], "#ffff00"); /* yellow */
	   strcpy (color[7], "#000000"); /* black */
	   colorinit=1;
	   }
/* Check to see if the widget exists and is a canvas */
	sprintf (command, "%s %s", "winfo class", namebuf);
	if (Tcl_Eval(tkInterp, command) != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}
	if (strcmp(tkInterp->result, "Canvas") != 0) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}
/* Get dimensions */
	sprintf (command, "%s %s", "winfo width", namebuf);
	if (Tcl_Eval(tkInterp, command) != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}
	sscanf(tkInterp->result, "%d", &imax);
	sprintf (command, "%s %s", "winfo height", namebuf);
	if (Tcl_Eval(tkInterp, command) != TCL_OK) {
		rbuf[1]=0.;	/* I think this is return error code */
		break;
		}
	sscanf(tkInterp->result, "%d", &jmax);
/* Initial default color */
	ic = 1;
	rbuf[1]=1;
/* Line weight */
	lw = 1;
	Tcl_ResetResult(tkInterp);
	break;

/*--- IFUNC=10, Close workstation ---------------------------------------*/

         case 10 :
/* We don't clear the canvas; just reset namebuf */
	strcpy (namebuf,"");
	Tcl_ResetResult(tkInterp);
	break;

/*--- IFUNC=11, Begin picture -------------------------------------------*/

         case 11 :

/* Clear the canvas */
/* Get a list of all the tags and delete them all */
		sprintf (command, "%s %s", namebuf, "find all");
/* On error, just plow ahead anyway */
		if (Tcl_Eval(tkInterp, command) == TCL_OK) {
		   if (strlen(tkInterp->result) > 0) {
/* Save result string in a temporary array (could be big) & use as argument
 * to the next command
*/
			temp = shMalloc(strlen(tkInterp->result)+1);
			strcpy (temp, tkInterp->result);
			Tcl_VarEval(tkInterp, namebuf, " delete ",temp, NULL);
			shFree(temp);
			}
		   }
	   Tcl_ResetResult(tkInterp);
		break;

/*--- IFUNC=12, Draw line -----------------------------------------------*/

         case 12 :

/* Decision time: pgPlot has x, y origin at lower left.  tk places it at
 * upper left.  I will use pgPlot convention, which means that pgPlot
 * coordinates don't map onto tk coordinates (such as would arise from
 * manipulating the widget directly
*/
         /* Translate input */

/* Reverse y (vertical) coordinates */
              x0 = rbuf[0] + imin;
              y0 = jmax - (rbuf[1] + jmin);
              x1 = rbuf[2] + imin;
              y1 = jmax - (rbuf[3] + jmin);

         /* Draw the line */
	   if (x0 < 0.) x0 = 0.;   if (x0 > imax) x0 = imax;
	   if (x1 < 0.) x1 = 0.;   if (x1 > imax) x1 = imax;
	   if (y0 < 0.) y0 = 0.;   if (y0 > jmax) y0 = jmax;
	   if (y1 < 0.) y1 = 0.;   if (y1 > jmax) y1 = jmax;

	sprintf (command, "%s %s %f %f %f %f -fill %s", namebuf,
	   "create line", x0, y0, x1, y1, color[ic-1]);
	Tcl_Eval (tkInterp, command);
/*	if (count++ == 500) {
	   sprintf (command, "update");
	   Tcl_Eval (tkInterp, command);
	   count=0;
	   }
*/
	Tcl_ResetResult(tkInterp);
	break;

/*--- IFUNC=13, Draw dot ------------------------------------------------*/

         case 13 :

/* Reverse y (vertical) coordinates */
              x0 = rbuf[0] + imin;
              y0 = jmax - (rbuf[1] + jmin);
              x1 = x0 + 1.;
              y1 = y0 - 1.;
         /* Draw the line */
	   if (x0 < 0.) x0 = 0.;   if (x0 > imax) x0 = imax;
	   if (x1 < 0.) x1 = 0.;   if (x1 > imax) x1 = imax;
	   if (y0 < 0.) y0 = 0.;   if (y0 > jmax) y0 = jmax;
	   if (y1 < 0.) y1 = 0.;   if (y1 > jmax) y1 = jmax;
/*
	if (count++ == 500) {
	   sprintf (command, "update");
	   Tcl_Eval (tkInterp, command);
	   count=0;
	   }
*/
	sprintf (command, "%s %s %f %f %f %f -fill %s", namebuf,
	   "create line", x0, y0, x1, y1, color[ic-1]);
	Tcl_Eval (tkInterp, command);
	break;

/*--- IFUNC=14, End picture ---------------------------------------------*/

         case 14 :

              break;

/*--- IFUNC=15, Select color index --------------------------------------*/

         case 15 :

         /* Translate input */

              ic = (int) (rbuf[0] + 0.5);

         /* Check input for proper range */

              if (ic < 1 || ic > ncolor) {
                 ic = 1;
                 rbuf[0] = (float) ic;
                 }

              break;

/*--- IFUNC=16, Flush buffer. -------------------------------------------*/

         case 16 :
/*
	   sprintf (command, "update");
	   Tcl_Eval (tkInterp, command);
*/
              break;

/*--- IFUNC=17, Read cursor. --------------------------------------------*/

         case 17 :

              break;

/*--- IFUNC=18, Erase alpha screen. -------------------------------------*/

         case 18 :

              break;

/*--- IFUNC=19, Set line style. -----------------------------------------*/

         case 19 :

              break;

/*--- IFUNC=20, Polygon fill. -------------------------------------------*/

/* We could do something here, but not for now */

         case 20 :

              break;

/*--- IFUNC=21, Set color representation. -------------------------------*/

         case 21 :

	{
	   int indx;
	   unsigned char red, green, blue;

	   indx = (int)(rbuf[0]+0.5);
	   if (indx < 1 || indx >= ncolor) break;
	   red = (unsigned short) (rbuf[1]*255);
	   green = (unsigned short) (rbuf[2]*255);
	   blue = (unsigned short) (rbuf[3]*255);
	   sprintf (color[indx-1],"#%02d%02d%02d", red, green, blue);
	   }
	break;

/*--- IFUNC=22, Set line width. -----------------------------------------*/

         case 22 :
	     lw = (int)(rbuf[0]+0.5);

              break;

/*--- IFUNC=23, Escape --------------------------------------------------*/

         case 23 :

              break;

/*--- IFUNC=24, Rectangle Fill. -----------------------------------------*/

         case 24 :

              break;

/*--- IFUNC=25, ---------------------------------------------------------*/

         case 25 :

              break;

/*--- IFUNC=26, Line of pixels ------------------------------------------*/

         case 26 :

              break;

/*--- IFUNC=?, ----------------------------------------------------------*/

         default :

              break;

              } /* End of switch */

         return;

} /* End of tkdriv */
