#interface routine to annulus that can be called from a mouse pointer
#Procedures:	ann6
#		xann6
#		annulus
#		binpix
#		fwhmr

source regutil.tcl
source vector.tcl

proc ann6 {reg row col} {annulus $reg $row $col 6}
proc xann6 {reg row col} {annulus $reg $row $col 6 "OLD"}

proc annulus {reg row col rad {new "NEW"}} {
#Get centroid
   set list [imgCentroid $reg $row $col $rad $rad]
#   echo $list
   set xc [keylget list xc]
   set yc [keylget list yc]
   set list {}
   for {set r 0} {$r <= $rad} {incr r} {
   set sum 0; set n 0;
	for {set j 0} {$j <= [expr { 6.28*$r}]} {incr j} {
	   set ang [expr { $j / [max 1 $r]}]
	   set x [expr { $xc + $r * [cos $ang]}]
	   set y [expr { $yc + $r * [sin $ang]}]
	   set p [binpix $reg $y $x]
	   set sum [expr { $sum + $p }]
	   incr n
	   }
	set p [expr { $sum / $n}]
	lappend list "$r $p"
	}
# Make a histogram format list
   set hlist [makehist $list]
# Drop 1st item
   lvarpop hlist
# Convert to two 1-d arrays
   global xlist
   global ylist
   list_to_vec $hlist xlist ylist
# find max value of histogram
   set narray [llength $ylist]
   set ymax 0; set ymin 65535
   for {set i 0} {$i < ($narray-1) } {incr i} {
        set yval [lindex $ylist $i]
	set ymin [min $ymin $yval]
	set ymax [max $ymax $yval]
	}
   set ymax [expr {$ymax * 1.2}]
   if { "$new" == "NEW" } then {
	pgAsk 0
	pgSls 1
	pgEnv -$rad $rad $ymin $ymax 0 0
	} else {pgSls 2}
    pgWindow $rad -$rad $ymin $ymax
   pgLine $xlist $ylist
   pgWindow -$rad $rad $ymin $ymax
   pgLine $xlist $ylist
#Compute fwhm
   list_to_vec $list xlist ylist
   set n 0; set sum 0; set sum2 0;
   set thresh [lindex $ylist $rad]
   set r1 [lvarpop xlist]
   set p1 [expr { [lvarpop ylist] - $thresh} ]
   for {set r 0} {$r < $rad} {incr r} {
	set r2 [lvarpop xlist]
	set p2 [expr { [lvarpop ylist] - $thresh} ]
	set avg [expr {($p1+$p2)/2.}]
	set sum [expr {$sum + $avg * ($r2*$r2 - $r1*$r1)}]
	set sum2 [expr {$sum2 + $avg * ($r2*$r2*$r2*$r2 - $r1*$r1*$r1*$r1)/2}]
	set r1 $r2
	set p1 $p2
	}
   if {$sum2 <= 0} then {return 0} else {}
   if {$sum  <= 0} then {return 0} else {}
   set fwhm [sqrt [ expr { $sum2 / $sum } ]  ]
   set fwhm [expr { $fwhm * 2.35 / 1.414} ]
   }

proc binpix {reg row col} {
   set r1 [floor $row]
   set c1 [floor $col]
   set r2 [expr {$r1 + 1}]
   set c2 [expr {$c1 + 1}]
   set ncol [regNcol $reg]
   set nrow [regNrow $reg]
   if {$r1 < 0} then {return 0}
   if {$r1 >= $nrow} then {return 0}
   if {$c1 < 0} then {return 0}
   if {$c1 >= $ncol} then {return 0}
   if {$r2 < 0} then {return 0}
   if {$r2 >= $nrow} then {return 0}
   if {$c2 < 0} then {return 0}
   if {$c2 >= $ncol} then {return 0}
   set pix1 [regPixGet $reg $r1 $c1]
   set pix2 [regPixGet $reg $r1 $c2]
   set pix3 [regPixGet $reg $r2 $c1]
   set pix4 [regPixGet $reg $r2 $c2]
   set pix1 [expr { ($r2-$row)*($c2-$col)*$pix1 }]
   set pix2 [expr { ($r2-$row)*($col-$c1)*$pix2 }]
   set pix3 [expr { ($row-$r1)*($c1+1-$col)*$pix3 }]
   set pix4 [expr { ($row-$r1)*($col-$c1)*$pix4 }]
   set result [expr {$pix1 + $pix2 + $pix3 + $pix4}]
   }

#Compute FWHM using annular rings.  Same as annulus without the plotting.
proc fwhmr {reg row col {rad 6}} {
#Get centroid
   set list [imgCentroid $reg $row $col $rad $rad]
   set xc [keylget list xc]
   set yc [keylget list yc]
   set list {}
   for {set r 0} {$r <= $rad} {incr r} {
   set sum 0; set n 0;
	for {set j 0} {$j <= [expr { 6.28*$r}]} {incr j} {
	   set ang [expr { $j / [max 1 $r]}]
	   set x [expr { $xc + $r * [cos $ang]}]
	   set y [expr { $yc + $r * [sin $ang]}]
	   set p [binpix $reg $y $x]
	   set sum [expr { $sum + $p }]
	   incr n
	   }
	set p [expr { $sum / $n}]
	lappend list "$r $p"
	}
# Convert to two 1-d arrays
   global xlist
   global ylist
#Compute fwhm
   list_to_vec $list xlist ylist
   set n 0; set sum 0; set sum2 0;
   set thresh [lindex $ylist $rad]
   set r1 [lvarpop xlist]
   set p1 [expr { [lvarpop ylist] - $thresh} ]
   for {set r 0} {$r < $rad} {incr r} {
	set r2 [lvarpop xlist]
	set p2 [expr { [lvarpop ylist] - $thresh} ]
	set avg [expr {($p1+$p2)/2.}]
	set sum [expr {$sum + $avg * ($r2*$r2 - $r1*$r1)}]
	set sum2 [expr {$sum2 + $avg * ($r2*$r2*$r2*$r2 - $r1*$r1*$r1*$r1)/2}]
	set r1 $r2
	set p1 $p2
	}
   if {$sum2 <= 0} then {return 0} else {}
   if {$sum  <= 0} then {return 0} else {}
   set fwhm [sqrt [ expr { $sum2 / $sum } ]  ]
   set fwhm [expr { $fwhm * 2.35 / 1.414} ]
   }
