# Auxiliary routines to make use of curses routine, screenEdit
#
# Format of screenEdit is:
#	screenEdit message beginIndex arrayName ncolName arrayVal ncolVal
#where message is a 1-line message, beginIndex is the starting cursor pos,
# arrayName is a tcl array with a set of names, indexed starting at 0
# (index 0 is a field label; index 1 is the start of the real data)
# arrayVal is another tcl array with a set of values for each name, same
# indexing, ncolName is the number of columns used to display arrayName,
# arrayVal is the number of columns to display arrayVal.  beginIndex is
# used to position the cursor on startup.  You can
# edit the Val fields (use arrow keys, <Ctrl-K>, <CR>, <Del>).
# The update values are copied back to the arrayVal array.
# Simple, huh?
#
# Procedures defined here:
#
# structsEdit <handle>		Edit the structure pointed to by the handle.

##############################################################################
# structsEdit does recursive translation like structPrint
#

proc structsEdit {hndl {outline ""}} {
#Complex handle expressions with dereferenced pointers mess up the following.
#Embed in parentheses to cure
   if {$outline != ""} then {upvar 1 $outline outlist}
   set arrayName(0) $hndl
   set arraySchema(0) STRUCT
   set arrayVal(0) [handleType $hndl]
   set begin 1
   while {1} {
	set nelem 0
	recursEdit "($hndl)" arrayName arraySchema arrayVal nelem
	set message(0) ""
	set response [screenEdit "" $begin arrayName -40 arrayVal 80]
	set command [lindex $response 0]
	set line [lindex $response 1]
	set begin $line
	set outlist \
	   [list $arrayName($line) $arraySchema($line) $arrayVal($line)]
#Update output values if CTRL-X was used to exit
	if {$command == "X"} {
	   for {set i 1} {$i <= $nelem} {incr i} {
	      if {$arraySchema($i) == "PRIM"} then {
		catch {handleSet $arrayName($i) $arrayVal($i)}
		}
	      if {$arraySchema($i) == "ENUM"} then {
		catch {handleSet $arrayName($i) $arrayVal($i)}
		}
	      if {$arraySchema($i) == "ARRAY"} then {
		set handle [lindex $arrayName($i) 0]
		set arg [lindex $arrayName($i) 1]
		catch {arraySet $handle $arg $arrayVal($i)}
		}
#	      if {$arrayName($i) == "PTR"}
	      }
#Redisplay after a CTRL-X
	   continue
	   }
#Follow a link
	if {$command == "L"} then {
	   set name [lindex $outlist 0]
	   set schema [lindex $outlist 1]
	   set val [lindex $outlist 2]
	   if {$schema != "PTR"} then {continue}
#Get the target of the pointer
	   set top [lvarpop name]
	   set var [lvarpop name]
	   set allschema [schemaGet $top]
	   set type [keylget allschema $var]
	   if {$type == "LIST*"} then {
		listEdit "*((LIST *)($top.$var))"
	   } else {
#There are lots of weird pointers out there.  Just catch -em.
		catch {structsEdit "*(($type)($top.$var))"}
		}
	   continue
	   } else break
	}
#For convenience, set response to null if CTRL-Z was used to exit.
   if {$command == "Z"} then {set response ""}
   return $response
   }

##############################################################################
proc recursEdit { hndl inName inSchema inVal inElem} {
   upvar 1 $inName arrayName
   upvar 1 $inSchema arraySchema
   upvar 1 $inVal arrayVal
   upvar 1 $inElem nelem
   set allschema [schemaGet $hndl]
   set test [schemaKindGet $hndl]
   if {$test == "PRIM" || $test == "ENUM"} {
# Handle is a primitive type.
	set arg ""
	elemEdit $hndl $arg arrayName arraySchema arrayVal nelem
	} else {
	foreach f $allschema {
# Handle is a complicated type.
	   set arg [lindex $f 0]
	   elemEdit $hndl $arg arrayName arraySchema arrayVal nelem
	   }
	}
}

##############################################################################
proc elemEdit { hndl arg inName inSchema inVal inElem} {
   upvar 1 $inName arrayName
   upvar 1 $inSchema arraySchema
   upvar 1 $inVal arrayVal
   upvar 1 $inElem nelem
# if arg = "", we have a primitive type
  set allschem [schemaGet $hndl]
  if {$arg == ""} {
	incr nelem
	set arrayName($nelem) $hndl
	set arraySchema($nelem) PRIM
	set arrayVal($nelem) [handleShow $hndl]
	return
	}
   set schema [keylget allschem $arg]
# Get the type of the handle.
   set type [handleType $hndl.$arg]
# If input is a pointer, check to see if it is an array
   if { $type == "PTR"} {
	if { [regexp {[][<>]} $schema] } {
# it is an array

# check to see if it is an array of primitives or of structs
	   set type [lindex [split $schema {[]<>}] 0]
	   set subschema [schemaGetFromType $type]
	   if {[llength $subschema] <= 1} {
#It's a primitive.  Print a bunch of values on 1 line.
		set val [arrayShow $hndl $arg]
#Trash double quotes that I stick on for character strings.
		set val [string trim $val \"]
		incr nelem
		set arrayName($nelem) "$hndl $arg"
		set arraySchema($nelem) ARRAY
		set arrayVal($nelem) $val
		return
	   } else {
#It's a complex type.  Print each array element one line at a time.
		arrayEditByElem $hndl $arg arrayName arraySchema arrayVal nelem
		return
		}
	   }
	# is it a known handle?
	set val [handleShow $hndl.$arg]
	if {![catch {handleNameGet $val} hval]} {
	   set val [format "(handle)%s" $hval]
	   }
	incr nelem
	set arrayName($nelem) "$hndl $arg"
	set arraySchema($nelem) PTR
	set arrayVal($nelem) $val
	return
	}
   recursEdit $hndl.$arg arrayName arraySchema arrayVal nelem
}
############################################################################
# New version of arrayEdit - print 1 line per array element regardless.
#
proc arrayEditByElem {hndl arg inarrayName inarraySchema inarrayVal innelem} {
   upvar 1 $inarrayName arrayName
   upvar 1 $inarraySchema arraySchema
   upvar 1 $inarrayVal arrayVal
   upvar 1 $innelem nelem
   set allschem [schemaGet $hndl]
   set schema [keylget allschem $arg]
   set sch [split $schema {[]<>}]
   set type [lindex $sch 0]
   set dimen {}
   for {set i 1} {$i < [llength $sch]} { set i [expr $i+2] } {
	lappend dimen [lindex $sch $i]
	}
# Is it a scalar after all?  More than 2 dimensions?
   if { [llength $dimen] == 0 || [llength $dimen] > 2} then {
	elemTest $hndl.$arg ""
	}
   if { [llength $dimen] == 1 } {
	for {set i 0} {$i < [lindex $dimen 0]} { incr i } {
	   recursEdit $hndl.$arg<$i> arrayName arraySchema arrayVal nelem
	   }
	}
   if { [llength $dimen] == 2 } {
	for {set i 0} {$i < [lindex $dimen 0]} { incr i } {
	   for {set j 0} {$j < [lindex $dimen 1]} { incr j } {
		recursEdit $hndl.$arg<$i><$j> arrayName arraySchema arrayVal nelem
		}
	   }
	}
   }

##############################################################################
proc listPage {listHandle} {
#Complex handle expressions with dereferenced pointers mess up the following.
#Embed in parentheses to cure
   if {[handleType $listHandle] != "LIST"} then {error "Not a LIST"}
   linkInit $listHandle iter
   set hndl [linkNext iter]
   while {$hndl != ""} {
	set response [structsEdit $hndl outline]
	if {$response == ""} then {
	   handleDel $hndl
	   handleDel [keylget iter ITERATOR]
	   return
	   }
	set command [lvarpop response]
	set line [lvarpop response]
	if {$command == "N"} then {
	   set hndl [linkNext iter]
	   }
	if {$command == "P"} then {
	   set hndl [linkPrev iter]
	   }
	}
   return
   }

##############################################################################

# Edit a list, one line per object.
proc listEdit {listHandle {field ""}} {

   if {[handleType $listHandle] != "LIST"} then {error "Not a LIST"}
#field can have several items.  Concatenate all but the last one into
#field1, let the last one be field2.  This is done so we can use arrayShow
#to extract a field value.
   set refresh 1
   set beginline 1
   while {1} {
	if {$refresh == 1} {
	   linkInit $listHandle iter
	   set hndl [linkNext iter]
	   if {$hndl == ""} then {return}
	   set arrayName(0) $listHandle
	   set class [handleType $hndl]
# Don't mess with blank fields
	   if {$field == ""} {
		puts stdout "Select Attribute from next display" nonewline
		flush stdout
		sleep 1
		schemaEdit $class field
		}
	   if {$field == ""} return
	   set field [split $field .]
	   set flen [llength $field]
	   set field1 ""
	   if {$flen > 1} {
		for {set i 1} {$i < $flen} {incr i} {
		   append field1 . [lindex $field [expr $i-1]]
		   }
		}
	   set field2 [lindex $field [expr $flen-1]]
	   set arrayVal(0) $class.$field
	   set index 0
#Create a new handle for each item on the list
	   while {$hndl != ""} {
		incr index
		set handleSave($index) [handleNew]
		handleBindFromHandle $handleSave($index) $hndl
		set arrayName($index) $handleSave($index)
		set err [catch {set arrayVal($index) \
		   [string trim [arrayShow $hndl$field1 $field2] \"]}]
		if {$err == 1} then {
		   set arrayVal($index) \
"Error: Tried to display $hndl$field1.$field2 in [handleType $hndl]"
		   }
		set hndl [linkNext iter]
		}
	   set nelem $index
	   }
	set refresh 0
#No field is editable
	set response [screenEdit "" $beginline arrayName -40 arrayVal -80]
	if {$response == ""} then break
#nelem is number of elements excluding index 0, which is a header
	set nelem [expr {[array size arrayName] - 1}]
#This is a simple link to follow.  All names are Link#'s
	set command [lindex $response 0]
	set line [lindex $response 1]
	set beginline $line
	if {$command == "Z"} then break
	if {$command == "L"} then {
	   structsEdit $arrayName($line)
	   }
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Edit the list
	if {$command == "X"} then {
	   }
	}
   for {set i 1} {$i <= $nelem} {incr i} {
	handleDel $handleSave($i)
	}
   return $response
   }

##############################################################################

#StructsEdit for database objects; main difference is that character strings
#are treated as vstrs, not shMalloc'ed things.

proc objStructsEdit {objHndl hndl begin {outline ""} {message ""}} {
#Complex handle expressions with dereferenced pointers mess up the following.
#Embed in parentheses to cure
#outline is use as an external variable to return the last line array elements
   if {$outline != ""} {upvar 1 $outline outlist}
   set nelem 0
   set arrayName($nelem) $objHndl
   set arraySchema($nelem) STRUCT
   set arrayVal($nelem) [handleType $hndl]
   recursEdit "($hndl)" arrayName arraySchema arrayVal nelem
   set msg(0) $message
   set response [screenEdit msg $begin arrayName -40 arrayVal 60]
   set command [lindex $response 0]
   set line [lindex $response 1]
   set outlist [list $arrayName($line) $arraySchema($line) $arrayVal($line)]
#Update values if CTRL-X was used to exit
   if {$command == "X"} {
	for {set i 1} {$i <= $nelem} {incr i} {
	   if {$arraySchema($i) == "PRIM"} then {
	      if {[string trim [schemaGet $arrayName($i)] {{}" }] != "char*"} \
		then {
		catch {handleSet $arrayName($i) $arrayVal($i)}
	      } else {
		p_vstrSet $arrayName($i) $arrayVal($i)
		}
	      continue
	      }
#	   if {$arraySchema($i) == "ARRAY"} then {
#		set handle [lindex $arrayName($i) 0]
#		set arg [lindex $arrayName($i) 1]
#		catch {arraySet $handle $arg $arrayVal($i)}
#		}
#For pointers, if we reset the value to NULL, then check to see if it
# is a Link#.  If so, so a select on the target class and rebind.
	   if {$arraySchema($i) == "PTR"} {
#Get the target of the pointer
		set name $arrayName($i)
		set top [lvarpop name]
		set var [lvarpop name]
		set allschema [schemaGet $top]
		set type [keylget allschema $var]
		if {$arrayVal($i) != ""} continue
		if {$type == "COD#*"} then {
#Get shiva type of the class
		   set toptype [handleType $top]
#Strip off leading Link#
		   set class [string range $toptype 5 end]
		   puts stdout \
"Pick a display argument from the next window and type <CTRL-X>" nonewline
		   flush stdout
		   sleep 2
		   schemaEdit $class field
		   if {$field == ""} continue
		   set selVstr [select $class]
#Be nice and sort the vstr.  If the field is complex, this will fail, so catch
# errors
		   catch {sortVstr $selVstr $field}
		   puts stdout \
"Pick an object from the next window and type <CTRL-Z>" nonewline
		   flush stdout
		   sleep 2
		   set subresponse [o_vstrEdit $selVstr.vstrptr $field $class]
		   set line [lindex $subresponse 1]		   
		   set line [expr $line-1]
		   handleSetFromHandle $top \
			"(($toptype *)($selVstr.vstrptr))<$line>"
		   vstrDel $selVstr
		   }
		}
	   }
	}
#For convenience, set response to null if CTRL-Z was used to exit.
   if {$command == "Z"} then {set response ""}
   return $response
   }

###############################################################################
#Input is a handle to a Link#.  Assumption is that we want to edit the
#object, so mark for update
proc objectEdit {hndl {message ""}} {
   if {[regexp "Link#" [handleType $hndl]] != 1} {error
		"Handle $hndl is not a Link#"}
#If handle has an invalid link, just catch.  This happens, e.g., if the
#Link is null.
   if {[catch {set temp [objLocate $hndl [handleNew]]}]} then {
	echo Object not found - may have been deleted?
	flush stdout
	sleep 2
	return
	}
#Enter a while loop.  If we follow links while editing, make sure that we
#return and redisplay the original object.
   set beginline 1
   while {1} {
	set response [objStructsEdit $hndl $temp $beginline outline $message]
	if {$response == ""} then break;
	set command [lindex $response 0]
	set beginline [lindex $response 1]
	if {$command == "L"} then {
	   set name [lvarpop outline]
	   set schema [lvarpop outline]
	   set val [lvarpop outline]
	   if {$schema != "PTR"} then {continue}
#Get the target of the pointer
	   set top [lvarpop name]
	   set var [lvarpop name]
	   set allschema [schemaGet $top]
	   set type [keylget allschema $var]
	   if {$type == "COD#*"} then {
		objectEdit $top $message
		}
#Are we following a LinkVstr?
	   if {[string range $type 0 4]  == "Link#"} then {
# Get length
		set size [o_vstrSize $top.$var]
# Get class
		set len [string length $type]
		set class [string range $type 5 [expr $len-2]]
		puts stdout \
"Vstr size is $size.  Pick a display argument from the next window" nonewline
		flush stdout
		sleep 1
		schemaEdit $class field
		if {$field == ""} {
		   o_vstrPage $top.$var
		} else {
		   o_vstrEdit $top.$var $field $class
		   }
		}
#Are we following a Vstr?
	   if {[string range $type 0 4]  == "Vstr#"} then {
# Get type
		set len [string length $type]
		set class [string range $type 5 [expr $len-2]]
#Set size to 0 to cover myself
		set size 0
		if {$class == "char" || $class == "unsigned char"} {set size 1}
		if {$class == "short" || $class == "unsigned short"} \
			{set size 2}
		if {$class == "int" || $class == "unsigned int"} {set size 4}
		if {$class == "float"} {set size 4}
		if {$class == "double"} {set size  8}
		arrayEdit $top.$var $class $size
		}
	   continue
#For convenience, if I do an update with CTRL-X, redisplay the window
	   }
	if {$command == "X"} {
	   catch {objMark $hndl}
	   continue
	   }
	break
	}
   handleDel $temp
   return $response
   }

##############################################################################

# Edit a vstr of elemental types, one line per element.

proc arrayEdit {vstrHandle class size} {
   set index 0
#Default variables for headers
   set arrayName(0) $vstrHandle
   set arrayVal(0) "$class* $vstrHandle"
   for {set i 1} {$i <= $size} {incr i} {
	if {$size == 0} break
	incr index
	set arrayName($index) "(($class *)$vstrHandle)<[expr $i-1]>"
	set arrayVal($index) [string trim \
		[handleShow "(($class *)$vstrHandle)<[expr $i-1]>"] \"]
	}
   set line 1
   while {1} {
	set response [screenEdit "" $line arrayName -40 arrayVal 60]
	set command [lindex $response 0]
	set line [lindex $response 1]
	if {$command == "X"} then {
	   for {set i 1} {$i <= $size} {incr i} {
	   	catch {handleSet $arrayName($i) $arrayVal($i)}
		}
	   continue
	   }
	break
	}
   return $response
   }

##############################################################################
# Step through a vstr (array) of objects
proc o_vstrPage {vstrHandle} {
   set size [o_vstrSize $vstrHandle]
   if {$size == 0} return
   set index 0
   while {$index >= 0 && $index < $size} {
	set message "Index $index of range 0 to [expr $size-1]"
	set response [objectEdit "((Link# *)$vstrHandle)<$index>" $message]
	if {$response == ""} return
	set command [lvarpop response]
	set line [lvarpop response]
	if {$command == "N"} then {
	   incr index
	   }
	if {$command == "P"} then {
	   set index [expr $index-1]
	   }
	if {$command == "I"} then {
	   puts stdout "Index (0 to $size): " nonewline
	   flush stdout
	   gets stdin index
	   }
	}
   return
   }

##############################################################################
proc vstrPage {vstrHandle} {
   if {[handleType $vstrHandle] != "VSTR"} then {error "Not a VSTR handle"}
   if {[handleShow $vstrHandle.len] == 0} return
   o_vstrPage $vstrHandle.vstrptr
   }

##############################################################################

# Edit an o_vstr, one line per o_object.  Must specify a window to look at
proc o_vstrEdit {vstrHandle field class} {
#field can have several items.  Concatenate all but the last one into
#field1, let the last one be field2.  This is done so we can use arrayShow
#to extract a field value.
# Don't mess with blank fields
   if {$field == ""} return
   set field [split $field .]
   set flen [llength $field]
   set field1 ""
   if {$flen > 1} {
	for {set i 1} {$i < $flen} {incr i} {
	   append field1 . [lindex $field [expr $i-1]]
	   }
	}
   set field2 [lindex $field [expr $flen-1]]
   set refresh 1
   set beginline 1
   while {1} {
	if {$refresh == 1} {
	   set index 0
	   set size [o_vstrSize $vstrHandle]
	   set arrayName(0) $vstrHandle
	   set arrayVal(0) $class.$field
	   for {set i 0} {$i < $size} {incr i} {
		if {$size == 0} break
		incr index
		set arrayName($index) "((Link# *)$vstrHandle)<$i>"
		set temp [handleNew]
		set err [catch {set temp [objLocate $arrayName($index) $temp]}]
		if {$err == 1} {
		   set arrayVal($index) \
			"Error: Invalid object (Possibly deleted?)"
		   handleDel $temp
		   continue
		   }
		set err [catch {set arrayVal($index) \
		   [string trim [arrayShow $temp$field1 $field2] \"]}]
		if {$err == 1} then {
		   set arrayVal($index) \
"Error: Invalid field. Expect class is $class, actual is [handleType $temp]"
		   }
		handleDel $temp
		}
	   }
	set refresh 0
#No field is editable
	set response [screenEdit "" $beginline arrayName -40 arrayVal -60]
	if {$response == ""} then break
# Reset dimensions of TCL arrays and vstr; they need not be the same
	set size [o_vstrSize $vstrHandle]
#nelem is number of elements excluding index 0, which is a header
	set nelem [expr {[array size arrayName] - 1}]
#This is a simple link to follow.  All names are Link#'s
	set command [lindex $response 0]
	set line [lindex $response 1]
	set beginline $line
	if {$command == "Z"} then break
	if {$command == "L"} then {
	   objectEdit $arrayName($line)
	   }
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Edit the list
	if {$command == "X"} then {

	   echo Selecting all objects in class $class ...
	   flush stdout
	   set selVstr [select $class]
#Be nice and sort the vstr.  If the field is complex, this will fail, so catch
# errors
	   echo Sorting objects in class $class by field $field ...
	   flush stdout
	   catch {sortVstr $selVstr $field}
	   set nvstr [handleShow $selVstr.len]
	   echo Identifying matching members ...
	   flush stdout
#Mark existing objects
	   set msg(0) "Select / unselect desired objects"
	   for {set i 0} {$i < $nvstr} {incr i} {
		for {set index 1} {$index <= $nelem} {incr index} {
		   if {[handleShow "((PTR *)($selVstr.vstrptr))<$i>"]
		   == [handleShow "((PTR *)$vstrHandle)<[expr $index-1]>"]} \
		   then {set msg([expr $i+1]) 1}
		   }
		}
	   set subresponse [o_vstrMarkEdit $selVstr.vstrptr $field $class msg]
	   set command [lindex $subresponse 0]
	   if {$command != "X"} {
		vstrDel $selVstr
		continue
		}
	   if {[handleShow $vstrHandle] != "0x0"} then {o_vstrDel $vstrHandle}
	   for {set i 1} {$i <= [handleShow $selVstr.len]} {incr i} {
		if {$msg($i) != ""} {
		   o_vstrAppend $vstrHandle \
			"((Link# *)($selVstr.vstrptr))<[expr $i-1]>"
		   }
		}
	   vstrDel $selVstr
	   unset arrayName
	   unset arrayVal
	   set refresh 1
	   }
	}
   return $response
   }

##############################################################################

# Edit an o_vstr, one line per o_object, and return marked objects.
# This is a stripped version of o_vstrEdit
proc o_vstrMarkEdit {vstrHandle field class inmsg} {
   upvar 1 $inmsg msg
   set size [o_vstrSize $vstrHandle]
   set index 0
#field can have several items.  Concatenate all but the last one into
#field1, let the last one be field2.  This is done so we can use arrayShow
#to extract a field value.
   set field [split $field .]
   set flen [llength $field]
   set field1 ""
   if {$flen > 1} {
	for {set i 1} {$i < $flen} {incr i} {
	   append field1 . [lindex $field [expr $i-1]]
	   }
	}
   set field2 [lindex $field [expr $flen-1]]
   set arrayName(0) $vstrHandle
#Default variables for headers
   set arrayName(0) $vstrHandle
   set arrayVal(0) $class.$field
   for {set i 0} {$i < $size} {incr i} {
	if {$size == 0} break
	incr index
	set arrayName($index) "((Link# *)$vstrHandle)<$i>"
	set temp [handleNew]
	set err [catch {set temp [objLocate $arrayName($index) $temp]}]
	if {$err == 1} {
	   set arrayVal($index) ""
	   handleDel $temp
	   continue
	   }
	   catch {set arrayVal($index) \
		[string trim [arrayShow $temp$field1 $field2] \"]}]
	handleDel $temp
	}
   set line 1
   while {1} {
	set msg(0) "Highlight desired objects and type <CTRL_X> to save"
#No editing allowed
	set response [screenEdit msg $line arrayName -40 arrayVal -60]
	set command [lindex $response 0]
	set line [lindex $response 1]
	if {$command == "L"} then {
	   objectEdit $arrayName($line)
	   continue
	   }
	break
	}
   return $response
   }

###############################################################################
# More user-friendly version of o_vstrEdit
proc vstrEdit {vstrHandle {field ""} {class ""}} {
   if {[handleType $vstrHandle] != "VSTR"} {error "Input handle is not a VSTR"}
#Take class and field from vstrHandle fields if present
   if {$field == ""} {
	set attr [handleShow $vstrHandle.attribute]
	regsub "::" $attr " " attr
	set field [lindex $attr [expr [llength $attr]-1]]
	}
   if {$field == "(null)"} then {set field ""}
#Class specifier is OK
   if {$class == ""} {
	set class [handleShow $vstrHandle.clss]
	}
   if {$class == "(null)"} then {set class ""}
# Query for class and attribute
   if {$class == ""} {
	puts stdout "Class = " nonewline
	flush stdout
	gets stdin class
#	if {$class != ""} {p_vstrSet $vstrHandle.clss $class}
	}
   if {$field == ""} {
	puts stdout "Select Attribute from next display" nonewline
	flush stdout
	sleep 1
	schemaEdit $class field
#	if {$field != ""} {p_vstrSet $vstrHandle.attribute $field}
	}
   if {([handleShow $vstrHandle.len] < 10000) && ($field != "")} {
	o_vstrEdit $vstrHandle.vstrptr $field $class
   } else {
	o_vstrPage $vstrHandle.vstrptr
	}
# Must reset length
   set size [o_vstrSize $vstrHandle.vstrptr]
   handleSet $vstrHandle.len $size
   }

##############################################################################

proc schemaEdit {schemaElem {inline ""}} {
   if {$inline != ""} then {
	upvar 1 $inline outline
	}
   if {[llength $schemaElem] == 1} then {
	set arg ""
	set type $schemaElem
   } else {
	set arg [lindex $schemaElem 0]
	set type [lindex $schemaElem 1]
	}
   set outline ""
   if {[catch {set schemaList [schemaArrayGetFromType $type]}]} {
	echo "Type is invalid or too complex"
	flush stdout
	sleep 1
	return ""
	}
   set len [llength $schemaList]
   set arrayName(0) $arg
   set arrayVal(0) $type
   for {set i 1} {$i <= $len} {incr i} {
	if {$len == 0} break
	set next [lvarpop schemaList]
	if {[llength $next] == 0} then {
	   set arrayName($i) $arg
	   set arrayVal($i) ""
	   continue
	   }
	if {[llength $next] == 1} then {
	   set arrayName($i) $arg
	   set arrayVal($i) [lindex $next 0]
	   continue
	   }
	set nextarg [lindex $next 0]
	if {[string index $nextarg 0] == "<" || $arg == ""} then {
	   set arrayName($i) $arg$nextarg
	} else {
	   set arrayName($i) $arg.$nextarg
	   }
	set arrayVal($i) [lindex $next 1]
	}
   set line 1
   while {1} {
	set msg(0) "<CTRL_X> to make a selection" 
	set response [screenEdit msg $line arrayName -40 arrayVal -40]
	set command [lindex $response 0]
	set line [lindex $response 1]
	if {$command == "N"} then {
	   set response [schemaEdit "$arrayName($line) $arrayVal($line)" \
		outline]
	   set command [lindex $response 0]
	   }
	if {$command == "X"} then {
	   set outline $arrayName($line)
	   set command "A"
	   break
	   }
# Control A: We are unwinding the stack and saving the result.  Build up
# the full argument name
	if {$command == "A"} then {
#	   set outline $arrayName($line).$outline
	   break
	   }
	if {$command == "R"} continue
	if {$command == "P"} then {
	   set command "R"
	   break
	   }
	if {$command == "Z"} then break
	}
   return "$command $line"
   }

############################################################################
# Enhance version of schemaGetFromType.  If the input type is of the form of
# an array, all elements of the array are returned.  Otherwise, function
# just like schemaGetFromType.  schemaElem can be of the form "name type"
# or just "type".  The former is needed to parse arrays because we want to
# return a list of the form {name<0> int} {name<1> int} ...
#
proc schemaArrayGetFromType {schema} {
   set sch [split $schema {[]<>}]
   set type [lindex $sch 0]
   set dimen {}
   for {set i 1} {$i < [llength $sch]} { set i [expr $i+2] } {
      lappend dimen [lindex $sch $i]
   }
# Is it a scalar after all?  More than 2 dimensions?
   if { [llength $dimen] == 0 || [llength $dimen] > 2} then {
        return [schemaGetFromType $schema]
        }
#Is the dimension really a number? If not, it is likely a Versant class
#specifier
   if { [llength $dimen] == 1 && [ctype digit [lindex $dimen 0]] == 0} {
        return [schemaGetFromType $schema]
        }

   set ans ""
   if { [llength $dimen] == 1 } {
      for {set i 0} {$i < [lindex $dimen 0]} { incr i } {
	lappend ans "<$i> $type"
	 }
      }
   if { [llength $dimen] == 2 } {
      for {set i 0} {$i < [lindex $dimen 0]} { incr i } {
         for {set j 0} {$j < [lindex $dimen 1]} { incr j } {
	    lappend ans "<$i><$j> $type"
            }
         }
      }
   return $ans
}

