#Read a .h file and parse off the typedef's.
#
#Top level routines operates on a group of files and load schema in one go.
#This is how the shiva makeio facility operates.

proc makeio {args} {
#Check 1st argument.  If it is -pragma, remove from the argument list and
#pass it as a flag to makeioSub
   if {[lindex $args 0] == "-pragma"} then {
	lvarpop args
	set flag 1
   } else {
	set flag 0
	}
   foreach arg $args {
	set files [glob $arg]
	foreach file $files {
	   makeioSub $flag $file
	   }
	}
   return
   }

proc makeioSub {flag file} {
   set fid [open $file]
   while {1} {
	set line [gets $fid]
	if {[eof $fid]} then break
#C allows braces to not have surrounding whitespace - an anathema to TCL
#Fix that here.  Also, convert tabs to spaces
	regsub -all \[\{] $line " \{ " line
	regsub -all \[\}] $line " \} " line
	regsub -all \t $line " " line
#Should also take care of character strings here
#Search for #define statements.
	if {[regsub #define $line "" line]} then {
	   set var [lindex $line 0]
	   set val [lindex $line 1]
#Let us make the variable name global; make it available to other files
	   global $var
	   set $var $val
	   continue
	   }
	append list $line " "
	}
   close $fid
#Find and zonk all comments.  I might have a "typedef" embedded in one!
#Convert all /* pragma SCHEMA */ statements to #schema#, which is not valid C
#Convert all /* pragma CONSTRUCTOR */ statements to #schema#, which is not valid
#No constructors, however, are allowed.
   regsub -all \n $list " " list
   regsub -all "pragma\[ ]*NOCONSTRUCTOR" $list \
	" " list
   regsub -all "/\[*]\[ ]*pragma\[ ]*SCHEMA\[ ]*\[*]/" $list \
	"#schema#" list
   regsub -all "/\[*]\[ ]*pragma\[ ]*CONSTRUCTOR\[ ]*\[*]/" $list \
	"#schema#" list
   regsub -all "/\[*]\[ ]*pragma\[ ]*USER\[ ]*\[*]/" $list \
	"#schema#" list
   while {1} {
	if {![regexp -indices (/\[*\]) $list c1]} then break
	set first [string range $list 0 [expr [lindex $c1 0]-1]]
	set last [string range $list [expr [lindex $c1 1]+1] end]
	if {![regexp -indices (\[*\]/) $last c2]} then {error "unmatched /* "}
	set last [string range $last [expr [lindex $c2 1]+1] end]
	set list "${first} ${last}"
	}
#Find and zonk all character strings.
   while {1} {
#First, zonk any embedded quotes
	regsub -all \\\" $list " " list
	if {![regexp -indices \" $list c1]} then break
	set first [string range $list 0 [expr [lindex $c1 0]-1]]
	set last [string range $list [expr [lindex $c1 1]+1] end]
	if {![regexp -indices \" $last c2]} then {error "unmatched \" "}
	set last [string range $last [expr [lindex $c2 1]+1] end]
	set list "${first} ${last}"
	}
#search for a "typedef" string
   while {1} {
	if {![regexp -indices "typedef " $list c1]} then break
	set list [string range $list [lindex $c1 0] end]
#We could use lvarpop, but it is slow
	set command [lindex $list 0]
	set list [lrange $list 1 end]
#Because of possible variations on specification of typedefs, I will
#just look for the next list item that ends with a semicolon.
#We could use lvarpop, but it is slow.
	while {1} {
	   set item [lindex $list 0]
	   if {$item == ""} then continue
	   set list [lrange $list 1 end]
	   if {[regexp -indices (\;\$) $item c1]} then {
		set item [string trimright $item \;]
		if {$item != ""} then {lappend command $item}
		break
		}
	   lappend command $item
	   }
#If flag is 1, we check for a #schema# following the typedef
	if {$flag == 0 || [lindex $list 0] == "#schema#"} then {
echo Defining [lindex $command [expr [llength $command]-1]]
	   eval $command
	   }
	}
   return
   }
