
proc makeCBindings {package } {
    global inonly fl
    global prefix
    set prefix sma
    set fl ""
    set flFile [open c_$package.par]
    while {[gets $flFile line] > 0} {
	set line [string trim $line]
	if {[string range $line 0 0] != "#"} {
	    set rn [lindex $line 0]
	    lappend fl $rn
	    set inonly($rn) [string toupper [lrange $line 1 end]]
	}
    }

    # Here are data we will use to write the files

    set hRoutineType(SUBROUTINE) void
    set hRoutineType(FUNCTION) float
    set hRoutineType(REAL_FUNCTION) float
    set hRoutineType(COMPLEX_FUNCTION) STCOMPLEX*
    set hRoutineType(DOUBLE_PRECISION_FUNCTION) double
    set hRoutineType(INTEGER) int
    set hRoutineType(REAL) float
    set hRoutineType(LOGICAL) short
    set hRoutineType(CHARACTER) char*

    set hArgType(INTEGER)  int
    set hArgType(REAL) float
    set hArgType(DOUBLE) double

    set bindFile [open ../src/$package.c w]
    set tmpFile  [open ../src/temp.c w]

    puts $bindFile "/*"
    puts $bindFile "WARNING:  this file is generated by"
    puts $bindFile "../etc/makeCBindings.tcl and should not be"
    puts $bindFile "modified or checked into the cvs repository."
    puts $bindFile "*/"

    puts $bindFile "/* <AUTO>"
    puts $bindFile " FILE: c_$package.c"
    puts $bindFile " c bindings for the $package package"
    puts $bindFile "</AUTO>"
    puts $bindFile "*/"

    puts $bindFile "\#include <stdlib.h>"
    puts $bindFile "\#include <stdio.h>"
    puts $bindFile "\#include <string.h>"
    puts $bindFile "\#include \"dervish.h\""
    puts $bindFile "\#include \"$package.h\""
    puts $bindFile "\#ifdef __cplusplus"
    puts $bindFile "extern \"C\"\{"
    puts $bindFile "\#endif"

    set incFile [open ../include/$package.h w]

    puts $incFile "/*"
    puts $incFile "WARNING:  this file is generated by"
    puts $incFile "../etc/makeCBindings.tcl and should not be"
    puts $incFile "modified or checked into the cvs repository."
    puts $incFile "*/"

    puts $incFile "\#ifndef [string toupper $package]_C_H"
    puts $incFile "\#define [string toupper $package]_C_H"
    puts $incFile "\#ifdef __cplusplus"
    puts $incFile "extern \"C\"\{"
    puts $incFile "\#endif"
    puts $incFile "/* HTML"
    puts $incFile "<HTML>"
    puts $incFile "<HEAD>"
    puts $incFile "<TITLE> SDSSMATH $package C Bindings </TITLE>"
    puts $incFile "</HEAD>"
    puts $incFile "<BODY>"
    puts $incFile "<H1> SDSSMATH $package C Bindings</H1>"
    puts $incFile "</BODY>"
    puts $incFile "</HTML>"
    puts $incFile "*/"

    
    set uname [exec uname]
    if {$uname == "IRIX64"} {
       set uname IRIX6
    }
   
    foreach froot $fl {
	if {[string first 1mach $froot] == 1} {
	    set fr2 [set froot]_$uname
	} else {
	    set fr2 $froot
	}
	set f [envscan \$SLATEC_DIR]/src/$fr2.f
	echo ............begin f=$f
	set name unknown
	set inFile [open $f r]
	set mode top
	gets $inFile line
	set argList ""
	while {$mode != "END"} {
	    gets $inFile line
	    if {[string range $line 0 3] == "C***"} {
		set sub [lindex $line 0]
		set prevMode $mode
		set mode [string range $sub 4 end]
	    }
	    case $mode in {
		top {
		    if {$name == "unknown"} {
			set index [string first \( $line]
			set sub [string range $line 0 [expr $index-1]]
			set name [lrange $sub end end]
			set routineType [string trim \
				[lrange $sub 0 [expr [llength $sub]-2]]]
			regsub -all " " $routineType _ routineType
			set line [string range $line $index end]
		    }
		    set line [string trim $line]
		    set line [string trim $line \(,+\)]
		    set temp [split $line ,]
		    foreach arg $temp {
			lappend argList [string trim $arg]
		    }
		}
		TYPE {
		    set index [string first \( $line]
		    set sub [string range $line 14 [expr $index-1]]
		    set type [string trim \
			    [lrange $sub 0 [expr [llength $sub]-1]]]
		}
	    }
	}
	foreach arg $argList {
	    set argFlavor($arg) "unknown"
	    set argArrayDim($arg) 0

	    if {[lsearch -exact $inonly($froot) $arg] >= 0} {
		set argIsInputOnly($arg) 1
	    } else {
		set argIsInputOnly($arg) 0
	    }
	}
	while {$mode != "FIRST"} {
	    gets $inFile line
	    if {[string range $line 0 3] == "C***"} {
		set sub [lindex $line 0]
		set prevMode $mode
		set mode [string range $sub 4 end]
	    }
	    if {[string range $line 0 0] != "C"} {
		case $mode in {
		    END {
			# Look for a declaration for each argument
			if {[string range $line 5 5] != " "} {
			    set flavor $prevFlavor
			} else {
			    set flavor [lindex $line 0]
			}
			if {$flavor != "DIMENSION"} {
			    set prevFlavor $flavor
			    set sub0 [lrange $line 1 end]
			    set sub1 [split $sub0 ,]
			    set sub2 ""
			    foreach s $sub1 {
				set s1 [string trim $s]
				foreach s1prime $s1 {
				    set first [string first "(" $s1prime]
				    if {$first == -1} {
					set s2 $s1prime
				    } else {
					set s2 [string range $s1prime \
						0 [expr $first-1]]
				    }
				    if {![cequal $s2 "DIMENSION"]} {
					lappend sub2 $s2
					if {![cequal $s1prime $s2]} {
					    set argArrayDim($s2) 1
					}
				    }
				}
			    }
			    foreach arg $argList {
				if {[lsearch $sub2 $arg] != -1} {
				    set argFlavor($arg) $flavor
				}
			    }
			}
		    }
		}
	    }
	}
	close $inFile
	# Now write to the top part of the .c file for this routine

	puts $bindFile $hRoutineType($routineType)
	puts $bindFile [string tolower $name]_\(
	set lastArg [lrange $argList end end]
	foreach arg $argList {
	    set af $argFlavor($arg)
	    if { ($af == "unknown") || ($af == "DIMENSION") } {
		set letter [string toupper [string range $arg 0 0]]
		if {[lsearch "I J K L M N" $letter] >= 0} {
		    set af INTEGER
		} else {
		    set af REAL
		}
		set argFlavor($arg) $af
	    }
	    if {$af == "EXTERNAL"} {
		set tempFileName $name.$arg
		if {![file exists $tempFileName]} {
		    error "I need the file $tempFileName to proceed"
		}
		set tempFile [open $name.$arg r]
		set icount 0
		while {[gets $tempFile tempLine] > 0} {
		    if {$icount > 0} {
			puts $bindFile " "
		    }
		    incr icount
		    puts -nonewline $bindFile $tempLine
		}
		close $tempFile
	    } else {
		puts -nonewline $bindFile "      $hArgType($af) *$arg"
	    }
	    if {$arg == $lastArg} {
		puts $bindFile \)\;
		puts $bindFile " "
	    } else {
		puts $bindFile ,
	    }
	}

	# Now write to the bottom part of the .c file for this routine
	# Now write to the .h file for this routine

	puts $tmpFile "/*<AUTO EXTRACT>"
	puts $tmpFile "ROUTINE: $prefix$name"
	puts $tmpFile "<HTML>"
	puts $tmpFile "Calls <a href=\"http://www-sdss.fnal.gov:8000/slatec/doc/www/$froot.f.html\">$froot</a>"
	puts $tmpFile "</HTML>"
	puts $tmpFile "</AUTO>*/"

	puts $incFile $hRoutineType($routineType)
	puts $tmpFile $hRoutineType($routineType)
	puts $incFile $prefix$name\(
	puts $tmpFile $prefix$name\(
	

	set lastArg [lrange $argList end end]
	foreach arg $argList {
	    set af $argFlavor($arg)
	    if {$af == "EXTERNAL"} {
		set tempFileName $name.$arg
		if {![file exists $tempFileName]} {
		    error "I need the file $tempFileName to proceed"
		}
		set tempFile [open $name.$arg r]
		set icount 0
		while {[gets $tempFile tempLine] > 0} {
		    if {$icount > 0} {
			puts $incFile " "
			puts $tmpFile " "
		    }
		    incr icount
		    puts -nonewline $incFile $tempLine
		    puts -nonewline $tmpFile $tempLine
		}
		close $tempFile
	    } else {
		if {$argIsInputOnly($arg)} {
		    set outLine \
			    "      $hArgType($af) $arg"
		} else {
		    set outLine \
			    "      $hArgType($af) *$arg"
		}
		puts -nonewline $incFile $outLine
		puts -nonewline $tmpFile $outLine
	    }

	    if {$arg == $lastArg} {
		puts $incFile "\)\;"
		puts $incFile " "
		puts $tmpFile "\) \{"
		puts $tmpFile " "
	    } else {
		puts $incFile ,
		puts $tmpFile ,
	    }
	}
	# Now write the actual call to the FORTRAN subroutine

	if {$routineType == "SUBROUTINE"} {
	    puts $tmpFile "[string tolower $name]_\("
	} else {
	    puts $tmpFile "return [string tolower $name]_\("
	}
	set lastArg [lrange $argList end end]
	foreach arg $argList {
		if {$argIsInputOnly($arg)} {
		    set outLine \
			    "      &$arg"
		} else {
		    set outLine \
			    "      $arg"
		}
	    puts -nonewline $tmpFile $outLine
	    if {$arg == $lastArg} {
		puts $tmpFile "\)\;"
		if {$routineType == "SUBROUTINE"} {
		    puts $tmpFile "return\;\}"
		} else {
		    puts $tmpFile "\;\}"
		}		
	    } else {
		puts $tmpFile ,
	    }
	}
    }

    close $tmpFile
    set tmpFile  [open ../src/temp.c r]
    while {[gets $tmpFile line] > 0} {
	puts $bindFile $line
    }
    close $bindFile

    
    puts $incFile "\#ifdef __cplusplus"
    puts $incFile "\}"
    puts $incFile "\#endif"
    puts $incFile "\#endif"
    close $incFile
}











