proc buildProd {name} {

    inipar $name.par
    echo ===> Begin building product $name with:
    foreach par "cvsTag declareVersion urlist brlist" {
	set $par [getpars $par]
	echo $par = [set $par]
    }
    if {$urlist == "none"} {set urlist ""}
    if {$brlist == "none"} {set brlist ""}

    set dashes ---------------------------------------------------------------
    echo $dashes
    set nerror 0
    if {[file exists $name]} {
	incr nerror
	echo "Error: The file $name already exists here."
    }

    set uname [exec uname]
    if {$uname == "IRIX64"} {set uname IRIX6}
    set installDir /p/$uname/$name/$declareVersion
    if {[file exists $installDir]} {
	incr nerror
	echo  "Error: The file $installDir already exists"
    }

    set sealLog seal.$name.$cvsTag.$declareVersion.$uname
    if {[file exists $sealLog]} {
	incr nerror
	echo "Error: sealLog=$sealLog already exists"
    }

    set ups [envscan \$UPS_DIR]/bin/ups
    catch "exec $ups list $name $declareVersion | grep NOT | grep DECLARED | wc -l" notdeclared
    set nd [lindex $notdeclared 0]
    if {$nd == 0} {
	incr nerror
	echo "Error: $name $declareVersion -f $uname is already declared."
    }	

    if {$nerror > 0} {
	error "nerror=$nerror"
    }

    # Make the build.csh file
    set file [open build.csh w]

    # unsetup all old versions
    set pdirlist [exec printenv | grep _DIR]
    foreach pdir $pdirlist {
	set index [expr [string first _ $pdir]-1]
	set pname [string tolower [string range $pdir 0 $index]]
	if {[lsearch {ups int} $pname] == -1} {
	    puts $file "unsetup $pname"
	} else {
	    # echo let $pname stay set up as $pdir
	}
    }

    # setup required versions
    foreach pair $brlist {
	set product [lindex $pair 0]
	set version [lindex $pair 1]
	puts $file "setup $product $version"
    }
    puts $file "setup sdsscvs"
    if {$uname == "OSF1"} {
	# make sure you have ncurses set up if this is OSF1.
	puts $file "if !(`printenv | grep NCURSES_DIR | wc -l`) then"
	puts $file "echo Need to setup ncurses"
	puts $file "setup ncurses"
	puts $file "endif"
    }

    # initialize the log file
    puts $file "echo $dashes"
    puts $file date
    puts $file \
	    "echo Build for product=$name cvsTag=$cvsTag \
	    declareVersion=$declareVersion"
    foreach pair $brlist {
	puts $file "echo $pair"
    }

    puts $file "echo Build with this environment:"
    puts $file "printenv | grep _DIR | sort"

    # checkout a fresh copy
    puts $file "echo $dashes"
    puts $file date
    set coName $name
    if {$coName == "dervish"} {set coName all_dervish}
    puts $file "echo Begin cvs co -r $cvsTag $coName"
    puts $file "cvs co -r $cvsTag $coName"
    puts $file "echo End cvs co -r $cvsTag $coName"
    
    # sdssmake all to compile the product
    puts $file "echo $dashes"
    puts $file date
    puts $file "echo Begin sdssmake all"
    puts $file "cd $name"
    set upname [string toupper [string toupper $name]_DIR]
    puts $file "setenv $upname \$cwd"
    puts $file "source ups/setup.csh"
    puts $file "sdssmake -nocvscheck all"
    puts $file "cd .."
    puts $file "echo End sdssmake all"
    
    # install the product
    puts $file "echo $dashes"
    puts $file date
    puts $file "echo Begin sdssmake install"
    puts $file "setenv [string toupper $name]_INSTALL_DIR $installDir"
    puts $file "mkdir $installDir"
    puts $file "cd $name"
    puts $file "sdssmake -nocvscheck install"
    puts $file "cd .."
    puts $file "echo End sdssmake install"
    
    # declare the product
    set dl "ups declare -f $uname -r $installDir"
    set ustring "none"
    if {[llength $urlist] > 0} {
	set first 1
	set dstring ""
	foreach pair $urlist {
	    if {$first} {
		set first 0
	    } else {
		lappend dstring ,
	    }
	    set product [lindex $pair 0]
	    set pversion [lindex $pair 1]
	    lappend dstring <
	    lappend dstring $product
	    lappend dstring $pversion
	    lappend dstring $uname
	}
	set ustring [format " -u \" %s \" " $dstring]
	echo ustring=$ustring
    }
    if {$ustring != "none"} {
	set dl "$dl $ustring"
    }
    lappend dl $name 
    lappend dl $declareVersion
    puts $file $dl

    # seal the product
    puts $file "echo Seal for $upname > $sealLog"
    puts $file "date >> $sealLog"
    puts $file "echo === _DIR environment >> $sealLog"
    puts $file "printenv | grep _DIR | sort >> $sealLog"
    puts $file "echo === sum for lib/.a >> $sealLog"
    puts $file "sum $installDir/lib/*.a >> $sealLog"
    puts $file "echo === sum for bin/ >> $sealLog"
    puts $file "sum $installDir/bin/* >> $sealLog"
    puts $file "echo === sum for etc/.tcl >> $sealLog"
    puts $file "sum $installDir/etc/*.tcl >> $sealLog"
    puts $file "echo === ls -lR for $upname >> $sealLog"
    puts $file "chmod -R a-w $installDir" 
    puts $file "chmod -R a+r $installDir" 
    puts $file "ls -lR $installDir >> $sealLog"
    puts $file "chmod  +w $installDir"
    puts $file "cp $sealLog $installDir"
    puts $file "chmod a-w $installDir"
    puts $file "chmod a-w $installDir/*"
    puts $file "echo DONE running build.csh"
    close $file


    echo NOW BEGIN THE BUILD -- inspect build.log to see what happened.
    set pid [exec nohup csh build.csh >& build.log &]
    echo Running in the background with pid=$pid
}


proc sealTest {name cvsTag declareVersion} {
    set uname [exec uname]
    if {$uname == "IRIX64"} {set uname IRIX6}
    set installDir /p/$uname/$name/$declareVersion
    set sealLog seal.$name.$cvsTag.$declareVersion.$uname
    if {![file readable $installDir/$sealLog]} {
	error "Can not read $sealLog in $installDir"
    }

    set newSealLog newSeal.log
    set upname [string toupper [string toupper $name]_DIR]
    set file [open sealtest.csh w]
    puts $file "echo Seal for $upname > $newSealLog"
    puts $file "date >> $newSealLog"
    puts $file "echo === _DIR environment >> $newSealLog"
    puts $file "printenv | grep _DIR | sort >> $newSealLog"
    puts $file "echo === sum for lib/.a >> $newSealLog"
    puts $file "sum $installDir/lib/*.a >> $newSealLog"
    puts $file "echo === sum for bin/ >> $newSealLog"
    puts $file "sum $installDir/bin/* >> $newSealLog"
    puts $file "echo === sum for etc/.tcl >> $newSealLog"
    puts $file "sum $installDir/etc/*.tcl >> $newSealLog"
    puts $file "echo === ls -lR for $upname >> $newSealLog"
    puts $file "ls -lR $installDir >> $newSealLog"
    close $file
    echo Make a comparison seal.log
    exec csh sealtest.csh > /dev/null 2>&1

    echo Compare newSeal.log to $installDir/$sealLog
    set f1 [open $installDir/$sealLog r]
    set f2 [open newSeal.log r]

    set ndiff 0
    while {[gets $f1 l1] > -1} {
	if {[string first "=== sum" $l1] > -1} {
	    # echo found beginning of f1 $l1
	    while {[gets $f2 l2] > -1} {
		if {[string first "=== sum" $l2] > -1} {
		    # echo found beginning of f2 $l2
		    while {[gets $f2 l2] > -1} {
			if {[string first $sealLog $l2] == -1} {
			    gets $f1 l1
			    if {$l1 != $l2} {
				if {[lindex $l1 0] != "total"} {
				    incr ndiff
				    echo difference number $ndiff
				    echo $l1
				    echo $l2
				}
			    }
			} else {
			    # echo don't compare $l2
			}   
		    }
		    echo Number of differences is $ndiff
		    if {[file exists $sealLog]} {
			echo Compare $sealLog here and in $installDir
			set sum1 [exec sum $sealLog]
			set sum2 [exec sum $installDir/$sealLog]
			set s1 [lindex $sum1 0]
			set s2 [lindex $sum2 0]
			if {$s1 != $s2} {
			    echo "trouble: $sealLog here and $installDir"
			    echo $sum1
			    echo $sum2
			    error "BAD SEAL TEST"
			}
		    }
		    echo DONE WITH COMPARE
		    return
		}
	    }
	    error Could not find === sum in newSeal.log
	}
    }
    error "Could not find === sum in $installDir/$sealLog"
}

