#!/usr/local/bin/bash
umask 002
unset PRODUCTS
. /sdss/ups/etc/setups.sh

# Following exported for use in sdssmake environment
export upsRoot=/sdss/ups
export upsSetup=/sdss/ups/etc/setups.sh
export newVers=v0_9b
export flavDir=Linux-2-4-2-3-2

cd Linux-2-4-2-3-2
echo "---------------------------------------------------------------"
date
echo -e "\nBegin cvs export -r v0_9b slatec"
setup sdsscvs
cvs export -r v0_9b slatec
rc=$?
if (( rc != 0 ))
then
    echo "error: \`cvs export -r v0_9b slatec' exited $rc"
    exit 1
fi
date
echo "Ended cvs export -r v0_9b slatec"
buildProds=(sdsstools tex)
echo "---------------------------------------------------------------"
echo -e "\nBegin setup of products required for building"
for item in ${buildProds[@]}
do
    echo "Begin \`setup $item'"
    setup $item
    rc=$?
    if (( rc != 0 ))
    then
        echo "error: \`setup $item' exited $rc"
        exit 1
    echo "Ended \`setup $item'"
    fi
done
date
echo "Ended setup of products required for building"
echo "---------------------------------------------------------------"
echo "native compiler desired (default), will unset gcc if already set"
[ ! -z "$SETUP_GCC" ] && unsetup gcc
date
echo "Using \"default\" gcc = $(which gcc)"
echo "Completed \`unsetup gcc' if previously set"
echo "---------------------------------------------------------------"
echo -e "\nBegin setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/slatec slatec "
setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/slatec slatec 
rc=$?
if (( rc != 0 ))
then
    echo "error: \`setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/slatec slatec ' exited $rc"
    exit 1
fi
echo "Ended \`setup  -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/slatec slatec '"
echo "---------------------------------------------------------------"
echo -e "\nBegin setup tex"
setup tex
rc=$?
if (( rc != 0 ))
then
    echo "error: \`setup tex' exited $rc"
    exit 1
fi
echo "Completed \`setup tex'"
if (( ${#SETUP_POSTGRES} > 0 ))
then
    unsetup postgres
fi
echo "---------------------------------------------------------------"
date
echo "Build for product=slatec cvsTag=v0_9b declareVersion=v0_9b"
echo "slatec -r /usrdevel/lnx9/jhendry/products/build/Linux-2-4-2-3-2/slatec"
echo -e "\nBuild with this environment:\n"
set | grep "_DIR" | sort
echo "---------------------------------------------------------------"
echo -e "\nTEXINPUTS=$TEXINPUTS"
echo -e "\n$PATH"
echo "---------------------------------------------------------------"
export SLATEC_INSTALL_DIR=/sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
echo "Using SLATEC_INSTALL_DIR=/sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2"
echo "---------------------------------------------------------------"
date
if [ ! -f slatec/Makefile ]
then
    if [ -f slatec/Makefile.in -a -f slatec/configure ]
    then
        cd slatec
        ./configure
        cd ..
    else
        echo "error: no Makefile.in or configure script"
        exit 2
    fi
fi
echo "Begin sdssmake -fullopt all"
cd slatec
pwd
eval sdssmake -fullopt all
rc=$?
if (( rc != 0 ))
then
    echo "error: \`sdssmake -fullopt all' exited $rc"
    exit 1
fi
cd ..
echo "Ended sdssmake -fullopt all"
echo "---------------------------------------------------------------"
date
echo "Begin sdssmake install"
export SLATEC_DIR=/sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
echo "SLATEC is $SLATEC"
echo -e "Making product installation directory"
mkdir -m 775 -p /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
cd slatec
sdssmake install
rc=$?
if (( rc != 0 ))
then
    echo "error: \`sdssmake install' exited $rc"
    exit 1
fi
cd ..
echo "Ended sdssmake install"
upsDclComArgs="slatec v0_9b  -f Linux+2.4-2.3.2 -r slatec/v0_9b/Linux-2-4-2-3-2 -z /sdss/ups/db"
if [ -f slatec/ups/slatec.table ]
then
    upsTblArgs="-m slatec.table -M ups"
else
    # product ups directory has not table file, copying from earlier version
    Gargs="-r /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2  slatec v0_9b -m v0_9b.table"
    ups copy -W -f Linux+2.4-2.3.2 -G "$Gargs" slatec v0_9
    # remove all -z flags from copied table file
    perl -p -i -e 's/\b -z .*\b//g' /sdss/ups/db/slatec/v0_9b.table
    upsTblArgs="-m v0_9b.table"
fi
echo "Begin \`ups declare $upsDclComArgs $upsTblArgs'"
ups declare $upsDclComArgs $upsTblArgs
echo "Ended \`ups declare $upsDclComArgs $upsTblArgs'"
chmod g+w /sdss/ups/prd/slatec/v0_9b
chown jhendry.products /sdss/ups/prd/slatec/v0_9b
echo -e "\nSeal the product"
{
    echo "Seal for SLATEC"
    date
    echo "=== _DIR environment"
    set | grep "_DIR" | sort
    for _dir in bin etc lib
    do
        if [ -d /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2/$_dir ]
        then
            for _f in /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2/$_dir/*
            do
                if [ -f $_f ]
                then
                    echo "=== sum for $_dir/*"
                    sum /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2/$_dir/*
                    break
                fi
            done
        fi
    done
    echo "=== ls -lR for SLATEC"
    chmod -R a-w /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
    ls -lR /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
} > seal.slatec.v0_9b.v0_9b.Linux+2.4-2.3.2
chmod +w /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
cp seal.slatec.v0_9b.v0_9b.Linux+2.4-2.3.2 /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
chgrp -R products /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
chmod a-w /sdss/ups/prd/slatec/v0_9b/Linux-2-4-2-3-2
echo "ALL DONE"
